@@ -11,21 +11,36 @@ print_help() {
 declare -rA GROUPED_DEPS=(
   [gcc]="binutils gcc build-essential cmake ccache curl unzip git pkg-config mold"
   [clang]="binutils clang make cmake ccache curl unzip git pkg-config mold"
-  [clang_ubuntu_20_04]="binutils clang make cmake ccache curl unzip git pkg-config"
+  [coverity]="binutils clang-12 make cmake ccache curl unzip git pkg-config"
+  [gcc_preprocess]="
+    binutils
+    build-essential
+    clang
+    cmake
+    curl
+    gcc
+    git
+    libclang-dev
+    ninja-build
+    python3-clang
+    python3-pip
+    unzip
+  "
 
   # Common dependencies for building OpenMW.
   [openmw-deps]="
-    libboost-filesystem-dev libboost-program-options-dev
+    libboost-program-options-dev
     libboost-system-dev libboost-iostreams-dev
 
     libavcodec-dev libavformat-dev libavutil-dev libswscale-dev libswresample-dev
-    libsdl2-dev libqt5opengl5-dev libopenal-dev libunshield-dev libtinyxml-dev
-    libbullet-dev liblz4-dev libpng-dev libjpeg-dev libluajit-5.1-dev
-    librecast-dev libsqlite3-dev ca-certificates libicu-dev libyaml-cpp-dev
+    libsdl2-dev libqt5opengl5-dev qttools5-dev qttools5-dev-tools libopenal-dev
+    libunshield-dev libtinyxml-dev libbullet-dev liblz4-dev libpng-dev libjpeg-dev
+    libluajit-5.1-dev librecast-dev libsqlite3-dev ca-certificates libicu-dev
+    libyaml-cpp-dev libqt5svg5 libqt5svg5-dev
   "
 
   # These dependencies can alternatively be built and linked statically.
-  [openmw-deps-dynamic]="libmygui-dev libopenscenegraph-dev libsqlite3-dev"
+  [openmw-deps-dynamic]="libmygui-dev libopenscenegraph-dev libsqlite3-dev libcollada-dom-dev"
   [clang-tidy]="clang-tidy"
 
   # Pre-requisites for building MyGUI and OSG for static linking.
@@ -52,7 +67,6 @@ declare -rA GROUPED_DEPS=(
     libavcodec58
     libavformat58
     libavutil56
-    libboost-filesystem1.74.0
     libboost-iostreams1.74.0
     libboost-program-options1.74.0
     libboost-system1.74.0
@@ -73,12 +87,25 @@ declare -rA GROUPED_DEPS=(
     libswresample3
     libswscale5
     libtinyxml2.6.2v5
-    libyaml-cpp0.7
+    libyaml-cpp0.8
     python3-pip
     xvfb
   "
 
+  [libasan6]="libasan6"
+
   [android]="binutils build-essential cmake ccache curl unzip git pkg-config"
+
+  [openmw-clang-format]="
+    clang-format-14
+    git-core
+  "
+
+  [openmw-qt-translations]="
+    qttools5-dev
+    qttools5-dev-tools
+    git-core
+  "
 )
 
 if [[ $# -eq 0 ]]; then
@@ -99,7 +126,24 @@ export APT_CACHE_DIR="${PWD}/apt-cache"
 export DEBIAN_FRONTEND=noninteractive
 set -x
 mkdir -pv "$APT_CACHE_DIR"
-apt-get update -yqq
+
+while true; do
+  apt-get update -yqq && break
+done
+
 apt-get -qq -o dir::cache::archives="$APT_CACHE_DIR" install -y --no-install-recommends software-properties-common gnupg >/dev/null
-add-apt-repository -y ppa:openmw/openmw
+
+while true; do
+  add-apt-repository -y ppa:openmw/openmw && break
+done
+
+while true; do
+  add-apt-repository -y ppa:openmw/openmw-daily && break
+done
+
+while true; do
+  add-apt-repository -y ppa:openmw/staging && break
+done
+
 apt-get -qq -o dir::cache::archives="$APT_CACHE_DIR" install -y --no-install-recommends "${deps[@]}" >/dev/null
+apt list --installed
