@@ -1,9 +1,11 @@
 #include <benchmark/benchmark.h>
 
 #include <components/detournavigator/navmeshtilescache.hpp>
+#include <components/detournavigator/stats.hpp>
 #include <components/esm3/loadland.hpp>
 
 #include <algorithm>
+#include <iterator>
 #include <random>
 
 namespace
@@ -23,29 +25,25 @@ namespace
         PreparedNavMeshData mValue;
     };
 
-    template <typename Random>
-    osg::Vec2i generateVec2i(int max, Random& random)
+    osg::Vec2i generateVec2i(int max, auto& random)
     {
         std::uniform_int_distribution<int> distribution(0, max);
         return osg::Vec2i(distribution(random), distribution(random));
     }
 
-    template <typename Random>
-    osg::Vec3f generateAgentHalfExtents(float min, float max, Random& random)
+    osg::Vec3f generateAgentHalfExtents(float min, float max, auto& random)
     {
         std::uniform_int_distribution<int> distribution(min, max);
         return osg::Vec3f(distribution(random), distribution(random), distribution(random));
     }
 
-    template <typename OutputIterator, typename Random>
-    void generateVertices(OutputIterator out, std::size_t number, Random& random)
+    void generateVertices(std::output_iterator<int> auto out, std::size_t number, auto& random)
     {
         std::uniform_real_distribution<float> distribution(0.0, 1.0);
         std::generate_n(out, 3 * (number - number % 3), [&] { return distribution(random); });
     }
 
-    template <typename OutputIterator, typename Random>
-    void generateIndices(OutputIterator out, int max, std::size_t number, Random& random)
+    void generateIndices(std::output_iterator<int> auto out, int max, std::size_t number, auto& random)
     {
         std::uniform_int_distribution<int> distribution(0, max);
         std::generate_n(out, number - number % 3, [&] { return distribution(random); });
@@ -55,39 +53,40 @@ namespace
     {
         switch (index)
         {
-            case 0: return AreaType_null;
-            case 1: return AreaType_water;
-            case 2: return AreaType_door;
-            case 3: return AreaType_pathgrid;
-            case 4: return AreaType_ground;
+            case 0:
+                return AreaType_null;
+            case 1:
+                return AreaType_water;
+            case 2:
+                return AreaType_door;
+            case 3:
+                return AreaType_pathgrid;
+            case 4:
+                return AreaType_ground;
         }
         return AreaType_null;
     }
 
-    template <typename Random>
-    AreaType generateAreaType(Random& random)
+    AreaType generateAreaType(auto& random)
     {
         std::uniform_int_distribution<int> distribution(0, 4);
         return toAreaType(distribution(random));
     }
 
-    template <typename OutputIterator, typename Random>
-    void generateAreaTypes(OutputIterator out, std::size_t triangles, Random& random)
+    void generateAreaTypes(std::output_iterator<AreaType> auto out, std::size_t triangles, auto& random)
     {
         std::generate_n(out, triangles, [&] { return generateAreaType(random); });
     }
 
-    template <typename OutputIterator, typename Random>
-    void generateWater(OutputIterator out, std::size_t count, Random& random)
+    void generateWater(std::output_iterator<CellWater> auto out, std::size_t count, auto& random)
     {
         std::uniform_real_distribution<float> distribution(0.0, 1.0);
         std::generate_n(out, count, [&] {
-            return CellWater {generateVec2i(1000, random), Water {ESM::Land::REAL_SIZE, distribution(random)}};
+            return CellWater{ generateVec2i(1000, random), Water{ ESM::Land::REAL_SIZE, distribution(random) } };
         });
     }
 
-    template <class Random>
-    Mesh generateMesh(std::size_t triangles, Random& random)
+    Mesh generateMesh(std::size_t triangles, auto& random)
     {
         std::uniform_real_distribution<float> distribution(0.0, 1.0);
         std::vector<float> vertices;
@@ -96,14 +95,14 @@ namespace
         if (distribution(random) < 0.939)
         {
             generateVertices(std::back_inserter(vertices), triangles * 2.467, random);
-            generateIndices(std::back_inserter(indices), static_cast<int>(vertices.size() / 3) - 1, vertices.size() * 1.279, random);
+            generateIndices(std::back_inserter(indices), static_cast<int>(vertices.size() / 3) - 1,
+                vertices.size() * 1.279, random);
             generateAreaTypes(std::back_inserter(areaTypes), indices.size() / 3, random);
         }
         return Mesh(std::move(indices), std::move(vertices), std::move(areaTypes));
     }
 
-    template <class Random>
-    Heightfield generateHeightfield(Random& random)
+    Heightfield generateHeightfield(auto& random)
     {
         std::uniform_real_distribution<float> distribution(0.0, 1.0);
         Heightfield result;
@@ -112,18 +111,15 @@ namespace
         result.mMinHeight = distribution(random);
         result.mMaxHeight = result.mMinHeight + 1.0;
         result.mLength = static_cast<std::uint8_t>(ESM::Land::LAND_SIZE);
-        std::generate_n(std::back_inserter(result.mHeights), ESM::Land::LAND_NUM_VERTS, [&]
-        {
-            return distribution(random);
-        });
+        std::generate_n(
+            std::back_inserter(result.mHeights), ESM::Land::LAND_NUM_VERTS, [&] { return distribution(random); });
         result.mOriginalSize = ESM::Land::LAND_SIZE;
         result.mMinX = 0;
         result.mMinY = 0;
         return result;
     }
 
-    template <class Random>
-    FlatHeightfield generateFlatHeightfield(Random& random)
+    FlatHeightfield generateFlatHeightfield(auto& random)
     {
         std::uniform_real_distribution<float> distribution(0.0, 1.0);
         FlatHeightfield result;
@@ -133,40 +129,38 @@ namespace
         return result;
     }
 
-    template <class Random>
-    Key generateKey(std::size_t triangles, Random& random)
+    Key generateKey(std::size_t triangles, auto& random)
     {
         const CollisionShapeType agentShapeType = CollisionShapeType::Aabb;
         const osg::Vec3f agentHalfExtents = generateAgentHalfExtents(0.5, 1.5, random);
         const TilePosition tilePosition = generateVec2i(10000, random);
-        const std::size_t generation = std::uniform_int_distribution<std::size_t>(0, 100)(random);
-        const std::size_t revision = std::uniform_int_distribution<std::size_t>(0, 10000)(random);
+        const Version version{
+            .mGeneration = std::uniform_int_distribution<std::size_t>(0, 100)(random),
+            .mRevision = std::uniform_int_distribution<std::size_t>(0, 10000)(random),
+        };
         Mesh mesh = generateMesh(triangles, random);
         std::vector<CellWater> water;
         generateWater(std::back_inserter(water), 1, random);
-        RecastMesh recastMesh(generation, revision, std::move(mesh), std::move(water),
-                              {generateHeightfield(random)}, {generateFlatHeightfield(random)}, {});
-        return Key {AgentBounds {agentShapeType, agentHalfExtents}, tilePosition, std::move(recastMesh)};
+        RecastMesh recastMesh(version, std::move(mesh), std::move(water), { generateHeightfield(random) },
+            { generateFlatHeightfield(random) }, {});
+        return Key{ AgentBounds{ agentShapeType, agentHalfExtents }, tilePosition, std::move(recastMesh) };
     }
 
     constexpr std::size_t trianglesPerTile = 239;
 
-    template <typename OutputIterator, typename Random>
-    void generateKeys(OutputIterator out, std::size_t count, Random& random)
+    void generateKeys(std::output_iterator<Key> auto out, std::size_t count, auto& random)
     {
         std::generate_n(out, count, [&] { return generateKey(trianglesPerTile, random); });
     }
 
-    template <typename OutputIterator, typename Random>
-    void fillCache(OutputIterator out, Random& random, NavMeshTilesCache& cache)
+    void fillCache(std::output_iterator<Key> auto out, auto& random, NavMeshTilesCache& cache)
     {
         std::size_t size = cache.getStats().mNavMeshCacheSize;
 
         while (true)
         {
             Key key = generateKey(trianglesPerTile, random);
-            cache.set(key.mAgentBounds, key.mTilePosition, key.mRecastMesh,
-                      std::make_unique<PreparedNavMeshData>());
+            cache.set(key.mAgentBounds, key.mTilePosition, key.mRecastMesh, std::make_unique<PreparedNavMeshData>());
             *out++ = std::move(key);
             const std::size_t newSize = cache.getStats().mNavMeshCacheSize;
             if (size >= newSize)
@@ -185,10 +179,10 @@ namespace
         generateKeys(std::back_inserter(keys), keys.size() * (100 - hitPercentage) / 100, random);
         std::size_t n = 0;
 
-        while (state.KeepRunning())
+        for ([[maybe_unused]] auto _ : state)
         {
             const auto& key = keys[n++ % keys.size()];
-            const auto result = cache.get(key.mAgentBounds, key.mTilePosition, key.mRecastMesh);
+            auto result = cache.get(key.mAgentBounds, key.mTilePosition, key.mRecastMesh);
             benchmark::DoNotOptimize(result);
         }
     }
@@ -247,8 +241,8 @@ namespace
         while (state.KeepRunning())
         {
             const auto& key = keys[n++ % keys.size()];
-            const auto result = cache.set(key.mAgentBounds, key.mTilePosition, key.mRecastMesh,
-                                          std::make_unique<PreparedNavMeshData>());
+            auto result = cache.set(
+                key.mAgentBounds, key.mTilePosition, key.mRecastMesh, std::make_unique<PreparedNavMeshData>());
             benchmark::DoNotOptimize(result);
         }
     }
