@@ -1,10 +1,13 @@
 #include "labels.hpp"
 
+#include <components/esm3/dialoguecondition.hpp>
+#include <components/esm3/loadalch.hpp>
 #include <components/esm3/loadbody.hpp>
 #include <components/esm3/loadcell.hpp>
 #include <components/esm3/loadcont.hpp>
 #include <components/esm3/loadcrea.hpp>
 #include <components/esm3/loadench.hpp>
+#include <components/esm3/loadland.hpp>
 #include <components/esm3/loadlevlist.hpp>
 #include <components/esm3/loadligh.hpp>
 #include <components/esm3/loadmgef.hpp>
@@ -13,13 +16,13 @@
 #include <components/esm3/loadspel.hpp>
 #include <components/esm3/loadweap.hpp>
 
-#include <components/misc/stringops.hpp>
+#include <components/misc/strings/format.hpp>
 
-std::string bodyPartLabel(int idx)
+std::string_view bodyPartLabel(int idx)
 {
     if (idx >= 0 && idx <= 26)
     {
-        static const char *bodyPartLabels[] =  {
+        static constexpr std::string_view bodyPartLabels[] = {
             "Head",
             "Hair",
             "Neck",
@@ -46,7 +49,7 @@ std::string bodyPartLabel(int idx)
             "Right Shoulder",
             "Left Shoulder",
             "Weapon",
-            "Tail"
+            "Tail",
         };
         return bodyPartLabels[idx];
     }
@@ -54,11 +57,11 @@ std::string bodyPartLabel(int idx)
         return "Invalid";
 }
 
-std::string meshPartLabel(int idx)
+std::string_view meshPartLabel(int idx)
 {
     if (idx >= 0 && idx <= ESM::BodyPart::MP_Tail)
     {
-        static const char *meshPartLabels[] =  {
+        static constexpr std::string_view meshPartLabels[] = {
             "Head",
             "Hair",
             "Neck",
@@ -73,7 +76,7 @@ std::string meshPartLabel(int idx)
             "Knee",
             "Upper Leg",
             "Clavicle",
-            "Tail"
+            "Tail",
         };
         return meshPartLabels[idx];
     }
@@ -81,14 +84,14 @@ std::string meshPartLabel(int idx)
         return "Invalid";
 }
 
-std::string meshTypeLabel(int idx)
+std::string_view meshTypeLabel(int idx)
 {
     if (idx >= 0 && idx <= ESM::BodyPart::MT_Armor)
     {
-        static const char *meshTypeLabels[] =  {
+        static constexpr std::string_view meshTypeLabels[] = {
             "Skin",
             "Clothing",
-            "Armor"
+            "Armor",
         };
         return meshTypeLabels[idx];
     }
@@ -96,11 +99,11 @@ std::string meshTypeLabel(int idx)
         return "Invalid";
 }
 
-std::string clothingTypeLabel(int idx)
+std::string_view clothingTypeLabel(int idx)
 {
     if (idx >= 0 && idx <= 9)
     {
-        static const char *clothingTypeLabels[] = {
+        static constexpr std::string_view clothingTypeLabels[] = {
             "Pants",
             "Shoes",
             "Shirt",
@@ -110,7 +113,7 @@ std::string clothingTypeLabel(int idx)
             "Left Glove",
             "Skirt",
             "Ring",
-            "Amulet"
+            "Amulet",
         };
         return clothingTypeLabels[idx];
     }
@@ -118,11 +121,11 @@ std::string clothingTypeLabel(int idx)
         return "Invalid";
 }
 
-std::string armorTypeLabel(int idx)
+std::string_view armorTypeLabel(int idx)
 {
     if (idx >= 0 && idx <= 10)
     {
-        static const char *armorTypeLabels[] = {
+        static constexpr std::string_view armorTypeLabels[] = {
             "Helmet",
             "Cuirass",
             "Left Pauldron",
@@ -133,7 +136,7 @@ std::string armorTypeLabel(int idx)
             "Right Gauntlet",
             "Shield",
             "Left Bracer",
-            "Right Bracer"
+            "Right Bracer",
         };
         return armorTypeLabels[idx];
     }
@@ -141,16 +144,16 @@ std::string armorTypeLabel(int idx)
         return "Invalid";
 }
 
-std::string dialogTypeLabel(int idx)
+std::string_view dialogTypeLabel(int idx)
 {
     if (idx >= 0 && idx <= 4)
     {
-        static const char *dialogTypeLabels[] = {
+        static constexpr std::string_view dialogTypeLabels[] = {
             "Topic",
             "Voice",
             "Greeting",
             "Persuasion",
-            "Journal"
+            "Journal",
         };
         return dialogTypeLabels[idx];
     }
@@ -160,16 +163,16 @@ std::string dialogTypeLabel(int idx)
         return "Invalid";
 }
 
-std::string questStatusLabel(int idx)
+std::string_view questStatusLabel(int idx)
 {
     if (idx >= 0 && idx <= 4)
     {
-        static const char *questStatusLabels[] = {
+        static constexpr std::string_view questStatusLabels[] = {
             "None",
             "Name",
             "Finished",
             "Restart",
-            "Deleted"
+            "Deleted",
         };
         return questStatusLabels[idx];
     }
@@ -177,11 +180,11 @@ std::string questStatusLabel(int idx)
         return "Invalid";
 }
 
-std::string creatureTypeLabel(int idx)
+std::string_view creatureTypeLabel(int idx)
 {
     if (idx >= 0 && idx <= 3)
     {
-        static const char *creatureTypeLabels[] = {
+        static constexpr std::string_view creatureTypeLabels[] = {
             "Creature",
             "Daedra",
             "Undead",
@@ -193,11 +196,11 @@ std::string creatureTypeLabel(int idx)
         return "Invalid";
 }
 
-std::string soundTypeLabel(int idx)
+std::string_view soundTypeLabel(int idx)
 {
     if (idx >= 0 && idx <= 7)
     {
-        static const char *soundTypeLabels[] = {
+        static constexpr std::string_view soundTypeLabels[] = {
             "Left Foot",
             "Right Foot",
             "Swim Left",
@@ -205,7 +208,7 @@ std::string soundTypeLabel(int idx)
             "Moan",
             "Roar",
             "Scream",
-            "Land"
+            "Land",
         };
         return soundTypeLabels[idx];
     }
@@ -213,11 +216,11 @@ std::string soundTypeLabel(int idx)
         return "Invalid";
 }
 
-std::string weaponTypeLabel(int idx)
+std::string_view weaponTypeLabel(int idx)
 {
     if (idx >= 0 && idx <= 13)
     {
-        static const char *weaponTypeLabels[] = {
+        static constexpr std::string_view weaponTypeLabels[] = {
             "Short Blade One Hand",
             "Long Blade One Hand",
             "Long Blade Two Hand",
@@ -231,7 +234,7 @@ std::string weaponTypeLabel(int idx)
             "Marksman Crossbow",
             "Marksman Thrown",
             "Arrow",
-            "Bolt"
+            "Bolt",
         };
         return weaponTypeLabels[idx];
     }
@@ -239,21 +242,29 @@ std::string weaponTypeLabel(int idx)
         return "Invalid";
 }
 
-std::string aiTypeLabel(int type)
+std::string_view aiTypeLabel(ESM::AiPackageType type)
 {
-    if (type == ESM::AI_Wander) return "Wander";
-    else if (type == ESM::AI_Travel) return "Travel";
-    else if (type == ESM::AI_Follow) return "Follow";
-    else if (type == ESM::AI_Escort) return "Escort";
-    else if (type == ESM::AI_Activate) return "Activate";
-    else return "Invalid";
+    switch (type)
+    {
+        case ESM::AI_Wander:
+            return "Wander";
+        case ESM::AI_Travel:
+            return "Travel";
+        case ESM::AI_Follow:
+            return "Follow";
+        case ESM::AI_Escort:
+            return "Escort";
+        case ESM::AI_Activate:
+            return "Activate";
+    }
+    return "Invalid";
 }
 
-std::string magicEffectLabel(int idx)
+std::string_view magicEffectLabel(int idx)
 {
     if (idx >= 0 && idx <= 142)
     {
-        const char* magicEffectLabels [] = {
+        static constexpr std::string_view magicEffectLabels[] = {
             "Water Breathing",
             "Swift Swim",
             "Water Walking",
@@ -396,7 +407,7 @@ std::string magicEffectLabel(int idx)
             "sEffectSummonCreature02",
             "sEffectSummonCreature03",
             "sEffectSummonCreature04",
-            "sEffectSummonCreature05"
+            "sEffectSummonCreature05",
         };
         return magicEffectLabels[idx];
     }
@@ -404,11 +415,11 @@ std::string magicEffectLabel(int idx)
         return "Invalid";
 }
 
-std::string attributeLabel(int idx)
+std::string_view attributeLabel(int idx)
 {
     if (idx >= 0 && idx <= 7)
     {
-        const char* attributeLabels [] = {
+        static constexpr std::string_view attributeLabels[] = {
             "Strength",
             "Intelligence",
             "Willpower",
@@ -416,7 +427,7 @@ std::string attributeLabel(int idx)
             "Speed",
             "Endurance",
             "Personality",
-            "Luck"
+            "Luck",
         };
         return attributeLabels[idx];
     }
@@ -424,17 +435,17 @@ std::string attributeLabel(int idx)
         return "Invalid";
 }
 
-std::string spellTypeLabel(int idx)
+std::string_view spellTypeLabel(int idx)
 {
     if (idx >= 0 && idx <= 5)
     {
-        const char* spellTypeLabels [] = {
+        static constexpr std::string_view spellTypeLabels[] = {
             "Spells",
             "Abilities",
             "Blight Disease",
             "Disease",
             "Curse",
-            "Powers"
+            "Powers",
         };
         return spellTypeLabels[idx];
     }
@@ -442,14 +453,14 @@ std::string spellTypeLabel(int idx)
         return "Invalid";
 }
 
-std::string specializationLabel(int idx)
+std::string_view specializationLabel(int idx)
 {
     if (idx >= 0 && idx <= 2)
     {
-        const char* specializationLabels [] = {
+        static constexpr std::string_view specializationLabels[] = {
             "Combat",
             "Magic",
-            "Stealth"
+            "Stealth",
         };
         return specializationLabels[idx];
     }
@@ -457,11 +468,11 @@ std::string specializationLabel(int idx)
         return "Invalid";
 }
 
-std::string skillLabel(int idx)
+std::string_view skillLabel(int idx)
 {
     if (idx >= 0 && idx <= 26)
     {
-        const char* skillLabels [] = {
+        static constexpr std::string_view skillLabels[] = {
             "Block",
             "Armorer",
             "Medium Armor",
@@ -488,7 +499,7 @@ std::string skillLabel(int idx)
             "Marksman",
             "Mercantile",
             "Speechcraft",
-            "Hand-to-hand"
+            "Hand-to-hand",
         };
         return skillLabels[idx];
     }
@@ -496,11 +507,11 @@ std::string skillLabel(int idx)
         return "Invalid";
 }
 
-std::string apparatusTypeLabel(int idx)
+std::string_view apparatusTypeLabel(int idx)
 {
     if (idx >= 0 && idx <= 3)
     {
-        const char* apparatusTypeLabels [] = {
+        static constexpr std::string_view apparatusTypeLabels[] = {
             "Mortar",
             "Alembic",
             "Calcinator",
@@ -512,14 +523,14 @@ std::string apparatusTypeLabel(int idx)
         return "Invalid";
 }
 
-std::string rangeTypeLabel(int idx)
+std::string_view rangeTypeLabel(int idx)
 {
     if (idx >= 0 && idx <= 2)
     {
-        const char* rangeTypeLabels [] = {
+        static constexpr std::string_view rangeTypeLabels[] = {
             "Self",
             "Touch",
-            "Target"
+            "Target",
         };
         return rangeTypeLabels[idx];
     }
@@ -527,17 +538,17 @@ std::string rangeTypeLabel(int idx)
         return "Invalid";
 }
 
-std::string schoolLabel(int idx)
+std::string_view schoolLabel(int idx)
 {
     if (idx >= 0 && idx <= 5)
     {
-        const char* schoolLabels [] = {
+        static constexpr std::string_view schoolLabels[] = {
             "Alteration",
             "Conjuration",
             "Destruction",
             "Illusion",
             "Mysticism",
-            "Restoration"
+            "Restoration",
         };
         return schoolLabels[idx];
     }
@@ -545,15 +556,15 @@ std::string schoolLabel(int idx)
         return "Invalid";
 }
 
-std::string enchantTypeLabel(int idx)
+std::string_view enchantTypeLabel(int idx)
 {
     if (idx >= 0 && idx <= 3)
     {
-        const char* enchantTypeLabels [] = {
+        static constexpr std::string_view enchantTypeLabels[] = {
             "Cast Once",
             "Cast When Strikes",
             "Cast When Used",
-            "Constant Effect"
+            "Constant Effect",
         };
         return enchantTypeLabels[idx];
     }
@@ -561,15 +572,16 @@ std::string enchantTypeLabel(int idx)
         return "Invalid";
 }
 
-std::string ruleFunction(int idx)
+std::string_view ruleFunction(int idx)
 {
-    if (idx >= 0 && idx <= 72)
+    if (idx >= ESM::DialogueCondition::Function_FacReactionLowest
+        && idx <= ESM::DialogueCondition::Function_PcWerewolfKills)
     {
-        std::string ruleFunctions[] = {
-            "Reaction Low",
-            "Reaction High",
+        static constexpr std::string_view ruleFunctions[] = {
+            "Lowest Faction Reaction",
+            "Highest Faction Reaction",
             "Rank Requirement",
-            "NPC? Reputation",
+            "NPC Reputation",
             "Health Percent",
             "Player Reputation",
             "NPC Level",
@@ -638,7 +650,8 @@ std::string ruleFunction(int idx)
             "Alarm",
             "Flee",
             "Should Attack",
-            "Werewolf"
+            "Werewolf",
+            "Werewolf Kills",
         };
         return ruleFunctions[idx];
     }
@@ -653,13 +666,15 @@ std::string ruleFunction(int idx)
 std::string bodyPartFlags(int flags)
 {
     std::string properties;
-    if (flags == 0) properties += "[None] ";
-    if (flags & ESM::BodyPart::BPF_Female) properties += "Female ";
-    if (flags & ESM::BodyPart::BPF_NotPlayable) properties += "NotPlayable ";
-    int unused = (0xFFFFFFFF ^
-                  (ESM::BodyPart::BPF_Female|
-                   ESM::BodyPart::BPF_NotPlayable));
-    if (flags & unused) properties += "Invalid ";
+    if (flags == 0)
+        properties += "[None] ";
+    if (flags & ESM::BodyPart::BPF_Female)
+        properties += "Female ";
+    if (flags & ESM::BodyPart::BPF_NotPlayable)
+        properties += "NotPlayable ";
+    int unused = (0xFFFFFFFF ^ (ESM::BodyPart::BPF_Female | ESM::BodyPart::BPF_NotPlayable));
+    if (flags & unused)
+        properties += "Invalid ";
     properties += Misc::StringUtils::format("(0x%08X)", flags);
     return properties;
 }
@@ -667,20 +682,23 @@ std::string bodyPartFlags(int flags)
 std::string cellFlags(int flags)
 {
     std::string properties;
-    if (flags == 0) properties += "[None] ";
-    if (flags & ESM::Cell::HasWater) properties += "HasWater ";
-    if (flags & ESM::Cell::Interior) properties += "Interior ";
-    if (flags & ESM::Cell::NoSleep) properties += "NoSleep ";
-    if (flags & ESM::Cell::QuasiEx) properties += "QuasiEx ";
+    if (flags == 0)
+        properties += "[None] ";
+    if (flags & ESM::Cell::HasWater)
+        properties += "HasWater ";
+    if (flags & ESM::Cell::Interior)
+        properties += "Interior ";
+    if (flags & ESM::Cell::NoSleep)
+        properties += "NoSleep ";
+    if (flags & ESM::Cell::QuasiEx)
+        properties += "QuasiEx ";
     // This used value is not in the ESM component.
-    if (flags & 0x00000040) properties += "Unknown ";
-    int unused = (0xFFFFFFFF ^
-                  (ESM::Cell::HasWater|
-                   ESM::Cell::Interior|
-                   ESM::Cell::NoSleep|
-                   ESM::Cell::QuasiEx|
-                   0x00000040));
-    if (flags & unused) properties += "Invalid ";
+    if (flags & 0x00000040)
+        properties += "Unknown ";
+    int unused = (0xFFFFFFFF
+        ^ (ESM::Cell::HasWater | ESM::Cell::Interior | ESM::Cell::NoSleep | ESM::Cell::QuasiEx | 0x00000040));
+    if (flags & unused)
+        properties += "Invalid ";
     properties += Misc::StringUtils::format("(0x%08X)", flags);
     return properties;
 }
@@ -688,15 +706,17 @@ std::string cellFlags(int flags)
 std::string containerFlags(int flags)
 {
     std::string properties;
-    if (flags == 0) properties += "[None] ";
-    if (flags & ESM::Container::Unknown) properties += "Unknown ";
-    if (flags & ESM::Container::Organic) properties += "Organic ";
-    if (flags & ESM::Container::Respawn) properties += "Respawn ";
-    int unused = (0xFFFFFFFF ^
-                  (ESM::Container::Unknown|
-                   ESM::Container::Organic|
-                   ESM::Container::Respawn));
-    if (flags & unused) properties += "Invalid ";
+    if (flags == 0)
+        properties += "[None] ";
+    if (flags & ESM::Container::Unknown)
+        properties += "Unknown ";
+    if (flags & ESM::Container::Organic)
+        properties += "Organic ";
+    if (flags & ESM::Container::Respawn)
+        properties += "Respawn ";
+    int unused = (0xFFFFFFFF ^ (ESM::Container::Unknown | ESM::Container::Organic | ESM::Container::Respawn));
+    if (flags & unused)
+        properties += "Invalid ";
     properties += Misc::StringUtils::format("(0x%08X)", flags);
     return properties;
 }
@@ -704,25 +724,29 @@ std::string containerFlags(int flags)
 std::string creatureFlags(int flags)
 {
     std::string properties;
-    if (flags == 0) properties += "[None] ";
-    if (flags & ESM::Creature::Base) properties += "Base ";
-    if (flags & ESM::Creature::Walks) properties += "Walks ";
-    if (flags & ESM::Creature::Swims) properties += "Swims ";
-    if (flags & ESM::Creature::Flies) properties += "Flies ";
-    if (flags & ESM::Creature::Bipedal) properties += "Bipedal ";
-    if (flags & ESM::Creature::Respawn) properties += "Respawn ";
-    if (flags & ESM::Creature::Weapon) properties += "Weapon ";
-    if (flags & ESM::Creature::Essential) properties += "Essential ";
-    int unused = (0xFFFFFFFF ^
-                  (ESM::Creature::Base|
-                   ESM::Creature::Walks|
-                   ESM::Creature::Swims|
-                   ESM::Creature::Flies|
-                   ESM::Creature::Bipedal|
-                   ESM::Creature::Respawn|
-                   ESM::Creature::Weapon|
-                   ESM::Creature::Essential));
-    if (flags & unused) properties += "Invalid ";
+    if (flags == 0)
+        properties += "[None] ";
+    if (flags & ESM::Creature::Base)
+        properties += "Base ";
+    if (flags & ESM::Creature::Walks)
+        properties += "Walks ";
+    if (flags & ESM::Creature::Swims)
+        properties += "Swims ";
+    if (flags & ESM::Creature::Flies)
+        properties += "Flies ";
+    if (flags & ESM::Creature::Bipedal)
+        properties += "Bipedal ";
+    if (flags & ESM::Creature::Respawn)
+        properties += "Respawn ";
+    if (flags & ESM::Creature::Weapon)
+        properties += "Weapon ";
+    if (flags & ESM::Creature::Essential)
+        properties += "Essential ";
+    int unused = (0xFFFFFFFF
+        ^ (ESM::Creature::Base | ESM::Creature::Walks | ESM::Creature::Swims | ESM::Creature::Flies
+            | ESM::Creature::Bipedal | ESM::Creature::Respawn | ESM::Creature::Weapon | ESM::Creature::Essential));
+    if (flags & unused)
+        properties += "Invalid ";
     properties += Misc::StringUtils::format("(0x%02X)", flags);
     return properties;
 }
@@ -730,24 +754,30 @@ std::string creatureFlags(int flags)
 std::string enchantmentFlags(int flags)
 {
     std::string properties;
-    if (flags == 0) properties += "[None] ";
-    if (flags & ESM::Enchantment::Autocalc) properties += "Autocalc ";
-    if (flags & (0xFFFFFFFF ^ ESM::Enchantment::Autocalc)) properties += "Invalid ";
+    if (flags == 0)
+        properties += "[None] ";
+    if (flags & ESM::Enchantment::Autocalc)
+        properties += "Autocalc ";
+    if (flags & (0xFFFFFFFF ^ ESM::Enchantment::Autocalc))
+        properties += "Invalid ";
     properties += Misc::StringUtils::format("(0x%08X)", flags);
     return properties;
 }
 
-std::string landFlags(int flags)
+std::string landFlags(std::uint32_t flags)
 {
     std::string properties;
-    // The ESM component says that this first four bits are used, but
-    // only the first three bits are used as far as I can tell.
-    // There's also no enumeration of the bit in the ESM component.
-    if (flags == 0) properties += "[None] ";
-    if (flags & 0x00000001) properties += "Unknown1 ";
-    if (flags & 0x00000004) properties += "Unknown3 ";
-    if (flags & 0x00000002) properties += "Unknown2 ";
-    if (flags & 0xFFFFFFF8) properties += "Invalid ";
+    if (flags == 0)
+        properties += "[None] ";
+    if (flags & ESM::Land::Flag_HeightsNormals)
+        properties += "HeightsNormals ";
+    if (flags & ESM::Land::Flag_Colors)
+        properties += "Colors ";
+    if (flags & ESM::Land::Flag_Textures)
+        properties += "Textures ";
+    int unused = 0xFFFFFFFF ^ (ESM::Land::Flag_HeightsNormals | ESM::Land::Flag_Colors | ESM::Land::Flag_Textures);
+    if (flags & unused)
+        properties += "Invalid ";
     properties += Misc::StringUtils::format("(0x%08X)", flags);
     return properties;
 }
@@ -755,13 +785,15 @@ std::string landFlags(int flags)
 std::string itemListFlags(int flags)
 {
     std::string properties;
-    if (flags == 0) properties += "[None] ";
-    if (flags & ESM::ItemLevList::AllLevels) properties += "AllLevels ";
-    if (flags & ESM::ItemLevList::Each) properties += "Each ";
-    int unused = (0xFFFFFFFF ^
-                  (ESM::ItemLevList::AllLevels|
-                   ESM::ItemLevList::Each));
-    if (flags & unused) properties += "Invalid ";
+    if (flags == 0)
+        properties += "[None] ";
+    if (flags & ESM::ItemLevList::AllLevels)
+        properties += "AllLevels ";
+    if (flags & ESM::ItemLevList::Each)
+        properties += "Each ";
+    int unused = (0xFFFFFFFF ^ (ESM::ItemLevList::AllLevels | ESM::ItemLevList::Each));
+    if (flags & unused)
+        properties += "Invalid ";
     properties += Misc::StringUtils::format("(0x%08X)", flags);
     return properties;
 }
@@ -769,10 +801,13 @@ std::string itemListFlags(int flags)
 std::string creatureListFlags(int flags)
 {
     std::string properties;
-    if (flags == 0) properties += "[None] ";
-    if (flags & ESM::CreatureLevList::AllLevels) properties += "AllLevels ";
+    if (flags == 0)
+        properties += "[None] ";
+    if (flags & ESM::CreatureLevList::AllLevels)
+        properties += "AllLevels ";
     int unused = (0xFFFFFFFF ^ ESM::CreatureLevList::AllLevels);
-    if (flags & unused) properties += "Invalid ";
+    if (flags & unused)
+        properties += "Invalid ";
     properties += Misc::StringUtils::format("(0x%08X)", flags);
     return properties;
 }
@@ -780,27 +815,31 @@ std::string creatureListFlags(int flags)
 std::string lightFlags(int flags)
 {
     std::string properties;
-    if (flags == 0) properties += "[None] ";
-    if (flags & ESM::Light::Dynamic) properties += "Dynamic ";
-    if (flags & ESM::Light::Fire) properties += "Fire ";
-    if (flags & ESM::Light::Carry) properties += "Carry ";
-    if (flags & ESM::Light::Flicker) properties += "Flicker ";
-    if (flags & ESM::Light::FlickerSlow) properties += "FlickerSlow ";
-    if (flags & ESM::Light::Pulse) properties += "Pulse ";
-    if (flags & ESM::Light::PulseSlow) properties += "PulseSlow ";
-    if (flags & ESM::Light::Negative) properties += "Negative ";
-    if (flags & ESM::Light::OffDefault) properties += "OffDefault ";
-    int unused = (0xFFFFFFFF ^
-                  (ESM::Light::Dynamic|
-                   ESM::Light::Fire|
-                   ESM::Light::Carry|
-                   ESM::Light::Flicker|
-                   ESM::Light::FlickerSlow|
-                   ESM::Light::Pulse|
-                   ESM::Light::PulseSlow|
-                   ESM::Light::Negative|
-                   ESM::Light::OffDefault));
-    if (flags & unused) properties += "Invalid ";
+    if (flags == 0)
+        properties += "[None] ";
+    if (flags & ESM::Light::Dynamic)
+        properties += "Dynamic ";
+    if (flags & ESM::Light::Fire)
+        properties += "Fire ";
+    if (flags & ESM::Light::Carry)
+        properties += "Carry ";
+    if (flags & ESM::Light::Flicker)
+        properties += "Flicker ";
+    if (flags & ESM::Light::FlickerSlow)
+        properties += "FlickerSlow ";
+    if (flags & ESM::Light::Pulse)
+        properties += "Pulse ";
+    if (flags & ESM::Light::PulseSlow)
+        properties += "PulseSlow ";
+    if (flags & ESM::Light::Negative)
+        properties += "Negative ";
+    if (flags & ESM::Light::OffDefault)
+        properties += "OffDefault ";
+    int unused = (0xFFFFFFFF
+        ^ (ESM::Light::Dynamic | ESM::Light::Fire | ESM::Light::Carry | ESM::Light::Flicker | ESM::Light::FlickerSlow
+            | ESM::Light::Pulse | ESM::Light::PulseSlow | ESM::Light::Negative | ESM::Light::OffDefault));
+    if (flags & unused)
+        properties += "Invalid ";
     properties += Misc::StringUtils::format("(0x%08X)", flags);
     return properties;
 }
@@ -808,27 +847,47 @@ std::string lightFlags(int flags)
 std::string magicEffectFlags(int flags)
 {
     std::string properties;
-    if (flags == 0) properties += "[None] ";
-    if (flags & ESM::MagicEffect::TargetAttribute) properties += "TargetAttribute ";
-    if (flags & ESM::MagicEffect::TargetSkill) properties += "TargetSkill ";
-    if (flags & ESM::MagicEffect::NoDuration) properties += "NoDuration ";
-    if (flags & ESM::MagicEffect::NoMagnitude) properties += "NoMagnitude ";
-    if (flags & ESM::MagicEffect::Harmful) properties += "Harmful ";
-    if (flags & ESM::MagicEffect::ContinuousVfx) properties += "ContinuousVFX ";
-    if (flags & ESM::MagicEffect::CastSelf) properties += "CastSelf ";
-    if (flags & ESM::MagicEffect::CastTouch) properties += "CastTouch ";
-    if (flags & ESM::MagicEffect::CastTarget) properties += "CastTarget ";
-    if (flags & ESM::MagicEffect::AppliedOnce) properties += "AppliedOnce ";
-    if (flags & ESM::MagicEffect::Stealth) properties += "Stealth ";
-    if (flags & ESM::MagicEffect::NonRecastable) properties += "NonRecastable ";
-    if (flags & ESM::MagicEffect::IllegalDaedra) properties += "IllegalDaedra ";
-    if (flags & ESM::MagicEffect::Unreflectable) properties += "Unreflectable ";
-    if (flags & ESM::MagicEffect::CasterLinked) properties += "CasterLinked ";
-    if (flags & ESM::MagicEffect::AllowSpellmaking) properties += "AllowSpellmaking ";
-    if (flags & ESM::MagicEffect::AllowEnchanting) properties += "AllowEnchanting ";
-    if (flags & ESM::MagicEffect::NegativeLight) properties += "NegativeLight ";
+    if (flags == 0)
+        properties += "[None] ";
+    if (flags & ESM::MagicEffect::TargetAttribute)
+        properties += "TargetAttribute ";
+    if (flags & ESM::MagicEffect::TargetSkill)
+        properties += "TargetSkill ";
+    if (flags & ESM::MagicEffect::NoDuration)
+        properties += "NoDuration ";
+    if (flags & ESM::MagicEffect::NoMagnitude)
+        properties += "NoMagnitude ";
+    if (flags & ESM::MagicEffect::Harmful)
+        properties += "Harmful ";
+    if (flags & ESM::MagicEffect::ContinuousVfx)
+        properties += "ContinuousVFX ";
+    if (flags & ESM::MagicEffect::CastSelf)
+        properties += "CastSelf ";
+    if (flags & ESM::MagicEffect::CastTouch)
+        properties += "CastTouch ";
+    if (flags & ESM::MagicEffect::CastTarget)
+        properties += "CastTarget ";
+    if (flags & ESM::MagicEffect::AppliedOnce)
+        properties += "AppliedOnce ";
+    if (flags & ESM::MagicEffect::Stealth)
+        properties += "Stealth ";
+    if (flags & ESM::MagicEffect::NonRecastable)
+        properties += "NonRecastable ";
+    if (flags & ESM::MagicEffect::IllegalDaedra)
+        properties += "IllegalDaedra ";
+    if (flags & ESM::MagicEffect::Unreflectable)
+        properties += "Unreflectable ";
+    if (flags & ESM::MagicEffect::CasterLinked)
+        properties += "CasterLinked ";
+    if (flags & ESM::MagicEffect::AllowSpellmaking)
+        properties += "AllowSpellmaking ";
+    if (flags & ESM::MagicEffect::AllowEnchanting)
+        properties += "AllowEnchanting ";
+    if (flags & ESM::MagicEffect::NegativeLight)
+        properties += "NegativeLight ";
 
-    if (flags & 0xFFFC0000) properties += "Invalid ";
+    if (flags & 0xFFFC0000)
+        properties += "Invalid ";
     properties += Misc::StringUtils::format("(0x%08X)", flags);
     return properties;
 }
@@ -836,21 +895,24 @@ std::string magicEffectFlags(int flags)
 std::string npcFlags(int flags)
 {
     std::string properties;
-    if (flags == 0) properties += "[None] ";
-    if (flags & ESM::NPC::Base) properties += "Base ";
-    if (flags & ESM::NPC::Autocalc) properties += "Autocalc ";
-    if (flags & ESM::NPC::Female) properties += "Female ";
-    if (flags & ESM::NPC::Respawn) properties += "Respawn ";
-    if (flags & ESM::NPC::Essential) properties += "Essential ";
+    if (flags == 0)
+        properties += "[None] ";
+    if (flags & ESM::NPC::Base)
+        properties += "Base ";
+    if (flags & ESM::NPC::Autocalc)
+        properties += "Autocalc ";
+    if (flags & ESM::NPC::Female)
+        properties += "Female ";
+    if (flags & ESM::NPC::Respawn)
+        properties += "Respawn ";
+    if (flags & ESM::NPC::Essential)
+        properties += "Essential ";
     // Whether corpses persist is a bit that is unaccounted for,
     // however relatively few NPCs have this bit set.
-    int unused = (0xFF ^
-                  (ESM::NPC::Base|
-                   ESM::NPC::Autocalc|
-                   ESM::NPC::Female|
-                   ESM::NPC::Respawn|
-                   ESM::NPC::Essential));
-    if (flags & unused) properties += "Invalid ";
+    int unused
+        = (0xFF ^ (ESM::NPC::Base | ESM::NPC::Autocalc | ESM::NPC::Female | ESM::NPC::Respawn | ESM::NPC::Essential));
+    if (flags & unused)
+        properties += "Invalid ";
     properties += Misc::StringUtils::format("(0x%02X)", flags);
     return properties;
 }
@@ -858,14 +920,16 @@ std::string npcFlags(int flags)
 std::string raceFlags(int flags)
 {
     std::string properties;
-    if (flags == 0) properties += "[None] ";
+    if (flags == 0)
+        properties += "[None] ";
     // All races have the playable flag set in Bethesda files.
-    if (flags & ESM::Race::Playable) properties += "Playable ";
-    if (flags & ESM::Race::Beast) properties += "Beast ";
-    int unused = (0xFFFFFFFF ^
-                  (ESM::Race::Playable|
-                   ESM::Race::Beast));
-    if (flags & unused) properties += "Invalid ";
+    if (flags & ESM::Race::Playable)
+        properties += "Playable ";
+    if (flags & ESM::Race::Beast)
+        properties += "Beast ";
+    int unused = (0xFFFFFFFF ^ (ESM::Race::Playable | ESM::Race::Beast));
+    if (flags & unused)
+        properties += "Invalid ";
     properties += Misc::StringUtils::format("(0x%08X)", flags);
     return properties;
 }
@@ -873,15 +937,17 @@ std::string raceFlags(int flags)
 std::string spellFlags(int flags)
 {
     std::string properties;
-    if (flags == 0) properties += "[None] ";
-    if (flags & ESM::Spell::F_Autocalc) properties += "Autocalc ";
-    if (flags & ESM::Spell::F_PCStart) properties += "PCStart ";
-    if (flags & ESM::Spell::F_Always) properties += "Always ";
-    int unused = (0xFFFFFFFF ^
-                  (ESM::Spell::F_Autocalc|
-                   ESM::Spell::F_PCStart|
-                   ESM::Spell::F_Always));
-    if (flags & unused) properties += "Invalid ";
+    if (flags == 0)
+        properties += "[None] ";
+    if (flags & ESM::Spell::F_Autocalc)
+        properties += "Autocalc ";
+    if (flags & ESM::Spell::F_PCStart)
+        properties += "PCStart ";
+    if (flags & ESM::Spell::F_Always)
+        properties += "Always ";
+    int unused = (0xFFFFFFFF ^ (ESM::Spell::F_Autocalc | ESM::Spell::F_PCStart | ESM::Spell::F_Always));
+    if (flags & unused)
+        properties += "Invalid ";
     properties += Misc::StringUtils::format("(0x%08X)", flags);
     return properties;
 }
@@ -889,16 +955,18 @@ std::string spellFlags(int flags)
 std::string weaponFlags(int flags)
 {
     std::string properties;
-    if (flags == 0) properties += "[None] ";
+    if (flags == 0)
+        properties += "[None] ";
     // The interpretation of the flags are still unclear to me.
     // Apparently you can't be Silver without being Magical?  Many of
     // the "Magical" weapons don't have enchantments of any sort.
-    if (flags & ESM::Weapon::Magical) properties += "Magical ";
-    if (flags & ESM::Weapon::Silver) properties += "Silver ";
-    int unused = (0xFFFFFFFF ^
-                  (ESM::Weapon::Magical|
-                   ESM::Weapon::Silver));
-    if (flags & unused) properties += "Invalid ";
+    if (flags & ESM::Weapon::Magical)
+        properties += "Magical ";
+    if (flags & ESM::Weapon::Silver)
+        properties += "Silver ";
+    int unused = (0xFFFFFFFF ^ (ESM::Weapon::Magical | ESM::Weapon::Silver));
+    if (flags & unused)
+        properties += "Invalid ";
     properties += Misc::StringUtils::format("(0x%08X)", flags);
     return properties;
 }
@@ -906,13 +974,32 @@ std::string weaponFlags(int flags)
 std::string recordFlags(uint32_t flags)
 {
     std::string properties;
-    if (flags == 0) properties += "[None] ";
-    if (flags & ESM::FLAG_Deleted) properties += "Deleted ";
-    if (flags & ESM::FLAG_Persistent) properties += "Persistent ";
-    if (flags & ESM::FLAG_Ignored) properties += "Ignored ";
-    if (flags & ESM::FLAG_Blocked) properties += "Blocked ";
+    if (flags == 0)
+        properties += "[None] ";
+    if (flags & ESM::FLAG_Deleted)
+        properties += "Deleted ";
+    if (flags & ESM::FLAG_Persistent)
+        properties += "Persistent ";
+    if (flags & ESM::FLAG_Ignored)
+        properties += "Ignored ";
+    if (flags & ESM::FLAG_Blocked)
+        properties += "Blocked ";
     int unused = ~(ESM::FLAG_Deleted | ESM::FLAG_Persistent | ESM::FLAG_Ignored | ESM::FLAG_Blocked);
-    if (flags & unused) properties += "Invalid ";
+    if (flags & unused)
+        properties += "Invalid ";
     properties += Misc::StringUtils::format("(0x%08X)", flags);
     return properties;
-}
\ No newline at end of file
+}
+
+std::string potionFlags(int flags)
+{
+    std::string properties;
+    if (flags == 0)
+        properties += "[None] ";
+    if (flags & ESM::Potion::Autocalc)
+        properties += "Autocalc ";
+    if (flags & (0xFFFFFFFF ^ ESM::Enchantment::Autocalc))
+        properties += "Invalid ";
+    properties += Misc::StringUtils::format("(0x%08X)", flags);
+    return properties;
+}
