@@ -5,14 +5,14 @@
 namespace ESSImport
 {
 
-    void DIAL::load(ESM::ESMReader &esm)
+    void DIAL::load(ESM::ESMReader& esm)
     {
         // See ESM::Dialogue::Type enum, not sure why we would need this here though
-        int type = 0;
+        int32_t type = 0;
         esm.getHNOT(type, "DATA");
 
         // Deleted dialogue in a savefile. No clue what this means...
-        int deleted = 0;
+        int32_t deleted = 0;
         esm.getHNOT(deleted, "DELE");
 
         mIndex = 0;
