@@ -1,5 +1,8 @@
 #ifndef OPENMW_ESSIMPORT_IMPORTDIAL_H
 #define OPENMW_ESSIMPORT_IMPORTDIAL_H
+
+#include <cstdint>
+
 namespace ESM
 {
     class ESMReader;
@@ -10,7 +13,7 @@ namespace ESSImport
 
     struct DIAL
     {
-        int mIndex; // Journal index
+        int32_t mIndex; // Journal index
 
         void load(ESM::ESMReader& esm);
     };
