@@ -2,13 +2,13 @@
 
 #include <components/esm3/esmreader.hpp>
 
-
 namespace ESSImport
 {
 
-    void SCPT::load(ESM::ESMReader &esm)
+    void SCPT::load(ESM::ESMReader& esm)
     {
-        esm.getHNT(mSCHD, "SCHD");
+        esm.getHNT("SCHD", mSCHD.mName.mData, mSCHD.mNumShorts, mSCHD.mNumLongs, mSCHD.mNumFloats,
+            mSCHD.mScriptDataSize, mSCHD.mStringTableSize);
 
         mSCRI.load(esm);
 
