@@ -1,10 +1,7 @@
 #ifndef MAINDIALOG_H
 #define MAINDIALOG_H
 
-
 #ifndef Q_MOC_RUN
-#include <components/files/configurationmanager.hpp>
-
 #include <components/process/processinvoker.hpp>
 
 #include <components/config/gamesettings.hpp>
@@ -15,18 +12,21 @@
 
 class QListWidgetItem;
 class QStackedWidget;
-class QStringList;
 class QStringListModel;
 class QString;
 
+namespace Files
+{
+    struct ConfigurationManager;
+}
+
 namespace Launcher
 {
-    class PlayPage;
     class GraphicsPage;
     class DataFilesPage;
     class UnshieldThread;
+    class ImportPage;
     class SettingsPage;
-    class AdvancedPage;
 
     enum FirstRunDialogResult
     {
@@ -44,7 +44,7 @@ namespace Launcher
         Q_OBJECT
 
     public:
-        explicit MainDialog(QWidget *parent = nullptr);
+        explicit MainDialog(const Files::ConfigurationManager& configurationManager, QWidget* parent = nullptr);
         ~MainDialog() override;
 
         FirstRunDialogResult showFirstRunDialog();
@@ -53,10 +53,16 @@ namespace Launcher
         bool writeSettings();
 
     public slots:
-        void changePage(QListWidgetItem *current, QListWidgetItem *previous);
+        void enableDataPage();
+        void enableGraphicsPage();
+        void enableSettingsPage();
+        void enableImportPage();
         void play();
         void help();
 
+    protected:
+        bool event(QEvent* event) override;
+
     private slots:
         void wizardStarted();
         void wizardFinished(int exitCode, QProcess::ExitStatus exitStatus);
@@ -77,25 +83,26 @@ namespace Launcher
         void loadSettings();
         void saveSettings();
 
-        inline bool startProgram(const QString &name, bool detached = false) { return startProgram(name, QStringList(), detached); }
-        bool startProgram(const QString &name, const QStringList &arguments, bool detached = false);
+        inline bool startProgram(const QString& name, bool detached = false)
+        {
+            return startProgram(name, QStringList(), detached);
+        }
+        bool startProgram(const QString& name, const QStringList& arguments, bool detached = false);
 
-        void closeEvent(QCloseEvent *event) override;
+        void closeEvent(QCloseEvent* event) override;
 
-        PlayPage *mPlayPage;
-        GraphicsPage *mGraphicsPage;
-        DataFilesPage *mDataFilesPage;
-        SettingsPage *mSettingsPage;
-        AdvancedPage *mAdvancedPage;
+        GraphicsPage* mGraphicsPage;
+        DataFilesPage* mDataFilesPage;
+        ImportPage* mImportPage;
+        SettingsPage* mSettingsPage;
 
-        Process::ProcessInvoker *mGameInvoker;
-        Process::ProcessInvoker *mWizardInvoker;
+        Process::ProcessInvoker* mGameInvoker;
+        Process::ProcessInvoker* mWizardInvoker;
 
-        Files::ConfigurationManager mCfgMgr;
+        const Files::ConfigurationManager& mCfgMgr;
 
         Config::GameSettings mGameSettings;
         Config::LauncherSettings mLauncherSettings;
-
     };
 }
 #endif
