@@ -1,7 +1,7 @@
 #include <cstring>
 #include <vector>
 
-#include <apps/openmw/mwsound/alext.h>
+#include "apps/openmw/mwsound/alext.h"
 
 #include "openalutil.hpp"
 
@@ -12,9 +12,9 @@
 std::vector<std::string> Launcher::enumerateOpenALDevices()
 {
     std::vector<std::string> devlist;
-    const ALCchar *devnames;
+    const ALCchar* devnames;
 
-    if(alcIsExtensionPresent(nullptr, "ALC_ENUMERATE_ALL_EXT"))
+    if (alcIsExtensionPresent(nullptr, "ALC_ENUMERATE_ALL_EXT"))
     {
         devnames = alcGetString(nullptr, ALC_ALL_DEVICES_SPECIFIER);
     }
@@ -23,10 +23,10 @@ std::vector<std::string> Launcher::enumerateOpenALDevices()
         devnames = alcGetString(nullptr, ALC_DEVICE_SPECIFIER);
     }
 
-    while(devnames && *devnames)
+    while (devnames && *devnames)
     {
         devlist.emplace_back(devnames);
-        devnames += strlen(devnames)+1;
+        devnames += strlen(devnames) + 1;
     }
     return devlist;
 }
@@ -35,10 +35,10 @@ std::vector<std::string> Launcher::enumerateOpenALDevicesHrtf()
 {
     std::vector<std::string> ret;
 
-    ALCdevice *device = alcOpenDevice(nullptr);
-    if(device)
+    ALCdevice* device = alcOpenDevice(nullptr);
+    if (device)
     {
-        if(alcIsExtensionPresent(device, "ALC_SOFT_HRTF"))
+        if (alcIsExtensionPresent(device, "ALC_SOFT_HRTF"))
         {
             LPALCGETSTRINGISOFT alcGetStringiSOFT = nullptr;
             void* funcPtr = alcGetProcAddress(device, "alcGetStringiSOFT");
@@ -46,10 +46,10 @@ std::vector<std::string> Launcher::enumerateOpenALDevicesHrtf()
             ALCint num_hrtf;
             alcGetIntegerv(device, ALC_NUM_HRTF_SPECIFIERS_SOFT, 1, &num_hrtf);
             ret.reserve(num_hrtf);
-            for(ALCint i = 0;i < num_hrtf;++i)
+            for (ALCint i = 0; i < num_hrtf; ++i)
             {
-                const ALCchar *entry = alcGetStringiSOFT(device, ALC_HRTF_SPECIFIER_SOFT, i);
-                if(strcmp(entry, "") == 0)
+                const ALCchar* entry = alcGetStringiSOFT(device, ALC_HRTF_SPECIFIER_SOFT, i);
+                if (strcmp(entry, "") == 0)
                     break;
                 ret.emplace_back(entry);
             }
