@@ -1,31 +1,31 @@
 #include "textinputdialog.hpp"
 
-#include <QDialogButtonBox>
 #include <QApplication>
+#include <QDialogButtonBox>
+#include <QLabel>
 #include <QPushButton>
 #include <QVBoxLayout>
 #include <QValidator>
-#include <QLabel>
 
-Launcher::TextInputDialog::TextInputDialog(const QString& title, const QString &text, QWidget *parent) :
-    QDialog(parent)
+Launcher::TextInputDialog::TextInputDialog(const QString& title, const QString& text, QWidget* parent)
+    : QDialog(parent)
 {
     setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
     mButtonBox = new QDialogButtonBox(this);
     mButtonBox->addButton(QDialogButtonBox::Ok);
     mButtonBox->addButton(QDialogButtonBox::Cancel);
-    mButtonBox->button(QDialogButtonBox::Ok)->setEnabled (false);
+    mButtonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
 
-    auto *label = new QLabel(this);
+    auto* label = new QLabel(this);
     label->setText(text);
 
     // Line edit
-    QValidator *validator = new QRegExpValidator(QRegExp("^[a-zA-Z0-9_]*$"), this); // Alpha-numeric + underscore
+    QValidator* validator = new QRegularExpressionValidator(QRegularExpression("^[a-zA-Z0-9_]*$"), this);
     mLineEdit = new LineEdit(this);
     mLineEdit->setValidator(validator);
     mLineEdit->setCompleter(nullptr);
 
-    auto *dialogLayout = new QVBoxLayout(this);
+    auto* dialogLayout = new QVBoxLayout(this);
     dialogLayout->addWidget(label);
     dialogLayout->addWidget(mLineEdit);
     dialogLayout->addWidget(mButtonBox);
@@ -39,12 +39,8 @@ Launcher::TextInputDialog::TextInputDialog(const QString& title, const QString &
 
     setModal(true);
 
-    connect(mButtonBox, SIGNAL(accepted()), this, SLOT(accept()));
-    connect(mButtonBox, SIGNAL(rejected()), this, SLOT(reject()));
-}
-
-Launcher::TextInputDialog::~TextInputDialog()
-{
+    connect(mButtonBox, &QDialogButtonBox::accepted, this, &TextInputDialog::accept);
+    connect(mButtonBox, &QDialogButtonBox::rejected, this, &TextInputDialog::reject);
 }
 
 int Launcher::TextInputDialog::exec()
@@ -56,15 +52,18 @@ int Launcher::TextInputDialog::exec()
 
 void Launcher::TextInputDialog::setOkButtonEnabled(bool enabled)
 {
-    QPushButton *okButton = mButtonBox->button(QDialogButtonBox::Ok);
+    QPushButton* okButton = mButtonBox->button(QDialogButtonBox::Ok);
     okButton->setEnabled(enabled);
 
     QPalette palette;
     palette.setColor(QPalette::Text, Qt::red);
 
-    if (enabled) {
+    if (enabled)
+    {
         mLineEdit->setPalette(QApplication::palette());
-    } else {
+    }
+    else
+    {
         // Existing profile name, make the text red
         mLineEdit->setPalette(palette);
     }
