@@ -1,8 +1,6 @@
 #ifndef OPENMW_NAVMESHTOOL_NAVMESH_H
 #define OPENMW_NAVMESHTOOL_NAVMESH_H
 
-#include <osg/Vec3f>
-
 #include <cstddef>
 
 namespace DetourNavigator
@@ -23,9 +21,9 @@ namespace NavMeshTool
         NotEnoughSpace,
     };
 
-    Status generateAllNavMeshTiles(const DetourNavigator::AgentBounds& agentBounds, const DetourNavigator::Settings& settings,
-        std::size_t threadsNumber, bool removeUnusedTiles, bool writeBinaryLog, WorldspaceData& cellsData,
-        DetourNavigator::NavMeshDb&& db);
+    Status generateAllNavMeshTiles(const DetourNavigator::AgentBounds& agentBounds,
+        const DetourNavigator::Settings& settings, std::size_t threadsNumber, bool removeUnusedTiles,
+        bool writeBinaryLog, WorldspaceData& cellsData, DetourNavigator::NavMeshDb&& db);
 }
 
 #endif
