@@ -1,8 +1,12 @@
 #include "unarynode.hpp"
 
-CSMFilter::UnaryNode::UnaryNode (std::shared_ptr<Node> child, const std::string& name)
-: mChild (child), mName (name)
-{}
+#include <apps/opencs/model/filter/node.hpp>
+
+CSMFilter::UnaryNode::UnaryNode(std::shared_ptr<Node> child, const std::string& name)
+    : mChild(std::move(child))
+    , mName(name)
+{
+}
 
 const CSMFilter::Node& CSMFilter::UnaryNode::getChild() const
 {
@@ -19,7 +23,7 @@ std::vector<int> CSMFilter::UnaryNode::getReferencedColumns() const
     return mChild->getReferencedColumns();
 }
 
-std::string CSMFilter::UnaryNode::toString (bool numericColumns) const
+std::string CSMFilter::UnaryNode::toString(bool numericColumns) const
 {
-    return mName + " " + mChild->toString (numericColumns);
+    return mName + " " + mChild->toString(numericColumns);
 }
