@@ -5,19 +5,22 @@
 #include <QLabel>
 #include <QMouseEvent>
 #include <QPushButton>
-#include <QWidget>
 #include <QString>
+#include <QWidget>
 
 #include <components/settings/settings.hpp>
 
-#include "state.hpp"
+#include <apps/opencs/model/prefs/category.hpp>
+#include <apps/opencs/model/prefs/setting.hpp>
+
 #include "shortcutmanager.hpp"
+#include "state.hpp"
 
 namespace CSMPrefs
 {
-    ShortcutSetting::ShortcutSetting(Category* parent, QMutex* mutex, const std::string& key,
-        const std::string& label)
-        : Setting(parent, mutex, key, label)
+    ShortcutSetting::ShortcutSetting(
+        Category* parent, QMutex* mutex, std::string_view key, const QString& label, Settings::Index& index)
+        : TypedSetting(parent, mutex, key, label, index)
         , mButton(nullptr)
         , mEditorActive(false)
         , mEditorPos(0)
@@ -28,14 +31,14 @@ namespace CSMPrefs
         }
     }
 
-    std::pair<QWidget*, QWidget*> ShortcutSetting::makeWidgets(QWidget* parent)
+    SettingWidgets ShortcutSetting::makeWidgets(QWidget* parent)
     {
         QKeySequence sequence;
         State::get().getShortcutManager().getSequence(getKey(), sequence);
 
         QString text = QString::fromUtf8(State::get().getShortcutManager().convertToString(sequence).c_str());
 
-        QLabel* label = new QLabel(QString::fromUtf8(getLabel().c_str()), parent);
+        QLabel* label = new QLabel(getLabel(), parent);
         QPushButton* widget = new QPushButton(text, parent);
 
         widget->setCheckable(true);
@@ -46,16 +49,16 @@ namespace CSMPrefs
 
         mButton = widget;
 
-        connect(widget, SIGNAL(toggled(bool)), this, SLOT(buttonToggled(bool)));
+        connect(widget, &QPushButton::toggled, this, &ShortcutSetting::buttonToggled);
 
-        return std::make_pair(label, widget);
+        return SettingWidgets{ .mLabel = label, .mInput = widget };
     }
 
     void ShortcutSetting::updateWidget()
     {
         if (mButton)
         {
-            std::string shortcut = Settings::Manager::getString(getKey(), getParent()->getKey());
+            const std::string shortcut = getValue();
 
             QKeySequence sequence;
             State::get().getShortcutManager().convertFromString(shortcut, sequence);
@@ -115,14 +118,7 @@ namespace CSMPrefs
     bool ShortcutSetting::handleEvent(QObject* target, int mod, int value, bool active)
     {
         // Modifiers are handled differently
-        const int Blacklist[] =
-        {
-            Qt::Key_Shift,
-            Qt::Key_Control,
-            Qt::Key_Meta,
-            Qt::Key_Alt,
-            Qt::Key_AltGr
-        };
+        const int Blacklist[] = { Qt::Key_Shift, Qt::Key_Control, Qt::Key_Meta, Qt::Key_Alt, Qt::Key_AltGr };
 
         const size_t BlacklistSize = sizeof(Blacklist) / sizeof(int);
 
@@ -175,15 +171,7 @@ namespace CSMPrefs
     void ShortcutSetting::storeValue(const QKeySequence& sequence)
     {
         State::get().getShortcutManager().setSequence(getKey(), sequence);
-
-        // Convert to string and assign
-        std::string value = State::get().getShortcutManager().convertToString(sequence);
-
-        {
-            QMutexLocker lock(getMutex());
-            Settings::Manager::setString(getKey(), getParent()->getKey(), value);
-        }
-
+        setValue(State::get().getShortcutManager().convertToString(sequence));
         getParent()->getState()->update(*this);
     }
 
