@@ -1,22 +1,34 @@
 #ifndef CSM_TOOLS_JOURNALCHECK_H
 #define CSM_TOOLS_JOURNALCHECK_H
 
-#include <components/esm3/loaddial.hpp>
-
 #include "../world/idcollection.hpp"
 #include "../world/infocollection.hpp"
 
 #include "../doc/stage.hpp"
 
+namespace CSMDoc
+{
+    class Messages;
+}
+
+namespace CSMWorld
+{
+    class InfoCollection;
+}
+
+namespace ESM
+{
+    struct Dialogue;
+}
+
 namespace CSMTools
 {
     /// \brief VerifyStage: make sure that journal infos are good
     class JournalCheckStage : public CSMDoc::Stage
     {
     public:
-
-        JournalCheckStage(const CSMWorld::IdCollection<ESM::Dialogue>& journals,
-            const CSMWorld::InfoCollection& journalInfos);
+        JournalCheckStage(
+            const CSMWorld::IdCollection<ESM::Dialogue>& journals, const CSMWorld::InfoCollection& journalInfos);
 
         int setup() override;
         ///< \return number of steps
@@ -25,11 +37,10 @@ namespace CSMTools
         ///< Messages resulting from this stage will be appended to \a messages
 
     private:
-
         const CSMWorld::IdCollection<ESM::Dialogue>& mJournals;
         const CSMWorld::InfoCollection& mJournalInfos;
         bool mIgnoreBaseRecords;
-
+        CSMWorld::InfosRecordPtrByTopic mInfosByTopic;
     };
 }
 
