@@ -1,31 +1,43 @@
 #include "magiceffectcheck.hpp"
 
+#include <apps/opencs/model/doc/messages.hpp>
+#include <apps/opencs/model/prefs/category.hpp>
+#include <apps/opencs/model/prefs/setting.hpp>
+#include <apps/opencs/model/world/idcollection.hpp>
+#include <apps/opencs/model/world/record.hpp>
+#include <apps/opencs/model/world/refidcollection.hpp>
+#include <apps/opencs/model/world/refiddata.hpp>
+#include <apps/opencs/model/world/resources.hpp>
+
+#include <components/esm3/loadmgef.hpp>
 #include <components/misc/resourcehelpers.hpp>
 
 #include "../prefs/state.hpp"
 
-std::string CSMTools::MagicEffectCheckStage::checkObject(const std::string &id, 
-                                                                const CSMWorld::UniversalId &type, 
-                                                                const std::string &column) const
+namespace ESM
+{
+    struct Sound;
+}
+
+std::string CSMTools::MagicEffectCheckStage::checkObject(
+    const ESM::RefId& id, const CSMWorld::UniversalId& type, const std::string& column) const
 {
     CSMWorld::RefIdData::LocalIndex index = mObjects.getDataSet().searchId(id);
-    if (index.first == -1) 
-        return (column + " '" + id + "' does not exist");
-    else if (index.second != type.getType()) 
-        return (column + " '" + id + "' does not have " + type.getTypeName() + " type");
+    if (index.first == -1)
+        return (column + " '" + id.getRefIdString() + "' does not exist");
+    else if (index.second != type.getType())
+        return (column + " '" + id.getRefIdString() + "' does not have " + type.getTypeName() + " type");
     return std::string();
 }
 
-CSMTools::MagicEffectCheckStage::MagicEffectCheckStage(const CSMWorld::IdCollection<ESM::MagicEffect> &effects,
-                                                       const CSMWorld::IdCollection<ESM::Sound> &sounds,
-                                                       const CSMWorld::RefIdCollection &objects,
-                                                       const CSMWorld::Resources &icons,
-                                                       const CSMWorld::Resources &textures)
-    : mMagicEffects(effects),
-      mSounds(sounds),
-      mObjects(objects),
-      mIcons(icons),
-      mTextures(textures)
+CSMTools::MagicEffectCheckStage::MagicEffectCheckStage(const CSMWorld::IdCollection<ESM::MagicEffect>& effects,
+    const CSMWorld::IdCollection<ESM::Sound>& sounds, const CSMWorld::RefIdCollection& objects,
+    const CSMWorld::Resources& icons, const CSMWorld::Resources& textures)
+    : mMagicEffects(effects)
+    , mSounds(sounds)
+    , mObjects(objects)
+    , mIcons(icons)
+    , mTextures(textures)
 {
     mIgnoreBaseRecords = false;
 }
@@ -37,16 +49,21 @@ int CSMTools::MagicEffectCheckStage::setup()
     return mMagicEffects.getSize();
 }
 
-void CSMTools::MagicEffectCheckStage::perform(int stage, CSMDoc::Messages &messages)
+void CSMTools::MagicEffectCheckStage::perform(int stage, CSMDoc::Messages& messages)
 {
-    const CSMWorld::Record<ESM::MagicEffect> &record = mMagicEffects.getRecord(stage);
+    const CSMWorld::Record<ESM::MagicEffect>& record = mMagicEffects.getRecord(stage);
 
     // Skip "Base" records (setting!) and "Deleted" records
     if ((mIgnoreBaseRecords && record.mState == CSMWorld::RecordBase::State_BaseOnly) || record.isDeleted())
         return;
 
     ESM::MagicEffect effect = record.get();
-    CSMWorld::UniversalId id(CSMWorld::UniversalId::Type_MagicEffect, effect.mId);
+    CSMWorld::UniversalId id(CSMWorld::UniversalId::Type_MagicEffect, CSMWorld::getRecordId(effect));
+
+    if (effect.mData.mSpeed <= 0.0f)
+    {
+        messages.add(id, "Speed is less than or equal to zero", "", CSMDoc::Message::Severity_Error);
+    }
 
     if (effect.mDescription.empty())
     {
@@ -78,7 +95,8 @@ void CSMTools::MagicEffectCheckStage::perform(int stage, CSMDoc::Messages &messa
         {
             std::string ddsParticle = effect.mParticle;
             if (!(Misc::ResourceHelpers::changeExtensionToDds(ddsParticle) && mTextures.searchId(ddsParticle) != -1))
-                messages.add(id, "Particle texture '" + effect.mParticle + "' does not exist", "", CSMDoc::Message::Severity_Error);
+                messages.add(id, "Particle texture '" + effect.mParticle + "' does not exist", "",
+                    CSMDoc::Message::Severity_Error);
         }
     }
 
@@ -111,11 +129,15 @@ void CSMTools::MagicEffectCheckStage::perform(int stage, CSMDoc::Messages &messa
     }
 
     if (!effect.mCastSound.empty() && mSounds.searchId(effect.mCastSound) == -1)
-        messages.add(id, "Casting sound '" + effect.mCastSound + "' does not exist", "", CSMDoc::Message::Severity_Error);
+        messages.add(id, "Casting sound '" + effect.mCastSound.getRefIdString() + "' does not exist", "",
+            CSMDoc::Message::Severity_Error);
     if (!effect.mHitSound.empty() && mSounds.searchId(effect.mHitSound) == -1)
-        messages.add(id, "Hit sound '" + effect.mHitSound + "' does not exist", "", CSMDoc::Message::Severity_Error);
+        messages.add(id, "Hit sound '" + effect.mHitSound.getRefIdString() + "' does not exist", "",
+            CSMDoc::Message::Severity_Error);
     if (!effect.mAreaSound.empty() && mSounds.searchId(effect.mAreaSound) == -1)
-        messages.add(id, "Area sound '" + effect.mAreaSound + "' does not exist", "", CSMDoc::Message::Severity_Error);
+        messages.add(id, "Area sound '" + effect.mAreaSound.getRefIdString() + "' does not exist", "",
+            CSMDoc::Message::Severity_Error);
     if (!effect.mBoltSound.empty() && mSounds.searchId(effect.mBoltSound) == -1)
-        messages.add(id, "Bolt sound '" + effect.mBoltSound + "' does not exist", "", CSMDoc::Message::Severity_Error);
+        messages.add(id, "Bolt sound '" + effect.mBoltSound.getRefIdString() + "' does not exist", "",
+            CSMDoc::Message::Severity_Error);
 }
