@@ -1,12 +1,21 @@
 #include "racecheck.hpp"
 
+#include <string>
+
 #include "../prefs/state.hpp"
 
 #include "../world/universalid.hpp"
 
-void CSMTools::RaceCheckStage::performPerRecord (int stage, CSMDoc::Messages& messages)
+#include <apps/opencs/model/doc/messages.hpp>
+#include <apps/opencs/model/prefs/category.hpp>
+#include <apps/opencs/model/prefs/setting.hpp>
+#include <apps/opencs/model/world/idcollection.hpp>
+#include <apps/opencs/model/world/record.hpp>
+#include <components/esm3/loadrace.hpp>
+
+void CSMTools::RaceCheckStage::performPerRecord(int stage, CSMDoc::Messages& messages)
 {
-    const CSMWorld::Record<ESM::Race>& record = mRaces.getRecord (stage);
+    const CSMWorld::Record<ESM::Race>& record = mRaces.getRecord(stage);
 
     if (record.isDeleted())
         return;
@@ -21,42 +30,44 @@ void CSMTools::RaceCheckStage::performPerRecord (int stage, CSMDoc::Messages& me
     if (mIgnoreBaseRecords && record.mState == CSMWorld::RecordBase::State_BaseOnly)
         return;
 
-    CSMWorld::UniversalId id (CSMWorld::UniversalId::Type_Race, race.mId);
+    CSMWorld::UniversalId id(CSMWorld::UniversalId::Type_Race, race.mId);
 
     // test for empty name and description
     if (race.mName.empty())
-        messages.add(id, "Name is missing", "", (race.mData.mFlags & 0x1) ? CSMDoc::Message::Severity_Error : CSMDoc::Message::Severity_Warning);
+        messages.add(id, "Name is missing", "",
+            (race.mData.mFlags & 0x1) ? CSMDoc::Message::Severity_Error : CSMDoc::Message::Severity_Warning);
 
     if (race.mDescription.empty())
         messages.add(id, "Description is missing", "", CSMDoc::Message::Severity_Warning);
 
     // test for positive height
-    if (race.mData.mHeight.mMale<=0)
+    if (race.mData.mMaleHeight <= 0)
         messages.add(id, "Male height is non-positive", "", CSMDoc::Message::Severity_Error);
 
-    if (race.mData.mHeight.mFemale<=0)
+    if (race.mData.mFemaleHeight <= 0)
         messages.add(id, "Female height is non-positive", "", CSMDoc::Message::Severity_Error);
 
     // test for non-negative weight
-    if (race.mData.mWeight.mMale<0)
+    if (race.mData.mMaleWeight < 0)
         messages.add(id, "Male weight is negative", "", CSMDoc::Message::Severity_Error);
 
-    if (race.mData.mWeight.mFemale<0)
+    if (race.mData.mFemaleWeight < 0)
         messages.add(id, "Female weight is negative", "", CSMDoc::Message::Severity_Error);
 
     /// \todo check data members that can't be edited in the table view
 }
 
-void CSMTools::RaceCheckStage::performFinal (CSMDoc::Messages& messages)
+void CSMTools::RaceCheckStage::performFinal(CSMDoc::Messages& messages)
 {
-    CSMWorld::UniversalId id (CSMWorld::UniversalId::Type_Races);
+    CSMWorld::UniversalId id(CSMWorld::UniversalId::Type_Races);
 
     if (!mPlayable)
         messages.add(id, "No playable race", "", CSMDoc::Message::Severity_SeriousError);
 }
 
-CSMTools::RaceCheckStage::RaceCheckStage (const CSMWorld::IdCollection<ESM::Race>& races)
-: mRaces (races), mPlayable (false)
+CSMTools::RaceCheckStage::RaceCheckStage(const CSMWorld::IdCollection<ESM::Race>& races)
+    : mRaces(races)
+    , mPlayable(false)
 {
     mIgnoreBaseRecords = false;
 }
@@ -66,13 +77,13 @@ int CSMTools::RaceCheckStage::setup()
     mPlayable = false;
     mIgnoreBaseRecords = CSMPrefs::get()["Reports"]["ignore-base-records"].isTrue();
 
-    return mRaces.getSize()+1;
+    return mRaces.getSize() + 1;
 }
 
-void CSMTools::RaceCheckStage::perform (int stage, CSMDoc::Messages& messages)
+void CSMTools::RaceCheckStage::perform(int stage, CSMDoc::Messages& messages)
 {
-    if (stage==mRaces.getSize())
-        performFinal (messages);
+    if (stage == mRaces.getSize())
+        performFinal(messages);
     else
-        performPerRecord (stage, messages);
+        performPerRecord(stage, messages);
 }
