@@ -1,13 +1,24 @@
 #include "soundcheck.hpp"
 
+#include <string>
+
 #include "../prefs/state.hpp"
 
 #include "../world/universalid.hpp"
 
-CSMTools::SoundCheckStage::SoundCheckStage (const CSMWorld::IdCollection<ESM::Sound> &sounds,
-                                            const CSMWorld::Resources &soundfiles)
-    : mSounds (sounds),
-      mSoundFiles (soundfiles)
+#include <apps/opencs/model/doc/messages.hpp>
+#include <apps/opencs/model/prefs/category.hpp>
+#include <apps/opencs/model/prefs/setting.hpp>
+#include <apps/opencs/model/world/idcollection.hpp>
+#include <apps/opencs/model/world/record.hpp>
+#include <apps/opencs/model/world/resources.hpp>
+
+#include <components/esm3/loadsoun.hpp>
+
+CSMTools::SoundCheckStage::SoundCheckStage(
+    const CSMWorld::IdCollection<ESM::Sound>& sounds, const CSMWorld::Resources& soundfiles)
+    : mSounds(sounds)
+    , mSoundFiles(soundfiles)
 {
     mIgnoreBaseRecords = false;
 }
@@ -19,9 +30,9 @@ int CSMTools::SoundCheckStage::setup()
     return mSounds.getSize();
 }
 
-void CSMTools::SoundCheckStage::perform (int stage, CSMDoc::Messages& messages)
+void CSMTools::SoundCheckStage::perform(int stage, CSMDoc::Messages& messages)
 {
-    const CSMWorld::Record<ESM::Sound>& record = mSounds.getRecord (stage);
+    const CSMWorld::Record<ESM::Sound>& record = mSounds.getRecord(stage);
 
     // Skip "Base" records (setting!) and "Deleted" records
     if ((mIgnoreBaseRecords && record.mState == CSMWorld::RecordBase::State_BaseOnly) || record.isDeleted())
@@ -29,9 +40,9 @@ void CSMTools::SoundCheckStage::perform (int stage, CSMDoc::Messages& messages)
 
     const ESM::Sound& sound = record.get();
 
-    CSMWorld::UniversalId id (CSMWorld::UniversalId::Type_Sound, sound.mId);
+    CSMWorld::UniversalId id(CSMWorld::UniversalId::Type_Sound, sound.mId);
 
-    if (sound.mData.mMinRange>sound.mData.mMaxRange)
+    if (sound.mData.mMinRange > sound.mData.mMaxRange)
     {
         messages.add(id, "Minimum range is larger than maximum range", "", CSMDoc::Message::Severity_Warning);
     }
