@@ -1,11 +1,15 @@
 #ifndef CSM_WOLRD_CELL_H
 #define CSM_WOLRD_CELL_H
 
-#include <vector>
 #include <string>
 
 #include <components/esm3/loadcell.hpp>
 
+namespace ESM
+{
+    class ESMReader;
+}
+
 namespace CSMWorld
 {
     /// \brief Wrapper for Cell record
@@ -14,10 +18,9 @@ namespace CSMWorld
     /// Exterior cell coordinates are encoded in the cell ID.
     struct Cell : public ESM::Cell
     {
-        std::string mId;
-
-        void load (ESM::ESMReader &esm, bool &isDeleted);
+        ESM::RefId mId;
 
+        void load(ESM::ESMReader& esm, bool& isDeleted);
     };
 }
 
