@@ -1,6 +1,9 @@
 #include "idtablebase.hpp"
 
-CSMWorld::IdTableBase::IdTableBase (unsigned int features) : mFeatures (features) {}
+CSMWorld::IdTableBase::IdTableBase(unsigned int features)
+    : mFeatures(features)
+{
+}
 
 unsigned int CSMWorld::IdTableBase::getFeatures() const
 {
