@@ -1,30 +1,38 @@
 #include "infotableproxymodel.hpp"
 
-#include <components/misc/stringops.hpp>
+#include <QModelIndex>
+#include <QObject>
+#include <QString>
+
+#include <apps/opencs/model/world/idtableproxymodel.hpp>
+#include <apps/opencs/model/world/universalid.hpp>
+
+#include <components/misc/strings/lower.hpp>
+
+#include <string>
 
-#include "idtablebase.hpp"
 #include "columns.hpp"
+#include "idtablebase.hpp"
 
 namespace
 {
-    QString toLower(const QString &str)
+    QString toLower(const QString& str)
     {
         return QString::fromUtf8(Misc::StringUtils::lowerCase(str.toUtf8().constData()).c_str());
     }
 }
 
-CSMWorld::InfoTableProxyModel::InfoTableProxyModel(CSMWorld::UniversalId::Type type, QObject *parent)
-    : IdTableProxyModel(parent),
-      mType(type),
-      mInfoColumnId(type == UniversalId::Type_TopicInfos ? Columns::ColumnId_Topic :
-                                                           Columns::ColumnId_Journal),
-      mInfoColumnIndex(-1),
-      mLastAddedSourceRow(-1)
+CSMWorld::InfoTableProxyModel::InfoTableProxyModel(CSMWorld::UniversalId::Type type, QObject* parent)
+    : IdTableProxyModel(parent)
+    , mType(type)
+    , mInfoColumnId(type == UniversalId::Type_TopicInfos ? Columns::ColumnId_Topic : Columns::ColumnId_Journal)
+    , mInfoColumnIndex(-1)
+    , mLastAddedSourceRow(-1)
 {
     Q_ASSERT(type == UniversalId::Type_TopicInfos || type == UniversalId::Type_JournalInfos);
 }
 
-void CSMWorld::InfoTableProxyModel::setSourceModel(QAbstractItemModel *sourceModel)
+void CSMWorld::InfoTableProxyModel::setSourceModel(QAbstractItemModel* sourceModel)
 {
     IdTableProxyModel::setSourceModel(sourceModel);
 
@@ -35,7 +43,7 @@ void CSMWorld::InfoTableProxyModel::setSourceModel(QAbstractItemModel *sourceMod
     }
 }
 
-bool CSMWorld::InfoTableProxyModel::lessThan(const QModelIndex &left, const QModelIndex &right) const
+bool CSMWorld::InfoTableProxyModel::lessThan(const QModelIndex& left, const QModelIndex& right) const
 {
     Q_ASSERT(mSourceModel != nullptr);
 
@@ -63,21 +71,21 @@ int CSMWorld::InfoTableProxyModel::getFirstInfoRow(int currentRow) const
         return mFirstRowCache[info];
     }
 
-    while (--row >= 0 && 
-           toLower(mSourceModel->data(mSourceModel->index(row, column)).toString()) == info);
+    while (--row >= 0 && toLower(mSourceModel->data(mSourceModel->index(row, column)).toString()) == info)
+        ;
     ++row;
 
     mFirstRowCache[info] = row;
     return row;
 }
 
-void CSMWorld::InfoTableProxyModel::sourceRowsRemoved(const QModelIndex &/*parent*/, int /*start*/, int /*end*/)
+void CSMWorld::InfoTableProxyModel::sourceRowsRemoved(const QModelIndex& /*parent*/, int /*start*/, int /*end*/)
 {
     refreshFilter();
     mFirstRowCache.clear();
 }
 
-void CSMWorld::InfoTableProxyModel::sourceRowsInserted(const QModelIndex &parent, int /*start*/, int end)
+void CSMWorld::InfoTableProxyModel::sourceRowsInserted(const QModelIndex& parent, int /*start*/, int end)
 {
     refreshFilter();
 
@@ -90,19 +98,18 @@ void CSMWorld::InfoTableProxyModel::sourceRowsInserted(const QModelIndex &parent
     }
 }
 
-void CSMWorld::InfoTableProxyModel::sourceDataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight)
+void CSMWorld::InfoTableProxyModel::sourceDataChanged(const QModelIndex& topLeft, const QModelIndex& bottomRight)
 {
     refreshFilter();
 
-    if (mLastAddedSourceRow != -1 && 
-        topLeft.row() <= mLastAddedSourceRow && bottomRight.row() >= mLastAddedSourceRow)
+    if (mLastAddedSourceRow != -1 && topLeft.row() <= mLastAddedSourceRow && bottomRight.row() >= mLastAddedSourceRow)
     {
-        // Now the topic of the last added row is set, 
+        // Now the topic of the last added row is set,
         // so we can re-sort the model to ensure the corrent position of this row
         int column = sortColumn();
         Qt::SortOrder order = sortOrder();
         sort(mInfoColumnIndex); // Restore the correct position of an added row
-        sort(column, order);    // Restore the original sort order
+        sort(column, order); // Restore the original sort order
         emit rowAdded(getRecordId(mLastAddedSourceRow).toUtf8().constData());
 
         // Make sure that we perform a re-sorting only in the first dataChanged() after a row insertion
