@@ -5,6 +5,11 @@
 
 #include <components/esm3/loadland.hpp>
 
+namespace ESM
+{
+    class ESMReader;
+}
+
 namespace CSMWorld
 {
     /// \brief Wrapper for Land record. Encodes X and Y cell index in the ID.
@@ -13,7 +18,7 @@ namespace CSMWorld
     struct Land : public ESM::Land
     {
         /// Loads the metadata and ID
-        void load (ESM::ESMReader &esm, bool &isDeleted);
+        void load(ESM::ESMReader& esm, bool& isDeleted);
 
         static std::string createUniqueRecordId(int x, int y);
         static void parseUniqueRecordId(const std::string& id, int& x, int& y);
