@@ -10,14 +10,13 @@ namespace CSMWorld
     template <typename ESXRecordT>
     struct Record;
 
-    template<typename ESXRecordT>
+    template <typename ESXRecordT>
     class NestedColumnAdapter
     {
     public:
+        NestedColumnAdapter() = default;
 
-        NestedColumnAdapter() {}
-
-        virtual ~NestedColumnAdapter() {}
+        virtual ~NestedColumnAdapter() = default;
 
         virtual void addRow(Record<ESXRecordT>& record, int position) const = 0;
 
@@ -29,7 +28,8 @@ namespace CSMWorld
 
         virtual QVariant getData(const Record<ESXRecordT>& record, int subRowIndex, int subColIndex) const = 0;
 
-        virtual void setData(Record<ESXRecordT>& record, const QVariant& value, int subRowIndex, int subColIndex) const = 0;
+        virtual void setData(
+            Record<ESXRecordT>& record, const QVariant& value, int subRowIndex, int subColIndex) const = 0;
 
         virtual int getColumnsCount(const Record<ESXRecordT>& record) const = 0;
 
