@@ -1,69 +1,82 @@
 #include "refcollection.hpp"
 
-#include <components/esm3/loadcell.hpp>
-
-#include "ref.hpp"
-#include "cell.hpp"
-#include "universalid.hpp"
-#include "record.hpp"
-
+#include <algorithm>
+#include <stdexcept>
 #include <string_view>
+#include <utility>
+
+#include <apps/opencs/model/world/collection.hpp>
+
+#include <components/esm3/cellref.hpp>
+#include <components/esm3/esmreader.hpp>
+#include <components/esm3/loadcell.hpp>
+#include <components/misc/strings/conversion.hpp>
+
+#include "cell.hpp"
+#include "record.hpp"
+#include "ref.hpp"
+#include "universalid.hpp"
+
+#include "../doc/messages.hpp"
 
 namespace CSMWorld
 {
-    template<>
-    void Collection<CellRef, IdAccessor<CellRef> >::removeRows (int index, int count)
+    template <>
+    void Collection<CellRef>::removeRows(int index, int count)
     {
-        mRecords.erase(mRecords.begin()+index, mRecords.begin()+index+count);
+        mRecords.erase(mRecords.begin() + index, mRecords.begin() + index + count);
 
         // index map is updated in RefCollection::removeRows()
     }
 
-    template<>
-    void Collection<CellRef, IdAccessor<CellRef> >::insertRecord (std::unique_ptr<RecordBase> record, int index,
-        UniversalId::Type type)
+    template <>
+    void Collection<CellRef>::insertRecord(std::unique_ptr<RecordBase> record, int index, UniversalId::Type type)
     {
         int size = static_cast<int>(mRecords.size());
         if (index < 0 || index > size)
             throw std::runtime_error("index out of range");
 
-        std::unique_ptr<Record<CellRef> > record2(static_cast<Record<CellRef>*>(record.release()));
+        std::unique_ptr<Record<CellRef>> record2(static_cast<Record<CellRef>*>(record.release()));
 
         if (index == size)
             mRecords.push_back(std::move(record2));
         else
-            mRecords.insert(mRecords.begin()+index, std::move(record2));
+            mRecords.insert(mRecords.begin() + index, std::move(record2));
 
         // index map is updated in RefCollection::insertRecord()
     }
 }
 
-void CSMWorld::RefCollection::load (ESM::ESMReader& reader, int cellIndex, bool base,
-    std::map<unsigned int, unsigned int>& cache, CSMDoc::Messages& messages)
+void CSMWorld::RefCollection::load(ESM::ESMReader& reader, int cellIndex, bool base,
+    std::map<ESM::RefNum, unsigned int>& cache, CSMDoc::Messages& messages)
 {
-    Record<Cell> cell = mCells.getRecord (cellIndex);
+    Record<Cell> cell = mCells.getRecord(cellIndex);
 
     Cell& cell2 = base ? cell.mBase : cell.mModified;
 
-    CellRef ref;
-    ref.mNew = false;
     ESM::MovedCellRef mref;
-    mref.mRefNum.mIndex = 0;
     bool isDeleted = false;
     bool isMoved = false;
 
-    while (ESM::Cell::getNextRef(reader, ref, isDeleted, mref, isMoved))
+    while (true)
     {
+        CellRef ref;
+        ref.mNew = false;
+
+        if (!ESM::Cell::getNextRef(reader, ref, isDeleted, mref, isMoved))
+            break;
+        if (!base && reader.getIndex() == ref.mRefNum.mContentFile)
+            ref.mRefNum.mContentFile = -1;
         // Keep mOriginalCell empty when in modified (as an indicator that the
         // original cell will always be equal the current cell).
-        ref.mOriginalCell = base ? cell2.mId : "";
+        ref.mOriginalCell = base ? cell2.mId : ESM::RefId();
 
         if (cell.get().isExterior())
         {
             // Autocalculate the cell index from coordinates first
             std::pair<int, int> index = ref.getCellIndex();
 
-            ref.mCell = "#" + std::to_string(index.first) + " " + std::to_string(index.second);
+            ref.mCell = ESM::RefId::stringRefId(ESM::RefId::esm3ExteriorCell(index.first, index.second).toString());
 
             // Handle non-base moved references
             if (!base && isMoved)
@@ -78,12 +91,13 @@ void CSMWorld::RefCollection::load (ESM::ESMReader& reader, int cellIndex, bool
                 // Log a warning if the record target cell is different
                 if (index.first != mref.mTarget[0] || index.second != mref.mTarget[1])
                 {
-                    std::string indexCell = ref.mCell;
-                    ref.mCell = "#" + std::to_string(mref.mTarget[0]) + " " + std::to_string(mref.mTarget[1]);
+                    ESM::RefId indexCell = ref.mCell;
+                    ref.mCell = ESM::RefId::stringRefId(
+                        ESM::RefId::esm3ExteriorCell(mref.mTarget[0], mref.mTarget[1]).toString());
 
-                    CSMWorld::UniversalId id(CSMWorld::UniversalId::Type_Cell, mCells.getId (cellIndex));
-                    messages.add(id, "The position of the moved reference " + ref.mRefID + " (cell " + indexCell + ")"
-                                     " does not match the target cell (" + ref.mCell + ")",
+                    CSMWorld::UniversalId id(CSMWorld::UniversalId::Type_Cell, mCells.getId(cellIndex));
+                    messages.add(id, "The position of the moved reference " + ref.mRefID.toDebugString() + " (cell " + indexCell.toDebugString() + ")"
+                                     " does not match the target cell (" + ref.mCell.toDebugString() + ")",
                                      std::string(), CSMDoc::Message::Severity_Warning);
                 }
             }
@@ -91,29 +105,19 @@ void CSMWorld::RefCollection::load (ESM::ESMReader& reader, int cellIndex, bool
         else
             ref.mCell = cell2.mId;
 
-        if (ref.mRefNum.mContentFile != -1 && !base)
-        {
-            ref.mRefNum.mContentFile = ref.mRefNum.mIndex >> 24;
-            ref.mRefNum.mIndex &= 0x00ffffff;
-        }
-
-        unsigned int  refNum = (ref.mRefNum.mIndex & 0x00ffffff) |
-            (ref.mRefNum.hasContentFile() ? ref.mRefNum.mContentFile : 0xff) << 24;
-
-        std::map<unsigned int, unsigned int>::iterator iter = cache.find(refNum);
+        auto iter = cache.find(ref.mRefNum);
 
         if (isMoved)
         {
             if (iter == cache.end())
             {
-                CSMWorld::UniversalId id(CSMWorld::UniversalId::Type_Cell,
-                    mCells.getId(cellIndex));
+                CSMWorld::UniversalId id(CSMWorld::UniversalId::Type_Cell, mCells.getId(cellIndex));
 
-                messages.add(id, "Attempt to move a non-existent reference - RefNum index "
-                    + std::to_string(ref.mRefNum.mIndex) + ", refID " + ref.mRefID + ", content file index "
-                    + std::to_string(ref.mRefNum.mContentFile),
-                    /*hint*/"",
-                    CSMDoc::Message::Severity_Warning);
+                messages.add(id,
+                    "Attempt to move a non-existent reference - RefNum index " + std::to_string(ref.mRefNum.mIndex)
+                        + ", refID " + ref.mRefID.toDebugString() + ", content file index "
+                        + std::to_string(ref.mRefNum.mContentFile),
+                    /*hint*/ "", CSMDoc::Message::Severity_Warning);
                 continue;
             }
 
@@ -121,7 +125,7 @@ void CSMWorld::RefCollection::load (ESM::ESMReader& reader, int cellIndex, bool
 
             // ensure we have the same record id for setRecord()
             ref.mId = getRecord(index).get().mId;
-            ref.mIdNum = extractIdNum(ref.mId);
+            ref.mIdNum = extractIdNum(ref.mId.getRefIdString());
 
             auto record = std::make_unique<Record<CellRef>>();
             // TODO: check whether a base record be moved
@@ -136,25 +140,24 @@ void CSMWorld::RefCollection::load (ESM::ESMReader& reader, int cellIndex, bool
 
         if (isDeleted)
         {
-            if (iter==cache.end())
+            if (iter == cache.end())
             {
-                CSMWorld::UniversalId id (CSMWorld::UniversalId::Type_Cell,
-                    mCells.getId (cellIndex));
+                CSMWorld::UniversalId id(CSMWorld::UniversalId::Type_Cell, mCells.getId(cellIndex));
 
-                messages.add (id, "Attempt to delete a non-existent reference - RefNum index "
-                        + std::to_string(ref.mRefNum.mIndex) + ", refID " + ref.mRefID + ", content file index "
+                messages.add(id,
+                    "Attempt to delete a non-existent reference - RefNum index " + std::to_string(ref.mRefNum.mIndex)
+                        + ", refID " + ref.mRefID.getRefIdString() + ", content file index "
                         + std::to_string(ref.mRefNum.mContentFile),
-                        /*hint*/"",
-                        CSMDoc::Message::Severity_Warning);
+                    /*hint*/ "", CSMDoc::Message::Severity_Warning);
                 continue;
             }
 
-            int index = getIntIndex (iter->second);
+            int index = getIntIndex(iter->second);
 
             if (base)
             {
-                removeRows (index, 1);
-                cache.erase (iter);
+                removeRows(index, 1);
+                cache.erase(iter);
             }
             else
             {
@@ -166,13 +169,13 @@ void CSMWorld::RefCollection::load (ESM::ESMReader& reader, int cellIndex, bool
             continue;
         }
 
-        if (iter==cache.end())
+        if (iter == cache.end())
         {
             // new reference
             ref.mIdNum = mNextId; // FIXME: fragile
-            ref.mId = getNewId();
+            ref.mId = ESM::RefId::stringRefId(getNewId());
 
-            cache.emplace(refNum, ref.mIdNum);
+            cache.emplace(ref.mRefNum, ref.mIdNum);
 
             auto record = std::make_unique<Record<CellRef>>();
             record->mState = base ? RecordBase::State_BaseOnly : RecordBase::State_ModifiedOnly;
@@ -203,7 +206,7 @@ void CSMWorld::RefCollection::load (ESM::ESMReader& reader, int cellIndex, bool
             }
 #endif
             ref.mId = getRecord(index).get().mId;
-            ref.mIdNum = extractIdNum(ref.mId);
+            ref.mIdNum = extractIdNum(ref.mId.getRefIdString());
 
             auto record = std::make_unique<Record<CellRef>>(getRecord(index));
             record->mState = base ? RecordBase::State_BaseOnly : RecordBase::State_Modified;
@@ -226,10 +229,10 @@ unsigned int CSMWorld::RefCollection::extractIdNum(std::string_view id) const
     if (separator == std::string::npos)
         throw std::runtime_error("invalid ref ID: " + std::string(id));
 
-    return static_cast<unsigned int>(std::stoi(std::string(id.substr(separator+1))));
+    return Misc::StringUtils::toNumeric<unsigned int>(id.substr(separator + 1), 0);
 }
 
-int CSMWorld::RefCollection::getIntIndex (unsigned int id) const
+int CSMWorld::RefCollection::getIntIndex(unsigned int id) const
 {
     int index = searchId(id);
 
@@ -239,7 +242,7 @@ int CSMWorld::RefCollection::getIntIndex (unsigned int id) const
     return index;
 }
 
-int CSMWorld::RefCollection::searchId (unsigned int id) const
+int CSMWorld::RefCollection::searchId(unsigned int id) const
 {
     std::map<unsigned int, int>::const_iterator iter = mRefIndex.find(id);
 
@@ -249,16 +252,16 @@ int CSMWorld::RefCollection::searchId (unsigned int id) const
     return iter->second;
 }
 
-void CSMWorld::RefCollection::removeRows (int index, int count)
+void CSMWorld::RefCollection::removeRows(int index, int count)
 {
-    Collection<CellRef, IdAccessor<CellRef> >::removeRows(index, count); // erase records only
+    Collection<CellRef>::removeRows(index, count); // erase records only
 
     std::map<unsigned int, int>::iterator iter = mRefIndex.begin();
     while (iter != mRefIndex.end())
     {
-        if (iter->second>=index)
+        if (iter->second >= index)
         {
-            if (iter->second >= index+count)
+            if (iter->second >= index + count)
             {
                 iter->second -= count;
                 ++iter;
@@ -271,7 +274,7 @@ void CSMWorld::RefCollection::removeRows (int index, int count)
     }
 }
 
-void  CSMWorld::RefCollection::appendBlankRecord (const std::string& id, UniversalId::Type type)
+void CSMWorld::RefCollection::appendBlankRecord(const ESM::RefId& id, UniversalId::Type type)
 {
     auto record = std::make_unique<Record<CellRef>>();
 
@@ -279,49 +282,57 @@ void  CSMWorld::RefCollection::appendBlankRecord (const std::string& id, Univers
     record->mModified.blank();
 
     record->get().mId = id;
-    record->get().mIdNum = extractIdNum(id);
+    record->get().mIdNum = extractIdNum(id.getRefIdString());
 
-    Collection<CellRef, IdAccessor<CellRef> >::appendRecord(std::move(record));
+    Collection<CellRef>::appendRecord(std::move(record));
 }
 
-void CSMWorld::RefCollection::cloneRecord (const std::string& origin,
-                                           const std::string& destination,
-                                           const UniversalId::Type type)
+void CSMWorld::RefCollection::cloneRecord(
+    const ESM::RefId& origin, const ESM::RefId& destination, const UniversalId::Type type)
 {
     auto copy = std::make_unique<Record<CellRef>>();
+    int index = getAppendIndex(ESM::RefId(), type);
 
     copy->mModified = getRecord(origin).get();
     copy->mState = RecordBase::State_ModifiedOnly;
 
     copy->get().mId = destination;
-    copy->get().mIdNum = extractIdNum(destination);
+    copy->get().mIdNum = extractIdNum(destination.getRefIdString());
+
+    if (copy->get().mRefNum.hasContentFile())
+    {
+        mRefIndex.insert(std::make_pair(static_cast<Record<CellRef>*>(copy.get())->get().mIdNum, index));
+        copy->get().mRefNum.mContentFile = -1;
+        copy->get().mRefNum.mIndex = index;
+    }
+    else
+        copy->get().mRefNum.mIndex = copy->get().mIdNum;
 
     insertRecord(std::move(copy), getAppendIndex(destination, type)); // call RefCollection::insertRecord()
 }
 
-int CSMWorld::RefCollection::searchId(std::string_view id) const
+int CSMWorld::RefCollection::searchId(const ESM::RefId& id) const
 {
-    return searchId(extractIdNum(id));
+    return searchId(extractIdNum(id.getRefIdString()));
 }
 
-void CSMWorld::RefCollection::appendRecord (std::unique_ptr<RecordBase> record, UniversalId::Type type)
+void CSMWorld::RefCollection::appendRecord(std::unique_ptr<RecordBase> record, UniversalId::Type type)
 {
-    int index = getAppendIndex(/*id*/"", type); // for CellRef records id is ignored
+    int index = getAppendIndex(/*id*/ ESM::RefId(), type); // for CellRef records id is ignored
 
     mRefIndex.insert(std::make_pair(static_cast<Record<CellRef>*>(record.get())->get().mIdNum, index));
 
-    Collection<CellRef, IdAccessor<CellRef> >::insertRecord(std::move(record), index, type); // add records only
+    Collection<CellRef>::insertRecord(std::move(record), index, type); // add records only
 }
 
-void CSMWorld::RefCollection::insertRecord (std::unique_ptr<RecordBase> record, int index,
-    UniversalId::Type type)
+void CSMWorld::RefCollection::insertRecord(std::unique_ptr<RecordBase> record, int index, UniversalId::Type type)
 {
-    int size = getAppendIndex(/*id*/"", type); // for CellRef records id is ignored
+    int size = getAppendIndex(/*id*/ ESM::RefId(), type); // for CellRef records id is ignored
     unsigned int idNum = static_cast<Record<CellRef>*>(record.get())->get().mIdNum;
 
-    Collection<CellRef, IdAccessor<CellRef> >::insertRecord(std::move(record), index, type); // add records only
+    Collection<CellRef>::insertRecord(std::move(record), index, type); // add records only
 
-    if (index < size-1)
+    if (index < size - 1)
     {
         for (std::map<unsigned int, int>::iterator iter(mRefIndex.begin()); iter != mRefIndex.end(); ++iter)
         {
