@@ -1,15 +1,21 @@
 #include "keybindingpage.hpp"
 
 #include <cassert>
+#include <string>
+#include <utility>
+#include <vector>
 
 #include <QComboBox>
 #include <QGridLayout>
+#include <QLabel>
 #include <QPushButton>
 #include <QStackedLayout>
 #include <QVBoxLayout>
 
-#include "../../model/prefs/setting.hpp"
+#include <apps/opencs/view/prefs/pagebase.hpp>
+
 #include "../../model/prefs/category.hpp"
+#include "../../model/prefs/setting.hpp"
 #include "../../model/prefs/state.hpp"
 
 namespace CSVPrefs
@@ -29,15 +35,16 @@ namespace CSVPrefs
         mStackedLayout = new QStackedLayout(stackedWidget);
 
         mPageSelector = new QComboBox();
-        connect(mPageSelector, SIGNAL(currentIndexChanged(int)), mStackedLayout, SLOT(setCurrentIndex(int)));
+        connect(mPageSelector, qOverload<int>(&QComboBox::currentIndexChanged), mStackedLayout,
+            &QStackedLayout::setCurrentIndex);
 
         QFrame* lineSeparator = new QFrame(topWidget);
         lineSeparator->setFrameShape(QFrame::HLine);
         lineSeparator->setFrameShadow(QFrame::Sunken);
 
         // Reset key bindings button
-        QPushButton* resetButton = new QPushButton ("Reset to Defaults", topWidget);
-        connect(resetButton, SIGNAL(clicked()), this, SLOT(resetKeyBindings()));
+        QPushButton* resetButton = new QPushButton("Reset to Defaults", topWidget);
+        connect(resetButton, &QPushButton::clicked, this, &KeyBindingPage::resetKeyBindings);
 
         topLayout->addWidget(mPageSelector);
         topLayout->addWidget(stackedWidget);
@@ -46,55 +53,44 @@ namespace CSVPrefs
         topLayout->setSizeConstraint(QLayout::SetMinAndMaxSize);
 
         // Add each option
-        for (CSMPrefs::Category::Iterator iter = category.begin(); iter!=category.end(); ++iter)
-            addSetting (*iter);
+        for (CSMPrefs::Category::Iterator iter = category.begin(); iter != category.end(); ++iter)
+            addSetting(*iter);
 
         setWidgetResizable(true);
         setWidget(topWidget);
     }
 
-    void KeyBindingPage::addSetting(CSMPrefs::Setting *setting)
+    void KeyBindingPage::addSetting(CSMPrefs::Setting* setting)
     {
-        std::pair<QWidget*, QWidget*> widgets = setting->makeWidgets (this);
+        const CSMPrefs::SettingWidgets widgets = setting->makeWidgets(this);
 
-        if (widgets.first)
+        if (widgets.mLabel != nullptr && widgets.mInput != nullptr)
         {
             // Label, Option widgets
             assert(mPageLayout);
 
             int next = mPageLayout->rowCount();
-            mPageLayout->addWidget(widgets.first, next, 0);
-            mPageLayout->addWidget(widgets.second, next, 1);
+            mPageLayout->addWidget(widgets.mLabel, next, 0);
+            mPageLayout->addWidget(widgets.mInput, next, 1);
         }
-        else if (widgets.second)
+        else if (widgets.mInput != nullptr)
         {
             // Wide single widget
             assert(mPageLayout);
 
             int next = mPageLayout->rowCount();
-            mPageLayout->addWidget(widgets.second, next, 0, 1, 2);
+            mPageLayout->addWidget(widgets.mInput, next, 0, 1, 2);
         }
         else
         {
-            if (setting->getLabel().empty())
-            {
-                // Insert empty space
-                assert(mPageLayout);
+            // Create new page
+            QWidget* pageWidget = new QWidget();
+            mPageLayout = new QGridLayout(pageWidget);
+            mPageLayout->setSizeConstraint(QLayout::SetMinAndMaxSize);
 
-                int next = mPageLayout->rowCount();
-                mPageLayout->addWidget(new QWidget(), next, 0);
-            }
-            else
-            {
-                // Create new page
-                QWidget* pageWidget = new QWidget();
-                mPageLayout = new QGridLayout(pageWidget);
-                mPageLayout->setSizeConstraint(QLayout::SetMinAndMaxSize);
+            mStackedLayout->addWidget(pageWidget);
 
-                mStackedLayout->addWidget(pageWidget);
-
-                mPageSelector->addItem(QString::fromUtf8(setting->getLabel().c_str()));
-            }
+            mPageSelector->addItem(setting->getLabel());
         }
     }
 
