@@ -1,15 +1,17 @@
 
 #include "pagebase.hpp"
 
-#include <QMenu>
 #include <QContextMenuEvent>
+#include <QMenu>
 
 #include "../../model/prefs/category.hpp"
 #include "../../model/prefs/state.hpp"
 
-CSVPrefs::PageBase::PageBase (CSMPrefs::Category& category, QWidget *parent)
-: QScrollArea (parent), mCategory (category)
-{}
+CSVPrefs::PageBase::PageBase(CSMPrefs::Category& category, QWidget* parent)
+    : QScrollArea(parent)
+    , mCategory(category)
+{
+}
 
 CSMPrefs::Category& CSVPrefs::PageBase::getCategory()
 {
