@@ -1,11 +1,22 @@
 #include "actor.hpp"
 
-#include <osg/Group>
-#include <osg/Node>
+#include <memory>
+#include <unordered_map>
+#include <utility>
 
+#include <osg/Group>
+#include <osg/MatrixTransform>
+#include <osg/Node>
+#include <osg/Vec3d>
+
+#include <apps/opencs/model/world/actoradapter.hpp>
+#include <apps/opencs/model/world/idcollection.hpp>
+#include <apps/opencs/model/world/record.hpp>
+
+#include <components/esm3/loadbody.hpp>
 #include <components/esm3/mappings.hpp>
 #include <components/misc/resourcehelpers.hpp>
-#include <components/resource/resourcemanager.hpp>
+#include <components/resource/resourcesystem.hpp>
 #include <components/resource/scenemanager.hpp>
 #include <components/sceneutil/attach.hpp>
 #include <components/sceneutil/skeleton.hpp>
@@ -16,15 +27,14 @@ namespace CSVRender
 {
     const std::string Actor::MeshPrefix = "meshes\\";
 
-    Actor::Actor(const std::string& id, CSMWorld::Data& data)
+    Actor::Actor(const ESM::RefId& id, CSMWorld::Data& data)
         : mId(id)
         , mData(data)
-        , mBaseNode(new osg::Group())
+        , mBaseNode(new osg::PositionAttitudeTransform())
         , mSkeleton(nullptr)
     {
         mActorData = mData.getActorAdapter()->getActorData(mId);
-        connect(mData.getActorAdapter(), SIGNAL(actorChanged(const std::string&)),
-                this, SLOT(handleActorChanged(const std::string&)));
+        connect(mData.getActorAdapter(), &CSMWorld::ActorAdapter::actorChanged, this, &Actor::handleActorChanged);
     }
 
     osg::Group* Actor::getBaseNode()
@@ -37,8 +47,9 @@ namespace CSVRender
         mBaseNode->removeChildren(0, mBaseNode->getNumChildren());
 
         // Load skeleton
-        std::string skeletonModel = mActorData->getSkeleton();
-        skeletonModel = Misc::ResourceHelpers::correctActorModelPath(skeletonModel, mData.getResourceSystem()->getVFS());
+        VFS::Path::Normalized skeletonModel = mActorData->getSkeleton();
+        skeletonModel
+            = Misc::ResourceHelpers::correctActorModelPath(skeletonModel, mData.getResourceSystem()->getVFS());
         loadSkeleton(skeletonModel);
 
         if (!mActorData->isCreature())
@@ -50,6 +61,10 @@ namespace CSVRender
 
             // Attach parts to skeleton
             loadBodyParts();
+
+            const osg::Vec2f& attributes = mActorData->getRaceWeightHeight();
+
+            mBaseNode->setScale(osg::Vec3d(attributes.x(), attributes.x(), attributes.y()));
         }
         else
         {
@@ -63,7 +78,7 @@ namespace CSVRender
         mSkeleton->setActive(SceneUtil::Skeleton::Active);
     }
 
-    void Actor::handleActorChanged(const std::string& refId)
+    void Actor::handleActorChanged(const ESM::RefId& refId)
     {
         if (mId == refId)
         {
@@ -75,7 +90,7 @@ namespace CSVRender
     {
         auto sceneMgr = mData.getResourceSystem()->getSceneManager();
 
-        osg::ref_ptr<osg::Node> temp = sceneMgr->getInstance(model);
+        osg::ref_ptr<osg::Node> temp = sceneMgr->getInstance(VFS::Path::toNormalized(model));
         mSkeleton = dynamic_cast<SceneUtil::Skeleton*>(temp.get());
         if (!mSkeleton)
         {
@@ -88,16 +103,14 @@ namespace CSVRender
         mNodeMap.clear();
         SceneUtil::NodeMapVisitor nmVisitor(mNodeMap);
         mSkeleton->accept(nmVisitor);
-
     }
 
     void Actor::loadBodyParts()
     {
         for (int i = 0; i < ESM::PRT_Count; ++i)
         {
-            auto type = (ESM::PartReferenceType) i;
-            const std::string_view partId = mActorData->getPart(type);
-            attachBodyPart(type, getBodyPartMesh(partId));
+            const auto type = static_cast<ESM::PartReferenceType>(i);
+            attachBodyPart(type, getBodyPartMesh(mActorData->getPart(type)));
         }
     }
 
@@ -110,16 +123,16 @@ namespace CSVRender
         auto node = mNodeMap.find(boneName);
         if (!mesh.empty() && node != mNodeMap.end())
         {
-            auto instance = sceneMgr->getInstance(mesh);
+            auto instance = sceneMgr->getInstance(VFS::Path::toNormalized(mesh));
             SceneUtil::attach(instance, mSkeleton, boneName, node->second, sceneMgr);
         }
     }
 
-    std::string Actor::getBodyPartMesh(std::string_view bodyPartId)
+    std::string Actor::getBodyPartMesh(const ESM::RefId& bodyPartId)
     {
         const auto& bodyParts = mData.getBodyParts();
 
-        int index = bodyParts.searchId(bodyPartId);
+        const int index = bodyParts.searchId(bodyPartId);
         if (index != -1 && !bodyParts.getRecord(index).isDeleted())
             return MeshPrefix + bodyParts.getRecord(index).get().mModel;
         else
