@@ -4,6 +4,8 @@
 #include <string>
 #include <string_view>
 
+#include <osg/Group>
+#include <osg/PositionAttitudeTransform>
 #include <osg/ref_ptr>
 
 #include <QObject>
@@ -13,11 +15,6 @@
 
 #include "../../model/world/actoradapter.hpp"
 
-namespace osg
-{
-    class Group;
-}
-
 namespace CSMWorld
 {
     class Data;
@@ -39,7 +36,7 @@ namespace CSVRender
         /// \param id       The referenceable id
         /// \param type     The record type
         /// \param data     The data store
-        Actor(const std::string& id, CSMWorld::Data& data);
+        Actor(const ESM::RefId& id, CSMWorld::Data& data);
 
         /// Retrieves the base node that meshes are attached to
         osg::Group* getBaseNode();
@@ -48,24 +45,24 @@ namespace CSVRender
         void update();
 
     private slots:
-        void handleActorChanged(const std::string& refId);
+        void handleActorChanged(const ESM::RefId& refId);
 
     private:
         void loadSkeleton(const std::string& model);
         void loadBodyParts();
         void attachBodyPart(ESM::PartReferenceType, const std::string& mesh);
 
-        std::string getBodyPartMesh(std::string_view bodyPartId);
+        std::string getBodyPartMesh(const ESM::RefId& bodyPartId);
 
         static const std::string MeshPrefix;
 
-        std::string mId;
+        ESM::RefId mId;
         CSMWorld::Data& mData;
         CSMWorld::ActorAdapter::ActorDataPtr mActorData;
 
-        osg::ref_ptr<osg::Group> mBaseNode;
+        osg::ref_ptr<osg::PositionAttitudeTransform> mBaseNode;
         SceneUtil::Skeleton* mSkeleton;
-        SceneUtil::NodeMapVisitor::NodeMap mNodeMap;
+        SceneUtil::NodeMap mNodeMap;
     };
 }
 
