@@ -1,16 +1,19 @@
 #include "lightingbright.hpp"
 
+#include <osg/Group>
+#include <osg/Light>
 #include <osg/LightSource>
+#include <osg/ref_ptr>
 
 CSVRender::LightingBright::LightingBright() {}
 
-void CSVRender::LightingBright::activate (osg::Group* rootNode, bool /*isExterior*/)
+void CSVRender::LightingBright::activate(osg::Group* rootNode, bool /*isExterior*/)
 {
     mRootNode = rootNode;
 
     mLightSource = (new osg::LightSource);
 
-    osg::ref_ptr<osg::Light> light (new osg::Light);
+    osg::ref_ptr<osg::Light> light(new osg::Light);
     light->setAmbient(osg::Vec4f(0.f, 0.f, 0.f, 1.f));
     light->setPosition(osg::Vec4f(0.f, 0.f, 1.f, 0.f));
     light->setDiffuse(osg::Vec4f(1.f, 1.f, 1.f, 1.f));
