@@ -1,12 +1,12 @@
 #include "pathgridmode.hpp"
 
-#include <QPoint>
+#include <QIcon>
 
+#include <components/misc/scalableicon.hpp>
 #include <components/sceneutil/pathgridutil.hpp>
 
 #include "../../model/prefs/state.hpp"
 
-#include "../../model/world/commands.hpp"
 #include "../../model/world/commandmacro.hpp"
 
 #include "../widget/scenetoolbar.hpp"
@@ -17,11 +17,28 @@
 #include "pathgridselectionmode.hpp"
 #include "worldspacewidget.hpp"
 
+#include <apps/opencs/model/doc/document.hpp>
+#include <apps/opencs/model/prefs/category.hpp>
+#include <apps/opencs/model/prefs/setting.hpp>
+#include <apps/opencs/view/render/editmode.hpp>
+#include <apps/opencs/view/render/tagbase.hpp>
+
+#include <osg/Camera>
+#include <osg/Matrixd>
+#include <osg/Vec3d>
+#include <osg/ref_ptr>
+
+#include <vector>
+
+class QPoint;
+class QUndoStack;
+class QWidget;
+
 namespace CSVRender
 {
     PathgridMode::PathgridMode(WorldspaceWidget* worldspaceWidget, QWidget* parent)
-        : EditMode(worldspaceWidget, QIcon(":placeholder"), Mask_Pathgrid | Mask_Terrain | Mask_Reference,
-            getTooltip(), parent)
+        : EditMode(worldspaceWidget, Misc::ScalableIcon::load(":scenetoolbar/editing-pathgrid"),
+            Mask_Pathgrid | Mask_Terrain | Mask_Reference, getTooltip(), parent)
         , mDragMode(DragMode_None)
         , mFromNode(0)
         , mSelectionMode(nullptr)
@@ -54,24 +71,22 @@ namespace CSVRender
     {
         if (mSelectionMode)
         {
-            toolbar->removeTool (mSelectionMode);
+            toolbar->removeTool(mSelectionMode);
             delete mSelectionMode;
             mSelectionMode = nullptr;
         }
     }
 
-    void PathgridMode::primaryOpenPressed(const WorldspaceHitResult& hitResult)
-    {
-    }
+    void PathgridMode::primaryOpenPressed(const WorldspaceHitResult& hitResult) {}
 
     void PathgridMode::primaryEditPressed(const WorldspaceHitResult& hitResult)
     {
-        if (CSMPrefs::get()["3D Scene Input"]["context-select"].isTrue() &&
-            dynamic_cast<PathgridTag*>(hitResult.tag.get()))
+        if (CSMPrefs::get()["3D Scene Input"]["context-select"].isTrue()
+            && dynamic_cast<PathgridTag*>(hitResult.tag.get()))
         {
             primarySelectPressed(hitResult);
         }
-        else if (Cell* cell = getWorldspaceWidget().getCell (hitResult.worldPos))
+        else if (Cell* cell = getWorldspaceWidget().getCell(hitResult.worldPos))
         {
             if (cell->getPathgrid())
             {
@@ -93,7 +108,7 @@ namespace CSVRender
             {
                 if (tag->getPathgrid()->isSelected())
                 {
-                    unsigned short node = SceneUtil::getPathgridNode(static_cast<unsigned short>(hit.index0));
+                    unsigned short node = SceneUtil::getPathgridNode(hit.index0);
 
                     QUndoStack& undoStack = getWorldspaceWidget().getDocument().getUndoStack();
                     QString description = "Connect node to selected nodes";
@@ -114,7 +129,7 @@ namespace CSVRender
             if (PathgridTag* tag = dynamic_cast<PathgridTag*>(hit.tag.get()))
             {
                 mLastId = tag->getPathgrid()->getId();
-                unsigned short node = SceneUtil::getPathgridNode(static_cast<unsigned short>(hit.index0));
+                unsigned short node = SceneUtil::getPathgridNode(hit.index0);
                 tag->getPathgrid()->toggleSelected(node);
             }
         }
@@ -132,7 +147,7 @@ namespace CSVRender
                     mLastId = tag->getPathgrid()->getId();
                 }
 
-                unsigned short node = SceneUtil::getPathgridNode(static_cast<unsigned short>(hit.index0));
+                unsigned short node = SceneUtil::getPathgridNode(hit.index0);
                 tag->getPathgrid()->toggleSelected(node);
 
                 return;
@@ -144,16 +159,16 @@ namespace CSVRender
 
     bool PathgridMode::primaryEditStartDrag(const QPoint& pos)
     {
-        std::vector<osg::ref_ptr<TagBase> > selection = getWorldspaceWidget().getSelection (Mask_Pathgrid);
+        std::vector<osg::ref_ptr<TagBase>> selection = getWorldspaceWidget().getSelection(Mask_Pathgrid);
 
         if (CSMPrefs::get()["3D Scene Input"]["context-select"].isTrue())
         {
-            WorldspaceHitResult hit = getWorldspaceWidget().mousePick (pos, getWorldspaceWidget().getInteractionMask());
+            WorldspaceHitResult hit = getWorldspaceWidget().mousePick(pos, getWorldspaceWidget().getInteractionMask());
 
             if (dynamic_cast<PathgridTag*>(hit.tag.get()))
             {
                 primarySelectPressed(hit);
-                selection = getWorldspaceWidget().getSelection (Mask_Pathgrid);
+                selection = getWorldspaceWidget().getSelection(Mask_Pathgrid);
             }
         }
 
@@ -168,14 +183,14 @@ namespace CSVRender
 
     bool PathgridMode::secondaryEditStartDrag(const QPoint& pos)
     {
-        WorldspaceHitResult hit = getWorldspaceWidget().mousePick (pos, getWorldspaceWidget().getInteractionMask());
+        WorldspaceHitResult hit = getWorldspaceWidget().mousePick(pos, getWorldspaceWidget().getInteractionMask());
         if (hit.tag)
         {
             if (PathgridTag* tag = dynamic_cast<PathgridTag*>(hit.tag.get()))
             {
                 mDragMode = DragMode_Edge;
                 mEdgeId = tag->getPathgrid()->getId();
-                mFromNode = SceneUtil::getPathgridNode(static_cast<unsigned short>(hit.index0));
+                mFromNode = SceneUtil::getPathgridNode(hit.index0);
 
                 tag->getPathgrid()->setDragOrigin(mFromNode);
                 return true;
@@ -189,14 +204,14 @@ namespace CSVRender
     {
         if (mDragMode == DragMode_Move)
         {
-            std::vector<osg::ref_ptr<TagBase> > selection = getWorldspaceWidget().getSelection(Mask_Pathgrid);
+            std::vector<osg::ref_ptr<TagBase>> selection = getWorldspaceWidget().getSelection(Mask_Pathgrid);
 
-            for (std::vector<osg::ref_ptr<TagBase> >::iterator it = selection.begin(); it != selection.end(); ++it)
+            for (std::vector<osg::ref_ptr<TagBase>>::iterator it = selection.begin(); it != selection.end(); ++it)
             {
                 if (PathgridTag* tag = dynamic_cast<PathgridTag*>(it->get()))
                 {
                     osg::Vec3d eye, center, up, offset;
-                    getWorldspaceWidget().getCamera()->getViewMatrix().getLookAt (eye, center, up);
+                    getWorldspaceWidget().getCamera()->getViewMatrix().getLookAt(eye, center, up);
 
                     offset = (up * diffY * speedFactor) + (((center - eye) ^ up) * diffX * speedFactor);
 
@@ -206,22 +221,22 @@ namespace CSVRender
         }
         else if (mDragMode == DragMode_Edge)
         {
-            WorldspaceHitResult hit = getWorldspaceWidget().mousePick (pos, getWorldspaceWidget().getInteractionMask());
+            WorldspaceHitResult hit = getWorldspaceWidget().mousePick(pos, getWorldspaceWidget().getInteractionMask());
 
             Cell* cell = getWorldspaceWidget().getCell(hit.worldPos);
             if (cell && cell->getPathgrid())
             {
                 PathgridTag* tag = nullptr;
-                if (hit.tag && (tag = dynamic_cast<PathgridTag*>(hit.tag.get())) && tag->getPathgrid()->getId() == mEdgeId)
+                if (hit.tag && (tag = dynamic_cast<PathgridTag*>(hit.tag.get()))
+                    && tag->getPathgrid()->getId() == mEdgeId)
                 {
-                    unsigned short node = SceneUtil::getPathgridNode(static_cast<unsigned short>(hit.index0));
+                    unsigned short node = SceneUtil::getPathgridNode(hit.index0);
                     cell->getPathgrid()->setDragEndpoint(node);
                 }
                 else
                 {
                     cell->getPathgrid()->setDragEndpoint(hit.worldPos);
                 }
-
             }
         }
     }
@@ -230,8 +245,8 @@ namespace CSVRender
     {
         if (mDragMode == DragMode_Move)
         {
-            std::vector<osg::ref_ptr<TagBase> > selection = getWorldspaceWidget().getSelection (Mask_Pathgrid);
-            for (std::vector<osg::ref_ptr<TagBase> >::iterator it = selection.begin(); it != selection.end(); ++it)
+            std::vector<osg::ref_ptr<TagBase>> selection = getWorldspaceWidget().getSelection(Mask_Pathgrid);
+            for (std::vector<osg::ref_ptr<TagBase>>::iterator it = selection.begin(); it != selection.end(); ++it)
             {
                 if (PathgridTag* tag = dynamic_cast<PathgridTag*>(it->get()))
                 {
@@ -253,7 +268,7 @@ namespace CSVRender
                 {
                     if (tag->getPathgrid()->getId() == mEdgeId)
                     {
-                        unsigned short toNode = SceneUtil::getPathgridNode(static_cast<unsigned short>(hit.index0));
+                        unsigned short toNode = SceneUtil::getPathgridNode(hit.index0);
 
                         QUndoStack& undoStack = getWorldspaceWidget().getDocument().getUndoStack();
                         QString description = "Add edge between nodes";
