@@ -1,6 +1,6 @@
 #include "completerpopup.hpp"
 
-CSVWidget::CompleterPopup::CompleterPopup(QWidget *parent)
+CSVWidget::CompleterPopup::CompleterPopup(QWidget* parent)
     : QListView(parent)
 {
     setEditTriggers(QAbstractItemView::NoEditTriggers);
@@ -22,7 +22,12 @@ int CSVWidget::CompleterPopup::sizeHintForRow(int row) const
 
     ensurePolished();
     QModelIndex index = model()->index(row, modelColumn());
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+    QStyleOptionViewItem option;
+    initViewItemOption(&option);
+#else
     QStyleOptionViewItem option = viewOptions();
-    QAbstractItemDelegate *delegate = itemDelegate(index);
+#endif
+    QAbstractItemDelegate* delegate = itemDelegate(index);
     return delegate->sizeHint(option, index).height();
 }
