@@ -2,24 +2,25 @@
 
 #include <QWheelEvent>
 
-CSVWorld::DialogueSpinBox::DialogueSpinBox(QWidget *parent) : QSpinBox(parent)
+CSVWorld::DialogueSpinBox::DialogueSpinBox(QWidget* parent)
+    : QSpinBox(parent)
 {
     setFocusPolicy(Qt::StrongFocus);
 }
 
-void CSVWorld::DialogueSpinBox::focusInEvent(QFocusEvent *event)
+void CSVWorld::DialogueSpinBox::focusInEvent(QFocusEvent* event)
 {
     setFocusPolicy(Qt::WheelFocus);
     QSpinBox::focusInEvent(event);
 }
 
-void CSVWorld::DialogueSpinBox::focusOutEvent(QFocusEvent *event)
+void CSVWorld::DialogueSpinBox::focusOutEvent(QFocusEvent* event)
 {
     setFocusPolicy(Qt::StrongFocus);
     QSpinBox::focusOutEvent(event);
 }
 
-void CSVWorld::DialogueSpinBox::wheelEvent(QWheelEvent *event)
+void CSVWorld::DialogueSpinBox::wheelEvent(QWheelEvent* event)
 {
     if (!hasFocus())
         event->ignore();
@@ -27,24 +28,25 @@ void CSVWorld::DialogueSpinBox::wheelEvent(QWheelEvent *event)
         QSpinBox::wheelEvent(event);
 }
 
-CSVWorld::DialogueDoubleSpinBox::DialogueDoubleSpinBox(QWidget *parent) : QDoubleSpinBox(parent)
+CSVWorld::DialogueDoubleSpinBox::DialogueDoubleSpinBox(QWidget* parent)
+    : QDoubleSpinBox(parent)
 {
     setFocusPolicy(Qt::StrongFocus);
 }
 
-void CSVWorld::DialogueDoubleSpinBox::focusInEvent(QFocusEvent *event)
+void CSVWorld::DialogueDoubleSpinBox::focusInEvent(QFocusEvent* event)
 {
     setFocusPolicy(Qt::WheelFocus);
     QDoubleSpinBox::focusInEvent(event);
 }
 
-void CSVWorld::DialogueDoubleSpinBox::focusOutEvent(QFocusEvent *event)
+void CSVWorld::DialogueDoubleSpinBox::focusOutEvent(QFocusEvent* event)
 {
     setFocusPolicy(Qt::StrongFocus);
     QDoubleSpinBox::focusOutEvent(event);
 }
 
-void CSVWorld::DialogueDoubleSpinBox::wheelEvent(QWheelEvent *event)
+void CSVWorld::DialogueDoubleSpinBox::wheelEvent(QWheelEvent* event)
 {
     if (!hasFocus())
         event->ignore();
