@@ -1,13 +1,19 @@
 #include "globalcreator.hpp"
 
+#include <apps/opencs/model/world/columns.hpp>
+#include <apps/opencs/model/world/data.hpp>
+#include <apps/opencs/view/world/genericcreator.hpp>
+
 #include <components/esm3/variant.hpp>
 
 #include "../../model/world/commands.hpp"
 #include "../../model/world/idtable.hpp"
 
+class QUndoStack;
+
 namespace CSVWorld
 {
-    void GlobalCreator::configureCreateCommand (CSMWorld::CreateCommand& command) const
+    void GlobalCreator::configureCreateCommand(CSMWorld::CreateCommand& command) const
     {
         CSMWorld::IdTable* table = static_cast<CSMWorld::IdTable*>(getData().getTableModel(getCollectionId()));
 
@@ -18,7 +24,7 @@ namespace CSVWorld
     }
 
     GlobalCreator::GlobalCreator(CSMWorld::Data& data, QUndoStack& undoStack, const CSMWorld::UniversalId& id)
-        : GenericCreator (data, undoStack, id, true)
+        : GenericCreator(data, undoStack, id, true)
     {
     }
 }
