@@ -1,10 +1,11 @@
-#include <components/version/version.hpp>
-#include <components/files/configurationmanager.hpp>
+#include <components/debug/debugging.hpp>
 #include <components/fallback/fallback.hpp>
 #include <components/fallback/validate.hpp>
-#include <components/debug/debugging.hpp>
+#include <components/files/configurationmanager.hpp>
+#include <components/misc/osgpluginchecker.hpp>
 #include <components/misc/rng.hpp>
 #include <components/platform/platform.hpp>
+#include <components/version/version.hpp>
 
 #include "mwgui/debugwindow.hpp"
 
@@ -14,25 +15,19 @@
 #include <boost/program_options/variables_map.hpp>
 
 #if defined(_WIN32)
-#include <components/windows.hpp>
+#include <components/misc/windows.hpp>
 // makes __argc and __argv available on windows
 #include <cstdlib>
 
 extern "C" __declspec(dllexport) DWORD AmdPowerXpressRequestHighPerformance = 0x00000001;
 #endif
 
+#include <filesystem>
+
 #if (defined(__APPLE__) || defined(__linux) || defined(__unix) || defined(__posix))
 #include <unistd.h>
 #endif
 
-
-using namespace Fallback;
-
-namespace
-{
-    constexpr std::string_view applicationName = "OpenMW";
-}
-
 /**
  * \brief Parses application command line and calls \ref Cfg::ConfigurationManager
  * to parse configuration files.
@@ -42,7 +37,7 @@ namespace
  * \retval true - Everything goes OK
  * \retval false - Error
  */
-bool parseOptions (int argc, char** argv, OMW::Engine& engine, Files::ConfigurationManager& cfgMgr)
+bool parseOptions(int argc, char** argv, OMW::Engine& engine, Files::ConfigurationManager& cfgMgr)
 {
     // Create a local alias for brevity
     namespace bpo = boost::program_options;
@@ -54,30 +49,29 @@ bool parseOptions (int argc, char** argv, OMW::Engine& engine, Files::Configurat
     Files::parseArgs(argc, argv, variables, desc);
     bpo::notify(variables);
 
-    if (variables.count ("help"))
+    if (variables.count("help"))
     {
-        getRawStdout() << desc << std::endl;
+        Debug::getRawStdout() << desc << std::endl;
         return false;
     }
 
-    if (variables.count ("version"))
+    if (variables.count("version"))
     {
-        cfgMgr.readConfiguration(variables, desc, true);
-
-        Version::Version v = Version::getOpenmwVersion(variables["resources"].as<Files::MaybeQuotedPath>().string());
-        getRawStdout() << v.describe() << std::endl;
+        Debug::getRawStdout() << Version::getOpenmwVersionDescription() << std::endl;
         return false;
     }
 
+    cfgMgr.processPaths(variables, std::filesystem::current_path());
+
     cfgMgr.readConfiguration(variables, desc);
+
+    Debug::setupLogging(cfgMgr.getLogPath(), "OpenMW");
+    Log(Debug::Info) << Version::getOpenmwVersionDescription();
+
     Settings::Manager::load(cfgMgr);
 
-    setupLogging(cfgMgr.getLogPath().string(), applicationName);
     MWGui::DebugWindow::startLogRecording();
 
-    Version::Version v = Version::getOpenmwVersion(variables["resources"].as<Files::MaybeQuotedPath>().string());
-    Log(Debug::Info) << v.describe();
-
     engine.setGrabMouse(!variables["no-grab"].as<bool>());
 
     // Font encoding settings
@@ -85,18 +79,21 @@ bool parseOptions (int argc, char** argv, OMW::Engine& engine, Files::Configurat
     Log(Debug::Info) << ToUTF8::encodingUsingMessage(encoding);
     engine.setEncoding(ToUTF8::calculateEncoding(encoding));
 
-    // directory settings
-    engine.enableFSStrict(variables["fs-strict"].as<bool>());
-
     Files::PathContainer dataDirs(asPathContainer(variables["data"].as<Files::MaybeQuotedPathContainer>()));
 
-    Files::PathContainer::value_type local(variables["data-local"].as<Files::MaybeQuotedPathContainer::value_type>());
+    Files::PathContainer::value_type local(variables["data-local"]
+                                               .as<Files::MaybeQuotedPathContainer::value_type>()
+                                               .u8string()); // This call to u8string is redundant, but required to
+                                                             // build on MSVC 14.26 due to implementation bugs.
     if (!local.empty())
         dataDirs.push_back(local);
 
     cfgMgr.filterOutNonExistingPaths(dataDirs);
 
-    engine.setResourceDir(variables["resources"].as<Files::MaybeQuotedPath>());
+    engine.setResourceDir(variables["resources"]
+                              .as<Files::MaybeQuotedPath>()
+                              .u8string()); // This call to u8string is redundant, but required to build on MSVC 14.26
+                                            // due to implementation bugs.
     engine.setDataDirs(dataDirs);
 
     // fallback archives
@@ -112,7 +109,8 @@ bool parseOptions (int argc, char** argv, OMW::Engine& engine, Files::Configurat
         Log(Debug::Error) << "No content file given (esm/esp, nor omwgame/omwaddon). Aborting...";
         return false;
     }
-    std::set<std::string> contentDedupe;
+    engine.addContentFile("builtin.omwscripts");
+    std::set<std::string> contentDedupe{ "builtin.omwscripts" };
     for (const auto& contentFile : content)
     {
         if (!contentDedupe.insert(contentFile).second)
@@ -141,24 +139,23 @@ bool parseOptions (int argc, char** argv, OMW::Engine& engine, Files::Configurat
 
     // startup-settings
     engine.setCell(variables["start"].as<std::string>());
-    engine.setSkipMenu (variables["skip-menu"].as<bool>(), variables["new-game"].as<bool>());
+    engine.setSkipMenu(variables["skip-menu"].as<bool>(), variables["new-game"].as<bool>());
     if (!variables["skip-menu"].as<bool>() && variables["new-game"].as<bool>())
         Log(Debug::Warning) << "Warning: new-game used without skip-menu -> ignoring it";
 
     // scripts
     engine.setCompileAll(variables["script-all"].as<bool>());
     engine.setCompileAllDialogue(variables["script-all-dialogue"].as<bool>());
-    engine.setScriptConsoleMode (variables["script-console"].as<bool>());
-    engine.setStartupScript (variables["script-run"].as<std::string>());
-    engine.setWarningsMode (variables["script-warn"].as<int>());
-    engine.setScriptBlacklist (variables["script-blacklist"].as<StringsVector>());
-    engine.setScriptBlacklistUse (variables["script-blacklist-use"].as<bool>());
-    engine.setSaveGameFile (variables["load-savegame"].as<Files::MaybeQuotedPath>().string());
+    engine.setScriptConsoleMode(variables["script-console"].as<bool>());
+    engine.setStartupScript(variables["script-run"].as<std::string>());
+    engine.setWarningsMode(variables["script-warn"].as<int>());
+    engine.setSaveGameFile(variables["load-savegame"].as<Files::MaybeQuotedPath>().u8string());
 
     // other settings
-    Fallback::Map::init(variables["fallback"].as<FallbackMap>().mMap);
+    Fallback::Map::init(variables["fallback"].as<Fallback::FallbackMap>().mMap);
     engine.setSoundUsage(!variables["no-sound"].as<bool>());
-    engine.setActivationDistanceOverride (variables["activate-dist"].as<int>());
+    engine.setActivationDistanceOverride(variables["activate-dist"].as<int>());
+    engine.enableFontExport(variables["export-fonts"].as<bool>());
     engine.setRandomSeed(variables["random-seed"].as<unsigned int>());
 
     return true;
@@ -178,21 +175,21 @@ namespace
             Debug::Level level;
             switch (severity)
             {
-            case osg::ALWAYS:
-            case osg::FATAL:
-                level = Debug::Error;
-                break;
-            case osg::WARN:
-            case osg::NOTICE:
-                level = Debug::Warning;
-                break;
-            case osg::INFO:
-                level = Debug::Info;
-                break;
-            case osg::DEBUG_INFO:
-            case osg::DEBUG_FP:
-            default:
-                level = Debug::Debug;
+                case osg::ALWAYS:
+                case osg::FATAL:
+                    level = Debug::Error;
+                    break;
+                case osg::WARN:
+                case osg::NOTICE:
+                    level = Debug::Warning;
+                    break;
+                case osg::INFO:
+                    level = Debug::Info;
+                    break;
+                case osg::DEBUG_INFO:
+                case osg::DEBUG_FP:
+                default:
+                    level = Debug::Debug;
             }
             std::string_view s(msgCopy);
             if (s.size() < 1024)
@@ -212,13 +209,11 @@ namespace
     };
 }
 
-int runApplication(int argc, char *argv[])
+int runApplication(int argc, char* argv[])
 {
     Platform::init();
 
 #ifdef __APPLE__
-    boost::filesystem::path binary_path = boost::filesystem::system_complete(boost::filesystem::path(argv[0]));
-    boost::filesystem::current_path(binary_path.parent_path());
     setenv("OSG_GL_TEXTURE_STORAGE", "OFF", 0);
 #endif
 
@@ -226,8 +221,13 @@ int runApplication(int argc, char *argv[])
     Files::ConfigurationManager cfgMgr;
     std::unique_ptr<OMW::Engine> engine = std::make_unique<OMW::Engine>(cfgMgr);
 
+    engine->setRecastMaxLogLevel(Debug::getRecastMaxLogLevel());
+
     if (parseOptions(argc, argv, *engine, cfgMgr))
     {
+        if (!Misc::checkRequiredOSGPluginsArePresent())
+            return 1;
+
         engine->go();
     }
 
@@ -235,12 +235,12 @@ int runApplication(int argc, char *argv[])
 }
 
 #ifdef ANDROID
-extern "C" int SDL_main(int argc, char**argv)
+extern "C" int SDL_main(int argc, char** argv)
 #else
-int main(int argc, char**argv)
+int main(int argc, char** argv)
 #endif
 {
-    return wrapApplication(&runApplication, argc, argv, applicationName);
+    return Debug::wrapApplication(&runApplication, argc, argv, "OpenMW");
 }
 
 // Platform specific for Windows when there is no console built into the executable.
