@@ -1,7 +1,9 @@
 #include "bodypart.hpp"
 
-#include "../mwrender/renderinginterface.hpp"
+#include <components/esm3/loadbody.hpp>
+
 #include "../mwrender/objects.hpp"
+#include "../mwrender/renderinginterface.hpp"
 
 #include "../mwworld/cellstore.hpp"
 
@@ -14,23 +16,25 @@ namespace MWClass
     {
     }
 
-    MWWorld::Ptr BodyPart::copyToCellImpl(const MWWorld::ConstPtr &ptr, MWWorld::CellStore &cell) const
+    MWWorld::Ptr BodyPart::copyToCellImpl(const MWWorld::ConstPtr& ptr, MWWorld::CellStore& cell) const
     {
-        const MWWorld::LiveCellRef<ESM::BodyPart> *ref = ptr.get<ESM::BodyPart>();
+        const MWWorld::LiveCellRef<ESM::BodyPart>* ref = ptr.get<ESM::BodyPart>();
 
         return MWWorld::Ptr(cell.insert(ref), &cell);
     }
 
-    void BodyPart::insertObjectRendering(const MWWorld::Ptr &ptr, const std::string &model, MWRender::RenderingInterface &renderingInterface) const
+    void BodyPart::insertObjectRendering(
+        const MWWorld::Ptr& ptr, const std::string& model, MWRender::RenderingInterface& renderingInterface) const
     {
-        if (!model.empty()) {
+        if (!model.empty())
+        {
             renderingInterface.getObjects().insertModel(ptr, model);
         }
     }
 
-    std::string BodyPart::getName(const MWWorld::ConstPtr &ptr) const
+    std::string_view BodyPart::getName(const MWWorld::ConstPtr& ptr) const
     {
-        return std::string();
+        return {};
     }
 
     bool BodyPart::hasToolTip(const MWWorld::ConstPtr& ptr) const
@@ -38,7 +42,7 @@ namespace MWClass
         return false;
     }
 
-    std::string BodyPart::getModel(const MWWorld::ConstPtr &ptr) const
+    std::string_view BodyPart::getModel(const MWWorld::ConstPtr& ptr) const
     {
         return getClassModel<ESM::BodyPart>(ptr);
     }
