@@ -2,65 +2,70 @@
 
 #include <algorithm>
 #include <list>
+#include <optional>
 #include <sstream>
 
 #include <components/debug/debuglog.hpp>
 
-#include <components/esm3/loaddial.hpp>
-#include <components/esm3/loadinfo.hpp>
 #include <components/esm3/dialoguestate.hpp>
 #include <components/esm3/esmwriter.hpp>
+#include <components/esm3/loaddial.hpp>
+#include <components/esm3/loadfact.hpp>
+#include <components/esm3/loadinfo.hpp>
 
-#include <components/compiler/exception.hpp>
 #include <components/compiler/errorhandler.hpp>
-#include <components/compiler/scanner.hpp>
+#include <components/compiler/exception.hpp>
 #include <components/compiler/locals.hpp>
 #include <components/compiler/output.hpp>
+#include <components/compiler/scanner.hpp>
 #include <components/compiler/scriptparser.hpp>
 
-#include <components/interpreter/interpreter.hpp>
 #include <components/interpreter/defines.hpp>
+#include <components/interpreter/interpreter.hpp>
 
-#include <components/settings/settings.hpp>
+#include <components/misc/resourcehelpers.hpp>
+
+#include <components/settings/values.hpp>
 
 #include "../mwbase/environment.hpp"
-#include "../mwbase/world.hpp"
 #include "../mwbase/journal.hpp"
-#include "../mwbase/scriptmanager.hpp"
-#include "../mwbase/windowmanager.hpp"
 #include "../mwbase/mechanicsmanager.hpp"
+#include "../mwbase/scriptmanager.hpp"
 #include "../mwbase/soundmanager.hpp"
+#include "../mwbase/windowmanager.hpp"
+#include "../mwbase/world.hpp"
 
 #include "../mwworld/class.hpp"
 #include "../mwworld/containerstore.hpp"
 #include "../mwworld/esmstore.hpp"
 
 #include "../mwscript/compilercontext.hpp"
-#include "../mwscript/interpretercontext.hpp"
 #include "../mwscript/extensions.hpp"
+#include "../mwscript/interpretercontext.hpp"
 
+#include "../mwmechanics/actorutil.hpp"
 #include "../mwmechanics/creaturestats.hpp"
 #include "../mwmechanics/npcstats.hpp"
-#include "../mwmechanics/actorutil.hpp"
 
 #include "filter.hpp"
 #include "hypertextparser.hpp"
 
 namespace MWDialogue
 {
-    DialogueManager::DialogueManager (const Compiler::Extensions& extensions, Translation::Storage& translationDataStorage) :
-      mTranslationDataStorage(translationDataStorage)
-      , mCompilerContext (MWScript::CompilerContext::Type_Dialogue)
-      , mErrorHandler()
-      , mTalkedTo(false)
-      , mOriginalDisposition(0)
-      , mCurrentDisposition(0)
-      , mPermanentDispositionChange(0)
+    DialogueManager::DialogueManager(
+        const Compiler::Extensions& extensions, Translation::Storage& translationDataStorage)
+        : mTranslationDataStorage(translationDataStorage)
+        , mCompilerContext(MWScript::CompilerContext::Type_Dialogue)
+        , mErrorHandler()
+        , mTalkedTo(false)
+        , mOriginalDisposition(0)
+        , mCurrentDisposition(0)
+        , mPermanentDispositionChange(0)
     {
         mChoice = -1;
         mIsInChoice = false;
         mGoodbye = false;
-        mCompilerContext.setExtensions (&extensions);
+        mCompilerContext.setExtensions(&extensions);
     }
 
     void DialogueManager::clear()
@@ -72,14 +77,14 @@ namespace MWDialogue
         mPermanentDispositionChange = 0;
     }
 
-    void DialogueManager::addTopic(std::string_view topic)
+    void DialogueManager::addTopic(const ESM::RefId& topic)
     {
-        mKnownTopics.insert( Misc::StringUtils::lowerCase(topic) );
+        mKnownTopics.insert(topic);
     }
 
-    std::vector<std::string> DialogueManager::parseTopicIdsFromText (const std::string& text)
+    std::vector<ESM::RefId> DialogueManager::parseTopicIdsFromText(const std::string& text)
     {
-        std::vector<std::string> topicIdList;
+        std::vector<ESM::RefId> topicIdList;
 
         std::vector<HyperTextParser::Token> hypertext = HyperTextParser::parseHyperText(text);
 
@@ -91,36 +96,36 @@ namespace MWDialogue
             {
                 // calculation of standard form for all hyperlinks
                 size_t asterisk_count = HyperTextParser::removePseudoAsterisks(topicId);
-                for(; asterisk_count > 0; --asterisk_count)
+                for (; asterisk_count > 0; --asterisk_count)
                     topicId.append("*");
 
                 topicId = mTranslationDataStorage.topicStandardForm(topicId);
             }
 
-            topicIdList.push_back(topicId);
+            topicIdList.push_back(ESM::RefId::stringRefId(topicId));
         }
 
         return topicIdList;
     }
 
-    void DialogueManager::addTopicsFromText (const std::string& text)
+    void DialogueManager::addTopicsFromText(const std::string& text)
     {
         updateActorKnownTopics();
 
         for (const auto& topicId : parseTopicIdsFromText(text))
         {
-            if (mActorKnownTopics.count( topicId ))
-                mKnownTopics.insert( topicId );
+            if (mActorKnownTopics.count(topicId))
+                mKnownTopics.insert(topicId);
         }
     }
 
     void DialogueManager::updateOriginalDisposition()
     {
-        if(mActor.getClass().isNpc())
+        if (mActor.getClass().isNpc())
         {
             const auto& stats = mActor.getClass().getNpcStats(mActor);
             // Disposition changed by script; discard our preconceived notions
-            if(stats.getBaseDisposition() != mCurrentDisposition)
+            if (stats.getBaseDisposition() != mCurrentDisposition)
             {
                 mCurrentDisposition = stats.getBaseDisposition();
                 mOriginalDisposition = mCurrentDisposition;
@@ -128,7 +133,7 @@ namespace MWDialogue
         }
     }
 
-    bool DialogueManager::startDialogue (const MWWorld::Ptr& actor, ResponseCallback* callback)
+    bool DialogueManager::startDialogue(const MWWorld::Ptr& actor, ResponseCallback* callback)
     {
         updateGlobals();
 
@@ -136,7 +141,7 @@ namespace MWDialogue
         if (actor.getClass().getCreatureStats(actor).isDead())
             return false;
 
-        mLastTopic.clear();
+        mLastTopic = ESM::RefId();
         // Note that we intentionally don't reset mPermanentDispositionChange
 
         mChoice = -1;
@@ -146,23 +151,22 @@ namespace MWDialogue
 
         mActor = actor;
 
-        MWMechanics::CreatureStats& creatureStats = actor.getClass().getCreatureStats (actor);
+        MWMechanics::CreatureStats& creatureStats = actor.getClass().getCreatureStats(actor);
         mTalkedTo = creatureStats.hasTalkedToPlayer();
 
         mActorKnownTopics.clear();
 
-        //greeting
-        const MWWorld::Store<ESM::Dialogue> &dialogs =
-            MWBase::Environment::get().getWorld()->getStore().get<ESM::Dialogue>();
+        // greeting
+        const MWWorld::Store<ESM::Dialogue>& dialogs = MWBase::Environment::get().getESMStore()->get<ESM::Dialogue>();
 
-        Filter filter (actor, mChoice, mTalkedTo);
+        Filter filter(actor, mChoice, mTalkedTo);
 
-        for (MWWorld::Store<ESM::Dialogue>::iterator it = dialogs.begin(); it != dialogs.end(); ++it)
+        for (const ESM::Dialogue& dialogue : dialogs)
         {
-            if(it->mType == ESM::Dialogue::Greeting)
+            if (dialogue.mType == ESM::Dialogue::Greeting)
             {
                 // Search a response (we do not accept a fallback to "Info refusal" here)
-                if (const ESM::DialInfo *info = filter.search (*it, false))
+                if (const ESM::DialInfo* info = filter.search(dialogue, false).second)
                 {
                     creatureStats.talkedToPlayer();
 
@@ -171,12 +175,12 @@ namespace MWDialogue
                         // TODO play sound
                     }
 
-                    MWScript::InterpreterContext interpreterContext(&mActor.getRefData().getLocals(),mActor);
-                    callback->addResponse("", Interpreter::fixDefinesDialog(info->mResponse, interpreterContext));
-                    executeScript (info->mResultScript, mActor);
-                    mLastTopic = it->mId;
+                    MWScript::InterpreterContext interpreterContext(&mActor.getRefData().getLocals(), mActor);
+                    callback->addResponse({}, Interpreter::fixDefinesDialog(info->mResponse, interpreterContext));
+                    executeScript(info->mResultScript, mActor);
+                    mLastTopic = dialogue.mId;
 
-                    addTopicsFromText (info->mResponse);
+                    addTopicsFromText(info->mResponse);
 
                     return true;
                 }
@@ -185,9 +189,10 @@ namespace MWDialogue
         return false;
     }
 
-    bool DialogueManager::compile (const std::string& cmd, std::vector<Interpreter::Type_Code>& code, const MWWorld::Ptr& actor)
+    std::optional<Interpreter::Program> DialogueManager::compile(const std::string& cmd, const MWWorld::Ptr& actor)
     {
         bool success = true;
+        std::optional<Interpreter::Program> program;
 
         try
         {
@@ -195,29 +200,29 @@ namespace MWDialogue
 
             mErrorHandler.setContext("[dialogue script]");
 
-            std::istringstream input (cmd + "\n");
+            std::istringstream input(cmd + "\n");
 
-            Compiler::Scanner scanner (mErrorHandler, input, mCompilerContext.getExtensions());
+            Compiler::Scanner scanner(mErrorHandler, input, mCompilerContext.getExtensions());
 
             Compiler::Locals locals;
 
-            std::string actorScript = actor.getClass().getScript (actor);
+            const ESM::RefId& actorScript = actor.getClass().getScript(actor);
 
             if (!actorScript.empty())
             {
                 // grab local variables from actor's script, if available.
-                locals = MWBase::Environment::get().getScriptManager()->getLocals (actorScript);
+                locals = MWBase::Environment::get().getScriptManager()->getLocals(actorScript);
             }
 
-            Compiler::ScriptParser parser(mErrorHandler,mCompilerContext, locals, false);
+            Compiler::ScriptParser parser(mErrorHandler, mCompilerContext, locals, false);
 
-            scanner.scan (parser);
+            scanner.scan(parser);
 
             if (!mErrorHandler.isGood())
                 success = false;
 
             if (success)
-                parser.getCode (code);
+                program = parser.getProgram();
         }
         catch (const Compiler::SourceException& /* error */)
         {
@@ -226,7 +231,7 @@ namespace MWDialogue
         }
         catch (const std::exception& error)
         {
-            Log(Debug::Error) << std::string ("Dialogue error: An exception has been thrown: ") + error.what();
+            Log(Debug::Error) << std::string("Dialogue error: An exception has been thrown: ") + error.what();
             success = false;
         }
 
@@ -235,32 +240,31 @@ namespace MWDialogue
             Log(Debug::Error) << "Error: compiling failed (dialogue script): \n" << cmd << "\n";
         }
 
-        return success;
+        return program;
     }
 
-    void DialogueManager::executeScript (const std::string& script, const MWWorld::Ptr& actor)
+    void DialogueManager::executeScript(const std::string& script, const MWWorld::Ptr& actor)
     {
-        std::vector<Interpreter::Type_Code> code;
-        if(compile(script, code, actor))
+        if (const std::optional<Interpreter::Program> program = compile(script, actor))
         {
             try
             {
                 MWScript::InterpreterContext interpreterContext(&actor.getRefData().getLocals(), actor);
                 Interpreter::Interpreter interpreter;
-                MWScript::installOpcodes (interpreter);
-                interpreter.run (&code[0], code.size(), interpreterContext);
+                MWScript::installOpcodes(interpreter);
+                interpreter.run(*program, interpreterContext);
             }
             catch (const std::exception& error)
             {
-               Log(Debug::Error) << std::string ("Dialogue error: An exception has been thrown: ") + error.what();
+                Log(Debug::Error) << std::string("Dialogue error: An exception has been thrown: ") + error.what();
             }
         }
     }
 
-    bool DialogueManager::inJournal(const std::string& topicId, const std::string& infoId) const
+    bool DialogueManager::inJournal(const ESM::RefId& topicId, const ESM::RefId& infoId) const
     {
-        const MWDialogue::Topic *topicHistory = nullptr;
-        MWBase::Journal *journal = MWBase::Environment::get().getJournal();
+        const MWDialogue::Topic* topicHistory = nullptr;
+        MWBase::Journal* journal = MWBase::Environment::get().getJournal();
         for (auto it = journal->topicBegin(); it != journal->topicEnd(); ++it)
         {
             if (it->first == topicId)
@@ -273,7 +277,7 @@ namespace MWDialogue
         if (!topicHistory)
             return false;
 
-        for(const auto& topic : *topicHistory)
+        for (const auto& topic : *topicHistory)
         {
             if (topic.mInfoId == infoId)
                 return true;
@@ -281,66 +285,58 @@ namespace MWDialogue
         return false;
     }
 
-    void DialogueManager::executeTopic (const std::string& topic, ResponseCallback* callback)
+    void DialogueManager::executeTopic(const ESM::RefId& topic, ResponseCallback* callback)
     {
-        Filter filter (mActor, mChoice, mTalkedTo);
+        Filter filter(mActor, mChoice, mTalkedTo);
 
-        const MWWorld::Store<ESM::Dialogue> &dialogues =
-            MWBase::Environment::get().getWorld()->getStore().get<ESM::Dialogue>();
+        const MWWorld::Store<ESM::Dialogue>& dialogues = MWBase::Environment::get().getESMStore()->get<ESM::Dialogue>();
 
-        const ESM::Dialogue& dialogue = *dialogues.find (topic);
+        const ESM::Dialogue& dialogue = *dialogues.find(topic);
+
+        const auto [responseTopic, info] = filter.search(dialogue, true);
 
-        const ESM::DialInfo* info = filter.search(dialogue, true);
-        
         if (info)
         {
-            std::string title;
-            if (dialogue.mType==ESM::Dialogue::Persuasion)
+            std::string_view title;
+            if (dialogue.mType == ESM::Dialogue::Persuasion)
             {
                 // Determine GMST from dialogue topic. GMSTs are:
                 // sAdmireSuccess, sAdmireFail, sIntimidateSuccess, sIntimidateFail,
                 // sTauntSuccess, sTauntFail, sBribeSuccess, sBribeFail
-                std::string modifiedTopic = "s" + topic;
+                std::string modifiedTopic = "s" + topic.getRefIdString();
 
-                modifiedTopic.erase (std::remove (modifiedTopic.begin(), modifiedTopic.end(), ' '), modifiedTopic.end());
+                modifiedTopic.erase(std::remove(modifiedTopic.begin(), modifiedTopic.end(), ' '), modifiedTopic.end());
 
-                const MWWorld::Store<ESM::GameSetting>& gmsts =
-                    MWBase::Environment::get().getWorld()->getStore().get<ESM::GameSetting>();
+                const MWWorld::Store<ESM::GameSetting>& gmsts
+                    = MWBase::Environment::get().getESMStore()->get<ESM::GameSetting>();
 
-                title = gmsts.find (modifiedTopic)->mValue.getString();
+                title = gmsts.find(modifiedTopic)->mValue.getString();
             }
             else
-                title = topic;
+                title = dialogue.mStringId;
 
-            MWScript::InterpreterContext interpreterContext(&mActor.getRefData().getLocals(),mActor);
+            MWScript::InterpreterContext interpreterContext(&mActor.getRefData().getLocals(), mActor);
             callback->addResponse(title, Interpreter::fixDefinesDialog(info->mResponse, interpreterContext));
 
             if (dialogue.mType == ESM::Dialogue::Topic)
             {
-                // Make sure the returned DialInfo is from the Dialogue we supplied. If could also be from the Info refusal group,
-                // in which case it should not be added to the journal.
-                for (ESM::Dialogue::InfoContainer::const_iterator iter = dialogue.mInfo.begin();
-                    iter!=dialogue.mInfo.end(); ++iter)
-                {
-                    if (iter->mId == info->mId)
-                    {
-                        MWBase::Environment::get().getJournal()->addTopic (Misc::StringUtils::lowerCase(topic), info->mId, mActor);
-                        break;
-                    }
-                }
+                // Make sure the returned DialInfo is from the Dialogue we supplied. If could also be from the Info
+                // refusal group, in which case it should not be added to the journal.
+                if (responseTopic == &dialogue)
+                    MWBase::Environment::get().getJournal()->addTopic(topic, info->mId, mActor);
             }
 
             mLastTopic = topic;
 
-            executeScript (info->mResultScript, mActor);
+            executeScript(info->mResultScript, mActor);
 
-            addTopicsFromText (info->mResponse);
+            addTopicsFromText(info->mResponse);
         }
     }
 
-    const ESM::Dialogue *DialogueManager::searchDialogue(const std::string& id)
+    const ESM::Dialogue* DialogueManager::searchDialogue(const ESM::RefId& id)
     {
-        return MWBase::Environment::get().getWorld()->getStore().get<ESM::Dialogue>().search(id);
+        return MWBase::Environment::get().getESMStore()->get<ESM::Dialogue>().search(id);
     }
 
     void DialogueManager::updateGlobals()
@@ -354,31 +350,30 @@ namespace MWDialogue
 
         mActorKnownTopics.clear();
 
-        const auto& dialogs = MWBase::Environment::get().getWorld()->getStore().get<ESM::Dialogue>();
+        const auto& dialogs = MWBase::Environment::get().getESMStore()->get<ESM::Dialogue>();
 
-        Filter filter (mActor, -1, mTalkedTo);
+        Filter filter(mActor, -1, mTalkedTo);
 
         for (const auto& dialog : dialogs)
         {
             if (dialog.mType == ESM::Dialogue::Topic)
             {
-                const auto* answer = filter.search(dialog, true);
-                auto topicId = Misc::StringUtils::lowerCase(dialog.mId);
+                const auto* answer = filter.search(dialog, true).second;
+                const auto& topicId = dialog.mId;
 
                 if (answer != nullptr)
                 {
                     int topicFlags = 0;
-                    if(!inJournal(topicId, answer->mId))
+                    if (!inJournal(topicId, answer->mId))
                     {
                         // Does this dialogue contains some actor-specific answer?
-                        if (Misc::StringUtils::ciEqual(answer->mActor, mActor.getCellRef().getRefId()))
+                        if (answer->mActor == mActor.getCellRef().getRefId())
                             topicFlags |= MWBase::DialogueManager::TopicType::Specific;
                     }
                     else
                         topicFlags |= MWBase::DialogueManager::TopicType::Exhausted;
-                    mActorKnownTopics.insert (std::make_pair(dialog.mId, ActorKnownTopicInfo {topicFlags, answer}));
+                    mActorKnownTopics.insert(std::make_pair(dialog.mId, ActorKnownTopicInfo{ topicFlags, answer }));
                 }
-
             }
         }
 
@@ -389,13 +384,12 @@ namespace MWDialogue
             // If the topic is not marked as exhausted, we don't need to do anything about it.
             // If the topic will not be shown to the player, the flag actually does not matter.
 
-            if (!(topicInfo.mFlags & MWBase::DialogueManager::TopicType::Exhausted) ||
-                !mKnownTopics.count(dialogId))
+            if (!(topicInfo.mFlags & MWBase::DialogueManager::TopicType::Exhausted) || !mKnownTopics.count(dialogId))
                 continue;
 
             for (const auto& topicId : parseTopicIdsFromText(topicInfo.mInfo->mResponse))
             {
-                if (mActorKnownTopics.count( topicId ) && !mKnownTopics.count( topicId ))
+                if (mActorKnownTopics.count(topicId) && !mKnownTopics.count(topicId))
                 {
                     topicInfo.mFlags &= ~MWBase::DialogueManager::TopicType::Exhausted;
                     break;
@@ -409,12 +403,12 @@ namespace MWDialogue
         updateActorKnownTopics();
 
         std::list<std::string> keywordList;
-
+        const auto& store = MWBase::Environment::get().getESMStore()->get<ESM::Dialogue>();
         for (const auto& [topic, topicInfo] : mActorKnownTopics)
         {
-            //does the player know the topic?
-            if (mKnownTopics.count(topic))
-                keywordList.push_back(topic);
+            // does the player know the topic?
+            if (mKnownTopics.contains(topic))
+                keywordList.push_back(store.find(topic)->mStringId);
         }
 
         // sort again, because the previous sort was case-sensitive
@@ -422,7 +416,7 @@ namespace MWDialogue
         return keywordList;
     }
 
-    int DialogueManager::getTopicFlag(const std::string& topicId) const
+    int DialogueManager::getTopicFlag(const ESM::RefId& topicId) const
     {
         auto known = mActorKnownTopics.find(topicId);
         if (known != mActorKnownTopics.end())
@@ -430,14 +424,14 @@ namespace MWDialogue
         return 0;
     }
 
-    void DialogueManager::keywordSelected (const std::string& keyword, ResponseCallback* callback)
+    void DialogueManager::keywordSelected(std::string_view keyword, ResponseCallback* callback)
     {
-        if(!mIsInChoice)
+        if (!mIsInChoice)
         {
-            const ESM::Dialogue* dialogue = searchDialogue(keyword);
+            const ESM::Dialogue* dialogue = searchDialogue(ESM::RefId::stringRefId(keyword));
             if (dialogue && dialogue->mType == ESM::Dialogue::Topic)
             {
-                executeTopic (keyword, callback);
+                executeTopic(dialogue->mId, callback);
             }
         }
     }
@@ -454,7 +448,8 @@ namespace MWDialogue
         {
             updateOriginalDisposition();
             MWMechanics::NpcStats& npcStats = mActor.getClass().getNpcStats(mActor);
-            // Clamp permanent disposition change so that final disposition doesn't go below 0 (could happen with intimidate)
+            // Clamp permanent disposition change so that final disposition doesn't go below 0 (could happen with
+            // intimidate)
             npcStats.setBaseDisposition(0);
             int zero = MWBase::Environment::get().getMechanicsManager()->getDerivedDisposition(mActor, false);
             int disposition = std::clamp(mOriginalDisposition + mPermanentDispositionChange, -zero, 100 - zero);
@@ -466,45 +461,39 @@ namespace MWDialogue
         mCurrentDisposition = 0;
     }
 
-    void DialogueManager::questionAnswered (int answer, ResponseCallback* callback)
+    void DialogueManager::questionAnswered(int answer, ResponseCallback* callback)
     {
         mChoice = answer;
 
         const ESM::Dialogue* dialogue = searchDialogue(mLastTopic);
         if (dialogue)
         {
-            Filter filter (mActor, mChoice, mTalkedTo);
+            Filter filter(mActor, mChoice, mTalkedTo);
 
             if (dialogue->mType == ESM::Dialogue::Topic || dialogue->mType == ESM::Dialogue::Greeting)
             {
-                if (const ESM::DialInfo *info = filter.search (*dialogue, true))
+                const auto [responseTopic, info] = filter.search(*dialogue, true);
+                if (info)
                 {
-                    std::string text = info->mResponse;
-                    addTopicsFromText (text);
+                    const std::string& text = info->mResponse;
+                    addTopicsFromText(text);
 
                     mChoice = -1;
                     mIsInChoice = false;
                     mChoices.clear();
 
-                    MWScript::InterpreterContext interpreterContext(&mActor.getRefData().getLocals(),mActor);
-                    callback->addResponse("", Interpreter::fixDefinesDialog(text, interpreterContext));
+                    MWScript::InterpreterContext interpreterContext(&mActor.getRefData().getLocals(), mActor);
+                    callback->addResponse({}, Interpreter::fixDefinesDialog(text, interpreterContext));
 
                     if (dialogue->mType == ESM::Dialogue::Topic)
                     {
-                        // Make sure the returned DialInfo is from the Dialogue we supplied. If could also be from the Info refusal group,
-                        // in which case it should not be added to the journal
-                        for (ESM::Dialogue::InfoContainer::const_iterator iter = dialogue->mInfo.begin();
-                            iter!=dialogue->mInfo.end(); ++iter)
-                        {
-                            if (iter->mId == info->mId)
-                            {
-                                MWBase::Environment::get().getJournal()->addTopic (Misc::StringUtils::lowerCase(mLastTopic), info->mId, mActor);
-                                break;
-                            }
-                        }
+                        // Make sure the returned DialInfo is from the Dialogue we supplied. If could also be from the
+                        // Info refusal group, in which case it should not be added to the journal
+                        if (responseTopic == dialogue)
+                            MWBase::Environment::get().getJournal()->addTopic(mLastTopic, info->mId, mActor);
                     }
 
-                    executeScript (info->mResultScript, mActor);
+                    executeScript(info->mResultScript, mActor);
                 }
                 else
                 {
@@ -545,21 +534,21 @@ namespace MWDialogue
         bool success;
         int temp, perm;
         MWBase::Environment::get().getMechanicsManager()->getPersuasionDispositionChange(
-                    mActor, MWBase::MechanicsManager::PersuasionType(type),
-                    success, temp, perm);
+            mActor, MWBase::MechanicsManager::PersuasionType(type), success, temp, perm);
         updateOriginalDisposition();
-        if(temp > 0 && perm > 0 && mOriginalDisposition + perm + mPermanentDispositionChange < 0)
+        if (temp > 0 && perm > 0 && mOriginalDisposition + perm + mPermanentDispositionChange < 0)
             perm = -(mOriginalDisposition + mPermanentDispositionChange);
         mCurrentDisposition += temp;
         mActor.getClass().getNpcStats(mActor).setBaseDisposition(mCurrentDisposition);
         mPermanentDispositionChange += perm;
 
         MWWorld::Ptr player = MWMechanics::getPlayer();
-        player.getClass().skillUsageSucceeded(player, ESM::Skill::Speechcraft, success ? 0 : 1);
+        player.getClass().skillUsageSucceeded(
+            player, ESM::Skill::Speechcraft, success ? ESM::Skill::Speechcraft_Success : ESM::Skill::Speechcraft_Fail);
 
         if (success)
         {
-            int gold=0;
+            int gold = 0;
             if (type == MWBase::MechanicsManager::PT_Bribe10)
                 gold = 10;
             else if (type == MWBase::MechanicsManager::PT_Bribe100)
@@ -569,8 +558,8 @@ namespace MWDialogue
 
             if (gold)
             {
-                player.getClass().getContainerStore(player).remove(MWWorld::ContainerStore::sGoldId, gold, player);
-                mActor.getClass().getContainerStore(mActor).add(MWWorld::ContainerStore::sGoldId, gold, mActor);
+                player.getClass().getContainerStore(player).remove(MWWorld::ContainerStore::sGoldId, gold);
+                mActor.getClass().getContainerStore(mActor).add(MWWorld::ContainerStore::sGoldId, gold);
             }
         }
 
@@ -582,91 +571,93 @@ namespace MWDialogue
             text = "Taunt";
         else if (type == MWBase::MechanicsManager::PT_Intimidate)
             text = "Intimidate";
-        else{
+        else
+        {
             text = "Bribe";
         }
 
-        executeTopic (text + (success ? " Success" : " Fail"), callback);
+        executeTopic(ESM::RefId::stringRefId(text + (success ? " Success" : " Fail")), callback);
     }
 
     void DialogueManager::applyBarterDispositionChange(int delta)
     {
-        if(mActor.getClass().isNpc())
+        if (!mActor.isEmpty() && mActor.getClass().isNpc())
         {
             updateOriginalDisposition();
             mCurrentDisposition += delta;
             mActor.getClass().getNpcStats(mActor).setBaseDisposition(mCurrentDisposition);
-            if (Settings::Manager::getBool("barter disposition change is permanent", "Game"))
+            if (Settings::game().mBarterDispositionChangeIsPermanent)
                 mPermanentDispositionChange += delta;
         }
     }
 
     bool DialogueManager::checkServiceRefused(ResponseCallback* callback, ServiceType service)
     {
-        Filter filter (mActor, service, mTalkedTo);
+        Filter filter(mActor, service, mTalkedTo);
 
-        const MWWorld::Store<ESM::Dialogue> &dialogues =
-            MWBase::Environment::get().getWorld()->getStore().get<ESM::Dialogue>();
+        const MWWorld::Store<ESM::Dialogue>& dialogues = MWBase::Environment::get().getESMStore()->get<ESM::Dialogue>();
 
-        const ESM::Dialogue& dialogue = *dialogues.find ("Service Refusal");
+        const ESM::Dialogue& dialogue = *dialogues.find(ESM::RefId::stringRefId("Service Refusal"));
 
-        std::vector<const ESM::DialInfo *> infos = filter.list (dialogue, false, false, true);
+        std::vector<Filter::Response> infos = filter.list(dialogue, false, false, true);
         if (!infos.empty())
         {
-            const ESM::DialInfo* info = infos[0];
+            const ESM::DialInfo* info = infos[0].second;
 
-            addTopicsFromText (info->mResponse);
+            addTopicsFromText(info->mResponse);
 
-            const MWWorld::Store<ESM::GameSetting>& gmsts =
-                MWBase::Environment::get().getWorld()->getStore().get<ESM::GameSetting>();
+            const MWWorld::Store<ESM::GameSetting>& gmsts
+                = MWBase::Environment::get().getESMStore()->get<ESM::GameSetting>();
 
-            MWScript::InterpreterContext interpreterContext(&mActor.getRefData().getLocals(),mActor);
+            MWScript::InterpreterContext interpreterContext(&mActor.getRefData().getLocals(), mActor);
 
-            callback->addResponse(gmsts.find ("sServiceRefusal")->mValue.getString(), Interpreter::fixDefinesDialog(info->mResponse, interpreterContext));
+            callback->addResponse(gmsts.find("sServiceRefusal")->mValue.getString(),
+                Interpreter::fixDefinesDialog(info->mResponse, interpreterContext));
 
-            executeScript (info->mResultScript, mActor);
+            executeScript(info->mResultScript, mActor);
             return true;
         }
         return false;
     }
 
-    void DialogueManager::say(const MWWorld::Ptr &actor, const std::string &topic)
+    bool DialogueManager::say(const MWWorld::Ptr& actor, const ESM::RefId& topic)
     {
-        MWBase::SoundManager *sndMgr = MWBase::Environment::get().getSoundManager();
-        if(sndMgr->sayActive(actor))
+        MWBase::SoundManager* sndMgr = MWBase::Environment::get().getSoundManager();
+        if (sndMgr->sayActive(actor))
         {
             // Actor is already saying something.
-            return;
+            return false;
         }
 
         if (actor.getClass().isNpc() && MWBase::Environment::get().getWorld()->isSwimming(actor))
         {
             // NPCs don't talk while submerged
-            return;
+            return false;
         }
 
         if (actor.getClass().getCreatureStats(actor).getKnockedDown())
         {
             // Unconscious actors can not speak
-            return;
+            return false;
         }
 
-        const MWWorld::ESMStore &store = MWBase::Environment::get().getWorld()->getStore();
-        const ESM::Dialogue *dial = store.get<ESM::Dialogue>().find(topic);
+        const MWWorld::ESMStore& store = *MWBase::Environment::get().getESMStore();
+        const ESM::Dialogue* dial = store.get<ESM::Dialogue>().find(topic);
 
         const MWMechanics::CreatureStats& creatureStats = actor.getClass().getCreatureStats(actor);
         Filter filter(actor, 0, creatureStats.hasTalkedToPlayer());
-        const ESM::DialInfo *info = filter.search(*dial, false);
-        if(info != nullptr)
+        const ESM::DialInfo* info = filter.search(*dial, false).second;
+        if (info != nullptr)
         {
-            MWBase::WindowManager *winMgr = MWBase::Environment::get().getWindowManager();
-            if(winMgr->getSubtitlesEnabled())
+            MWBase::WindowManager* winMgr = MWBase::Environment::get().getWindowManager();
+            if (Settings::gui().mSubtitles)
                 winMgr->messageBox(info->mResponse);
             if (!info->mSound.empty())
-                sndMgr->say(actor, info->mSound);
+                sndMgr->say(actor, Misc::ResourceHelpers::correctSoundPath(VFS::Path::Normalized(info->mSound)));
             if (!info->mResultScript.empty())
                 executeScript(info->mResultScript, actor);
         }
+        return info != nullptr;
     }
 
     int DialogueManager::countSavedGameRecords() const
@@ -674,7 +665,7 @@ namespace MWDialogue
         return 1; // known topics
     }
 
-    void DialogueManager::write (ESM::ESMWriter& writer, Loading::Listener& progress) const
+    void DialogueManager::write(ESM::ESMWriter& writer, Loading::Listener& progress) const
     {
         ESM::DialogueState state;
 
@@ -683,83 +674,88 @@ namespace MWDialogue
 
         state.mChangedFactionReaction = mChangedFactionReaction;
 
-        writer.startRecord (ESM::REC_DIAS);
-        state.save (writer);
-        writer.endRecord (ESM::REC_DIAS);
+        writer.startRecord(ESM::REC_DIAS);
+        state.save(writer);
+        writer.endRecord(ESM::REC_DIAS);
     }
 
-    void DialogueManager::readRecord (ESM::ESMReader& reader, uint32_t type)
+    void DialogueManager::readRecord(ESM::ESMReader& reader, uint32_t type)
     {
-        if (type==ESM::REC_DIAS)
+        if (type == ESM::REC_DIAS)
         {
-            const MWWorld::ESMStore& store = MWBase::Environment::get().getWorld()->getStore();
+            const MWWorld::ESMStore& store = *MWBase::Environment::get().getESMStore();
 
             ESM::DialogueState state;
-            state.load (reader);
+            state.load(reader);
 
-            for (std::vector<std::string>::const_iterator iter (state.mKnownTopics.begin());
-                iter!=state.mKnownTopics.end(); ++iter)
-                if (store.get<ESM::Dialogue>().search (*iter))
-                    mKnownTopics.insert (*iter);
+            for (const auto& knownTopic : state.mKnownTopics)
+                if (store.get<ESM::Dialogue>().search(knownTopic))
+                    mKnownTopics.insert(knownTopic);
 
             mChangedFactionReaction = state.mChangedFactionReaction;
         }
     }
 
-    void DialogueManager::modFactionReaction(std::string_view faction1, std::string_view faction2, int diff)
+    void DialogueManager::modFactionReaction(const ESM::RefId& faction1, const ESM::RefId& faction2, int diff)
     {
-        std::string fact1 = Misc::StringUtils::lowerCase(faction1);
-        std::string fact2 = Misc::StringUtils::lowerCase(faction2);
-
         // Make sure the factions exist
-        MWBase::Environment::get().getWorld()->getStore().get<ESM::Faction>().find(fact1);
-        MWBase::Environment::get().getWorld()->getStore().get<ESM::Faction>().find(fact2);
+        MWBase::Environment::get().getESMStore()->get<ESM::Faction>().find(faction1);
+        MWBase::Environment::get().getESMStore()->get<ESM::Faction>().find(faction2);
 
         int newValue = getFactionReaction(faction1, faction2) + diff;
 
-        std::map<std::string, int>& map = mChangedFactionReaction[fact1];
-        map[fact2] = newValue;
+        auto& map = mChangedFactionReaction[faction1];
+        map[faction2] = newValue;
     }
 
-    void DialogueManager::setFactionReaction(std::string_view faction1, std::string_view faction2, int absolute)
+    void DialogueManager::setFactionReaction(const ESM::RefId& faction1, const ESM::RefId& faction2, int absolute)
     {
-        std::string fact1 = Misc::StringUtils::lowerCase(faction1);
-        std::string fact2 = Misc::StringUtils::lowerCase(faction2);
-
         // Make sure the factions exist
-        MWBase::Environment::get().getWorld()->getStore().get<ESM::Faction>().find(fact1);
-        MWBase::Environment::get().getWorld()->getStore().get<ESM::Faction>().find(fact2);
+        MWBase::Environment::get().getESMStore()->get<ESM::Faction>().find(faction1);
+        MWBase::Environment::get().getESMStore()->get<ESM::Faction>().find(faction2);
 
-        std::map<std::string, int>& map = mChangedFactionReaction[fact1];
-        map[fact2] = absolute;
+        auto& map = mChangedFactionReaction[faction1];
+        map[faction2] = absolute;
     }
 
-    int DialogueManager::getFactionReaction(std::string_view faction1, std::string_view faction2) const
+    int DialogueManager::getFactionReaction(const ESM::RefId& faction1, const ESM::RefId& faction2) const
     {
-        std::string fact1 = Misc::StringUtils::lowerCase(faction1);
-        std::string fact2 = Misc::StringUtils::lowerCase(faction2);
+        ModFactionReactionMap::const_iterator map = mChangedFactionReaction.find(faction1);
+        if (map != mChangedFactionReaction.end())
+        {
+            auto it = map->second.find(faction2);
+            if (it != map->second.end())
+                return it->second;
+        }
 
-        ModFactionReactionMap::const_iterator map = mChangedFactionReaction.find(fact1);
-        if (map != mChangedFactionReaction.end() && map->second.find(fact2) != map->second.end())
-            return map->second.at(fact2);
+        const ESM::Faction* faction = MWBase::Environment::get().getESMStore()->get<ESM::Faction>().find(faction1);
 
-        const ESM::Faction* faction = MWBase::Environment::get().getWorld()->getStore().get<ESM::Faction>().find(fact1);
-
-        std::map<std::string, int>::const_iterator it = faction->mReactions.begin();
+        auto it = faction->mReactions.begin();
         for (; it != faction->mReactions.end(); ++it)
         {
-            if (Misc::StringUtils::ciEqual(it->first, fact2))
-                    return it->second;
+            if (it->first == faction2)
+                return it->second;
         }
         return 0;
     }
 
-    void DialogueManager::clearInfoActor(const MWWorld::Ptr &actor) const
+    const std::map<ESM::RefId, int>* DialogueManager::getFactionReactionOverrides(const ESM::RefId& faction) const
+    {
+        // Make sure the faction exists
+        MWBase::Environment::get().getESMStore()->get<ESM::Faction>().find(faction);
+
+        const auto found = mChangedFactionReaction.find(faction);
+        if (found != mChangedFactionReaction.end())
+            return &found->second;
+        return nullptr;
+    }
+
+    void DialogueManager::clearInfoActor(const MWWorld::Ptr& actor) const
     {
         if (actor == mActor && !mLastTopic.empty())
         {
             MWBase::Environment::get().getJournal()->removeLastAddedTopicResponse(
-                        Misc::StringUtils::lowerCase(mLastTopic), actor.getClass().getName(actor));
+                mLastTopic, actor.getClass().getName(actor));
         }
     }
 }
