@@ -11,19 +11,23 @@ namespace MWGui
      */
     class BackgroundImage final : public MyGUI::ImageBox
     {
-    MYGUI_RTTI_DERIVED(BackgroundImage)
+        MYGUI_RTTI_DERIVED(BackgroundImage)
 
     public:
-        BackgroundImage() : mChild(nullptr), mAspect(0) {}
+        BackgroundImage()
+            : mChild(nullptr)
+            , mAspect(0)
+        {
+        }
 
         /**
          * @param fixedRatio Use a fixed ratio of 4:3, regardless of the image dimensions
          * @param stretch Stretch to fill the whole screen, or add black bars?
          */
-        void setBackgroundImage (const std::string& image, bool fixedRatio=true, bool stretch=true);
+        void setBackgroundImage(const std::string& image, bool fixedRatio = true, bool stretch = true);
 
-        void setSize (const MyGUI::IntSize &_value) override;
-        void setCoord (const MyGUI::IntCoord &_value) override;
+        void setSize(const MyGUI::IntSize& _value) override;
+        void setCoord(const MyGUI::IntCoord& _value) override;
 
     private:
         MyGUI::ImageBox* mChild;
