@@ -7,7 +7,7 @@ namespace
 
     void modifyProfit(const MWWorld::Ptr& actor, int diff)
     {
-        std::string script = actor.getClass().getScript(actor);
+        const ESM::RefId& script = actor.getClass().getScript(actor);
         if (!script.empty())
         {
             int profit = actor.getRefData().getLocals().getIntVar(script, "minimumprofit");
@@ -20,12 +20,20 @@ namespace
 
 namespace MWGui
 {
-    CompanionItemModel::CompanionItemModel(const MWWorld::Ptr &actor)
+    CompanionItemModel::CompanionItemModel(const MWWorld::Ptr& actor)
         : InventoryItemModel(actor)
     {
     }
 
-    MWWorld::Ptr CompanionItemModel::copyItem (const ItemStack& item, size_t count, bool allowAutoEquip)
+    MWWorld::Ptr CompanionItemModel::addItem(const ItemStack& item, size_t count, bool allowAutoEquip)
+    {
+        if (hasProfit(mActor))
+            modifyProfit(mActor, item.mBase.getClass().getValue(item.mBase) * count);
+
+        return InventoryItemModel::addItem(item, count, allowAutoEquip);
+    }
+
+    MWWorld::Ptr CompanionItemModel::copyItem(const ItemStack& item, size_t count, bool allowAutoEquip)
     {
         if (hasProfit(mActor))
             modifyProfit(mActor, item.mBase.getClass().getValue(item.mBase) * count);
@@ -33,7 +41,7 @@ namespace MWGui
         return InventoryItemModel::copyItem(item, count, allowAutoEquip);
     }
 
-    void CompanionItemModel::removeItem (const ItemStack& item, size_t count)
+    void CompanionItemModel::removeItem(const ItemStack& item, size_t count)
     {
         if (hasProfit(mActor))
             modifyProfit(mActor, -item.mBase.getClass().getValue(item.mBase) * count);
@@ -41,9 +49,9 @@ namespace MWGui
         InventoryItemModel::removeItem(item, count);
     }
 
-    bool CompanionItemModel::hasProfit(const MWWorld::Ptr &actor)
+    bool CompanionItemModel::hasProfit(const MWWorld::Ptr& actor)
     {
-        std::string script = actor.getClass().getScript(actor);
+        const ESM::RefId& script = actor.getClass().getScript(actor);
         if (script.empty())
             return false;
         return actor.getRefData().getLocals().hasVar(script, "minimumprofit");
