@@ -7,11 +7,9 @@ namespace MWGui
 {
     namespace Controllers
     {
-        void ControllerFollowMouse::prepareItem(MyGUI::Widget *_widget)
-        {
-        }
+        void ControllerFollowMouse::prepareItem(MyGUI::Widget* _widget) {}
 
-        bool ControllerFollowMouse::addTime(MyGUI::Widget *_widget, float _time)
+        bool ControllerFollowMouse::addTime(MyGUI::Widget* _widget, float _time)
         {
             _widget->setPosition(MyGUI::InputManager::getInstance().getMousePosition());
             return true;
