@@ -1,23 +1,20 @@
 #include "cursor.hpp"
 
-#include <MyGUI_PointerManager.h>
-#include <MyGUI_InputManager.h>
-#include <MyGUI_RotatingSkin.h>
 #include <MyGUI_Gui.h>
+#include <MyGUI_InputManager.h>
+#include <MyGUI_PointerManager.h>
+#include <MyGUI_RotatingSkin.h>
 
 namespace MWGui
 {
 
-
     ResourceImageSetPointerFix::ResourceImageSetPointerFix()
         : mImageSet(nullptr)
         , mRotation(0)
     {
     }
 
-    ResourceImageSetPointerFix::~ResourceImageSetPointerFix()
-    {
-    }
+    ResourceImageSetPointerFix::~ResourceImageSetPointerFix() {}
 
     void ResourceImageSetPointerFix::deserialization(MyGUI::xml::ElementPtr _node, MyGUI::Version _version)
     {
@@ -26,8 +23,8 @@ namespace MWGui
         MyGUI::xml::ElementEnumerator info = _node->getElementEnumerator();
         while (info.next("Property"))
         {
-            const std::string& key = info->findAttribute("key");
-            const std::string& value = info->findAttribute("value");
+            auto key = info->findAttribute("key");
+            auto value = info->findAttribute("value");
 
             if (key == "Point")
                 mPoint = MyGUI::IntPoint::parse(value);
@@ -56,7 +53,7 @@ namespace MWGui
         _image->setCoord(_point.left - mPoint.left, _point.top - mPoint.top, mSize.width, mSize.height);
     }
 
-    MyGUI::ResourceImageSetPtr ResourceImageSetPointerFix:: getImageSet()
+    MyGUI::ResourceImageSetPtr ResourceImageSetPointerFix::getImageSet()
     {
         return mImageSet;
     }
