@@ -9,12 +9,12 @@ namespace MWGui
 
     /// \brief Allows us to get the members of
     ///        ResourceImageSetPointer that we need.
-    /// \example MyGUI::FactoryManager::getInstance().registerFactory<ResourceImageSetPointerFix>("Resource", "ResourceImageSetPointer");
+    /// \example MyGUI::FactoryManager::getInstance().registerFactory<ResourceImageSetPointerFix>("Resource",
+    /// "ResourceImageSetPointer");
     ///          MyGUI::ResourceManager::getInstance().load("core.xml");
-    class ResourceImageSetPointerFix final :
-        public MyGUI::IPointer
+    class ResourceImageSetPointerFix final : public MyGUI::IPointer
     {
-        MYGUI_RTTI_DERIVED( ResourceImageSetPointerFix )
+        MYGUI_RTTI_DERIVED(ResourceImageSetPointerFix)
 
     public:
         ResourceImageSetPointerFix();
@@ -25,8 +25,8 @@ namespace MWGui
         void setImage(MyGUI::ImageBox* _image) override;
         void setPosition(MyGUI::ImageBox* _image, const MyGUI::IntPoint& _point) override;
 
-        //and now for the whole point of this class, allow us to get
-        //the hot spot, the image and the size of the cursor.
+        // and now for the whole point of this class, allow us to get
+        // the hot spot, the image and the size of the cursor.
         MyGUI::ResourceImageSetPtr getImageSet();
         MyGUI::IntPoint getHotSpot();
         MyGUI::IntSize getSize();
