@@ -1,66 +1,54 @@
 #include "dialogue.hpp"
 
+#include <MyGUI_Button.h>
 #include <MyGUI_LanguageManager.h>
-#include <MyGUI_Window.h>
 #include <MyGUI_ProgressBar.h>
 #include <MyGUI_ScrollBar.h>
-#include <MyGUI_Button.h>
+#include <MyGUI_UString.h>
+#include <MyGUI_Window.h>
 
 #include <components/debug/debuglog.hpp>
+#include <components/esm3/loadcrea.hpp>
+#include <components/settings/values.hpp>
+#include <components/translation/translation.hpp>
 #include <components/widgets/box.hpp>
 #include <components/widgets/list.hpp>
-#include <components/translation/translation.hpp>
 
-#include "../mwbase/environment.hpp"
-#include "../mwbase/windowmanager.hpp"
-#include "../mwbase/mechanicsmanager.hpp"
-#include "../mwbase/world.hpp"
 #include "../mwbase/dialoguemanager.hpp"
+#include "../mwbase/environment.hpp"
+#include "../mwbase/mechanicsmanager.hpp"
+#include "../mwbase/windowmanager.hpp"
+#include "../mwbase/world.hpp"
 
 #include "../mwworld/class.hpp"
 #include "../mwworld/containerstore.hpp"
 #include "../mwworld/esmstore.hpp"
+#include "../mwworld/player.hpp"
 
-#include "../mwmechanics/creaturestats.hpp"
 #include "../mwmechanics/actorutil.hpp"
+#include "../mwmechanics/creaturestats.hpp"
+#include "../mwmechanics/npcstats.hpp"
 
 #include "bookpage.hpp"
 #include "textcolours.hpp"
-#include "tooltips.hpp"
 
 #include "journalbooks.hpp" // to_utf8_span
 
 namespace MWGui
 {
-
-    class ResponseCallback : public MWBase::DialogueManager::ResponseCallback
+    void ResponseCallback::addResponse(std::string_view title, std::string_view text)
     {
-    public:
-        ResponseCallback(DialogueWindow* win, bool needMargin=true)
-            : mWindow(win)
-            , mNeedMargin(needMargin)
-        {
+        mWindow->addResponse(title, text, mNeedMargin);
+    }
 
-        }
+    void ResponseCallback::updateTopics() const
+    {
+        mWindow->updateTopics();
+    }
 
-        void addResponse(const std::string& title, const std::string& text) override
-        {
-            mWindow->addResponse(title, text, mNeedMargin);
-        }
-
-        void updateTopics()
-        {
-            mWindow->updateTopics();
-        }
-
-    private:
-        DialogueWindow* mWindow;
-        bool mNeedMargin;
-    };
-
-    PersuasionDialog::PersuasionDialog(ResponseCallback* callback)
+    PersuasionDialog::PersuasionDialog(std::unique_ptr<ResponseCallback> callback)
         : WindowModal("openmw_persuasion_dialog.layout")
-        , mCallback(callback)
+        , mCallback(std::move(callback))
         , mInitialGoldLabelWidth(0)
         , mInitialMainWidgetWidth(0)
     {
@@ -104,23 +92,26 @@ namespace MWGui
 
     void PersuasionDialog::adjustAction(MyGUI::Widget* action, int& totalHeight)
     {
-        const int lineHeight = MWBase::Environment::get().getWindowManager()->getFontHeight() + 2;
+        const int lineHeight = Settings::gui().mFontSize + 2;
         auto currentCoords = action->getCoord();
         action->setCoord(currentCoords.left, totalHeight, currentCoords.width, lineHeight);
         totalHeight += lineHeight;
     }
 
-    void PersuasionDialog::onCancel(MyGUI::Widget *sender)
+    void PersuasionDialog::onCancel(MyGUI::Widget* sender)
     {
         setVisible(false);
     }
 
-    void PersuasionDialog::onPersuade(MyGUI::Widget *sender)
+    void PersuasionDialog::onPersuade(MyGUI::Widget* sender)
     {
         MWBase::MechanicsManager::PersuasionType type;
-        if (sender == mAdmireButton) type = MWBase::MechanicsManager::PT_Admire;
-        else if (sender == mIntimidateButton) type = MWBase::MechanicsManager::PT_Intimidate;
-        else if (sender == mTauntButton) type = MWBase::MechanicsManager::PT_Taunt;
+        if (sender == mAdmireButton)
+            type = MWBase::MechanicsManager::PT_Admire;
+        else if (sender == mIntimidateButton)
+            type = MWBase::MechanicsManager::PT_Intimidate;
+        else if (sender == mTauntButton)
+            type = MWBase::MechanicsManager::PT_Taunt;
         else if (sender == mBribe10Button)
             type = MWBase::MechanicsManager::PT_Bribe10;
         else if (sender == mBribe100Button)
@@ -141,9 +132,9 @@ namespace MWGui
         MWWorld::Ptr player = MWMechanics::getPlayer();
         int playerGold = player.getClass().getContainerStore(player).count(MWWorld::ContainerStore::sGoldId);
 
-        mBribe10Button->setEnabled (playerGold >= 10);
-        mBribe100Button->setEnabled (playerGold >= 100);
-        mBribe1000Button->setEnabled (playerGold >= 1000);
+        mBribe10Button->setEnabled(playerGold >= 10);
+        mBribe100Button->setEnabled(playerGold >= 100);
+        mBribe1000Button->setEnabled(playerGold >= 1000);
 
         mGoldLabel->setCaptionWithReplacing("#{sGold}: " + MyGUI::utility::toString(playerGold));
 
@@ -163,21 +154,24 @@ namespace MWGui
 
     // --------------------------------------------------------------------------------------------------
 
-    Response::Response(const std::string &text, const std::string &title, bool needMargin)
-        : mTitle(title), mNeedMargin(needMargin)
+    Response::Response(std::string_view text, std::string_view title, bool needMargin)
+        : mTitle(title)
+        , mNeedMargin(needMargin)
     {
         mText = text;
     }
 
-    void Response::write(BookTypesetter::Ptr typesetter, KeywordSearchT* keywordSearch, std::map<std::string, Link*>& topicLinks) const
+    void Response::write(BookTypesetter::Ptr typesetter, KeywordSearchT* keywordSearch,
+        std::map<std::string, std::unique_ptr<Link>>& topicLinks) const
     {
         typesetter->sectionBreak(mNeedMargin ? 9 : 0);
+        auto windowManager = MWBase::Environment::get().getWindowManager();
 
-        if (mTitle != "")
+        if (!mTitle.empty())
         {
-            const MyGUI::Colour& headerColour = MWBase::Environment::get().getWindowManager()->getTextColours().header;
-            BookTypesetter::Style* title = typesetter->createStyle("", headerColour, false);
-            typesetter->write(title, to_utf8_span(mTitle.c_str()));
+            const MyGUI::Colour& headerColour = windowManager->getTextColours().header;
+            BookTypesetter::Style* title = typesetter->createStyle({}, headerColour, false);
+            typesetter->write(title, to_utf8_span(mTitle));
             typesetter->sectionBreak();
         }
 
@@ -188,7 +182,7 @@ namespace MWGui
         std::string text = mText;
 
         size_t pos_end = std::string::npos;
-        for(;;)
+        for (;;)
         {
             size_t pos_begin = text.find('@');
             if (pos_begin != std::string::npos)
@@ -199,36 +193,37 @@ namespace MWGui
                 std::string link = text.substr(pos_begin + 1, pos_end - pos_begin - 1);
                 const char specialPseudoAsteriskCharacter = 127;
                 std::replace(link.begin(), link.end(), specialPseudoAsteriskCharacter, '*');
-                std::string topicName = MWBase::Environment::get().getWindowManager()->
-                        getTranslationDataStorage().topicStandardForm(link);
+                std::string topicName
+                    = Misc::StringUtils::lowerCase(windowManager->getTranslationDataStorage().topicStandardForm(link));
 
-                std::string displayName = link;
-                while (displayName[displayName.size()-1] == '*')
-                    displayName.erase(displayName.size()-1, 1);
+                std::string displayName = std::move(link);
+                while (displayName[displayName.size() - 1] == '*')
+                    displayName.erase(displayName.size() - 1, 1);
 
-                text.replace(pos_begin, pos_end+1-pos_begin, displayName);
+                text.replace(pos_begin, pos_end + 1 - pos_begin, displayName);
 
-                if (topicLinks.find(Misc::StringUtils::lowerCase(topicName)) != topicLinks.end())
-                    hyperLinks[std::make_pair(pos_begin, pos_begin+displayName.size())] = intptr_t(topicLinks[Misc::StringUtils::lowerCase(topicName)]);
+                if (topicLinks.find(topicName) != topicLinks.end())
+                    hyperLinks[std::make_pair(pos_begin, pos_begin + displayName.size())]
+                        = intptr_t(topicLinks[topicName].get());
             }
             else
                 break;
         }
 
-        typesetter->addContent(to_utf8_span(text.c_str()));
+        typesetter->addContent(to_utf8_span(text));
 
-        if (hyperLinks.size() && MWBase::Environment::get().getWindowManager()->getTranslationDataStorage().hasTranslation())
+        if (hyperLinks.size()
+            && MWBase::Environment::get().getWindowManager()->getTranslationDataStorage().hasTranslation())
         {
             const TextColours& textColours = MWBase::Environment::get().getWindowManager()->getTextColours();
 
-            BookTypesetter::Style* style = typesetter->createStyle("", textColours.normal, false);
+            BookTypesetter::Style* style = typesetter->createStyle({}, textColours.normal, false);
             size_t formatted = 0; // points to the first character that is not laid out yet
             for (auto& hyperLink : hyperLinks)
             {
                 intptr_t topicId = hyperLink.second;
-                BookTypesetter::Style* hotStyle = typesetter->createHotStyle (style, textColours.link,
-                                                                              textColours.linkOver, textColours.linkPressed,
-                                                                              topicId);
+                BookTypesetter::Style* hotStyle = typesetter->createHotStyle(
+                    style, textColours.link, textColours.linkOver, textColours.linkPressed, topicId);
                 if (formatted < hyperLink.first.first)
                     typesetter->write(style, formatted, hyperLink.first.first);
                 typesetter->write(hotStyle, hyperLink.first.first, hyperLink.first.second);
@@ -242,18 +237,18 @@ namespace MWGui
             std::vector<KeywordSearchT::Match> matches;
             keywordSearch->highlightKeywords(text.begin(), text.end(), matches);
 
-            std::string::const_iterator i = text.begin ();
+            std::string::const_iterator i = text.begin();
             for (KeywordSearchT::Match& match : matches)
             {
                 if (i != match.mBeg)
-                    addTopicLink (typesetter, 0, i - text.begin (), match.mBeg - text.begin ());
+                    addTopicLink(typesetter, 0, i - text.begin(), match.mBeg - text.begin());
 
-                addTopicLink (typesetter, match.mValue, match.mBeg - text.begin (), match.mEnd - text.begin ());
+                addTopicLink(typesetter, match.mValue, match.mBeg - text.begin(), match.mEnd - text.begin());
 
                 i = match.mEnd;
             }
-            if (i != text.end ())
-                addTopicLink (typesetter, 0, i - text.begin (), text.size ());
+            if (i != text.end())
+                addTopicLink(std::move(typesetter), 0, i - text.begin(), text.size());
         }
     }
 
@@ -261,44 +256,45 @@ namespace MWGui
     {
         const TextColours& textColours = MWBase::Environment::get().getWindowManager()->getTextColours();
 
-        BookTypesetter::Style* style = typesetter->createStyle("", textColours.normal, false);
-
+        BookTypesetter::Style* style = typesetter->createStyle({}, textColours.normal, false);
 
         if (topicId)
-            style = typesetter->createHotStyle (style, textColours.link, textColours.linkOver, textColours.linkPressed, topicId);
-        typesetter->write (style, begin, end);
+            style = typesetter->createHotStyle(
+                style, textColours.link, textColours.linkOver, textColours.linkPressed, topicId);
+        typesetter->write(style, begin, end);
     }
 
-    Message::Message(const std::string& text)
+    Message::Message(std::string_view text)
     {
         mText = text;
     }
 
-    void Message::write(BookTypesetter::Ptr typesetter, KeywordSearchT* keywordSearch, std::map<std::string, Link*>& topicLinks) const
+    void Message::write(BookTypesetter::Ptr typesetter, KeywordSearchT* keywordSearch,
+        std::map<std::string, std::unique_ptr<Link>>& topicLinks) const
     {
         const MyGUI::Colour& textColour = MWBase::Environment::get().getWindowManager()->getTextColours().notify;
-        BookTypesetter::Style* title = typesetter->createStyle("", textColour, false);
+        BookTypesetter::Style* title = typesetter->createStyle({}, textColour, false);
         typesetter->sectionBreak(9);
-        typesetter->write(title, to_utf8_span(mText.c_str()));
+        typesetter->write(title, to_utf8_span(mText));
     }
 
     // --------------------------------------------------------------------------------------------------
 
     void Choice::activated()
     {
-        MWBase::Environment::get().getWindowManager()->playSound("Menu Click");
+        MWBase::Environment::get().getWindowManager()->playSound(ESM::RefId::stringRefId("Menu Click"));
         eventChoiceActivated(mChoiceId);
     }
 
     void Topic::activated()
     {
-        MWBase::Environment::get().getWindowManager()->playSound("Menu Click");
+        MWBase::Environment::get().getWindowManager()->playSound(ESM::RefId::stringRefId("Menu Click"));
         eventTopicActivated(mTopicId);
     }
 
     void Goodbye::activated()
     {
-        MWBase::Environment::get().getWindowManager()->playSound("Menu Click");
+        MWBase::Environment::get().getWindowManager()->playSound(ESM::RefId::stringRefId("Menu Click"));
         eventActivated();
     }
 
@@ -308,19 +304,19 @@ namespace MWGui
         : WindowBase("openmw_dialogue_window.layout")
         , mIsCompanion(false)
         , mGoodbye(false)
-        , mPersuasionDialog(new ResponseCallback(this))
-        , mCallback(new ResponseCallback(this))
-        , mGreetingCallback(new ResponseCallback(this, false))
+        , mPersuasionDialog(std::make_unique<ResponseCallback>(this))
+        , mCallback(std::make_unique<ResponseCallback>(this))
+        , mGreetingCallback(std::make_unique<ResponseCallback>(this, false))
     {
         // Centre dialog
         center();
 
         mPersuasionDialog.setVisible(false);
 
-        //History view
+        // History view
         getWidget(mHistory, "History");
 
-        //Topics list
+        // Topics list
         getWidget(mTopicsList, "TopicsList");
         mTopicsList->eventItemSelected += MyGUI::newDelegate(this, &DialogueWindow::onSelectListItem);
 
@@ -328,32 +324,23 @@ namespace MWGui
         mGoodbyeButton->eventMouseButtonClick += MyGUI::newDelegate(this, &DialogueWindow::onByeClicked);
 
         getWidget(mDispositionBar, "Disposition");
-        getWidget(mDispositionText,"DispositionText");
+        getWidget(mDispositionText, "DispositionText");
         getWidget(mScrollBar, "VScroll");
 
         mScrollBar->eventScrollChangePosition += MyGUI::newDelegate(this, &DialogueWindow::onScrollbarMoved);
         mHistory->eventMouseWheel += MyGUI::newDelegate(this, &DialogueWindow::onMouseWheel);
 
-        BookPage::ClickCallback callback = std::bind (&DialogueWindow::notifyLinkClicked, this, std::placeholders::_1);
-        mHistory->adviseLinkClicked(callback);
+        BookPage::ClickCallback callback = [this](TypesetBook::InteractiveId link) { notifyLinkClicked(link); };
+        mHistory->adviseLinkClicked(std::move(callback));
 
-        mMainWidget->castType<MyGUI::Window>()->eventWindowChangeCoord += MyGUI::newDelegate(this, &DialogueWindow::onWindowResize);
-    }
-
-    DialogueWindow::~DialogueWindow()
-    {
-        deleteLater();
-        for (Link* link : mLinks)
-            delete link;
-        for (const auto& link : mTopicLinks)
-            delete link.second;
-        for (auto history : mHistoryContents)
-            delete history;
+        mMainWidget->castType<MyGUI::Window>()->eventWindowChangeCoord
+            += MyGUI::newDelegate(this, &DialogueWindow::onWindowResize);
     }
 
     void DialogueWindow::onTradeComplete()
     {
-        addResponse("", MyGUI::LanguageManager::getInstance().replaceTags("#{sBarterDialog5}"));
+        MyGUI::UString message = MyGUI::LanguageManager::getInstance().replaceTags("#{sBarterDialog5}");
+        addResponse({}, message);
     }
 
     bool DialogueWindow::exit()
@@ -374,11 +361,11 @@ namespace MWGui
     void DialogueWindow::onWindowResize(MyGUI::Window* _sender)
     {
         // if the window has only been moved, not resized, we don't need to update
-        if (mCurrentWindowSize == _sender->getSize()) return;
+        if (mCurrentWindowSize == _sender->getSize())
+            return;
 
-        mTopicsList->adjustSize();
+        redrawTopicsList();
         updateHistory();
-        updateTopicFormat();
         mCurrentWindowSize = _sender->getSize();
     }
 
@@ -386,7 +373,8 @@ namespace MWGui
     {
         if (!mScrollBar->getVisible())
             return;
-        mScrollBar->setScrollPosition(std::clamp<int>(mScrollBar->getScrollPosition() - _rel*0.3, 0, mScrollBar->getScrollRange() - 1));
+        mScrollBar->setScrollPosition(
+            std::clamp<int>(mScrollBar->getScrollPosition() - _rel * 0.3, 0, mScrollBar->getScrollRange() - 1));
         onScrollbarMoved(mScrollBar, mScrollBar->getScrollPosition());
     }
 
@@ -405,7 +393,8 @@ namespace MWGui
         if (mGoodbye || dialogueManager->isInChoice())
             return;
 
-        const MWWorld::Store<ESM::GameSetting> &gmst = MWBase::Environment::get().getWorld()->getStore().get<ESM::GameSetting>();
+        const MWWorld::Store<ESM::GameSetting>& gmst
+            = MWBase::Environment::get().getESMStore()->get<ESM::GameSetting>();
 
         const std::string& sPersuasion = gmst.find("sPersuasion")->mValue.getString();
         const std::string& sCompanionShare = gmst.find("sCompanionShare")->mValue.getString();
@@ -417,9 +406,9 @@ namespace MWGui
         const std::string& sServiceTrainingTitle = gmst.find("sServiceTrainingTitle")->mValue.getString();
         const std::string& sRepair = gmst.find("sRepair")->mValue.getString();
 
-        if (topic != sPersuasion && topic != sCompanionShare && topic != sBarter 
-         && topic != sSpells && topic != sTravel && topic != sSpellMakingMenuTitle 
-         && topic != sEnchanting && topic != sServiceTrainingTitle && topic != sRepair)
+        if (topic != sPersuasion && topic != sCompanionShare && topic != sBarter && topic != sSpells && topic != sTravel
+            && topic != sSpellMakingMenuTitle && topic != sEnchanting && topic != sServiceTrainingTitle
+            && topic != sRepair)
         {
             onTopicActivated(topic);
             if (mGoodbyeButton->getEnabled())
@@ -431,19 +420,26 @@ namespace MWGui
             MWBase::Environment::get().getWindowManager()->pushGuiMode(GM_Companion, mPtr);
         else if (!dialogueManager->checkServiceRefused(mCallback.get()))
         {
-            if (topic == sBarter && !dialogueManager->checkServiceRefused(mCallback.get(), MWBase::DialogueManager::Barter))
+            if (topic == sBarter
+                && !dialogueManager->checkServiceRefused(mCallback.get(), MWBase::DialogueManager::Barter))
                 MWBase::Environment::get().getWindowManager()->pushGuiMode(GM_Barter, mPtr);
-            else if (topic == sSpells && !dialogueManager->checkServiceRefused(mCallback.get(), MWBase::DialogueManager::Spells))
+            else if (topic == sSpells
+                && !dialogueManager->checkServiceRefused(mCallback.get(), MWBase::DialogueManager::Spells))
                 MWBase::Environment::get().getWindowManager()->pushGuiMode(GM_SpellBuying, mPtr);
-            else if (topic == sTravel && !dialogueManager->checkServiceRefused(mCallback.get(), MWBase::DialogueManager::Travel))
+            else if (topic == sTravel
+                && !dialogueManager->checkServiceRefused(mCallback.get(), MWBase::DialogueManager::Travel))
                 MWBase::Environment::get().getWindowManager()->pushGuiMode(GM_Travel, mPtr);
-            else if (topic == sSpellMakingMenuTitle && !dialogueManager->checkServiceRefused(mCallback.get(), MWBase::DialogueManager::Spellmaking))
+            else if (topic == sSpellMakingMenuTitle
+                && !dialogueManager->checkServiceRefused(mCallback.get(), MWBase::DialogueManager::Spellmaking))
                 MWBase::Environment::get().getWindowManager()->pushGuiMode(GM_SpellCreation, mPtr);
-            else if (topic == sEnchanting && !dialogueManager->checkServiceRefused(mCallback.get(), MWBase::DialogueManager::Enchanting))
+            else if (topic == sEnchanting
+                && !dialogueManager->checkServiceRefused(mCallback.get(), MWBase::DialogueManager::Enchanting))
                 MWBase::Environment::get().getWindowManager()->pushGuiMode(GM_Enchanting, mPtr);
-            else if (topic == sServiceTrainingTitle && !dialogueManager->checkServiceRefused(mCallback.get(), MWBase::DialogueManager::Training))
+            else if (topic == sServiceTrainingTitle
+                && !dialogueManager->checkServiceRefused(mCallback.get(), MWBase::DialogueManager::Training))
                 MWBase::Environment::get().getWindowManager()->pushGuiMode(GM_Training, mPtr);
-            else if (topic == sRepair && !dialogueManager->checkServiceRefused(mCallback.get(), MWBase::DialogueManager::Repair))
+            else if (topic == sRepair
+                && !dialogueManager->checkServiceRefused(mCallback.get(), MWBase::DialogueManager::Repair))
                 MWBase::Environment::get().getWindowManager()->pushGuiMode(GM_MerchantRepair, mPtr);
         }
         else
@@ -452,7 +448,7 @@ namespace MWGui
 
     void DialogueWindow::setPtr(const MWWorld::Ptr& actor)
     {
-        if (!actor.getClass().isActor())
+        if (actor.isEmpty() || !actor.getClass().isActor())
         {
             Log(Debug::Warning) << "Warning: can not talk with non-actor object.";
             return;
@@ -464,8 +460,9 @@ namespace MWGui
             // The history is not reset here
             mKeywords.clear();
             mTopicsList->clear();
-            for (Link* link : mLinks)
-                mDeleteLater.push_back(link); // Links are not deleted right away to prevent issues with event handlers
+            for (auto& link : mLinks)
+                mDeleteLater.push_back(
+                    std::move(link)); // Links are not deleted right away to prevent issues with event handlers
             mLinks.clear();
         }
 
@@ -497,8 +494,12 @@ namespace MWGui
 
     void DialogueWindow::restock()
     {
-        MWMechanics::CreatureStats &sellerStats = mPtr.getClass().getCreatureStats(mPtr);
-        float delay = MWBase::Environment::get().getWorld()->getStore().get<ESM::GameSetting>().find("fBarterGoldResetDelay")->mValue.getFloat();
+        MWMechanics::CreatureStats& sellerStats = mPtr.getClass().getCreatureStats(mPtr);
+        float delay = MWBase::Environment::get()
+                          .getESMStore()
+                          ->get<ESM::GameSetting>()
+                          .find("fBarterGoldResetDelay")
+                          ->mValue.getFloat();
 
         // Gold is restocked every 24h
         if (MWBase::Environment::get().getWorld()->getTimeStamp() >= sellerStats.getLastRestockTime() + delay)
@@ -511,8 +512,6 @@ namespace MWGui
 
     void DialogueWindow::deleteLater()
     {
-        for (Link* link : mDeleteLater)
-            delete link;
         mDeleteLater.clear();
     }
 
@@ -521,8 +520,6 @@ namespace MWGui
         if (MWBase::Environment::get().getWindowManager()->containsMode(GM_Dialogue))
             return;
         // Reset history
-        for (DialogueText* text : mHistoryContents)
-            delete text;
         mHistoryContents.clear();
     }
 
@@ -536,21 +533,30 @@ namespace MWGui
         return true;
     }
 
+    void DialogueWindow::redrawTopicsList()
+    {
+        mTopicsList->adjustSize();
+
+        // The topics list has been regenerated so topic formatting needs to be updated
+        updateTopicFormat();
+    }
+
     void DialogueWindow::updateTopicsPane()
     {
         mTopicsList->clear();
         for (auto& linkPair : mTopicLinks)
-            mDeleteLater.push_back(linkPair.second);
+            mDeleteLater.push_back(std::move(linkPair.second));
         mTopicLinks.clear();
         mKeywordSearch.clear();
 
         int services = mPtr.getClass().getServices(mPtr);
 
         bool travel = (mPtr.getType() == ESM::NPC::sRecordId && !mPtr.get<ESM::NPC>()->mBase->getTransport().empty())
-                || (mPtr.getType() == ESM::Creature::sRecordId && !mPtr.get<ESM::Creature>()->mBase->getTransport().empty());
+            || (mPtr.getType() == ESM::Creature::sRecordId
+                && !mPtr.get<ESM::Creature>()->mBase->getTransport().empty());
 
-        const MWWorld::Store<ESM::GameSetting> &gmst =
-            MWBase::Environment::get().getWorld()->getStore().get<ESM::GameSetting>();
+        const MWWorld::Store<ESM::GameSetting>& gmst
+            = MWBase::Environment::get().getESMStore()->get<ESM::GameSetting>();
 
         if (mPtr.getType() == ESM::NPC::sRecordId)
             mTopicsList->addItem(gmst.find("sPersuasion")->mValue.getString());
@@ -582,44 +588,40 @@ namespace MWGui
         if (mTopicsList->getItemCount() > 0)
             mTopicsList->addSeparator();
 
-
-        for(const auto& keyword : mKeywords)
+        for (const auto& keyword : mKeywords)
         {
             std::string topicId = Misc::StringUtils::lowerCase(keyword);
             mTopicsList->addItem(keyword);
 
-            Topic* t = new Topic(keyword);
+            auto t = std::make_unique<Topic>(keyword);
+            mKeywordSearch.seed(topicId, intptr_t(t.get()));
             t->eventTopicActivated += MyGUI::newDelegate(this, &DialogueWindow::onTopicActivated);
-            mTopicLinks[topicId] = t;
-
-            mKeywordSearch.seed(topicId, intptr_t(t));
+            mTopicLinks[topicId] = std::move(t);
         }
-        mTopicsList->adjustSize();
 
+        redrawTopicsList();
         updateHistory();
-        // The topics list has been regenerated so topic formatting needs to be updated
-        updateTopicFormat();
     }
 
     void DialogueWindow::updateHistory(bool scrollbar)
     {
         if (!scrollbar && mScrollBar->getVisible())
         {
-            mHistory->setSize(mHistory->getSize()+MyGUI::IntSize(mScrollBar->getWidth(),0));
+            mHistory->setSize(mHistory->getSize() + MyGUI::IntSize(mScrollBar->getWidth(), 0));
             mScrollBar->setVisible(false);
         }
         if (scrollbar && !mScrollBar->getVisible())
         {
-            mHistory->setSize(mHistory->getSize()-MyGUI::IntSize(mScrollBar->getWidth(),0));
+            mHistory->setSize(mHistory->getSize() - MyGUI::IntSize(mScrollBar->getWidth(), 0));
             mScrollBar->setVisible(true);
         }
 
-        BookTypesetter::Ptr typesetter = BookTypesetter::create (mHistory->getWidth(), std::numeric_limits<int>::max());
+        BookTypesetter::Ptr typesetter = BookTypesetter::create(mHistory->getWidth(), std::numeric_limits<int>::max());
 
-        for (DialogueText* text : mHistoryContents)
+        for (const auto& text : mHistoryContents)
             text->write(typesetter, &mKeywordSearch, mTopicLinks);
 
-        BookTypesetter::Style* body = typesetter->createStyle("", MyGUI::Colour::White, false);
+        BookTypesetter::Style* body = typesetter->createStyle({}, MyGUI::Colour::White, false);
 
         typesetter->sectionBreak(9);
         // choices
@@ -627,29 +629,33 @@ namespace MWGui
         mChoices = MWBase::Environment::get().getDialogueManager()->getChoices();
         for (std::pair<std::string, int>& choice : mChoices)
         {
-            Choice* link = new Choice(choice.second);
+            auto link = std::make_unique<Choice>(choice.second);
             link->eventChoiceActivated += MyGUI::newDelegate(this, &DialogueWindow::onChoiceActivated);
-            mLinks.push_back(link);
+            auto interactiveId = TypesetBook::InteractiveId(link.get());
+            mLinks.push_back(std::move(link));
 
             typesetter->lineBreak();
-            BookTypesetter::Style* questionStyle = typesetter->createHotStyle(body, textColours.answer, textColours.answerOver,
-                                                                              textColours.answerPressed,
-                                                                              TypesetBook::InteractiveId(link));
-            typesetter->write(questionStyle, to_utf8_span(choice.first.c_str()));
+            BookTypesetter::Style* questionStyle = typesetter->createHotStyle(
+                body, textColours.answer, textColours.answerOver, textColours.answerPressed, interactiveId);
+            typesetter->write(questionStyle, to_utf8_span(choice.first));
         }
 
         mGoodbye = MWBase::Environment::get().getDialogueManager()->isGoodbye();
         if (mGoodbye)
         {
-            Goodbye* link = new Goodbye();
+            auto link = std::make_unique<Goodbye>();
             link->eventActivated += MyGUI::newDelegate(this, &DialogueWindow::onGoodbyeActivated);
-            mLinks.push_back(link);
-            const std::string& goodbye = MWBase::Environment::get().getWorld()->getStore().get<ESM::GameSetting>().find("sGoodbye")->mValue.getString();
-            BookTypesetter::Style* questionStyle = typesetter->createHotStyle(body, textColours.answer, textColours.answerOver,
-                textColours.answerPressed,
-                TypesetBook::InteractiveId(link));
+            auto interactiveId = TypesetBook::InteractiveId(link.get());
+            mLinks.push_back(std::move(link));
+            const std::string& goodbye = MWBase::Environment::get()
+                                             .getESMStore()
+                                             ->get<ESM::GameSetting>()
+                                             .find("sGoodbye")
+                                             ->mValue.getString();
+            BookTypesetter::Style* questionStyle = typesetter->createHotStyle(
+                body, textColours.answer, textColours.answerOver, textColours.answerPressed, interactiveId);
             typesetter->lineBreak();
-            typesetter->write(questionStyle, to_utf8_span(goodbye.c_str()));
+            typesetter->write(questionStyle, to_utf8_span(goodbye));
         }
 
         TypesetBook::Ptr book = typesetter->complete();
@@ -660,11 +666,13 @@ namespace MWGui
         else if (scrollbar)
         {
             mHistory->setSize(MyGUI::IntSize(mHistory->getWidth(), book->getSize().second));
-            size_t range = book->getSize().second - viewHeight;
+            // Scroll range should be >= 2 to enable scrolling and prevent a crash
+            size_t range = std::max(book->getSize().second - viewHeight, size_t(2));
             mScrollBar->setScrollRange(range);
-            mScrollBar->setScrollPosition(range-1);
-            mScrollBar->setTrackSize(static_cast<int>(viewHeight / static_cast<float>(book->getSize().second) * mScrollBar->getLineSize()));
-            onScrollbarMoved(mScrollBar, range-1);
+            mScrollBar->setScrollPosition(range - 1);
+            mScrollBar->setTrackSize(
+                static_cast<int>(viewHeight / static_cast<float>(book->getSize().second) * mScrollBar->getLineSize()));
+            onScrollbarMoved(mScrollBar, range - 1);
         }
         else
         {
@@ -682,12 +690,12 @@ namespace MWGui
         mTopicsList->setEnabled(topicsEnabled);
     }
 
-    void DialogueWindow::notifyLinkClicked (TypesetBook::InteractiveId link)
+    void DialogueWindow::notifyLinkClicked(TypesetBook::InteractiveId link)
     {
         reinterpret_cast<Link*>(link)->activated();
     }
 
-    void DialogueWindow::onTopicActivated(const std::string &topicId)
+    void DialogueWindow::onTopicActivated(const std::string& topicId)
     {
         if (mGoodbye)
             return;
@@ -714,20 +722,20 @@ namespace MWGui
         resetReference();
     }
 
-    void DialogueWindow::onScrollbarMoved(MyGUI::ScrollBar *sender, size_t pos)
+    void DialogueWindow::onScrollbarMoved(MyGUI::ScrollBar* sender, size_t pos)
     {
         mHistory->setPosition(0, static_cast<int>(pos) * -1);
     }
 
-    void DialogueWindow::addResponse(const std::string &title, const std::string &text, bool needMargin)
+    void DialogueWindow::addResponse(std::string_view title, std::string_view text, bool needMargin)
     {
-        mHistoryContents.push_back(new Response(text, title, needMargin));
+        mHistoryContents.push_back(std::make_unique<Response>(text, title, needMargin));
         updateHistory();
     }
 
-    void DialogueWindow::addMessageBox(const std::string& text)
+    void DialogueWindow::addMessageBox(std::string_view text)
     {
-        mHistoryContents.push_back(new Message(text));
+        mHistoryContents.push_back(std::make_unique<Message>(text));
         updateHistory();
     }
 
@@ -736,27 +744,30 @@ namespace MWGui
         bool dispositionVisible = false;
         if (!mPtr.isEmpty() && mPtr.getClass().isNpc())
         {
+            // If actor was a witness to a crime which was payed off,
+            // restore original disposition immediately.
+            MWMechanics::NpcStats& npcStats = mPtr.getClass().getNpcStats(mPtr);
+            if (npcStats.getCrimeId() != -1 && npcStats.getCrimeDispositionModifier() != 0)
+            {
+                if (npcStats.getCrimeId() <= MWBase::Environment::get().getWorld()->getPlayer().getCrimeId())
+                    npcStats.setCrimeDispositionModifier(0);
+            }
+
             dispositionVisible = true;
             mDispositionBar->setProgressRange(100);
-            mDispositionBar->setProgressPosition(MWBase::Environment::get().getMechanicsManager()->getDerivedDisposition(mPtr));
-            mDispositionText->setCaption(MyGUI::utility::toString(MWBase::Environment::get().getMechanicsManager()->getDerivedDisposition(mPtr))+std::string("/100"));
+            mDispositionBar->setProgressPosition(
+                MWBase::Environment::get().getMechanicsManager()->getDerivedDisposition(mPtr));
+            mDispositionText->setCaption(
+                MyGUI::utility::toString(MWBase::Environment::get().getMechanicsManager()->getDerivedDisposition(mPtr))
+                + std::string("/100"));
         }
 
-        bool dispositionWasVisible = mDispositionBar->getVisible();
-
-        if (dispositionVisible && !dispositionWasVisible)
+        if (mDispositionBar->getVisible() != dispositionVisible)
         {
-            mDispositionBar->setVisible(true);
-            int offset = mDispositionBar->getHeight()+5;
-            mTopicsList->setCoord(mTopicsList->getCoord() + MyGUI::IntCoord(0,offset,0,-offset));
-            mTopicsList->adjustSize();
-        }
-        else if (!dispositionVisible && dispositionWasVisible)
-        {
-            mDispositionBar->setVisible(false);
-            int offset = mDispositionBar->getHeight()+5;
-            mTopicsList->setCoord(mTopicsList->getCoord() - MyGUI::IntCoord(0,offset,0,-offset));
-            mTopicsList->adjustSize();
+            mDispositionBar->setVisible(dispositionVisible);
+            const int offset = (mDispositionBar->getHeight() + 5) * (dispositionVisible ? 1 : -1);
+            mTopicsList->setCoord(mTopicsList->getCoord() + MyGUI::IntCoord(0, offset, 0, -offset));
+            redrawTopicsList();
         }
     }
 
@@ -775,27 +786,43 @@ namespace MWGui
         deleteLater();
 
         if (mChoices != MWBase::Environment::get().getDialogueManager()->getChoices()
-                || mGoodbye != MWBase::Environment::get().getDialogueManager()->isGoodbye())
+            || mGoodbye != MWBase::Environment::get().getDialogueManager()->isGoodbye())
             updateHistory();
     }
 
     void DialogueWindow::updateTopicFormat()
     {
-        if (!Settings::Manager::getBool("color topic enable", "GUI"))
+        if (!Settings::gui().mColorTopicEnable)
             return;
 
-        std::string specialColour = Settings::Manager::getString("color topic specific", "GUI");
-        std::string oldColour = Settings::Manager::getString("color topic exhausted", "GUI");
-
         for (const std::string& keyword : mKeywords)
         {
-            int flag = MWBase::Environment::get().getDialogueManager()->getTopicFlag(keyword);
+            int flag = MWBase::Environment::get().getDialogueManager()->getTopicFlag(ESM::RefId::stringRefId(keyword));
             MyGUI::Button* button = mTopicsList->getItemWidget(keyword);
+            const auto oldCaption = button->getCaption();
+            const MyGUI::IntSize oldSize = button->getSize();
 
-            if (!specialColour.empty() && flag & MWBase::DialogueManager::TopicType::Specific)
-                button->getSubWidgetText()->setTextColour(MyGUI::Colour::parse(specialColour));
-            else if (!oldColour.empty() && flag & MWBase::DialogueManager::TopicType::Exhausted)
-                button->getSubWidgetText()->setTextColour(MyGUI::Colour::parse(oldColour));
+            bool changed = false;
+            if (flag & MWBase::DialogueManager::TopicType::Specific)
+            {
+                button->changeWidgetSkin("MW_ListLine_Specific");
+                changed = true;
+            }
+            else if (flag & MWBase::DialogueManager::TopicType::Exhausted)
+            {
+                button->changeWidgetSkin("MW_ListLine_Exhausted");
+                changed = true;
+            }
+
+            if (changed)
+            {
+                button->setCaption(oldCaption);
+                button->setTextAlign(MyGUI::Align::Left);
+                MyGUI::ISubWidgetText* text = button->getSubWidgetText();
+                if (text != nullptr)
+                    text->setWordWrap(true);
+                button->setSize(oldSize);
+            }
         }
     }
 
@@ -817,7 +844,7 @@ namespace MWGui
             return false;
 
         return !actor.getClass().getScript(actor).empty()
-                && actor.getRefData().getLocals().getIntVar(actor.getClass().getScript(actor), "companion");
+            && actor.getRefData().getLocals().getIntVar(actor.getClass().getScript(actor), "companion");
     }
 
 }
