@@ -1,31 +1,31 @@
 #ifndef MWGUI_ENCHANTINGDIALOG_H
 #define MWGUI_ENCHANTINGDIALOG_H
 
-#include "spellcreationdialog.hpp"
+#include <memory>
 
-#include "../mwbase/windowmanager.hpp"
+#include "itemselection.hpp"
+#include "spellcreationdialog.hpp"
 
 #include "../mwmechanics/enchanting.hpp"
 
 namespace MWGui
 {
 
-    class ItemSelectionDialog;
     class ItemWidget;
 
     class EnchantingDialog : public WindowBase, public ReferenceInterface, public EffectEditorBase
     {
     public:
         EnchantingDialog();
-        virtual ~EnchantingDialog();
+        virtual ~EnchantingDialog() = default;
 
         void onOpen() override;
 
         void onFrame(float dt) override { checkReferenceAvailable(); }
         void clear() override { resetReference(); }
 
-        void setSoulGem (const MWWorld::Ptr& gem);
-        void setItem (const MWWorld::Ptr& item);
+        void setSoulGem(const MWWorld::Ptr& gem);
+        void setItem(const MWWorld::Ptr& item);
 
         /// Actor Ptr: buy enchantment from this actor
         /// Soulgem Ptr: player self-enchant
@@ -33,13 +33,15 @@ namespace MWGui
 
         void resetReference() override;
 
+        std::string_view getWindowIdForLua() const override { return "EnchantingDialog"; }
+
     protected:
         void onReferenceUnavailable() override;
         void notifyEffectsChanged() override;
 
         void onCancelButtonClicked(MyGUI::Widget* sender);
-        void onSelectItem (MyGUI::Widget* sender);
-        void onSelectSoul (MyGUI::Widget* sender);
+        void onSelectItem(MyGUI::Widget* sender);
+        void onSelectSoul(MyGUI::Widget* sender);
 
         void onItemSelected(MWWorld::Ptr item);
         void onItemCancel();
@@ -50,7 +52,7 @@ namespace MWGui
         void onTypeButtonClicked(MyGUI::Widget* sender);
         void onAccept(MyGUI::EditBox* sender);
 
-        ItemSelectionDialog* mItemSelectionDialog;
+        std::unique_ptr<ItemSelectionDialog> mItemSelectionDialog;
 
         MyGUI::Widget* mChanceLayout;
 
