@@ -1,15 +1,17 @@
 #include "hud.hpp"
 
-#include <MyGUI_RenderManager.h>
-#include <MyGUI_ProgressBar.h>
 #include <MyGUI_Button.h>
-#include <MyGUI_InputManager.h>
 #include <MyGUI_ImageBox.h>
+#include <MyGUI_InputManager.h>
+#include <MyGUI_ProgressBar.h>
+#include <MyGUI_RenderManager.h>
 #include <MyGUI_ScrollView.h>
 
-#include <components/settings/settings.hpp>
+#include <components/esm3/loadgmst.hpp>
+#include <components/esm3/loadmgef.hpp>
 #include <components/misc/resourcehelpers.hpp>
 #include <components/resource/resourcesystem.hpp>
+#include <components/settings/values.hpp>
 
 #include "../mwbase/environment.hpp"
 #include "../mwbase/windowmanager.hpp"
@@ -18,13 +20,13 @@
 #include "../mwworld/class.hpp"
 #include "../mwworld/esmstore.hpp"
 
-#include "../mwmechanics/npcstats.hpp"
 #include "../mwmechanics/actorutil.hpp"
+#include "../mwmechanics/npcstats.hpp"
 
-#include "inventorywindow.hpp"
-#include "spellicons.hpp"
-#include "itemmodel.hpp"
 #include "draganddrop.hpp"
+#include "inventorywindow.hpp"
+#include "itemmodel.hpp"
+#include "spellicons.hpp"
 
 #include "itemwidget.hpp"
 
@@ -37,27 +39,45 @@ namespace MWGui
     class WorldItemModel : public ItemModel
     {
     public:
-        WorldItemModel(float left, float top) : mLeft(left), mTop(top) {}
+        WorldItemModel(float left, float top)
+            : mLeft(left)
+            , mTop(top)
+        {
+        }
         virtual ~WorldItemModel() override {}
-        MWWorld::Ptr copyItem (const ItemStack& item, size_t count, bool /*allowAutoEquip*/) override
+
+        MWWorld::Ptr dropItemImpl(const ItemStack& item, size_t count, bool copy)
         {
             MWBase::World* world = MWBase::Environment::get().getWorld();
 
             MWWorld::Ptr dropped;
             if (world->canPlaceObject(mLeft, mTop))
-                dropped = world->placeObject(item.mBase, mLeft, mTop, count);
+                dropped = world->placeObject(item.mBase, mLeft, mTop, count, copy);
             else
-                dropped = world->dropObjectOnGround(world->getPlayerPtr(), item.mBase, count);
-            dropped.getCellRef().setOwner("");
+                dropped = world->dropObjectOnGround(world->getPlayerPtr(), item.mBase, count, copy);
+            dropped.getCellRef().setOwner(ESM::RefId());
 
             return dropped;
         }
 
-        void removeItem (const ItemStack& item, size_t count) override { throw std::runtime_error("removeItem not implemented"); }
-        ModelIndex getIndex (const ItemStack &item) override { throw std::runtime_error("getIndex not implemented"); }
+        MWWorld::Ptr addItem(const ItemStack& item, size_t count, bool /*allowAutoEquip*/) override
+        {
+            return dropItemImpl(item, count, false);
+        }
+
+        MWWorld::Ptr copyItem(const ItemStack& item, size_t count, bool /*allowAutoEquip*/) override
+        {
+            return dropItemImpl(item, count, true);
+        }
+
+        void removeItem(const ItemStack& item, size_t count) override
+        {
+            throw std::runtime_error("removeItem not implemented");
+        }
+        ModelIndex getIndex(const ItemStack& item) override { throw std::runtime_error("getIndex not implemented"); }
         void update() override {}
         size_t getItemCount() override { return 0; }
-        ItemStack getItem (ModelIndex index) override { throw std::runtime_error("getItem not implemented"); }
+        ItemStack getItem(ModelIndex index) override { throw std::runtime_error("getItem not implemented"); }
         bool usesContainer(const MWWorld::Ptr&) override { return false; }
 
     private:
@@ -66,10 +86,9 @@ namespace MWGui
         float mTop;
     };
 
-
-    HUD::HUD(CustomMarkerCollection &customMarkers, DragAndDrop* dragAndDrop, MWRender::LocalMap* localMapRender)
+    HUD::HUD(CustomMarkerCollection& customMarkers, DragAndDrop* dragAndDrop, MWRender::LocalMap* localMapRender)
         : WindowBase("openmw_hud.layout")
-        , LocalMapBase(customMarkers, localMapRender, Settings::Manager::getBool("local map hud fog of war", "Map"))
+        , LocalMapBase(customMarkers, localMapRender, Settings::map().mLocalMapHudFogOfWar)
         , mHealth(nullptr)
         , mMagicka(nullptr)
         , mStamina(nullptr)
@@ -117,7 +136,7 @@ namespace MWGui
         magickaFrame->eventMouseButtonClick += MyGUI::newDelegate(this, &HUD::onHMSClicked);
         fatigueFrame->eventMouseButtonClick += MyGUI::newDelegate(this, &HUD::onHMSClicked);
 
-        //Drowning bar
+        // Drowning bar
         getWidget(mDrowningBar, "DrowningBar");
         getWidget(mDrowningFrame, "DrowningFrame");
         getWidget(mDrowning, "Drowning");
@@ -163,7 +182,7 @@ namespace MWGui
         mMainWidget->eventMouseMove += MyGUI::newDelegate(this, &HUD::onWorldMouseOver);
         mMainWidget->eventMouseLostFocus += MyGUI::newDelegate(this, &HUD::onWorldMouseLostFocus);
 
-        mSpellIcons = new SpellIcons();
+        mSpellIcons = std::make_unique<SpellIcons>();
     }
 
     HUD::~HUD()
@@ -171,15 +190,12 @@ namespace MWGui
         mMainWidget->eventMouseLostFocus.clear();
         mMainWidget->eventMouseMove.clear();
         mMainWidget->eventMouseButtonClick.clear();
-
-        delete mSpellIcons;
     }
 
-    void HUD::setValue(const std::string& id, const MWMechanics::DynamicStat<float>& value)
+    void HUD::setValue(std::string_view id, const MWMechanics::DynamicStat<float>& value)
     {
         int current = static_cast<int>(value.getCurrent());
         int modified = static_cast<int>(value.getModified());
-
         // Fatigue can be negative
         if (id != "FBar")
             current = std::max(0, current);
@@ -229,22 +245,21 @@ namespace MWGui
 
     void HUD::onWorldClicked(MyGUI::Widget* _sender)
     {
-        if (!MWBase::Environment::get().getWindowManager ()->isGuiMode ())
+        if (!MWBase::Environment::get().getWindowManager()->isGuiMode())
             return;
 
-        MWBase::WindowManager *winMgr = MWBase::Environment::get().getWindowManager();
+        MWBase::WindowManager* winMgr = MWBase::Environment::get().getWindowManager();
         if (mDragAndDrop->mIsOnDragAndDrop)
         {
             // drop item into the gameworld
-            MWBase::Environment::get().getWorld()->breakInvisibility(
-                        MWMechanics::getPlayer());
+            MWBase::Environment::get().getWorld()->breakInvisibility(MWMechanics::getPlayer());
 
             MyGUI::IntSize viewSize = MyGUI::RenderManager::getInstance().getViewSize();
             MyGUI::IntPoint cursorPosition = MyGUI::InputManager::getInstance().getMousePosition();
             float mouseX = cursorPosition.left / float(viewSize.width);
             float mouseY = cursorPosition.top / float(viewSize.height);
 
-            WorldItemModel drop (mouseX, mouseY);
+            WorldItemModel drop(mouseX, mouseY);
             mDragAndDrop->drop(&drop, nullptr);
 
             winMgr->changePointer("arrow");
@@ -260,7 +275,7 @@ namespace MWGui
 
             if (winMgr->isConsoleMode())
                 winMgr->setConsoleSelectedObject(object);
-            else //if ((mode == GM_Container) || (mode == GM_Inventory))
+            else // if ((mode == GM_Container) || (mode == GM_Inventory))
             {
                 // pick up object
                 if (!object.isEmpty())
@@ -289,7 +304,6 @@ namespace MWGui
                 MWBase::Environment::get().getWindowManager()->changePointer("drop_ground");
             else
                 MWBase::Environment::get().getWindowManager()->changePointer("arrow");
-
         }
         else
         {
@@ -365,7 +379,7 @@ namespace MWGui
         if (mEnemyHealth->getVisible() && mEnemyHealthTimer < 0)
         {
             mEnemyHealth->setVisible(false);
-            mWeaponSpellBox->setPosition(mWeaponSpellBox->getPosition() + MyGUI::IntPoint(0,20));
+            mWeaponSpellBox->setPosition(mWeaponSpellBox->getPosition() + MyGUI::IntPoint(0, 20));
         }
 
         mSpellIcons->updateWidgets(mEffectBox, true);
@@ -376,11 +390,12 @@ namespace MWGui
         }
 
         if (mDrowningBar->getVisible())
-            mDrowningBar->setPosition(mMainWidget->getWidth()/2 - mDrowningFrame->getWidth()/2, mMainWidget->getTop());
+            mDrowningBar->setPosition(
+                mMainWidget->getWidth() / 2 - mDrowningFrame->getWidth() / 2, mMainWidget->getTop());
 
         if (mIsDrowning)
         {
-            mDrowningFlashTheta += dt * osg::PI*2;
+            mDrowningFlashTheta += dt * osg::PI * 2;
 
             float intensity = (cos(mDrowningFlashTheta) + 2.0f) / 3.0f;
 
@@ -388,12 +403,11 @@ namespace MWGui
         }
     }
 
-    void HUD::setSelectedSpell(const std::string& spellId, int successChancePercent)
+    void HUD::setSelectedSpell(const ESM::RefId& spellId, int successChancePercent)
     {
-        const ESM::Spell* spell =
-            MWBase::Environment::get().getWorld()->getStore().get<ESM::Spell>().find(spellId);
+        const ESM::Spell* spell = MWBase::Environment::get().getESMStore()->get<ESM::Spell>().find(spellId);
 
-        std::string spellName = spell->mName;
+        const std::string& spellName = spell->mName;
         if (spellName != mSpellName && mSpellVisible)
         {
             mWeaponSpellTimer = 5.0f;
@@ -406,23 +420,29 @@ namespace MWGui
         mSpellStatus->setProgressPosition(successChancePercent);
 
         mSpellBox->setUserString("ToolTipType", "Spell");
-        mSpellBox->setUserString("Spell", spellId);
+        mSpellBox->setUserString("Spell", spellId.serialize());
+        mSpellBox->setUserData(MyGUI::Any::Null);
 
-        // use the icon of the first effect
-        const ESM::MagicEffect* effect =
-            MWBase::Environment::get().getWorld()->getStore().get<ESM::MagicEffect>().find(spell->mEffects.mList.front().mEffectID);
-
-        std::string icon = effect->mIcon;
-        int slashPos = icon.rfind('\\');
-        icon.insert(slashPos+1, "b_");
-        icon = Misc::ResourceHelpers::correctIconPath(icon, MWBase::Environment::get().getResourceSystem()->getVFS());
-
-        mSpellImage->setSpellIcon(icon);
+        if (!spell->mEffects.mList.empty())
+        {
+            // use the icon of the first effect
+            const ESM::MagicEffect* effect = MWBase::Environment::get().getESMStore()->get<ESM::MagicEffect>().find(
+                spell->mEffects.mList.front().mData.mEffectID);
+            std::string icon = effect->mIcon;
+            std::replace(icon.begin(), icon.end(), '/', '\\');
+            size_t slashPos = icon.rfind('\\');
+            icon.insert(slashPos + 1, "b_");
+            icon = Misc::ResourceHelpers::correctIconPath(
+                icon, MWBase::Environment::get().getResourceSystem()->getVFS());
+            mSpellImage->setSpellIcon(icon);
+        }
+        else
+            mSpellImage->setSpellIcon({});
     }
 
     void HUD::setSelectedEnchantItem(const MWWorld::Ptr& item, int chargePercent)
     {
-        std::string itemName = item.getClass().getName(item);
+        std::string_view itemName = item.getClass().getName(item);
         if (itemName != mSpellName && mSpellVisible)
         {
             mWeaponSpellTimer = 5.0f;
@@ -442,7 +462,7 @@ namespace MWGui
 
     void HUD::setSelectedWeapon(const MWWorld::Ptr& item, int durabilityPercent)
     {
-        std::string itemName = item.getClass().getName(item);
+        std::string_view itemName = item.getClass().getName(item);
         if (itemName != mWeaponName && mWeaponVisible)
         {
             mWeaponSpellTimer = 5.0f;
@@ -463,7 +483,7 @@ namespace MWGui
 
     void HUD::unsetSelectedSpell()
     {
-        std::string spellName = "#{sNone}";
+        std::string_view spellName = "#{Interface:None}";
         if (spellName != mSpellName && mSpellVisible)
         {
             mWeaponSpellTimer = 5.0f;
@@ -476,6 +496,7 @@ namespace MWGui
         mSpellStatus->setProgressPosition(0);
         mSpellImage->setItem(MWWorld::Ptr());
         mSpellBox->clearUserStrings();
+        mSpellBox->setUserData(MyGUI::Any::Null);
     }
 
     void HUD::unsetSelectedWeapon()
@@ -492,11 +513,12 @@ namespace MWGui
         mWeapStatus->setProgressRange(100);
         mWeapStatus->setProgressPosition(0);
 
-        MWBase::World *world = MWBase::Environment::get().getWorld();
+        MWBase::World* world = MWBase::Environment::get().getWorld();
         MWWorld::Ptr player = world->getPlayerPtr();
 
         mWeapImage->setItem(MWWorld::Ptr());
-        std::string icon = (player.getClass().getNpcStats(player).isWerewolf()) ? "icons\\k\\tx_werewolf_hand.dds" : "icons\\k\\stealth_handtohand.dds";
+        std::string icon = (player.getClass().getNpcStats(player).isWerewolf()) ? "icons\\k\\tx_werewolf_hand.dds"
+                                                                                : "icons\\k\\stealth_handtohand.dds";
         mWeapImage->setIcon(icon);
 
         mWeapBox->clearUserStrings();
@@ -504,16 +526,17 @@ namespace MWGui
         mWeapBox->setUserString("ToolTipLayout", "HandToHandToolTip");
         mWeapBox->setUserString("Caption_HandToHandText", itemName);
         mWeapBox->setUserString("ImageTexture_HandToHandImage", icon);
+        mWeapBox->setUserData(MyGUI::Any::Null);
     }
 
     void HUD::setCrosshairVisible(bool visible)
     {
-        mCrosshair->setVisible (visible);
+        mCrosshair->setVisible(visible);
     }
-    
+
     void HUD::setCrosshairOwned(bool owned)
     {
-        if(owned)
+        if (owned)
         {
             mCrosshair->changeWidgetSkin("HUD_Crosshair_Owned");
         }
@@ -522,7 +545,7 @@ namespace MWGui
             mCrosshair->changeWidgetSkin("HUD_Crosshair");
         }
     }
-    
+
     void HUD::setHmsVisible(bool visible)
     {
         mHealth->setVisible(visible);
@@ -551,13 +574,13 @@ namespace MWGui
 
     void HUD::setEffectVisible(bool visible)
     {
-        mEffectBox->setVisible (visible);
+        mEffectBox->setVisible(visible);
         updatePositions();
     }
 
     void HUD::setMinimapVisible(bool visible)
     {
-        mMinimapBox->setVisible (visible);
+        mMinimapBox->setVisible(visible);
         updatePositions();
     }
 
@@ -589,14 +612,15 @@ namespace MWGui
 
         // effect box can have variable width -> variable left coordinate
         int effectsDx = 0;
-        if (!mMinimapBox->getVisible ())
+        if (!mMinimapBox->getVisible())
             effectsDx = mEffectBoxBaseRight - mMinimapBoxBaseRight;
 
-        mMapVisible = mMinimapBox->getVisible ();
+        mMapVisible = mMinimapBox->getVisible();
         if (!mMapVisible)
             mCellNameBox->setVisible(false);
 
-        mEffectBox->setPosition((viewSize.width - mEffectBoxBaseRight) - mEffectBox->getWidth() + effectsDx, mEffectBox->getTop());
+        mEffectBox->setPosition(
+            (viewSize.width - mEffectBoxBaseRight) - mEffectBox->getWidth() + effectsDx, mEffectBox->getTop());
     }
 
     void HUD::updateEnemyHealthBar()
@@ -610,41 +634,62 @@ namespace MWGui
         // Therefore any value < 1 should show as an empty health bar. We do the same in statswindow :)
         mEnemyHealth->setProgressPosition(static_cast<size_t>(stats.getHealth().getRatio() * 100));
 
-        static const float fNPCHealthBarFade = MWBase::Environment::get().getWorld()->getStore().get<ESM::GameSetting>().find("fNPCHealthBarFade")->mValue.getFloat();
+        static const float fNPCHealthBarFade = MWBase::Environment::get()
+                                                   .getESMStore()
+                                                   ->get<ESM::GameSetting>()
+                                                   .find("fNPCHealthBarFade")
+                                                   ->mValue.getFloat();
         if (fNPCHealthBarFade > 0.f)
             mEnemyHealth->setAlpha(std::clamp(mEnemyHealthTimer / fNPCHealthBarFade, 0.f, 1.f));
-
     }
 
-    void HUD::setEnemy(const MWWorld::Ptr &enemy)
+    void HUD::setEnemy(const MWWorld::Ptr& enemy)
     {
         mEnemyActorId = enemy.getClass().getCreatureStats(enemy).getActorId();
-        mEnemyHealthTimer = MWBase::Environment::get().getWorld()->getStore().get<ESM::GameSetting>().find("fNPCHealthBarTime")->mValue.getFloat();
+        mEnemyHealthTimer = MWBase::Environment::get()
+                                .getESMStore()
+                                ->get<ESM::GameSetting>()
+                                .find("fNPCHealthBarTime")
+                                ->mValue.getFloat();
         if (!mEnemyHealth->getVisible())
-            mWeaponSpellBox->setPosition(mWeaponSpellBox->getPosition() - MyGUI::IntPoint(0,20));
+            mWeaponSpellBox->setPosition(mWeaponSpellBox->getPosition() - MyGUI::IntPoint(0, 20));
         mEnemyHealth->setVisible(true);
         updateEnemyHealthBar();
     }
 
-    void HUD::resetEnemy()
+    void HUD::clear()
     {
         mEnemyActorId = -1;
         mEnemyHealthTimer = -1;
+
+        mWeaponSpellTimer = 0.f;
+        mWeaponName = std::string();
+        mSpellName = std::string();
+        mWeaponSpellBox->setVisible(false);
+
+        mWeapStatus->setProgressRange(100);
+        mWeapStatus->setProgressPosition(0);
+        mSpellStatus->setProgressRange(100);
+        mSpellStatus->setProgressPosition(0);
+
+        mWeapImage->setItem(MWWorld::Ptr());
+        mSpellImage->setItem(MWWorld::Ptr());
+
+        mWeapBox->clearUserStrings();
+        mWeapBox->setUserData(MyGUI::Any::Null);
+        mSpellBox->clearUserStrings();
+        mSpellBox->setUserData(MyGUI::Any::Null);
+
+        mActiveCell = nullptr;
+        mHasALastActiveCell = false;
     }
 
-    void HUD::clear()
-    {
-        unsetSelectedSpell();
-        unsetSelectedWeapon();
-        resetEnemy();
-    }
-
-    void HUD::customMarkerCreated(MyGUI::Widget *marker)
+    void HUD::customMarkerCreated(MyGUI::Widget* marker)
     {
         marker->eventMouseButtonClick += MyGUI::newDelegate(this, &HUD::onMapClicked);
     }
 
-    void HUD::doorMarkerCreated(MyGUI::Widget *marker)
+    void HUD::doorMarkerCreated(MyGUI::Widget* marker)
     {
         marker->eventMouseButtonClick += MyGUI::newDelegate(this, &HUD::onMapClicked);
     }
