@@ -3,41 +3,43 @@
 #include <cmath>
 #include <stdexcept>
 
-#include <MyGUI_Window.h>
-#include <MyGUI_ImageBox.h>
-#include <MyGUI_RenderManager.h>
-#include <MyGUI_InputManager.h>
 #include <MyGUI_Button.h>
 #include <MyGUI_EditBox.h>
+#include <MyGUI_ImageBox.h>
+#include <MyGUI_InputManager.h>
+#include <MyGUI_RenderManager.h>
+#include <MyGUI_Window.h>
 
 #include <osg/Texture2D>
 
-#include <components/misc/stringops.hpp>
+#include <components/misc/strings/algorithm.hpp>
 
 #include <components/myguiplatform/myguitexture.hpp>
 
-#include <components/settings/settings.hpp>
+#include <components/settings/values.hpp>
 
-#include "../mwbase/world.hpp"
 #include "../mwbase/environment.hpp"
-#include "../mwbase/windowmanager.hpp"
+#include "../mwbase/luamanager.hpp"
 #include "../mwbase/mechanicsmanager.hpp"
+#include "../mwbase/windowmanager.hpp"
+#include "../mwbase/world.hpp"
 
-#include "../mwworld/inventorystore.hpp"
+#include "../mwworld/action.hpp"
 #include "../mwworld/class.hpp"
-#include "../mwworld/actionequip.hpp"
+#include "../mwworld/inventorystore.hpp"
 
-#include "../mwmechanics/npcstats.hpp"
 #include "../mwmechanics/actorutil.hpp"
+#include "../mwmechanics/npcstats.hpp"
 
-#include "itemview.hpp"
-#include "inventoryitemmodel.hpp"
-#include "sortfilteritemmodel.hpp"
-#include "tradeitemmodel.hpp"
 #include "countdialog.hpp"
-#include "tradewindow.hpp"
 #include "draganddrop.hpp"
+#include "inventoryitemmodel.hpp"
+#include "itemview.hpp"
+#include "settings.hpp"
+#include "sortfilteritemmodel.hpp"
 #include "tooltips.hpp"
+#include "tradeitemmodel.hpp"
+#include "tradewindow.hpp"
 
 namespace
 {
@@ -54,8 +56,26 @@ namespace
 
 namespace MWGui
 {
+    namespace
+    {
+        WindowSettingValues getModeSettings(GuiMode mode)
+        {
+            switch (mode)
+            {
+                case GM_Container:
+                    return makeInventoryContainerWindowSettingValues();
+                case GM_Companion:
+                    return makeInventoryCompanionWindowSettingValues();
+                case GM_Barter:
+                    return makeInventoryBarterWindowSettingValues();
+                default:
+                    return makeInventoryWindowSettingValues();
+            }
+        }
+    }
 
-    InventoryWindow::InventoryWindow(DragAndDrop* dragAndDrop, osg::Group* parent, Resource::ResourceSystem* resourceSystem)
+    InventoryWindow::InventoryWindow(
+        DragAndDrop* dragAndDrop, osg::Group* parent, Resource::ResourceSystem* resourceSystem)
         : WindowPinnableBase("openmw_inventory_window.layout")
         , mDragAndDrop(dragAndDrop)
         , mSelectedItem(-1)
@@ -68,10 +88,12 @@ namespace MWGui
         , mTrading(false)
         , mUpdateTimer(0.f)
     {
-        mPreviewTexture = std::make_unique<osgMyGUI::OSGTexture>(mPreview->getTexture(), mPreview->getTextureStateSet());
+        mPreviewTexture
+            = std::make_unique<osgMyGUI::OSGTexture>(mPreview->getTexture(), mPreview->getTextureStateSet());
         mPreview->rebuild();
 
-        mMainWidget->castType<MyGUI::Window>()->eventWindowChangeCoord += MyGUI::newDelegate(this, &InventoryWindow::onWindowResize);
+        mMainWidget->castType<MyGUI::Window>()->eventWindowChangeCoord
+            += MyGUI::newDelegate(this, &InventoryWindow::onWindowResize);
 
         getWidget(mAvatar, "Avatar");
         getWidget(mAvatarImage, "AvatarImage");
@@ -112,27 +134,28 @@ namespace MWGui
     {
         const float aspect = 0.5; // fixed aspect ratio for the avatar image
         int leftPaneWidth = static_cast<int>((mMainWidget->getSize().height - 44 - mArmorRating->getHeight()) * aspect);
-        mLeftPane->setSize( leftPaneWidth, mMainWidget->getSize().height-44 );
-        mRightPane->setCoord( mLeftPane->getPosition().left + leftPaneWidth + 4,
-                              mRightPane->getPosition().top,
-                              mMainWidget->getSize().width - 12 - leftPaneWidth - 15,
-                              mMainWidget->getSize().height-44 );
+        mLeftPane->setSize(leftPaneWidth, mMainWidget->getSize().height - 44);
+        mRightPane->setCoord(mLeftPane->getPosition().left + leftPaneWidth + 4, mRightPane->getPosition().top,
+            mMainWidget->getSize().width - 12 - leftPaneWidth - 15, mMainWidget->getSize().height - 44);
     }
 
     void InventoryWindow::updatePlayer()
     {
-        mPtr = MWBase::Environment::get().getWorld ()->getPlayerPtr();
-        mTradeModel = new TradeItemModel(new InventoryItemModel(mPtr), MWWorld::Ptr());
+        mPtr = MWBase::Environment::get().getWorld()->getPlayerPtr();
+        auto tradeModel = std::make_unique<TradeItemModel>(std::make_unique<InventoryItemModel>(mPtr), MWWorld::Ptr());
+        mTradeModel = tradeModel.get();
 
         if (mSortModel) // reuse existing SortModel when possible to keep previous category/filter settings
-            mSortModel->setSourceModel(mTradeModel);
+            mSortModel->setSourceModel(std::move(tradeModel));
         else
-            mSortModel = new SortFilterItemModel(mTradeModel);
+        {
+            auto sortModel = std::make_unique<SortFilterItemModel>(std::move(tradeModel));
+            mSortModel = sortModel.get();
+            mItemView->setModel(std::move(sortModel));
+        }
 
         mSortModel->setNameFilter(mFilterEdit->getCaption());
 
-        mItemView->setModel(mSortModel);
-
         mFilterAll->setStateSelected(true);
         mFilterWeapon->setStateSelected(false);
         mFilterApparel->setStateSelected(false);
@@ -162,24 +185,18 @@ namespace MWGui
 
     void InventoryWindow::toggleMaximized()
     {
-        std::string setting = getModeSetting();
-
-        bool maximized = !Settings::Manager::getBool(setting + " maximized", "Windows");
-        if (maximized)
-            setting += " maximized";
+        const WindowSettingValues settings = getModeSettings(mGuiMode);
+        const WindowRectSettingValues& rect = settings.mIsMaximized ? settings.mRegular : settings.mMaximized;
 
         MyGUI::IntSize viewSize = MyGUI::RenderManager::getInstance().getViewSize();
-        float x = Settings::Manager::getFloat(setting + " x", "Windows") * float(viewSize.width);
-        float y = Settings::Manager::getFloat(setting + " y", "Windows") * float(viewSize.height);
-        float w = Settings::Manager::getFloat(setting + " w", "Windows") * float(viewSize.width);
-        float h = Settings::Manager::getFloat(setting + " h", "Windows") * float(viewSize.height);
+        const float x = rect.mX * viewSize.width;
+        const float y = rect.mY * viewSize.height;
+        const float w = rect.mW * viewSize.width;
+        const float h = rect.mH * viewSize.height;
         MyGUI::Window* window = mMainWidget->castType<MyGUI::Window>();
         window->setCoord(x, y, w, h);
 
-        if (maximized)
-            Settings::Manager::setBool(setting, "Windows", maximized);
-        else
-            Settings::Manager::setBool(setting + " maximized", "Windows", maximized);
+        settings.mIsMaximized.set(!settings.mIsMaximized);
 
         adjustPanes();
         updatePreviewSize();
@@ -188,17 +205,14 @@ namespace MWGui
     void InventoryWindow::setGuiMode(GuiMode mode)
     {
         mGuiMode = mode;
-        std::string setting = getModeSetting();
+        const WindowSettingValues settings = getModeSettings(mGuiMode);
         setPinButtonVisible(mode == GM_Inventory);
 
-        if (Settings::Manager::getBool(setting + " maximized", "Windows"))
-            setting += " maximized";
+        const WindowRectSettingValues& rect = settings.mIsMaximized ? settings.mMaximized : settings.mRegular;
 
         MyGUI::IntSize viewSize = MyGUI::RenderManager::getInstance().getViewSize();
-        MyGUI::IntPoint pos(static_cast<int>(Settings::Manager::getFloat(setting + " x", "Windows") * viewSize.width),
-                            static_cast<int>(Settings::Manager::getFloat(setting + " y", "Windows") * viewSize.height));
-        MyGUI::IntSize size(static_cast<int>(Settings::Manager::getFloat(setting + " w", "Windows") * viewSize.width),
-                            static_cast<int>(Settings::Manager::getFloat(setting + " h", "Windows") * viewSize.height));
+        MyGUI::IntPoint pos(static_cast<int>(rect.mX * viewSize.width), static_cast<int>(rect.mY * viewSize.height));
+        MyGUI::IntSize size(static_cast<int>(rect.mW * viewSize.width), static_cast<int>(rect.mH * viewSize.height));
 
         bool needUpdate = (size.width != mMainWidget->getWidth() || size.height != mMainWidget->getHeight());
 
@@ -232,12 +246,12 @@ namespace MWGui
             mDragAndDrop->drop(mTradeModel, mItemView);
     }
 
-    void InventoryWindow::onItemSelected (int index)
+    void InventoryWindow::onItemSelected(int index)
     {
-        onItemSelectedFromSourceModel (mSortModel->mapToSource(index));
+        onItemSelectedFromSourceModel(mSortModel->mapToSource(index));
     }
 
-    void InventoryWindow::onItemSelectedFromSourceModel (int index)
+    void InventoryWindow::onItemSelectedFromSourceModel(int index)
     {
         if (mDragAndDrop->mIsOnDragAndDrop)
         {
@@ -246,7 +260,7 @@ namespace MWGui
         }
 
         const ItemStack& item = mTradeModel->getItem(index);
-        std::string sound = item.mBase.getClass().getDownSoundId(item.mBase);
+        const ESM::RefId& sound = item.mBase.getClass().getDownSoundId(item.mBase);
 
         MWWorld::Ptr object = item.mBase;
         int count = item.mCount;
@@ -270,8 +284,7 @@ namespace MWGui
             if (!object.getClass().canSell(object, services))
             {
                 MWBase::Environment::get().getWindowManager()->playSound(sound);
-                MWBase::Environment::get().getWindowManager()->
-                        messageBox("#{sBarterDialog4}");
+                MWBase::Environment::get().getWindowManager()->messageBox("#{sBarterDialog4}");
                 return;
             }
         }
@@ -293,7 +306,8 @@ namespace MWGui
         {
             CountDialog* dialog = MWBase::Environment::get().getWindowManager()->getCountDialog();
             std::string message = mTrading ? "#{sQuanityMenuMessage01}" : "#{sTake}";
-            std::string name = object.getClass().getName(object) + MWGui::ToolTips::getSoulString(object.getCellRef());
+            std::string name{ object.getClass().getName(object) };
+            name += MWGui::ToolTips::getSoulString(object.getCellRef());
             dialog->openCountDialog(name, message, count);
             dialog->eventOkClicked.clear();
             if (mTrading)
@@ -306,9 +320,9 @@ namespace MWGui
         {
             mSelectedItem = index;
             if (mTrading)
-                sellItem (nullptr, count);
+                sellItem(nullptr, count);
             else
-                dragItem (nullptr, count);
+                dragItem(nullptr, count);
         }
     }
 
@@ -318,17 +332,17 @@ namespace MWGui
         if (item.mType == ItemStack::Type_Equipped)
         {
             MWWorld::InventoryStore& invStore = mPtr.getClass().getInventoryStore(mPtr);
-            MWWorld::Ptr newStack = *invStore.unequipItemQuantity(item.mBase, mPtr, count);
+            MWWorld::Ptr newStack = *invStore.unequipItemQuantity(item.mBase, count);
 
             // The unequipped item was re-stacked. We have to update the index
             // since the item pointed does not exist anymore.
             if (item.mBase != newStack)
             {
-                updateItemView();  // Unequipping can produce a new stack, not yet in the window...
+                updateItemView(); // Unequipping can produce a new stack, not yet in the window...
 
                 // newIndex will store the index of the ItemStack the item was stacked on
                 int newIndex = -1;
-                for (size_t i=0; i < mTradeModel->getItemCount(); ++i)
+                for (size_t i = 0; i < mTradeModel->getItemCount(); ++i)
                 {
                     if (mTradeModel->getItem(i).mBase == newStack)
                     {
@@ -356,7 +370,7 @@ namespace MWGui
     {
         ensureSelectedItemUnequipped(count);
         const ItemStack& item = mTradeModel->getItem(mSelectedItem);
-        std::string sound = item.mBase.getClass().getUpSoundId(item.mBase);
+        const ESM::RefId& sound = item.mBase.getClass().getUpSoundId(item.mBase);
         MWBase::Environment::get().getWindowManager()->playSound(sound);
 
         if (item.mType == ItemStack::Type_Barter)
@@ -401,44 +415,20 @@ namespace MWGui
         adjustPanes();
     }
 
-    std::string InventoryWindow::getModeSetting() const
-    {
-        std::string setting = "inventory";
-        switch(mGuiMode)
-        {
-            case GM_Container:
-                setting += " container";
-                break;
-            case GM_Companion:
-                setting += " companion";
-                break;
-            case GM_Barter:
-                setting += " barter";
-                break;
-            default:
-                break;
-        }
-
-        return setting;
-    }
-
     void InventoryWindow::onWindowResize(MyGUI::Window* _sender)
     {
+        WindowBase::clampWindowCoordinates(_sender);
+
         adjustPanes();
-        std::string setting = getModeSetting();
+        const WindowSettingValues settings = getModeSettings(mGuiMode);
 
         MyGUI::IntSize viewSize = MyGUI::RenderManager::getInstance().getViewSize();
-        float x = _sender->getPosition().left / float(viewSize.width);
-        float y = _sender->getPosition().top / float(viewSize.height);
-        float w = _sender->getSize().width / float(viewSize.width);
-        float h = _sender->getSize().height / float(viewSize.height);
-        Settings::Manager::setFloat(setting + " x", "Windows", x);
-        Settings::Manager::setFloat(setting + " y", "Windows", y);
-        Settings::Manager::setFloat(setting + " w", "Windows", w);
-        Settings::Manager::setFloat(setting + " h", "Windows", h);
-        bool maximized = Settings::Manager::getBool(setting + " maximized", "Windows");
-        if (maximized)
-            Settings::Manager::setBool(setting + " maximized", "Windows", false);
+
+        settings.mRegular.mX.set(_sender->getPosition().left / static_cast<float>(viewSize.width));
+        settings.mRegular.mY.set(_sender->getPosition().top / static_cast<float>(viewSize.height));
+        settings.mRegular.mW.set(_sender->getSize().width / static_cast<float>(viewSize.width));
+        settings.mRegular.mH.set(_sender->getSize().height / static_cast<float>(viewSize.height));
+        settings.mIsMaximized.set(false);
 
         if (mMainWidget->getSize().width != mLastXSize || mMainWidget->getSize().height != mLastYSize)
         {
@@ -452,18 +442,23 @@ namespace MWGui
 
     void InventoryWindow::updateArmorRating()
     {
-        mArmorRating->setCaptionWithReplacing ("#{sArmor}: "
-            + MyGUI::utility::toString(static_cast<int>(mPtr.getClass().getArmorRating(mPtr))));
+        if (mPtr.isEmpty())
+            return;
+
+        mArmorRating->setCaptionWithReplacing(
+            "#{sArmor}: " + MyGUI::utility::toString(static_cast<int>(mPtr.getClass().getArmorRating(mPtr))));
         if (mArmorRating->getTextSize().width > mArmorRating->getSize().width)
-            mArmorRating->setCaptionWithReplacing (MyGUI::utility::toString(static_cast<int>(mPtr.getClass().getArmorRating(mPtr))));
+            mArmorRating->setCaptionWithReplacing(
+                MyGUI::utility::toString(static_cast<int>(mPtr.getClass().getArmorRating(mPtr))));
     }
 
     void InventoryWindow::updatePreviewSize()
     {
         const MyGUI::IntSize viewport = getPreviewViewportSize();
         mPreview->setViewport(viewport.width, viewport.height);
-        mAvatarImage->getSubWidgetMain()->_setUVSet(MyGUI::FloatRect(0.f, 0.f,
-                                                                     viewport.width / float(mPreview->getTextureWidth()), viewport.height / float(mPreview->getTextureHeight())));
+        mAvatarImage->getSubWidgetMain()->_setUVSet(
+            MyGUI::FloatRect(0.f, 0.f, viewport.width / float(mPreview->getTextureWidth()),
+                viewport.height / float(mPreview->getTextureHeight())));
     }
 
     void InventoryWindow::onNameFilterChanged(MyGUI::EditBox* _sender)
@@ -497,7 +492,7 @@ namespace MWGui
 
     void InventoryWindow::onPinToggled()
     {
-        Settings::Manager::setBool("inventory pin", "Windows", mPinned);
+        Settings::windows().mInventoryPin.set(mPinned);
 
         MWBase::Environment::get().getWindowManager()->setWeaponVisibility(!mPinned);
     }
@@ -510,9 +505,9 @@ namespace MWGui
             MWBase::Environment::get().getWindowManager()->toggleVisible(GW_Inventory);
     }
 
-    void InventoryWindow::useItem(const MWWorld::Ptr &ptr, bool force)
+    void InventoryWindow::useItem(const MWWorld::Ptr& ptr, bool force)
     {
-        const std::string& script = ptr.getClass().getScript(ptr);
+        const ESM::RefId& script = ptr.getClass().getScript(ptr);
         if (!script.empty())
         {
             // Don't try to equip the item if PCSkipEquip is set to 1
@@ -526,7 +521,8 @@ namespace MWGui
 
         MWWorld::Ptr player = MWMechanics::getPlayer();
 
-        // early-out for items that need to be equipped, but can't be equipped: we don't want to set OnPcEquip in that case
+        // early-out for items that need to be equipped, but can't be equipped: we don't want to set OnPcEquip in that
+        // case
         if (!ptr.getClass().getEquipmentSlots(ptr).first.empty())
         {
             if (ptr.getClass().hasItemHealth(ptr) && ptr.getCellRef().getCharge() == 0)
@@ -538,7 +534,7 @@ namespace MWGui
 
             if (!force)
             {
-                std::pair<int, std::string> canEquip = ptr.getClass().canBeEquipped(ptr, player);
+                auto canEquip = ptr.getClass().canBeEquipped(ptr, player);
 
                 if (canEquip.first == 0)
                 {
@@ -565,6 +561,20 @@ namespace MWGui
         std::unique_ptr<MWWorld::Action> action = ptr.getClass().use(ptr, force);
         action->execute(player);
 
+        // Handles partial equipping (final part)
+        if (mEquippedStackableCount.has_value())
+        {
+            // the count to unequip
+            int count = ptr.getCellRef().getCount() - mDragAndDrop->mDraggedCount - mEquippedStackableCount.value();
+            if (count > 0)
+            {
+                MWWorld::InventoryStore& invStore = mPtr.getClass().getInventoryStore(mPtr);
+                invStore.unequipItemQuantity(ptr, count);
+                updateItemView();
+            }
+            mEquippedStackableCount.reset();
+        }
+
         if (isVisible())
         {
             mItemView->update();
@@ -585,14 +595,30 @@ namespace MWGui
             if (mDragAndDrop->mSourceModel != mTradeModel)
             {
                 // Move item to the player's inventory
-                ptr = mDragAndDrop->mSourceModel->moveItem(mDragAndDrop->mItem, mDragAndDrop->mDraggedCount, mTradeModel);
+                ptr = mDragAndDrop->mSourceModel->moveItem(
+                    mDragAndDrop->mItem, mDragAndDrop->mDraggedCount, mTradeModel);
             }
 
-            useItem(ptr);
+            // Handles partial equipping
+            mEquippedStackableCount.reset();
+            const auto slots = ptr.getClass().getEquipmentSlots(ptr);
+            if (!slots.first.empty() && slots.second)
+            {
+                MWWorld::InventoryStore& invStore = mPtr.getClass().getInventoryStore(mPtr);
+                MWWorld::ConstContainerStoreIterator slotIt = invStore.getSlot(slots.first.front());
 
-            // If item is ingredient or potion don't stop drag and drop to simplify action of taking more than one 1 item
-            if ((ptr.getType() == ESM::Potion::sRecordId ||
-                 ptr.getType() == ESM::Ingredient::sRecordId)
+                // Save the currently equipped count before useItem()
+                if (slotIt != invStore.end() && slotIt->getCellRef().getRefId() == ptr.getCellRef().getRefId())
+                    mEquippedStackableCount = slotIt->getCellRef().getCount();
+                else
+                    mEquippedStackableCount = 0;
+            }
+
+            MWBase::Environment::get().getLuaManager()->useItem(ptr, MWMechanics::getPlayer(), false);
+
+            // If item is ingredient or potion don't stop drag and drop to simplify action of taking more than one 1
+            // item
+            if ((ptr.getType() == ESM::Potion::sRecordId || ptr.getType() == ESM::Ingredient::sRecordId)
                 && mDragAndDrop->mDraggedCount > 1)
             {
                 // Item can be provided from other window for example container.
@@ -603,14 +629,15 @@ namespace MWGui
         }
         else
         {
-            MyGUI::IntPoint mousePos = MyGUI::InputManager::getInstance ().getLastPressedPosition (MyGUI::MouseButton::Left);
-            MyGUI::IntPoint relPos = mousePos - mAvatarImage->getAbsolutePosition ();
+            MyGUI::IntPoint mousePos
+                = MyGUI::InputManager::getInstance().getLastPressedPosition(MyGUI::MouseButton::Left);
+            MyGUI::IntPoint relPos = mousePos - mAvatarImage->getAbsolutePosition();
 
-            MWWorld::Ptr itemSelected = getAvatarSelectedItem (relPos.left, relPos.top);
-            if (itemSelected.isEmpty ())
+            MWWorld::Ptr itemSelected = getAvatarSelectedItem(relPos.left, relPos.top);
+            if (itemSelected.isEmpty())
                 return;
 
-            for (size_t i=0; i < mTradeModel->getItemCount (); ++i)
+            for (size_t i = 0; i < mTradeModel->getItemCount(); ++i)
             {
                 if (mTradeModel->getItem(i).mBase == itemSelected)
                 {
@@ -631,7 +658,7 @@ namespace MWGui
             return MWWorld::Ptr();
 
         MWWorld::InventoryStore& invStore = mPtr.getClass().getInventoryStore(mPtr);
-        if(invStore.getSlot(slot) != invStore.end())
+        if (invStore.getSlot(slot) != invStore.end())
         {
             MWWorld::Ptr item = *invStore.getSlot(slot);
             if (!item.getClass().showsInInventory(item))
@@ -690,72 +717,67 @@ namespace MWGui
         // update the spell window just in case new enchanted items were added to inventory
         MWBase::Environment::get().getWindowManager()->updateSpellWindow();
 
-        MWBase::Environment::get().getMechanicsManager()->updateMagicEffects(
-                    MWMechanics::getPlayer());
+        MWBase::Environment::get().getMechanicsManager()->updateMagicEffects(MWMechanics::getPlayer());
 
         dirtyPreview();
     }
 
-    void InventoryWindow::pickUpObject (MWWorld::Ptr object)
+    void InventoryWindow::pickUpObject(MWWorld::Ptr object)
     {
         // If the inventory is not yet enabled, don't pick anything up
         if (!MWBase::Environment::get().getWindowManager()->isAllowed(GW_Inventory))
             return;
         // make sure the object is of a type that can be picked up
         auto type = object.getType();
-        if ( (type != ESM::Apparatus::sRecordId)
-            && (type != ESM::Armor::sRecordId)
-            && (type != ESM::Book::sRecordId)
-            && (type != ESM::Clothing::sRecordId)
-            && (type != ESM::Ingredient::sRecordId)
-            && (type != ESM::Light::sRecordId)
-            && (type != ESM::Miscellaneous::sRecordId)
-            && (type != ESM::Lockpick::sRecordId)
-            && (type != ESM::Probe::sRecordId)
-            && (type != ESM::Repair::sRecordId)
-            && (type != ESM::Weapon::sRecordId)
-            && (type != ESM::Potion::sRecordId))
+        if ((type != ESM::Apparatus::sRecordId) && (type != ESM::Armor::sRecordId) && (type != ESM::Book::sRecordId)
+            && (type != ESM::Clothing::sRecordId) && (type != ESM::Ingredient::sRecordId)
+            && (type != ESM::Light::sRecordId) && (type != ESM::Miscellaneous::sRecordId)
+            && (type != ESM::Lockpick::sRecordId) && (type != ESM::Probe::sRecordId) && (type != ESM::Repair::sRecordId)
+            && (type != ESM::Weapon::sRecordId) && (type != ESM::Potion::sRecordId))
             return;
 
         // An object that can be picked up must have a tooltip.
         if (!object.getClass().hasToolTip(object))
             return;
 
-        int count = object.getRefData().getCount();
+        int count = object.getCellRef().getCount();
         if (object.getClass().isGold(object))
             count *= object.getClass().getValue(object);
 
         MWWorld::Ptr player = MWMechanics::getPlayer();
         MWBase::Environment::get().getWorld()->breakInvisibility(player);
-        
+
         if (!object.getRefData().activate())
             return;
 
         // Player must not be paralyzed, knocked down, or dead to pick up an item.
         const MWMechanics::NpcStats& playerStats = player.getClass().getNpcStats(player);
-        bool godmode = MWBase::Environment::get().getWorld()->getGodModeState();
-        if ((!godmode && playerStats.isParalyzed()) || playerStats.getKnockedDown() || playerStats.isDead())
+        if (playerStats.isParalyzed() || playerStats.getKnockedDown() || playerStats.isDead())
             return;
 
         MWBase::Environment::get().getMechanicsManager()->itemTaken(player, object, MWWorld::Ptr(), count);
 
         // add to player inventory
         // can't use ActionTake here because we need an MWWorld::Ptr to the newly inserted object
-        MWWorld::Ptr newObject = *player.getClass().getContainerStore (player).add (object, object.getRefData().getCount(), player);
+        MWWorld::Ptr newObject = *player.getClass().getContainerStore(player).add(object, count);
 
         // remove from world
-        MWBase::Environment::get().getWorld()->deleteObject (object);
+        MWBase::Environment::get().getWorld()->deleteObject(object);
 
         // get ModelIndex to the item
         mTradeModel->update();
-        size_t i=0;
-        for (; i<mTradeModel->getItemCount(); ++i)
+        size_t i = 0;
+        for (; i < mTradeModel->getItemCount(); ++i)
         {
             if (mTradeModel->getItem(i).mBase == newObject)
                 break;
         }
         if (i == mTradeModel->getItemCount())
             throw std::runtime_error("Added item not found");
+
+        if (mDragAndDrop->mIsOnDragAndDrop)
+            mDragAndDrop->finish();
+
         mDragAndDrop->startDrag(i, mSortModel, mTradeModel, mItemView, count);
 
         MWBase::Environment::get().getWindowManager()->updateSpellWindow();
@@ -768,20 +790,19 @@ namespace MWGui
         if (MWBase::Environment::get().getMechanicsManager()->isAttackingOrSpell(player))
             return;
 
-        const MWMechanics::CreatureStats &stats = player.getClass().getCreatureStats(player);
-        bool godmode = MWBase::Environment::get().getWorld()->getGodModeState();
-        if ((!godmode && stats.isParalyzed()) || stats.getKnockedDown() || stats.isDead() || stats.getHitRecovery())
+        const MWMechanics::CreatureStats& stats = player.getClass().getCreatureStats(player);
+        if (stats.isParalyzed() || stats.getKnockedDown() || stats.isDead() || stats.getHitRecovery())
             return;
 
         ItemModel::ModelIndex selected = -1;
         // not using mSortFilterModel as we only need sorting, not filtering
-        SortFilterItemModel model(new InventoryItemModel(player));
+        SortFilterItemModel model(std::make_unique<InventoryItemModel>(player));
         model.setSortByType(false);
         model.update();
         if (model.getItemCount() == 0)
             return;
 
-        for (ItemModel::ModelIndex i=0; i<int(model.getItemCount()); ++i)
+        for (ItemModel::ModelIndex i = 0; i < int(model.getItemCount()); ++i)
         {
             MWWorld::Ptr item = model.getItem(i).mBase;
             if (model.getItem(i).mType & ItemStack::Type_Equipped && isRightHandWeapon(item))
@@ -790,26 +811,26 @@ namespace MWGui
 
         int incr = next ? 1 : -1;
         bool found = false;
-        std::string lastId;
+        ESM::RefId lastId;
         if (selected != -1)
             lastId = model.getItem(selected).mBase.getCellRef().getRefId();
         ItemModel::ModelIndex cycled = selected;
-        for (unsigned int i=0; i<model.getItemCount(); ++i)
+        for (size_t i = 0; i < model.getItemCount(); ++i)
         {
             cycled += incr;
             cycled = (cycled + model.getItemCount()) % model.getItemCount();
 
             MWWorld::Ptr item = model.getItem(cycled).mBase;
 
-            // skip different stacks of the same item, or we will get stuck as stacking/unstacking them may change their relative ordering
-            if (Misc::StringUtils::ciEqual(lastId, item.getCellRef().getRefId()))
+            // skip different stacks of the same item, or we will get stuck as stacking/unstacking them may change their
+            // relative ordering
+            if (lastId == item.getCellRef().getRefId())
                 continue;
 
             lastId = item.getCellRef().getRefId();
 
-            if (item.getClass().getType() == ESM::Weapon::sRecordId &&
-                isRightHandWeapon(item) &&
-                item.getClass().canBeEquipped(item, player).first)
+            if (item.getClass().getType() == ESM::Weapon::sRecordId && isRightHandWeapon(item)
+                && item.getClass().canBeEquipped(item, player).first)
             {
                 found = true;
                 break;
@@ -833,7 +854,7 @@ namespace MWGui
         const float scale = MWBase::Environment::get().getWindowManager()->getScalingFactor();
 
         return MyGUI::IntSize(std::min<int>(mPreview->getTextureWidth(), previewWindowSize.width * scale),
-                              std::min<int>(mPreview->getTextureHeight(), previewWindowSize.height * scale));
+            std::min<int>(mPreview->getTextureHeight(), previewWindowSize.height * scale));
     }
 
     osg::Vec2f InventoryWindow::mapPreviewWindowToViewport(int x, int y) const
@@ -843,9 +864,6 @@ namespace MWGui
         const float normalisedY = y / std::max<float>(1.0f, previewWindowSize.height);
 
         const MyGUI::IntSize viewport = getPreviewViewportSize();
-        return osg::Vec2f(
-            normalisedX * float(viewport.width - 1),
-            (1.0 - normalisedY) * float(viewport.height - 1)
-        );
+        return osg::Vec2f(normalisedX * float(viewport.width - 1), (1.0 - normalisedY) * float(viewport.height - 1));
     }
 }
