@@ -14,7 +14,6 @@ namespace MWGui
 {
     class ItemView;
     class SortFilterItemModel;
-    class InventoryItemModel;
 
     class ItemSelectionDialog : public WindowModal
     {
@@ -23,20 +22,21 @@ namespace MWGui
 
         bool exit() override;
 
-        typedef MyGUI::delegates::CMultiDelegate0 EventHandle_Void;
-        typedef MyGUI::delegates::CMultiDelegate1<MWWorld::Ptr> EventHandle_Item;
+        typedef MyGUI::delegates::MultiDelegate<> EventHandle_Void;
+        typedef MyGUI::delegates::MultiDelegate<MWWorld::Ptr> EventHandle_Item;
 
         EventHandle_Item eventItemSelected;
         EventHandle_Void eventDialogCanceled;
 
-        void openContainer (const MWWorld::Ptr& container);
+        void openContainer(const MWWorld::Ptr& container);
         void setCategory(int category);
         void setFilter(int filter);
 
+        SortFilterItemModel* getSortModel() { return mSortModel; }
+
     private:
         ItemView* mItemView;
         SortFilterItemModel* mSortModel;
-        InventoryItemModel* mModel;
 
         void onSelectedItem(int index);
 
