@@ -1,10 +1,10 @@
 #ifndef MWGUI_JOURNALVIEWMODEL_HPP
 #define MWGUI_JOURNALVIEWMODEL_HPP
 
-#include <string>
-#include <memory>
-#include <functional>
 #include <cstdint>
+#include <functional>
+#include <memory>
+#include <string>
 
 #include <components/misc/utf8stream.hpp>
 
@@ -18,12 +18,12 @@ namespace MWGui
     /// game data store.
     struct JournalViewModel
     {
-        typedef std::shared_ptr <JournalViewModel> Ptr;
+        typedef std::shared_ptr<JournalViewModel> Ptr;
 
         typedef intptr_t QuestId;
         typedef intptr_t TopicId;
-        typedef uint8_t const * Utf8Point;
-        typedef std::pair <Utf8Point, Utf8Point> Utf8Span;
+        typedef uint8_t const* Utf8Point;
+        typedef std::pair<Utf8Point, Utf8Point> Utf8Span;
 
         /// The base interface for both journal entries and topics.
         struct Entry
@@ -33,12 +33,12 @@ namespace MWGui
             /// This function returns a borrowed reference to the body of the
             /// journal entry. The returned reference becomes invalid when the
             /// entry is destroyed.
-            virtual Utf8Span body () const = 0;
+            virtual Utf8Span body() const = 0;
 
             /// Visits each subset of text in the body, delivering the beginning
             /// and end of the span relative to the body, and a valid topic ID if
             /// the span represents a keyword, or zero if not.
-            virtual void visitSpans (std::function <void (TopicId, size_t, size_t)> visitor) const = 0;
+            virtual void visitSpans(std::function<void(TopicId, size_t, size_t)> visitor) const = 0;
 
             virtual ~Entry() = default;
         };
@@ -48,7 +48,7 @@ namespace MWGui
         {
             /// Returns a pre-formatted span of UTF8 encoded text representing
             /// the name of the NPC this portion of dialog was heard from.
-            virtual Utf8Span source () const = 0;
+            virtual Utf8Span source() const = 0;
 
             virtual ~TopicEntry() = default;
         };
@@ -58,38 +58,40 @@ namespace MWGui
         {
             /// Returns a pre-formatted span of UTF8 encoded text representing
             /// the in-game date this entry was added to the journal.
-            virtual Utf8Span timestamp () const = 0;
+            virtual Utf8Span timestamp() const = 0;
 
             virtual ~JournalEntry() = default;
         };
 
         /// called prior to journal opening
-        virtual void load () = 0;
+        virtual void load() = 0;
 
         /// called prior to journal closing
-        virtual void unload () = 0;
+        virtual void unload() = 0;
 
         /// returns true if their are no journal entries to display
-        virtual bool isEmpty () const = 0;
+        virtual bool isEmpty() const = 0;
 
         /// walks the active and optionally completed, quests providing the name and completed status
-        virtual void visitQuestNames (bool active_only, std::function <void (const std::string&, bool)> visitor) const = 0;
+        virtual void visitQuestNames(bool active_only, std::function<void(std::string_view, bool)> visitor) const = 0;
 
         /// walks over the journal entries related to all quests with the given name
         /// If \a questName is empty, simply visits all journal entries
-        virtual void visitJournalEntries (const std::string& questName, std::function <void (JournalEntry const &)> visitor) const = 0;
+        virtual void visitJournalEntries(
+            std::string_view questName, std::function<void(JournalEntry const&)> visitor) const = 0;
 
         /// provides the name of the topic specified by its id
-        virtual void visitTopicName (TopicId topicId, std::function <void (Utf8Span)> visitor) const = 0;
+        virtual void visitTopicName(TopicId topicId, std::function<void(Utf8Span)> visitor) const = 0;
 
         /// walks over the topics whose names start with the character
-        virtual void visitTopicNamesStartingWith (Utf8Stream::UnicodeChar character, std::function < void (const std::string&) > visitor) const = 0;
+        virtual void visitTopicNamesStartingWith(
+            Utf8Stream::UnicodeChar character, std::function<void(std::string_view)> visitor) const = 0;
 
         /// walks over the topic entries for the topic specified by its identifier
-        virtual void visitTopicEntries (TopicId topicId, std::function <void (TopicEntry const &)> visitor) const = 0;
+        virtual void visitTopicEntries(TopicId topicId, std::function<void(TopicEntry const&)> visitor) const = 0;
 
         // create an instance of the default journal view model implementation
-        static Ptr create ();
+        static Ptr create();
 
         virtual ~JournalViewModel() = default;
     };
