@@ -1,30 +1,41 @@
 #include "levelupdialog.hpp"
 
 #include <MyGUI_Button.h>
-#include <MyGUI_ImageBox.h>
 #include <MyGUI_EditBox.h>
+#include <MyGUI_ImageBox.h>
+#include <MyGUI_ScrollView.h>
+#include <MyGUI_TextBox.h>
+#include <MyGUI_UString.h>
 
 #include <components/fallback/fallback.hpp>
+#include <components/widgets/box.hpp>
 
-#include "../mwbase/windowmanager.hpp"
 #include "../mwbase/environment.hpp"
-#include "../mwbase/world.hpp"
 #include "../mwbase/soundmanager.hpp"
+#include "../mwbase/windowmanager.hpp"
+#include "../mwbase/world.hpp"
 
 #include "../mwworld/class.hpp"
+#include "../mwworld/esmstore.hpp"
 
+#include "../mwmechanics/actorutil.hpp"
 #include "../mwmechanics/creaturestats.hpp"
 #include "../mwmechanics/npcstats.hpp"
-#include "../mwmechanics/actorutil.hpp"
+
+#include "../mwsound/constants.hpp"
 
 #include "class.hpp"
 
+namespace
+{
+    constexpr unsigned int sMaxCoins = 3;
+    constexpr int sColumnOffsets[] = { 32, 218 };
+}
 namespace MWGui
 {
-    const unsigned int LevelupDialog::sMaxCoins = 3;
     LevelupDialog::LevelupDialog()
-        : WindowBase("openmw_levelup_dialog.layout"),
-          mCoinCount(sMaxCoins)
+        : WindowBase("openmw_levelup_dialog.layout")
+        , mCoinCount(sMaxCoins)
     {
         getWidget(mOkButton, "OkButton");
         getWidget(mClassImage, "ClassImage");
@@ -35,26 +46,47 @@ namespace MWGui
 
         mOkButton->eventMouseButtonClick += MyGUI::newDelegate(this, &LevelupDialog::onOkButtonClicked);
 
-        for (int i=1; i<9; ++i)
         {
-            MyGUI::TextBox* t;
-            getWidget(t, "AttribVal" + MyGUI::utility::toString(i));
-            mAttributeValues.push_back(t);
+            const auto& store = MWBase::Environment::get().getESMStore()->get<ESM::Attribute>();
+            const size_t perCol
+                = static_cast<size_t>(std::ceil(store.getSize() / static_cast<float>(std::size(sColumnOffsets))));
+            size_t i = 0;
+            for (const ESM::Attribute& attribute : store)
+            {
+                const int offset = sColumnOffsets[i / perCol];
+                const int row = static_cast<int>(i % perCol);
+                Widgets widgets;
+                widgets.mMultiplier = mAssignWidget->createWidget<MyGUI::TextBox>(
+                    "SandTextVCenter", { offset, 20 * row, 100, 20 }, MyGUI::Align::Default);
+                auto* hbox = mAssignWidget->createWidget<Gui::HBox>(
+                    {}, { offset + 20, 20 * row, 200, 20 }, MyGUI::Align::Default);
+                widgets.mButton = hbox->createWidget<Gui::AutoSizedButton>("SandTextButton", {}, MyGUI::Align::Default);
+                widgets.mButton->setUserData(attribute.mId);
+                widgets.mButton->eventMouseButtonClick += MyGUI::newDelegate(this, &LevelupDialog::onAttributeClicked);
+                widgets.mButton->setUserString("TextPadding", "0 0");
+                widgets.mButton->setUserString("ToolTipType", "Layout");
+                widgets.mButton->setUserString("ToolTipLayout", "AttributeToolTip");
+                widgets.mButton->setUserString("Caption_AttributeName", attribute.mName);
+                widgets.mButton->setUserString("Caption_AttributeDescription", attribute.mDescription);
+                widgets.mButton->setUserString("ImageTexture_AttributeImage", attribute.mIcon);
+                widgets.mButton->setCaption(attribute.mName);
+                widgets.mValue = hbox->createWidget<Gui::AutoSizedTextBox>("SandText", {}, MyGUI::Align::Default);
+                mAttributeWidgets.emplace(attribute.mId, widgets);
+                ++i;
+            }
 
-            MyGUI::Button* b;
-            getWidget(b, "Attrib" + MyGUI::utility::toString(i));
-            b->setUserData (i-1);
-            b->eventMouseButtonClick += MyGUI::newDelegate(this, &LevelupDialog::onAttributeClicked);
-            mAttributes.push_back(b);
-
-            getWidget(t, "AttribMultiplier" + MyGUI::utility::toString(i));
-            mAttributeMultipliers.push_back(t);
+            mAssignWidget->setVisibleVScroll(false);
+            mAssignWidget->setCanvasSize(MyGUI::IntSize(
+                mAssignWidget->getWidth(), std::max(mAssignWidget->getHeight(), static_cast<int>(20 * perCol))));
+            mAssignWidget->setVisibleVScroll(true);
+            mAssignWidget->setViewOffset(MyGUI::IntPoint());
         }
 
-        for (unsigned int i = 0; i < mCoinCount; ++i)
+        for (unsigned int i = 0; i < sMaxCoins; ++i)
         {
-            MyGUI::ImageBox* image = mCoinBox->createWidget<MyGUI::ImageBox>("ImageBox", MyGUI::IntCoord(0,0,16,16), MyGUI::Align::Default);
-            image->setImageTexture ("icons\\tx_goldicon.dds");
+            MyGUI::ImageBox* image = mCoinBox->createWidget<MyGUI::ImageBox>(
+                "ImageBox", MyGUI::IntCoord(0, 0, 16, 16), MyGUI::Align::Default);
+            image->setImageTexture("icons\\tx_goldicon.dds");
             mCoins.push_back(image);
         }
 
@@ -63,31 +95,30 @@ namespace MWGui
 
     void LevelupDialog::setAttributeValues()
     {
-        MWWorld::Ptr player = MWBase::Environment::get().getWorld ()->getPlayerPtr();
+        MWWorld::Ptr player = MWBase::Environment::get().getWorld()->getPlayerPtr();
         MWMechanics::CreatureStats& creatureStats = player.getClass().getCreatureStats(player);
-        MWMechanics::NpcStats& pcStats = player.getClass().getNpcStats (player);
+        MWMechanics::NpcStats& pcStats = player.getClass().getNpcStats(player);
 
-        for (int i = 0; i < 8; ++i)
+        for (const ESM::Attribute& attribute : MWBase::Environment::get().getESMStore()->get<ESM::Attribute>())
         {
-            int val = creatureStats.getAttribute(i).getBase();
-            if (std::find(mSpentAttributes.begin(), mSpentAttributes.end(), i) != mSpentAttributes.end())
+            int val = creatureStats.getAttribute(attribute.mId).getBase();
+            if (std::find(mSpentAttributes.begin(), mSpentAttributes.end(), attribute.mId) != mSpentAttributes.end())
             {
-                val += pcStats.getLevelupAttributeMultiplier(i);
+                val += pcStats.getLevelupAttributeMultiplier(attribute.mId);
             }
 
             if (val >= 100)
                 val = 100;
 
-            mAttributeValues[i]->setCaption(MyGUI::utility::toString(val));
+            mAttributeWidgets[attribute.mId].mValue->setCaption(MyGUI::utility::toString(val));
         }
     }
 
-
     void LevelupDialog::resetCoins()
     {
-        const int coinSpacing = 33;
-        int curX = mCoinBox->getWidth()/2 - (coinSpacing*(mCoinCount - 1) + 16*mCoinCount)/2;
-        for (unsigned int i=0; i<sMaxCoins; ++i)
+        constexpr int coinSpacing = 33;
+        int curX = mCoinBox->getWidth() / 2 - (coinSpacing * (mCoinCount - 1) + 16 * mCoinCount) / 2;
+        for (unsigned int i = 0; i < sMaxCoins; ++i)
         {
             MyGUI::ImageBox* image = mCoins[i];
             image->detachFromWidget();
@@ -95,8 +126,8 @@ namespace MWGui
             if (i < mCoinCount)
             {
                 mCoins[i]->setVisible(true);
-                image->setCoord(MyGUI::IntCoord(curX,0,16,16));
-                curX += 16+coinSpacing;
+                image->setCoord(MyGUI::IntCoord(curX, 0, 16, 16));
+                curX += 16 + coinSpacing;
             }
             else
                 mCoins[i]->setVisible(false);
@@ -106,18 +137,21 @@ namespace MWGui
     void LevelupDialog::assignCoins()
     {
         resetCoins();
-        for (unsigned int i=0; i<mSpentAttributes.size(); ++i)
+        for (size_t i = 0; i < mSpentAttributes.size(); ++i)
         {
             MyGUI::ImageBox* image = mCoins[i];
             image->detachFromWidget();
             image->attachToWidget(mAssignWidget);
 
-            int attribute = mSpentAttributes[i];
+            const auto& attribute = mSpentAttributes[i];
+            const auto& widgets = mAttributeWidgets[attribute];
 
-            int xdiff = mAttributeMultipliers[attribute]->getCaption() == "" ? 0 : 20;
+            const int xdiff = widgets.mMultiplier->getCaption().empty() ? 0 : 20;
+            const auto* hbox = widgets.mButton->getParent();
 
-            MyGUI::IntPoint pos = mAttributes[attribute]->getAbsolutePosition() - mAssignWidget->getAbsolutePosition() - MyGUI::IntPoint(22+xdiff,0);
-            pos.top += (mAttributes[attribute]->getHeight() - image->getHeight())/2;
+            MyGUI::IntPoint pos = hbox->getPosition();
+            pos.left -= 22 + xdiff;
+            pos.top += (hbox->getHeight() - image->getHeight()) / 2;
             image->setPosition(pos);
         }
 
@@ -126,46 +160,51 @@ namespace MWGui
 
     void LevelupDialog::onOpen()
     {
-        MWBase::World *world = MWBase::Environment::get().getWorld();
+        MWBase::World* world = MWBase::Environment::get().getWorld();
         MWWorld::Ptr player = world->getPlayerPtr();
-        MWMechanics::CreatureStats& creatureStats = player.getClass().getCreatureStats(player);
-        MWMechanics::NpcStats& pcStats = player.getClass().getNpcStats(player);
+        const MWMechanics::CreatureStats& creatureStats = player.getClass().getCreatureStats(player);
+        const MWMechanics::NpcStats& pcStats = player.getClass().getNpcStats(player);
 
-        setClassImage(mClassImage, getLevelupClassImage(pcStats.getSkillIncreasesForSpecialization(0),
-                                                        pcStats.getSkillIncreasesForSpecialization(1),
-                                                        pcStats.getSkillIncreasesForSpecialization(2)));
+        setClassImage(mClassImage,
+            ESM::RefId::stringRefId(
+                getLevelupClassImage(pcStats.getSkillIncreasesForSpecialization(ESM::Class::Specialization::Combat),
+                    pcStats.getSkillIncreasesForSpecialization(ESM::Class::Specialization::Magic),
+                    pcStats.getSkillIncreasesForSpecialization(ESM::Class::Specialization::Stealth))));
 
-        int level = creatureStats.getLevel ()+1;
+        int level = creatureStats.getLevel() + 1;
         mLevelText->setCaptionWithReplacing("#{sLevelUpMenu1} " + MyGUI::utility::toString(level));
 
-        std::string levelupdescription;
-        levelupdescription = Fallback::Map::getString("Level_Up_Level"+MyGUI::utility::toString(level));
+        std::string_view levelupdescription;
+        levelupdescription = Fallback::Map::getString("Level_Up_Level" + MyGUI::utility::toString(level));
 
-        if (levelupdescription == "")
+        if (levelupdescription.empty())
             levelupdescription = Fallback::Map::getString("Level_Up_Default");
 
-        mLevelDescription->setCaption (levelupdescription);
+        mLevelDescription->setCaption(MyGUI::UString(levelupdescription));
 
         unsigned int availableAttributes = 0;
-        for (int i = 0; i < 8; ++i)
+        for (const ESM::Attribute& attribute : MWBase::Environment::get().getESMStore()->get<ESM::Attribute>())
         {
-            MyGUI::TextBox* text = mAttributeMultipliers[i];
-            if (pcStats.getAttribute(i).getBase() < 100)
+            const auto& widgets = mAttributeWidgets[attribute.mId];
+            if (pcStats.getAttribute(attribute.mId).getBase() < 100)
             {
-                mAttributes[i]->setEnabled(true);
-                mAttributeValues[i]->setEnabled(true);
+                widgets.mButton->setEnabled(true);
+                widgets.mValue->setEnabled(true);
                 availableAttributes++;
 
-                float mult = pcStats.getLevelupAttributeMultiplier (i);
-                mult = std::min(mult, 100-pcStats.getAttribute(i).getBase());
-                text->setCaption(mult <= 1 ? "" : "x" + MyGUI::utility::toString(mult));
+                float mult = pcStats.getLevelupAttributeMultiplier(attribute.mId);
+                mult = std::min(mult, 100 - pcStats.getAttribute(attribute.mId).getBase());
+                if (mult <= 1)
+                    widgets.mMultiplier->setCaption({});
+                else
+                    widgets.mMultiplier->setCaption("x" + MyGUI::utility::toString(mult));
             }
             else
             {
-                mAttributes[i]->setEnabled(false);
-                mAttributeValues[i]->setEnabled(false);
+                widgets.mButton->setEnabled(false);
+                widgets.mValue->setEnabled(false);
 
-                text->setCaption("");
+                widgets.mMultiplier->setCaption({});
             }
         }
 
@@ -179,13 +218,13 @@ namespace MWGui
         center();
 
         // Play LevelUp Music
-        MWBase::Environment::get().getSoundManager()->streamMusic("Special/MW_Triumph.mp3");
+        MWBase::Environment::get().getSoundManager()->streamMusic(MWSound::triumphMusic, MWSound::MusicType::Normal);
     }
 
     void LevelupDialog::onOkButtonClicked(MyGUI::Widget* sender)
     {
         MWWorld::Ptr player = MWMechanics::getPlayer();
-        MWMechanics::NpcStats& pcStats = player.getClass().getNpcStats (player);
+        MWMechanics::NpcStats& pcStats = player.getClass().getNpcStats(player);
 
         if (mSpentAttributes.size() < mCoinCount)
             MWBase::Environment::get().getWindowManager()->messageBox("#{sNotifyMessage36}");
@@ -206,14 +245,13 @@ namespace MWGui
 
             MWBase::Environment::get().getWindowManager()->removeGuiMode(GM_Levelup);
         }
-
     }
 
-    void LevelupDialog::onAttributeClicked(MyGUI::Widget *sender)
+    void LevelupDialog::onAttributeClicked(MyGUI::Widget* sender)
     {
-        int attribute = *sender->getUserData<int>();
+        auto attribute = *sender->getUserData<ESM::Attribute::AttributeID>();
 
-        std::vector<int>::iterator found = std::find(mSpentAttributes.begin(), mSpentAttributes.end(), attribute);
+        auto found = std::find(mSpentAttributes.begin(), mSpentAttributes.end(), attribute);
         if (found != mSpentAttributes.end())
             mSpentAttributes.erase(found);
         else
@@ -226,9 +264,10 @@ namespace MWGui
         assignCoins();
     }
 
-    std::string LevelupDialog::getLevelupClassImage(const int combatIncreases, const int magicIncreases, const int stealthIncreases)
+    std::string_view LevelupDialog::getLevelupClassImage(
+        const int combatIncreases, const int magicIncreases, const int stealthIncreases)
     {
-        std::string ret = "acrobat";
+        std::string_view ret = "acrobat";
 
         int total = combatIncreases + magicIncreases + stealthIncreases;
         if (total == 0)
