@@ -1,6 +1,8 @@
 #ifndef MWGUI_LEVELUPDIALOG_H
 #define MWGUI_LEVELUPDIALOG_H
 
+#include <components/esm/attr.hpp>
+
 #include "windowbase.hpp"
 
 namespace MWGui
@@ -13,24 +15,29 @@ namespace MWGui
 
         void onOpen() override;
 
+        std::string_view getWindowIdForLua() const override { return "LevelUpDialog"; }
+
     private:
+        struct Widgets
+        {
+            MyGUI::Button* mButton;
+            MyGUI::TextBox* mValue;
+            MyGUI::TextBox* mMultiplier;
+        };
         MyGUI::Button* mOkButton;
         MyGUI::ImageBox* mClassImage;
         MyGUI::TextBox* mLevelText;
         MyGUI::EditBox* mLevelDescription;
 
         MyGUI::Widget* mCoinBox;
-        MyGUI::Widget* mAssignWidget;
+        MyGUI::ScrollView* mAssignWidget;
 
-        std::vector<MyGUI::Button*> mAttributes;
-        std::vector<MyGUI::TextBox*> mAttributeValues;
-        std::vector<MyGUI::TextBox*> mAttributeMultipliers;
+        std::map<ESM::Attribute::AttributeID, Widgets> mAttributeWidgets;
         std::vector<MyGUI::ImageBox*> mCoins;
 
-        std::vector<int> mSpentAttributes;
+        std::vector<ESM::Attribute::AttributeID> mSpentAttributes;
 
         unsigned int mCoinCount;
-        static const unsigned int sMaxCoins;
 
         void onOkButtonClicked(MyGUI::Widget* sender);
         void onAttributeClicked(MyGUI::Widget* sender);
@@ -40,7 +47,8 @@ namespace MWGui
 
         void setAttributeValues();
 
-        std::string getLevelupClassImage(const int combatIncreases, const int magicIncreases, const int stealthIncreases);
+        std::string_view getLevelupClassImage(
+            const int combatIncreases, const int magicIncreases, const int stealthIncreases);
     };
 
 }
