@@ -1,64 +1,131 @@
 #include "mainmenu.hpp"
 
-#include <MyGUI_TextBox.h>
 #include <MyGUI_Gui.h>
 #include <MyGUI_RenderManager.h>
+#include <MyGUI_TextBox.h>
 
-#include <components/widgets/imagebutton.hpp>
-#include <components/settings/settings.hpp>
+#include <components/misc/frameratelimiter.hpp>
+#include <components/settings/values.hpp>
 #include <components/vfs/manager.hpp>
+#include <components/vfs/pathutil.hpp>
+#include <components/widgets/imagebutton.hpp>
 
 #include "../mwbase/environment.hpp"
+#include "../mwbase/statemanager.hpp"
 #include "../mwbase/windowmanager.hpp"
 #include "../mwbase/world.hpp"
-#include "../mwbase/statemanager.hpp"
 
-#include "savegamedialog.hpp"
-#include "confirmationdialog.hpp"
+#include "../mwworld/globals.hpp"
+
 #include "backgroundimage.hpp"
+#include "confirmationdialog.hpp"
+#include "savegamedialog.hpp"
+#include "settingswindow.hpp"
 #include "videowidget.hpp"
 
 namespace MWGui
 {
+    void MenuVideo::run()
+    {
+        Misc::FrameRateLimiter frameRateLimiter
+            = Misc::makeFrameRateLimiter(MWBase::Environment::get().getFrameRateLimit());
+        const MWBase::WindowManager& windowManager = *MWBase::Environment::get().getWindowManager();
+        bool paused = false;
+        while (mRunning)
+        {
+            if (windowManager.isWindowVisible())
+            {
+                if (paused)
+                {
+                    mVideo->resume();
+                    paused = false;
+                }
+                // If finished playing, start again
+                if (!mVideo->update())
+                    mVideo->playVideo("video\\menu_background.bik");
+            }
+            else if (!paused)
+            {
+                paused = true;
+                mVideo->pause();
+            }
+            frameRateLimiter.limit();
+        }
+    }
+
+    MenuVideo::MenuVideo(const VFS::Manager* vfs)
+        : mRunning(true)
+    {
+        // Use black background to correct aspect ratio
+        mVideoBackground = MyGUI::Gui::getInstance().createWidgetReal<MyGUI::ImageBox>(
+            "ImageBox", 0, 0, 1, 1, MyGUI::Align::Default, "MainMenuBackground");
+        mVideoBackground->setImageTexture("black");
+
+        mVideo = mVideoBackground->createWidget<VideoWidget>(
+            "ImageBox", 0, 0, 1, 1, MyGUI::Align::Stretch, "MainMenuBackground");
+        mVideo->setVFS(vfs);
+
+        mVideo->playVideo("video\\menu_background.bik");
+        mThread = std::thread([this] { run(); });
+    }
+
+    void MenuVideo::resize(int screenWidth, int screenHeight)
+    {
+        const bool stretch = Settings::gui().mStretchMenuBackground;
+        mVideoBackground->setSize(screenWidth, screenHeight);
+        mVideo->autoResize(stretch);
+        mVideo->setVisible(true);
+    }
+
+    MenuVideo::~MenuVideo()
+    {
+        mRunning = false;
+        mThread.join();
+        try
+        {
+            MyGUI::Gui::getInstance().destroyWidget(mVideoBackground);
+        }
+        catch (const MyGUI::Exception& e)
+        {
+            Log(Debug::Error) << "Error in the destructor: " << e.what();
+        }
+    }
 
     MainMenu::MainMenu(int w, int h, const VFS::Manager* vfs, const std::string& versionDescription)
         : WindowBase("openmw_mainmenu.layout")
-        , mWidth (w), mHeight (h)
-        , mVFS(vfs), mButtonBox(nullptr)
+        , mWidth(w)
+        , mHeight(h)
+        , mVFS(vfs)
+        , mButtonBox(nullptr)
         , mBackground(nullptr)
-        , mVideoBackground(nullptr)
-        , mVideo(nullptr)
-        , mSaveGameDialog(nullptr)
     {
         getWidget(mVersionText, "VersionText");
         mVersionText->setCaption(versionDescription);
 
-        mHasAnimatedMenu = mVFS->exists("video/menu_background.bik");
+        constexpr VFS::Path::NormalizedView menuBackgroundVideo("video/menu_background.bik");
+
+        mHasAnimatedMenu = mVFS->exists(menuBackgroundVideo);
 
         updateMenu();
     }
 
-    MainMenu::~MainMenu()
-    {
-        delete mSaveGameDialog;
-    }
-
     void MainMenu::onResChange(int w, int h)
     {
         mWidth = w;
         mHeight = h;
 
         updateMenu();
+        if (mVideo)
+            mVideo->resize(w, h);
     }
 
-    void MainMenu::setVisible (bool visible)
+    void MainMenu::setVisible(bool visible)
     {
         if (visible)
             updateMenu();
 
-        bool isMainMenu =
-                MWBase::Environment::get().getWindowManager()->containsMode(MWGui::GM_MainMenu) &&
-                MWBase::Environment::get().getStateManager()->getState() == MWBase::StateManager::State_NoGame;
+        bool isMainMenu = MWBase::Environment::get().getWindowManager()->containsMode(MWGui::GM_MainMenu)
+            && MWBase::Environment::get().getStateManager()->getState() == MWBase::StateManager::State_NoGame;
 
         showBackground(isMainMenu);
 
@@ -75,12 +142,12 @@ namespace MWGui
                 MWBase::Environment::get().getWindowManager()->setKeyFocusWidget(mButtons["return"]);
         }
 
-        Layout::setVisible (visible);
+        Layout::setVisible(visible);
     }
 
     void MainMenu::onNewGameConfirmed()
     {
-        MWBase::Environment::get().getWindowManager()->removeGuiMode (MWGui::GM_MainMenu);
+        MWBase::Environment::get().getWindowManager()->removeGuiMode(MWGui::GM_MainMenu);
         MWBase::Environment::get().getStateManager()->newGame();
     }
 
@@ -89,18 +156,16 @@ namespace MWGui
         MWBase::Environment::get().getStateManager()->requestQuit();
     }
 
-    void MainMenu::onButtonClicked(MyGUI::Widget *sender)
+    void MainMenu::onButtonClicked(MyGUI::Widget* sender)
     {
-        MWBase::WindowManager *winMgr = MWBase::Environment::get().getWindowManager();
+        MWBase::WindowManager* winMgr = MWBase::Environment::get().getWindowManager();
 
-        std::string name = *sender->getUserData<std::string>();
-        winMgr->playSound("Menu Click");
+        const std::string& name = *sender->getUserData<std::string>();
+        winMgr->playSound(ESM::RefId::stringRefId("Menu Click"));
         if (name == "return")
         {
-            winMgr->removeGuiMode (GM_MainMenu);
+            winMgr->removeGuiMode(GM_MainMenu);
         }
-        else if (name == "options")
-            winMgr->pushGuiMode (GM_Settings);
         else if (name == "credits")
             winMgr->playVideo("mw_credits.bik", true);
         else if (name == "exitgame")
@@ -110,7 +175,7 @@ namespace MWGui
             else
             {
                 ConfirmationDialog* dialog = winMgr->getConfirmationDialog();
-                dialog->askForConfirmation("#{sMessage2}");
+                dialog->askForConfirmation("#{OMWEngine:QuitGameConfirmation}");
                 dialog->eventOkClicked.clear();
                 dialog->eventOkClicked += MyGUI::newDelegate(this, &MainMenu::onExitConfirmed);
                 dialog->eventCancelClicked.clear();
@@ -123,32 +188,31 @@ namespace MWGui
             else
             {
                 ConfirmationDialog* dialog = winMgr->getConfirmationDialog();
-                dialog->askForConfirmation("#{sNotifyMessage54}");
+                dialog->askForConfirmation("#{OMWEngine:NewGameConfirmation}");
                 dialog->eventOkClicked.clear();
                 dialog->eventOkClicked += MyGUI::newDelegate(this, &MainMenu::onNewGameConfirmed);
                 dialog->eventCancelClicked.clear();
             }
         }
-
-        else
+        else if (name == "loadgame" || name == "savegame")
         {
             if (!mSaveGameDialog)
-                mSaveGameDialog = new SaveGameDialog();
-            if (name == "loadgame")
-                mSaveGameDialog->setLoadOrSave(true);
-            else if (name == "savegame")
-                mSaveGameDialog->setLoadOrSave(false);
+                mSaveGameDialog = std::make_unique<SaveGameDialog>();
+            mSaveGameDialog->setLoadOrSave(name == "loadgame");
             mSaveGameDialog->setVisible(true);
         }
+
+        if (winMgr->isSettingsWindowVisible() || name == "options")
+        {
+            winMgr->toggleSettingsWindow();
+        }
     }
 
     void MainMenu::showBackground(bool show)
     {
         if (mVideo && !show)
         {
-            MyGUI::Gui::getInstance().destroyWidget(mVideoBackground);
-            mVideoBackground = nullptr;
-            mVideo = nullptr;
+            mVideo.reset();
         }
         if (mBackground && !show)
         {
@@ -159,68 +223,48 @@ namespace MWGui
         if (!show)
             return;
 
-        bool stretch = Settings::Manager::getBool("stretch menu background", "GUI");
+        const bool stretch = Settings::gui().mStretchMenuBackground;
 
         if (mHasAnimatedMenu)
         {
             if (!mVideo)
-            {
-                // Use black background to correct aspect ratio
-                mVideoBackground = MyGUI::Gui::getInstance().createWidgetReal<MyGUI::ImageBox>("ImageBox", 0,0,1,1,
-                    MyGUI::Align::Default, "Menu");
-                mVideoBackground->setImageTexture("black");
+                mVideo.emplace(mVFS);
 
-                mVideo = mVideoBackground->createWidget<VideoWidget>("ImageBox", 0,0,1,1,
-                    MyGUI::Align::Stretch, "Menu");
-                mVideo->setVFS(mVFS);
-
-                mVideo->playVideo("video\\menu_background.bik");
-            }
-
-            MyGUI::IntSize viewSize = MyGUI::RenderManager::getInstance().getViewSize();
+            const auto& viewSize = MyGUI::RenderManager::getInstance().getViewSize();
             int screenWidth = viewSize.width;
             int screenHeight = viewSize.height;
-            mVideoBackground->setSize(screenWidth, screenHeight);
-
-            mVideo->autoResize(stretch);
-
-            mVideo->setVisible(true);
+            mVideo->resize(screenWidth, screenHeight);
         }
         else
         {
             if (!mBackground)
             {
-                mBackground = MyGUI::Gui::getInstance().createWidgetReal<BackgroundImage>("ImageBox", 0,0,1,1,
-                    MyGUI::Align::Stretch, "Menu");
+                mBackground = MyGUI::Gui::getInstance().createWidgetReal<BackgroundImage>(
+                    "ImageBox", 0, 0, 1, 1, MyGUI::Align::Stretch, "MainMenuBackground");
                 mBackground->setBackgroundImage("textures\\menu_morrowind.dds", true, stretch);
             }
             mBackground->setVisible(true);
         }
     }
 
-    void MainMenu::onFrame(float dt)
-    {
-        if (mVideo)
-        {
-            if (!mVideo->update())
-            {
-                // If finished playing, start again
-                mVideo->playVideo("video\\menu_background.bik");
-            }
-        }
-    }
-
     bool MainMenu::exit()
     {
+        if (MWBase::Environment::get().getWindowManager()->isSettingsWindowVisible())
+        {
+            MWBase::Environment::get().getWindowManager()->toggleSettingsWindow();
+            return false;
+        }
+
         return MWBase::Environment::get().getStateManager()->getState() == MWBase::StateManager::State_Running;
     }
 
     void MainMenu::updateMenu()
     {
-        setCoord(0,0, mWidth, mHeight);
+        setCoord(0, 0, mWidth, mHeight);
 
         if (!mButtonBox)
-            mButtonBox = mMainWidget->createWidget<MyGUI::Widget>("", MyGUI::IntCoord(0, 0, 0, 0), MyGUI::Align::Default);
+            mButtonBox
+                = mMainWidget->createWidget<MyGUI::Widget>({}, MyGUI::IntCoord(0, 0, 0, 0), MyGUI::Align::Default);
 
         int curH = 0;
 
@@ -230,47 +274,46 @@ namespace MWGui
 
         std::vector<std::string> buttons;
 
-        if (state==MWBase::StateManager::State_Running)
+        if (state == MWBase::StateManager::State_Running)
             buttons.emplace_back("return");
 
         buttons.emplace_back("newgame");
 
-        if (state==MWBase::StateManager::State_Running &&
-            MWBase::Environment::get().getWorld()->getGlobalInt ("chargenstate")==-1 &&
-                MWBase::Environment::get().getWindowManager()->isSavingAllowed())
+        if (state == MWBase::StateManager::State_Running
+            && MWBase::Environment::get().getWorld()->getGlobalInt(MWWorld::Globals::sCharGenState) == -1
+            && MWBase::Environment::get().getWindowManager()->isSavingAllowed())
             buttons.emplace_back("savegame");
 
-        if (MWBase::Environment::get().getStateManager()->characterBegin()!=
-            MWBase::Environment::get().getStateManager()->characterEnd())
+        if (MWBase::Environment::get().getStateManager()->characterBegin()
+            != MWBase::Environment::get().getStateManager()->characterEnd())
             buttons.emplace_back("loadgame");
 
         buttons.emplace_back("options");
 
-        if (state==MWBase::StateManager::State_NoGame)
+        if (state == MWBase::StateManager::State_NoGame)
             buttons.emplace_back("credits");
 
         buttons.emplace_back("exitgame");
 
         // Create new buttons if needed
-        std::vector<std::string> allButtons { "return", "newgame", "savegame", "loadgame", "options", "credits", "exitgame"};
-        for (std::string& buttonId : allButtons)
+        for (std::string_view id : { "return", "newgame", "savegame", "loadgame", "options", "credits", "exitgame" })
         {
-            if (mButtons.find(buttonId) == mButtons.end())
+            if (mButtons.find(id) == mButtons.end())
             {
-                Gui::ImageButton* button = mButtonBox->createWidget<Gui::ImageButton>
-                        ("ImageBox", MyGUI::IntCoord(0, curH, 0, 0), MyGUI::Align::Default);
+                Gui::ImageButton* button = mButtonBox->createWidget<Gui::ImageButton>(
+                    "ImageBox", MyGUI::IntCoord(0, curH, 0, 0), MyGUI::Align::Default);
+                const std::string& buttonId = mButtons.emplace(id, button).first->first;
                 button->setProperty("ImageHighlighted", "textures\\menu_" + buttonId + "_over.dds");
                 button->setProperty("ImageNormal", "textures\\menu_" + buttonId + ".dds");
                 button->setProperty("ImagePushed", "textures\\menu_" + buttonId + "_pressed.dds");
                 button->eventMouseButtonClick += MyGUI::newDelegate(this, &MainMenu::onButtonClicked);
-                button->setUserData(std::string(buttonId));
-                mButtons[buttonId] = button;
+                button->setUserData(buttonId);
             }
         }
 
         // Start by hiding all buttons
         int maxwidth = 0;
-        for (auto& buttonPair : mButtons)
+        for (const auto& buttonPair : mButtons)
         {
             buttonPair.second->setVisible(false);
             MyGUI::IntSize requested = buttonPair.second->getRequestedSize();
@@ -279,10 +322,11 @@ namespace MWGui
         }
 
         // Now show and position the ones we want
-        for (std::string& buttonId : buttons)
+        for (const std::string& buttonId : buttons)
         {
-            assert(mButtons.find(buttonId) != mButtons.end());
-            Gui::ImageButton* button = mButtons[buttonId];
+            auto it = mButtons.find(buttonId);
+            assert(it != mButtons.end());
+            Gui::ImageButton* button = it->second;
             button->setVisible(true);
 
             // By default, assume that all menu buttons textures should have 64 height.
@@ -294,19 +338,19 @@ namespace MWGui
             // Trim off some of the excessive padding
             // TODO: perhaps do this within ImageButton?
             int height = requested.height;
-            button->setImageTile(MyGUI::IntSize(requested.width, requested.height-16*scale));
-            button->setCoord((maxwidth-requested.width/scale) / 2, curH, requested.width/scale, height/scale-16);
-            curH += height/scale-16;
+            button->setImageTile(MyGUI::IntSize(requested.width, requested.height - 16 * scale));
+            button->setCoord(
+                (maxwidth - requested.width / scale) / 2, curH, requested.width / scale, height / scale - 16);
+            curH += height / scale - 16;
         }
 
         if (state == MWBase::StateManager::State_NoGame)
         {
             // Align with the background image
-            int bottomPadding=24;
-            mButtonBox->setCoord (mWidth/2 - maxwidth/2, mHeight - curH - bottomPadding, maxwidth, curH);
+            int bottomPadding = 24;
+            mButtonBox->setCoord(mWidth / 2 - maxwidth / 2, mHeight - curH - bottomPadding, maxwidth, curH);
         }
         else
-            mButtonBox->setCoord (mWidth/2 - maxwidth/2, mHeight/2 - curH/2, maxwidth, curH);
-
+            mButtonBox->setCoord(mWidth / 2 - maxwidth / 2, mHeight / 2 - curH / 2, maxwidth, curH);
     }
 }
