@@ -1,35 +1,34 @@
 #include "messagebox.hpp"
 
-#include <MyGUI_LanguageManager.h>
-#include <MyGUI_EditBox.h>
-#include <MyGUI_RenderManager.h>
 #include <MyGUI_Button.h>
+#include <MyGUI_EditBox.h>
+#include <MyGUI_LanguageManager.h>
+#include <MyGUI_RenderManager.h>
+#include <MyGUI_UString.h>
 
 #include <components/debug/debuglog.hpp>
-#include <components/misc/stringops.hpp>
+#include <components/misc/strings/algorithm.hpp>
 
 #include "../mwbase/environment.hpp"
-#include "../mwbase/soundmanager.hpp"
 #include "../mwbase/inputmanager.hpp"
 #include "../mwbase/windowmanager.hpp"
 
 namespace MWGui
 {
 
-    MessageBoxManager::MessageBoxManager (float timePerChar)
+    MessageBoxManager::MessageBoxManager(float timePerChar)
     {
-        mInterMessageBoxe = nullptr;
         mStaticMessageBox = nullptr;
         mLastButtonPressed = -1;
         mMessageBoxSpeed = timePerChar;
     }
 
-    MessageBoxManager::~MessageBoxManager ()
+    MessageBoxManager::~MessageBoxManager()
     {
         MessageBoxManager::clear();
     }
 
-    int MessageBoxManager::getMessagesCount()
+    std::size_t MessageBoxManager::getMessagesCount()
     {
         return mMessageBoxes.size();
     }
@@ -39,31 +38,36 @@ namespace MWGui
         if (mInterMessageBoxe)
         {
             mInterMessageBoxe->setVisible(false);
-
-            delete mInterMessageBoxe;
-            mInterMessageBoxe = nullptr;
+            mInterMessageBoxe.reset();
         }
 
-        for (MessageBox* messageBox : mMessageBoxes)
-        {
-            if (messageBox == mStaticMessageBox)
-                mStaticMessageBox = nullptr;
-            delete messageBox;
-        }
         mMessageBoxes.clear();
+        mStaticMessageBox = nullptr;
 
         mLastButtonPressed = -1;
     }
 
-    void MessageBoxManager::onFrame (float frameDuration)
+    void MessageBoxManager::resetInteractiveMessageBox()
     {
-        std::vector<MessageBox*>::iterator it;
-        for(it = mMessageBoxes.begin(); it != mMessageBoxes.end();)
+        if (mInterMessageBoxe)
+        {
+            mInterMessageBoxe->setVisible(false);
+            mInterMessageBoxe.reset();
+        }
+    }
+
+    void MessageBoxManager::setLastButtonPressed(int index)
+    {
+        mLastButtonPressed = index;
+    }
+
+    void MessageBoxManager::onFrame(float frameDuration)
+    {
+        for (auto it = mMessageBoxes.begin(); it != mMessageBoxes.end();)
         {
             (*it)->mCurrentTime += frameDuration;
-            if((*it)->mCurrentTime >= (*it)->mMaxTime && *it != mStaticMessageBox)
+            if ((*it)->mCurrentTime >= (*it)->mMaxTime && it->get() != mStaticMessageBox)
             {
-                delete *it;
                 it = mMessageBoxes.erase(it);
             }
             else
@@ -71,86 +75,84 @@ namespace MWGui
         }
 
         float height = 0;
-        it = mMessageBoxes.begin();
-        while(it != mMessageBoxes.end())
+        auto it = mMessageBoxes.begin();
+        while (it != mMessageBoxes.end())
         {
             (*it)->update(static_cast<int>(height));
             height += (*it)->getHeight();
             ++it;
         }
 
-        if(mInterMessageBoxe != nullptr && mInterMessageBoxe->mMarkedToDelete) {
+        if (mInterMessageBoxe != nullptr && mInterMessageBoxe->mMarkedToDelete)
+        {
             mLastButtonPressed = mInterMessageBoxe->readPressedButton();
             mInterMessageBoxe->setVisible(false);
-            delete mInterMessageBoxe;
-            mInterMessageBoxe = nullptr;
+            mInterMessageBoxe.reset();
             MWBase::Environment::get().getInputManager()->changeInputMode(
-                        MWBase::Environment::get().getWindowManager()->isGuiMode());
+                MWBase::Environment::get().getWindowManager()->isGuiMode());
         }
     }
 
-    void MessageBoxManager::createMessageBox (const std::string& message, bool stat)
+    void MessageBoxManager::createMessageBox(std::string_view message, bool stat)
     {
-        MessageBox *box = new MessageBox(*this, message);
+        auto box = std::make_unique<MessageBox>(*this, message);
         box->mCurrentTime = 0;
-        std::string realMessage = MyGUI::LanguageManager::getInstance().replaceTags(message);
-        box->mMaxTime = realMessage.length()*mMessageBoxSpeed;
+        auto realMessage = MyGUI::LanguageManager::getInstance().replaceTags({ message.data(), message.size() });
+        box->mMaxTime = realMessage.length() * mMessageBoxSpeed;
 
-        if(stat)
-            mStaticMessageBox = box;
+        if (stat)
+            mStaticMessageBox = box.get();
 
         box->setVisible(mVisible);
 
-        mMessageBoxes.push_back(box);
+        mMessageBoxes.push_back(std::move(box));
 
-        if(mMessageBoxes.size() > 3) {
-            delete *mMessageBoxes.begin();
+        if (mMessageBoxes.size() > 3)
+        {
             mMessageBoxes.erase(mMessageBoxes.begin());
         }
 
         int height = 0;
-        for (MessageBox* messageBox : mMessageBoxes)
+        for (const auto& messageBox : mMessageBoxes)
         {
             messageBox->update(height);
             height += messageBox->getHeight();
         }
     }
 
-    void MessageBoxManager::removeStaticMessageBox ()
+    void MessageBoxManager::removeStaticMessageBox()
     {
         removeMessageBox(mStaticMessageBox);
         mStaticMessageBox = nullptr;
     }
 
-    bool MessageBoxManager::createInteractiveMessageBox (const std::string& message, const std::vector<std::string>& buttons)
+    bool MessageBoxManager::createInteractiveMessageBox(
+        std::string_view message, const std::vector<std::string>& buttons, bool immediate, int defaultFocus)
     {
         if (mInterMessageBoxe != nullptr)
         {
             Log(Debug::Warning) << "Warning: replacing an interactive message box that was not answered yet";
             mInterMessageBoxe->setVisible(false);
-            delete mInterMessageBoxe;
-            mInterMessageBoxe = nullptr;
         }
 
-        mInterMessageBoxe = new InteractiveMessageBox(*this, message, buttons);
+        mInterMessageBoxe
+            = std::make_unique<InteractiveMessageBox>(*this, std::string{ message }, buttons, immediate, defaultFocus);
         mLastButtonPressed = -1;
 
         return true;
     }
 
-    bool MessageBoxManager::isInteractiveMessageBox ()
+    bool MessageBoxManager::isInteractiveMessageBox()
     {
         return mInterMessageBoxe != nullptr;
     }
 
-    bool MessageBoxManager::removeMessageBox (MessageBox *msgbox)
+    bool MessageBoxManager::removeMessageBox(MessageBox* msgbox)
     {
-        std::vector<MessageBox*>::iterator it;
-        for(it = mMessageBoxes.begin(); it != mMessageBoxes.end(); ++it)
+        for (auto it = mMessageBoxes.begin(); it != mMessageBoxes.end(); ++it)
         {
-            if((*it) == msgbox)
+            if (it->get() == msgbox)
             {
-                delete (*it);
                 mMessageBoxes.erase(it);
                 return true;
             }
@@ -158,12 +160,12 @@ namespace MWGui
         return false;
     }
 
-    const std::vector<MessageBox*> MessageBoxManager::getActiveMessageBoxes()
+    const std::vector<std::unique_ptr<MessageBox>>& MessageBoxManager::getActiveMessageBoxes() const
     {
         return mMessageBoxes;
     }
 
-    int MessageBoxManager::readPressedButton (bool reset)
+    int MessageBoxManager::readPressedButton(bool reset)
     {
         int pressed = mLastButtonPressed;
         if (reset)
@@ -174,16 +176,16 @@ namespace MWGui
     void MessageBoxManager::setVisible(bool value)
     {
         mVisible = value;
-        for (MessageBox* messageBox : mMessageBoxes)
+        for (const auto& messageBox : mMessageBoxes)
             messageBox->setVisible(value);
     }
 
-    MessageBox::MessageBox(MessageBoxManager& parMessageBoxManager, const std::string& message)
-      : Layout("openmw_messagebox.layout")
-      , mCurrentTime(0)
-      , mMaxTime(0)
-      , mMessageBoxManager(parMessageBoxManager)
-      , mMessage(message)
+    MessageBox::MessageBox(MessageBoxManager& parMessageBoxManager, std::string_view message)
+        : Layout("openmw_messagebox.layout")
+        , mCurrentTime(0)
+        , mMaxTime(0)
+        , mMessageBoxManager(parMessageBoxManager)
+        , mMessage(message)
     {
         // defines
         mBottomPadding = 48;
@@ -194,19 +196,19 @@ namespace MWGui
         mMessageWidget->setCaptionWithReplacing(mMessage);
     }
 
-    void MessageBox::update (int height)
+    void MessageBox::update(int height)
     {
         MyGUI::IntSize gameWindowSize = MyGUI::RenderManager::getInstance().getViewSize();
         MyGUI::IntPoint pos;
-        pos.left = (gameWindowSize.width - mMainWidget->getWidth())/2;
+        pos.left = (gameWindowSize.width - mMainWidget->getWidth()) / 2;
         pos.top = (gameWindowSize.height - mMainWidget->getHeight() - height - mBottomPadding);
 
         mMainWidget->setPosition(pos);
     }
 
-    int MessageBox::getHeight ()
+    int MessageBox::getHeight()
     {
-        return mMainWidget->getHeight()+mNextBoxPadding;
+        return mMainWidget->getHeight() + mNextBoxPadding;
     }
 
     void MessageBox::setVisible(bool value)
@@ -214,10 +216,15 @@ namespace MWGui
         mMainWidget->setVisible(value);
     }
 
-    InteractiveMessageBox::InteractiveMessageBox(MessageBoxManager& parMessageBoxManager, const std::string& message, const std::vector<std::string>& buttons)
-        : WindowModal(MWBase::Environment::get().getWindowManager()->isGuiMode() ? "openmw_interactive_messagebox_notransp.layout" : "openmw_interactive_messagebox.layout")
-      , mMessageBoxManager(parMessageBoxManager)
-      , mButtonPressed(-1)
+    InteractiveMessageBox::InteractiveMessageBox(MessageBoxManager& parMessageBoxManager, const std::string& message,
+        const std::vector<std::string>& buttons, bool immediate, int defaultFocus)
+        : WindowModal(MWBase::Environment::get().getWindowManager()->isGuiMode()
+                ? "openmw_interactive_messagebox_notransp.layout"
+                : "openmw_interactive_messagebox.layout")
+        , mMessageBoxManager(parMessageBoxManager)
+        , mButtonPressed(-1)
+        , mDefaultFocus(defaultFocus)
+        , mImmediate(immediate)
     {
         int textPadding = 10; // padding between text-widget and main-widget
         int textButtonPadding = 10; // padding between the text-widget und the button-widget
@@ -229,7 +236,6 @@ namespace MWGui
 
         mMarkedToDelete = false;
 
-
         getWidget(mMessageWidget, "message");
         getWidget(mButtonsWidget, "buttons");
 
@@ -246,13 +252,10 @@ namespace MWGui
         int buttonHeight = 0;
         MyGUI::IntCoord dummyCoord(0, 0, 0, 0);
 
-        for(const std::string& buttonId : buttons)
+        for (const std::string& buttonId : buttons)
         {
             MyGUI::Button* button = mButtonsWidget->createWidget<MyGUI::Button>(
-                MyGUI::WidgetStyle::Child,
-                std::string("MW_Button"),
-                dummyCoord,
-                MyGUI::Align::Default);
+                MyGUI::WidgetStyle::Child, std::string("MW_Button"), dummyCoord, MyGUI::Align::Default);
             button->setCaptionWithReplacing(buttonId);
 
             button->eventMouseButtonClick += MyGUI::newDelegate(this, &InteractiveMessageBox::mousePressed);
@@ -262,41 +265,42 @@ namespace MWGui
             if (buttonsWidth != 0)
                 buttonsWidth += buttonLeftPadding;
 
-            int buttonWidth = button->getTextSize().width + 2*buttonLabelLeftPadding;
+            int buttonWidth = button->getTextSize().width + 2 * buttonLabelLeftPadding;
             buttonsWidth += buttonWidth;
 
-            buttonHeight = button->getTextSize().height + 2*buttonLabelTopPadding;
+            buttonHeight = button->getTextSize().height + 2 * buttonLabelTopPadding;
 
             if (buttonsHeight != 0)
                 buttonsHeight += buttonTopPadding;
             buttonsHeight += buttonHeight;
 
-            if(buttonWidth > biggestButtonWidth)
+            if (buttonWidth > biggestButtonWidth)
             {
                 biggestButtonWidth = buttonWidth;
             }
         }
 
         MyGUI::IntSize mainWidgetSize;
-        if(buttonsWidth < textSize.width)
+        if (buttonsWidth < textSize.width)
         {
             // on one line
-            mainWidgetSize.width = textSize.width + 3*textPadding;
-            mainWidgetSize.height = textPadding + textSize.height + textButtonPadding + buttonHeight + buttonMainPadding;
+            mainWidgetSize.width = textSize.width + 3 * textPadding;
+            mainWidgetSize.height
+                = textPadding + textSize.height + textButtonPadding + buttonHeight + buttonMainPadding;
 
             MyGUI::IntSize realSize = mainWidgetSize +
-                    // To account for borders
-                    (mMainWidget->getSize() - mMainWidget->getClientWidget()->getSize());
+                // To account for borders
+                (mMainWidget->getSize() - mMainWidget->getClientWidget()->getSize());
 
             MyGUI::IntPoint absPos;
-            absPos.left = (gameWindowSize.width - realSize.width)/2;
-            absPos.top = (gameWindowSize.height - realSize.height)/2;
+            absPos.left = (gameWindowSize.width - realSize.width) / 2;
+            absPos.top = (gameWindowSize.height - realSize.height) / 2;
 
             mMainWidget->setPosition(absPos);
             mMainWidget->setSize(realSize);
 
             MyGUI::IntCoord messageWidgetCoord;
-            messageWidgetCoord.left = (mainWidgetSize.width - textSize.width)/2;
+            messageWidgetCoord.left = (mainWidgetSize.width - textSize.width) / 2;
             messageWidgetCoord.top = textPadding;
             mMessageWidget->setCoord(messageWidgetCoord);
 
@@ -304,15 +308,15 @@ namespace MWGui
 
             MyGUI::IntCoord buttonCord;
             MyGUI::IntSize buttonSize(0, buttonHeight);
-            int left = (mainWidgetSize.width - buttonsWidth)/2;
+            int left = (mainWidgetSize.width - buttonsWidth) / 2;
 
-            for(MyGUI::Button* button : mButtons)
+            for (MyGUI::Button* button : mButtons)
             {
                 buttonCord.left = left;
                 buttonCord.top = messageWidgetCoord.top + textSize.height + textButtonPadding;
 
-                buttonSize.width = button->getTextSize().width + 2*buttonLabelLeftPadding;
-                buttonSize.height = button->getTextSize().height + 2*buttonLabelTopPadding;
+                buttonSize.width = button->getTextSize().width + 2 * buttonLabelLeftPadding;
+                buttonSize.height = button->getTextSize().height + 2 * buttonLabelTopPadding;
 
                 button->setCoord(buttonCord);
                 button->setSize(buttonSize);
@@ -323,11 +327,13 @@ namespace MWGui
         else
         {
             // among each other
-            if(biggestButtonWidth > textSize.width) {
-                mainWidgetSize.width = biggestButtonWidth + buttonTopPadding*2;
+            if (biggestButtonWidth > textSize.width)
+            {
+                mainWidgetSize.width = biggestButtonWidth + buttonTopPadding * 2;
             }
-            else {
-                mainWidgetSize.width = textSize.width + 3*textPadding;
+            else
+            {
+                mainWidgetSize.width = textSize.width + 3 * textPadding;
             }
 
             MyGUI::IntCoord buttonCord;
@@ -335,13 +341,13 @@ namespace MWGui
 
             int top = textPadding + textSize.height + textButtonPadding;
 
-            for(MyGUI::Button* button : mButtons)
+            for (MyGUI::Button* button : mButtons)
             {
-                buttonSize.width = button->getTextSize().width + buttonLabelLeftPadding*2;
-                buttonSize.height = button->getTextSize().height + buttonLabelTopPadding*2;
+                buttonSize.width = button->getTextSize().width + buttonLabelLeftPadding * 2;
+                buttonSize.height = button->getTextSize().height + buttonLabelTopPadding * 2;
 
                 buttonCord.top = top;
-                buttonCord.left = (mainWidgetSize.width - buttonSize.width)/2;
+                buttonCord.left = (mainWidgetSize.width - buttonSize.width) / 2;
 
                 button->setCoord(buttonCord);
                 button->setSize(buttonSize);
@@ -349,19 +355,20 @@ namespace MWGui
                 top += buttonSize.height + buttonTopPadding;
             }
 
-            mainWidgetSize.height = textPadding + textSize.height + textButtonPadding + buttonsHeight + buttonMainPadding;
+            mainWidgetSize.height
+                = textPadding + textSize.height + textButtonPadding + buttonsHeight + buttonMainPadding;
             mMainWidget->setSize(mainWidgetSize +
-                                 // To account for borders
-                                 (mMainWidget->getSize() - mMainWidget->getClientWidget()->getSize()));
+                // To account for borders
+                (mMainWidget->getSize() - mMainWidget->getClientWidget()->getSize()));
 
             MyGUI::IntPoint absPos;
-            absPos.left = (gameWindowSize.width - mainWidgetSize.width)/2;
-            absPos.top = (gameWindowSize.height - mainWidgetSize.height)/2;
+            absPos.left = (gameWindowSize.width - mainWidgetSize.width) / 2;
+            absPos.top = (gameWindowSize.height - mainWidgetSize.height) / 2;
 
             mMainWidget->setPosition(absPos);
 
             MyGUI::IntCoord messageWidgetCoord;
-            messageWidgetCoord.left = (mainWidgetSize.width - textSize.width)/2;
+            messageWidgetCoord.left = (mainWidgetSize.width - textSize.width) / 2;
             messageWidgetCoord.top = textPadding;
             messageWidgetCoord.width = textSize.width;
             messageWidgetCoord.height = textSize.height;
@@ -373,14 +380,17 @@ namespace MWGui
 
     MyGUI::Widget* InteractiveMessageBox::getDefaultKeyFocus()
     {
-        std::vector<std::string> keywords { "sOk", "sYes" };
-        for(MyGUI::Button* button : mButtons)
+        if (mDefaultFocus >= 0 && mDefaultFocus < static_cast<int>(mButtons.size()))
+            return mButtons[mDefaultFocus];
+        auto& languageManager = MyGUI::LanguageManager::getInstance();
+        std::vector<MyGUI::UString> keywords{ languageManager.replaceTags("#{sOk}"),
+            languageManager.replaceTags("#{sYes}") };
+
+        for (MyGUI::Button* button : mButtons)
         {
-            for (const std::string& keyword : keywords)
+            for (const MyGUI::UString& keyword : keywords)
             {
-                if (Misc::StringUtils::ciEqual(
-                        MyGUI::LanguageManager::getInstance().replaceTags("#{" + keyword + "}").asUTF8(),
-                        button->getCaption().asUTF8()))
+                if (Misc::StringUtils::ciEqual(keyword, button->getCaption()))
                 {
                     return button;
                 }
@@ -389,28 +399,34 @@ namespace MWGui
         return nullptr;
     }
 
-    void InteractiveMessageBox::mousePressed (MyGUI::Widget* pressed)
+    void InteractiveMessageBox::mousePressed(MyGUI::Widget* pressed)
     {
-        buttonActivated (pressed);
+        buttonActivated(pressed);
     }
 
-    void InteractiveMessageBox::buttonActivated (MyGUI::Widget* pressed)
+    void InteractiveMessageBox::buttonActivated(MyGUI::Widget* pressed)
     {
         mMarkedToDelete = true;
         int index = 0;
-        for(const MyGUI::Button* button : mButtons)
+        for (const MyGUI::Button* button : mButtons)
         {
-            if(button == pressed)
+            if (button == pressed)
             {
                 mButtonPressed = index;
                 mMessageBoxManager.onButtonPressed(mButtonPressed);
+                if (!mImmediate)
+                    return;
+
+                mMessageBoxManager.setLastButtonPressed(mButtonPressed);
+                MWBase::Environment::get().getInputManager()->changeInputMode(
+                    MWBase::Environment::get().getWindowManager()->isGuiMode());
                 return;
             }
             index++;
         }
     }
 
-    int InteractiveMessageBox::readPressedButton ()
+    int InteractiveMessageBox::readPressedButton()
     {
         return mButtonPressed;
     }
