@@ -1,29 +1,42 @@
 #include "postprocessorhud.hpp"
 
-#include <MyGUI_Window.h>
+#include <MyGUI_Align.h>
 #include <MyGUI_Button.h>
-#include <MyGUI_TabItem.h>
-#include <MyGUI_ScrollView.h>
+#include <MyGUI_Delegate.h>
+#include <MyGUI_EditBox.h>
+#include <MyGUI_EventPair.h>
 #include <MyGUI_FactoryManager.h>
+#include <MyGUI_Gui.h>
+#include <MyGUI_InputManager.h>
+#include <MyGUI_Macros.h>
+#include <MyGUI_ScrollView.h>
+#include <MyGUI_TabItem.h>
+#include <MyGUI_UString.h>
+#include <MyGUI_Widget.h>
+#include <MyGUI_WidgetDefines.h>
+#include <MyGUI_WidgetInput.h>
+#include <MyGUI_Window.h>
 
-#include <components/fx/widgets.hpp>
 #include <components/fx/technique.hpp>
+#include <components/fx/widgets.hpp>
 
+#include <components/misc/strings/algorithm.hpp>
 #include <components/misc/utf8stream.hpp>
 
 #include <components/widgets/box.hpp>
 
 #include "../mwrender/postprocessor.hpp"
 
-#include "../mwbase/world.hpp"
 #include "../mwbase/environment.hpp"
 #include "../mwbase/windowmanager.hpp"
+#include "../mwbase/world.hpp"
 
 namespace MWGui
 {
     void PostProcessorHud::ListWrapper::onKeyButtonPressed(MyGUI::KeyCode key, MyGUI::Char ch)
     {
-        if (MyGUI::InputManager::getInstance().isShiftPressed() && (key == MyGUI::KeyCode::ArrowUp || key == MyGUI::KeyCode::ArrowDown))
+        if (MyGUI::InputManager::getInstance().isShiftPressed()
+            && (key == MyGUI::KeyCode::ArrowUp || key == MyGUI::KeyCode::ArrowDown))
             return;
 
         MyGUI::ListBox::onKeyButtonPressed(key, ch);
@@ -42,7 +55,8 @@ namespace MWGui
         getWidget(mButtonDown, "ButtonDown");
 
         mButtonActivate->eventMouseButtonClick += MyGUI::newDelegate(this, &PostProcessorHud::notifyActivatePressed);
-        mButtonDeactivate->eventMouseButtonClick += MyGUI::newDelegate(this, &PostProcessorHud::notifyDeactivatePressed);
+        mButtonDeactivate->eventMouseButtonClick
+            += MyGUI::newDelegate(this, &PostProcessorHud::notifyDeactivatePressed);
         mButtonUp->eventMouseButtonClick += MyGUI::newDelegate(this, &PostProcessorHud::notifyShaderUpPressed);
         mButtonDown->eventMouseButtonClick += MyGUI::newDelegate(this, &PostProcessorHud::notifyShaderDownPressed);
 
@@ -54,7 +68,8 @@ namespace MWGui
 
         mFilter->eventEditTextChange += MyGUI::newDelegate(this, &PostProcessorHud::notifyFilterChanged);
 
-        mMainWidget->castType<MyGUI::Window>()->eventWindowChangeCoord += MyGUI::newDelegate(this, &PostProcessorHud::notifyWindowResize);
+        mMainWidget->castType<MyGUI::Window>()->eventWindowChangeCoord
+            += MyGUI::newDelegate(this, &PostProcessorHud::notifyWindowResize);
 
         mShaderInfo = mConfigLayout->createWidget<Gui::AutoSizedEditBox>("HeaderText", {}, MyGUI::Align::Default);
         mShaderInfo->setUserString("VStretch", "true");
@@ -67,7 +82,7 @@ namespace MWGui
 
         mConfigLayout->setVisibleVScroll(true);
 
-        mConfigArea = mConfigLayout->createWidget<MyGUI::Widget>("", {}, MyGUI::Align::Default);
+        mConfigArea = mConfigLayout->createWidget<MyGUI::Widget>({}, {}, MyGUI::Align::Default);
 
         mConfigLayout->eventMouseWheel += MyGUI::newDelegate(this, &PostProcessorHud::notifyMouseWheel);
         mConfigArea->eventMouseWheel += MyGUI::newDelegate(this, &PostProcessorHud::notifyMouseWheel);
@@ -121,9 +136,9 @@ namespace MWGui
                 return;
 
             if (enabled)
-                processor->enableTechnique(technique);
+                processor->enableTechnique(std::move(technique));
             else
-                processor->disableTechnique(technique);
+                processor->disableTechnique(std::move(technique));
             processor->saveChain();
         }
     }
@@ -153,10 +168,11 @@ namespace MWGui
         if (static_cast<size_t>(index) != selected)
         {
             auto technique = *mActiveList->getItemDataAt<std::shared_ptr<fx::Technique>>(selected);
-            if (technique->getDynamic())
+            if (technique->getDynamic() || technique->getInternal())
                 return;
 
-            if (processor->enableTechnique(technique, index) != MWRender::PostProcessor::Status_Error)
+            if (processor->enableTechnique(std::move(technique), index - mOffset)
+                != MWRender::PostProcessor::Status_Error)
                 processor->saveChain();
         }
     }
@@ -204,7 +220,8 @@ namespace MWGui
                 select(mActiveList, 0);
             }
         }
-        else if (list == mActiveList && MyGUI::InputManager::getInstance().isShiftPressed() && (key == MyGUI::KeyCode::ArrowUp || key == MyGUI::KeyCode::ArrowDown))
+        else if (list == mActiveList && MyGUI::InputManager::getInstance().isShiftPressed()
+            && (key == MyGUI::KeyCode::ArrowUp || key == MyGUI::KeyCode::ArrowDown))
         {
             moveShader(key == MyGUI::KeyCode::ArrowUp ? Direction::Up : Direction::Down);
         }
@@ -225,19 +242,21 @@ namespace MWGui
     {
         constexpr int padding = 12;
         constexpr int padding2 = padding * 2;
-        mShaderInfo->setCoord(padding, padding, mConfigLayout->getSize().width - padding2 - padding, mShaderInfo->getTextSize().height);
+        mShaderInfo->setCoord(
+            padding, padding, mConfigLayout->getSize().width - padding2 - padding, mShaderInfo->getTextSize().height);
 
         int totalHeight = mShaderInfo->getTop() + mShaderInfo->getTextSize().height + padding;
 
-        mConfigArea->setCoord({padding, totalHeight, mShaderInfo->getSize().width, mConfigLayout->getHeight()});
+        mConfigArea->setCoord({ padding, totalHeight, mShaderInfo->getSize().width, mConfigLayout->getHeight() });
 
         int childHeights = 0;
-		MyGUI::EnumeratorWidgetPtr enumerator = mConfigArea->getEnumerator();
-		while (enumerator.next())
-		{
-			enumerator.current()->setCoord(padding, childHeights + padding, mShaderInfo->getSize().width - padding2, enumerator.current()->getHeight());
+        MyGUI::EnumeratorWidgetPtr enumerator = mConfigArea->getEnumerator();
+        while (enumerator.next())
+        {
+            enumerator.current()->setCoord(padding, childHeights + padding, mShaderInfo->getSize().width - padding2,
+                enumerator.current()->getHeight());
             childHeights += enumerator.current()->getHeight() + padding;
-		}
+        }
         totalHeight += childHeights;
 
         mConfigArea->setSize(mConfigArea->getWidth(), childHeights);
@@ -246,7 +265,7 @@ namespace MWGui
         mConfigLayout->setSize(mConfigLayout->getWidth(), mConfigLayout->getParentSize().height - padding2);
     }
 
-    void PostProcessorHud::notifyMouseWheel(MyGUI::Widget *sender, int rel)
+    void PostProcessorHud::notifyMouseWheel(MyGUI::Widget* sender, int rel)
     {
         int offset = mConfigLayout->getViewOffset().top + rel * 0.3;
         if (offset > 0)
@@ -288,20 +307,18 @@ namespace MWGui
         while (mConfigArea->getChildCount() > 0)
             MyGUI::Gui::getInstance().destroyWidget(mConfigArea->getChildAt(0));
 
-        mShaderInfo->setCaption("");
+        mShaderInfo->setCaption({});
 
         std::ostringstream ss;
 
-        const std::string NA = "#{Interface:NotAvailableShort}";
-        const std::string endl = "\n";
+        const std::string_view NA = "#{Interface:NotAvailableShort}";
+        const char endl = '\n';
 
-        std::string author = technique->getAuthor().empty() ? NA : std::string(technique->getAuthor());
-        std::string version = technique->getVersion().empty() ? NA : std::string(technique->getVersion());
-        std::string description = technique->getDescription().empty() ? NA : std::string(technique->getDescription());
+        std::string_view author = technique->getAuthor().empty() ? NA : technique->getAuthor();
+        std::string_view version = technique->getVersion().empty() ? NA : technique->getVersion();
+        std::string_view description = technique->getDescription().empty() ? NA : technique->getDescription();
 
-        auto serializeBool = [](bool value) {
-            return value ? "#{sYes}" : "#{sNo}";
-        };
+        auto serializeBool = [](bool value) { return value ? "#{Interface:Yes}" : "#{Interface:No}"; };
 
         const auto flags = technique->getFlags();
 
@@ -316,19 +333,30 @@ namespace MWGui
             case fx::Technique::Status::Uncompiled:
             {
                 if (technique->getDynamic())
-                    ss  << "#{fontcolourhtml=header}#{PostProcessing:ShaderLocked}:      #{fontcolourhtml=normal} #{PostProcessing:ShaderLockedDescription}" << endl << endl;
-                ss  << "#{fontcolourhtml=header}#{PostProcessing:Author}:      #{fontcolourhtml=normal} " << author << endl << endl
-                    << "#{fontcolourhtml=header}#{PostProcessing:Version}:     #{fontcolourhtml=normal} " << version << endl << endl
-                    << "#{fontcolourhtml=header}#{PostProcessing:Description}: #{fontcolourhtml=normal} " << description << endl << endl
-                    << "#{fontcolourhtml=header}#{PostProcessing:InInteriors}: #{fontcolourhtml=normal} " << flag_interior
-                    << "#{fontcolourhtml=header}   #{PostProcessing:InExteriors}: #{fontcolourhtml=normal} " << flag_exterior
-                    << "#{fontcolourhtml=header}   #{PostProcessing:Underwater}: #{fontcolourhtml=normal} " << flag_underwater
-                    << "#{fontcolourhtml=header}   #{PostProcessing:Abovewater}: #{fontcolourhtml=normal} " << flag_abovewater;
+                    ss << "#{fontcolourhtml=header}#{OMWShaders:ShaderLocked}:      #{fontcolourhtml=normal} "
+                          "#{OMWShaders:ShaderLockedDescription}"
+                       << endl
+                       << endl;
+                ss << "#{fontcolourhtml=header}#{OMWShaders:Author}:      #{fontcolourhtml=normal} " << author << endl
+                   << endl
+                   << "#{fontcolourhtml=header}#{OMWShaders:Version}:     #{fontcolourhtml=normal} " << version << endl
+                   << endl
+                   << "#{fontcolourhtml=header}#{OMWShaders:Description}: #{fontcolourhtml=normal} " << description
+                   << endl
+                   << endl
+                   << "#{fontcolourhtml=header}#{OMWShaders:InInteriors}: #{fontcolourhtml=normal} " << flag_interior
+                   << "#{fontcolourhtml=header}   #{OMWShaders:InExteriors}: #{fontcolourhtml=normal} " << flag_exterior
+                   << "#{fontcolourhtml=header}   #{OMWShaders:Underwater}: #{fontcolourhtml=normal} "
+                   << flag_underwater
+                   << "#{fontcolourhtml=header}   #{OMWShaders:Abovewater}: #{fontcolourhtml=normal} "
+                   << flag_abovewater;
                 break;
             }
             case fx::Technique::Status::Parse_Error:
-                ss  << "#{fontcolourhtml=negative}Shader Compile Error: #{fontcolourhtml=normal} <" << std::string(technique->getName()) << "> failed to compile." << endl << endl
-                    << technique->getLastError();
+                ss << "#{fontcolourhtml=negative}Shader Compile Error: #{fontcolourhtml=normal} <"
+                   << std::string(technique->getName()) << "> failed to compile." << endl
+                   << endl
+                   << technique->getLastError();
                 break;
             case fx::Technique::Status::File_Not_exists:
                 break;
@@ -340,11 +368,13 @@ namespace MWGui
         {
             if (technique->getUniformMap().size() > 0)
             {
-                MyGUI::Button* resetButton = mConfigArea->createWidget<MyGUI::Button>("MW_Button", {0,0,0,24}, MyGUI::Align::Default);
-                resetButton->setCaptionWithReplacing("#{PostProcessing:ResetShader}");
+                MyGUI::Button* resetButton
+                    = mConfigArea->createWidget<MyGUI::Button>("MW_Button", { 0, 0, 0, 24 }, MyGUI::Align::Default);
+                resetButton->setCaptionWithReplacing("#{OMWShaders:ResetShader}");
                 resetButton->setTextAlign(MyGUI::Align::Center);
                 resetButton->eventMouseWheel += MyGUI::newDelegate(this, &PostProcessorHud::notifyMouseWheel);
-                resetButton->eventMouseButtonClick += MyGUI::newDelegate(this, &PostProcessorHud::notifyResetButtonClicked);
+                resetButton->eventMouseButtonClick
+                    += MyGUI::newDelegate(this, &PostProcessorHud::notifyResetButtonClicked);
             }
 
             for (const auto& uniform : technique->getUniformMap())
@@ -354,12 +384,14 @@ namespace MWGui
 
                 if (!uniform->mHeader.empty())
                 {
-                    Gui::AutoSizedTextBox* divider = mConfigArea->createWidget<Gui::AutoSizedTextBox>("MW_UniformGroup", {0,0,0,34}, MyGUI::Align::Default);
+                    Gui::AutoSizedTextBox* divider = mConfigArea->createWidget<Gui::AutoSizedTextBox>(
+                        "MW_UniformGroup", { 0, 0, 0, 34 }, MyGUI::Align::Default);
                     divider->setNeedMouseFocus(false);
                     divider->setCaptionWithReplacing(uniform->mHeader);
                 }
 
-                fx::Widgets::UniformBase* uwidget = mConfigArea->createWidget<fx::Widgets::UniformBase>("MW_UniformEdit", {0,0,0,22}, MyGUI::Align::Default);
+                fx::Widgets::UniformBase* uwidget = mConfigArea->createWidget<fx::Widgets::UniformBase>(
+                    "MW_UniformEdit", { 0, 0, 0, 22 }, MyGUI::Align::Default);
                 uwidget->init(uniform);
                 uwidget->getLabel()->eventMouseWheel += MyGUI::newDelegate(this, &PostProcessorHud::notifyMouseWheel);
             }
@@ -391,7 +423,12 @@ namespace MWGui
 
         auto* processor = MWBase::Environment::get().getWorld()->getPostProcessor();
 
+        std::vector<std::string> techniques;
         for (const auto& [name, _] : processor->getTechniqueMap())
+            techniques.push_back(name);
+        std::sort(techniques.begin(), techniques.end(), Misc::StringUtils::ciLess);
+
+        for (const std::string& name : techniques)
         {
             auto technique = processor->loadTechnique(name);
 
@@ -408,18 +445,22 @@ namespace MWGui
             }
         }
 
+        mOffset = 0;
         for (auto technique : processor->getTechniques())
         {
             if (!technique->getHidden())
+            {
                 mActiveList->addItem(technique->getName(), technique);
+
+                if (technique->getInternal())
+                    mOffset++;
+            }
         }
 
-        auto tryFocus = [this](ListWrapper* widget, const std::string& hint)
-        {
+        auto tryFocus = [this](ListWrapper* widget, const std::string& hint) {
             MyGUI::Widget* oldFocus = MyGUI::InputManager::getInstance().getKeyFocusWidget();
             if (oldFocus == mFilter)
                 return;
-
             size_t index = widget->findItemIndexWith(hint);
 
             if (index != MyGUI::ITEM_NONE)
@@ -450,6 +491,7 @@ namespace MWGui
         factory.registerFactory<fx::Widgets::EditNumberFloat>("Widget");
         factory.registerFactory<fx::Widgets::EditNumberInt>("Widget");
         factory.registerFactory<fx::Widgets::EditBool>("Widget");
+        factory.registerFactory<fx::Widgets::EditChoice>("Widget");
         factory.registerFactory<ListWrapper>("Widget");
     }
 }
