@@ -1,35 +1,35 @@
 #include "quickkeysmenu.hpp"
 
-#include <MyGUI_EditBox.h>
 #include <MyGUI_Button.h>
+#include <MyGUI_EditBox.h>
 #include <MyGUI_Gui.h>
 #include <MyGUI_ImageBox.h>
 #include <MyGUI_RenderManager.h>
 
 #include <components/esm3/esmwriter.hpp>
+#include <components/esm3/loadmgef.hpp>
 #include <components/esm3/quickkeys.hpp>
-#include <components/resource/resourcesystem.hpp>
 #include <components/misc/resourcehelpers.hpp>
+#include <components/resource/resourcesystem.hpp>
 
-#include "../mwworld/inventorystore.hpp"
 #include "../mwworld/class.hpp"
-#include "../mwworld/player.hpp"
 #include "../mwworld/esmstore.hpp"
+#include "../mwworld/inventorystore.hpp"
+#include "../mwworld/player.hpp"
 
 #include "../mwbase/environment.hpp"
 #include "../mwbase/mechanicsmanager.hpp"
-#include "../mwbase/world.hpp"
 #include "../mwbase/windowmanager.hpp"
+#include "../mwbase/world.hpp"
 
-#include "../mwmechanics/spellutil.hpp"
-#include "../mwmechanics/creaturestats.hpp"
 #include "../mwmechanics/actorutil.hpp"
+#include "../mwmechanics/creaturestats.hpp"
+#include "../mwmechanics/spellutil.hpp"
 
 #include "itemselection.hpp"
-#include "spellview.hpp"
 #include "itemwidget.hpp"
 #include "sortfilteritemmodel.hpp"
-
+#include "spellview.hpp"
 
 namespace MWGui
 {
@@ -39,25 +39,21 @@ namespace MWGui
         , mKey(std::vector<keyData>(10))
         , mSelected(nullptr)
         , mActivated(nullptr)
-        , mAssignDialog(nullptr)
-        , mItemSelectionDialog(nullptr)
-        , mMagicSelectionDialog(nullptr)
 
     {
         getWidget(mOkButton, "OKButton");
         getWidget(mInstructionLabel, "InstructionLabel");
 
         mMainWidget->setSize(mMainWidget->getWidth(),
-                             mMainWidget->getHeight() +
-                             (mInstructionLabel->getTextSize().height - mInstructionLabel->getHeight()));
+            mMainWidget->getHeight() + (mInstructionLabel->getTextSize().height - mInstructionLabel->getHeight()));
 
         mOkButton->eventMouseButtonClick += MyGUI::newDelegate(this, &QuickKeysMenu::onOkButtonClicked);
         center();
 
         for (int i = 0; i < 10; ++i)
         {
-            mKey[i].index = i+1;
-            getWidget(mKey[i].button, "QuickKey" + MyGUI::utility::toString(i+1));
+            mKey[i].index = i + 1;
+            getWidget(mKey[i].button, "QuickKey" + MyGUI::utility::toString(i + 1));
             mKey[i].button->eventMouseButtonClick += MyGUI::newDelegate(this, &QuickKeysMenu::onQuickKeyButtonClicked);
 
             unassign(&mKey[i]);
@@ -68,42 +64,34 @@ namespace MWGui
     {
         mActivated = nullptr;
 
-        for (int i=0; i<10; ++i)
+        for (int i = 0; i < 10; ++i)
         {
             unassign(&mKey[i]);
         }
     }
 
-    QuickKeysMenu::~QuickKeysMenu()
-    {
-        delete mAssignDialog;
-        delete mItemSelectionDialog;
-        delete mMagicSelectionDialog;
-    }
-
     inline void QuickKeysMenu::validate(int index)
     {
         MWWorld::Ptr player = MWMechanics::getPlayer();
         MWWorld::InventoryStore& store = player.getClass().getInventoryStore(player);
         switch (mKey[index].type)
         {
-            case Type_Unassigned:
-            case Type_HandToHand:
-            case Type_Magic:
+            case ESM::QuickKeys::Type::Unassigned:
+            case ESM::QuickKeys::Type::HandToHand:
+            case ESM::QuickKeys::Type::Magic:
                 break;
-            case Type_Item:
-            case Type_MagicItem:
+            case ESM::QuickKeys::Type::Item:
+            case ESM::QuickKeys::Type::MagicItem:
             {
                 MWWorld::Ptr item = *mKey[index].button->getUserData<MWWorld::Ptr>();
                 // Make sure the item is available and is not broken
-                if (!item || item.getRefData().getCount() < 1 ||
-                    (item.getClass().hasItemHealth(item) &&
-                        item.getClass().getItemHealth(item) <= 0))
+                if (item.isEmpty() || item.getCellRef().getCount() < 1
+                    || (item.getClass().hasItemHealth(item) && item.getClass().getItemHealth(item) <= 0))
                 {
                     // Try searching for a compatible replacement
                     item = store.findReplacement(mKey[index].id);
 
-                    if (item)
+                    if (!item.isEmpty())
                         mKey[index].button->setUserData(MWWorld::Ptr(item));
 
                     break;
@@ -145,22 +133,22 @@ namespace MWGui
 
         if (key->index == 10)
         {
-            key->type = Type_HandToHand;
+            key->type = ESM::QuickKeys::Type::HandToHand;
 
-            MyGUI::ImageBox* image = key->button->createWidget<MyGUI::ImageBox>("ImageBox",
-                MyGUI::IntCoord(14, 13, 32, 32), MyGUI::Align::Default);
+            MyGUI::ImageBox* image = key->button->createWidget<MyGUI::ImageBox>(
+                "ImageBox", MyGUI::IntCoord(14, 13, 32, 32), MyGUI::Align::Default);
 
             image->setImageTexture("icons\\k\\stealth_handtohand.dds");
             image->setNeedMouseFocus(false);
         }
         else
         {
-            key->type = Type_Unassigned;
-            key->id.clear();
+            key->type = ESM::QuickKeys::Type::Unassigned;
+            key->id = ESM::RefId();
             key->name.clear();
 
-            MyGUI::TextBox* textBox = key->button->createWidgetReal<MyGUI::TextBox>("SandText",
-                MyGUI::FloatCoord(0,0,1,1), MyGUI::Align::Default);
+            MyGUI::TextBox* textBox = key->button->createWidgetReal<MyGUI::TextBox>(
+                "SandText", MyGUI::FloatCoord(0, 0, 1, 1), MyGUI::Align::Default);
 
             textBox->setTextAlign(MyGUI::Align::Center);
             textBox->setCaption(MyGUI::utility::toString(key->index));
@@ -188,18 +176,18 @@ namespace MWGui
 
         mSelected = &mKey[index];
 
-        // prevent reallocation of zero key from Type_HandToHand
-        if(mSelected->index == 10)
+        // prevent reallocation of zero key from ESM::QuickKeys::Type::HandToHand
+        if (mSelected->index == 10)
             return;
 
         // open assign dialog
         if (!mAssignDialog)
-            mAssignDialog = new QuickKeysMenuAssign(this);
+            mAssignDialog = std::make_unique<QuickKeysMenuAssign>(this);
 
         mAssignDialog->setVisible(true);
     }
 
-    void QuickKeysMenu::onOkButtonClicked (MyGUI::Widget *sender)
+    void QuickKeysMenu::onOkButtonClicked(MyGUI::Widget* sender)
     {
         MWBase::Environment::get().getWindowManager()->removeGuiMode(GM_QuickKeysMenu);
     }
@@ -208,7 +196,7 @@ namespace MWGui
     {
         if (!mItemSelectionDialog)
         {
-            mItemSelectionDialog = new ItemSelectionDialog("#{sQuickMenu6}");
+            mItemSelectionDialog = std::make_unique<ItemSelectionDialog>("#{sQuickMenu6}");
             mItemSelectionDialog->eventItemSelected += MyGUI::newDelegate(this, &QuickKeysMenu::onAssignItem);
             mItemSelectionDialog->eventDialogCanceled += MyGUI::newDelegate(this, &QuickKeysMenu::onAssignItemCancel);
         }
@@ -223,7 +211,7 @@ namespace MWGui
     {
         if (!mMagicSelectionDialog)
         {
-            mMagicSelectionDialog = new MagicSelectionDialog(this);
+            mMagicSelectionDialog = std::make_unique<MagicSelectionDialog>(this);
         }
         mMagicSelectionDialog->setVisible(true);
 
@@ -248,7 +236,7 @@ namespace MWGui
         while (mSelected->button->getChildCount()) // Destroy number label
             MyGUI::Gui::getInstance().destroyWidget(mSelected->button->getChildAt(0));
 
-        mSelected->type = Type_Item;
+        mSelected->type = ESM::QuickKeys::Type::Item;
         mSelected->id = item.getCellRef().getRefId();
         mSelected->name = item.getClass().getName(item);
 
@@ -272,16 +260,18 @@ namespace MWGui
         while (mSelected->button->getChildCount()) // Destroy number label
             MyGUI::Gui::getInstance().destroyWidget(mSelected->button->getChildAt(0));
 
-        mSelected->type = Type_MagicItem;
+        mSelected->type = ESM::QuickKeys::Type::MagicItem;
         mSelected->id = item.getCellRef().getRefId();
         mSelected->name = item.getClass().getName(item);
 
         float scale = 1.f;
-        MyGUI::ITexture* texture = MyGUI::RenderManager::getInstance().getTexture("textures\\menu_icon_select_magic_magic.dds");
+        MyGUI::ITexture* texture
+            = MyGUI::RenderManager::getInstance().getTexture("textures\\menu_icon_select_magic_magic.dds");
         if (texture)
             scale = texture->getHeight() / 64.f;
 
-        mSelected->button->setFrame("textures\\menu_icon_select_magic_magic.dds", MyGUI::IntCoord(0, 0, 44*scale, 44*scale));
+        mSelected->button->setFrame(
+            "textures\\menu_icon_select_magic_magic.dds", MyGUI::IntCoord(0, 0, 44 * scale, 44 * scale));
         mSelected->button->setIcon(item);
 
         mSelected->button->setUserString("ToolTipType", "ItemPtr");
@@ -291,37 +281,41 @@ namespace MWGui
             mMagicSelectionDialog->setVisible(false);
     }
 
-    void QuickKeysMenu::onAssignMagic(const std::string& spellId)
+    void QuickKeysMenu::onAssignMagic(const ESM::RefId& spellId)
     {
         assert(mSelected);
         while (mSelected->button->getChildCount()) // Destroy number label
             MyGUI::Gui::getInstance().destroyWidget(mSelected->button->getChildAt(0));
 
-        const MWWorld::ESMStore &esmStore = MWBase::Environment::get().getWorld()->getStore();
+        const MWWorld::ESMStore& esmStore = *MWBase::Environment::get().getESMStore();
         const ESM::Spell* spell = esmStore.get<ESM::Spell>().find(spellId);
 
-        mSelected->type = Type_Magic;
+        mSelected->type = ESM::QuickKeys::Type::Magic;
         mSelected->id = spellId;
         mSelected->name = spell->mName;
 
         mSelected->button->setItem(MWWorld::Ptr());
         mSelected->button->setUserString("ToolTipType", "Spell");
-        mSelected->button->setUserString("Spell", spellId);
+        mSelected->button->setUserString("Spell", spellId.serialize());
 
         // use the icon of the first effect
-        const ESM::MagicEffect* effect = esmStore.get<ESM::MagicEffect>().find(spell->mEffects.mList.front().mEffectID);
+        const ESM::MagicEffect* effect
+            = esmStore.get<ESM::MagicEffect>().find(spell->mEffects.mList.front().mData.mEffectID);
 
         std::string path = effect->mIcon;
+        std::replace(path.begin(), path.end(), '/', '\\');
         int slashPos = path.rfind('\\');
-        path.insert(slashPos+1, "b_");
+        path.insert(slashPos + 1, "b_");
         path = Misc::ResourceHelpers::correctIconPath(path, MWBase::Environment::get().getResourceSystem()->getVFS());
 
         float scale = 1.f;
-        MyGUI::ITexture* texture = MyGUI::RenderManager::getInstance().getTexture("textures\\menu_icon_select_magic.dds");
+        MyGUI::ITexture* texture
+            = MyGUI::RenderManager::getInstance().getTexture("textures\\menu_icon_select_magic.dds");
         if (texture)
             scale = texture->getHeight() / 64.f;
 
-        mSelected->button->setFrame("textures\\menu_icon_select_magic.dds", MyGUI::IntCoord(0, 0, 44*scale, 44*scale));
+        mSelected->button->setFrame(
+            "textures\\menu_icon_select_magic.dds", MyGUI::IntCoord(0, 0, 44 * scale, 44 * scale));
         mSelected->button->setIcon(path);
 
         if (mMagicSelectionDialog)
@@ -346,22 +340,20 @@ namespace MWGui
     {
         assert(index >= 1 && index <= 10);
 
-        keyData *key = &mKey[index-1];
-        
+        keyData* key = &mKey[index - 1];
+
         MWWorld::Ptr player = MWMechanics::getPlayer();
         MWWorld::InventoryStore& store = player.getClass().getInventoryStore(player);
-        const MWMechanics::CreatureStats &playerStats = player.getClass().getCreatureStats(player);
+        const MWMechanics::CreatureStats& playerStats = player.getClass().getCreatureStats(player);
 
-        validate(index-1);
+        validate(index - 1);
 
         // Delay action executing,
         // if player is busy for now (casting a spell, attacking someone, etc.)
         bool isDelayNeeded = MWBase::Environment::get().getMechanicsManager()->isAttackingOrSpell(player)
-                || playerStats.getKnockedDown()
-                || playerStats.getHitRecovery();
+            || playerStats.getKnockedDown() || playerStats.getHitRecovery();
 
-        bool godmode = MWBase::Environment::get().getWorld()->getGodModeState();
-        bool isReturnNeeded = (!godmode && playerStats.isParalyzed()) || playerStats.isDead();
+        bool isReturnNeeded = playerStats.isParalyzed() || playerStats.isDead();
 
         if (isReturnNeeded)
         {
@@ -377,7 +369,7 @@ namespace MWGui
             mActivated = nullptr;
         }
 
-        if (key->type == Type_Item || key->type == Type_MagicItem)
+        if (key->type == ESM::QuickKeys::Type::Item || key->type == ESM::QuickKeys::Type::MagicItem)
         {
             MWWorld::Ptr item = *key->button->getUserData<MWWorld::Ptr>();
 
@@ -391,32 +383,32 @@ namespace MWGui
                 item = nullptr;
 
             // check the item is available and not broken
-            if (!item || item.getRefData().getCount() < 1 ||
-               (item.getClass().hasItemHealth(item) && item.getClass().getItemHealth(item) <= 0))
+            if (item.isEmpty() || item.getCellRef().getCount() < 1
+                || (item.getClass().hasItemHealth(item) && item.getClass().getItemHealth(item) <= 0))
             {
                 item = store.findReplacement(key->id);
 
-                if (!item || item.getRefData().getCount() < 1)
+                if (item.isEmpty() || item.getCellRef().getCount() < 1)
                 {
-                    MWBase::Environment::get().getWindowManager()->messageBox(
-                        "#{sQuickMenu5} " + key->name);
+                    MWBase::Environment::get().getWindowManager()->messageBox("#{sQuickMenu5} " + key->name);
 
                     return;
                 }
             }
 
-            if (key->type == Type_Item)
+            if (key->type == ESM::QuickKeys::Type::Item)
             {
                 if (!store.isEquipped(item))
                     MWBase::Environment::get().getWindowManager()->useItem(item);
-                MWWorld::ConstContainerStoreIterator rightHand = store.getSlot(MWWorld::InventoryStore::Slot_CarriedRight);
+                MWWorld::ConstContainerStoreIterator rightHand
+                    = store.getSlot(MWWorld::InventoryStore::Slot_CarriedRight);
                 // change draw state only if the item is in player's right hand
                 if (rightHand != store.end() && item == *rightHand)
                 {
                     MWBase::Environment::get().getWorld()->getPlayer().setDrawState(MWMechanics::DrawState::Weapon);
                 }
             }
-            else if (key->type == Type_MagicItem)
+            else if (key->type == ESM::QuickKeys::Type::MagicItem)
             {
                 // equip, if it can be equipped and isn't yet equipped
                 if (!item.getClass().getEquipmentSlots(item).first.empty() && !store.isEquipped(item))
@@ -432,9 +424,9 @@ namespace MWGui
                 MWBase::Environment::get().getWorld()->getPlayer().setDrawState(MWMechanics::DrawState::Spell);
             }
         }
-        else if (key->type == Type_Magic)
+        else if (key->type == ESM::QuickKeys::Type::Magic)
         {
-            std::string spellId = key->id;
+            const ESM::RefId& spellId = key->id;
 
             // Make sure the player still has this spell
             MWMechanics::CreatureStats& stats = player.getClass().getCreatureStats(player);
@@ -447,20 +439,20 @@ namespace MWGui
             }
 
             store.setSelectedEnchantItem(store.end());
-            MWBase::Environment::get().getWindowManager()
-                ->setSelectedSpell(spellId, int(MWMechanics::getSpellSuccessChance(spellId, player)));
+            MWBase::Environment::get().getWindowManager()->setSelectedSpell(
+                spellId, int(MWMechanics::getSpellSuccessChance(spellId, player)));
             MWBase::Environment::get().getWorld()->getPlayer().setDrawState(MWMechanics::DrawState::Spell);
         }
-        else if (key->type == Type_HandToHand)
+        else if (key->type == ESM::QuickKeys::Type::HandToHand)
         {
-            store.unequipSlot(MWWorld::InventoryStore::Slot_CarriedRight, player);
+            store.unequipSlot(MWWorld::InventoryStore::Slot_CarriedRight);
             MWBase::Environment::get().getWorld()->getPlayer().setDrawState(MWMechanics::DrawState::Weapon);
         }
     }
 
     // ---------------------------------------------------------------------------------------------------------
 
-    QuickKeysMenuAssign::QuickKeysMenuAssign (QuickKeysMenu* parent)
+    QuickKeysMenuAssign::QuickKeysMenuAssign(QuickKeysMenu* parent)
         : WindowModal("openmw_quickkeys_menu_assign.layout")
         , mParent(parent)
     {
@@ -475,67 +467,57 @@ namespace MWGui
         mUnassignButton->eventMouseButtonClick += MyGUI::newDelegate(mParent, &QuickKeysMenu::onUnassignButtonClicked);
         mCancelButton->eventMouseButtonClick += MyGUI::newDelegate(mParent, &QuickKeysMenu::onCancelButtonClicked);
 
-
-        int maxWidth = mLabel->getTextSize ().width + 24;
-        maxWidth = std::max(maxWidth, mItemButton->getTextSize ().width + 24);
-        maxWidth = std::max(maxWidth, mMagicButton->getTextSize ().width + 24);
-        maxWidth = std::max(maxWidth, mUnassignButton->getTextSize ().width + 24);
-        maxWidth = std::max(maxWidth, mCancelButton->getTextSize ().width + 24);
+        int maxWidth = mLabel->getTextSize().width + 24;
+        maxWidth = std::max(maxWidth, mItemButton->getTextSize().width + 24);
+        maxWidth = std::max(maxWidth, mMagicButton->getTextSize().width + 24);
+        maxWidth = std::max(maxWidth, mUnassignButton->getTextSize().width + 24);
+        maxWidth = std::max(maxWidth, mCancelButton->getTextSize().width + 24);
 
         mMainWidget->setSize(maxWidth + 24, mMainWidget->getHeight());
         mLabel->setSize(maxWidth, mLabel->getHeight());
 
-        mItemButton->setCoord((maxWidth - mItemButton->getTextSize().width-24)/2 + 8,
-                              mItemButton->getTop(),
-                              mItemButton->getTextSize().width + 24,
-                              mItemButton->getHeight());
-        mMagicButton->setCoord((maxWidth - mMagicButton->getTextSize().width-24)/2 + 8,
-                              mMagicButton->getTop(),
-                              mMagicButton->getTextSize().width + 24,
-                              mMagicButton->getHeight());
-        mUnassignButton->setCoord((maxWidth - mUnassignButton->getTextSize().width-24)/2 + 8,
-                              mUnassignButton->getTop(),
-                              mUnassignButton->getTextSize().width + 24,
-                              mUnassignButton->getHeight());
-        mCancelButton->setCoord((maxWidth - mCancelButton->getTextSize().width-24)/2 + 8,
-                              mCancelButton->getTop(),
-                              mCancelButton->getTextSize().width + 24,
-                              mCancelButton->getHeight());
+        mItemButton->setCoord((maxWidth - mItemButton->getTextSize().width - 24) / 2 + 8, mItemButton->getTop(),
+            mItemButton->getTextSize().width + 24, mItemButton->getHeight());
+        mMagicButton->setCoord((maxWidth - mMagicButton->getTextSize().width - 24) / 2 + 8, mMagicButton->getTop(),
+            mMagicButton->getTextSize().width + 24, mMagicButton->getHeight());
+        mUnassignButton->setCoord((maxWidth - mUnassignButton->getTextSize().width - 24) / 2 + 8,
+            mUnassignButton->getTop(), mUnassignButton->getTextSize().width + 24, mUnassignButton->getHeight());
+        mCancelButton->setCoord((maxWidth - mCancelButton->getTextSize().width - 24) / 2 + 8, mCancelButton->getTop(),
+            mCancelButton->getTextSize().width + 24, mCancelButton->getHeight());
 
         center();
     }
 
-    void QuickKeysMenu::write(ESM::ESMWriter &writer)
+    void QuickKeysMenu::write(ESM::ESMWriter& writer)
     {
         writer.startRecord(ESM::REC_KEYS);
 
         ESM::QuickKeys keys;
 
         // NB: The quick key with index 9 always has Hand-to-Hand type and must not be saved
-        for (int i=0; i<9; ++i)
+        for (int i = 0; i < 9; ++i)
         {
             ItemWidget* button = mKey[i].button;
 
-            int type = mKey[i].type;
+            const ESM::QuickKeys::Type type = mKey[i].type;
 
             ESM::QuickKeys::QuickKey key;
             key.mType = type;
 
             switch (type)
             {
-                case Type_Unassigned:
-                case Type_HandToHand:
+                case ESM::QuickKeys::Type::Unassigned:
+                case ESM::QuickKeys::Type::HandToHand:
                     break;
-                case Type_Item:
-                case Type_MagicItem:
+                case ESM::QuickKeys::Type::Item:
+                case ESM::QuickKeys::Type::MagicItem:
                 {
                     MWWorld::Ptr item = *button->getUserData<MWWorld::Ptr>();
                     key.mId = item.getCellRef().getRefId();
                     break;
                 }
-                case Type_Magic:
-                    std::string spellId = button->getUserString("Spell");
-                    key.mId = spellId;
+                case ESM::QuickKeys::Type::Magic:
+                    key.mId = ESM::RefId::deserialize(button->getUserString("Spell"));
                     break;
             }
 
@@ -547,7 +529,7 @@ namespace MWGui
         writer.endRecord(ESM::REC_KEYS);
     }
 
-    void QuickKeysMenu::readRecord(ESM::ESMReader &reader, uint32_t type)
+    void QuickKeysMenu::readRecord(ESM::ESMReader& reader, uint32_t type)
     {
         if (type != ESM::REC_KEYS)
             return;
@@ -558,7 +540,7 @@ namespace MWGui
         MWWorld::Ptr player = MWMechanics::getPlayer();
         MWWorld::InventoryStore& store = player.getClass().getInventoryStore(player);
 
-        int i=0;
+        int i = 0;
         for (ESM::QuickKeys::QuickKey& quickKey : keys.mKeys)
         {
             // NB: The quick key with index 9 always has Hand-to-Hand type and must not be loaded
@@ -569,32 +551,32 @@ namespace MWGui
 
             switch (quickKey.mType)
             {
-            case Type_Magic:
-                if (MWBase::Environment::get().getWorld()->getStore().get<ESM::Spell>().search(quickKey.mId))
-                    onAssignMagic(quickKey.mId);
-                break;
-            case Type_Item:
-            case Type_MagicItem:
-            {
-                // Find the item by id
-                MWWorld::Ptr item = store.findReplacement(quickKey.mId);
-
-                if (item.isEmpty())
-                    unassign(mSelected);
-                else
+                case ESM::QuickKeys::Type::Magic:
+                    if (MWBase::Environment::get().getESMStore()->get<ESM::Spell>().search(quickKey.mId))
+                        onAssignMagic(quickKey.mId);
+                    break;
+                case ESM::QuickKeys::Type::Item:
+                case ESM::QuickKeys::Type::MagicItem:
                 {
-                    if (quickKey.mType == Type_Item)
-                        onAssignItem(item);
-                    else // if (quickKey.mType == Type_MagicItem)
-                        onAssignMagicItem(item);
-                }
+                    // Find the item by id
+                    MWWorld::Ptr item = store.findReplacement(quickKey.mId);
 
-                break;
-            }
-            case Type_Unassigned:
-            case Type_HandToHand:
-                unassign(mSelected);
-                break;
+                    if (item.isEmpty())
+                        unassign(mSelected);
+                    else
+                    {
+                        if (quickKey.mType == ESM::QuickKeys::Type::Item)
+                            onAssignItem(item);
+                        else // if (quickKey.mType == ESM::QuickKeys::Type::MagicItem)
+                            onAssignMagicItem(item);
+                    }
+
+                    break;
+                }
+                case ESM::QuickKeys::Type::Unassigned:
+                case ESM::QuickKeys::Type::HandToHand:
+                    unassign(mSelected);
+                    break;
             }
 
             ++i;
@@ -618,7 +600,7 @@ namespace MWGui
         center();
     }
 
-    void MagicSelectionDialog::onCancelButtonClicked (MyGUI::Widget *sender)
+    void MagicSelectionDialog::onCancelButtonClicked(MyGUI::Widget* sender)
     {
         exit();
     }
@@ -629,7 +611,7 @@ namespace MWGui
         return true;
     }
 
-    void MagicSelectionDialog::onOpen ()
+    void MagicSelectionDialog::onOpen()
     {
         WindowModal::onOpen();
 
