@@ -8,7 +8,8 @@ namespace MWGui
     /// \brief this class is intended for GUI interfaces that access an MW-Reference
     /// for example dialogue window accesses an NPC, or Container window accesses a Container
     /// these classes have to be automatically closed if the reference becomes unavailable
-    /// make sure that checkReferenceAvailable() is called every frame and that onReferenceUnavailable() has been overridden
+    /// make sure that checkReferenceAvailable() is called every frame and that onReferenceUnavailable() has been
+    /// overridden
     class ReferenceInterface
     {
     public:
@@ -17,15 +18,12 @@ namespace MWGui
 
         void checkReferenceAvailable(); ///< closes the window, if the MW-reference has become unavailable
 
-        virtual void resetReference() { mPtr = MWWorld::Ptr(); mCurrentPlayerCell = NULL; }
+        virtual void resetReference() { mPtr = MWWorld::Ptr(); }
 
     protected:
         virtual void onReferenceUnavailable() = 0; ///< called when reference has become unavailable
 
         MWWorld::Ptr mPtr;
-
-    private:
-        MWWorld::CellStore* mCurrentPlayerCell;
     };
 }
 
