@@ -3,21 +3,27 @@
 #include <cmath>
 
 #include <MyGUI_Button.h>
-#include <MyGUI_ScrollView.h>
-#include <MyGUI_ImageBox.h>
 #include <MyGUI_Gui.h>
+#include <MyGUI_ImageBox.h>
+#include <MyGUI_ScrollView.h>
+#include <MyGUI_UString.h>
+
+#include <components/esm3/loadbsgn.hpp>
+#include <components/esm3/loadrace.hpp>
+#include <components/esm3/loadspel.hpp>
+#include <components/settings/values.hpp>
 
 #include "../mwbase/environment.hpp"
-#include "../mwbase/world.hpp"
 #include "../mwbase/windowmanager.hpp"
-#include "../mwworld/esmstore.hpp"
+#include "../mwbase/world.hpp"
 #include "../mwmechanics/autocalcspell.hpp"
+#include "../mwworld/esmstore.hpp"
 
 #include "tooltips.hpp"
 
 namespace
 {
-    void adjustButtonSize(MyGUI::Button *button)
+    void adjustButtonSize(MyGUI::Button* button)
     {
         // adjust size of button to fit its text
         MyGUI::IntSize size = button->getTextSize();
@@ -28,8 +34,8 @@ namespace
 namespace MWGui
 {
     ReviewDialog::ReviewDialog()
-        : WindowModal("openmw_chargen_review.layout"),
-          mUpdateSkillArea(false)
+        : WindowModal("openmw_chargen_review.layout")
+        , mUpdateSkillArea(false)
     {
         // Centre dialog
         center();
@@ -58,36 +64,45 @@ namespace MWGui
 
         // Setup dynamic stats
         getWidget(mHealth, "Health");
-        mHealth->setTitle(MWBase::Environment::get().getWindowManager()->getGameSettingString("sHealth", ""));
+        mHealth->setTitle(MWBase::Environment::get().getWindowManager()->getGameSettingString("sHealth", {}));
         mHealth->setValue(45, 45);
 
         getWidget(mMagicka, "Magicka");
-        mMagicka->setTitle(MWBase::Environment::get().getWindowManager()->getGameSettingString("sMagic", ""));
+        mMagicka->setTitle(MWBase::Environment::get().getWindowManager()->getGameSettingString("sMagic", {}));
         mMagicka->setValue(50, 50);
 
         getWidget(mFatigue, "Fatigue");
-        mFatigue->setTitle(MWBase::Environment::get().getWindowManager()->getGameSettingString("sFatigue", ""));
+        mFatigue->setTitle(MWBase::Environment::get().getWindowManager()->getGameSettingString("sFatigue", {}));
         mFatigue->setValue(160, 160);
 
         // Setup attributes
 
-        Widgets::MWAttributePtr attribute;
-        for (int idx = 0; idx < ESM::Attribute::Length; ++idx)
+        MyGUI::Widget* attributes = getWidget("Attributes");
+        const auto& store = MWBase::Environment::get().getWorld()->getStore().get<ESM::Attribute>();
+        MyGUI::IntCoord coord{ 8, 4, 250, 18 };
+        for (const ESM::Attribute& attribute : store)
         {
-            getWidget(attribute, std::string("Attribute") + MyGUI::utility::toString(idx));
-            mAttributeWidgets.insert(std::make_pair(static_cast<int>(ESM::Attribute::sAttributeIds[idx]), attribute));
-            attribute->setAttributeId(ESM::Attribute::sAttributeIds[idx]);
-            attribute->setAttributeValue(Widgets::MWAttribute::AttributeValue());
+            auto* widget
+                = attributes->createWidget<Widgets::MWAttribute>("MW_StatNameValue", coord, MyGUI::Align::Default);
+            mAttributeWidgets.emplace(attribute.mId, widget);
+            widget->setUserString("ToolTipType", "Layout");
+            widget->setUserString("ToolTipLayout", "AttributeToolTip");
+            widget->setUserString("Caption_AttributeName", attribute.mName);
+            widget->setUserString("Caption_AttributeDescription", attribute.mDescription);
+            widget->setUserString("ImageTexture_AttributeImage", attribute.mIcon);
+            widget->setAttributeId(attribute.mId);
+            widget->setAttributeValue(Widgets::MWAttribute::AttributeValue());
+            coord.top += coord.height;
         }
 
         // Setup skills
         getWidget(mSkillView, "SkillView");
         mSkillView->eventMouseWheel += MyGUI::newDelegate(this, &ReviewDialog::onMouseWheel);
 
-        for (int i = 0; i < ESM::Skill::Length; ++i)
+        for (const ESM::Skill& skill : MWBase::Environment::get().getESMStore()->get<ESM::Skill>())
         {
-            mSkillValues.insert(std::make_pair(i, MWMechanics::SkillValue()));
-            mSkillWidgetMap.insert(std::make_pair(i, static_cast<MyGUI::TextBox*> (nullptr)));
+            mSkillValues.emplace(skill.mId, MWMechanics::SkillValue());
+            mSkillWidgetMap.emplace(skill.mId, static_cast<MyGUI::TextBox*>(nullptr));
         }
 
         MyGUI::Button* backButton;
@@ -114,17 +129,16 @@ namespace MWGui
         }
     }
 
-    void ReviewDialog::setPlayerName(const std::string &name)
+    void ReviewDialog::setPlayerName(const std::string& name)
     {
         mNameWidget->setCaption(name);
     }
 
-    void ReviewDialog::setRace(const std::string &raceId)
+    void ReviewDialog::setRace(const ESM::RefId& raceId)
     {
         mRaceId = raceId;
 
-        const ESM::Race *race =
-            MWBase::Environment::get().getWorld()->getStore().get<ESM::Race>().search(mRaceId);
+        const ESM::Race* race = MWBase::Environment::get().getESMStore()->get<ESM::Race>().search(mRaceId);
         if (race)
         {
             ToolTips::createRaceToolTip(mRaceWidget, race);
@@ -134,19 +148,19 @@ namespace MWGui
         mUpdateSkillArea = true;
     }
 
-    void ReviewDialog::setClass(const ESM::Class& class_)
+    void ReviewDialog::setClass(const ESM::Class& playerClass)
     {
-        mKlass = class_;
-        mClassWidget->setCaption(mKlass.mName);
-        ToolTips::createClassToolTip(mClassWidget, mKlass);
+        mClass = playerClass;
+        mClassWidget->setCaption(mClass.mName);
+        ToolTips::createClassToolTip(mClassWidget, mClass);
     }
 
-    void ReviewDialog::setBirthSign(const std::string& signId)
+    void ReviewDialog::setBirthSign(const ESM::RefId& signId)
     {
         mBirthSignId = signId;
 
-        const ESM::BirthSign *sign =
-            MWBase::Environment::get().getWorld()->getStore().get<ESM::BirthSign>().search(mBirthSignId);
+        const ESM::BirthSign* sign
+            = MWBase::Environment::get().getESMStore()->get<ESM::BirthSign>().search(mBirthSignId);
         if (sign)
         {
             mBirthSignWidget->setCaption(sign->mName);
@@ -162,7 +176,7 @@ namespace MWGui
         int modified = static_cast<int>(value.getModified());
 
         mHealth->setValue(current, modified);
-        std::string valStr =  MyGUI::utility::toString(current) + " / " + MyGUI::utility::toString(modified);
+        std::string valStr = MyGUI::utility::toString(current) + " / " + MyGUI::utility::toString(modified);
         mHealth->setUserString("Caption_HealthDescription", "#{sHealthDesc}\n" + valStr);
     }
 
@@ -172,7 +186,7 @@ namespace MWGui
         int modified = static_cast<int>(value.getModified());
 
         mMagicka->setValue(current, modified);
-        std::string valStr =  MyGUI::utility::toString(current) + " / " + MyGUI::utility::toString(modified);
+        std::string valStr = MyGUI::utility::toString(current) + " / " + MyGUI::utility::toString(modified);
         mMagicka->setUserString("Caption_HealthDescription", "#{sMagDesc}\n" + valStr);
     }
 
@@ -182,13 +196,13 @@ namespace MWGui
         int modified = static_cast<int>(value.getModified());
 
         mFatigue->setValue(current, modified);
-        std::string valStr =  MyGUI::utility::toString(current) + " / " + MyGUI::utility::toString(modified);
+        std::string valStr = MyGUI::utility::toString(current) + " / " + MyGUI::utility::toString(modified);
         mFatigue->setUserString("Caption_HealthDescription", "#{sFatDesc}\n" + valStr);
     }
 
-    void ReviewDialog::setAttribute(ESM::Attribute::AttributeID attributeId, const MWMechanics::AttributeValue& value)
+    void ReviewDialog::setAttribute(ESM::RefId attributeId, const MWMechanics::AttributeValue& value)
     {
-        std::map<int, Widgets::MWAttributePtr>::iterator attr = mAttributeWidgets.find(static_cast<int>(attributeId));
+        auto attr = mAttributeWidgets.find(attributeId);
         if (attr == mAttributeWidgets.end())
             return;
 
@@ -199,13 +213,14 @@ namespace MWGui
         }
     }
 
-    void ReviewDialog::setSkillValue(ESM::Skill::SkillEnum skillId, const MWMechanics::SkillValue& value)
+    void ReviewDialog::setSkillValue(ESM::RefId id, const MWMechanics::SkillValue& value)
     {
-        mSkillValues[skillId] = value;
-        MyGUI::TextBox* widget = mSkillWidgetMap[skillId];
+        mSkillValues[id] = value;
+        MyGUI::TextBox* widget = mSkillWidgetMap[id];
         if (widget)
         {
-            float modified = static_cast<float>(value.getModified()), base = static_cast<float>(value.getBase());
+            float modified = value.getModified();
+            float base = value.getBase();
             std::string text = MyGUI::utility::toString(std::floor(modified));
             std::string state = "normal";
             if (modified > base)
@@ -220,28 +235,30 @@ namespace MWGui
         mUpdateSkillArea = true;
     }
 
-    void ReviewDialog::configureSkills(const std::vector<int>& major, const std::vector<int>& minor)
+    void ReviewDialog::configureSkills(const std::vector<ESM::RefId>& major, const std::vector<ESM::RefId>& minor)
     {
         mMajorSkills = major;
         mMinorSkills = minor;
 
         // Update misc skills with the remaining skills not in major or minor
-        std::set<int> skillSet;
+        std::set<ESM::RefId> skillSet;
         std::copy(major.begin(), major.end(), std::inserter(skillSet, skillSet.begin()));
         std::copy(minor.begin(), minor.end(), std::inserter(skillSet, skillSet.begin()));
         mMiscSkills.clear();
-        for (const int skill : ESM::Skill::sSkillIds)
+        const auto& store = MWBase::Environment::get().getWorld()->getStore().get<ESM::Skill>();
+        for (const ESM::Skill& skill : store)
         {
-            if (skillSet.find(skill) == skillSet.end())
-                mMiscSkills.push_back(skill);
+            if (!skillSet.contains(skill.mId))
+                mMiscSkills.push_back(skill.mId);
         }
 
         mUpdateSkillArea = true;
     }
 
-    void ReviewDialog::addSeparator(MyGUI::IntCoord &coord1, MyGUI::IntCoord &coord2)
+    void ReviewDialog::addSeparator(MyGUI::IntCoord& coord1, MyGUI::IntCoord& coord2)
     {
-        MyGUI::ImageBox* separator = mSkillView->createWidget<MyGUI::ImageBox>("MW_HLine", MyGUI::IntCoord(10, coord1.top, coord1.width + coord2.width - 4, 18), MyGUI::Align::Default);
+        MyGUI::ImageBox* separator = mSkillView->createWidget<MyGUI::ImageBox>(
+            "MW_HLine", MyGUI::IntCoord(10, coord1.top, coord1.width + coord2.width - 4, 18), MyGUI::Align::Default);
         separator->eventMouseWheel += MyGUI::newDelegate(this, &ReviewDialog::onMouseWheel);
 
         mSkillWidgets.push_back(separator);
@@ -250,25 +267,27 @@ namespace MWGui
         coord2.top += separator->getHeight();
     }
 
-    void ReviewDialog::addGroup(const std::string &label, MyGUI::IntCoord &coord1, MyGUI::IntCoord &coord2)
+    void ReviewDialog::addGroup(std::string_view label, MyGUI::IntCoord& coord1, MyGUI::IntCoord& coord2)
     {
-        MyGUI::TextBox* groupWidget = mSkillView->createWidget<MyGUI::TextBox>("SandBrightText", MyGUI::IntCoord(0, coord1.top, coord1.width + coord2.width, coord1.height), MyGUI::Align::Default);
+        MyGUI::TextBox* groupWidget = mSkillView->createWidget<MyGUI::TextBox>("SandBrightText",
+            MyGUI::IntCoord(0, coord1.top, coord1.width + coord2.width, coord1.height), MyGUI::Align::Default);
         groupWidget->eventMouseWheel += MyGUI::newDelegate(this, &ReviewDialog::onMouseWheel);
-        groupWidget->setCaption(label);
+        groupWidget->setCaption(MyGUI::UString(label));
         mSkillWidgets.push_back(groupWidget);
 
-        int lineHeight = MWBase::Environment::get().getWindowManager()->getFontHeight() + 2;
+        const int lineHeight = Settings::gui().mFontSize + 2;
         coord1.top += lineHeight;
         coord2.top += lineHeight;
     }
 
-    MyGUI::TextBox* ReviewDialog::addValueItem(const std::string& text, const std::string &value, const std::string& state, MyGUI::IntCoord &coord1, MyGUI::IntCoord &coord2)
+    MyGUI::TextBox* ReviewDialog::addValueItem(std::string_view text, const std::string& value,
+        const std::string& state, MyGUI::IntCoord& coord1, MyGUI::IntCoord& coord2)
     {
         MyGUI::TextBox* skillNameWidget;
         MyGUI::TextBox* skillValueWidget;
 
         skillNameWidget = mSkillView->createWidget<MyGUI::TextBox>("SandText", coord1, MyGUI::Align::Default);
-        skillNameWidget->setCaption(text);
+        skillNameWidget->setCaption(MyGUI::UString(text));
         skillNameWidget->eventMouseWheel += MyGUI::newDelegate(this, &ReviewDialog::onMouseWheel);
 
         skillValueWidget = mSkillView->createWidget<MyGUI::TextBox>("SandTextRight", coord2, MyGUI::Align::Default);
@@ -279,44 +298,47 @@ namespace MWGui
         mSkillWidgets.push_back(skillNameWidget);
         mSkillWidgets.push_back(skillValueWidget);
 
-        int lineHeight = MWBase::Environment::get().getWindowManager()->getFontHeight() + 2;
+        const int lineHeight = Settings::gui().mFontSize + 2;
         coord1.top += lineHeight;
         coord2.top += lineHeight;
 
         return skillValueWidget;
     }
 
-    void ReviewDialog::addItem(const std::string& text, MyGUI::IntCoord &coord1, MyGUI::IntCoord &coord2)
+    void ReviewDialog::addItem(const std::string& text, MyGUI::IntCoord& coord1, MyGUI::IntCoord& coord2)
     {
         MyGUI::TextBox* skillNameWidget;
 
-        skillNameWidget = mSkillView->createWidget<MyGUI::TextBox>("SandText", coord1 + MyGUI::IntSize(coord2.width, 0), MyGUI::Align::Default);
+        skillNameWidget = mSkillView->createWidget<MyGUI::TextBox>(
+            "SandText", coord1 + MyGUI::IntSize(coord2.width, 0), MyGUI::Align::Default);
         skillNameWidget->setCaption(text);
         skillNameWidget->eventMouseWheel += MyGUI::newDelegate(this, &ReviewDialog::onMouseWheel);
 
         mSkillWidgets.push_back(skillNameWidget);
 
-        int lineHeight = MWBase::Environment::get().getWindowManager()->getFontHeight() + 2;
+        const int lineHeight = Settings::gui().mFontSize + 2;
         coord1.top += lineHeight;
         coord2.top += lineHeight;
     }
 
     void ReviewDialog::addItem(const ESM::Spell* spell, MyGUI::IntCoord& coord1, MyGUI::IntCoord& coord2)
     {
-        Widgets::MWSpellPtr widget = mSkillView->createWidget<Widgets::MWSpell>("MW_StatName", coord1 + MyGUI::IntSize(coord2.width, 0), MyGUI::Align::Default);
+        Widgets::MWSpellPtr widget = mSkillView->createWidget<Widgets::MWSpell>(
+            "MW_StatName", coord1 + MyGUI::IntSize(coord2.width, 0), MyGUI::Align::Default);
         widget->setSpellId(spell->mId);
         widget->setUserString("ToolTipType", "Spell");
-        widget->setUserString("Spell", spell->mId);
+        widget->setUserString("Spell", spell->mId.serialize());
         widget->eventMouseWheel += MyGUI::newDelegate(this, &ReviewDialog::onMouseWheel);
 
         mSkillWidgets.push_back(widget);
 
-        int lineHeight = MWBase::Environment::get().getWindowManager()->getFontHeight() + 2;
+        const int lineHeight = Settings::gui().mFontSize + 2;
         coord1.top += lineHeight;
         coord2.top += lineHeight;
     }
 
-    void ReviewDialog::addSkills(const SkillList &skills, const std::string &titleId, const std::string &titleDefault, MyGUI::IntCoord &coord1, MyGUI::IntCoord &coord2)
+    void ReviewDialog::addSkills(const std::vector<ESM::RefId>& skills, const std::string& titleId,
+        const std::string& titleDefault, MyGUI::IntCoord& coord1, MyGUI::IntCoord& coord2)
     {
         // Add a line separator if there are items above
         if (!mSkillWidgets.empty())
@@ -324,15 +346,24 @@ namespace MWGui
             addSeparator(coord1, coord2);
         }
 
-        addGroup(MWBase::Environment::get().getWindowManager()->getGameSettingString(titleId, titleDefault), coord1, coord2);
+        addGroup(
+            MWBase::Environment::get().getWindowManager()->getGameSettingString(titleId, titleDefault), coord1, coord2);
 
-        for (const int& skillId : skills)
+        for (const ESM::RefId& skillId : skills)
         {
-            if (skillId < 0 || skillId >= ESM::Skill::Length) // Skip unknown skill indexes
+            const ESM::Skill* skill = MWBase::Environment::get().getESMStore()->get<ESM::Skill>().search(skillId);
+            if (!skill) // Skip unknown skills
                 continue;
-            assert(skillId >= 0 && skillId < ESM::Skill::Length);
-            const std::string &skillNameId = ESM::Skill::sSkillNameIds[skillId];
-            const MWMechanics::SkillValue &stat = mSkillValues.find(skillId)->second;
+
+            auto skillValue = mSkillValues.find(skill->mId);
+            if (skillValue == mSkillValues.end())
+            {
+                Log(Debug::Error) << "Failed to update stats review window: can not find value for skill "
+                                  << skill->mId;
+                continue;
+            }
+
+            const MWMechanics::SkillValue& stat = skillValue->second;
             int base = stat.getBase();
             int modified = stat.getModified();
 
@@ -341,14 +372,15 @@ namespace MWGui
                 state = "increased";
             else if (modified < base)
                 state = "decreased";
-            MyGUI::TextBox* widget = addValueItem(MWBase::Environment::get().getWindowManager()->getGameSettingString(skillNameId, skillNameId), MyGUI::utility::toString(static_cast<int>(modified)), state, coord1, coord2);
+            MyGUI::TextBox* widget = addValueItem(
+                skill->mName, MyGUI::utility::toString(static_cast<int>(modified)), state, coord1, coord2);
 
-            for (int i=0; i<2; ++i)
+            for (int i = 0; i < 2; ++i)
             {
-                ToolTips::createSkillToolTip(mSkillWidgets[mSkillWidgets.size()-1-i], skillId);
+                ToolTips::createSkillToolTip(mSkillWidgets[mSkillWidgets.size() - 1 - i], skill->mId);
             }
 
-            mSkillWidgetMap[skillId] = widget;
+            mSkillWidgetMap[skill->mId] = widget;
         }
     }
 
@@ -374,80 +406,78 @@ namespace MWGui
             addSkills(mMiscSkills, "sSkillClassMisc", "Misc Skills", coord1, coord2);
 
         // starting spells
-        std::vector<std::string> spells;
+        std::vector<ESM::RefId> spells;
 
         const ESM::Race* race = nullptr;
         if (!mRaceId.empty())
-            race = MWBase::Environment::get().getWorld()->getStore().get<ESM::Race>().find(mRaceId);
+            race = MWBase::Environment::get().getESMStore()->get<ESM::Race>().find(mRaceId);
 
-        int skills[ESM::Skill::Length];
-        for (int i=0; i<ESM::Skill::Length; ++i)
-            skills[i] = mSkillValues.find(i)->second.getBase();
+        std::map<ESM::RefId, MWMechanics::AttributeValue> attributes;
+        for (const auto& [key, value] : mAttributeWidgets)
+            attributes[key] = value->getAttributeValue();
 
-        int attributes[ESM::Attribute::Length];
-        for (int i=0; i<ESM::Attribute::Length; ++i)
-            attributes[i] = mAttributeWidgets[i]->getAttributeValue().getBase();
-
-        std::vector<std::string> selectedSpells = MWMechanics::autoCalcPlayerSpells(skills, attributes, race);
-        for (std::string& spellId : selectedSpells)
+        std::vector<ESM::RefId> selectedSpells = MWMechanics::autoCalcPlayerSpells(mSkillValues, attributes, race);
+        for (ESM::RefId& spellId : selectedSpells)
         {
-            std::string lower = Misc::StringUtils::lowerCase(spellId);
-            if (std::find(spells.begin(), spells.end(), lower) == spells.end())
-                spells.push_back(lower);
+            if (std::find(spells.begin(), spells.end(), spellId) == spells.end())
+                spells.push_back(spellId);
         }
 
         if (race)
         {
-            for (const std::string& spellId : race->mPowers.mList)
+            for (const ESM::RefId& spellId : race->mPowers.mList)
             {
-                std::string lower = Misc::StringUtils::lowerCase(spellId);
-                if (std::find(spells.begin(), spells.end(), lower) == spells.end())
-                    spells.push_back(lower);
+                if (std::find(spells.begin(), spells.end(), spellId) == spells.end())
+                    spells.push_back(spellId);
             }
         }
 
         if (!mBirthSignId.empty())
         {
-            const ESM::BirthSign* sign = MWBase::Environment::get().getWorld()->getStore().get<ESM::BirthSign>().find(mBirthSignId);
-            for (const std::string& spellId : sign->mPowers.mList)
+            const ESM::BirthSign* sign
+                = MWBase::Environment::get().getESMStore()->get<ESM::BirthSign>().find(mBirthSignId);
+            for (const auto& spellId : sign->mPowers.mList)
             {
-                std::string lower = Misc::StringUtils::lowerCase(spellId);
-                if (std::find(spells.begin(), spells.end(), lower) == spells.end())
-                    spells.push_back(lower);
+                if (std::find(spells.begin(), spells.end(), spellId) == spells.end())
+                    spells.push_back(spellId);
             }
         }
 
         if (!mSkillWidgets.empty())
             addSeparator(coord1, coord2);
-        addGroup(MWBase::Environment::get().getWindowManager()->getGameSettingString("sTypeAbility", "Abilities"), coord1, coord2);
-        for (std::string& spellId : spells)
+        addGroup(MWBase::Environment::get().getWindowManager()->getGameSettingString("sTypeAbility", "Abilities"),
+            coord1, coord2);
+        for (auto& spellId : spells)
         {
-            const ESM::Spell* spell = MWBase::Environment::get().getWorld()->getStore().get<ESM::Spell>().find(spellId);
+            const ESM::Spell* spell = MWBase::Environment::get().getESMStore()->get<ESM::Spell>().find(spellId);
             if (spell->mData.mType == ESM::Spell::ST_Ability)
                 addItem(spell, coord1, coord2);
         }
 
         addSeparator(coord1, coord2);
-        addGroup(MWBase::Environment::get().getWindowManager()->getGameSettingString("sTypePower", "Powers"), coord1, coord2);
-        for (std::string& spellId : spells)
+        addGroup(MWBase::Environment::get().getWindowManager()->getGameSettingString("sTypePower", "Powers"), coord1,
+            coord2);
+        for (auto& spellId : spells)
         {
-            const ESM::Spell* spell = MWBase::Environment::get().getWorld()->getStore().get<ESM::Spell>().find(spellId);
+            const ESM::Spell* spell = MWBase::Environment::get().getESMStore()->get<ESM::Spell>().find(spellId);
             if (spell->mData.mType == ESM::Spell::ST_Power)
                 addItem(spell, coord1, coord2);
         }
 
         addSeparator(coord1, coord2);
-        addGroup(MWBase::Environment::get().getWindowManager()->getGameSettingString("sTypeSpell", "Spells"), coord1, coord2);
-        for (std::string& spellId : spells)
+        addGroup(MWBase::Environment::get().getWindowManager()->getGameSettingString("sTypeSpell", "Spells"), coord1,
+            coord2);
+        for (auto& spellId : spells)
         {
-            const ESM::Spell* spell = MWBase::Environment::get().getWorld()->getStore().get<ESM::Spell>().find(spellId);
+            const ESM::Spell* spell = MWBase::Environment::get().getESMStore()->get<ESM::Spell>().find(spellId);
             if (spell->mData.mType == ESM::Spell::ST_Spell)
                 addItem(spell, coord1, coord2);
         }
 
-        // Canvas size must be expressed with VScroll disabled, otherwise MyGUI would expand the scroll area when the scrollbar is hidden
+        // Canvas size must be expressed with VScroll disabled, otherwise MyGUI would expand the scroll area when the
+        // scrollbar is hidden
         mSkillView->setVisibleVScroll(false);
-        mSkillView->setCanvasSize (mSkillView->getWidth(), std::max(mSkillView->getHeight(), coord1.top));
+        mSkillView->setCanvasSize(mSkillView->getWidth(), std::max(mSkillView->getHeight(), coord1.top));
         mSkillView->setVisibleVScroll(true);
     }
 
@@ -485,10 +515,11 @@ namespace MWGui
 
     void ReviewDialog::onMouseWheel(MyGUI::Widget* _sender, int _rel)
     {
-        if (mSkillView->getViewOffset().top + _rel*0.3 > 0)
+        if (mSkillView->getViewOffset().top + _rel * 0.3 > 0)
             mSkillView->setViewOffset(MyGUI::IntPoint(0, 0));
         else
-            mSkillView->setViewOffset(MyGUI::IntPoint(0, static_cast<int>(mSkillView->getViewOffset().top + _rel*0.3)));
+            mSkillView->setViewOffset(
+                MyGUI::IntPoint(0, static_cast<int>(mSkillView->getViewOffset().top + _rel * 0.3)));
     }
 
 }
