@@ -1,10 +1,10 @@
 #ifndef MWGUI_REVIEW_H
 #define MWGUI_REVIEW_H
 
-#include <components/esm/attr.hpp>
-#include <components/esm3/loadclas.hpp>
-#include "windowbase.hpp"
 #include "widgets.hpp"
+#include "windowbase.hpp"
+#include <components/esm/refid.hpp>
+#include <components/esm3/loadclas.hpp>
 
 namespace ESM
 {
@@ -16,39 +16,39 @@ namespace MWGui
     class ReviewDialog : public WindowModal
     {
     public:
-        enum Dialogs {
+        enum Dialogs
+        {
             NAME_DIALOG,
             RACE_DIALOG,
             CLASS_DIALOG,
             BIRTHSIGN_DIALOG
         };
-        typedef std::vector<int> SkillList;
 
         ReviewDialog();
 
         bool exit() override { return false; }
 
-        void setPlayerName(const std::string &name);
-        void setRace(const std::string &raceId);
-        void setClass(const ESM::Class& class_);
-        void setBirthSign (const std::string &signId);
+        void setPlayerName(const std::string& name);
+        void setRace(const ESM::RefId& raceId);
+        void setClass(const ESM::Class& playerClass);
+        void setBirthSign(const ESM::RefId& signId);
 
         void setHealth(const MWMechanics::DynamicStat<float>& value);
         void setMagicka(const MWMechanics::DynamicStat<float>& value);
         void setFatigue(const MWMechanics::DynamicStat<float>& value);
 
-        void setAttribute(ESM::Attribute::AttributeID attributeId, const MWMechanics::AttributeValue& value);
+        void setAttribute(ESM::RefId attributeId, const MWMechanics::AttributeValue& value);
 
-        void configureSkills(const SkillList& major, const SkillList& minor);
-        void setSkillValue(ESM::Skill::SkillEnum skillId, const MWMechanics::SkillValue& value);
+        void configureSkills(const std::vector<ESM::RefId>& major, const std::vector<ESM::RefId>& minor);
+        void setSkillValue(ESM::RefId id, const MWMechanics::SkillValue& value);
 
         void onOpen() override;
 
         void onFrame(float duration) override;
 
         // Events
-        typedef MyGUI::delegates::CMultiDelegate0 EventHandle_Void;
-        typedef MyGUI::delegates::CMultiDelegate1<int> EventHandle_Int;
+        typedef MyGUI::delegates::MultiDelegate<> EventHandle_Void;
+        typedef MyGUI::delegates::MultiDelegate<int> EventHandle_Int;
 
         /** Event : Back button clicked.\n
         signature : void method()\n
@@ -74,12 +74,14 @@ namespace MWGui
         void onMouseWheel(MyGUI::Widget* _sender, int _rel);
 
     private:
-        void addSkills(const SkillList &skills, const std::string &titleId, const std::string &titleDefault, MyGUI::IntCoord &coord1, MyGUI::IntCoord &coord2);
-        void addSeparator(MyGUI::IntCoord &coord1, MyGUI::IntCoord &coord2);
-        void addGroup(const std::string &label, MyGUI::IntCoord &coord1, MyGUI::IntCoord &coord2);
-        MyGUI::TextBox* addValueItem(const std::string& text, const std::string &value, const std::string& state, MyGUI::IntCoord &coord1, MyGUI::IntCoord &coord2);
-        void addItem(const std::string& text, MyGUI::IntCoord &coord1, MyGUI::IntCoord &coord2);
-        void addItem(const ESM::Spell* spell, MyGUI::IntCoord &coord1, MyGUI::IntCoord &coord2);
+        void addSkills(const std::vector<ESM::RefId>& skills, const std::string& titleId,
+            const std::string& titleDefault, MyGUI::IntCoord& coord1, MyGUI::IntCoord& coord2);
+        void addSeparator(MyGUI::IntCoord& coord1, MyGUI::IntCoord& coord2);
+        void addGroup(std::string_view label, MyGUI::IntCoord& coord1, MyGUI::IntCoord& coord2);
+        MyGUI::TextBox* addValueItem(std::string_view text, const std::string& value, const std::string& state,
+            MyGUI::IntCoord& coord1, MyGUI::IntCoord& coord2);
+        void addItem(const std::string& text, MyGUI::IntCoord& coord1, MyGUI::IntCoord& coord2);
+        void addItem(const ESM::Spell* spell, MyGUI::IntCoord& coord1, MyGUI::IntCoord& coord2);
         void updateSkillArea();
 
         MyGUI::TextBox *mNameWidget, *mRaceWidget, *mClassWidget, *mBirthSignWidget;
@@ -87,13 +89,14 @@ namespace MWGui
 
         Widgets::MWDynamicStatPtr mHealth, mMagicka, mFatigue;
 
-        std::map<int, Widgets::MWAttributePtr> mAttributeWidgets;
+        std::map<ESM::RefId, Widgets::MWAttributePtr> mAttributeWidgets;
 
-        SkillList mMajorSkills, mMinorSkills, mMiscSkills;
-        std::map<int, MWMechanics::SkillValue > mSkillValues;
-        std::map<int, MyGUI::TextBox*> mSkillWidgetMap;
-        std::string mName, mRaceId, mBirthSignId;
-        ESM::Class mKlass;
+        std::vector<ESM::RefId> mMajorSkills, mMinorSkills, mMiscSkills;
+        std::map<ESM::RefId, MWMechanics::SkillValue> mSkillValues;
+        std::map<ESM::RefId, MyGUI::TextBox*> mSkillWidgetMap;
+        ESM::RefId mRaceId, mBirthSignId;
+        std::string mName;
+        ESM::Class mClass;
         std::vector<MyGUI::Widget*> mSkillWidgets; //< Skills and other information
 
         bool mUpdateSkillArea;
