@@ -1,30 +1,32 @@
 #include "savegamedialog.hpp"
 
-#include <sstream>
 #include <iomanip>
+#include <sstream>
 
 #include <MyGUI_ComboBox.h>
 #include <MyGUI_ImageBox.h>
 #include <MyGUI_InputManager.h>
 #include <MyGUI_LanguageManager.h>
+#include <MyGUI_UString.h>
 
-#include <osgDB/ReadFile>
 #include <osg/Texture2D>
+#include <osgDB/ReadFile>
 
 #include <components/debug/debuglog.hpp>
-
-#include <components/myguiplatform/myguitexture.hpp>
-
-#include <components/misc/stringops.hpp>
-
-#include <components/settings/settings.hpp>
-
+#include <components/esm3/loadclas.hpp>
+#include <components/files/conversion.hpp>
 #include <components/files/memorystream.hpp>
+#include <components/l10n/manager.hpp>
+#include <components/misc/strings/lower.hpp>
+#include <components/misc/timeconvert.hpp>
+#include <components/myguiplatform/myguitexture.hpp>
+#include <components/settings/values.hpp>
 
-#include "../mwbase/statemanager.hpp"
 #include "../mwbase/environment.hpp"
-#include "../mwbase/world.hpp"
+#include "../mwbase/statemanager.hpp"
 #include "../mwbase/windowmanager.hpp"
+#include "../mwbase/world.hpp"
+#include "../mwworld/datetimemanager.hpp"
 #include "../mwworld/esmstore.hpp"
 
 #include "../mwstate/character.hpp"
@@ -63,7 +65,7 @@ namespace MWGui
         mDeleteButton->setNeedKeyFocus(false);
     }
 
-    void SaveGameDialog::onSlotActivated(MyGUI::ListBox *sender, size_t pos)
+    void SaveGameDialog::onSlotActivated(MyGUI::ListBox* sender, size_t pos)
     {
         onSlotSelected(sender, pos);
         accept();
@@ -80,7 +82,7 @@ namespace MWGui
     void SaveGameDialog::confirmDeleteSave()
     {
         ConfirmationDialog* dialog = MWBase::Environment::get().getWindowManager()->getConfirmationDialog();
-        dialog->askForConfirmation("#{sMessage3}");
+        dialog->askForConfirmation("#{OMWEngine:DeleteGameConfirmation}");
         dialog->eventOkClicked.clear();
         dialog->eventOkClicked += MyGUI::newDelegate(this, &SaveGameDialog::onDeleteSlotConfirmed);
         dialog->eventCancelClicked.clear();
@@ -89,7 +91,7 @@ namespace MWGui
 
     void SaveGameDialog::onDeleteSlotConfirmed()
     {
-        MWBase::Environment::get().getStateManager()->deleteGame (mCurrentCharacter, mCurrentSlot);
+        MWBase::Environment::get().getStateManager()->deleteGame(mCurrentCharacter, mCurrentSlot);
         mSaveList->removeItemAt(mSaveList->getIndexSelected());
         onSlotSelected(mSaveList, mSaveList->getIndexSelected());
         MWBase::Environment::get().getWindowManager()->setKeyFocusWidget(mSaveList);
@@ -101,12 +103,12 @@ namespace MWGui
             mCharacterSelection->removeItemAt(previousIndex);
             if (mCharacterSelection->getItemCount())
             {
-                size_t nextCharacter = std::min(previousIndex, mCharacterSelection->getItemCount()-1);
+                size_t nextCharacter = std::min(previousIndex, mCharacterSelection->getItemCount() - 1);
                 mCharacterSelection->setIndexSelected(nextCharacter);
                 onCharacterSelected(mCharacterSelection, nextCharacter);
             }
             else
-                fillSaveList();
+                mCharacterSelection->setIndexSelected(MyGUI::ITEM_NONE);
         }
     }
 
@@ -115,14 +117,14 @@ namespace MWGui
         MWBase::Environment::get().getWindowManager()->setKeyFocusWidget(mSaveList);
     }
 
-    void SaveGameDialog::onSaveNameChanged(MyGUI::EditBox *sender)
+    void SaveGameDialog::onSaveNameChanged(MyGUI::EditBox* sender)
     {
         // This might have previously been a save slot from the list. If so, that is no longer the case
         mSaveList->setIndexSelected(MyGUI::ITEM_NONE);
         onSlotSelected(mSaveList, MyGUI::ITEM_NONE);
     }
 
-    void SaveGameDialog::onEditSelectAccept(MyGUI::EditBox *sender)
+    void SaveGameDialog::onEditSelectAccept(MyGUI::EditBox* sender)
     {
         accept();
 
@@ -141,7 +143,7 @@ namespace MWGui
     {
         WindowModal::onOpen();
 
-        mSaveNameEdit->setCaption ("");
+        mSaveNameEdit->setCaption({});
         if (mSaving)
             MWBase::Environment::get().getWindowManager()->setKeyFocusWidget(mSaveNameEdit);
         else
@@ -149,7 +151,7 @@ namespace MWGui
 
         center();
 
-        mCharacterSelection->setCaption("");
+        mCharacterSelection->setCaption({});
         mCharacterSelection->removeAllItems();
         mCurrentCharacter = nullptr;
         mCurrentSlot = nullptr;
@@ -162,54 +164,56 @@ namespace MWGui
 
         mCurrentCharacter = mgr->getCurrentCharacter();
 
-        std::string directory =
-            Misc::StringUtils::lowerCase (Settings::Manager::getString ("character", "Saves"));
+        const std::string& directory = Settings::saves().mCharacter;
 
         size_t selectedIndex = MyGUI::ITEM_NONE;
 
         for (MWBase::StateManager::CharacterIterator it = mgr->characterBegin(); it != mgr->characterEnd(); ++it)
         {
-            if (it->begin()!=it->end())
+            if (it->begin() != it->end())
             {
+                const ESM::SavedGame& signature = it->getSignature();
+
                 std::stringstream title;
-                title << it->getSignature().mPlayerName;
+                title << signature.mPlayerName;
 
                 // For a custom class, we will not find it in the store (unless we loaded the savegame first).
                 // Fall back to name stored in savegame header in that case.
-                std::string className;
-                if (it->getSignature().mPlayerClassId.empty())
-                    className = it->getSignature().mPlayerClassName;
+                std::string_view className;
+                if (signature.mPlayerClassId.empty())
+                    className = signature.mPlayerClassName;
                 else
                 {
                     // Find the localised name for this class from the store
-                    const ESM::Class* class_ = MWBase::Environment::get().getWorld()->getStore().get<ESM::Class>().search(
-                                it->getSignature().mPlayerClassId);
+                    const ESM::Class* class_
+                        = MWBase::Environment::get().getESMStore()->get<ESM::Class>().search(signature.mPlayerClassId);
                     if (class_)
                         className = class_->mName;
                     else
                         className = "?"; // From an older savegame format that did not support custom classes properly.
                 }
 
-                title << " (#{sLevel} " << it->getSignature().mPlayerLevel << " " << MyGUI::TextIterator::toTagsString(className) << ")";
+                title << " (#{OMWEngine:Level} " << signature.mPlayerLevel << " "
+                      << MyGUI::TextIterator::toTagsString(MyGUI::UString(className)) << ")";
 
-                mCharacterSelection->addItem (MyGUI::LanguageManager::getInstance().replaceTags(title.str()));
+                mCharacterSelection->addItem(MyGUI::LanguageManager::getInstance().replaceTags(title.str()));
 
-                if (mCurrentCharacter == &*it ||
-                    (!mCurrentCharacter && !mSaving && directory==Misc::StringUtils::lowerCase (
-                    it->begin()->mPath.parent_path().filename().string())))
+                if (mCurrentCharacter == &*it
+                    || (!mCurrentCharacter && !mSaving
+                        && Misc::StringUtils::ciEqual(
+                            directory, Files::pathToUnicodeString(it->begin()->mPath.parent_path().filename()))))
                 {
                     mCurrentCharacter = &*it;
-                    selectedIndex = mCharacterSelection->getItemCount()-1;
+                    selectedIndex = mCharacterSelection->getItemCount() - 1;
                 }
             }
         }
 
         mCharacterSelection->setIndexSelected(selectedIndex);
         if (selectedIndex == MyGUI::ITEM_NONE)
-            mCharacterSelection->setCaptionWithReplacing("#{SavegameMenu:SelectCharacter}");
+            mCharacterSelection->setCaptionWithReplacing("#{OMWEngine:SelectCharacter}");
 
         fillSaveList();
-
     }
 
     void SaveGameDialog::setLoadOrSave(bool load)
@@ -230,12 +234,12 @@ namespace MWGui
         center();
     }
 
-    void SaveGameDialog::onCancelButtonClicked(MyGUI::Widget *sender)
+    void SaveGameDialog::onCancelButtonClicked(MyGUI::Widget* sender)
     {
         setVisible(false);
     }
 
-    void SaveGameDialog::onDeleteButtonClicked(MyGUI::Widget *sender)
+    void SaveGameDialog::onDeleteButtonClicked(MyGUI::Widget* sender)
     {
         if (mCurrentSlot)
             confirmDeleteSave();
@@ -259,7 +263,7 @@ namespace MWGui
             if (mCurrentSlot != nullptr && !reallySure)
             {
                 ConfirmationDialog* dialog = MWBase::Environment::get().getWindowManager()->getConfirmationDialog();
-                dialog->askForConfirmation("#{sMessage4}");
+                dialog->askForConfirmation("#{OMWEngine:OverwriteGameConfirmation}");
                 dialog->eventOkClicked.clear();
                 dialog->eventOkClicked += MyGUI::newDelegate(this, &SaveGameDialog::onConfirmationGiven);
                 dialog->eventCancelClicked.clear();
@@ -268,7 +272,7 @@ namespace MWGui
             }
             if (mSaveNameEdit->getCaption().empty())
             {
-                MWBase::Environment::get().getWindowManager()->messageBox("#{sNotifyMessage65}");
+                MWBase::Environment::get().getWindowManager()->messageBox("#{OMWEngine:EmptySaveNameError}");
                 return;
             }
         }
@@ -280,7 +284,7 @@ namespace MWGui
             if (state == MWBase::StateManager::State_Running && !reallySure)
             {
                 ConfirmationDialog* dialog = MWBase::Environment::get().getWindowManager()->getConfirmationDialog();
-                dialog->askForConfirmation("#{sMessage1}");
+                dialog->askForConfirmation("#{OMWEngine:LoadGameConfirmation}");
                 dialog->eventOkClicked.clear();
                 dialog->eventOkClicked += MyGUI::newDelegate(this, &SaveGameDialog::onConfirmationGiven);
                 dialog->eventCancelClicked.clear();
@@ -290,16 +294,16 @@ namespace MWGui
         }
 
         setVisible(false);
-        MWBase::Environment::get().getWindowManager()->removeGuiMode (MWGui::GM_MainMenu);
+        MWBase::Environment::get().getWindowManager()->removeGuiMode(MWGui::GM_MainMenu);
 
         if (mSaving)
         {
-            MWBase::Environment::get().getStateManager()->saveGame (mSaveNameEdit->getCaption(), mCurrentSlot);
+            MWBase::Environment::get().getStateManager()->saveGame(mSaveNameEdit->getCaption(), mCurrentSlot);
         }
         else
         {
-            assert (mCurrentCharacter && mCurrentSlot);
-            MWBase::Environment::get().getStateManager()->loadGame (mCurrentCharacter, mCurrentSlot->mPath.string());
+            assert(mCurrentCharacter && mCurrentSlot);
+            MWBase::Environment::get().getStateManager()->loadGame(mCurrentCharacter, mCurrentSlot->mPath);
         }
     }
 
@@ -309,16 +313,16 @@ namespace MWGui
             confirmDeleteSave();
     }
 
-    void SaveGameDialog::onOkButtonClicked(MyGUI::Widget *sender)
+    void SaveGameDialog::onOkButtonClicked(MyGUI::Widget* sender)
     {
         accept();
     }
 
-    void SaveGameDialog::onCharacterSelected(MyGUI::ComboBox *sender, size_t pos)
+    void SaveGameDialog::onCharacterSelected(MyGUI::ComboBox* sender, size_t pos)
     {
         MWBase::StateManager* mgr = MWBase::Environment::get().getStateManager();
 
-        unsigned int i=0;
+        unsigned int i = 0;
         const MWState::Character* character = nullptr;
         for (MWBase::StateManager::CharacterIterator it = mgr->characterBegin(); it != mgr->characterEnd(); ++it, ++i)
         {
@@ -359,21 +363,26 @@ namespace MWGui
 
     std::string formatTimeplayed(const double timeInSeconds)
     {
-        int timePlayed = (int)floor(timeInSeconds);
-        int days = timePlayed / 60 / 60 / 24;
-        int hours = (timePlayed / 60 / 60) % 24;
-        int minutes = (timePlayed / 60) % 60;
-        int seconds = timePlayed % 60;
+        auto l10n = MWBase::Environment::get().getL10nManager()->getContext("Interface");
+        int duration = static_cast<int>(timeInSeconds);
+        if (duration <= 0)
+            return l10n->formatMessage("DurationSecond", { "seconds" }, { 0 });
 
-        std::stringstream stream;
-        stream << std::setfill('0') << std::setw(2) << days << ":";
-        stream << std::setfill('0') << std::setw(2) << hours << ":";
-        stream << std::setfill('0') << std::setw(2) << minutes << ":";
-        stream << std::setfill('0') << std::setw(2) << seconds;
-        return stream.str();
+        std::string result;
+        int hours = duration / 3600;
+        int minutes = (duration / 60) % 60;
+        int seconds = duration % 60;
+        if (hours)
+            result += l10n->formatMessage("DurationHour", { "hours" }, { hours });
+        if (minutes)
+            result += l10n->formatMessage("DurationMinute", { "minutes" }, { minutes });
+        if (seconds)
+            result += l10n->formatMessage("DurationSecond", { "seconds" }, { seconds });
+
+        return result;
     }
 
-    void SaveGameDialog::onSlotSelected(MyGUI::ListBox *sender, size_t pos)
+    void SaveGameDialog::onSlotSelected(MyGUI::ListBox* sender, size_t pos)
     {
         mOkButton->setEnabled(pos != MyGUI::ITEM_NONE || mSaving);
         mDeleteButton->setEnabled(pos != MyGUI::ITEM_NONE);
@@ -381,8 +390,8 @@ namespace MWGui
         if (pos == MyGUI::ITEM_NONE || !mCurrentCharacter)
         {
             mCurrentSlot = nullptr;
-            mInfoText->setCaption("");
-            mScreenshot->setImageTexture("");
+            mInfoText->setCaption({});
+            mScreenshot->setImageTexture({});
             return;
         }
 
@@ -390,8 +399,9 @@ namespace MWGui
             mSaveNameEdit->setCaption(sender->getItemNameAt(pos));
 
         mCurrentSlot = nullptr;
-        unsigned int i=0;
-        for (MWState::Character::SlotIterator it = mCurrentCharacter->begin(); it != mCurrentCharacter->end(); ++it, ++i)
+        unsigned int i = 0;
+        for (MWState::Character::SlotIterator it = mCurrentCharacter->begin(); it != mCurrentCharacter->end();
+             ++it, ++i)
         {
             if (i == pos)
                 mCurrentSlot = &*it;
@@ -400,52 +410,71 @@ namespace MWGui
             throw std::runtime_error("Can't find selected slot");
 
         std::stringstream text;
-        time_t time = mCurrentSlot->mTimeStamp;
-        struct tm* timeinfo;
-        timeinfo = localtime(&time);
 
-        text << std::put_time(timeinfo, "%Y.%m.%d %T") << "\n";
+        text << Misc::fileTimeToString(mCurrentSlot->mTimeStamp, "%Y.%m.%d %T") << "\n";
 
-        text << "#{sLevel} " << mCurrentSlot->mProfile.mPlayerLevel << "\n";
-        text << "#{sCell=" << mCurrentSlot->mProfile.mPlayerCell << "}\n";
+        if (mCurrentSlot->mProfile.mMaximumHealth > 0)
+            text << "#{OMWEngine:Health} " << static_cast<int>(mCurrentSlot->mProfile.mCurrentHealth) << "/"
+                 << static_cast<int>(mCurrentSlot->mProfile.mMaximumHealth) << "\n";
+
+        text << "#{OMWEngine:Level} " << mCurrentSlot->mProfile.mPlayerLevel << "\n";
+        text << "#{sCell=" << mCurrentSlot->mProfile.mPlayerCellName << "}\n";
 
         int hour = int(mCurrentSlot->mProfile.mInGameTime.mGameHour);
         bool pm = hour >= 12;
-        if (hour >= 13) hour -= 12;
-        if (hour == 0) hour = 12;
+        if (hour >= 13)
+            hour -= 12;
+        if (hour == 0)
+            hour = 12;
 
-        text
-            << mCurrentSlot->mProfile.mInGameTime.mDay << " "
-            << MWBase::Environment::get().getWorld()->getMonthName(mCurrentSlot->mProfile.mInGameTime.mMonth)
-            <<  " " << hour << " " << (pm ? "#{sSaveMenuHelp05}" : "#{sSaveMenuHelp04}");
+        if (mCurrentSlot->mProfile.mCurrentDay > 0)
+            text << "#{Calendar:day} " << mCurrentSlot->mProfile.mCurrentDay << "\n";
 
-        if (Settings::Manager::getBool("timeplayed","Saves"))
+        text << mCurrentSlot->mProfile.mInGameTime.mDay << " "
+             << MWBase::Environment::get().getWorld()->getTimeManager()->getMonthName(
+                    mCurrentSlot->mProfile.mInGameTime.mMonth)
+             << " " << hour << " " << (pm ? "#{Calendar:pm}" : "#{Calendar:am}");
+
+        if (mCurrentSlot->mProfile.mTimePlayed > 0)
         {
-            text << "\n" << "#{SavegameMenu:TimePlayed}: " << formatTimeplayed(mCurrentSlot->mProfile.mTimePlayed);
+            text << "\n"
+                 << "#{OMWEngine:TimePlayed}: " << formatTimeplayed(mCurrentSlot->mProfile.mTimePlayed);
         }
 
         mInfoText->setCaptionWithReplacing(text.str());
 
+        // Reset the image for the case we're unable to recover a screenshot
+        mScreenshotTexture.reset();
+        mScreenshot->setRenderItemTexture(nullptr);
+        mScreenshot->getSubWidgetMain()->_setUVSet(MyGUI::FloatRect(0.f, 0.f, 1.f, 1.f));
 
         // Decode screenshot
         const std::vector<char>& data = mCurrentSlot->mProfile.mScreenshot;
-        Files::IMemStream instream (&data[0], data.size());
+        if (!data.size())
+        {
+            Log(Debug::Warning) << "Selected save file '" << Files::pathToUnicodeString(mCurrentSlot->mPath.filename())
+                                << "' has no savegame screenshot";
+            return;
+        }
+
+        Files::IMemStream instream(data.data(), data.size());
 
         osgDB::ReaderWriter* readerwriter = osgDB::Registry::instance()->getReaderWriterForExtension("jpg");
         if (!readerwriter)
         {
-            Log(Debug::Error) << "Error: Can't open savegame screenshot, no jpg readerwriter found";
+            Log(Debug::Error) << "Can't open savegame screenshot, no jpg readerwriter found";
             return;
         }
 
         osgDB::ReaderWriter::ReadResult result = readerwriter->readImage(instream);
         if (!result.success())
         {
-            Log(Debug::Error) << "Error: Failed to read savegame screenshot: " << result.message() << " code " << result.status();
+            Log(Debug::Error) << "Failed to read savegame screenshot: " << result.message() << " code "
+                              << result.status();
             return;
         }
 
-        osg::ref_ptr<osg::Texture2D> texture (new osg::Texture2D);
+        osg::ref_ptr<osg::Texture2D> texture(new osg::Texture2D);
         texture->setImage(result.getImage());
         texture->setInternalFormat(GL_RGB);
         texture->setWrap(osg::Texture::WRAP_S, osg::Texture::CLAMP_TO_EDGE);
@@ -456,8 +485,6 @@ namespace MWGui
         texture->setUnRefImageDataAfterApply(true);
 
         mScreenshotTexture = std::make_unique<osgMyGUI::OSGTexture>(texture);
-
         mScreenshot->setRenderItemTexture(mScreenshotTexture.get());
-        mScreenshot->getSubWidgetMain()->_setUVSet(MyGUI::FloatRect(0.f, 0.f, 1.f, 1.f));
     }
 }
