@@ -1,43 +1,43 @@
 #include "spellcreationdialog.hpp"
 
 #include <MyGUI_Button.h>
-#include <MyGUI_ImageBox.h>
 #include <MyGUI_Gui.h>
+#include <MyGUI_ImageBox.h>
 #include <MyGUI_ScrollBar.h>
 
-#include <components/esm/records.hpp>
-#include <components/widgets/list.hpp>
 #include <components/misc/resourcehelpers.hpp>
 #include <components/resource/resourcesystem.hpp>
+#include <components/widgets/list.hpp>
+
+#include <components/esm3/loadgmst.hpp>
 
-#include "../mwbase/windowmanager.hpp"
-#include "../mwbase/mechanicsmanager.hpp"
 #include "../mwbase/environment.hpp"
-#include "../mwbase/world.hpp"
+#include "../mwbase/mechanicsmanager.hpp"
+#include "../mwbase/windowmanager.hpp"
 
-#include "../mwworld/containerstore.hpp"
 #include "../mwworld/class.hpp"
-#include "../mwworld/store.hpp"
+#include "../mwworld/containerstore.hpp"
 #include "../mwworld/esmstore.hpp"
+#include "../mwworld/store.hpp"
 
-#include "../mwmechanics/spellutil.hpp"
 #include "../mwmechanics/actorutil.hpp"
 #include "../mwmechanics/creaturestats.hpp"
+#include "../mwmechanics/spellutil.hpp"
 
-#include "tooltips.hpp"
 #include "class.hpp"
+#include "tooltips.hpp"
 #include "widgets.hpp"
 
 namespace
 {
 
-    bool sortMagicEffects (short id1, short id2)
+    bool sortMagicEffects(short id1, short id2)
     {
-        const MWWorld::Store<ESM::GameSetting> &gmst =
-            MWBase::Environment::get().getWorld()->getStore().get<ESM::GameSetting>();
+        const MWWorld::Store<ESM::GameSetting>& gmst
+            = MWBase::Environment::get().getESMStore()->get<ESM::GameSetting>();
 
-        return gmst.find(ESM::MagicEffect::effectIdToString (id1))->mValue.getString()
-                < gmst.find(ESM::MagicEffect::effectIdToString  (id2))->mValue.getString();
+        return gmst.find(ESM::MagicEffect::indexToGmstString(id1))->mValue.getString()
+            < gmst.find(ESM::MagicEffect::indexToGmstString(id2))->mValue.getString();
     }
 
     void init(ESM::ENAMstruct& effect)
@@ -89,8 +89,10 @@ namespace MWGui
         mCancelButton->eventMouseButtonClick += MyGUI::newDelegate(this, &EditEffectDialog::onCancelButtonClicked);
         mDeleteButton->eventMouseButtonClick += MyGUI::newDelegate(this, &EditEffectDialog::onDeleteButtonClicked);
 
-        mMagnitudeMinSlider->eventScrollChangePosition += MyGUI::newDelegate(this, &EditEffectDialog::onMagnitudeMinChanged);
-        mMagnitudeMaxSlider->eventScrollChangePosition += MyGUI::newDelegate(this, &EditEffectDialog::onMagnitudeMaxChanged);
+        mMagnitudeMinSlider->eventScrollChangePosition
+            += MyGUI::newDelegate(this, &EditEffectDialog::onMagnitudeMinChanged);
+        mMagnitudeMaxSlider->eventScrollChangePosition
+            += MyGUI::newDelegate(this, &EditEffectDialog::onMagnitudeMaxChanged);
         mDurationSlider->eventScrollChangePosition += MyGUI::newDelegate(this, &EditEffectDialog::onDurationChanged);
         mAreaSlider->eventScrollChangePosition += MyGUI::newDelegate(this, &EditEffectDialog::onAreaChanged);
     }
@@ -108,22 +110,22 @@ namespace MWGui
 
     bool EditEffectDialog::exit()
     {
-        if(mEditing)
+        if (mEditing)
             eventEffectModified(mOldEffect);
         else
             eventEffectRemoved(mEffect);
         return true;
     }
 
-    void EditEffectDialog::newEffect (const ESM::MagicEffect *effect)
+    void EditEffectDialog::newEffect(const ESM::MagicEffect* effect)
     {
-        bool allowSelf = (effect->mData.mFlags & ESM::MagicEffect::CastSelf) != 0;
+        bool allowSelf = (effect->mData.mFlags & ESM::MagicEffect::CastSelf) != 0 || mConstantEffect;
         bool allowTouch = (effect->mData.mFlags & ESM::MagicEffect::CastTouch) && !mConstantEffect;
 
         setMagicEffect(effect);
         mEditing = false;
 
-        mDeleteButton->setVisible (false);
+        mDeleteButton->setVisible(false);
 
         mEffect.mRange = ESM::RT_Self;
         if (!allowSelf)
@@ -140,14 +142,14 @@ namespace MWGui
 
         onRangeButtonClicked(mRangeButton);
 
-        mMagnitudeMinSlider->setScrollPosition (0);
-        mMagnitudeMaxSlider->setScrollPosition (0);
-        mAreaSlider->setScrollPosition (0);
-        mDurationSlider->setScrollPosition (0);
+        mMagnitudeMinSlider->setScrollPosition(0);
+        mMagnitudeMaxSlider->setScrollPosition(0);
+        mAreaSlider->setScrollPosition(0);
+        mDurationSlider->setScrollPosition(0);
 
         mDurationValue->setCaption("1");
         mMagnitudeMinValue->setCaption("1");
-        const std::string to = MWBase::Environment::get().getWindowManager()->getGameSettingString("sTo", "-");
+        const std::string to{ MWBase::Environment::get().getWindowManager()->getGameSettingString("sTo", "-") };
 
         mMagnitudeMaxValue->setCaption(to + " 1");
         mAreaValue->setCaption("0");
@@ -155,45 +157,45 @@ namespace MWGui
         setVisible(true);
     }
 
-    void EditEffectDialog::editEffect (ESM::ENAMstruct effect)
+    void EditEffectDialog::editEffect(ESM::ENAMstruct effect)
     {
-        const ESM::MagicEffect* magicEffect =
-            MWBase::Environment::get().getWorld()->getStore().get<ESM::MagicEffect>().find(effect.mEffectID);
+        const ESM::MagicEffect* magicEffect
+            = MWBase::Environment::get().getESMStore()->get<ESM::MagicEffect>().find(effect.mEffectID);
 
         setMagicEffect(magicEffect);
         mOldEffect = effect;
         mEffect = effect;
         mEditing = true;
 
-        mDeleteButton->setVisible (true);
+        mDeleteButton->setVisible(true);
 
-        mMagnitudeMinSlider->setScrollPosition (effect.mMagnMin-1);
-        mMagnitudeMaxSlider->setScrollPosition (effect.mMagnMax-1);
-        mAreaSlider->setScrollPosition (effect.mArea);
-        mDurationSlider->setScrollPosition (effect.mDuration-1);
+        mMagnitudeMinSlider->setScrollPosition(effect.mMagnMin - 1);
+        mMagnitudeMaxSlider->setScrollPosition(effect.mMagnMax - 1);
+        mAreaSlider->setScrollPosition(effect.mArea);
+        mDurationSlider->setScrollPosition(effect.mDuration - 1);
 
         if (mEffect.mRange == ESM::RT_Self)
-            mRangeButton->setCaptionWithReplacing ("#{sRangeSelf}");
+            mRangeButton->setCaptionWithReplacing("#{sRangeSelf}");
         else if (mEffect.mRange == ESM::RT_Target)
-            mRangeButton->setCaptionWithReplacing ("#{sRangeTarget}");
+            mRangeButton->setCaptionWithReplacing("#{sRangeTarget}");
         else if (mEffect.mRange == ESM::RT_Touch)
-            mRangeButton->setCaptionWithReplacing ("#{sRangeTouch}");
+            mRangeButton->setCaptionWithReplacing("#{sRangeTouch}");
 
-        onMagnitudeMinChanged (mMagnitudeMinSlider, effect.mMagnMin-1);
-        onMagnitudeMaxChanged (mMagnitudeMinSlider, effect.mMagnMax-1);
-        onAreaChanged (mAreaSlider, effect.mArea);
-        onDurationChanged (mDurationSlider, effect.mDuration-1);
+        onMagnitudeMinChanged(mMagnitudeMinSlider, effect.mMagnMin - 1);
+        onMagnitudeMaxChanged(mMagnitudeMinSlider, effect.mMagnMax - 1);
+        onAreaChanged(mAreaSlider, effect.mArea);
+        onDurationChanged(mDurationSlider, effect.mDuration - 1);
         eventEffectModified(mEffect);
 
         updateBoxes();
     }
 
-    void EditEffectDialog::setMagicEffect (const ESM::MagicEffect *effect)
+    void EditEffectDialog::setMagicEffect(const ESM::MagicEffect* effect)
     {
-        mEffectImage->setImageTexture(Misc::ResourceHelpers::correctIconPath(effect->mIcon,
-            MWBase::Environment::get().getResourceSystem()->getVFS()));
+        mEffectImage->setImageTexture(Misc::ResourceHelpers::correctIconPath(
+            effect->mIcon, MWBase::Environment::get().getResourceSystem()->getVFS()));
 
-        mEffectName->setCaptionWithReplacing("#{"+ESM::MagicEffect::effectIdToString  (effect->mIndex)+"}");
+        mEffectName->setCaptionWithReplacing("#{" + ESM::MagicEffect::indexToGmstString(effect->mIndex) + "}");
 
         mEffect.mEffectID = effect->mIndex;
 
@@ -207,129 +209,131 @@ namespace MWGui
         static int startY = mMagnitudeBox->getPosition().top;
         int curY = startY;
 
-        mMagnitudeBox->setVisible (false);
-        mDurationBox->setVisible (false);
-        mAreaBox->setVisible (false);
+        mMagnitudeBox->setVisible(false);
+        mDurationBox->setVisible(false);
+        mAreaBox->setVisible(false);
 
         if (!(mMagicEffect->mData.mFlags & ESM::MagicEffect::NoMagnitude))
         {
             mMagnitudeBox->setPosition(mMagnitudeBox->getPosition().left, curY);
-            mMagnitudeBox->setVisible (true);
+            mMagnitudeBox->setVisible(true);
             curY += mMagnitudeBox->getSize().height;
         }
-        if (!(mMagicEffect->mData.mFlags & ESM::MagicEffect::NoDuration)&&mConstantEffect==false)
+        if (!(mMagicEffect->mData.mFlags & ESM::MagicEffect::NoDuration) && mConstantEffect == false)
         {
             mDurationBox->setPosition(mDurationBox->getPosition().left, curY);
-            mDurationBox->setVisible (true);
+            mDurationBox->setVisible(true);
             curY += mDurationBox->getSize().height;
         }
         if (mEffect.mRange != ESM::RT_Self)
         {
             mAreaBox->setPosition(mAreaBox->getPosition().left, curY);
-            mAreaBox->setVisible (true);
-            //curY += mAreaBox->getSize().height;
+            mAreaBox->setVisible(true);
+            // curY += mAreaBox->getSize().height;
         }
     }
 
-    void EditEffectDialog::onRangeButtonClicked (MyGUI::Widget* sender)
+    void EditEffectDialog::onRangeButtonClicked(MyGUI::Widget* sender)
     {
-        mEffect.mRange = (mEffect.mRange+1)%3;
+        mEffect.mRange = (mEffect.mRange + 1) % 3;
 
         // cycle through range types until we find something that's allowed
-        // does not handle the case where nothing is allowed (this should be prevented before opening the Add Effect dialog)
-        bool allowSelf = (mMagicEffect->mData.mFlags & ESM::MagicEffect::CastSelf) != 0;
+        // does not handle the case where nothing is allowed (this should be prevented before opening the Add Effect
+        // dialog)
+        bool allowSelf = (mMagicEffect->mData.mFlags & ESM::MagicEffect::CastSelf) != 0 || mConstantEffect;
         bool allowTouch = (mMagicEffect->mData.mFlags & ESM::MagicEffect::CastTouch) && !mConstantEffect;
         bool allowTarget = (mMagicEffect->mData.mFlags & ESM::MagicEffect::CastTarget) && !mConstantEffect;
         if (mEffect.mRange == ESM::RT_Self && !allowSelf)
-            mEffect.mRange = (mEffect.mRange+1)%3;
+            mEffect.mRange = (mEffect.mRange + 1) % 3;
         if (mEffect.mRange == ESM::RT_Touch && !allowTouch)
-            mEffect.mRange = (mEffect.mRange+1)%3;
+            mEffect.mRange = (mEffect.mRange + 1) % 3;
         if (mEffect.mRange == ESM::RT_Target && !allowTarget)
-            mEffect.mRange = (mEffect.mRange+1)%3;
+            mEffect.mRange = (mEffect.mRange + 1) % 3;
 
-        if(mEffect.mRange == ESM::RT_Self)
+        if (mEffect.mRange == ESM::RT_Self)
         {
             mAreaSlider->setScrollPosition(0);
-            onAreaChanged(mAreaSlider,0);
+            onAreaChanged(mAreaSlider, 0);
         }
 
         if (mEffect.mRange == ESM::RT_Self)
-            mRangeButton->setCaptionWithReplacing ("#{sRangeSelf}");
+            mRangeButton->setCaptionWithReplacing("#{sRangeSelf}");
         else if (mEffect.mRange == ESM::RT_Target)
-            mRangeButton->setCaptionWithReplacing ("#{sRangeTarget}");
+            mRangeButton->setCaptionWithReplacing("#{sRangeTarget}");
         else if (mEffect.mRange == ESM::RT_Touch)
-            mRangeButton->setCaptionWithReplacing ("#{sRangeTouch}");
+            mRangeButton->setCaptionWithReplacing("#{sRangeTouch}");
 
         updateBoxes();
         eventEffectModified(mEffect);
     }
 
-    void EditEffectDialog::onDeleteButtonClicked (MyGUI::Widget* sender)
+    void EditEffectDialog::onDeleteButtonClicked(MyGUI::Widget* sender)
     {
         setVisible(false);
 
         eventEffectRemoved(mEffect);
     }
 
-    void EditEffectDialog::onOkButtonClicked (MyGUI::Widget* sender)
+    void EditEffectDialog::onOkButtonClicked(MyGUI::Widget* sender)
     {
         setVisible(false);
     }
 
-    void EditEffectDialog::onCancelButtonClicked (MyGUI::Widget* sender)
+    void EditEffectDialog::onCancelButtonClicked(MyGUI::Widget* sender)
     {
         setVisible(false);
         exit();
     }
 
-    void EditEffectDialog::setSkill (int skill)
+    void EditEffectDialog::setSkill(ESM::RefId skill)
     {
-        mEffect.mSkill = skill;
+        mEffect.mSkill = ESM::Skill::refIdToIndex(skill);
         eventEffectModified(mEffect);
     }
 
-    void EditEffectDialog::setAttribute (int attribute)
+    void EditEffectDialog::setAttribute(ESM::RefId attribute)
     {
-        mEffect.mAttribute = attribute;
+        mEffect.mAttribute = ESM::Attribute::refIdToIndex(attribute);
         eventEffectModified(mEffect);
     }
 
-    void EditEffectDialog::onMagnitudeMinChanged (MyGUI::ScrollBar* sender, size_t pos)
+    void EditEffectDialog::onMagnitudeMinChanged(MyGUI::ScrollBar* sender, size_t pos)
     {
-        mMagnitudeMinValue->setCaption(MyGUI::utility::toString(pos+1));
-        mEffect.mMagnMin = pos+1;
+        mMagnitudeMinValue->setCaption(MyGUI::utility::toString(pos + 1));
+        mEffect.mMagnMin = pos + 1;
 
         // trigger the check again (see below)
-        onMagnitudeMaxChanged(mMagnitudeMaxSlider, mMagnitudeMaxSlider->getScrollPosition ());
+        onMagnitudeMaxChanged(mMagnitudeMaxSlider, mMagnitudeMaxSlider->getScrollPosition());
         eventEffectModified(mEffect);
     }
 
-    void EditEffectDialog::onMagnitudeMaxChanged (MyGUI::ScrollBar* sender, size_t pos)
+    void EditEffectDialog::onMagnitudeMaxChanged(MyGUI::ScrollBar* sender, size_t pos)
     {
         // make sure the max value is actually larger or equal than the min value
-        size_t magnMin = std::abs(mEffect.mMagnMin); // should never be < 0, this is just here to avoid the compiler warning
-        if (pos+1 < magnMin)
+        size_t magnMin
+            = std::abs(mEffect.mMagnMin); // should never be < 0, this is just here to avoid the compiler warning
+        if (pos + 1 < magnMin)
         {
-            pos = mEffect.mMagnMin-1;
-            sender->setScrollPosition (pos);
+            pos = mEffect.mMagnMin - 1;
+            sender->setScrollPosition(pos);
         }
 
-        mEffect.mMagnMax = pos+1;
-        const std::string to = MWBase::Environment::get().getWindowManager()->getGameSettingString("sTo", "-");
+        mEffect.mMagnMax = pos + 1;
+        const std::string to{ MWBase::Environment::get().getWindowManager()->getGameSettingString("sTo", "-") };
 
-        mMagnitudeMaxValue->setCaption(to + " " + MyGUI::utility::toString(pos+1));
+        mMagnitudeMaxValue->setCaption(to + " " + MyGUI::utility::toString(pos + 1));
 
         eventEffectModified(mEffect);
     }
 
-    void EditEffectDialog::onDurationChanged (MyGUI::ScrollBar* sender, size_t pos)
+    void EditEffectDialog::onDurationChanged(MyGUI::ScrollBar* sender, size_t pos)
     {
-        mDurationValue->setCaption(MyGUI::utility::toString(pos+1));
-        mEffect.mDuration = pos+1;
+        mDurationValue->setCaption(MyGUI::utility::toString(pos + 1));
+        mEffect.mDuration = pos + 1;
         eventEffectModified(mEffect);
     }
 
-    void EditEffectDialog::onAreaChanged (MyGUI::ScrollBar* sender, size_t pos)
+    void EditEffectDialog::onAreaChanged(MyGUI::ScrollBar* sender, size_t pos)
     {
         mAreaValue->setCaption(MyGUI::utility::toString(pos));
         mEffect.mArea = pos;
@@ -358,36 +362,39 @@ namespace MWGui
         setWidgets(mAvailableEffectsList, mUsedEffectsView);
     }
 
-    void SpellCreationDialog::setPtr (const MWWorld::Ptr& actor)
+    void SpellCreationDialog::setPtr(const MWWorld::Ptr& actor)
     {
+        if (actor.isEmpty() || !actor.getClass().isActor())
+            throw std::runtime_error("Invalid argument in SpellCreationDialog::setPtr");
+
         mPtr = actor;
-        mNameEdit->setCaption("");
+        mNameEdit->setCaption({});
 
         startEditing();
     }
 
-    void SpellCreationDialog::onCancelButtonClicked (MyGUI::Widget* sender)
+    void SpellCreationDialog::onCancelButtonClicked(MyGUI::Widget* sender)
     {
-        MWBase::Environment::get().getWindowManager()->removeGuiMode (MWGui::GM_SpellCreation);
+        MWBase::Environment::get().getWindowManager()->removeGuiMode(MWGui::GM_SpellCreation);
     }
 
-    void SpellCreationDialog::onBuyButtonClicked (MyGUI::Widget* sender)
+    void SpellCreationDialog::onBuyButtonClicked(MyGUI::Widget* sender)
     {
         if (mEffects.size() <= 0)
         {
-            MWBase::Environment::get().getWindowManager()->messageBox ("#{sNotifyMessage30}");
+            MWBase::Environment::get().getWindowManager()->messageBox("#{sNotifyMessage30}");
             return;
         }
 
-        if (mNameEdit->getCaption () == "")
+        if (mNameEdit->getCaption().empty())
         {
-            MWBase::Environment::get().getWindowManager()->messageBox ("#{sNotifyMessage10}");
+            MWBase::Environment::get().getWindowManager()->messageBox("#{sNotifyMessage10}");
             return;
         }
 
         if (mMagickaCost->getCaption() == "0")
         {
-            MWBase::Environment::get().getWindowManager()->messageBox ("#{sEnchantmentMenu8}");
+            MWBase::Environment::get().getWindowManager()->messageBox("#{sEnchantmentMenu8}");
             return;
         }
 
@@ -397,30 +404,30 @@ namespace MWGui
         int price = MyGUI::utility::parseInt(mPriceLabel->getCaption());
         if (price > playerGold)
         {
-            MWBase::Environment::get().getWindowManager()->messageBox ("#{sNotifyMessage18}");
+            MWBase::Environment::get().getWindowManager()->messageBox("#{sNotifyMessage18}");
             return;
         }
 
         mSpell.mName = mNameEdit->getCaption();
 
-        player.getClass().getContainerStore(player).remove(MWWorld::ContainerStore::sGoldId, price, player);
+        player.getClass().getContainerStore(player).remove(MWWorld::ContainerStore::sGoldId, price);
 
         // add gold to NPC trading gold pool
         MWMechanics::CreatureStats& npcStats = mPtr.getClass().getCreatureStats(mPtr);
         npcStats.setGoldPool(npcStats.getGoldPool() + price);
 
-        MWBase::Environment::get().getWindowManager()->playSound ("Mysticism Hit");
+        MWBase::Environment::get().getWindowManager()->playSound(ESM::RefId::stringRefId("Mysticism Hit"));
 
-        const ESM::Spell* spell = MWBase::Environment::get().getWorld()->createRecord(mSpell);
+        const ESM::Spell* spell = MWBase::Environment::get().getESMStore()->insert(mSpell);
 
         MWMechanics::CreatureStats& stats = player.getClass().getCreatureStats(player);
         MWMechanics::Spells& spells = stats.getSpells();
-        spells.add (spell->mId);
+        spells.add(spell->mId);
 
-        MWBase::Environment::get().getWindowManager()->removeGuiMode (GM_SpellCreation);
+        MWBase::Environment::get().getWindowManager()->removeGuiMode(GM_SpellCreation);
     }
 
-    void SpellCreationDialog::onAccept(MyGUI::EditBox *sender)
+    void SpellCreationDialog::onAccept(MyGUI::EditBox* sender)
     {
         onBuyButtonClicked(sender);
 
@@ -434,13 +441,13 @@ namespace MWGui
         MWBase::Environment::get().getWindowManager()->setKeyFocusWidget(mNameEdit);
     }
 
-    void SpellCreationDialog::onReferenceUnavailable ()
+    void SpellCreationDialog::onReferenceUnavailable()
     {
-        MWBase::Environment::get().getWindowManager()->removeGuiMode (GM_Dialogue);
-        MWBase::Environment::get().getWindowManager()->removeGuiMode (GM_SpellCreation);
+        MWBase::Environment::get().getWindowManager()->removeGuiMode(GM_Dialogue);
+        MWBase::Environment::get().getWindowManager()->removeGuiMode(GM_SpellCreation);
     }
 
-    void SpellCreationDialog::notifyEffectsChanged ()
+    void SpellCreationDialog::notifyEffectsChanged()
     {
         if (mEffects.empty())
         {
@@ -452,28 +459,25 @@ namespace MWGui
 
         float y = 0;
 
-        const MWWorld::ESMStore &store =
-            MWBase::Environment::get().getWorld()->getStore();
+        const MWWorld::ESMStore& store = *MWBase::Environment::get().getESMStore();
 
         for (const ESM::ENAMstruct& effect : mEffects)
         {
-            y += std::max(1.f, MWMechanics::calcEffectCost(effect, nullptr, MWMechanics::EffectCostMethod::PlayerSpell));
+            y += std::max(
+                1.f, MWMechanics::calcEffectCost(effect, nullptr, MWMechanics::EffectCostMethod::PlayerSpell));
 
             if (effect.mRange == ESM::RT_Target)
                 y *= 1.5;
         }
 
-        ESM::EffectList effectList;
-        effectList.mList = mEffects;
-        mSpell.mEffects = effectList;
+        mSpell.mEffects.populate(mEffects);
         mSpell.mData.mCost = int(y);
         mSpell.mData.mType = ESM::Spell::ST_Spell;
         mSpell.mData.mFlags = 0;
 
         mMagickaCost->setCaption(MyGUI::utility::toString(int(y)));
 
-        float fSpellMakingValueMult =
-            store.get<ESM::GameSetting>().find("fSpellMakingValueMult")->mValue.getFloat();
+        float fSpellMakingValueMult = store.get<ESM::GameSetting>().find("fSpellMakingValueMult")->mValue.getFloat();
 
         int price = std::max(1, static_cast<int>(y * fSpellMakingValueMult));
         price = MWBase::Environment::get().getMechanicsManager()->getBarterOffer(mPtr, price, true);
@@ -488,13 +492,10 @@ namespace MWGui
 
     // ------------------------------------------------------------------------------------------------
 
-
     EffectEditorBase::EffectEditorBase(Type type)
         : mAvailableEffectsList(nullptr)
         , mUsedEffectsView(nullptr)
         , mAddEffectDialog()
-        , mSelectAttributeDialog(nullptr)
-        , mSelectSkillDialog(nullptr)
         , mSelectedEffect(0)
         , mSelectedKnownEffectId(0)
         , mConstantEffect(false)
@@ -504,14 +505,12 @@ namespace MWGui
         mAddEffectDialog.eventEffectModified += MyGUI::newDelegate(this, &EffectEditorBase::onEffectModified);
         mAddEffectDialog.eventEffectRemoved += MyGUI::newDelegate(this, &EffectEditorBase::onEffectRemoved);
 
-        mAddEffectDialog.setVisible (false);
+        mAddEffectDialog.setVisible(false);
     }
 
-    EffectEditorBase::~EffectEditorBase()
-    {
-    }
+    EffectEditorBase::~EffectEditorBase() {}
 
-    void EffectEditorBase::startEditing ()
+    void EffectEditorBase::startEditing()
     {
         // get the list of magic effects that are known to the player
 
@@ -527,90 +526,95 @@ namespace MWGui
             if (spell->mData.mType != ESM::Spell::ST_Spell)
                 continue;
 
-            for (const ESM::ENAMstruct& effectInfo : spell->mEffects.mList)
+            for (const ESM::IndexedENAMstruct& effectInfo : spell->mEffects.mList)
             {
-                const ESM::MagicEffect * effect = MWBase::Environment::get().getWorld()->getStore().get<ESM::MagicEffect>().find(effectInfo.mEffectID);
+                int16_t effectId = effectInfo.mData.mEffectID;
+                const ESM::MagicEffect* effect
+                    = MWBase::Environment::get().getESMStore()->get<ESM::MagicEffect>().find(effectId);
 
                 // skip effects that do not allow spellmaking/enchanting
-                int requiredFlags = (mType == Spellmaking) ? ESM::MagicEffect::AllowSpellmaking : ESM::MagicEffect::AllowEnchanting;
+                int requiredFlags
+                    = (mType == Spellmaking) ? ESM::MagicEffect::AllowSpellmaking : ESM::MagicEffect::AllowEnchanting;
                 if (!(effect->mData.mFlags & requiredFlags))
                     continue;
 
-                if (std::find(knownEffects.begin(), knownEffects.end(), effectInfo.mEffectID) == knownEffects.end())
-                    knownEffects.push_back(effectInfo.mEffectID);
+                if (std::find(knownEffects.begin(), knownEffects.end(), effectId) == knownEffects.end())
+                    knownEffects.push_back(effectId);
             }
         }
 
         std::sort(knownEffects.begin(), knownEffects.end(), sortMagicEffects);
 
-        mAvailableEffectsList->clear ();
+        mAvailableEffectsList->clear();
 
-        int i=0;
+        int i = 0;
         for (const short effectId : knownEffects)
         {
-            mAvailableEffectsList->addItem(MWBase::Environment::get().getWorld ()->getStore ().get<ESM::GameSetting>().find(
-                                               ESM::MagicEffect::effectIdToString(effectId))->mValue.getString());
+            mAvailableEffectsList->addItem(MWBase::Environment::get()
+                                               .getESMStore()
+                                               ->get<ESM::GameSetting>()
+                                               .find(ESM::MagicEffect::indexToGmstString(effectId))
+                                               ->mValue.getString());
             mButtonMapping[i] = effectId;
             ++i;
         }
-        mAvailableEffectsList->adjustSize ();
+        mAvailableEffectsList->adjustSize();
         mAvailableEffectsList->scrollToTop();
 
         for (const short effectId : knownEffects)
         {
-            std::string name = MWBase::Environment::get().getWorld ()->getStore ().get<ESM::GameSetting>().find(
-                                               ESM::MagicEffect::effectIdToString(effectId))->mValue.getString();
+            const std::string& name = MWBase::Environment::get()
+                                          .getESMStore()
+                                          ->get<ESM::GameSetting>()
+                                          .find(ESM::MagicEffect::indexToGmstString(effectId))
+                                          ->mValue.getString();
             MyGUI::Widget* w = mAvailableEffectsList->getItemWidget(name);
 
-            ToolTips::createMagicEffectToolTip (w, effectId);
+            ToolTips::createMagicEffectToolTip(w, effectId);
         }
 
         mEffects.clear();
-        updateEffectsView ();
+        updateEffectsView();
     }
 
-    void EffectEditorBase::setWidgets (Gui::MWList *availableEffectsList, MyGUI::ScrollView *usedEffectsView)
+    void EffectEditorBase::setWidgets(Gui::MWList* availableEffectsList, MyGUI::ScrollView* usedEffectsView)
     {
         mAvailableEffectsList = availableEffectsList;
         mUsedEffectsView = usedEffectsView;
 
-        mAvailableEffectsList->eventWidgetSelected += MyGUI::newDelegate(this, &EffectEditorBase::onAvailableEffectClicked);
+        mAvailableEffectsList->eventWidgetSelected
+            += MyGUI::newDelegate(this, &EffectEditorBase::onAvailableEffectClicked);
     }
 
-    void EffectEditorBase::onSelectAttribute ()
+    void EffectEditorBase::onSelectAttribute()
     {
-        const ESM::MagicEffect* effect =
-            MWBase::Environment::get().getWorld()->getStore().get<ESM::MagicEffect>().find(mSelectedKnownEffectId);
+        const ESM::MagicEffect* effect
+            = MWBase::Environment::get().getESMStore()->get<ESM::MagicEffect>().find(mSelectedKnownEffectId);
 
         mAddEffectDialog.newEffect(effect);
-        mAddEffectDialog.setAttribute (mSelectAttributeDialog->getAttributeId());
-        MWBase::Environment::get().getWindowManager ()->removeDialog (mSelectAttributeDialog);
-        mSelectAttributeDialog = nullptr;
+        mAddEffectDialog.setAttribute(mSelectAttributeDialog->getAttributeId());
+        MWBase::Environment::get().getWindowManager()->removeDialog(std::move(mSelectAttributeDialog));
     }
 
-    void EffectEditorBase::onSelectSkill ()
+    void EffectEditorBase::onSelectSkill()
     {
-        const ESM::MagicEffect* effect =
-            MWBase::Environment::get().getWorld()->getStore().get<ESM::MagicEffect>().find(mSelectedKnownEffectId);
+        const ESM::MagicEffect* effect
+            = MWBase::Environment::get().getESMStore()->get<ESM::MagicEffect>().find(mSelectedKnownEffectId);
 
         mAddEffectDialog.newEffect(effect);
-        mAddEffectDialog.setSkill (mSelectSkillDialog->getSkillId());
-        MWBase::Environment::get().getWindowManager ()->removeDialog (mSelectSkillDialog);
-        mSelectSkillDialog = nullptr;
+        mAddEffectDialog.setSkill(mSelectSkillDialog->getSkillId());
+        MWBase::Environment::get().getWindowManager()->removeDialog(std::move(mSelectSkillDialog));
     }
 
-    void EffectEditorBase::onAttributeOrSkillCancel ()
+    void EffectEditorBase::onAttributeOrSkillCancel()
     {
-        if (mSelectSkillDialog)
-            MWBase::Environment::get().getWindowManager ()->removeDialog (mSelectSkillDialog);
-        if (mSelectAttributeDialog)
-            MWBase::Environment::get().getWindowManager ()->removeDialog (mSelectAttributeDialog);
-
-        mSelectSkillDialog = nullptr;
-        mSelectAttributeDialog = nullptr;
+        if (mSelectSkillDialog != nullptr)
+            MWBase::Environment::get().getWindowManager()->removeDialog(std::move(mSelectSkillDialog));
+        if (mSelectAttributeDialog != nullptr)
+            MWBase::Environment::get().getWindowManager()->removeDialog(std::move(mSelectAttributeDialog));
     }
 
-    void EffectEditorBase::onAvailableEffectClicked (MyGUI::Widget* sender)
+    void EffectEditorBase::onAvailableEffectClicked(MyGUI::Widget* sender)
     {
         if (mEffects.size() >= 8)
         {
@@ -621,10 +625,10 @@ namespace MWGui
         int buttonId = *sender->getUserData<int>();
         mSelectedKnownEffectId = mButtonMapping[buttonId];
 
-        const ESM::MagicEffect* effect =
-            MWBase::Environment::get().getWorld()->getStore().get<ESM::MagicEffect>().find(mSelectedKnownEffectId);
+        const ESM::MagicEffect* effect
+            = MWBase::Environment::get().getESMStore()->get<ESM::MagicEffect>().find(mSelectedKnownEffectId);
 
-        bool allowSelf = (effect->mData.mFlags & ESM::MagicEffect::CastSelf) != 0;
+        bool allowSelf = (effect->mData.mFlags & ESM::MagicEffect::CastSelf) != 0 || mConstantEffect;
         bool allowTouch = (effect->mData.mFlags & ESM::MagicEffect::CastTouch) && !mConstantEffect;
         bool allowTarget = (effect->mData.mFlags & ESM::MagicEffect::CastTarget) && !mConstantEffect;
 
@@ -633,19 +637,19 @@ namespace MWGui
 
         if (effect->mData.mFlags & ESM::MagicEffect::TargetSkill)
         {
-            delete mSelectSkillDialog;
-            mSelectSkillDialog = new SelectSkillDialog();
+            mSelectSkillDialog = std::make_unique<SelectSkillDialog>();
             mSelectSkillDialog->eventCancel += MyGUI::newDelegate(this, &SpellCreationDialog::onAttributeOrSkillCancel);
             mSelectSkillDialog->eventItemSelected += MyGUI::newDelegate(this, &SpellCreationDialog::onSelectSkill);
-            mSelectSkillDialog->setVisible (true);
+            mSelectSkillDialog->setVisible(true);
         }
         else if (effect->mData.mFlags & ESM::MagicEffect::TargetAttribute)
         {
-            delete mSelectAttributeDialog;
-            mSelectAttributeDialog = new SelectAttributeDialog();
-            mSelectAttributeDialog->eventCancel += MyGUI::newDelegate(this, &SpellCreationDialog::onAttributeOrSkillCancel);
-            mSelectAttributeDialog->eventItemSelected += MyGUI::newDelegate(this, &SpellCreationDialog::onSelectAttribute);
-            mSelectAttributeDialog->setVisible (true);
+            mSelectAttributeDialog = std::make_unique<SelectAttributeDialog>();
+            mSelectAttributeDialog->eventCancel
+                += MyGUI::newDelegate(this, &SpellCreationDialog::onAttributeOrSkillCancel);
+            mSelectAttributeDialog->eventItemSelected
+                += MyGUI::newDelegate(this, &SpellCreationDialog::onSelectAttribute);
+            mSelectAttributeDialog->setVisible(true);
         }
         else
         {
@@ -653,7 +657,7 @@ namespace MWGui
             {
                 if (effectInfo.mEffectID == mSelectedKnownEffectId)
                 {
-                    MWBase::Environment::get().getWindowManager()->messageBox ("#{sOnetypeEffectMessage}");
+                    MWBase::Environment::get().getWindowManager()->messageBox("#{sOnetypeEffectMessage}");
                     return;
                 }
             }
@@ -662,33 +666,33 @@ namespace MWGui
         }
     }
 
-    void EffectEditorBase::onEffectModified (ESM::ENAMstruct effect)
+    void EffectEditorBase::onEffectModified(ESM::ENAMstruct effect)
     {
         mEffects[mSelectedEffect] = effect;
 
         updateEffectsView();
     }
 
-    void EffectEditorBase::onEffectRemoved (ESM::ENAMstruct effect)
+    void EffectEditorBase::onEffectRemoved(ESM::ENAMstruct effect)
     {
         mEffects.erase(mEffects.begin() + mSelectedEffect);
         updateEffectsView();
     }
 
-    void EffectEditorBase::updateEffectsView ()
+    void EffectEditorBase::updateEffectsView()
     {
-        MyGUI::EnumeratorWidgetPtr oldWidgets = mUsedEffectsView->getEnumerator ();
-        MyGUI::Gui::getInstance ().destroyWidgets (oldWidgets);
+        MyGUI::EnumeratorWidgetPtr oldWidgets = mUsedEffectsView->getEnumerator();
+        MyGUI::Gui::getInstance().destroyWidgets(oldWidgets);
 
-        MyGUI::IntSize size(0,0);
+        MyGUI::IntSize size(0, 0);
 
         int i = 0;
         for (const ESM::ENAMstruct& effectInfo : mEffects)
         {
             Widgets::SpellEffectParams params;
             params.mEffectID = effectInfo.mEffectID;
-            params.mSkill = effectInfo.mSkill;
-            params.mAttribute = effectInfo.mAttribute;
+            params.mSkill = ESM::Skill::indexToRefId(effectInfo.mSkill);
+            params.mAttribute = ESM::Attribute::indexToRefId(effectInfo.mAttribute);
             params.mDuration = effectInfo.mDuration;
             params.mMagnMin = effectInfo.mMagnMin;
             params.mMagnMax = effectInfo.mMagnMax;
@@ -696,25 +700,28 @@ namespace MWGui
             params.mArea = effectInfo.mArea;
             params.mIsConstant = mConstantEffect;
 
-            MyGUI::Button* button = mUsedEffectsView->createWidget<MyGUI::Button>("", MyGUI::IntCoord(0, size.height, 0, 24), MyGUI::Align::Default);
+            MyGUI::Button* button = mUsedEffectsView->createWidget<MyGUI::Button>(
+                {}, MyGUI::IntCoord(0, size.height, 0, 24), MyGUI::Align::Default);
             button->setUserData(i);
             button->eventMouseButtonClick += MyGUI::newDelegate(this, &SpellCreationDialog::onEditEffect);
-            button->setNeedMouseFocus (true);
+            button->setNeedMouseFocus(true);
 
-            Widgets::MWSpellEffectPtr effect = button->createWidget<Widgets::MWSpellEffect>("MW_EffectImage", MyGUI::IntCoord(0,0,0,24), MyGUI::Align::Default);
+            Widgets::MWSpellEffectPtr effect = button->createWidget<Widgets::MWSpellEffect>(
+                "MW_EffectImage", MyGUI::IntCoord(0, 0, 0, 24), MyGUI::Align::Default);
 
-            effect->setNeedMouseFocus (false);
-            effect->setSpellEffect (params);
+            effect->setNeedMouseFocus(false);
+            effect->setSpellEffect(params);
 
-            effect->setSize(effect->getRequestedWidth (), 24);
-            button->setSize(effect->getRequestedWidth (), 24);
+            effect->setSize(effect->getRequestedWidth(), 24);
+            button->setSize(effect->getRequestedWidth(), 24);
 
-            size.width = std::max(size.width, effect->getRequestedWidth ());
+            size.width = std::max(size.width, effect->getRequestedWidth());
             size.height += 24;
             ++i;
         }
 
-        // Canvas size must be expressed with HScroll disabled, otherwise MyGUI would expand the scroll area when the scrollbar is hidden
+        // Canvas size must be expressed with HScroll disabled, otherwise MyGUI would expand the scroll area when the
+        // scrollbar is hidden
         mUsedEffectsView->setVisibleHScroll(false);
         mUsedEffectsView->setCanvasSize(size);
         mUsedEffectsView->setVisibleHScroll(true);
@@ -722,46 +729,30 @@ namespace MWGui
         notifyEffectsChanged();
     }
 
-    void EffectEditorBase::onEffectAdded (ESM::ENAMstruct effect)
+    void EffectEditorBase::onEffectAdded(ESM::ENAMstruct effect)
     {
         mEffects.push_back(effect);
-        mSelectedEffect=mEffects.size()-1;
+        mSelectedEffect = mEffects.size() - 1;
 
         updateEffectsView();
     }
 
-    void EffectEditorBase::onEditEffect (MyGUI::Widget *sender)
+    void EffectEditorBase::onEditEffect(MyGUI::Widget* sender)
     {
         int id = *sender->getUserData<int>();
 
         mSelectedEffect = id;
 
-        mAddEffectDialog.editEffect (mEffects[id]);
-        mAddEffectDialog.setVisible (true);
+        mAddEffectDialog.editEffect(mEffects[id]);
+        mAddEffectDialog.setVisible(true);
     }
 
     void EffectEditorBase::setConstantEffect(bool constant)
     {
         mAddEffectDialog.setConstantEffect(constant);
+        if (!mConstantEffect && constant)
+            for (ESM::ENAMstruct& effect : mEffects)
+                effect.mRange = ESM::RT_Self;
         mConstantEffect = constant;
-
-        if (!constant)
-            return;
-
-        for (auto it = mEffects.begin(); it != mEffects.end();)
-        {
-            if (it->mRange != ESM::RT_Self)
-            {
-                auto& store = MWBase::Environment::get().getWorld()->getStore();
-                auto magicEffect = store.get<ESM::MagicEffect>().find(it->mEffectID);
-                if ((magicEffect->mData.mFlags & ESM::MagicEffect::CastSelf) == 0)
-                {
-                    it = mEffects.erase(it);
-                    continue;
-                }
-                it->mRange = ESM::RT_Self;
-            }
-            ++it;
-        }
     }
 }
