@@ -3,16 +3,18 @@
 #include <components/debug/debuglog.hpp>
 #include <components/misc/utf8stream.hpp>
 
+#include <components/esm3/loadench.hpp>
+#include <components/esm3/loadmgef.hpp>
+
 #include "../mwbase/environment.hpp"
-#include "../mwbase/world.hpp"
 #include "../mwbase/windowmanager.hpp"
 
 #include "../mwmechanics/creaturestats.hpp"
 #include "../mwmechanics/spellutil.hpp"
 
+#include "../mwworld/class.hpp"
 #include "../mwworld/esmstore.hpp"
 #include "../mwworld/inventorystore.hpp"
-#include "../mwworld/class.hpp"
 
 namespace
 {
@@ -21,11 +23,7 @@ namespace
     {
         if (left.mType != right.mType)
             return left.mType < right.mType;
-
-        std::string leftName = Misc::StringUtils::lowerCase(left.mName);
-        std::string rightName = Misc::StringUtils::lowerCase(right.mName);
-
-        return leftName.compare(rightName) < 0;
+        return Misc::StringUtils::ciLess(left.mName, right.mName);
     }
 
 }
@@ -33,43 +31,33 @@ namespace
 namespace MWGui
 {
 
-    SpellModel::SpellModel(const MWWorld::Ptr &actor, const std::string& filter)
-        : mActor(actor), mFilter(filter)
+    SpellModel::SpellModel(const MWWorld::Ptr& actor, const std::string& filter)
+        : mActor(actor)
+        , mFilter(filter)
     {
     }
 
-    SpellModel::SpellModel(const MWWorld::Ptr &actor)
+    SpellModel::SpellModel(const MWWorld::Ptr& actor)
         : mActor(actor)
     {
     }
 
-    bool SpellModel::matchingEffectExists(std::string filter, const ESM::EffectList &effects)
+    bool SpellModel::matchingEffectExists(std::string filter, const ESM::EffectList& effects)
     {
-        auto wm = MWBase::Environment::get().getWindowManager();
-        const MWWorld::ESMStore &store =
-            MWBase::Environment::get().getWorld()->getStore();
+        const MWWorld::ESMStore& store = *MWBase::Environment::get().getESMStore();
 
         for (const auto& effect : effects.mList)
         {
-            short effectId = effect.mEffectID;
+            short effectId = effect.mData.mEffectID;
 
             if (effectId != -1)
             {
-                const ESM::MagicEffect *magicEffect =
-                    store.get<ESM::MagicEffect>().find(effectId);
-                std::string effectIDStr = ESM::MagicEffect::effectIdToString(effectId);
-                std::string fullEffectName = wm->getGameSettingString(effectIDStr, "");
-
-                if (magicEffect->mData.mFlags & ESM::MagicEffect::TargetSkill && effect.mSkill != -1)
-                {
-                    fullEffectName += " " + wm->getGameSettingString(ESM::Skill::sSkillNameIds[effect.mSkill], "");
-                }
-
-                if (magicEffect->mData.mFlags & ESM::MagicEffect::TargetAttribute && effect.mAttribute != -1)
-                {
-                    fullEffectName += " " + wm->getGameSettingString(ESM::Attribute::sGmstAttributeIds[effect.mAttribute], "");
-                }
+                const ESM::MagicEffect* magicEffect = store.get<ESM::MagicEffect>().find(effectId);
+                const ESM::Attribute* attribute
+                    = store.get<ESM::Attribute>().search(ESM::Attribute::indexToRefId(effect.mData.mAttribute));
+                const ESM::Skill* skill = store.get<ESM::Skill>().search(ESM::Skill::indexToRefId(effect.mData.mSkill));
 
+                std::string fullEffectName = MWMechanics::getMagicEffectString(*magicEffect, attribute, skill);
                 std::string convert = Utf8Stream::lowerCaseUtf8(fullEffectName);
                 if (convert.find(filter) != std::string::npos)
                 {
@@ -88,8 +76,7 @@ namespace MWGui
         MWMechanics::CreatureStats& stats = mActor.getClass().getCreatureStats(mActor);
         const MWMechanics::Spells& spells = stats.getSpells();
 
-        const MWWorld::ESMStore &esmStore =
-            MWBase::Environment::get().getWorld()->getStore();
+        const MWWorld::ESMStore& esmStore = *MWBase::Environment::get().getESMStore();
 
         std::string filter = Utf8Stream::lowerCaseUtf8(mFilter);
 
@@ -99,9 +86,8 @@ namespace MWGui
                 continue;
 
             std::string name = Utf8Stream::lowerCaseUtf8(spell->mName);
- 
-            if (name.find(filter) == std::string::npos 
-                && !matchingEffectExists(filter, spell->mEffects))
+
+            if (name.find(filter) == std::string::npos && !matchingEffectExists(filter, spell->mEffects))
                 continue;
 
             Spell newSpell;
@@ -127,30 +113,31 @@ namespace MWGui
         for (MWWorld::ContainerStoreIterator it = invStore.begin(); it != invStore.end(); ++it)
         {
             MWWorld::Ptr item = *it;
-            const std::string enchantId = item.getClass().getEnchantment(item);
+            const ESM::RefId& enchantId = item.getClass().getEnchantment(item);
             if (enchantId.empty())
                 continue;
             const ESM::Enchantment* enchant = esmStore.get<ESM::Enchantment>().search(enchantId);
             if (!enchant)
             {
-                Log(Debug::Warning) << "Warning: Can't find enchantment '" << enchantId << "' on item " << item.getCellRef().getRefId();
+                Log(Debug::Warning) << "Warning: Can't find enchantment '" << enchantId << "' on item "
+                                    << item.getCellRef().getRefId();
                 continue;
             }
 
-            if (enchant->mData.mType != ESM::Enchantment::WhenUsed && enchant->mData.mType != ESM::Enchantment::CastOnce)
+            if (enchant->mData.mType != ESM::Enchantment::WhenUsed
+                && enchant->mData.mType != ESM::Enchantment::CastOnce)
                 continue;
 
             std::string name = Utf8Stream::lowerCaseUtf8(item.getClass().getName(item));
 
-            if (name.find(filter) == std::string::npos
-                && !matchingEffectExists(filter, enchant->mEffects))
+            if (name.find(filter) == std::string::npos && !matchingEffectExists(filter, enchant->mEffects))
                 continue;
 
             Spell newSpell;
             newSpell.mItem = item;
             newSpell.mId = item.getCellRef().getRefId();
             newSpell.mName = item.getClass().getName(item);
-            newSpell.mCount = item.getRefData().getCount();
+            newSpell.mCount = item.getCellRef().getCount();
             newSpell.mType = Spell::Type_EnchantedItem;
             newSpell.mSelected = invStore.getSelectedEnchantItem() == it;
 
@@ -163,15 +150,15 @@ namespace MWGui
             else
             {
                 if (!item.getClass().getEquipmentSlots(item).first.empty()
-                        && item.getClass().canBeEquipped(item, mActor).first == 0)
+                    && item.getClass().canBeEquipped(item, mActor).first == 0)
                     continue;
 
-                int castCost = MWMechanics::getEffectiveEnchantmentCastCost(static_cast<float>(enchant->mData.mCost), mActor);
+                int castCost = MWMechanics::getEffectiveEnchantmentCastCost(*enchant, mActor);
 
                 std::string cost = std::to_string(castCost);
                 int currentCharge = int(item.getCellRef().getEnchantmentCharge());
-                if (currentCharge ==  -1)
-                    currentCharge = enchant->mData.mCharge;
+                if (currentCharge == -1)
+                    currentCharge = MWMechanics::getEnchantmentCharge(*enchant);
                 std::string charge = std::to_string(currentCharge);
                 newSpell.mCostColumn = cost + "/" + charge;
 
@@ -191,9 +178,10 @@ namespace MWGui
     SpellModel::ModelIndex SpellModel::getSelectedIndex() const
     {
         ModelIndex selected = -1;
-        for (SpellModel::ModelIndex i = 0; i<int(getItemCount()); ++i)
+        for (SpellModel::ModelIndex i = 0; i < int(getItemCount()); ++i)
         {
-            if (getItem(i).mSelected) {
+            if (getItem(i).mSelected)
+            {
                 selected = i;
                 break;
             }
