@@ -1,20 +1,20 @@
 #ifndef MWGUI_SPELLWINDOW_H
 #define MWGUI_SPELLWINDOW_H
 
-#include "windowpinnablebase.hpp"
+#include <memory>
 
+#include "spellicons.hpp"
 #include "spellmodel.hpp"
+#include "windowpinnablebase.hpp"
 
 namespace MWGui
 {
-    class SpellIcons;
     class SpellView;
 
     class SpellWindow : public WindowPinnableBase, public NoDrop
     {
     public:
         SpellWindow(DragAndDrop* drag);
-        virtual ~SpellWindow();
 
         void updateSpells();
 
@@ -23,25 +23,27 @@ namespace MWGui
         /// Cycle to next/previous spell
         void cycle(bool next);
 
+        std::string_view getWindowIdForLua() const override { return "Magic"; }
+
     protected:
         MyGUI::Widget* mEffectBox;
 
-        std::string mSpellToDelete;
+        ESM::RefId mSpellToDelete;
 
         void onEnchantedItemSelected(MWWorld::Ptr item, bool alreadyEquipped);
-        void onSpellSelected(const std::string& spellId);
+        void onSpellSelected(const ESM::RefId& spellId);
         void onModelIndexSelected(SpellModel::ModelIndex index);
-        void onFilterChanged(MyGUI::EditBox *sender);
-        void onDeleteClicked(MyGUI::Widget *widget);
+        void onFilterChanged(MyGUI::EditBox* sender);
+        void onDeleteClicked(MyGUI::Widget* widget);
         void onDeleteSpellAccept();
-        void askDeleteSpell(const std::string& spellId);
+        void askDeleteSpell(const ESM::RefId& spellId);
 
         void onPinToggled() override;
         void onTitleDoubleClicked() override;
         void onOpen() override;
 
         SpellView* mSpellView;
-        SpellIcons* mSpellIcons;
+        std::unique_ptr<SpellIcons> mSpellIcons;
         MyGUI::EditBox* mFilterEdit;
 
     private:
