@@ -1,15 +1,15 @@
 #include "statswatcher.hpp"
 
+#include <components/esm3/loadclas.hpp>
+#include <components/esm3/loadrace.hpp>
+
 #include "../mwbase/environment.hpp"
 #include "../mwbase/windowmanager.hpp"
-#include "../mwbase/world.hpp"
 
 #include "../mwmechanics/npcstats.hpp"
-#include "../mwmechanics/spellutil.hpp"
 
 #include "../mwworld/class.hpp"
 #include "../mwworld/esmstore.hpp"
-#include "../mwworld/inventorystore.hpp"
 
 #include <string>
 
@@ -18,7 +18,9 @@ namespace MWGui
     // mWatchedTimeToStartDrowning = -1 for correct drowning state check,
     // if stats.getTimeToStartDrowning() == 0 already on game start
     StatsWatcher::StatsWatcher()
-      : mWatchedLevel(-1), mWatchedTimeToStartDrowning(-1), mWatchedStatsEmpty(true)
+        : mWatchedLevel(-1)
+        , mWatchedTimeToStartDrowning(-1)
+        , mWatchedStatsEmpty(true)
     {
     }
 
@@ -32,46 +34,48 @@ namespace MWGui
         if (mWatched.isEmpty())
             return;
 
-        MWBase::WindowManager *winMgr = MWBase::Environment::get().getWindowManager();
-        const MWMechanics::NpcStats &stats = mWatched.getClass().getNpcStats(mWatched);
-        for (int i = 0;i < ESM::Attribute::Length;++i)
+        const auto& store = MWBase::Environment::get().getESMStore();
+        MWBase::WindowManager* winMgr = MWBase::Environment::get().getWindowManager();
+        const MWMechanics::NpcStats& stats = mWatched.getClass().getNpcStats(mWatched);
+        for (const ESM::Attribute& attribute : store->get<ESM::Attribute>())
         {
-            if (stats.getAttribute(i) != mWatchedAttributes[i] || mWatchedStatsEmpty)
+            const auto& value = stats.getAttribute(attribute.mId);
+            if (value != mWatchedAttributes[attribute.mId] || mWatchedStatsEmpty)
             {
-                mWatchedAttributes[i] = stats.getAttribute(i);
-                setValue("AttribVal" + std::to_string(i + 1), stats.getAttribute(i));
+                mWatchedAttributes[attribute.mId] = value;
+                setAttribute(attribute.mId, value);
             }
         }
 
         if (stats.getHealth() != mWatchedHealth || mWatchedStatsEmpty)
         {
-            static const std::string hbar("HBar");
             mWatchedHealth = stats.getHealth();
-            setValue(hbar, stats.getHealth());
+            setValue("HBar", stats.getHealth());
         }
         if (stats.getMagicka() != mWatchedMagicka || mWatchedStatsEmpty)
         {
-            static const std::string mbar("MBar");
             mWatchedMagicka = stats.getMagicka();
-            setValue(mbar, stats.getMagicka());
+            setValue("MBar", stats.getMagicka());
         }
         if (stats.getFatigue() != mWatchedFatigue || mWatchedStatsEmpty)
         {
-            static const std::string fbar("FBar");
             mWatchedFatigue = stats.getFatigue();
-            setValue(fbar, stats.getFatigue());
+            setValue("FBar", stats.getFatigue());
         }
 
         float timeToDrown = stats.getTimeToStartDrowning();
 
         if (timeToDrown != mWatchedTimeToStartDrowning)
         {
-            static const float fHoldBreathTime = MWBase::Environment::get().getWorld()->getStore().get<ESM::GameSetting>()
-                    .find("fHoldBreathTime")->mValue.getFloat();
+            static const float fHoldBreathTime = MWBase::Environment::get()
+                                                     .getESMStore()
+                                                     ->get<ESM::GameSetting>()
+                                                     .find("fHoldBreathTime")
+                                                     ->mValue.getFloat();
 
             mWatchedTimeToStartDrowning = timeToDrown;
 
-            if(timeToDrown >= fHoldBreathTime || timeToDrown == -1.0) // -1.0 is a special value during initialization
+            if (timeToDrown >= fHoldBreathTime || timeToDrown == -1.0) // -1.0 is a special value during initialization
                 winMgr->setDrowningBarVisibility(false);
             else
             {
@@ -80,13 +84,13 @@ namespace MWGui
             }
         }
 
-        //Loop over ESM::Skill::SkillEnum
-        for (int i = 0; i < ESM::Skill::Length; ++i)
+        for (const ESM::Skill& skill : store->get<ESM::Skill>())
         {
-            if(stats.getSkill(i) != mWatchedSkills[i] || mWatchedStatsEmpty)
+            const auto& value = stats.getSkill(skill.mId);
+            if (value != mWatchedSkills[skill.mId] || mWatchedStatsEmpty)
             {
-                mWatchedSkills[i] = stats.getSkill(i);
-                setValue((ESM::Skill::SkillEnum)i, stats.getSkill(i));
+                mWatchedSkills[skill.mId] = value;
+                setValue(skill.mId, value);
             }
         }
 
@@ -98,7 +102,7 @@ namespace MWGui
 
         if (mWatched.getClass().isNpc())
         {
-            const ESM::NPC *watchedRecord = mWatched.get<ESM::NPC>()->mBase;
+            const ESM::NPC* watchedRecord = mWatched.get<ESM::NPC>()->mBase;
 
             if (watchedRecord->mName != mWatchedName || mWatchedStatsEmpty)
             {
@@ -109,25 +113,24 @@ namespace MWGui
             if (watchedRecord->mRace != mWatchedRace || mWatchedStatsEmpty)
             {
                 mWatchedRace = watchedRecord->mRace;
-                const ESM::Race *race = MWBase::Environment::get().getWorld()->getStore()
-                    .get<ESM::Race>().find(watchedRecord->mRace);
+                const ESM::Race* race = store->get<ESM::Race>().find(watchedRecord->mRace);
                 setValue("race", race->mName);
             }
 
             if (watchedRecord->mClass != mWatchedClass || mWatchedStatsEmpty)
             {
                 mWatchedClass = watchedRecord->mClass;
-                const ESM::Class *cls = MWBase::Environment::get().getWorld()->getStore()
-                    .get<ESM::Class>().find(watchedRecord->mClass);
+                const ESM::Class* cls = store->get<ESM::Class>().find(watchedRecord->mClass);
                 setValue("class", cls->mName);
 
-                MWBase::WindowManager::SkillList majorSkills (5);
-                MWBase::WindowManager::SkillList minorSkills (5);
+                size_t size = cls->mData.mSkills.size();
+                std::vector<ESM::RefId> majorSkills(size);
+                std::vector<ESM::RefId> minorSkills(size);
 
-                for (int i=0; i<5; ++i)
+                for (size_t i = 0; i < size; ++i)
                 {
-                    minorSkills[i] = cls->mData.mSkills[i][0];
-                    majorSkills[i] = cls->mData.mSkills[i][1];
+                    minorSkills[i] = ESM::Skill::indexToRefId(cls->mData.mSkills[i][0]);
+                    majorSkills[i] = ESM::Skill::indexToRefId(cls->mData.mSkills[i][1]);
                 }
 
                 configureSkills(majorSkills, minorSkills);
@@ -147,39 +150,37 @@ namespace MWGui
         mListeners.erase(listener);
     }
 
-    void StatsWatcher::setValue(const std::string& id, const MWMechanics::AttributeValue& value)
+    void StatsWatcher::setAttribute(ESM::RefId id, const MWMechanics::AttributeValue& value)
+    {
+        for (StatsListener* listener : mListeners)
+            listener->setAttribute(id, value);
+    }
+
+    void StatsWatcher::setValue(ESM::RefId id, const MWMechanics::SkillValue& value)
     {
         for (StatsListener* listener : mListeners)
             listener->setValue(id, value);
     }
 
-    void StatsWatcher::setValue(ESM::Skill::SkillEnum parSkill, const MWMechanics::SkillValue& value)
-    {
-        /// \todo Don't use the skill enum as a parameter type (we will have to drop it anyway, once we
-        /// allow custom skills.
-        for (StatsListener* listener : mListeners)
-            listener->setValue(parSkill, value);
-    }
-
-    void StatsWatcher::setValue(const std::string& id, const MWMechanics::DynamicStat<float>& value)
+    void StatsWatcher::setValue(std::string_view id, const MWMechanics::DynamicStat<float>& value)
     {
         for (StatsListener* listener : mListeners)
             listener->setValue(id, value);
     }
 
-    void StatsWatcher::setValue(const std::string& id, const std::string& value)
+    void StatsWatcher::setValue(std::string_view id, const std::string& value)
     {
         for (StatsListener* listener : mListeners)
             listener->setValue(id, value);
     }
 
-    void StatsWatcher::setValue(const std::string& id, int value)
+    void StatsWatcher::setValue(std::string_view id, int value)
     {
         for (StatsListener* listener : mListeners)
             listener->setValue(id, value);
     }
 
-    void StatsWatcher::configureSkills(const std::vector<int>& major, const std::vector<int>& minor)
+    void StatsWatcher::configureSkills(const std::vector<ESM::RefId>& major, const std::vector<ESM::RefId>& minor)
     {
         for (StatsListener* listener : mListeners)
             listener->configureSkills(major, minor);
