@@ -3,11 +3,6 @@
 
 #include "windowbase.hpp"
 
-namespace MWGui
-{
-    class WindowManager;
-}
-
 namespace MWGui
 {
     class TextInputDialog : public WindowModal
@@ -16,13 +11,13 @@ namespace MWGui
         TextInputDialog();
 
         std::string getTextInput() const;
-        void setTextInput(const std::string &text);
+        void setTextInput(const std::string& text);
 
         void setNextButtonShow(bool shown);
-        void setTextLabel(const std::string &label);
-        virtual void onOpen();
+        void setTextLabel(std::string_view label);
+        void onOpen() override;
 
-        bool exit() { return false; }
+        bool exit() override { return false; }
 
         /** Event : Dialog finished, OK button clicked.\n
             signature : void method()\n
@@ -31,7 +26,7 @@ namespace MWGui
 
     protected:
         void onOkClicked(MyGUI::Widget* _sender);
-        void onTextAccepted(MyGUI::Edit* _sender);
+        void onTextAccepted(MyGUI::EditBox* _sender);
 
     private:
         MyGUI::EditBox* mTextEdit;
