@@ -1,8 +1,8 @@
 #ifndef MWGUI_TOOLTIPS_H
 #define MWGUI_TOOLTIPS_H
 
-#include "layout.hpp"
 #include "../mwworld/ptr.hpp"
+#include "layout.hpp"
 
 #include "widgets.hpp"
 
@@ -24,15 +24,17 @@ namespace MWGui
             , isPotion(false)
             , isIngredient(false)
             , wordWrap(true)
-        {}
+        {
+        }
 
         std::string caption;
         std::string text;
+        std::string extra;
         std::string icon;
         int imageSize;
 
         // enchantment (for cloth, armor, weapons)
-        std::string enchant;
+        ESM::RefId enchant;
         int remainingEnchantCharge;
 
         // effects (for potions, ingredients)
@@ -59,12 +61,10 @@ namespace MWGui
         bool toggleFullHelp(); ///< show extra info in item tooltips (owner, script)
         bool getFullHelp() const;
 
-        void setDelay(float delay);
-
         void clear();
 
         void setFocusObject(const MWWorld::Ptr& focus);
-        void setFocusObjectScreenCoords(float min_x, float min_y, float max_x, float max_y);
+        void setFocusObjectScreenCoords(float x, float y);
         ///< set the screen-space position of the tooltip for focused object
 
         static std::string getWeightString(const float weight, const std::string& prefix);
@@ -87,28 +87,28 @@ namespace MWGui
         static std::string getCellRefString(const MWWorld::CellRef& cellref);
         ///< Returns a string containing debug tooltip information about the given cellref.
 
-        static std::string getDurationString (float duration, const std::string& prefix);
+        static std::string getDurationString(float duration, const std::string& prefix);
         ///< Returns duration as two largest time units, rounded down. Note: not localized; no line break.
 
         // these do not create an actual tooltip, but they fill in the data that is required so the tooltip
         // system knows what to show in case this widget is hovered
-        static void createSkillToolTip(MyGUI::Widget* widget, int skillId);
-        static void createAttributeToolTip(MyGUI::Widget* widget, int attributeId);
+        static void createSkillToolTip(MyGUI::Widget* widget, ESM::RefId skillId);
+        static void createAttributeToolTip(MyGUI::Widget* widget, ESM::RefId attributeId);
         static void createSpecializationToolTip(MyGUI::Widget* widget, const std::string& name, int specId);
-        static void createBirthsignToolTip(MyGUI::Widget* widget, const std::string& birthsignId);
+        static void createBirthsignToolTip(MyGUI::Widget* widget, const ESM::RefId& birthsignId);
         static void createRaceToolTip(MyGUI::Widget* widget, const ESM::Race* playerRace);
         static void createClassToolTip(MyGUI::Widget* widget, const ESM::Class& playerClass);
         static void createMagicEffectToolTip(MyGUI::Widget* widget, short id);
-        
+
         bool checkOwned();
         /// Returns True if taking mFocusObject would be crime
- 
+
     private:
         MyGUI::Widget* mDynamicToolTipBox;
 
         MWWorld::Ptr mFocusObject;
 
-        MyGUI::IntSize getToolTipViaPtr (int count, bool image = true, bool isOwned = false);
+        MyGUI::IntSize getToolTipViaPtr(int count, bool image = true, bool isOwned = false);
         ///< @return requested tooltip size
 
         MyGUI::IntSize createToolTip(const ToolTipInfo& info, bool isOwned = false);
@@ -121,12 +121,8 @@ namespace MWGui
         /// Adjust position for a tooltip so that it doesn't leave the screen and does not obscure the mouse cursor
         void position(MyGUI::IntPoint& position, MyGUI::IntSize size, MyGUI::IntSize viewportSize);
 
-        static std::string sSchoolNames[6];
-
         int mHorizontalScrollIndex;
 
-
-        float mDelay;
         float mRemainingDelay; // remaining time until tooltip will show
 
         int mLastMouseX;
@@ -135,8 +131,6 @@ namespace MWGui
         bool mEnabled;
 
         bool mFullHelp;
-        
-        int mShowOwned;
 
         float mFrameDuration;
     };
