@@ -1,17 +1,19 @@
 #include "tradewindow.hpp"
 
 #include <MyGUI_Button.h>
-#include <MyGUI_InputManager.h>
 #include <MyGUI_ControllerManager.h>
 #include <MyGUI_ControllerRepeatClick.h>
+#include <MyGUI_InputManager.h>
 
+#include <components/misc/rng.hpp>
+#include <components/misc/strings/format.hpp>
 #include <components/widgets/numericeditbox.hpp>
 
-#include "../mwbase/environment.hpp"
-#include "../mwbase/world.hpp"
-#include "../mwbase/windowmanager.hpp"
-#include "../mwbase/mechanicsmanager.hpp"
 #include "../mwbase/dialoguemanager.hpp"
+#include "../mwbase/environment.hpp"
+#include "../mwbase/mechanicsmanager.hpp"
+#include "../mwbase/windowmanager.hpp"
+#include "../mwbase/world.hpp"
 
 #include "../mwworld/class.hpp"
 #include "../mwworld/containerstore.hpp"
@@ -20,18 +22,18 @@
 #include "../mwmechanics/actorutil.hpp"
 #include "../mwmechanics/creaturestats.hpp"
 
+#include "containeritemmodel.hpp"
+#include "countdialog.hpp"
 #include "inventorywindow.hpp"
 #include "itemview.hpp"
 #include "sortfilteritemmodel.hpp"
-#include "containeritemmodel.hpp"
-#include "tradeitemmodel.hpp"
-#include "countdialog.hpp"
 #include "tooltips.hpp"
+#include "tradeitemmodel.hpp"
 
 namespace
 {
 
-    int getEffectiveValue (MWWorld::Ptr item, int count)
+    int getEffectiveValue(MWWorld::Ptr item, int count)
     {
         float price = static_cast<float>(item.getClass().getValue(item));
         if (item.getClass().hasItemHealth(item))
@@ -41,6 +43,75 @@ namespace
         return static_cast<int>(price * count);
     }
 
+    bool haggle(const MWWorld::Ptr& player, const MWWorld::Ptr& merchant, int playerOffer, int merchantOffer)
+    {
+        // accept if merchant offer is better than player offer
+        if (playerOffer <= merchantOffer)
+        {
+            return true;
+        }
+
+        // reject if npc is a creature
+        if (merchant.getType() != ESM::NPC::sRecordId)
+        {
+            return false;
+        }
+
+        const MWWorld::Store<ESM::GameSetting>& gmst
+            = MWBase::Environment::get().getESMStore()->get<ESM::GameSetting>();
+
+        // Is the player buying?
+        bool buying = (merchantOffer < 0);
+        int a = std::abs(merchantOffer);
+        int b = std::abs(playerOffer);
+        int d = (buying) ? int(100 * (a - b) / a) : int(100 * (b - a) / b);
+
+        int clampedDisposition = MWBase::Environment::get().getMechanicsManager()->getDerivedDisposition(merchant);
+
+        const MWMechanics::CreatureStats& merchantStats = merchant.getClass().getCreatureStats(merchant);
+        const MWMechanics::CreatureStats& playerStats = player.getClass().getCreatureStats(player);
+
+        float a1 = static_cast<float>(player.getClass().getSkill(player, ESM::Skill::Mercantile));
+        float b1 = 0.1f * playerStats.getAttribute(ESM::Attribute::Luck).getModified();
+        float c1 = 0.2f * playerStats.getAttribute(ESM::Attribute::Personality).getModified();
+        float d1 = static_cast<float>(merchant.getClass().getSkill(merchant, ESM::Skill::Mercantile));
+        float e1 = 0.1f * merchantStats.getAttribute(ESM::Attribute::Luck).getModified();
+        float f1 = 0.2f * merchantStats.getAttribute(ESM::Attribute::Personality).getModified();
+
+        float dispositionTerm = gmst.find("fDispositionMod")->mValue.getFloat() * (clampedDisposition - 50);
+        float pcTerm = (dispositionTerm + a1 + b1 + c1) * playerStats.getFatigueTerm();
+        float npcTerm = (d1 + e1 + f1) * merchantStats.getFatigueTerm();
+        float x = gmst.find("fBargainOfferMulti")->mValue.getFloat() * d
+            + gmst.find("fBargainOfferBase")->mValue.getFloat() + int(pcTerm - npcTerm);
+
+        auto& prng = MWBase::Environment::get().getWorld()->getPrng();
+        int roll = Misc::Rng::rollDice(100, prng) + 1;
+
+        // reject if roll fails
+        // (or if player tries to buy things and get money)
+        if (roll > x || (merchantOffer < 0 && 0 < playerOffer))
+        {
+            return false;
+        }
+
+        // apply skill gain on successful barter
+        float skillGain = 0.f;
+        int finalPrice = std::abs(playerOffer);
+        int initialMerchantOffer = std::abs(merchantOffer);
+
+        if (!buying && (finalPrice > initialMerchantOffer))
+        {
+            skillGain = std::floor(100.f * (finalPrice - initialMerchantOffer) / finalPrice);
+        }
+        else if (buying && (finalPrice < initialMerchantOffer))
+        {
+            skillGain = std::floor(100.f * (initialMerchantOffer - finalPrice) / initialMerchantOffer);
+        }
+        player.getClass().skillUsageSucceeded(
+            player, ESM::Skill::Mercantile, ESM::Skill::Mercantile_Success, skillGain);
+
+        return true;
+    }
 }
 
 namespace MWGui
@@ -93,13 +164,16 @@ namespace MWGui
 
         mTotalBalance->eventValueChanged += MyGUI::newDelegate(this, &TradeWindow::onBalanceValueChanged);
         mTotalBalance->eventEditSelectAccept += MyGUI::newDelegate(this, &TradeWindow::onAccept);
-        mTotalBalance->setMinValue(std::numeric_limits<int>::min()+1); // disallow INT_MIN since abs(INT_MIN) is undefined
+        mTotalBalance->setMinValue(
+            std::numeric_limits<int>::min() + 1); // disallow INT_MIN since abs(INT_MIN) is undefined
 
         setCoord(400, 0, 400, 300);
     }
 
     void TradeWindow::setPtr(const MWWorld::Ptr& actor)
     {
+        if (actor.isEmpty() || !actor.getClass().isActor())
+            throw std::runtime_error("Invalid argument in TradeWindow::setPtr");
         mPtr = actor;
 
         mCurrentBalance = 0;
@@ -113,9 +187,12 @@ namespace MWGui
         std::vector<MWWorld::Ptr> worldItems;
         MWBase::Environment::get().getWorld()->getItemsOwnedBy(actor, worldItems);
 
-        mTradeModel = new TradeItemModel(new ContainerItemModel(itemSources, worldItems), mPtr);
-        mSortModel = new SortFilterItemModel(mTradeModel);
-        mItemView->setModel (mSortModel);
+        auto tradeModel
+            = std::make_unique<TradeItemModel>(std::make_unique<ContainerItemModel>(itemSources, worldItems), mPtr);
+        mTradeModel = tradeModel.get();
+        auto sortModel = std::make_unique<SortFilterItemModel>(std::move(tradeModel));
+        mSortModel = sortModel.get();
+        mItemView->setModel(std::move(sortModel));
         mItemView->resetScrollBars();
 
         updateLabels();
@@ -123,7 +200,7 @@ namespace MWGui
         setTitle(actor.getClass().getName(actor));
 
         onFilterChanged(mFilterAll);
-        mFilterEdit->setCaption("");
+        mFilterEdit->setCaption({});
     }
 
     void TradeWindow::onFrame(float dt)
@@ -173,7 +250,7 @@ namespace MWGui
         return true;
     }
 
-    void TradeWindow::onItemSelected (int index)
+    void TradeWindow::onItemSelected(int index)
     {
         const ItemStack& item = mSortModel->getItem(index);
 
@@ -187,7 +264,8 @@ namespace MWGui
         {
             CountDialog* dialog = MWBase::Environment::get().getWindowManager()->getCountDialog();
             std::string message = "#{sQuanityMenuMessage02}";
-            std::string name = object.getClass().getName(object) + MWGui::ToolTips::getSoulString(object.getCellRef());
+            std::string name{ object.getClass().getName(object) };
+            name += MWGui::ToolTips::getSoulString(object.getCellRef());
             dialog->openCountDialog(name, message, count);
             dialog->eventOkClicked.clear();
             dialog->eventOkClicked += MyGUI::newDelegate(this, &TradeWindow::sellItem);
@@ -196,17 +274,18 @@ namespace MWGui
         else
         {
             mItemToSell = mSortModel->mapToSource(index);
-            sellItem (nullptr, count);
+            sellItem(nullptr, count);
         }
     }
 
     void TradeWindow::sellItem(MyGUI::Widget* sender, int count)
     {
         const ItemStack& item = mTradeModel->getItem(mItemToSell);
-        std::string sound = item.mBase.getClass().getUpSoundId(item.mBase);
+        const ESM::RefId& sound = item.mBase.getClass().getUpSoundId(item.mBase);
         MWBase::Environment::get().getWindowManager()->playSound(sound);
 
-        TradeItemModel* playerTradeModel = MWBase::Environment::get().getWindowManager()->getInventoryWindow()->getTradeModel();
+        TradeItemModel* playerTradeModel
+            = MWBase::Environment::get().getWindowManager()->getInventoryWindow()->getTradeModel();
 
         if (item.mType == ItemStack::Type_Barter)
         {
@@ -227,17 +306,19 @@ namespace MWGui
         mItemView->update();
     }
 
-    void TradeWindow::borrowItem (int index, size_t count)
+    void TradeWindow::borrowItem(int index, size_t count)
     {
-        TradeItemModel* playerTradeModel = MWBase::Environment::get().getWindowManager()->getInventoryWindow()->getTradeModel();
+        TradeItemModel* playerTradeModel
+            = MWBase::Environment::get().getWindowManager()->getInventoryWindow()->getTradeModel();
         mTradeModel->borrowItemToUs(index, playerTradeModel, count);
         mItemView->update();
         sellToNpc(playerTradeModel->getItem(index).mBase, count, false);
     }
 
-    void TradeWindow::returnItem (int index, size_t count)
+    void TradeWindow::returnItem(int index, size_t count)
     {
-        TradeItemModel* playerTradeModel = MWBase::Environment::get().getWindowManager()->getInventoryWindow()->getTradeModel();
+        TradeItemModel* playerTradeModel
+            = MWBase::Environment::get().getWindowManager()->getInventoryWindow()->getTradeModel();
         const ItemStack& item = playerTradeModel->getItem(index);
         mTradeModel->returnItemBorrowedFromUs(index, playerTradeModel, count);
         mItemView->update();
@@ -250,22 +331,24 @@ namespace MWGui
 
         if (amount > 0)
         {
-            store.add(MWWorld::ContainerStore::sGoldId, amount, actor);
+            store.add(MWWorld::ContainerStore::sGoldId, amount);
         }
         else
         {
-            store.remove(MWWorld::ContainerStore::sGoldId, - amount, actor);
+            store.remove(MWWorld::ContainerStore::sGoldId, -amount);
         }
     }
 
     void TradeWindow::onOfferButtonClicked(MyGUI::Widget* _sender)
     {
-        TradeItemModel* playerItemModel = MWBase::Environment::get().getWindowManager()->getInventoryWindow()->getTradeModel();
+        TradeItemModel* playerItemModel
+            = MWBase::Environment::get().getWindowManager()->getInventoryWindow()->getTradeModel();
 
-        const MWWorld::Store<ESM::GameSetting> &gmst =
-            MWBase::Environment::get().getWorld()->getStore().get<ESM::GameSetting>();
+        const MWWorld::Store<ESM::GameSetting>& gmst
+            = MWBase::Environment::get().getESMStore()->get<ESM::GameSetting>();
 
-        if (mTotalBalance->getValue() == 0) mCurrentBalance = 0;
+        if (mTotalBalance->getValue() == 0)
+            mCurrentBalance = 0;
 
         // were there any items traded at all?
         const std::vector<ItemStack>& playerBought = playerItemModel->getItemsBorrowedToUs();
@@ -273,8 +356,7 @@ namespace MWGui
         if (playerBought.empty() && merchantBought.empty())
         {
             // user notification
-            MWBase::Environment::get().getWindowManager()->
-                messageBox("#{sBarterDialog11}");
+            MWBase::Environment::get().getWindowManager()->messageBox("#{sBarterDialog11}");
             return;
         }
 
@@ -285,8 +367,7 @@ namespace MWGui
         if (mCurrentBalance < 0 && playerGold < std::abs(mCurrentBalance))
         {
             // user notification
-            MWBase::Environment::get().getWindowManager()->
-                messageBox("#{sBarterDialog1}");
+            MWBase::Environment::get().getWindowManager()->messageBox("#{sBarterDialog1}");
             return;
         }
 
@@ -294,21 +375,22 @@ namespace MWGui
         if (mCurrentBalance > 0 && getMerchantGold() < mCurrentBalance)
         {
             // user notification
-            MWBase::Environment::get().getWindowManager()->
-                messageBox("#{sBarterDialog2}");
+            MWBase::Environment::get().getWindowManager()->messageBox("#{sBarterDialog2}");
             return;
         }
 
         // check if the player is attempting to sell back an item stolen from this actor
         for (const ItemStack& itemStack : merchantBought)
         {
-            if (MWBase::Environment::get().getMechanicsManager()->isItemStolenFrom(itemStack.mBase.getCellRef().getRefId(), mPtr))
+            if (MWBase::Environment::get().getMechanicsManager()->isItemStolenFrom(
+                    itemStack.mBase.getCellRef().getRefId(), mPtr))
             {
                 std::string msg = gmst.find("sNotifyMessage49")->mValue.getString();
                 msg = Misc::StringUtils::format(msg, itemStack.mBase.getClass().getName(itemStack.mBase));
                 MWBase::Environment::get().getWindowManager()->messageBox(msg);
 
-                MWBase::Environment::get().getMechanicsManager()->confiscateStolenItemToOwner(player, itemStack.mBase, mPtr, itemStack.mCount);
+                MWBase::Environment::get().getMechanicsManager()->confiscateStolenItemToOwner(
+                    player, itemStack.mBase, mPtr, itemStack.mCount);
 
                 onCancelButtonClicked(mCancelButton);
                 MWBase::Environment::get().getWindowManager()->exitCurrentGuiMode();
@@ -316,21 +398,21 @@ namespace MWGui
             }
         }
 
-        bool offerAccepted = mTrading.haggle(player, mPtr, mCurrentBalance, mCurrentMerchantOffer);
+        bool offerAccepted = haggle(player, mPtr, mCurrentBalance, mCurrentMerchantOffer);
 
         // apply disposition change if merchant is NPC
-        if ( mPtr.getClass().isNpc() ) {
-            int dispositionDelta = offerAccepted
-                ? gmst.find("iBarterSuccessDisposition")->mValue.getInteger()
-                : gmst.find("iBarterFailDisposition")->mValue.getInteger();
+        if (mPtr.getClass().isNpc())
+        {
+            int dispositionDelta = offerAccepted ? gmst.find("iBarterSuccessDisposition")->mValue.getInteger()
+                                                 : gmst.find("iBarterFailDisposition")->mValue.getInteger();
 
             MWBase::Environment::get().getDialogueManager()->applyBarterDispositionChange(dispositionDelta);
         }
 
         // display message on haggle failure
-        if ( !offerAccepted ) {
-            MWBase::Environment::get().getWindowManager()->
-                messageBox("#{sNotifyMessage9}");
+        if (!offerAccepted)
+        {
+            MWBase::Environment::get().getWindowManager()->messageBox("#{sNotifyMessage9}");
             return;
         }
 
@@ -343,16 +425,16 @@ namespace MWGui
         {
             addOrRemoveGold(mCurrentBalance, player);
             mPtr.getClass().getCreatureStats(mPtr).setGoldPool(
-                        mPtr.getClass().getCreatureStats(mPtr).getGoldPool() - mCurrentBalance );
+                mPtr.getClass().getCreatureStats(mPtr).getGoldPool() - mCurrentBalance);
         }
 
         eventTradeDone();
 
-        MWBase::Environment::get().getWindowManager()->playSound("Item Gold Up");
+        MWBase::Environment::get().getWindowManager()->playSound(ESM::RefId::stringRefId("Item Gold Up"));
         MWBase::Environment::get().getWindowManager()->removeGuiMode(GM_Barter);
     }
 
-    void TradeWindow::onAccept(MyGUI::EditBox *sender)
+    void TradeWindow::onAccept(MyGUI::EditBox* sender)
     {
         onOfferButtonClicked(sender);
 
@@ -372,9 +454,10 @@ namespace MWGui
         updateLabels();
     }
 
-    void TradeWindow::addRepeatController(MyGUI::Widget *widget)
+    void TradeWindow::addRepeatController(MyGUI::Widget* widget)
     {
-        MyGUI::ControllerItem* item = MyGUI::ControllerManager::getInstance().createItem(MyGUI::ControllerRepeatClick::getClassTypeName());
+        MyGUI::ControllerItem* item
+            = MyGUI::ControllerManager::getInstance().createItem(MyGUI::ControllerRepeatClick::getClassTypeName());
         MyGUI::ControllerRepeatClick* controller = static_cast<MyGUI::ControllerRepeatClick*>(item);
         controller->eventRepeatClick += newDelegate(this, &TradeWindow::onRepeatClick);
         MyGUI::ControllerManager::getInstance().addItem(widget, controller);
@@ -400,7 +483,7 @@ namespace MWGui
             onDecreaseButtonTriggered();
     }
 
-    void TradeWindow::onBalanceButtonReleased(MyGUI::Widget *_sender, int _left, int _top, MyGUI::MouseButton _id)
+    void TradeWindow::onBalanceButtonReleased(MyGUI::Widget* _sender, int _left, int _top, MyGUI::MouseButton _id)
     {
         MyGUI::ControllerManager::getInstance().removeItem(_sender);
     }
@@ -423,19 +506,26 @@ namespace MWGui
     void TradeWindow::onIncreaseButtonTriggered()
     {
         // prevent overflows, and prevent entering INT_MIN since abs(INT_MIN) is undefined
-        if (mCurrentBalance == std::numeric_limits<int>::max() || mCurrentBalance == std::numeric_limits<int>::min()+1)
+        if (mCurrentBalance == std::numeric_limits<int>::max()
+            || mCurrentBalance == std::numeric_limits<int>::min() + 1)
             return;
-        if (mTotalBalance->getValue() == 0) mCurrentBalance = 0;
-        if (mCurrentBalance < 0) mCurrentBalance -= 1;
-        else mCurrentBalance += 1;
+        if (mTotalBalance->getValue() == 0)
+            mCurrentBalance = 0;
+        if (mCurrentBalance < 0)
+            mCurrentBalance -= 1;
+        else
+            mCurrentBalance += 1;
         updateLabels();
     }
 
     void TradeWindow::onDecreaseButtonTriggered()
     {
-        if (mTotalBalance->getValue() == 0) mCurrentBalance = 0;
-        if (mCurrentBalance < 0) mCurrentBalance += 1;
-        else mCurrentBalance -= 1;
+        if (mTotalBalance->getValue() == 0)
+            mCurrentBalance = 0;
+        if (mCurrentBalance < 0)
+            mCurrentBalance += 1;
+        else
+            mCurrentBalance -= 1;
         updateLabels();
     }
 
@@ -445,7 +535,8 @@ namespace MWGui
         int playerGold = player.getClass().getContainerStore(player).count(MWWorld::ContainerStore::sGoldId);
         mPlayerGold->setCaptionWithReplacing("#{sYourGold} " + MyGUI::utility::toString(playerGold));
 
-        TradeItemModel* playerTradeModel = MWBase::Environment::get().getWindowManager()->getInventoryWindow()->getTradeModel();
+        TradeItemModel* playerTradeModel
+            = MWBase::Environment::get().getWindowManager()->getInventoryWindow()->getTradeModel();
         const std::vector<ItemStack>& playerBorrowed = playerTradeModel->getItemsBorrowedToUs();
         const std::vector<ItemStack>& merchantBorrowed = mTradeModel->getItemsBorrowedToUs();
 
@@ -470,7 +561,8 @@ namespace MWGui
 
     void TradeWindow::updateOffer()
     {
-        TradeItemModel* playerTradeModel = MWBase::Environment::get().getWindowManager()->getInventoryWindow()->getTradeModel();
+        TradeItemModel* playerTradeModel
+            = MWBase::Environment::get().getWindowManager()->getInventoryWindow()->getTradeModel();
 
         int merchantOffer = 0;
 
@@ -482,8 +574,10 @@ namespace MWGui
         for (const ItemStack& itemStack : playerBorrowed)
         {
             const int basePrice = getEffectiveValue(itemStack.mBase, itemStack.mCount);
-            const int cap = static_cast<int>(std::max(1.f, 0.75f * basePrice)); // Minimum buying price -- 75% of the base
-            const int buyingPrice = MWBase::Environment::get().getMechanicsManager()->getBarterOffer(mPtr, basePrice, true);
+            const int cap
+                = static_cast<int>(std::max(1.f, 0.75f * basePrice)); // Minimum buying price -- 75% of the base
+            const int buyingPrice
+                = MWBase::Environment::get().getMechanicsManager()->getBarterOffer(mPtr, basePrice, true);
             merchantOffer -= std::max(cap, buyingPrice);
         }
 
@@ -491,8 +585,10 @@ namespace MWGui
         for (const ItemStack& itemStack : merchantBorrowed)
         {
             const int basePrice = getEffectiveValue(itemStack.mBase, itemStack.mCount);
-            const int cap = static_cast<int>(std::max(1.f, 0.75f * basePrice)); // Maximum selling price -- 75% of the base
-            const int sellingPrice = MWBase::Environment::get().getMechanicsManager()->getBarterOffer(mPtr, basePrice, false);
+            const int cap
+                = static_cast<int>(std::max(1.f, 0.75f * basePrice)); // Maximum selling price -- 75% of the base
+            const int sellingPrice
+                = MWBase::Environment::get().getMechanicsManager()->getBarterOffer(mPtr, basePrice, false);
             merchantOffer += mPtr.getClass().isNpc() ? std::min(cap, sellingPrice) : sellingPrice;
         }
 
@@ -514,7 +610,8 @@ namespace MWGui
 
     void TradeWindow::onReferenceUnavailable()
     {
-        // remove both Trade and Dialogue (since you always trade with the NPC/creature that you have previously talked to)
+        // remove both Trade and Dialogue (since you always trade with the NPC/creature that you have previously talked
+        // to)
         MWBase::Environment::get().getWindowManager()->removeGuiMode(GM_Barter);
         MWBase::Environment::get().getWindowManager()->exitCurrentGuiMode();
     }
@@ -543,7 +640,7 @@ namespace MWGui
 
     void TradeWindow::onDeleteCustomData(const MWWorld::Ptr& ptr)
     {
-        if(mTradeModel && mTradeModel->usesContainer(ptr))
+        if (mTradeModel && mTradeModel->usesContainer(ptr))
             MWBase::Environment::get().getWindowManager()->removeGuiMode(GM_Barter);
     }
 }
