@@ -1,10 +1,15 @@
 #ifndef MWGUI_TRAININGWINDOW_H
 #define MWGUI_TRAININGWINDOW_H
 
-#include "windowbase.hpp"
 #include "referenceinterface.hpp"
 #include "timeadvancer.hpp"
 #include "waitdialog.hpp"
+#include "windowbase.hpp"
+
+namespace MWMechanics
+{
+    class NpcStats;
+}
 
 namespace MWGui
 {
@@ -14,27 +19,33 @@ namespace MWGui
     public:
         TrainingWindow();
 
-        virtual void onOpen();
+        void onOpen() override;
 
-        bool exit();
+        bool exit() override;
 
-        void setPtr(const MWWorld::Ptr& actor);
+        void setPtr(const MWWorld::Ptr& actor) override;
 
-        void onFrame(float dt);
+        void onFrame(float dt) override;
 
         WindowBase* getProgressBar() { return &mProgressBar; }
 
-        void clear() { resetReference(); }
+        void clear() override { resetReference(); }
+
+        std::string_view getWindowIdForLua() const override { return "Training"; }
 
     protected:
-        virtual void onReferenceUnavailable ();
+        void onReferenceUnavailable() override;
 
-        void onCancelButtonClicked (MyGUI::Widget* sender);
+        void onCancelButtonClicked(MyGUI::Widget* sender);
         void onTrainingSelected(MyGUI::Widget* sender);
 
         void onTrainingProgressChanged(int cur, int total);
         void onTrainingFinished();
 
+        // Retrieve the base skill value if the setting 'training skills based on base skill' is set;
+        // otherwise returns the modified skill
+        float getSkillForTraining(const MWMechanics::NpcStats& stats, ESM::RefId id) const;
+
         MyGUI::Widget* mTrainingOptions;
         MyGUI::Button* mCancelButton;
         MyGUI::TextBox* mPlayerGold;
