@@ -1,32 +1,34 @@
 #include "travelwindow.hpp"
 
 #include <MyGUI_Button.h>
-#include <MyGUI_ScrollView.h>
 #include <MyGUI_Gui.h>
+#include <MyGUI_ScrollView.h>
 
-#include <components/settings/settings.hpp>
+#include <components/esm3/loadcrea.hpp>
 #include <components/esm3/loadgmst.hpp>
+#include <components/misc/strings/conversion.hpp>
+#include <components/settings/values.hpp>
 
 #include "../mwbase/environment.hpp"
-#include "../mwbase/world.hpp"
-#include "../mwbase/windowmanager.hpp"
 #include "../mwbase/mechanicsmanager.hpp"
+#include "../mwbase/windowmanager.hpp"
+#include "../mwbase/world.hpp"
 
-#include "../mwworld/class.hpp"
-#include "../mwworld/containerstore.hpp"
 #include "../mwworld/actionteleport.hpp"
 #include "../mwworld/cellstore.hpp"
-#include "../mwworld/cellutils.hpp"
-#include "../mwworld/store.hpp"
+#include "../mwworld/class.hpp"
+#include "../mwworld/containerstore.hpp"
 #include "../mwworld/esmstore.hpp"
+#include "../mwworld/store.hpp"
+#include "../mwworld/worldmodel.hpp"
 
 #include "../mwmechanics/actorutil.hpp"
 #include "../mwmechanics/creaturestats.hpp"
 
 namespace MWGui
 {
-    TravelWindow::TravelWindow() :
-        WindowBase("openmw_travel_window.layout")
+    TravelWindow::TravelWindow()
+        : WindowBase("openmw_travel_window.layout")
         , mCurrentY(0)
     {
         getWidget(mCancelButton, "CancelButton");
@@ -37,24 +39,19 @@ namespace MWGui
 
         mCancelButton->eventMouseButtonClick += MyGUI::newDelegate(this, &TravelWindow::onCancelButtonClicked);
 
-        mDestinations->setCoord(450/2-mDestinations->getTextSize().width/2,
-                          mDestinations->getTop(),
-                          mDestinations->getTextSize().width,
-                          mDestinations->getHeight());
-        mSelect->setCoord(8,
-                          mSelect->getTop(),
-                          mSelect->getTextSize().width,
-                          mSelect->getHeight());
+        mDestinations->setCoord(450 / 2 - mDestinations->getTextSize().width / 2, mDestinations->getTop(),
+            mDestinations->getTextSize().width, mDestinations->getHeight());
+        mSelect->setCoord(8, mSelect->getTop(), mSelect->getTextSize().width, mSelect->getHeight());
     }
 
-    void TravelWindow::addDestination(const std::string& name, const ESM::Position &pos, bool interior)
+    void TravelWindow::addDestination(const ESM::RefId& name, const ESM::Position& pos, bool interior)
     {
         int price;
 
-        const MWWorld::Store<ESM::GameSetting> &gmst =
-            MWBase::Environment::get().getWorld()->getStore().get<ESM::GameSetting>();
+        const MWWorld::Store<ESM::GameSetting>& gmst
+            = MWBase::Environment::get().getESMStore()->get<ESM::GameSetting>();
 
-        MWWorld::Ptr player = MWBase::Environment::get().getWorld ()->getPlayerPtr();
+        MWWorld::Ptr player = MWBase::Environment::get().getWorld()->getPlayerPtr();
         int playerGold = player.getClass().getContainerStore(player).count(MWWorld::ContainerStore::sGoldId);
 
         if (!mPtr.getCell()->isExterior())
@@ -64,7 +61,8 @@ namespace MWGui
         else
         {
             ESM::Position PlayerPos = player.getRefData().getPosition();
-            float d = sqrt(pow(pos.pos[0] - PlayerPos.pos[0], 2) + pow(pos.pos[1] - PlayerPos.pos[1], 2) + pow(pos.pos[2] - PlayerPos.pos[2], 2));
+            float d = sqrt(pow(pos.pos[0] - PlayerPos.pos[0], 2) + pow(pos.pos[1] - PlayerPos.pos[1], 2)
+                + pow(pos.pos[2] - PlayerPos.pos[2], 2));
             float fTravelMult = gmst.find("fTravelMult")->mValue.getFloat();
             if (fTravelMult != 0)
                 price = static_cast<int>(d / fTravelMult);
@@ -82,28 +80,31 @@ namespace MWGui
         // Apply followers cost, unlike vanilla the first follower doesn't travel for free
         price *= 1 + static_cast<int>(followers.size());
 
-        int lineHeight = MWBase::Environment::get().getWindowManager()->getFontHeight() + 2;
+        const int lineHeight = Settings::gui().mFontSize + 2;
 
-        MyGUI::Button* toAdd = mDestinationsView->createWidget<MyGUI::Button>("SandTextButton", 0, mCurrentY, 200, lineHeight, MyGUI::Align::Default);
+        MyGUI::Button* toAdd = mDestinationsView->createWidget<MyGUI::Button>(
+            "SandTextButton", 0, mCurrentY, 200, lineHeight, MyGUI::Align::Default);
         toAdd->setEnabled(price <= playerGold);
         mCurrentY += lineHeight;
-        if(interior)
-            toAdd->setUserString("interior","y");
+        if (interior)
+            toAdd->setUserString("interior", "y");
         else
-            toAdd->setUserString("interior","n");
+            toAdd->setUserString("interior", "n");
 
+        const std::string& nameString = name.getRefIdString();
         toAdd->setUserString("price", std::to_string(price));
-        toAdd->setCaptionWithReplacing("#{sCell=" + name + "}   -   " + MyGUI::utility::toString(price)+"#{sgp}");
-        toAdd->setSize(mDestinationsView->getWidth(),lineHeight);
+        toAdd->setCaptionWithReplacing(
+            "#{sCell=" + nameString + "}   -   " + MyGUI::utility::toString(price) + "#{sgp}");
+        toAdd->setSize(mDestinationsView->getWidth(), lineHeight);
         toAdd->eventMouseWheel += MyGUI::newDelegate(this, &TravelWindow::onMouseWheel);
-        toAdd->setUserString("Destination", name);
+        toAdd->setUserString("Destination", nameString);
         toAdd->setUserData(pos);
         toAdd->eventMouseButtonClick += MyGUI::newDelegate(this, &TravelWindow::onTravelButtonClick);
     }
 
     void TravelWindow::clearDestinations()
     {
-        mDestinationsView->setViewOffset(MyGUI::IntPoint(0,0));
+        mDestinationsView->setViewOffset(MyGUI::IntPoint(0, 0));
         mCurrentY = 0;
         while (mDestinationsView->getChildCount())
             MyGUI::Gui::getInstance().destroyWidget(mDestinationsView->getChildAt(0));
@@ -111,6 +112,9 @@ namespace MWGui
 
     void TravelWindow::setPtr(const MWWorld::Ptr& actor)
     {
+        if (actor.isEmpty() || !actor.getClass().isActor())
+            throw std::runtime_error("Invalid argument in TravelWindow::setPtr");
+
         center();
         mPtr = actor;
         clearDestinations();
@@ -121,37 +125,38 @@ namespace MWGui
         else if (mPtr.getType() == ESM::Creature::sRecordId)
             transport = mPtr.get<ESM::Creature>()->mBase->getTransport();
 
-        for(unsigned int i = 0;i<transport.size();i++)
+        for (const auto& dest : transport)
         {
-            std::string cellname = transport[i].mCellName;
+            std::string_view cellname = dest.mCellName;
             bool interior = true;
-            const osg::Vec2i cellIndex = MWWorld::positionToCellIndex(transport[i].mPos.pos[0], transport[i].mPos.pos[1]);
-            if (cellname == "")
+            const ESM::ExteriorCellLocation cellIndex
+                = ESM::positionToExteriorCellLocation(dest.mPos.pos[0], dest.mPos.pos[1]);
+            if (cellname.empty())
             {
-                MWWorld::CellStore* cell = MWBase::Environment::get().getWorld()->getExterior(cellIndex.x(), cellIndex.y());
-                cellname = MWBase::Environment::get().getWorld()->getCellName(cell);
+                MWWorld::CellStore& cell = MWBase::Environment::get().getWorldModel()->getExterior(cellIndex);
+                cellname = MWBase::Environment::get().getWorld()->getCellName(&cell);
                 interior = false;
             }
-            addDestination(cellname,transport[i].mPos,interior);
+            addDestination(ESM::RefId::stringRefId(cellname), dest.mPos, interior);
         }
 
         updateLabels();
-        // Canvas size must be expressed with VScroll disabled, otherwise MyGUI would expand the scroll area when the scrollbar is hidden
+        // Canvas size must be expressed with VScroll disabled, otherwise MyGUI would expand the scroll area when the
+        // scrollbar is hidden
         mDestinationsView->setVisibleVScroll(false);
-        mDestinationsView->setCanvasSize (MyGUI::IntSize(mDestinationsView->getWidth(), std::max(mDestinationsView->getHeight(), mCurrentY)));
+        mDestinationsView->setCanvasSize(
+            MyGUI::IntSize(mDestinationsView->getWidth(), std::max(mDestinationsView->getHeight(), mCurrentY)));
         mDestinationsView->setVisibleVScroll(true);
     }
 
     void TravelWindow::onTravelButtonClick(MyGUI::Widget* _sender)
     {
-        std::istringstream iss(_sender->getUserString("price"));
-        int price;
-        iss >> price;
+        const int price = Misc::StringUtils::toNumeric<int>(_sender->getUserString("price"), 0);
 
         MWWorld::Ptr player = MWMechanics::getPlayer();
         int playerGold = player.getClass().getContainerStore(player).count(MWWorld::ContainerStore::sGoldId);
 
-        if (playerGold<price)
+        if (playerGold < price)
             return;
 
         // Set "traveling" flag, so GetPCTraveling can detect teleportation.
@@ -160,9 +165,9 @@ namespace MWGui
 
         if (!mPtr.getCell()->isExterior())
             // Interior cell -> mages guild transport
-            MWBase::Environment::get().getWindowManager()->playSound("mysticism cast");
+            MWBase::Environment::get().getWindowManager()->playSound(ESM::RefId::stringRefId("mysticism cast"));
 
-        player.getClass().getContainerStore(player).remove(MWWorld::ContainerStore::sGoldId, price, player);
+        player.getClass().getContainerStore(player).remove(MWWorld::ContainerStore::sGoldId, price);
 
         // add gold to NPC trading gold pool
         MWMechanics::CreatureStats& npcStats = mPtr.getClass().getCreatureStats(mPtr);
@@ -170,14 +175,20 @@ namespace MWGui
 
         MWBase::Environment::get().getWindowManager()->fadeScreenOut(1);
         ESM::Position pos = *_sender->getUserData<ESM::Position>();
-        std::string cellname = _sender->getUserString("Destination");
+        std::string_view cellname = _sender->getUserString("Destination");
         bool interior = _sender->getUserString("interior") == "y";
         if (mPtr.getCell()->isExterior())
         {
             ESM::Position playerPos = player.getRefData().getPosition();
-            float d = (osg::Vec3f(pos.pos[0], pos.pos[1], 0) - osg::Vec3f(playerPos.pos[0], playerPos.pos[1], 0)).length();
-            int hours = static_cast<int>(d /MWBase::Environment::get().getWorld()->getStore().get<ESM::GameSetting>().find("fTravelTimeMult")->mValue.getFloat());
-            MWBase::Environment::get().getMechanicsManager ()->rest (hours, true);
+            float d
+                = (osg::Vec3f(pos.pos[0], pos.pos[1], 0) - osg::Vec3f(playerPos.pos[0], playerPos.pos[1], 0)).length();
+            int hours = static_cast<int>(d
+                / MWBase::Environment::get()
+                      .getESMStore()
+                      ->get<ESM::GameSetting>()
+                      .find("fTravelTimeMult")
+                      ->mValue.getFloat());
+            MWBase::Environment::get().getMechanicsManager()->rest(hours, true);
             MWBase::Environment::get().getWorld()->advanceTime(hours);
         }
 
@@ -185,9 +196,11 @@ namespace MWGui
         MWBase::Environment::get().getWindowManager()->exitCurrentGuiMode();
 
         MWBase::Environment::get().getWindowManager()->fadeScreenOut(1);
+        const ESM::ExteriorCellLocation posCell = ESM::positionToExteriorCellLocation(pos.pos[0], pos.pos[1]);
+        ESM::RefId cellId = ESM::Cell::generateIdForCell(!interior, cellname, posCell.mX, posCell.mY);
 
         // Teleports any followers, too.
-        MWWorld::ActionTeleport action(interior ? cellname : "", pos, true);
+        MWWorld::ActionTeleport action(cellId, pos, true);
         action.execute(player);
 
         MWBase::Environment::get().getWindowManager()->fadeScreenOut(0);
@@ -201,14 +214,11 @@ namespace MWGui
 
     void TravelWindow::updateLabels()
     {
-        MWWorld::Ptr player = MWBase::Environment::get().getWorld ()->getPlayerPtr();
+        MWWorld::Ptr player = MWBase::Environment::get().getWorld()->getPlayerPtr();
         int playerGold = player.getClass().getContainerStore(player).count(MWWorld::ContainerStore::sGoldId);
 
         mPlayerGold->setCaptionWithReplacing("#{sGold}: " + MyGUI::utility::toString(playerGold));
-        mPlayerGold->setCoord(8,
-                              mPlayerGold->getTop(),
-                              mPlayerGold->getTextSize().width,
-                              mPlayerGold->getHeight());
+        mPlayerGold->setCoord(8, mPlayerGold->getTop(), mPlayerGold->getTextSize().width, mPlayerGold->getHeight());
     }
 
     void TravelWindow::onReferenceUnavailable()
@@ -219,10 +229,10 @@ namespace MWGui
 
     void TravelWindow::onMouseWheel(MyGUI::Widget* _sender, int _rel)
     {
-        if (mDestinationsView->getViewOffset().top + _rel*0.3f > 0)
+        if (mDestinationsView->getViewOffset().top + _rel * 0.3f > 0)
             mDestinationsView->setViewOffset(MyGUI::IntPoint(0, 0));
         else
-            mDestinationsView->setViewOffset(MyGUI::IntPoint(0, static_cast<int>(mDestinationsView->getViewOffset().top + _rel*0.3f)));
+            mDestinationsView->setViewOffset(
+                MyGUI::IntPoint(0, static_cast<int>(mDestinationsView->getViewOffset().top + _rel * 0.3f)));
     }
 }
-
