@@ -12,7 +12,7 @@ namespace MWGui
 {
     class DragAndDrop;
 
-    class WindowBase: public Layout
+    class WindowBase : public Layout
     {
     public:
         WindowBase(std::string_view parLayout);
@@ -20,7 +20,7 @@ namespace MWGui
         virtual MyGUI::Widget* getDefaultKeyFocus() { return nullptr; }
 
         // Events
-        typedef MyGUI::delegates::CMultiDelegate1<WindowBase*> EventHandle_WindowBase;
+        typedef MyGUI::delegates::MultiDelegate<WindowBase*> EventHandle_WindowBase;
 
         /// Open this object in the GUI, for windows that support it
         virtual void setPtr(const MWWorld::Ptr& ptr) {}
@@ -31,13 +31,13 @@ namespace MWGui
         /// Notify that window has been made visible
         virtual void onOpen() {}
         /// Notify that window has been hidden
-        virtual void onClose () {}
+        virtual void onClose() {}
         /// Gracefully exits the window
-        virtual bool exit() {return true;}
+        virtual bool exit() { return true; }
         /// Sets the visibility of the window
         void setVisible(bool visible) override;
         /// Returns the visibility state of the window
-        bool isVisible();
+        bool isVisible() const;
 
         void center();
 
@@ -49,11 +49,18 @@ namespace MWGui
 
         virtual void onDeleteCustomData(const MWWorld::Ptr& ptr) {}
 
+        virtual std::string_view getWindowIdForLua() const { return ""; }
+        void setDisabledByLua(bool disabled) { mDisabledByLua = disabled; }
+
+        static void clampWindowCoordinates(MyGUI::Window* window);
+
     protected:
         virtual void onTitleDoubleClicked();
 
     private:
         void onDoubleClick(MyGUI::Widget* _sender);
+
+        bool mDisabledByLua = false;
     };
 
     /*
@@ -65,7 +72,7 @@ namespace MWGui
         WindowModal(const std::string& parLayout);
         void onOpen() override;
         void onClose() override;
-        bool exit() override {return true;}
+        bool exit() override { return true; }
     };
 
     /// A window that cannot be the target of a drag&drop action.
