@@ -17,43 +17,29 @@ namespace MWInput
     class ActionManager
     {
     public:
-
-        ActionManager(BindingsManager* bindingsManager,
-            osgViewer::ScreenCaptureHandler::CaptureOperation* screenCaptureOperation,
-            osg::ref_ptr<osgViewer::Viewer> viewer,
+        ActionManager(BindingsManager* bindingsManager, osg::ref_ptr<osgViewer::Viewer> viewer,
             osg::ref_ptr<osgViewer::ScreenCaptureHandler> screenCaptureHandler);
 
-        void update(float dt, bool triedToMove);
+        void update(float dt);
 
         void executeAction(int action);
 
         bool checkAllowedToUseItems() const;
 
         void toggleMainMenu();
-        void toggleSpell();
-        void toggleWeapon();
-        void toggleInventory();
         void toggleConsole();
         void screenshot();
-        void toggleJournal();
         void activate();
-        void toggleWalking();
-        void toggleSneaking();
-        void toggleAutoMove();
         void rest();
         void quickLoad();
         void quickSave();
 
-        void quickKey (int index);
-        void showQuickKeysMenu();
+        void quickKey(int index);
 
         void resetIdleTime();
         float getIdleTime() const { return mTimeIdle; }
 
-        bool isAlwaysRunActive() const { return mAlwaysRunActive; };
-        bool isSneaking() const { return mSneaking; };
-
-        void setAttemptJump(bool enabled) { mAttemptJump = enabled; }
+        bool isSneaking() const;
 
     private:
         void handleGuiArrowKey(int action);
@@ -61,11 +47,6 @@ namespace MWInput
         BindingsManager* mBindingsManager;
         osg::ref_ptr<osgViewer::Viewer> mViewer;
         osg::ref_ptr<osgViewer::ScreenCaptureHandler> mScreenCaptureHandler;
-        osgViewer::ScreenCaptureHandler::CaptureOperation* mScreenCaptureOperation;
-
-        bool mAlwaysRunActive;
-        bool mSneaking;
-        bool mAttemptJump;
 
         float mTimeIdle;
     };
