@@ -1,24 +1,26 @@
 #include "bindingsmanager.hpp"
 
+#include <filesystem>
+
 #include <MyGUI_EditBox.h>
 
 #include <extern/oics/ICSChannelListener.h>
 #include <extern/oics/ICSInputControlSystem.h>
 
+#include <components/debug/debuglog.hpp>
+#include <components/files/conversion.hpp>
 #include <components/sdlutil/sdlmappings.hpp>
 
 #include "../mwbase/environment.hpp"
 #include "../mwbase/inputmanager.hpp"
 #include "../mwbase/windowmanager.hpp"
-#include "../mwbase/world.hpp"
-
-#include "../mwworld/player.hpp"
 
 #include "actions.hpp"
 
 namespace MWInput
 {
-    static const int sFakeDeviceId = 1; //As we only support one controller at a time, use a fake deviceID so we don't lose bindings when switching controllers
+    static const int sFakeDeviceId = 1; // As we only support one controller at a time, use a fake deviceID so we don't
+                                        // lose bindings when switching controllers
 
     void clearAllKeyBindings(ICS::InputControlSystem* inputBinder, ICS::Control* control)
     {
@@ -27,7 +29,8 @@ namespace MWInput
             inputBinder->removeKeyBinding(inputBinder->getKeyBinding(control, ICS::Control::INCREASE));
         if (inputBinder->getMouseButtonBinding(control, ICS::Control::INCREASE) != ICS_MAX_DEVICE_BUTTONS)
             inputBinder->removeMouseButtonBinding(inputBinder->getMouseButtonBinding(control, ICS::Control::INCREASE));
-        if (inputBinder->getMouseWheelBinding(control, ICS::Control::INCREASE) != ICS::InputControlSystem::MouseWheelClick::UNASSIGNED)
+        if (inputBinder->getMouseWheelBinding(control, ICS::Control::INCREASE)
+            != ICS::InputControlSystem::MouseWheelClick::UNASSIGNED)
             inputBinder->removeMouseWheelBinding(inputBinder->getMouseWheelBinding(control, ICS::Control::INCREASE));
     }
 
@@ -35,23 +38,24 @@ namespace MWInput
     {
         // right now we don't really need multiple bindings for the same action, so remove all others first
         if (inputBinder->getJoystickAxisBinding(control, sFakeDeviceId, ICS::Control::INCREASE) != SDL_SCANCODE_UNKNOWN)
-            inputBinder->removeJoystickAxisBinding(sFakeDeviceId, inputBinder->getJoystickAxisBinding(control, sFakeDeviceId, ICS::Control::INCREASE));
-        if (inputBinder->getJoystickButtonBinding(control, sFakeDeviceId, ICS::Control::INCREASE) != ICS_MAX_DEVICE_BUTTONS)
-            inputBinder->removeJoystickButtonBinding(sFakeDeviceId, inputBinder->getJoystickButtonBinding(control, sFakeDeviceId, ICS::Control::INCREASE));
+            inputBinder->removeJoystickAxisBinding(
+                sFakeDeviceId, inputBinder->getJoystickAxisBinding(control, sFakeDeviceId, ICS::Control::INCREASE));
+        if (inputBinder->getJoystickButtonBinding(control, sFakeDeviceId, ICS::Control::INCREASE)
+            != ICS_MAX_DEVICE_BUTTONS)
+            inputBinder->removeJoystickButtonBinding(
+                sFakeDeviceId, inputBinder->getJoystickButtonBinding(control, sFakeDeviceId, ICS::Control::INCREASE));
     }
 
     class InputControlSystem : public ICS::InputControlSystem
     {
     public:
-        InputControlSystem(const std::string& bindingsFile)
-            : ICS::InputControlSystem(bindingsFile, true, nullptr, nullptr, A_Last)
+        InputControlSystem(const std::filesystem::path& bindingsFile)
+            : ICS::InputControlSystem(Files::pathToUnicodeString(bindingsFile), true, nullptr, nullptr, A_Last)
         {
         }
     };
 
-    class BindingsListener :
-            public ICS::ChannelListener,
-            public ICS::DetectingBindingListener
+    class BindingsListener : public ICS::ChannelListener, public ICS::DetectingBindingListener
     {
     public:
         BindingsListener(ICS::InputControlSystem* inputBinder, BindingsManager* bindingsManager)
@@ -69,13 +73,13 @@ namespace MWInput
             mBindingsManager->actionValueChanged(action, currentValue, previousValue);
         }
 
-        void keyBindingDetected(ICS::InputControlSystem* ICS, ICS::Control* control
-            , SDL_Scancode key, ICS::Control::ControlChangingDirection direction) override
+        void keyBindingDetected(ICS::InputControlSystem* ICS, ICS::Control* control, SDL_Scancode key,
+            ICS::Control::ControlChangingDirection direction) override
         {
-            //Disallow binding escape key
-            if (key==SDL_SCANCODE_ESCAPE)
+            // Disallow binding escape key
+            if (key == SDL_SCANCODE_ESCAPE)
             {
-                //Stop binding if esc pressed
+                // Stop binding if esc pressed
                 mInputBinder->cancelDetectingBindingState();
                 MWBase::Environment::get().getWindowManager()->notifyInputActionBound();
                 return;
@@ -85,11 +89,11 @@ namespace MWInput
             if (key == SDL_SCANCODE_F3 || key == SDL_SCANCODE_F4 || key == SDL_SCANCODE_F10)
                 return;
 
-            #ifndef __APPLE__
+#ifndef __APPLE__
             // Disallow binding Windows/Meta keys
             if (key == SDL_SCANCODE_LGUI || key == SDL_SCANCODE_RGUI)
                 return;
-            #endif
+#endif
 
             if (!mDetectingKeyboard)
                 return;
@@ -100,15 +104,15 @@ namespace MWInput
             MWBase::Environment::get().getWindowManager()->notifyInputActionBound();
         }
 
-        void mouseAxisBindingDetected(ICS::InputControlSystem* ICS, ICS::Control* control
-            , ICS::InputControlSystem::NamedAxis axis, ICS::Control::ControlChangingDirection direction) override
+        void mouseAxisBindingDetected(ICS::InputControlSystem* ICS, ICS::Control* control,
+            ICS::InputControlSystem::NamedAxis axis, ICS::Control::ControlChangingDirection direction) override
         {
             // we don't want mouse movement bindings
             return;
         }
 
-        void mouseButtonBindingDetected(ICS::InputControlSystem* ICS, ICS::Control* control
-            , unsigned int button, ICS::Control::ControlChangingDirection direction) override
+        void mouseButtonBindingDetected(ICS::InputControlSystem* ICS, ICS::Control* control, unsigned int button,
+            ICS::Control::ControlChangingDirection direction) override
         {
             if (!mDetectingKeyboard)
                 return;
@@ -118,8 +122,8 @@ namespace MWInput
             MWBase::Environment::get().getWindowManager()->notifyInputActionBound();
         }
 
-        void mouseWheelBindingDetected(ICS::InputControlSystem* ICS, ICS::Control* control
-            , ICS::InputControlSystem::MouseWheelClick click, ICS::Control::ControlChangingDirection direction) override
+        void mouseWheelBindingDetected(ICS::InputControlSystem* ICS, ICS::Control* control,
+            ICS::InputControlSystem::MouseWheelClick click, ICS::Control::ControlChangingDirection direction) override
         {
             if (!mDetectingKeyboard)
                 return;
@@ -129,30 +133,30 @@ namespace MWInput
             MWBase::Environment::get().getWindowManager()->notifyInputActionBound();
         }
 
-        void joystickAxisBindingDetected(ICS::InputControlSystem* ICS, int deviceID, ICS::Control* control
-            , int axis, ICS::Control::ControlChangingDirection direction) override
+        void joystickAxisBindingDetected(ICS::InputControlSystem* ICS, int deviceID, ICS::Control* control, int axis,
+            ICS::Control::ControlChangingDirection direction) override
         {
-            //only allow binding to the trigers
+            // only allow binding to the trigers
             if (axis != SDL_CONTROLLER_AXIS_TRIGGERLEFT && axis != SDL_CONTROLLER_AXIS_TRIGGERRIGHT)
                 return;
             if (mDetectingKeyboard)
                 return;
 
             clearAllControllerBindings(mInputBinder, control);
-            control->setValue(0.5f); //axis bindings must start at 0.5
+            control->setValue(0.5f); // axis bindings must start at 0.5
             control->setInitialValue(0.5f);
             ICS::DetectingBindingListener::joystickAxisBindingDetected(ICS, deviceID, control, axis, direction);
             MWBase::Environment::get().getWindowManager()->notifyInputActionBound();
         }
 
-        void joystickButtonBindingDetected(ICS::InputControlSystem* ICS, int deviceID, ICS::Control* control
-            , unsigned int button, ICS::Control::ControlChangingDirection direction) override
+        void joystickButtonBindingDetected(ICS::InputControlSystem* ICS, int deviceID, ICS::Control* control,
+            unsigned int button, ICS::Control::ControlChangingDirection direction) override
         {
             if (mDetectingKeyboard)
                 return;
-            clearAllControllerBindings(mInputBinder,control);
+            clearAllControllerBindings(mInputBinder, control);
             control->setInitialValue(0.0f);
-            ICS::DetectingBindingListener::joystickButtonBindingDetected (ICS, deviceID, control, button, direction);
+            ICS::DetectingBindingListener::joystickButtonBindingDetected(ICS, deviceID, control, button, direction);
             MWBase::Environment::get().getWindowManager()->notifyInputActionBound();
         }
 
@@ -167,11 +171,11 @@ namespace MWInput
         bool mDetectingKeyboard;
     };
 
-    BindingsManager::BindingsManager(const std::string& userFile, bool userFileExists)
+    BindingsManager::BindingsManager(const std::filesystem::path& userFile, bool userFileExists)
         : mUserFile(userFile)
         , mDragDrop(false)
     {
-        std::string file = userFileExists ? userFile : "";
+        const auto file = userFileExists ? userFile : std::filesystem::path();
         mInputBinder = std::make_unique<InputControlSystem>(file);
         mListener = std::make_unique<BindingsListener>(mInputBinder.get(), this);
         mInputBinder->setDetectingBindingListener(mListener.get());
@@ -192,7 +196,23 @@ namespace MWInput
 
     BindingsManager::~BindingsManager()
     {
-        mInputBinder->save(mUserFile);
+        const std::string newFileName = Files::pathToUnicodeString(mUserFile) + ".new";
+        try
+        {
+            if (mInputBinder->save(newFileName))
+            {
+                std::filesystem::rename(Files::pathFromUnicodeString(newFileName), mUserFile);
+                Log(Debug::Info) << "Saved input bindings: " << mUserFile;
+            }
+            else
+            {
+                Log(Debug::Error) << "Failed to save input bindings to " << newFileName;
+            }
+        }
+        catch (const std::exception& e)
+        {
+            Log(Debug::Error) << "Failed to save input bindings to " << newFileName << ": " << e.what();
+        }
     }
 
     void BindingsManager::update(float dt)
@@ -203,10 +223,12 @@ namespace MWInput
 
     bool BindingsManager::isLeftOrRightButton(int action, bool joystick) const
     {
-        int mouseBinding = mInputBinder->getMouseButtonBinding(mInputBinder->getControl(action), ICS::Control::INCREASE);
+        int mouseBinding
+            = mInputBinder->getMouseButtonBinding(mInputBinder->getControl(action), ICS::Control::INCREASE);
         if (mouseBinding != ICS_MAX_DEVICE_BUTTONS)
             return true;
-        int buttonBinding = mInputBinder->getJoystickButtonBinding(mInputBinder->getControl(action), sFakeDeviceId, ICS::Control::INCREASE);
+        int buttonBinding = mInputBinder->getJoystickButtonBinding(
+            mInputBinder->getControl(action), sFakeDeviceId, ICS::Control::INCREASE);
         if (joystick && (buttonBinding == 0 || buttonBinding == 1))
             return true;
         return false;
@@ -214,13 +236,11 @@ namespace MWInput
 
     void BindingsManager::setPlayerControlsEnabled(bool enabled)
     {
-        int playerChannels[] = {A_AutoMove, A_AlwaysRun, A_ToggleWeapon,
-                                A_ToggleSpell, A_Rest, A_QuickKey1, A_QuickKey2,
-                                A_QuickKey3, A_QuickKey4, A_QuickKey5, A_QuickKey6,
-                                A_QuickKey7, A_QuickKey8, A_QuickKey9, A_QuickKey10,
-                                A_Use, A_Journal};
+        int playerChannels[] = { A_AutoMove, A_AlwaysRun, A_ToggleWeapon, A_ToggleSpell, A_Rest, A_QuickKey1,
+            A_QuickKey2, A_QuickKey3, A_QuickKey4, A_QuickKey5, A_QuickKey6, A_QuickKey7, A_QuickKey8, A_QuickKey9,
+            A_QuickKey10, A_Use, A_Journal };
 
-        for(int pc : playerChannels)
+        for (int pc : playerChannels)
         {
             mInputBinder->getChannel(pc)->setEnabled(enabled);
         }
@@ -231,23 +251,23 @@ namespace MWInput
         mInputBinder->setJoystickDeadZone(deadZone);
     }
 
-    float BindingsManager::getActionValue (int id) const
+    float BindingsManager::getActionValue(int id) const
     {
         return mInputBinder->getChannel(id)->getValue();
     }
 
-    bool BindingsManager::actionIsActive (int id) const
+    bool BindingsManager::actionIsActive(int id) const
     {
         return getActionValue(id) == 1.0;
     }
 
-    void BindingsManager::loadKeyDefaults (bool force)
+    void BindingsManager::loadKeyDefaults(bool force)
     {
         // using hardcoded key defaults is inevitable, if we want the configuration files to stay valid
         // across different versions of OpenMW (in the case where another input action is added)
         std::map<int, SDL_Scancode> defaultKeyBindings;
 
-        //Gets the Keyvalue from the Scancode; gives the button in the same place reguardless of keyboard format
+        // Gets the Keyvalue from the Scancode; gives the button in the same place reguardless of keyboard format
         defaultKeyBindings[A_Activate] = SDL_SCANCODE_SPACE;
         defaultKeyBindings[A_MoveBackward] = SDL_SCANCODE_S;
         defaultKeyBindings[A_MoveForward] = SDL_SCANCODE_W;
@@ -311,38 +331,41 @@ namespace MWInput
                 control = mInputBinder->getChannel(i)->getAttachedControls().front().control;
             }
 
-            if (!controlExists || force ||
-                    (mInputBinder->getKeyBinding(control, ICS::Control::INCREASE) == SDL_SCANCODE_UNKNOWN
-                      && mInputBinder->getMouseButtonBinding(control, ICS::Control::INCREASE) == ICS_MAX_DEVICE_BUTTONS
-                      && mInputBinder->getMouseWheelBinding(control, ICS::Control::INCREASE) == ICS::InputControlSystem::MouseWheelClick::UNASSIGNED))
+            if (!controlExists || force
+                || (mInputBinder->getKeyBinding(control, ICS::Control::INCREASE) == SDL_SCANCODE_UNKNOWN
+                    && mInputBinder->getMouseButtonBinding(control, ICS::Control::INCREASE) == ICS_MAX_DEVICE_BUTTONS
+                    && mInputBinder->getMouseWheelBinding(control, ICS::Control::INCREASE)
+                        == ICS::InputControlSystem::MouseWheelClick::UNASSIGNED))
             {
                 clearAllKeyBindings(mInputBinder.get(), control);
 
                 if (defaultKeyBindings.find(i) != defaultKeyBindings.end()
-                        && (force || !mInputBinder->isKeyBound(defaultKeyBindings[i])))
+                    && (force || !mInputBinder->isKeyBound(defaultKeyBindings[i])))
                 {
                     control->setInitialValue(0.0f);
                     mInputBinder->addKeyBinding(control, defaultKeyBindings[i], ICS::Control::INCREASE);
                 }
                 else if (defaultMouseButtonBindings.find(i) != defaultMouseButtonBindings.end()
-                         && (force || !mInputBinder->isMouseButtonBound(defaultMouseButtonBindings[i])))
+                    && (force || !mInputBinder->isMouseButtonBound(defaultMouseButtonBindings[i])))
                 {
                     control->setInitialValue(0.0f);
                     mInputBinder->addMouseButtonBinding(control, defaultMouseButtonBindings[i], ICS::Control::INCREASE);
                 }
                 else if (defaultMouseWheelBindings.find(i) != defaultMouseWheelBindings.end()
-                        && (force || !mInputBinder->isMouseWheelBound(defaultMouseWheelBindings[i])))
+                    && (force || !mInputBinder->isMouseWheelBound(defaultMouseWheelBindings[i])))
                 {
                     control->setInitialValue(0.f);
                     mInputBinder->addMouseWheelBinding(control, defaultMouseWheelBindings[i], ICS::Control::INCREASE);
                 }
 
-                if (i == A_LookLeftRight && !mInputBinder->isKeyBound(SDL_SCANCODE_KP_4) && !mInputBinder->isKeyBound(SDL_SCANCODE_KP_6))
+                if (i == A_LookLeftRight && !mInputBinder->isKeyBound(SDL_SCANCODE_KP_4)
+                    && !mInputBinder->isKeyBound(SDL_SCANCODE_KP_6))
                 {
                     mInputBinder->addKeyBinding(control, SDL_SCANCODE_KP_6, ICS::Control::INCREASE);
                     mInputBinder->addKeyBinding(control, SDL_SCANCODE_KP_4, ICS::Control::DECREASE);
                 }
-                if (i == A_LookUpDown && !mInputBinder->isKeyBound(SDL_SCANCODE_KP_8) && !mInputBinder->isKeyBound(SDL_SCANCODE_KP_2))
+                if (i == A_LookUpDown && !mInputBinder->isKeyBound(SDL_SCANCODE_KP_8)
+                    && !mInputBinder->isKeyBound(SDL_SCANCODE_KP_2))
                 {
                     mInputBinder->addKeyBinding(control, SDL_SCANCODE_KP_2, ICS::Control::INCREASE);
                     mInputBinder->addKeyBinding(control, SDL_SCANCODE_KP_8, ICS::Control::DECREASE);
@@ -360,7 +383,8 @@ namespace MWInput
         defaultButtonBindings[A_Activate] = SDL_CONTROLLER_BUTTON_A;
         defaultButtonBindings[A_ToggleWeapon] = SDL_CONTROLLER_BUTTON_X;
         defaultButtonBindings[A_ToggleSpell] = SDL_CONTROLLER_BUTTON_Y;
-        //defaultButtonBindings[A_QuickButtonsMenu] = SDL_GetButtonFromScancode(SDL_SCANCODE_F1); // Need to implement, should be ToggleSpell(5) AND Wait(9)
+        // defaultButtonBindings[A_QuickButtonsMenu] = SDL_GetButtonFromScancode(SDL_SCANCODE_F1); // Need to implement,
+        // should be ToggleSpell(5) AND Wait(9)
         defaultButtonBindings[A_Sneak] = SDL_CONTROLLER_BUTTON_LEFTSTICK;
         defaultButtonBindings[A_Journal] = SDL_CONTROLLER_BUTTON_LEFTSHOULDER;
         defaultButtonBindings[A_Rest] = SDL_CONTROLLER_BUTTON_RIGHTSHOULDER;
@@ -368,10 +392,6 @@ namespace MWInput
         defaultButtonBindings[A_Inventory] = SDL_CONTROLLER_BUTTON_B;
         defaultButtonBindings[A_GameMenu] = SDL_CONTROLLER_BUTTON_START;
         defaultButtonBindings[A_QuickSave] = SDL_CONTROLLER_BUTTON_GUIDE;
-        defaultButtonBindings[A_MoveForward] = SDL_CONTROLLER_BUTTON_DPAD_UP;
-        defaultButtonBindings[A_MoveLeft] = SDL_CONTROLLER_BUTTON_DPAD_LEFT;
-        defaultButtonBindings[A_MoveBackward] = SDL_CONTROLLER_BUTTON_DPAD_DOWN;
-        defaultButtonBindings[A_MoveRight] = SDL_CONTROLLER_BUTTON_DPAD_RIGHT;
 
         std::map<int, int> defaultAxisBindings;
         defaultAxisBindings[A_MoveForwardBackward] = SDL_CONTROLLER_AXIS_LEFTY;
@@ -390,7 +410,8 @@ namespace MWInput
                 float initial;
                 if (defaultAxisBindings.find(i) == defaultAxisBindings.end())
                     initial = 0.0f;
-                else initial = 0.5f;
+                else
+                    initial = 0.5f;
                 control = new ICS::Control(std::to_string(i), false, true, initial, ICS::ICS_MAX, ICS::ICS_MAX);
                 mInputBinder->addControl(control);
                 control->attachChannel(mInputBinder->getChannel(i), ICS::Channel::DIRECT);
@@ -400,39 +421,45 @@ namespace MWInput
                 control = mInputBinder->getChannel(i)->getAttachedControls().front().control;
             }
 
-            if (!controlExists || force || (mInputBinder->getJoystickAxisBinding(control, sFakeDeviceId, ICS::Control::INCREASE) == ICS::InputControlSystem::UNASSIGNED &&
-                mInputBinder->getJoystickButtonBinding(control, sFakeDeviceId, ICS::Control::INCREASE) == ICS_MAX_DEVICE_BUTTONS))
+            if (!controlExists || force
+                || (mInputBinder->getJoystickAxisBinding(control, sFakeDeviceId, ICS::Control::INCREASE)
+                        == ICS::InputControlSystem::UNASSIGNED
+                    && mInputBinder->getJoystickButtonBinding(control, sFakeDeviceId, ICS::Control::INCREASE)
+                        == ICS_MAX_DEVICE_BUTTONS))
             {
                 clearAllControllerBindings(mInputBinder.get(), control);
 
                 if (defaultButtonBindings.find(i) != defaultButtonBindings.end()
-                        && (force || !mInputBinder->isJoystickButtonBound(sFakeDeviceId, defaultButtonBindings[i])))
+                    && (force || !mInputBinder->isJoystickButtonBound(sFakeDeviceId, defaultButtonBindings[i])))
                 {
                     control->setInitialValue(0.0f);
-                    mInputBinder->addJoystickButtonBinding(control, sFakeDeviceId, defaultButtonBindings[i], ICS::Control::INCREASE);
+                    mInputBinder->addJoystickButtonBinding(
+                        control, sFakeDeviceId, defaultButtonBindings[i], ICS::Control::INCREASE);
                 }
-                else if (defaultAxisBindings.find(i) != defaultAxisBindings.end() && (force || !mInputBinder->isJoystickAxisBound(sFakeDeviceId, defaultAxisBindings[i])))
+                else if (defaultAxisBindings.find(i) != defaultAxisBindings.end()
+                    && (force || !mInputBinder->isJoystickAxisBound(sFakeDeviceId, defaultAxisBindings[i])))
                 {
                     control->setValue(0.5f);
                     control->setInitialValue(0.5f);
-                    mInputBinder->addJoystickAxisBinding(control, sFakeDeviceId, defaultAxisBindings[i], ICS::Control::INCREASE);
+                    mInputBinder->addJoystickAxisBinding(
+                        control, sFakeDeviceId, defaultAxisBindings[i], ICS::Control::INCREASE);
                 }
             }
         }
     }
 
-    std::string BindingsManager::getActionDescription(int action)
+    std::string_view BindingsManager::getActionDescription(int action)
     {
         switch (action)
         {
             case A_Screenshot:
-                return "#{SettingsMenu:Screenshot}";
+                return "#{OMWEngine:Screenshot}";
             case A_ZoomIn:
-                return "#{SettingsMenu:CameraZoomIn}";
+                return "#{OMWEngine:CameraZoomIn}";
             case A_ZoomOut:
-                return "#{SettingsMenu:CameraZoomOut}";
+                return "#{OMWEngine:CameraZoomOut}";
             case A_ToggleHUD:
-                return "#{SettingsMenu:ToggleHUD}";
+                return "#{OMWEngine:ToggleHUD}";
             case A_Use:
                 return "#{sUse}";
             case A_Activate:
@@ -458,7 +485,7 @@ namespace MWInput
             case A_CycleWeaponRight:
                 return "#{sNextWeapon}";
             case A_Console:
-                return "#{sConsoleTitle}";
+                return "#{OMWEngine:ConsoleWindow}";
             case A_Run:
                 return "#{sRun}";
             case A_Sneak:
@@ -504,16 +531,16 @@ namespace MWInput
             case A_QuickLoad:
                 return "#{sQuickLoadCmd}";
             case A_TogglePostProcessorHUD:
-                return "#{SettingsMenu:TogglePostProcessorHUD}";
+                return "#{OMWEngine:TogglePostProcessorHUD}";
             default:
-                return std::string(); // not configurable
+                return {}; // not configurable
         }
     }
 
     std::string BindingsManager::getActionKeyBindingName(int action)
     {
         if (mInputBinder->getChannel(action)->getControlsCount() == 0)
-            return "#{sNone}";
+            return "#{Interface:None}";
 
         ICS::Control* c = mInputBinder->getChannel(action)->getAttachedControls().front().control;
 
@@ -536,52 +563,50 @@ namespace MWInput
                 case ICS::InputControlSystem::MouseWheelClick::LEFT:
                     return "Mouse Wheel Left";
                 default:
-                    return "#{sNone}";
+                    return "#{Interface:None}";
             }
         else
-            return "#{sNone}";
+            return "#{Interface:None}";
     }
 
     std::string BindingsManager::getActionControllerBindingName(int action)
     {
         if (mInputBinder->getChannel(action)->getControlsCount() == 0)
-            return "#{sNone}";
+            return "#{Interface:None}";
 
         ICS::Control* c = mInputBinder->getChannel(action)->getAttachedControls().front().control;
 
-        if (mInputBinder->getJoystickAxisBinding(c, sFakeDeviceId, ICS::Control::INCREASE) != ICS::InputControlSystem::UNASSIGNED)
-            return SDLUtil::sdlControllerAxisToString(mInputBinder->getJoystickAxisBinding(c, sFakeDeviceId, ICS::Control::INCREASE));
-        else if (mInputBinder->getJoystickButtonBinding(c, sFakeDeviceId, ICS::Control::INCREASE) != ICS_MAX_DEVICE_BUTTONS)
-            return SDLUtil::sdlControllerButtonToString(mInputBinder->getJoystickButtonBinding(c, sFakeDeviceId, ICS::Control::INCREASE));
+        if (mInputBinder->getJoystickAxisBinding(c, sFakeDeviceId, ICS::Control::INCREASE)
+            != ICS::InputControlSystem::UNASSIGNED)
+            return SDLUtil::sdlControllerAxisToString(
+                mInputBinder->getJoystickAxisBinding(c, sFakeDeviceId, ICS::Control::INCREASE));
+        else if (mInputBinder->getJoystickButtonBinding(c, sFakeDeviceId, ICS::Control::INCREASE)
+            != ICS_MAX_DEVICE_BUTTONS)
+            return SDLUtil::sdlControllerButtonToString(
+                mInputBinder->getJoystickButtonBinding(c, sFakeDeviceId, ICS::Control::INCREASE));
         else
-            return "#{sNone}";
+            return "#{Interface:None}";
     }
 
-    std::vector<int> BindingsManager::getActionKeySorting()
+    const std::initializer_list<int>& BindingsManager::getActionKeySorting()
     {
-        static const std::vector<int> actions
-        {
-            A_MoveForward, A_MoveBackward, A_MoveLeft, A_MoveRight, A_TogglePOV, A_ZoomIn, A_ZoomOut,
-            A_Run, A_AlwaysRun, A_Sneak, A_Activate, A_Use, A_ToggleWeapon, A_ToggleSpell,
-            A_CycleSpellLeft, A_CycleSpellRight, A_CycleWeaponLeft, A_CycleWeaponRight, A_AutoMove,
-            A_Jump, A_Inventory, A_Journal, A_Rest, A_Console, A_QuickSave, A_QuickLoad,
-            A_ToggleHUD, A_Screenshot, A_QuickKeysMenu, A_QuickKey1, A_QuickKey2, A_QuickKey3,
-            A_QuickKey4, A_QuickKey5, A_QuickKey6, A_QuickKey7, A_QuickKey8, A_QuickKey9, A_QuickKey10,
-            A_TogglePostProcessorHUD
-        };
+        static const std::initializer_list<int> actions{ A_MoveForward, A_MoveBackward, A_MoveLeft, A_MoveRight,
+            A_TogglePOV, A_ZoomIn, A_ZoomOut, A_Run, A_AlwaysRun, A_Sneak, A_Activate, A_Use, A_ToggleWeapon,
+            A_ToggleSpell, A_CycleSpellLeft, A_CycleSpellRight, A_CycleWeaponLeft, A_CycleWeaponRight, A_AutoMove,
+            A_Jump, A_Inventory, A_Journal, A_Rest, A_Console, A_QuickSave, A_QuickLoad, A_ToggleHUD, A_Screenshot,
+            A_QuickKeysMenu, A_QuickKey1, A_QuickKey2, A_QuickKey3, A_QuickKey4, A_QuickKey5, A_QuickKey6, A_QuickKey7,
+            A_QuickKey8, A_QuickKey9, A_QuickKey10, A_TogglePostProcessorHUD };
 
         return actions;
     }
-    std::vector<int> BindingsManager::getActionControllerSorting()
+    const std::initializer_list<int>& BindingsManager::getActionControllerSorting()
     {
-        static const std::vector<int> actions
-        {
-            A_TogglePOV, A_ZoomIn, A_ZoomOut, A_Sneak, A_Activate, A_Use, A_ToggleWeapon, A_ToggleSpell,
-            A_AutoMove, A_Jump, A_Inventory, A_Journal, A_Rest, A_QuickSave, A_QuickLoad, A_ToggleHUD,
-            A_Screenshot, A_QuickKeysMenu, A_QuickKey1, A_QuickKey2, A_QuickKey3, A_QuickKey4,
-            A_QuickKey5, A_QuickKey6, A_QuickKey7, A_QuickKey8, A_QuickKey9, A_QuickKey10,
-            A_CycleSpellLeft, A_CycleSpellRight, A_CycleWeaponLeft, A_CycleWeaponRight
-        };
+        static const std::initializer_list<int> actions{ A_MoveForward, A_MoveBackward, A_MoveLeft, A_MoveRight,
+            A_TogglePOV, A_ZoomIn, A_ZoomOut, A_Sneak, A_Activate, A_Use, A_ToggleWeapon, A_ToggleSpell, A_AutoMove,
+            A_Jump, A_Inventory, A_Journal, A_Rest, A_QuickSave, A_QuickLoad, A_ToggleHUD, A_Screenshot,
+            A_QuickKeysMenu, A_QuickKey1, A_QuickKey2, A_QuickKey3, A_QuickKey4, A_QuickKey5, A_QuickKey6, A_QuickKey7,
+            A_QuickKey8, A_QuickKey9, A_QuickKey10, A_CycleSpellLeft, A_CycleSpellRight, A_CycleWeaponLeft,
+            A_CycleWeaponRight };
 
         return actions;
     }
@@ -598,57 +623,57 @@ namespace MWInput
         return mInputBinder->detectingBindingState();
     }
 
-    void BindingsManager::mousePressed(const SDL_MouseButtonEvent &arg, int deviceID)
+    void BindingsManager::mousePressed(const SDL_MouseButtonEvent& arg, Uint8 deviceID)
     {
         mInputBinder->mousePressed(arg, deviceID);
     }
 
-    void BindingsManager::mouseReleased(const SDL_MouseButtonEvent &arg, int deviceID)
+    void BindingsManager::mouseReleased(const SDL_MouseButtonEvent& arg, Uint8 deviceID)
     {
         mInputBinder->mouseReleased(arg, deviceID);
     }
 
-    void BindingsManager::mouseMoved(const SDLUtil::MouseMotionEvent &arg)
+    void BindingsManager::mouseMoved(const SDLUtil::MouseMotionEvent& arg)
     {
         mInputBinder->mouseMoved(arg);
     }
 
-    void BindingsManager::mouseWheelMoved(const SDL_MouseWheelEvent &arg)
+    void BindingsManager::mouseWheelMoved(const SDL_MouseWheelEvent& arg)
     {
         mInputBinder->mouseWheelMoved(arg);
     }
 
-    void BindingsManager::keyPressed(const SDL_KeyboardEvent &arg)
+    void BindingsManager::keyPressed(const SDL_KeyboardEvent& arg)
     {
         mInputBinder->keyPressed(arg);
     }
 
-    void BindingsManager::keyReleased(const SDL_KeyboardEvent &arg)
+    void BindingsManager::keyReleased(const SDL_KeyboardEvent& arg)
     {
         mInputBinder->keyReleased(arg);
     }
 
-    void BindingsManager::controllerAdded(int deviceID, const SDL_ControllerDeviceEvent &arg)
+    void BindingsManager::controllerAdded(int deviceID, const SDL_ControllerDeviceEvent& arg)
     {
         mInputBinder->controllerAdded(deviceID, arg);
     }
 
-    void BindingsManager::controllerRemoved(const SDL_ControllerDeviceEvent &arg)
+    void BindingsManager::controllerRemoved(const SDL_ControllerDeviceEvent& arg)
     {
         mInputBinder->controllerRemoved(arg);
     }
 
-    void BindingsManager::controllerButtonPressed(int deviceID, const SDL_ControllerButtonEvent &arg)
+    void BindingsManager::controllerButtonPressed(int deviceID, const SDL_ControllerButtonEvent& arg)
     {
         mInputBinder->buttonPressed(deviceID, arg);
     }
 
-    void BindingsManager::controllerButtonReleased(int deviceID, const SDL_ControllerButtonEvent &arg)
+    void BindingsManager::controllerButtonReleased(int deviceID, const SDL_ControllerButtonEvent& arg)
     {
         mInputBinder->buttonReleased(deviceID, arg);
     }
 
-    void BindingsManager::controllerAxisMoved(int deviceID, const SDL_ControllerAxisEvent &arg)
+    void BindingsManager::controllerAxisMoved(int deviceID, const SDL_ControllerAxisEvent& arg)
     {
         mInputBinder->axisMoved(deviceID, arg);
     }
@@ -669,64 +694,25 @@ namespace MWInput
 
     void BindingsManager::actionValueChanged(int action, float currentValue, float previousValue)
     {
-        MWBase::Environment::get().getInputManager()->resetIdleTime();
+        auto manager = MWBase::Environment::get().getInputManager();
+        manager->resetIdleTime();
 
         if (mDragDrop && action != A_GameMenu && action != A_Inventory)
             return;
 
-        if ((previousValue == 1 || previousValue == 0) && (currentValue==1 || currentValue==0))
+        if (manager->joystickLastUsed() && manager->getControlSwitch("playercontrols"))
         {
-            //Is a normal button press, so don't change it at all
-        }
-        //Otherwise only trigger button presses as they go through specific points
-        else if (previousValue >= 0.8 && currentValue < 0.8)
-        {
-            currentValue = 0.0;
-            previousValue = 1.0;
-        }
-        else if (previousValue <= 0.6 && currentValue > 0.6)
-        {
-            currentValue = 1.0;
-            previousValue = 0.0;
-        }
-        else
-        {
-            //If it's not switching between those values, ignore the channel change.
-            return;
+            if (action == A_Use && actionIsActive(A_ToggleWeapon))
+                action = A_CycleWeaponRight;
+            else if (action == A_Use && actionIsActive(A_ToggleSpell))
+                action = A_CycleSpellRight;
+            else if (action == A_Jump && actionIsActive(A_ToggleWeapon))
+                action = A_CycleWeaponLeft;
+            else if (action == A_Jump && actionIsActive(A_ToggleSpell))
+                action = A_CycleSpellLeft;
         }
 
-        if (MWBase::Environment::get().getInputManager()->getControlSwitch("playercontrols"))
-        {
-            bool joystickUsed = MWBase::Environment::get().getInputManager()->joystickLastUsed();
-            if (action == A_Use)
-            {
-                if (joystickUsed && currentValue == 1.0 && actionIsActive(A_ToggleWeapon))
-                    action = A_CycleWeaponRight;
-
-                else if (joystickUsed && currentValue == 1.0 && actionIsActive(A_ToggleSpell))
-                    action = A_CycleSpellRight;
-
-                else
-                {
-                    MWWorld::Player& player = MWBase::Environment::get().getWorld()->getPlayer();
-                    MWMechanics::DrawState state = player.getDrawState();
-                    player.setAttackingOrSpell(currentValue != 0 && state != MWMechanics::DrawState::Nothing);
-                }
-            }
-            else if (action == A_Jump)
-            {
-                if (joystickUsed && currentValue == 1.0 && actionIsActive(A_ToggleWeapon))
-                    action = A_CycleWeaponLeft;
-
-                else if (joystickUsed && currentValue == 1.0 && actionIsActive(A_ToggleSpell))
-                    action = A_CycleSpellLeft;
-
-                else
-                    MWBase::Environment::get().getInputManager()->setAttemptJump(currentValue == 1.0 && previousValue == 0.0);
-            }
-        }
-
-        if (currentValue == 1)
-            MWBase::Environment::get().getInputManager()->executeAction(action);
+        if (previousValue <= 0.6 && currentValue > 0.6)
+            manager->executeAction(action);
     }
 }
