@@ -1,42 +1,39 @@
 #ifndef MWINPUT_MWCONTROLLERMANAGER_H
 #define MWINPUT_MWCONTROLLERMANAGER_H
 
+#include <array>
+#include <filesystem>
 #include <string>
 
-#include <components/settings/settings.hpp>
 #include <components/sdlutil/events.hpp>
+#include <components/settings/settings.hpp>
 
 namespace MWInput
 {
-    class ActionManager;
     class BindingsManager;
     class MouseManager;
 
     class ControllerManager : public SDLUtil::ControllerListener
     {
     public:
-        ControllerManager(BindingsManager* bindingsManager,
-            ActionManager* actionManager,
-            MouseManager* mouseManager,
-            const std::string& userControllerBindingsFile,
-            const std::string& controllerBindingsFile);
+        ControllerManager(BindingsManager* bindingsManager, MouseManager* mouseManager,
+            const std::filesystem::path& userControllerBindingsFile,
+            const std::filesystem::path& controllerBindingsFile);
 
         virtual ~ControllerManager() = default;
 
-        bool update(float dt);
+        void update(float dt);
 
-        void buttonPressed(int deviceID, const SDL_ControllerButtonEvent &arg) override;
-        void buttonReleased(int deviceID, const SDL_ControllerButtonEvent &arg) override;
-        void axisMoved(int deviceID, const SDL_ControllerAxisEvent &arg) override;
-        void controllerAdded(int deviceID, const SDL_ControllerDeviceEvent &arg) override;
-        void controllerRemoved(const SDL_ControllerDeviceEvent &arg) override;
+        void buttonPressed(int deviceID, const SDL_ControllerButtonEvent& arg) override;
+        void buttonReleased(int deviceID, const SDL_ControllerButtonEvent& arg) override;
+        void axisMoved(int deviceID, const SDL_ControllerAxisEvent& arg) override;
+        void controllerAdded(int deviceID, const SDL_ControllerDeviceEvent& arg) override;
+        void controllerRemoved(const SDL_ControllerDeviceEvent& arg) override;
 
         void touchpadMoved(int deviceId, const SDLUtil::TouchEvent& arg) override;
         void touchpadPressed(int deviceId, const SDLUtil::TouchEvent& arg) override;
         void touchpadReleased(int deviceId, const SDLUtil::TouchEvent& arg) override;
 
-        void processChangedSettings(const Settings::CategorySettingVector& changed);
-
         void setJoystickLastUsed(bool enabled) { mJoystickLastUsed = enabled; }
         bool joystickLastUsed() const { return mJoystickLastUsed; }
 
@@ -45,7 +42,7 @@ namespace MWInput
         void setGamepadGuiCursorEnabled(bool enabled) { mGamepadGuiCursorEnabled = enabled; }
         bool gamepadGuiCursorEnabled() const { return mGamepadGuiCursorEnabled; }
 
-        float getAxisValue(SDL_GameControllerAxis axis) const;  // returns value in range [-1, 1]
+        float getAxisValue(SDL_GameControllerAxis axis) const; // returns value in range [-1, 1]
         bool isButtonPressed(SDL_GameControllerButton button) const;
 
         bool isGyroAvailable() const;
@@ -53,23 +50,18 @@ namespace MWInput
 
     private:
         // Return true if GUI consumes input.
-        bool gamepadToGuiControl(const SDL_ControllerButtonEvent &arg);
-        bool gamepadToGuiControl(const SDL_ControllerAxisEvent &arg);
+        bool gamepadToGuiControl(const SDL_ControllerButtonEvent& arg);
+        bool gamepadToGuiControl(const SDL_ControllerAxisEvent& arg);
 
         void enableGyroSensor();
 
         BindingsManager* mBindingsManager;
-        ActionManager* mActionManager;
         MouseManager* mMouseManager;
 
-        bool mJoystickEnabled;
         bool mGyroAvailable;
-        float mGamepadCursorSpeed;
-        float mSneakToggleShortcutTimer;
         bool mGamepadGuiCursorEnabled;
         bool mGuiCursorEnabled;
         bool mJoystickLastUsed;
-        bool mSneakGamepadShortcut;
     };
 }
 #endif
