@@ -11,8 +11,6 @@
 #include "../mwbase/luamanager.hpp"
 #include "../mwbase/windowmanager.hpp"
 
-#include "../mwworld/player.hpp"
-
 #include "actions.hpp"
 #include "bindingsmanager.hpp"
 
@@ -23,7 +21,7 @@ namespace MWInput
     {
     }
 
-    void KeyboardManager::textInput(const SDL_TextInputEvent &arg)
+    void KeyboardManager::textInput(const SDL_TextInputEvent& arg)
     {
         MyGUI::UString ustring(&arg.text[0]);
         MyGUI::UString::utf32string utf32string = ustring.asUTF32();
@@ -31,21 +29,21 @@ namespace MWInput
             MyGUI::InputManager::getInstance().injectKeyPress(MyGUI::KeyCode::None, *it);
     }
 
-    void KeyboardManager::keyPressed(const SDL_KeyboardEvent &arg)
+    void KeyboardManager::keyPressed(const SDL_KeyboardEvent& arg)
     {
         // HACK: to make default keybinding for the console work without printing an extra "^" upon closing
         // This assumes that SDL_TextInput events always come *after* the key event
         // (which is somewhat reasonable, and hopefully true for all SDL platforms)
         auto kc = SDLUtil::sdlKeyToMyGUI(arg.keysym.sym);
         if (mBindingsManager->getKeyBinding(A_Console) == arg.keysym.scancode
-                && MWBase::Environment::get().getWindowManager()->isConsoleMode())
+            && MWBase::Environment::get().getWindowManager()->isConsoleMode())
             SDL_StopTextInput();
 
-        bool consumed = SDL_IsTextInputActive() &&  // Little trick to check if key is printable
-                        (!(SDLK_SCANCODE_MASK & arg.keysym.sym) &&
-                        // Don't trust isprint for symbols outside the extended ASCII range
-                        ((kc == MyGUI::KeyCode::None && arg.keysym.sym > 0xff) ||
-                        (arg.keysym.sym >= 0 && arg.keysym.sym <= 255 && std::isprint(arg.keysym.sym))));
+        bool consumed = SDL_IsTextInputActive() && // Little trick to check if key is printable
+            (!(SDLK_SCANCODE_MASK & arg.keysym.sym) &&
+                // Don't trust isprint for symbols outside the extended ASCII range
+                ((kc == MyGUI::KeyCode::None && arg.keysym.sym > 0xff)
+                    || (arg.keysym.sym >= 0 && arg.keysym.sym <= 255 && std::isprint(arg.keysym.sym))));
         if (kc != MyGUI::KeyCode::None && !mBindingsManager->isDetectingBindingState())
         {
             if (MWBase::Environment::get().getWindowManager()->injectKeyPress(kc, 0, arg.repeat))
@@ -63,13 +61,13 @@ namespace MWInput
         if (!consumed)
         {
             MWBase::Environment::get().getLuaManager()->inputEvent(
-                {MWBase::LuaManager::InputEvent::KeyPressed, arg.keysym});
+                { MWBase::LuaManager::InputEvent::KeyPressed, arg.keysym });
         }
 
         input->setJoystickLastUsed(false);
     }
 
-    void KeyboardManager::keyReleased(const SDL_KeyboardEvent &arg)
+    void KeyboardManager::keyReleased(const SDL_KeyboardEvent& arg)
     {
         MWBase::Environment::get().getInputManager()->setJoystickLastUsed(false);
         auto kc = SDLUtil::sdlKeyToMyGUI(arg.keysym.sym);
@@ -77,6 +75,7 @@ namespace MWInput
         if (!mBindingsManager->isDetectingBindingState())
             mBindingsManager->setPlayerControlsEnabled(!MyGUI::InputManager::getInstance().injectKeyRelease(kc));
         mBindingsManager->keyReleased(arg);
-        MWBase::Environment::get().getLuaManager()->inputEvent({MWBase::LuaManager::InputEvent::KeyReleased, arg.keysym});
+        MWBase::Environment::get().getLuaManager()->inputEvent(
+            { MWBase::LuaManager::InputEvent::KeyReleased, arg.keysym });
     }
 }
