@@ -4,15 +4,18 @@
 #include <MyGUI_InputManager.h>
 #include <MyGUI_RenderManager.h>
 
-#include <components/debug/debuglog.hpp>
 #include <components/sdlutil/sdlinputwrapper.hpp>
 #include <components/sdlutil/sdlmappings.hpp>
+#include <components/settings/values.hpp>
 
 #include "../mwbase/environment.hpp"
 #include "../mwbase/inputmanager.hpp"
+#include "../mwbase/luamanager.hpp"
 #include "../mwbase/windowmanager.hpp"
 #include "../mwbase/world.hpp"
 
+#include "../mwgui/settingswindow.hpp"
+
 #include "../mwworld/player.hpp"
 
 #include "actions.hpp"
@@ -20,13 +23,9 @@
 
 namespace MWInput
 {
-    MouseManager::MouseManager(BindingsManager* bindingsManager, SDLUtil::InputWrapper* inputWrapper, SDL_Window* window)
-        : mInvertX(Settings::Manager::getBool("invert x axis", "Input"))
-        , mInvertY(Settings::Manager::getBool("invert y axis", "Input"))
-        , mGrabCursor(Settings::Manager::getBool("grab cursor", "Input"))
-        , mCameraSensitivity(Settings::Manager::getFloat("camera sensitivity", "Input"))
-        , mCameraYMultiplier(Settings::Manager::getFloat("camera y multiplier", "Input"))
-        , mBindingsManager(bindingsManager)
+    MouseManager::MouseManager(
+        BindingsManager* bindingsManager, SDLUtil::InputWrapper* inputWrapper, SDL_Window* window)
+        : mBindingsManager(bindingsManager)
         , mInputWrapper(inputWrapper)
         , mGuiCursorX(0)
         , mGuiCursorY(0)
@@ -36,7 +35,7 @@ namespace MWInput
         , mMouseMoveX(0)
         , mMouseMoveY(0)
     {
-        int w,h;
+        int w, h;
         SDL_GetWindowSize(window, &w, &h);
 
         float uiScale = MWBase::Environment::get().getWindowManager()->getScalingFactor();
@@ -44,25 +43,7 @@ namespace MWInput
         mGuiCursorY = h / (2.f * uiScale);
     }
 
-    void MouseManager::processChangedSettings(const Settings::CategorySettingVector& changed)
-    {
-        for (const auto& setting : changed)
-        {
-            if (setting.first == "Input" && setting.second == "invert x axis")
-                mInvertX = Settings::Manager::getBool("invert x axis", "Input");
-
-            if (setting.first == "Input" && setting.second == "invert y axis")
-                mInvertY = Settings::Manager::getBool("invert y axis", "Input");
-
-            if (setting.first == "Input" && setting.second == "camera sensitivity")
-                mCameraSensitivity = Settings::Manager::getFloat("camera sensitivity", "Input");
-
-            if (setting.first == "Input" && setting.second == "grab cursor")
-                mGrabCursor = Settings::Manager::getBool("grab cursor", "Input");
-        }
-    }
-
-    void MouseManager::mouseMoved(const SDLUtil::MouseMotionEvent &arg)
+    void MouseManager::mouseMoved(const SDLUtil::MouseMotionEvent& arg)
     {
         mBindingsManager->mouseMoved(arg);
 
@@ -82,9 +63,12 @@ namespace MWInput
 
             mMouseWheel = static_cast<int>(arg.z);
 
-            MyGUI::InputManager::getInstance().injectMouseMove(static_cast<int>(mGuiCursorX), static_cast<int>(mGuiCursorY), mMouseWheel);
-            // FIXME: inject twice to force updating focused widget states (tooltips) resulting from changing the viewport by scroll wheel
-            MyGUI::InputManager::getInstance().injectMouseMove(static_cast<int>(mGuiCursorX), static_cast<int>(mGuiCursorY), mMouseWheel);
+            MyGUI::InputManager::getInstance().injectMouseMove(
+                static_cast<int>(mGuiCursorX), static_cast<int>(mGuiCursorY), mMouseWheel);
+            // FIXME: inject twice to force updating focused widget states (tooltips) resulting from changing the
+            // viewport by scroll wheel
+            MyGUI::InputManager::getInstance().injectMouseMove(
+                static_cast<int>(mGuiCursorX), static_cast<int>(mGuiCursorY), mMouseWheel);
 
             MWBase::Environment::get().getWindowManager()->setCursorActive(true);
         }
@@ -93,8 +77,10 @@ namespace MWInput
         {
             MWBase::World* world = MWBase::Environment::get().getWorld();
 
-            float x = arg.xrel * mCameraSensitivity * (mInvertX ? -1 : 1) / 256.f;
-            float y = arg.yrel * mCameraSensitivity * (mInvertY ? -1 : 1) * mCameraYMultiplier / 256.f;
+            const float cameraSensitivity = Settings::input().mCameraSensitivity;
+            float x = arg.xrel * cameraSensitivity * (Settings::input().mInvertXAxis ? -1 : 1) / 256.f;
+            float y = arg.yrel * cameraSensitivity * (Settings::input().mInvertYAxis ? -1 : 1)
+                * Settings::input().mCameraYMultiplier / 256.f;
 
             float rot[3];
             rot[0] = -y;
@@ -113,7 +99,7 @@ namespace MWInput
         }
     }
 
-    void MouseManager::mouseReleased(const SDL_MouseButtonEvent &arg, Uint8 id)
+    void MouseManager::mouseReleased(const SDL_MouseButtonEvent& arg, Uint8 id)
     {
         MWBase::Environment::get().getInputManager()->setJoystickLastUsed(false);
 
@@ -124,11 +110,9 @@ namespace MWInput
         else
         {
             bool guiMode = MWBase::Environment::get().getWindowManager()->isGuiMode();
-            guiMode = MyGUI::InputManager::getInstance().injectMouseRelease(
-                static_cast<int>(mGuiCursorX),
-                static_cast<int>(mGuiCursorY),
-                SDLUtil::sdlMouseButtonToMyGui(id)
-            ) && guiMode;
+            guiMode = MyGUI::InputManager::getInstance().injectMouseRelease(static_cast<int>(mGuiCursorX),
+                          static_cast<int>(mGuiCursorY), SDLUtil::sdlMouseButtonToMyGui(id))
+                && guiMode;
 
             if (mBindingsManager->isDetectingBindingState())
                 return; // don't allow same mouseup to bind as initiated bind
@@ -136,18 +120,25 @@ namespace MWInput
             mBindingsManager->setPlayerControlsEnabled(!guiMode);
             mBindingsManager->mouseReleased(arg, id);
         }
+
+        MWBase::Environment::get().getLuaManager()->inputEvent(
+            { MWBase::LuaManager::InputEvent::MouseButtonReleased, arg.button });
     }
 
-    void MouseManager::mouseWheelMoved(const SDL_MouseWheelEvent &arg)
+    void MouseManager::mouseWheelMoved(const SDL_MouseWheelEvent& arg)
     {
         MWBase::InputManager* input = MWBase::Environment::get().getInputManager();
         if (mBindingsManager->isDetectingBindingState() || !input->controlsDisabled())
+        {
             mBindingsManager->mouseWheelMoved(arg);
+        }
 
         input->setJoystickLastUsed(false);
+        MWBase::Environment::get().getLuaManager()->inputEvent({ MWBase::LuaManager::InputEvent::MouseWheel,
+            MWBase::LuaManager::InputEvent::WheelChange{ arg.x, arg.y } });
     }
 
-    void MouseManager::mousePressed(const SDL_MouseButtonEvent &arg, Uint8 id)
+    void MouseManager::mousePressed(const SDL_MouseButtonEvent& arg, Uint8 id)
     {
         MWBase::InputManager* input = MWBase::Environment::get().getInputManager();
         input->setJoystickLastUsed(false);
@@ -156,17 +147,16 @@ namespace MWInput
         if (id == SDL_BUTTON_LEFT || id == SDL_BUTTON_RIGHT) // MyGUI only uses these mouse events
         {
             guiMode = MWBase::Environment::get().getWindowManager()->isGuiMode();
-            guiMode = MyGUI::InputManager::getInstance().injectMousePress(
-                static_cast<int>(mGuiCursorX),
-                static_cast<int>(mGuiCursorY),
-                SDLUtil::sdlMouseButtonToMyGui(id)
-            ) && guiMode;
-            if (MyGUI::InputManager::getInstance().getMouseFocusWidget () != nullptr)
+            guiMode = MyGUI::InputManager::getInstance().injectMousePress(static_cast<int>(mGuiCursorX),
+                          static_cast<int>(mGuiCursorY), SDLUtil::sdlMouseButtonToMyGui(id))
+                && guiMode;
+            if (MyGUI::InputManager::getInstance().getMouseFocusWidget() != nullptr)
             {
-                MyGUI::Button* b = MyGUI::InputManager::getInstance().getMouseFocusWidget()->castType<MyGUI::Button>(false);
+                MyGUI::Button* b
+                    = MyGUI::InputManager::getInstance().getMouseFocusWidget()->castType<MyGUI::Button>(false);
                 if (b && b->getEnabled() && id == SDL_BUTTON_LEFT)
                 {
-                    MWBase::Environment::get().getWindowManager()->playSound("Menu Click");
+                    MWBase::Environment::get().getWindowManager()->playSound(ESM::RefId::stringRefId("Menu Click"));
                 }
             }
             MWBase::Environment::get().getWindowManager()->setCursorActive(true);
@@ -176,14 +166,18 @@ namespace MWInput
 
         // Don't trigger any mouse bindings while in settings menu, otherwise rebinding controls becomes impossible
         // Also do not trigger bindings when input controls are disabled, e.g. during save loading
-        if (MWBase::Environment::get().getWindowManager()->getMode() != MWGui::GM_Settings && !input->controlsDisabled())
+        if (!MWBase::Environment::get().getWindowManager()->isSettingsWindowVisible() && !input->controlsDisabled())
+        {
             mBindingsManager->mousePressed(arg, id);
+        }
+        MWBase::Environment::get().getLuaManager()->inputEvent(
+            { MWBase::LuaManager::InputEvent::MouseButtonPressed, arg.button });
     }
 
     void MouseManager::updateCursorMode()
     {
         bool grab = !MWBase::Environment::get().getWindowManager()->containsMode(MWGui::GM_MainMenu)
-             && !MWBase::Environment::get().getWindowManager()->isConsoleMode();
+            && !MWBase::Environment::get().getWindowManager()->isConsoleMode();
 
         bool wasRelative = mInputWrapper->getMouseRelative();
         bool isRelative = !MWBase::Environment::get().getWindowManager()->isGuiMode();
@@ -192,11 +186,11 @@ namespace MWInput
         // stop using raw mouse motions and switch to system cursor movements
         mInputWrapper->setMouseRelative(isRelative);
 
-        //we let the mouse escape in the main menu
-        mInputWrapper->setGrabPointer(grab && (mGrabCursor || isRelative));
+        // we let the mouse escape in the main menu
+        mInputWrapper->setGrabPointer(grab && (Settings::input().mGrabCursor || isRelative));
 
-        //we switched to non-relative mode, move our cursor to where the in-game
-        //cursor is
+        // we switched to non-relative mode, move our cursor to where the in-game
+        // cursor is
         if (!isRelative && wasRelative != isRelative)
         {
             warpMouse();
@@ -215,20 +209,23 @@ namespace MWInput
         if (xAxis == 0 && yAxis == 0)
             return;
 
-        float rot[3];
-        rot[0] = -yAxis * dt * 1000.0f * mCameraSensitivity * (mInvertY ? -1 : 1) * mCameraYMultiplier / 256.f;
-        rot[1] = 0.0f;
-        rot[2] = -xAxis * dt * 1000.0f * mCameraSensitivity * (mInvertX ? -1 : 1) / 256.f;
+        const float cameraSensitivity = Settings::input().mCameraSensitivity;
+        const float rot[3] = {
+            -yAxis * dt * 1000.0f * cameraSensitivity * (Settings::input().mInvertYAxis ? -1 : 1)
+                * Settings::input().mCameraYMultiplier / 256.f,
+            0.0f,
+            -xAxis * dt * 1000.0f * cameraSensitivity * (Settings::input().mInvertXAxis ? -1 : 1) / 256.f,
+        };
 
         // Only actually turn player when we're not in vanity mode
-        bool controls = MWBase::Environment::get().getInputManager()->getControlSwitch("playercontrols");
-        if (!MWBase::Environment::get().getWorld()->vanityRotateCamera(rot) && controls)
+        bool playerLooking = MWBase::Environment::get().getInputManager()->getControlSwitch("playerlooking");
+        if (!MWBase::Environment::get().getWorld()->vanityRotateCamera(rot) && playerLooking)
         {
             MWWorld::Player& player = MWBase::Environment::get().getWorld()->getPlayer();
             player.yaw(-rot[2]);
             player.pitch(-rot[0]);
         }
-        else if (!controls)
+        else if (!playerLooking)
             MWBase::Environment::get().getWorld()->disableDeferredPreviewRotation();
 
         MWBase::Environment::get().getInputManager()->resetIdleTime();
@@ -237,17 +234,13 @@ namespace MWInput
     bool MouseManager::injectMouseButtonPress(Uint8 button)
     {
         return MyGUI::InputManager::getInstance().injectMousePress(
-            static_cast<int>(mGuiCursorX),
-            static_cast<int>(mGuiCursorY),
-            SDLUtil::sdlMouseButtonToMyGui(button));
+            static_cast<int>(mGuiCursorX), static_cast<int>(mGuiCursorY), SDLUtil::sdlMouseButtonToMyGui(button));
     }
 
     bool MouseManager::injectMouseButtonRelease(Uint8 button)
     {
         return MyGUI::InputManager::getInstance().injectMouseRelease(
-            static_cast<int>(mGuiCursorX),
-            static_cast<int>(mGuiCursorY),
-            SDLUtil::sdlMouseButtonToMyGui(button));
+            static_cast<int>(mGuiCursorX), static_cast<int>(mGuiCursorY), SDLUtil::sdlMouseButtonToMyGui(button));
     }
 
     void MouseManager::injectMouseMove(float xMove, float yMove, float mouseWheelMove)
@@ -260,12 +253,14 @@ namespace MWInput
         mGuiCursorX = std::clamp<float>(mGuiCursorX, 0.f, viewSize.width - 1);
         mGuiCursorY = std::clamp<float>(mGuiCursorY, 0.f, viewSize.height - 1);
 
-        MyGUI::InputManager::getInstance().injectMouseMove(static_cast<int>(mGuiCursorX), static_cast<int>(mGuiCursorY), static_cast<int>(mMouseWheel));
+        MyGUI::InputManager::getInstance().injectMouseMove(
+            static_cast<int>(mGuiCursorX), static_cast<int>(mGuiCursorY), static_cast<int>(mMouseWheel));
     }
 
     void MouseManager::warpMouse()
     {
-        float uiScale = MWBase::Environment::get().getWindowManager()->getScalingFactor();
-        mInputWrapper->warpMouse(static_cast<int>(mGuiCursorX*uiScale), static_cast<int>(mGuiCursorY*uiScale));
+        float guiUiScale = Settings::gui().mScalingFactor;
+        mInputWrapper->warpMouse(
+            static_cast<int>(mGuiCursorX * guiUiScale), static_cast<int>(mGuiCursorY * guiUiScale));
     }
 }
