@@ -1,12 +1,7 @@
 #include "sensormanager.hpp"
 
 #include <components/debug/debuglog.hpp>
-
-#include "../mwbase/environment.hpp"
-#include "../mwbase/inputmanager.hpp"
-#include "../mwbase/world.hpp"
-
-#include "../mwworld/player.hpp"
+#include <components/settings/values.hpp>
 
 namespace MWInput
 {
@@ -36,7 +31,7 @@ namespace MWInput
 
     void SensorManager::correctGyroscopeAxes()
     {
-        if (!Settings::Manager::getBool("enable gyroscope", "Input"))
+        if (!Settings::input().mEnableGyroscope)
             return;
 
         // Treat setting from config as axes for landscape mode.
@@ -47,7 +42,7 @@ namespace MWInput
 
         float angle = 0;
 
-        SDL_DisplayOrientation currentOrientation = SDL_GetDisplayOrientation(Settings::Manager::getInt("screen", "Video"));
+        SDL_DisplayOrientation currentOrientation = SDL_GetDisplayOrientation(Settings::video().mScreen);
         switch (currentOrientation)
         {
             case SDL_ORIENTATION_UNKNOWN:
@@ -76,7 +71,7 @@ namespace MWInput
 
     void SensorManager::updateSensors()
     {
-        if (Settings::Manager::getBool("enable gyroscope", "Input"))
+        if (Settings::input().mEnableGyroscope)
         {
             int numSensors = SDL_NumSensors();
             for (int i = 0; i < numSensors; ++i)
@@ -84,7 +79,8 @@ namespace MWInput
                 if (SDL_SensorGetDeviceType(i) == SDL_SENSOR_GYRO)
                 {
                     // It is unclear how to handle several enabled gyroscopes, so use the first one.
-                    // Note: Android registers some gyroscope as two separate sensors, for non-wake-up mode and for wake-up mode.
+                    // Note: Android registers some gyroscope as two separate sensors, for non-wake-up mode and for
+                    // wake-up mode.
                     if (mGyroscope != nullptr)
                     {
                         SDL_SensorClose(mGyroscope);
@@ -93,9 +89,10 @@ namespace MWInput
                     }
 
                     // FIXME: SDL2 does not provide a way to configure a sensor update frequency so far.
-                    SDL_Sensor *sensor = SDL_SensorOpen(i);
+                    SDL_Sensor* sensor = SDL_SensorOpen(i);
                     if (sensor == nullptr)
-                        Log(Debug::Error) << "Couldn't open sensor " << SDL_SensorGetDeviceName(i) << ": " << SDL_GetError();
+                        Log(Debug::Error)
+                            << "Couldn't open sensor " << SDL_SensorGetDeviceName(i) << ": " << SDL_GetError();
                     else
                     {
                         mGyroscope = sensor;
@@ -129,12 +126,12 @@ namespace MWInput
         correctGyroscopeAxes();
     }
 
-    void SensorManager::sensorUpdated(const SDL_SensorEvent &arg)
+    void SensorManager::sensorUpdated(const SDL_SensorEvent& arg)
     {
-        if (!Settings::Manager::getBool("enable gyroscope", "Input"))
+        if (!Settings::input().mEnableGyroscope)
             return;
 
-        SDL_Sensor *sensor = SDL_SensorFromInstanceID(arg.which);
+        SDL_Sensor* sensor = SDL_SensorFromInstanceID(arg.which);
         if (!sensor)
         {
             Log(Debug::Info) << "Couldn't get sensor for sensor event";
@@ -151,9 +148,9 @@ namespace MWInput
                 mGyroValues = mRotation * gyro;
                 mGyroUpdateTimer = 0.f;
                 break;
-        }
-        default:
-            break;
+            }
+            default:
+                break;
         }
     }
 
