@@ -4,10 +4,12 @@
 #include <memory>
 
 #include "character.hpp"
+#include "creaturestats.hpp"
 #include "greetingstate.hpp"
 
 #include "../mwbase/environment.hpp"
 #include "../mwbase/world.hpp"
+#include "../mwworld/class.hpp"
 
 #include <components/misc/timer.hpp>
 
@@ -28,8 +30,9 @@ namespace MWMechanics
     public:
         Actor(const MWWorld::Ptr& ptr, MWRender::Animation* animation)
             : mCharacterController(ptr, animation)
-            , mPositionAdjusted(false)
-        {}
+            , mPositionAdjusted(ptr.getClass().getCreatureStats(ptr).getFallHeight() > 0)
+        {
+        }
 
         const MWWorld::Ptr& getPtr() const { return mCharacterController.getPtr(); }
 
@@ -61,11 +64,12 @@ namespace MWMechanics
 
     private:
         CharacterController mCharacterController;
-        int mGreetingTimer{0};
-        float mTargetAngleRadians{0.f};
-        GreetingState mGreetingState{Greet_None};
-        bool mIsTurningToPlayer{false};
-        Misc::DeviatingPeriodicTimer mEngageCombat{1.0f, 0.25f, Misc::Rng::deviate(0, 0.25f, MWBase::Environment::get().getWorld()->getPrng())};
+        int mGreetingTimer{ 0 };
+        float mTargetAngleRadians{ 0.f };
+        GreetingState mGreetingState{ Greet_None };
+        bool mIsTurningToPlayer{ false };
+        Misc::DeviatingPeriodicTimer mEngageCombat{ 1.0f, 0.25f,
+            Misc::Rng::deviate(0, 0.25f, MWBase::Environment::get().getWorld()->getPrng()) };
         bool mPositionAdjusted;
     };
 
