@@ -1,13 +1,13 @@
 #include "actorutil.hpp"
 
-#include "../mwbase/world.hpp"
 #include "../mwbase/environment.hpp"
+#include "../mwbase/world.hpp"
 
 #include "../mwworld/class.hpp"
 #include "../mwworld/player.hpp"
 
-#include "../mwmechanics/magiceffects.hpp"
 #include "../mwmechanics/creaturestats.hpp"
+#include "../mwmechanics/magiceffects.hpp"
 
 #include <components/esm3/loadmgef.hpp>
 
@@ -32,13 +32,13 @@ namespace MWMechanics
     bool hasWaterWalking(const MWWorld::Ptr& actor)
     {
         const MWMechanics::MagicEffects& effects = actor.getClass().getCreatureStats(actor).getMagicEffects();
-        return effects.get(ESM::MagicEffect::WaterWalking).getMagnitude() > 0;
+        return effects.getOrDefault(ESM::MagicEffect::WaterWalking).getMagnitude() > 0;
     }
 
     bool isTargetMagicallyHidden(const MWWorld::Ptr& actor)
     {
         const MagicEffects& magicEffects = actor.getClass().getCreatureStats(actor).getMagicEffects();
-        return (magicEffects.get(ESM::MagicEffect::Invisibility).getMagnitude() > 0)
-            || (magicEffects.get(ESM::MagicEffect::Chameleon).getMagnitude() > 75);
+        return (magicEffects.getOrDefault(ESM::MagicEffect::Invisibility).getMagnitude() > 0)
+            || (magicEffects.getOrDefault(ESM::MagicEffect::Chameleon).getMagnitude() >= 75);
     }
 }
