@@ -3,18 +3,19 @@
 
 #include <map>
 #include <set>
-#include <string>
 #include <stdexcept>
+#include <string>
 
-#include "stat.hpp"
-#include "magiceffects.hpp"
-#include "spells.hpp"
 #include "activespells.hpp"
 #include "aisequence.hpp"
-#include "drawstate.hpp"
 #include "aisetting.hpp"
+#include "drawstate.hpp"
+#include "magiceffects.hpp"
+#include "spells.hpp"
+#include "stat.hpp"
 
 #include <components/esm/attr.hpp>
+#include <components/esm/refid.hpp>
 #include <components/esm3/magiceffects.hpp>
 
 namespace ESM
@@ -38,51 +39,49 @@ namespace MWMechanics
     class CreatureStats
     {
         static int sActorId;
-        DrawState mDrawState;
-        AttributeValue mAttributes[ESM::Attribute::Length];
+        std::map<ESM::RefId, AttributeValue> mAttributes;
         DynamicStat<float> mDynamic[3]; // health, magicka, fatigue
+        DrawState mDrawState = DrawState::Nothing;
         Spells mSpells;
         ActiveSpells mActiveSpells;
         MagicEffects mMagicEffects;
         Stat<int> mAiSettings[4];
         AiSequence mAiSequence;
-        bool mDead;
-        bool mDeathAnimationFinished;
-        bool mDied; // flag for OnDeath script function
-        bool mMurdered;
-        int mFriendlyHits;
-        bool mTalkedTo;
-        bool mAlarmed;
-        bool mAttacked;
-        bool mKnockdown;
-        bool mKnockdownOneFrame;
-        bool mKnockdownOverOneFrame;
-        bool mHitRecovery;
-        bool mBlock;
-        unsigned int mMovementFlags;
+        bool mDead = false;
+        bool mDeathAnimationFinished = false;
+        bool mDied = false; // flag for OnDeath script function
+        bool mMurdered = false;
+        int mFriendlyHits = 0;
+        bool mTalkedTo = false;
+        bool mAlarmed = false;
+        bool mAttacked = false;
+        bool mKnockdown = false;
+        bool mKnockdownOneFrame = false;
+        bool mKnockdownOverOneFrame = false;
+        bool mHitRecovery = false;
+        bool mBlock = false;
+        unsigned int mMovementFlags = 0;
 
-        float mFallHeight;
+        float mFallHeight = 0.f;
 
-        std::string mLastHitObject; // The last object to hit this actor
-        std::string mLastHitAttemptObject; // The last object to attempt to hit this actor
+        ESM::RefId mLastHitObject; // The last object to hit this actor
+        ESM::RefId mLastHitAttemptObject; // The last object to attempt to hit this actor
 
         // For merchants: the last time items were restocked and gold pool refilled.
         MWWorld::TimeStamp mLastRestock;
 
         // The pool of merchant gold (not in inventory)
-        int mGoldPool;
+        int mGoldPool = 0;
 
-        int mActorId;
-        int mHitAttemptActorId; // Stores an actor that attacked this actor. Only one is stored at a time,
-                                // and it is not changed if a different actor attacks. It is cleared when combat ends.
-
-        // The index of the death animation that was played, or -1 if none played
-        signed char mDeathAnimation;
-
-        MWWorld::TimeStamp mTimeOfDeath;
+        int mActorId = -1;
+        // Stores an actor that attacked this actor. Only one is stored at a time, and it is not changed if a different
+        // actor attacks. It is cleared when combat ends.
+        int mHitAttemptActorId = -1;
 
         // The difference between view direction and lower body direction.
-        float mSideMovementAngle;
+        float mSideMovementAngle = 0;
+
+        MWWorld::TimeStamp mTimeOfDeath;
 
     private:
         std::multimap<int, int> mSummonedCreatures; // <Effect, ActorId>
@@ -92,8 +91,15 @@ namespace MWMechanics
         std::vector<int> mSummonGraveyard;
 
     protected:
-        int mLevel;
-        bool mAttackingOrSpell;
+        std::string mAttackType;
+        int mLevel = 0;
+        bool mAttackingOrSpell = false;
+
+    private:
+        // The index of the death animation that was played, or -1 if none played
+        signed char mDeathAnimation = -1;
+
+        bool mTeleported = false;
 
     public:
         CreatureStats();
@@ -108,56 +114,56 @@ namespace MWMechanics
 
         /// Reset the fall height
         /// @return total fall height
-        float land(bool isPlayer=false);
+        float land(bool isPlayer = false);
 
-        const AttributeValue & getAttribute(int index) const;
+        const AttributeValue& getAttribute(ESM::RefId id) const;
 
-        const DynamicStat<float> & getHealth() const;
+        const DynamicStat<float>& getHealth() const;
 
-        const DynamicStat<float> & getMagicka() const;
+        const DynamicStat<float>& getMagicka() const;
 
-        const DynamicStat<float> & getFatigue() const;
+        const DynamicStat<float>& getFatigue() const;
 
-        const DynamicStat<float> & getDynamic (int index) const;
+        const DynamicStat<float>& getDynamic(int index) const;
 
-        const Spells & getSpells() const;
+        const Spells& getSpells() const;
 
-        const ActiveSpells & getActiveSpells() const;
+        const ActiveSpells& getActiveSpells() const;
 
-        const MagicEffects & getMagicEffects() const;
+        const MagicEffects& getMagicEffects() const;
 
         bool getAttackingOrSpell() const { return mAttackingOrSpell; }
+        std::string_view getAttackType() const { return mAttackType; }
 
         int getLevel() const;
 
-        Spells & getSpells();
+        Spells& getSpells();
 
-        ActiveSpells & getActiveSpells();
+        ActiveSpells& getActiveSpells();
 
-        MagicEffects & getMagicEffects();
+        MagicEffects& getMagicEffects();
 
-        void setAttribute(int index, const AttributeValue &value);
+        void setAttribute(ESM::RefId id, const AttributeValue& value);
         // Shortcut to set only the base
-        void setAttribute(int index, float base);
+        void setAttribute(ESM::RefId id, float base);
 
-        void setHealth(const DynamicStat<float> &value);
+        void setHealth(const DynamicStat<float>& value);
 
-        void setMagicka(const DynamicStat<float> &value);
+        void setMagicka(const DynamicStat<float>& value);
 
-        void setFatigue(const DynamicStat<float> &value);
+        void setFatigue(const DynamicStat<float>& value);
 
-        void setDynamic (int index, const DynamicStat<float> &value);
-
-        /// Set Modifier for each magic effect according to \a effects. Does not touch Base values.
-        void modifyMagicEffects(const MagicEffects &effects);
+        void setDynamic(int index, const DynamicStat<float>& value);
 
         void setAttackingOrSpell(bool attackingOrSpell) { mAttackingOrSpell = attackingOrSpell; }
 
+        void setAttackType(std::string_view attackType) { mAttackType = attackType; }
+
         void setLevel(int level);
 
-        void setAiSetting (AiSetting index, Stat<int> value);
-        void setAiSetting (AiSetting index, int base);
-        Stat<int> getAiSetting (AiSetting index) const;
+        void setAiSetting(AiSetting index, Stat<int> value);
+        void setAiSetting(AiSetting index, int base);
+        Stat<int> getAiSetting(AiSetting index) const;
 
         const AiSequence& getAiSequence() const;
 
@@ -197,16 +203,18 @@ namespace MWMechanics
         void friendlyHit();
         ///< Increase number of friendly hits by one.
 
+        void resetFriendlyHits();
+
         bool hasTalkedToPlayer() const;
         ///< Has this creature talked with the player before?
 
         void talkedToPlayer();
 
         bool isAlarmed() const;
-        void setAlarmed (bool alarmed);
+        void setAlarmed(bool alarmed);
 
         bool getAttacked() const;
-        void setAttacked (bool attacked);
+        void setAttacked(bool attacked);
 
         float getEvasion() const;
 
@@ -215,10 +223,11 @@ namespace MWMechanics
         /// including transition animations (falling down & standing up)
         bool getKnockedDown() const;
         void setKnockedDownOneFrame(bool value);
-        ///Returns true only for the first frame of the actor being knocked out; used for "onKnockedOut" command
+        /// Returns true only for the first frame of the actor being knocked out; used for "onKnockedOut" command
         bool getKnockedDownOneFrame() const;
         void setKnockedDownOverOneFrame(bool value);
-        ///Returns true for all but the first frame of being knocked out; used to know to not reset mKnockedDownOneFrame
+        /// Returns true for all but the first frame of being knocked out; used to know to not reset
+        /// mKnockedDownOneFrame
         bool getKnockedDownOverOneFrame() const;
         void setHitRecovery(bool value);
         bool getHitRecovery() const;
@@ -243,26 +252,26 @@ namespace MWMechanics
             Stance_Sneak
         };
 
-        bool getMovementFlag (Flag flag) const;
-        void setMovementFlag (Flag flag, bool state);
+        bool getMovementFlag(Flag flag) const;
+        void setMovementFlag(Flag flag, bool state);
         /// Like getMovementFlag, but also takes into account if the flag is Forced
-        bool getStance (Stance flag) const;
+        bool getStance(Stance flag) const;
 
-        void setLastHitObject(const std::string &objectid);
+        void setLastHitObject(const ESM::RefId& objectid);
         void clearLastHitObject();
-        const std::string &getLastHitObject() const;
-        void setLastHitAttemptObject(const std::string &objectid);
+        const ESM::RefId& getLastHitObject() const;
+        void setLastHitAttemptObject(const ESM::RefId& objectid);
         void clearLastHitAttemptObject();
-        const std::string &getLastHitAttemptObject() const;
+        const ESM::RefId& getLastHitAttemptObject() const;
         void setHitAttemptActorId(const int actorId);
         int getHitAttemptActorId() const;
 
-        void writeState (ESM::CreatureStats& state) const;
+        void writeState(ESM::CreatureStats& state) const;
 
-        void readState (const ESM::CreatureStats& state);
+        void readState(const ESM::CreatureStats& state);
 
-        static void writeActorIdCounter (ESM::ESMWriter& esm);
-        static void readActorIdCounter (ESM::ESMReader& esm);
+        static void writeActorIdCounter(ESM::ESMWriter& esm);
+        static void readActorIdCounter(ESM::ESMReader& esm);
 
         void setLastRestockTime(MWWorld::TimeStamp tradeTime);
         MWWorld::TimeStamp getLastRestockTime() const;
@@ -278,7 +287,7 @@ namespace MWMechanics
         int getActorId();
         ///< Will generate an actor ID, if the actor does not have one yet.
 
-        bool matchesActorId (int id) const;
+        bool matchesActorId(int id) const;
         ///< Check if \a id matches the actor ID of *this (if the actor does not have an ID
         /// assigned this function will return false).
 
@@ -286,6 +295,11 @@ namespace MWMechanics
 
         float getSideMovementAngle() const { return mSideMovementAngle; }
         void setSideMovementAngle(float angle) { mSideMovementAngle = angle; }
+
+        bool wasTeleported() const { return mTeleported; }
+        void setTeleported(bool v) { mTeleported = v; }
+
+        const std::map<ESM::RefId, AttributeValue>& getAttributes() const { return mAttributes; }
     };
 }
 
