@@ -3,10 +3,10 @@
 
 #include "character.hpp"
 
-#include <string>
-#include <map>
-#include <vector>
 #include <list>
+#include <map>
+#include <string>
+#include <vector>
 
 namespace osg
 {
@@ -27,16 +27,16 @@ namespace MWMechanics
         std::map<const MWWorld::LiveCellRefBase*, std::list<CharacterController>::iterator> mIndex;
 
     public:
-        void addObject (const MWWorld::Ptr& ptr);
+        void addObject(const MWWorld::Ptr& ptr);
         ///< Register an animated object
 
-        void removeObject (const MWWorld::Ptr& ptr);
+        void removeObject(const MWWorld::Ptr& ptr);
         ///< Deregister an object
 
-        void updateObject(const MWWorld::Ptr &old, const MWWorld::Ptr& ptr);
+        void updateObject(const MWWorld::Ptr& old, const MWWorld::Ptr& ptr);
         ///< Updates an object with a new Ptr
 
-        void dropObjects(const MWWorld::CellStore *cellStore);
+        void dropObjects(const MWWorld::CellStore* cellStore);
         ///< Deregister all objects in the given cell.
 
         void update(float duration, bool paused);
@@ -45,16 +45,18 @@ namespace MWMechanics
         bool onOpen(const MWWorld::Ptr& ptr);
         void onClose(const MWWorld::Ptr& ptr);
 
-        bool playAnimationGroup(const MWWorld::Ptr& ptr, const std::string& groupName, int mode, int number, bool persist=false);
+        bool playAnimationGroup(
+            const MWWorld::Ptr& ptr, std::string_view groupName, int mode, uint32_t number, bool scripted = false);
+        bool playAnimationGroupLua(const MWWorld::Ptr& ptr, std::string_view groupName, uint32_t loops, float speed,
+            std::string_view startKey, std::string_view stopKey, bool forceLoop);
+        void enableLuaAnimations(const MWWorld::Ptr& ptr, bool enable);
         void skipAnimation(const MWWorld::Ptr& ptr);
         void persistAnimationStates();
+        void clearAnimationQueue(const MWWorld::Ptr& ptr, bool clearScripted);
 
         void getObjectsInRange(const osg::Vec3f& position, float radius, std::vector<MWWorld::Ptr>& out) const;
 
-        std::size_t size() const
-        {
-            return mObjects.size();
-        }
+        std::size_t size() const { return mObjects.size(); }
     };
 }
 
