@@ -5,29 +5,28 @@
 
 #include <osg/io_utils>
 
-#include <components/detournavigator/navigatorutils.hpp>
-#include <components/detournavigator/debug.hpp>
 #include <components/debug/debuglog.hpp>
+#include <components/detournavigator/debug.hpp>
+#include <components/detournavigator/navigatorutils.hpp>
 #include <components/misc/coordinateconverter.hpp>
 #include <components/misc/math.hpp>
 
-#include "../mwbase/world.hpp"
 #include "../mwbase/environment.hpp"
+#include "../mwbase/world.hpp"
 
-#include "../mwphysics/collisiontype.hpp"
+#include "../mwphysics/raycasting.hpp"
 
 #include "../mwworld/cellstore.hpp"
 #include "../mwworld/class.hpp"
 
-#include "pathgrid.hpp"
 #include "actorutil.hpp"
+#include "pathgrid.hpp"
 
 namespace
 {
     // Chooses a reachable end pathgrid point.  start is assumed reachable.
-    std::pair<int, bool> getClosestReachablePoint(const ESM::Pathgrid* grid,
-                                                  const MWMechanics::PathgridGraph *graph,
-                                                  const osg::Vec3f& pos, int start)
+    std::pair<int, bool> getClosestReachablePoint(
+        const ESM::Pathgrid* grid, const MWMechanics::PathgridGraph* graph, const osg::Vec3f& pos, int start)
     {
         assert(grid && !grid->mPoints.empty());
 
@@ -37,7 +36,7 @@ namespace
         int closestReachableIndex = 0;
         // TODO: if this full scan causes performance problems mapping pathgrid
         //       points to a quadtree may help
-        for(unsigned int counter = 0; counter < grid->mPoints.size(); counter++)
+        for (size_t counter = 0; counter < grid->mPoints.size(); counter++)
         {
             float potentialDistBetween = MWMechanics::PathFinder::distanceSquared(grid->mPoints[counter], pos);
             if (potentialDistBetween < closestDistanceReachable)
@@ -63,8 +62,7 @@ namespace
         // allowed nodes if not.  Hence a path needs to be created even if the start
         // and the end points are the same.
 
-        return std::pair<int, bool>
-            (closestReachableIndex, closestReachableIndex == closestIndex);
+        return std::pair<int, bool>(closestReachableIndex, closestReachableIndex == closestIndex);
     }
 
     float sqrDistance(const osg::Vec2f& lhs, const osg::Vec2f& rhs)
@@ -77,13 +75,6 @@ namespace
         return sqrDistance(osg::Vec2f(lhs.x(), lhs.y()), osg::Vec2f(rhs.x(), rhs.y()));
     }
 
-    float getPathStepSize(const MWWorld::ConstPtr& actor)
-    {
-        const auto world = MWBase::Environment::get().getWorld();
-        const auto realHalfExtents = world->getHalfExtents(actor);
-        return 2 * std::max(realHalfExtents.x(), realHalfExtents.y());
-    }
-
     float getHeight(const MWWorld::ConstPtr& actor)
     {
         const auto world = MWBase::Environment::get().getWorld();
@@ -92,7 +83,8 @@ namespace
     }
 
     // Returns true if turn in `p2` is less than 10 degrees and all the 3 points are almost on one line.
-    bool isAlmostStraight(const osg::Vec3f& p1, const osg::Vec3f& p2, const osg::Vec3f& p3, float pointTolerance) {
+    bool isAlmostStraight(const osg::Vec3f& p1, const osg::Vec3f& p2, const osg::Vec3f& p3, float pointTolerance)
+    {
         osg::Vec3f v1 = p1 - p2;
         osg::Vec3f v3 = p3 - p2;
         v1.z() = v3.z() = 0;
@@ -118,7 +110,8 @@ namespace
         bool operator()(const osg::Vec3f& start, const osg::Vec3f& end) const
         {
             const auto position = DetourNavigator::raycast(*mNavigator, mAgentBounds, start, end, mFlags);
-            return position.has_value() && std::abs((position.value() - start).length2() - (end - start).length2()) <= 1;
+            return position.has_value()
+                && std::abs((position.value() - start).length2() - (end - start).length2()) <= 1;
         }
     };
 }
@@ -138,10 +131,12 @@ namespace MWMechanics
         dir.z() = 0;
         dir.normalize();
         float verticalOffset = 200; // instead of '200' here we want the height of the actor
-        osg::Vec3f _from = from + dir*offsetXY + osg::Z_AXIS * verticalOffset;
+        osg::Vec3f _from = from + dir * offsetXY + osg::Z_AXIS * verticalOffset;
 
         // cast up-down ray and find height of hit in world space
-        float h = _from.z() - MWBase::Environment::get().getWorld()->getDistToNearestRayHit(_from, -osg::Z_AXIS, verticalOffset + PATHFIND_Z_REACH + 1);
+        float h = _from.z()
+            - MWBase::Environment::get().getWorld()->getDistToNearestRayHit(
+                _from, -osg::Z_AXIS, verticalOffset + PATHFIND_Z_REACH + 1);
 
         return (std::abs(from.z() - h) <= PATHFIND_Z_REACH);
     }
@@ -187,14 +182,14 @@ namespace MWMechanics
     {
         const auto pathgrid = pathgridGraph.getPathgrid();
 
-        // Refer to AiWander reseach topic on openmw forums for some background.
+        // Refer to AiWander research topic on openmw forums for some background.
         // Maybe there is no pathgrid for this cell.  Just go to destination and let
         // physics take care of any blockages.
-        if(!pathgrid || pathgrid->mPoints.empty())
+        if (!pathgrid || pathgrid->mPoints.empty())
             return;
 
         // NOTE: getClosestPoint expects local coordinates
-        Misc::CoordinateConverter converter(mCell->getCell());
+        const Misc::CoordinateConverter converter = Misc::makeCoordinateConverter(*mCell->getCell());
 
         // NOTE: It is possible that getClosestPoint returns a pathgrind point index
         //       that is unreachable in some situations. e.g. actor is standing
@@ -205,9 +200,8 @@ namespace MWMechanics
         int startNode = getClosestPoint(pathgrid, startPointInLocalCoords);
 
         osg::Vec3f endPointInLocalCoords(converter.toLocalVec3(endPoint));
-        std::pair<int, bool> endNode = getClosestReachablePoint(pathgrid, &pathgridGraph,
-            endPointInLocalCoords,
-                startNode);
+        std::pair<int, bool> endNode
+            = getClosestReachablePoint(pathgrid, &pathgridGraph, endPointInLocalCoords, startNode);
 
         // if it's shorter for actor to travel from start to end, than to travel from either
         // start or end to nearest pathgrid point, just travel from start to end.
@@ -225,7 +219,7 @@ namespace MWMechanics
         // even if the start and the end points are the same.
         // NOTE: aStarSearch will return an empty path if the start and end
         //       nodes are the same
-        if(startNode == endNode.first)
+        if (startNode == endNode.first)
         {
             ESM::Pathgrid::Point temp(pathgrid->mPoints[startNode]);
             converter.toWorld(temp);
@@ -251,20 +245,22 @@ namespace MWMechanics
                     // Add Z offset since path node can overlap with other objects.
                     // Also ignore doors in raytesting.
                     const int mask = MWPhysics::CollisionType_World;
-                    bool isPathClear = !MWBase::Environment::get().getWorld()->castRay(
-                        startPoint.x(), startPoint.y(), startPoint.z() + 16, temp.mX, temp.mY, temp.mZ + 16, mask);
+                    bool isPathClear = !MWBase::Environment::get()
+                                            .getWorld()
+                                            ->getRayCasting()
+                                            ->castRay(osg::Vec3f(startPoint.x(), startPoint.y(), startPoint.z() + 16),
+                                                osg::Vec3f(temp.mX, temp.mY, temp.mZ + 16), mask)
+                                            .mHit;
                     if (isPathClear)
                         path.pop_front();
                 }
             }
 
             // convert supplied path to world coordinates
-            std::transform(path.begin(), path.end(), out,
-                [&] (ESM::Pathgrid::Point& point)
-                {
-                    converter.toWorld(point);
-                    return makeOsgVec3(point);
-                });
+            std::transform(path.begin(), path.end(), out, [&](ESM::Pathgrid::Point& point) {
+                converter.toWorld(point);
+                return makeOsgVec3(point);
+            });
         }
 
         // If endNode found is NOT the closest PathGrid point to the endPoint,
@@ -287,7 +283,7 @@ namespace MWMechanics
     {
         // This should never happen (programmers should have an if statement checking
         // isPathConstructed that prevents this call if otherwise).
-        if(mPath.empty())
+        if (mPath.empty())
             return 0.;
 
         const auto& nextPoint = mPath.front();
@@ -301,7 +297,7 @@ namespace MWMechanics
     {
         // This should never happen (programmers should have an if statement checking
         // isPathConstructed that prevents this call if otherwise).
-        if(mPath.empty())
+        if (mPath.empty())
             return 0.;
 
         const osg::Vec3f dir = mPath.front() - osg::Vec3f(x, y, z);
@@ -310,8 +306,7 @@ namespace MWMechanics
     }
 
     void PathFinder::update(const osg::Vec3f& position, float pointTolerance, float destinationTolerance,
-        bool shortenIfAlmostStraight, bool canMoveByZ, const DetourNavigator::AgentBounds& agentBounds,
-        const DetourNavigator::Flags flags)
+        UpdateFlags updateFlags, const DetourNavigator::AgentBounds& agentBounds, DetourNavigator::Flags pathFlags)
     {
         if (mPath.empty())
             return;
@@ -319,22 +314,20 @@ namespace MWMechanics
         while (mPath.size() > 1 && sqrDistanceIgnoreZ(mPath.front(), position) < pointTolerance * pointTolerance)
             mPath.pop_front();
 
-        const IsValidShortcut isValidShortcut {
-            MWBase::Environment::get().getWorld()->getNavigator(),
-            agentBounds, flags
-        };
+        const IsValidShortcut isValidShortcut{ MWBase::Environment::get().getWorld()->getNavigator(), agentBounds,
+            pathFlags };
 
-        if (shortenIfAlmostStraight)
+        if ((updateFlags & UpdateFlag_ShortenIfAlmostStraight) != 0)
         {
             while (mPath.size() > 2 && isAlmostStraight(mPath[0], mPath[1], mPath[2], pointTolerance)
-                   && isValidShortcut(mPath[0], mPath[2]))
+                && isValidShortcut(mPath[0], mPath[2]))
                 mPath.erase(mPath.begin() + 1);
             if (mPath.size() > 1 && isAlmostStraight(position, mPath[0], mPath[1], pointTolerance)
-                    && isValidShortcut(position, mPath[1]))
+                && isValidShortcut(position, mPath[1]))
                 mPath.pop_front();
         }
 
-        if (mPath.size() > 1)
+        if ((updateFlags & UpdateFlag_RemoveLoops) != 0 && mPath.size() > 1)
         {
             std::size_t begin = 0;
             for (std::size_t i = 1; i < mPath.size(); ++i)
@@ -350,7 +343,7 @@ namespace MWMechanics
         if (mPath.size() == 1)
         {
             float distSqr;
-            if (canMoveByZ)
+            if ((updateFlags & UpdateFlag_CanMoveByZ) != 0)
                 distSqr = (mPath.front() - position).length2();
             else
                 distSqr = sqrDistanceIgnoreZ(mPath.front(), position);
@@ -397,9 +390,9 @@ namespace MWMechanics
     }
 
     void PathFinder::buildPath(const MWWorld::ConstPtr& actor, const osg::Vec3f& startPoint, const osg::Vec3f& endPoint,
-        const MWWorld::CellStore* cell, const PathgridGraph& pathgridGraph, const DetourNavigator::AgentBounds& agentBounds,
-        const DetourNavigator::Flags flags, const DetourNavigator::AreaCosts& areaCosts, float endTolerance,
-        PathType pathType)
+        const MWWorld::CellStore* cell, const PathgridGraph& pathgridGraph,
+        const DetourNavigator::AgentBounds& agentBounds, const DetourNavigator::Flags flags,
+        const DetourNavigator::AreaCosts& areaCosts, float endTolerance, PathType pathType)
     {
         mPath.clear();
         mCell = cell;
@@ -408,16 +401,18 @@ namespace MWMechanics
 
         if (!actor.getClass().isPureWaterCreature(actor) && !actor.getClass().isPureFlyingCreature(actor))
         {
-            status = buildPathByNavigatorImpl(actor, startPoint, endPoint, agentBounds, flags, areaCosts,
-                                              endTolerance, pathType, std::back_inserter(mPath));
+            status = buildPathByNavigatorImpl(actor, startPoint, endPoint, agentBounds, flags, areaCosts, endTolerance,
+                pathType, std::back_inserter(mPath));
             if (status != DetourNavigator::Status::Success)
                 mPath.clear();
         }
 
-        if (status != DetourNavigator::Status::NavMeshNotFound && mPath.empty() && (flags & DetourNavigator::Flag_usePathgrid) == 0)
+        if (status != DetourNavigator::Status::NavMeshNotFound && mPath.empty()
+            && (flags & DetourNavigator::Flag_usePathgrid) == 0)
         {
             status = buildPathByNavigatorImpl(actor, startPoint, endPoint, agentBounds,
-                flags | DetourNavigator::Flag_usePathgrid, areaCosts, endTolerance, pathType, std::back_inserter(mPath));
+                flags | DetourNavigator::Flag_usePathgrid, areaCosts, endTolerance, pathType,
+                std::back_inserter(mPath));
             if (status != DetourNavigator::Status::Success)
                 mPath.clear();
         }
@@ -431,16 +426,15 @@ namespace MWMechanics
         mConstructed = !mPath.empty();
     }
 
-    DetourNavigator::Status PathFinder::buildPathByNavigatorImpl(const MWWorld::ConstPtr& actor, const osg::Vec3f& startPoint,
-        const osg::Vec3f& endPoint, const DetourNavigator::AgentBounds& agentBounds, const DetourNavigator::Flags flags,
-        const DetourNavigator::AreaCosts& areaCosts, float endTolerance, PathType pathType,
-        std::back_insert_iterator<std::deque<osg::Vec3f>> out)
+    DetourNavigator::Status PathFinder::buildPathByNavigatorImpl(const MWWorld::ConstPtr& actor,
+        const osg::Vec3f& startPoint, const osg::Vec3f& endPoint, const DetourNavigator::AgentBounds& agentBounds,
+        const DetourNavigator::Flags flags, const DetourNavigator::AreaCosts& areaCosts, float endTolerance,
+        PathType pathType, std::back_insert_iterator<std::deque<osg::Vec3f>> out)
     {
         const auto world = MWBase::Environment::get().getWorld();
-        const auto stepSize = getPathStepSize(actor);
         const auto navigator = world->getNavigator();
-        const auto status = DetourNavigator::findPath(*navigator, agentBounds, stepSize,
-            startPoint, endPoint, flags, areaCosts, endTolerance, out);
+        const auto status = DetourNavigator::findPath(
+            *navigator, agentBounds, startPoint, endPoint, flags, areaCosts, endTolerance, out);
 
         if (pathType == PathType::Partial && status == DetourNavigator::Status::PartialPath)
             return DetourNavigator::Status::Success;
@@ -448,70 +442,27 @@ namespace MWMechanics
         if (status != DetourNavigator::Status::Success)
         {
             Log(Debug::Debug) << "Build path by navigator error: \"" << DetourNavigator::getMessage(status)
-                << "\" for \"" << actor.getClass().getName(actor) << "\" (" << actor.getBase()
-                << ") from " << startPoint << " to " << endPoint << " with flags ("
-                << DetourNavigator::WriteFlags {flags} << ")";
+                              << "\" for \"" << actor.getClass().getName(actor) << "\" (" << actor.getBase()
+                              << ") from " << startPoint << " to " << endPoint << " with flags ("
+                              << DetourNavigator::WriteFlags{ flags } << ")";
         }
 
         return status;
     }
 
-    void PathFinder::buildPathByNavMeshToNextPoint(const MWWorld::ConstPtr& actor,
-        const DetourNavigator::AgentBounds& agentBounds, const DetourNavigator::Flags flags,
-        const DetourNavigator::AreaCosts& areaCosts)
-    {
-        if (mPath.empty())
-            return;
-
-        const auto stepSize = getPathStepSize(actor);
-        const auto startPoint = actor.getRefData().getPosition().asVec3();
-
-        if (sqrDistanceIgnoreZ(mPath.front(), startPoint) <= 4 * stepSize * stepSize)
-            return;
-
-        const auto navigator = MWBase::Environment::get().getWorld()->getNavigator();
-        std::deque<osg::Vec3f> prePath;
-        auto prePathInserter = std::back_inserter(prePath);
-        const float endTolerance = 0;
-        const auto status = DetourNavigator::findPath(*navigator, agentBounds, stepSize,
-            startPoint, mPath.front(), flags, areaCosts, endTolerance, prePathInserter);
-
-        if (status == DetourNavigator::Status::NavMeshNotFound)
-            return;
-
-        if (status != DetourNavigator::Status::Success)
-        {
-            Log(Debug::Debug) << "Build path by navigator error: \"" << DetourNavigator::getMessage(status)
-                << "\" for \"" << actor.getClass().getName(actor) << "\" (" << actor.getBase()
-                << ") from " << startPoint << " to " << mPath.front() << " with flags ("
-                << DetourNavigator::WriteFlags {flags} << ")";
-            return;
-        }
-
-        while (!prePath.empty() && sqrDistanceIgnoreZ(prePath.front(), startPoint) < stepSize * stepSize)
-            prePath.pop_front();
-
-        while (!prePath.empty() && sqrDistanceIgnoreZ(prePath.back(), mPath.front()) < stepSize * stepSize)
-            prePath.pop_back();
-
-        std::copy(prePath.rbegin(), prePath.rend(), std::front_inserter(mPath));
-    }
-
-    void PathFinder::buildLimitedPath(const MWWorld::ConstPtr& actor, const osg::Vec3f& startPoint, const osg::Vec3f& endPoint,
-        const MWWorld::CellStore* cell, const PathgridGraph& pathgridGraph,
+    void PathFinder::buildLimitedPath(const MWWorld::ConstPtr& actor, const osg::Vec3f& startPoint,
+        const osg::Vec3f& endPoint, const MWWorld::CellStore* cell, const PathgridGraph& pathgridGraph,
         const DetourNavigator::AgentBounds& agentBounds, const DetourNavigator::Flags flags,
         const DetourNavigator::AreaCosts& areaCosts, float endTolerance, PathType pathType)
     {
         const auto navigator = MWBase::Environment::get().getWorld()->getNavigator();
-        const auto maxDistance = std::min(
-            navigator->getMaxNavmeshAreaRealRadius(),
-            static_cast<float>(Constants::CellSizeInUnits)
-        );
+        const auto maxDistance
+            = std::min(navigator->getMaxNavmeshAreaRealRadius(), static_cast<float>(Constants::CellSizeInUnits));
         const auto startToEnd = endPoint - startPoint;
         const auto distance = startToEnd.length();
         if (distance <= maxDistance)
             return buildPath(actor, startPoint, endPoint, cell, pathgridGraph, agentBounds, flags, areaCosts,
-                             endTolerance, pathType);
+                endTolerance, pathType);
         const auto end = startPoint + startToEnd * maxDistance / distance;
         buildPath(actor, startPoint, end, cell, pathgridGraph, agentBounds, flags, areaCosts, endTolerance, pathType);
     }
