@@ -6,16 +6,16 @@
 #include "../mwworld/containerstore.hpp"
 #include "../mwworld/esmstore.hpp"
 
-#include "../mwbase/world.hpp"
 #include "../mwbase/environment.hpp"
 #include "../mwbase/mechanicsmanager.hpp"
+#include "../mwbase/world.hpp"
 
 #include "creaturestats.hpp"
 
 namespace MWMechanics
 {
 
-    Security::Security(const MWWorld::Ptr &actor)
+    Security::Security(const MWWorld::Ptr& actor)
         : mActor(actor)
     {
         CreatureStats& creatureStats = actor.getClass().getCreatureStats(actor);
@@ -25,12 +25,12 @@ namespace MWMechanics
         mFatigueTerm = creatureStats.getFatigueTerm();
     }
 
-    void Security::pickLock(const MWWorld::Ptr &lock, const MWWorld::Ptr &lockpick,
-                            std::string& resultMessage, std::string& resultSound)
+    void Security::pickLock(const MWWorld::Ptr& lock, const MWWorld::Ptr& lockpick, std::string_view& resultMessage,
+        std::string_view& resultSound)
     {
-        if (lock.getCellRef().getLockLevel() <= 0 ||
-            lock.getCellRef().getLockLevel() == ESM::UnbreakableLock ||
-            !lock.getClass().hasToolTip(lock)) //If it's unlocked or can not be unlocked back out immediately
+        // If it's unlocked or can not be unlocked back out immediately. Note that we're not strictly speaking checking
+        // if the ref is locked, lock levels <= 0 can exist but they cannot be picked
+        if (lock.getCellRef().getLockLevel() <= 0 || !lock.getClass().hasToolTip(lock))
             return;
 
         int uses = lockpick.getClass().getItemHealth(lockpick);
@@ -41,7 +41,11 @@ namespace MWMechanics
 
         float pickQuality = lockpick.get<ESM::Lockpick>()->mBase->mData.mQuality;
 
-        float fPickLockMult = MWBase::Environment::get().getWorld()->getStore().get<ESM::GameSetting>().find("fPickLockMult")->mValue.getFloat();
+        float fPickLockMult = MWBase::Environment::get()
+                                  .getESMStore()
+                                  ->get<ESM::GameSetting>()
+                                  .find("fPickLockMult")
+                                  ->mValue.getFloat();
 
         float x = 0.2f * mAgility + 0.1f * mLuck + mSecuritySkill;
         x *= pickQuality * mFatigueTerm;
@@ -60,7 +64,7 @@ namespace MWMechanics
                 lock.getCellRef().unlock();
                 resultMessage = "#{sLockSuccess}";
                 resultSound = "Open Lock";
-                mActor.getClass().skillUsageSucceeded(mActor, ESM::Skill::Security, 1);
+                mActor.getClass().skillUsageSucceeded(mActor, ESM::Skill::Security, ESM::Skill::Security_PickLock);
             }
             else
                 resultMessage = "#{sLockFail}";
@@ -68,11 +72,11 @@ namespace MWMechanics
 
         lockpick.getCellRef().setCharge(--uses);
         if (!uses)
-            lockpick.getContainerStore()->remove(lockpick, 1, mActor);
+            lockpick.getContainerStore()->remove(lockpick, 1);
     }
 
-    void Security::probeTrap(const MWWorld::Ptr &trap, const MWWorld::Ptr &probe,
-                             std::string& resultMessage, std::string& resultSound)
+    void Security::probeTrap(const MWWorld::Ptr& trap, const MWWorld::Ptr& probe, std::string_view& resultMessage,
+        std::string_view& resultSound)
     {
         if (trap.getCellRef().getTrap().empty())
             return;
@@ -83,10 +87,15 @@ namespace MWMechanics
 
         float probeQuality = probe.get<ESM::Probe>()->mBase->mData.mQuality;
 
-        const ESM::Spell* trapSpell = MWBase::Environment::get().getWorld()->getStore().get<ESM::Spell>().find(trap.getCellRef().getTrap());
+        const ESM::Spell* trapSpell
+            = MWBase::Environment::get().getESMStore()->get<ESM::Spell>().find(trap.getCellRef().getTrap());
         int trapSpellPoints = trapSpell->mData.mCost;
 
-        float fTrapCostMult = MWBase::Environment::get().getWorld()->getStore().get<ESM::GameSetting>().find("fTrapCostMult")->mValue.getFloat();
+        float fTrapCostMult = MWBase::Environment::get()
+                                  .getESMStore()
+                                  ->get<ESM::GameSetting>()
+                                  .find("fTrapCostMult")
+                                  ->mValue.getFloat();
 
         float x = 0.2f * mAgility + 0.1f * mLuck + mSecuritySkill;
         x += fTrapCostMult * trapSpellPoints;
@@ -102,11 +111,11 @@ namespace MWMechanics
             auto& prng = MWBase::Environment::get().getWorld()->getPrng();
             if (Misc::Rng::roll0to99(prng) <= x)
             {
-                trap.getCellRef().setTrap("");
+                trap.getCellRef().setTrap(ESM::RefId());
 
                 resultSound = "Disarm Trap";
                 resultMessage = "#{sTrapSuccess}";
-                mActor.getClass().skillUsageSucceeded(mActor, ESM::Skill::Security, 0);
+                mActor.getClass().skillUsageSucceeded(mActor, ESM::Skill::Security, ESM::Skill::Security_DisarmTrap);
             }
             else
                 resultMessage = "#{sTrapFail}";
@@ -114,7 +123,7 @@ namespace MWMechanics
 
         probe.getCellRef().setCharge(--uses);
         if (!uses)
-            probe.getContainerStore()->remove(probe, 1, mActor);
+            probe.getContainerStore()->remove(probe, 1);
     }
 
 }
