@@ -4,19 +4,17 @@
 #include <string>
 
 #include "../mwbase/environment.hpp"
-#include "../mwbase/world.hpp"
 
 #include "../mwworld/esmstore.hpp"
 
 #include "aisetting.hpp"
-#include "creaturestats.hpp"
 
 namespace MWMechanics
 {
-    template<class T>
-    void setBaseAISetting(const std::string& id, MWMechanics::AiSetting setting, int value)
+    template <class T>
+    void setBaseAISetting(const ESM::RefId& id, MWMechanics::AiSetting setting, int value)
     {
-        T copy = *MWBase::Environment::get().getWorld()->getStore().get<T>().find(id);
+        T copy = *MWBase::Environment::get().getESMStore()->get<T>().find(id);
         switch (setting)
         {
             case MWMechanics::AiSetting::Hello:
@@ -34,7 +32,7 @@ namespace MWMechanics
             default:
                 assert(false);
         }
-        MWBase::Environment::get().getWorld()->createOverrideRecord(copy);
+        MWBase::Environment::get().getESMStore()->overrideRecord(copy);
     }
 }
 
