@@ -1,47 +1,72 @@
 #include "stat.hpp"
 
+#include <algorithm>
+
 #include <components/esm3/statstate.hpp>
 
 namespace MWMechanics
 {
-    template<typename T>
-    Stat<T>::Stat() : mBase (0), mModifier (0) {}
-    template<typename T>
-    Stat<T>::Stat(T base, T modified) : mBase (base), mModifier (modified) {}
+    template <typename T>
+    Stat<T>::Stat()
+        : mBase(0)
+        , mModifier(0)
+    {
+    }
+    template <typename T>
+    Stat<T>::Stat(T base, T modified)
+        : mBase(base)
+        , mModifier(modified)
+    {
+    }
 
-    template<typename T>
+    template <typename T>
     T Stat<T>::getModified(bool capped) const
     {
-        if(capped)
+        if (capped)
             return std::max({}, mModifier + mBase);
         return mModifier + mBase;
     }
 
-    template<typename T>
-    void Stat<T>::writeState (ESM::StatState<T>& state) const
+    template <typename T>
+    void Stat<T>::writeState(ESM::StatState<T>& state) const
     {
         state.mBase = mBase;
         state.mMod = mModifier;
     }
-    template<typename T>
-    void Stat<T>::readState (const ESM::StatState<T>& state)
+    template <typename T>
+    void Stat<T>::readState(const ESM::StatState<T>& state)
     {
         mBase = state.mBase;
         mModifier = state.mMod;
     }
 
+    template <typename T>
+    DynamicStat<T>::DynamicStat()
+        : mStatic(0, 0)
+        , mCurrent(0)
+    {
+    }
+    template <typename T>
+    DynamicStat<T>::DynamicStat(T base)
+        : mStatic(base, 0)
+        , mCurrent(base)
+    {
+    }
+    template <typename T>
+    DynamicStat<T>::DynamicStat(T base, T modified, T current)
+        : mStatic(base, modified)
+        , mCurrent(current)
+    {
+    }
+    template <typename T>
+    DynamicStat<T>::DynamicStat(const Stat<T>& stat, T current)
+        : mStatic(stat)
+        , mCurrent(current)
+    {
+    }
 
-    template<typename T>
-    DynamicStat<T>::DynamicStat() : mStatic(0, 0), mCurrent(0) {}
-    template<typename T>
-    DynamicStat<T>::DynamicStat(T base) : mStatic(base, 0), mCurrent(base) {}
-    template<typename T>
-    DynamicStat<T>::DynamicStat(T base, T modified, T current) : mStatic(base, modified), mCurrent (current) {}
-    template<typename T>
-    DynamicStat<T>::DynamicStat(const Stat<T> &stat, T current) : mStatic(stat), mCurrent (current) {}
-
-    template<typename T>
-    void DynamicStat<T>::setCurrent (const T& value, bool allowDecreaseBelowZero, bool allowIncreaseAboveModified)
+    template <typename T>
+    void DynamicStat<T>::setCurrent(const T& value, bool allowDecreaseBelowZero, bool allowIncreaseAboveModified)
     {
         if (value > mCurrent)
         {
@@ -65,34 +90,36 @@ namespace MWMechanics
         }
     }
 
-    template<typename T>
+    template <typename T>
     T DynamicStat<T>::getRatio(bool nanIsZero) const
     {
         T modified = getModified();
-        if(modified == T{})
+        if (modified == T{})
         {
-            if(nanIsZero)
+            if (nanIsZero)
                 return modified;
-            return {1};
+            return { 1 };
         }
         return getCurrent() / modified;
     }
 
-    template<typename T>
-    void DynamicStat<T>::writeState (ESM::StatState<T>& state) const
+    template <typename T>
+    void DynamicStat<T>::writeState(ESM::StatState<T>& state) const
     {
-        mStatic.writeState (state);
+        mStatic.writeState(state);
         state.mCurrent = mCurrent;
     }
-    template<typename T>
-    void DynamicStat<T>::readState (const ESM::StatState<T>& state)
+    template <typename T>
+    void DynamicStat<T>::readState(const ESM::StatState<T>& state)
     {
-        mStatic.readState (state);
+        mStatic.readState(state);
         mCurrent = state.mCurrent;
     }
 
-    AttributeValue::AttributeValue() :
-        mBase(0.f), mModifier(0.f), mDamage(0.f)
+    AttributeValue::AttributeValue()
+        : mBase(0.f)
+        , mModifier(0.f)
+        , mDamage(0.f)
     {
     }
 
@@ -112,7 +139,7 @@ namespace MWMechanics
     void AttributeValue::setBase(float base, bool clearModifier)
     {
         mBase = base;
-        if(clearModifier)
+        if (clearModifier)
         {
             mModifier = 0.f;
             mDamage = 0.f;
@@ -121,7 +148,7 @@ namespace MWMechanics
 
     void AttributeValue::setModifier(float mod)
     {
-        if(mod < 0)
+        if (mod < 0)
         {
             mModifier = 0.f;
             mDamage -= mod;
@@ -136,7 +163,8 @@ namespace MWMechanics
     }
     void AttributeValue::restore(float amount)
     {
-        if (mDamage <= 0) return;
+        if (mDamage <= 0)
+            return;
 
         mDamage -= std::min(mDamage, amount);
     }
@@ -146,22 +174,22 @@ namespace MWMechanics
         return mDamage;
     }
 
-    void AttributeValue::writeState (ESM::StatState<float>& state) const
+    void AttributeValue::writeState(ESM::StatState<float>& state) const
     {
         state.mBase = mBase;
         state.mMod = mModifier;
         state.mDamage = mDamage;
     }
 
-    void AttributeValue::readState (const ESM::StatState<float>& state)
+    void AttributeValue::readState(const ESM::StatState<float>& state)
     {
         mBase = state.mBase;
         mModifier = state.mMod;
         mDamage = state.mDamage;
     }
 
-    SkillValue::SkillValue() :
-        mProgress(0)
+    SkillValue::SkillValue()
+        : mProgress(0)
     {
     }
 
@@ -174,15 +202,15 @@ namespace MWMechanics
         mProgress = progress;
     }
 
-    void SkillValue::writeState (ESM::StatState<float>& state) const
+    void SkillValue::writeState(ESM::StatState<float>& state) const
     {
-        AttributeValue::writeState (state);
+        AttributeValue::writeState(state);
         state.mProgress = mProgress;
     }
 
-    void SkillValue::readState (const ESM::StatState<float>& state)
+    void SkillValue::readState(const ESM::StatState<float>& state)
     {
-        AttributeValue::readState (state);
+        AttributeValue::readState(state);
         mProgress = state.mProgress;
     }
 }
