@@ -1,6 +1,8 @@
 #include "weaponpriority.hpp"
 
+#include <components/esm3/loadcrea.hpp>
 #include <components/esm3/loadench.hpp>
+#include <components/esm3/loadgmst.hpp>
 
 #include "../mwbase/environment.hpp"
 #include "../mwbase/world.hpp"
@@ -9,16 +11,16 @@
 #include "../mwworld/esmstore.hpp"
 #include "../mwworld/inventorystore.hpp"
 
-#include "combat.hpp"
 #include "aicombataction.hpp"
+#include "combat.hpp"
 #include "spellpriority.hpp"
 #include "spellutil.hpp"
 #include "weapontype.hpp"
 
 namespace MWMechanics
 {
-    float rateWeapon (const MWWorld::Ptr &item, const MWWorld::Ptr& actor, const MWWorld::Ptr& enemy, int type,
-                      float arrowRating, float boltRating)
+    float rateWeapon(const MWWorld::Ptr& item, const MWWorld::Ptr& actor, const MWWorld::Ptr& enemy, int type,
+        float arrowRating, float boltRating)
     {
         if (enemy.isEmpty() || item.getType() != ESM::Weapon::sRecordId)
             return 0.f;
@@ -38,7 +40,7 @@ namespace MWMechanics
         if (type == -1 && weapclass == ESM::WeaponType::Ammo)
             return 0.f;
 
-        float rating=0.f;
+        float rating = 0.f;
         static const float fAIMeleeWeaponMult = gmst.find("fAIMeleeWeaponMult")->mValue.getFloat();
         float ratingMult = fAIMeleeWeaponMult;
 
@@ -46,7 +48,7 @@ namespace MWMechanics
         {
             // Underwater ranged combat is impossible
             if (world->isUnderwater(MWWorld::ConstPtr(actor), 0.75f)
-             || world->isUnderwater(MWWorld::ConstPtr(enemy), 0.75f))
+                || world->isUnderwater(MWWorld::ConstPtr(enemy), 0.75f))
                 return 0.f;
 
             // Use a higher rating multiplier if the actor is out of enemy's reach, use the normal mult otherwise
@@ -106,24 +108,29 @@ namespace MWMechanics
             const ESM::Enchantment* enchantment = world->getStore().get<ESM::Enchantment>().find(weapon->mEnchant);
             if (enchantment->mData.mType == ESM::Enchantment::WhenStrikes)
             {
-                int castCost = getEffectiveEnchantmentCastCost(static_cast<float>(enchantment->mData.mCost), actor);
+                int castCost = getEffectiveEnchantmentCastCost(*enchantment, actor);
                 float charge = item.getCellRef().getEnchantmentCharge();
 
-                if (charge == -1 || charge >= castCost || weapclass == ESM::WeaponType::Thrown || weapclass == ESM::WeaponType::Ammo)
-                    rating += rateEffects(enchantment->mEffects, actor, enemy);
+                if (charge == -1 || charge >= castCost || weapclass == ESM::WeaponType::Thrown
+                    || weapclass == ESM::WeaponType::Ammo)
+                    rating += rateEffects(enchantment->mEffects, actor, enemy, false);
             }
         }
 
         int value = 50.f;
-        if (actor.getClass().isNpc())
+        ESM::RefId skill = item.getClass().getEquipmentSkill(item);
+        if (!skill.empty())
+            value = actor.getClass().getSkill(actor, skill);
+        // Prefer hand-to-hand if our skill is 0 (presumably due to magic)
+        if (value <= 0.f)
+            return 0.f;
+        // Note that a creature with a dagger and 0 Stealth will forgo the weapon despite using Combat for hit chance.
+        // The same creature will use a sword provided its Combat stat isn't 0. We're using the "skill" value here to
+        // decide whether to use the weapon at all, but adjusting the final rating based on actual hit chance - i.e. the
+        // Combat stat.
+        if (!actor.getClass().isNpc())
         {
-            int skill = item.getClass().getEquipmentSkill(item);
-            if (skill != -1)
-               value = actor.getClass().getSkill(actor, skill);
-        }
-        else
-        {
-            MWWorld::LiveCellRef<ESM::Creature> *ref = actor.get<ESM::Creature>();
+            MWWorld::LiveCellRef<ESM::Creature>* ref = actor.get<ESM::Creature>();
             value = ref->mBase->mData.mCombat;
         }
 
@@ -136,7 +143,7 @@ namespace MWMechanics
         return rating * ratingMult;
     }
 
-    float rateAmmo(const MWWorld::Ptr &actor, const MWWorld::Ptr &enemy, MWWorld::Ptr &bestAmmo, int ammoType)
+    float rateAmmo(const MWWorld::Ptr& actor, const MWWorld::Ptr& enemy, MWWorld::Ptr& bestAmmo, int ammoType)
     {
         float bestAmmoRating = 0.f;
         if (!actor.getClass().hasInventoryStore(actor))
@@ -157,15 +164,17 @@ namespace MWMechanics
         return bestAmmoRating;
     }
 
-    float rateAmmo(const MWWorld::Ptr &actor, const MWWorld::Ptr &enemy, int ammoType)
+    float rateAmmo(const MWWorld::Ptr& actor, const MWWorld::Ptr& enemy, int ammoType)
     {
         MWWorld::Ptr emptyPtr;
         return rateAmmo(actor, enemy, emptyPtr, ammoType);
     }
 
-    float vanillaRateWeaponAndAmmo(const MWWorld::Ptr& weapon, const MWWorld::Ptr& ammo, const MWWorld::Ptr& actor, const MWWorld::Ptr& enemy)
+    float vanillaRateWeaponAndAmmo(
+        const MWWorld::Ptr& weapon, const MWWorld::Ptr& ammo, const MWWorld::Ptr& actor, const MWWorld::Ptr& enemy)
     {
-        const MWWorld::Store<ESM::GameSetting>& gmst = MWBase::Environment::get().getWorld()->getStore().get<ESM::GameSetting>();
+        const MWWorld::Store<ESM::GameSetting>& gmst
+            = MWBase::Environment::get().getESMStore()->get<ESM::GameSetting>();
 
         static const float fAIMeleeWeaponMult = gmst.find("fAIMeleeWeaponMult")->mValue.getFloat();
         static const float fAIMeleeArmorMult = gmst.find("fAIMeleeArmorMult")->mValue.getFloat();
@@ -196,7 +205,7 @@ namespace MWMechanics
         float thrustRating = esmWeap->mData.mThrust[1] * skillMult * fAIMeleeWeaponMult;
 
         return actor.getClass().getArmorRating(actor) * fAIMeleeArmorMult
-                    + std::max(std::max(chopRating, slashRating), thrustRating);
+            + std::max(std::max(chopRating, slashRating), thrustRating);
     }
 
 }
