@@ -14,16 +14,19 @@ namespace MWPhysics
 {
     class ContactTestResultCallback : public btCollisionWorld::ContactResultCallback
     {
-        const btCollisionObject* mTestedAgainst;
-
     public:
-        ContactTestResultCallback(const btCollisionObject* testedAgainst);
+        explicit ContactTestResultCallback(const btCollisionObject* testedAgainst)
+            : mTestedAgainst(testedAgainst)
+        {
+        }
 
-        btScalar addSingleResult(btManifoldPoint& cp,
-                                         const btCollisionObjectWrapper* col0Wrap,int partId0,int index0,
-                                         const btCollisionObjectWrapper* col1Wrap,int partId1,int index1) override;
+        btScalar addSingleResult(btManifoldPoint& cp, const btCollisionObjectWrapper* col0Wrap, int partId0, int index0,
+            const btCollisionObjectWrapper* col1Wrap, int partId1, int index1) override;
 
         std::vector<ContactPoint> mResult;
+
+    private:
+        const btCollisionObject* mTestedAgainst;
     };
 }
 
