@@ -16,15 +16,16 @@ namespace MWPhysics
     class Stepper
     {
     private:
-        const btCollisionWorld *mColWorld;
-        const btCollisionObject *mColObj;
+        const btCollisionWorld* mColWorld;
+        const btCollisionObject* mColObj;
 
         ActorTracer mTracer, mUpStepper, mDownStepper;
 
     public:
-        Stepper(const btCollisionWorld *colWorld, const btCollisionObject *colObj);
+        Stepper(const btCollisionWorld* colWorld, const btCollisionObject* colObj);
 
-        bool step(osg::Vec3f &position, osg::Vec3f &velocity, float &remainingTime, const bool & onGround, bool firstIteration);
+        bool step(osg::Vec3f& position, osg::Vec3f& velocity, float& remainingTime, const bool& onGround,
+            bool firstIteration);
     };
 }
 
