@@ -6,7 +6,6 @@
 class btCollisionObject;
 class btCollisionWorld;
 
-
 namespace MWPhysics
 {
     class Actor;
@@ -20,8 +19,10 @@ namespace MWPhysics
 
         float mFraction;
 
-        void doTrace(const btCollisionObject *actor, const osg::Vec3f& start, const osg::Vec3f& end, const btCollisionWorld* world, bool attempt_short_trace = false);
-        void findGround(const Actor* actor, const osg::Vec3f& start, const osg::Vec3f& end, const btCollisionWorld* world);
+        void doTrace(const btCollisionObject* actor, const osg::Vec3f& start, const osg::Vec3f& end,
+            const btCollisionWorld* world, bool attempt_short_trace = false);
+        void findGround(
+            const Actor* actor, const osg::Vec3f& start, const osg::Vec3f& end, const btCollisionWorld* world);
     };
 }
 
