@@ -1,17 +1,17 @@
 #ifndef OPENMW_MWRENDER_AGENTSPATHS_H
 #define OPENMW_MWRENDER_AGENTSPATHS_H
 
-#include <apps/openmw/mwworld/ptr.hpp>
+#include "apps/openmw/mwworld/ptr.hpp"
 
 #include <osg/ref_ptr>
 
-#include <unordered_map>
 #include <deque>
 #include <map>
 
 namespace osg
 {
     class Group;
+    class StateSet;
 }
 
 namespace DetourNavigator
@@ -31,8 +31,8 @@ namespace MWRender
         bool toggle();
 
         void update(const MWWorld::ConstPtr& actor, const std::deque<osg::Vec3f>& path,
-                const DetourNavigator::AgentBounds& agentBounds, const osg::Vec3f& start, const osg::Vec3f& end,
-                const DetourNavigator::Settings& settings);
+            const DetourNavigator::AgentBounds& agentBounds, const osg::Vec3f& start, const osg::Vec3f& end,
+            const DetourNavigator::Settings& settings);
 
         void remove(const MWWorld::ConstPtr& actor);
 
@@ -56,6 +56,8 @@ namespace MWRender
         osg::ref_ptr<osg::Group> mRootNode;
         Groups mGroups;
         bool mEnabled;
+        osg::ref_ptr<osg::StateSet> mGroupStateSet;
+        osg::ref_ptr<osg::StateSet> mDebugDrawStateSet;
     };
 }
 
