@@ -3,9 +3,8 @@
 #include <osg/Camera>
 
 #include <components/misc/mathutil.hpp>
-#include <components/sceneutil/positionattitudetransform.hpp>
-#include <components/settings/settings.hpp>
 #include <components/sceneutil/nodecallback.hpp>
+#include <components/sceneutil/positionattitudetransform.hpp>
 
 #include "../mwbase/environment.hpp"
 #include "../mwbase/windowmanager.hpp"
@@ -15,9 +14,7 @@
 #include "../mwworld/ptr.hpp"
 #include "../mwworld/refdata.hpp"
 
-#include "../mwmechanics/drawstate.hpp"
 #include "../mwmechanics/movement.hpp"
-#include "../mwmechanics/npcstats.hpp"
 
 #include "../mwphysics/raycasting.hpp"
 
@@ -26,55 +23,57 @@
 namespace
 {
 
-class UpdateRenderCameraCallback : public SceneUtil::NodeCallback<UpdateRenderCameraCallback, osg::Camera*>
-{
-public:
-    UpdateRenderCameraCallback(MWRender::Camera* cam)
-        : mCamera(cam)
+    class UpdateRenderCameraCallback : public SceneUtil::NodeCallback<UpdateRenderCameraCallback, osg::Camera*>
     {
-    }
+    public:
+        UpdateRenderCameraCallback(MWRender::Camera* cam)
+            : mCamera(cam)
+        {
+        }
 
-    void operator()(osg::Camera* cam, osg::NodeVisitor* nv)
-    {
-        // traverse first to update animations, in case the camera is attached to an animated node
-        traverse(cam, nv);
+        void operator()(osg::Camera* cam, osg::NodeVisitor* nv)
+        {
+            // traverse first to update animations, in case the camera is attached to an animated node
+            traverse(cam, nv);
 
-        mCamera->updateCamera(cam);
-    }
+            mCamera->updateCamera(cam);
+        }
 
-private:
-    MWRender::Camera* mCamera;
-};
+    private:
+        MWRender::Camera* mCamera;
+    };
 
 }
 
 namespace MWRender
 {
 
-    Camera::Camera (osg::Camera* camera)
-    : mHeightScale(1.f),
-      mCollisionType((MWPhysics::CollisionType::CollisionType_Default & ~MWPhysics::CollisionType::CollisionType_Actor) | MWPhysics::CollisionType_CameraOnly),
-      mCamera(camera),
-      mAnimation(nullptr),
-      mFirstPersonView(true),
-      mMode(Mode::FirstPerson),
-      mVanityAllowed(true),
-      mDeferredRotationAllowed(true),
-      mProcessViewChange(false),
-      mHeight(124.f),
-      mPitch(0.f),
-      mYaw(0.f),
-      mRoll(0.f),
-      mCameraDistance(0.f),
-      mPreferredCameraDistance(0.f),
-      mFocalPointCurrentOffset(osg::Vec2d()),
-      mFocalPointTargetOffset(osg::Vec2d()),
-      mFocalPointTransitionSpeedCoef(1.f),
-      mSkipFocalPointTransition(true),
-      mPreviousTransitionInfluence(0.f),
-      mShowCrosshair(false),
-      mDeferredRotation(osg::Vec3f()),
-      mDeferredRotationDisabled(false)
+    Camera::Camera(osg::Camera* camera)
+        : mHeightScale(1.f)
+        , mCollisionType(
+              (MWPhysics::CollisionType::CollisionType_Default & ~MWPhysics::CollisionType::CollisionType_Actor)
+              | MWPhysics::CollisionType_CameraOnly)
+        , mCamera(camera)
+        , mAnimation(nullptr)
+        , mFirstPersonView(true)
+        , mMode(Mode::FirstPerson)
+        , mVanityAllowed(true)
+        , mDeferredRotationAllowed(true)
+        , mProcessViewChange(false)
+        , mHeight(124.f)
+        , mPitch(0.f)
+        , mYaw(0.f)
+        , mRoll(0.f)
+        , mCameraDistance(0.f)
+        , mPreferredCameraDistance(0.f)
+        , mFocalPointCurrentOffset(osg::Vec2d())
+        , mFocalPointTargetOffset(osg::Vec2d())
+        , mFocalPointTransitionSpeedCoef(1.f)
+        , mSkipFocalPointTransition(true)
+        , mPreviousTransitionInfluence(0.f)
+        , mShowCrosshair(false)
+        , mDeferredRotation(osg::Vec3f())
+        , mDeferredRotationDisabled(false)
     {
         mUpdateCallback = new UpdateRenderCameraCallback(this);
         mCamera->addUpdateCallback(mUpdateCallback);
@@ -108,13 +107,11 @@ namespace MWRender
         return offset;
     }
 
-    void Camera::updateCamera(osg::Camera *cam)
+    void Camera::updateCamera(osg::Camera* cam)
     {
-        osg::Quat orient = osg::Quat(mRoll + mExtraRoll, osg::Vec3d(0, 1, 0)) *
-                           osg::Quat(mPitch + mExtraPitch, osg::Vec3d(1, 0, 0)) *
-                           osg::Quat(mYaw + mExtraYaw, osg::Vec3d(0, 0, 1));
-        osg::Vec3d forward = orient * osg::Vec3d(0,1,0);
-        osg::Vec3d up = orient * osg::Vec3d(0,0,1);
+        osg::Quat orient = getOrient();
+        osg::Vec3d forward = orient * osg::Vec3d(0, 1, 0);
+        osg::Vec3d up = orient * osg::Vec3d(0, 0, 1);
 
         osg::Vec3d pos = mPosition;
         if (mMode == Mode::FirstPerson)
@@ -129,6 +126,7 @@ namespace MWRender
         }
         cam->setViewMatrixAsLookAt(pos, pos + forward, up);
         mViewMatrix = cam->getViewMatrix();
+        mProjectionMatrix = cam->getProjectionMatrix();
     }
 
     void Camera::update(float duration, bool paused)
@@ -139,21 +137,20 @@ namespace MWRender
         if (mProcessViewChange)
             processViewChange();
 
-        if (paused)
-            return;
-
         // only show the crosshair in game mode
-        MWBase::WindowManager *wm = MWBase::Environment::get().getWindowManager();
+        MWBase::WindowManager* wm = MWBase::Environment::get().getWindowManager();
         wm->showCrosshair(!wm->isGuiMode() && mShowCrosshair);
 
-        updateFocalPointOffset(duration);
+        if (!paused)
+            updateFocalPointOffset(duration);
         updatePosition();
     }
 
     osg::Vec3d Camera::calculateFirstPersonPosition(const osg::Vec3d& trackedPosition) const
     {
         osg::Vec3d res = trackedPosition;
-        osg::Vec2f horizontalOffset = Misc::rotateVec2f(osg::Vec2f(mFirstPersonOffset.x(), mFirstPersonOffset.y()), mYaw);
+        osg::Vec2f horizontalOffset
+            = Misc::rotateVec2f(osg::Vec2f(mFirstPersonOffset.x(), mFirstPersonOffset.y()), mYaw);
         res.x() += horizontalOffset.x();
         res.y() += horizontalOffset.y();
         res.z() += mFirstPersonOffset.z();
@@ -180,24 +177,28 @@ namespace MWRender
         // Adjust focal point to prevent clipping.
         osg::Vec3d focalOffset = getFocalPointOffset();
         osg::Vec3d focal = mTrackedPosition + focalOffset;
-        focalOffset.z() += 10.f;  // Needed to avoid camera clipping through the ceiling because
-                                  // character's head can be a bit higher than the collision area.
+        focalOffset.z() += 10.f; // Needed to avoid camera clipping through the ceiling because
+                                 // character's head can be a bit higher than the collision area.
         float offsetLen = focalOffset.length();
         if (offsetLen > 0)
         {
-            MWPhysics::RayCastingResult result = rayCasting->castSphere(focal - focalOffset, focal, focalObstacleLimit, mCollisionType);
+            MWPhysics::RayCastingResult result
+                = rayCasting->castSphere(focal - focalOffset, focal, focalObstacleLimit, mCollisionType);
             if (result.mHit)
             {
-                double adjustmentCoef = -(result.mHitPos + result.mHitNormal * focalObstacleLimit - focal).length() / offsetLen;
+                double adjustmentCoef
+                    = -(result.mHitPos + result.mHitNormal * focalObstacleLimit - focal).length() / offsetLen;
                 focal += focalOffset * std::max(-1.0, adjustmentCoef);
             }
         }
 
         // Adjust camera distance.
         mCameraDistance = mPreferredCameraDistance;
-        osg::Quat orient =  osg::Quat(mPitch + mExtraPitch, osg::Vec3d(1,0,0)) * osg::Quat(mYaw + mExtraYaw, osg::Vec3d(0,0,1));
+        osg::Quat orient
+            = osg::Quat(mPitch + mExtraPitch, osg::Vec3d(1, 0, 0)) * osg::Quat(mYaw + mExtraYaw, osg::Vec3d(0, 0, 1));
         osg::Vec3d offset = orient * osg::Vec3d(0.f, -mCameraDistance, 0.f);
-        MWPhysics::RayCastingResult result = rayCasting->castSphere(focal, focal + offset, cameraObstacleLimit, mCollisionType);
+        MWPhysics::RayCastingResult result
+            = rayCasting->castSphere(focal, focal + offset, cameraObstacleLimit, mCollisionType);
         if (result.mHit)
         {
             mCameraDistance = (result.mHitPos + result.mHitNormal * cameraObstacleLimit - focal).length();
@@ -207,6 +208,12 @@ namespace MWRender
         mPosition = focal + offset;
     }
 
+    osg::Quat Camera::getOrient() const
+    {
+        return osg::Quat(mRoll + mExtraRoll, osg::Vec3d(0, 1, 0)) * osg::Quat(mPitch + mExtraPitch, osg::Vec3d(1, 0, 0))
+            * osg::Quat(mYaw + mExtraYaw, osg::Vec3d(0, 0, 1));
+    }
+
     void Camera::setMode(Mode newMode, bool force)
     {
         if (mMode == newMode)
@@ -215,7 +222,8 @@ namespace MWRender
             return;
         }
         Mode oldMode = mMode;
-        if (!force && (newMode == Mode::FirstPerson || oldMode == Mode::FirstPerson) && mAnimation && !mAnimation->upperBodyReady())
+        if (!force && (newMode == Mode::FirstPerson || oldMode == Mode::FirstPerson) && mAnimation
+            && !mAnimation->upperBodyReady())
         {
             // Changing the view will stop all playing animations, so if we are playing
             // anything important, queue the view change for later
@@ -261,9 +269,10 @@ namespace MWRender
         if (mPreviousTransitionInfluence > 0)
         {
             mFocalPointCurrentOffset -= mPreviousExtraOffset;
-            mPreviousExtraOffset = mPreviousExtraOffset / mPreviousTransitionInfluence + mPreviousTransitionSpeed * duration;
-            mPreviousTransitionInfluence =
-                std::max(0.f, mPreviousTransitionInfluence - duration * mFocalPointTransitionSpeedCoef);
+            mPreviousExtraOffset
+                = mPreviousExtraOffset / mPreviousTransitionInfluence + mPreviousTransitionSpeed * duration;
+            mPreviousTransitionInfluence
+                = std::max(0.f, mPreviousTransitionInfluence - duration * mFocalPointTransitionSpeedCoef);
             mPreviousExtraOffset *= mPreviousTransitionInfluence;
             mFocalPointCurrentOffset += mPreviousExtraOffset;
         }
@@ -271,8 +280,8 @@ namespace MWRender
         osg::Vec2d delta = mFocalPointTargetOffset - mFocalPointCurrentOffset;
         if (delta.length2() > 0)
         {
-            float coef = duration * (1.0 + 5.0 / delta.length()) *
-                         mFocalPointTransitionSpeedCoef * (1.0f - mPreviousTransitionInfluence);
+            float coef = duration * (1.0 + 5.0 / delta.length()) * mFocalPointTransitionSpeedCoef
+                * (1.0f - mPreviousTransitionInfluence);
             mFocalPointCurrentOffset += delta * std::min(coef, 1.0f);
         }
         else
@@ -300,7 +309,7 @@ namespace MWRender
 
     void Camera::setSneakOffset(float offset)
     {
-        mAnimation->setFirstPersonOffset(osg::Vec3f(0,0,-offset));
+        mAnimation->setFirstPersonOffset(osg::Vec3f(0, 0, -offset));
     }
 
     void Camera::setYaw(float angle, bool force)
@@ -328,7 +337,7 @@ namespace MWRender
         mPosition = pos;
     }
 
-    void Camera::setAnimation(NpcAnimation *anim)
+    void Camera::setAnimation(NpcAnimation* anim)
     {
         mAnimation = anim;
         mProcessViewChange = true;
@@ -387,7 +396,7 @@ namespace MWRender
             float c = std::cos(mDeferredRotation.z());
             float x = movement.mPosition[0];
             float y = movement.mPosition[1];
-            movement.mPosition[0] = x *  c + y * s;
+            movement.mPosition[0] = x * c + y * s;
             movement.mPosition[1] = x * -s + y * c;
         }
     }
