@@ -1,8 +1,8 @@
 #ifndef MWRENDER_CHARACTERPREVIEW_H
 #define MWRENDER_CHARACTERPREVIEW_H
 
-#include <osg/ref_ptr>
 #include <memory>
+#include <osg/ref_ptr>
 
 #include <osg/PositionAttitudeTransform>
 
@@ -31,8 +31,8 @@ namespace MWRender
     class CharacterPreview
     {
     public:
-        CharacterPreview(osg::Group* parent, Resource::ResourceSystem* resourceSystem, const MWWorld::Ptr& character, int sizeX, int sizeY,
-                         const osg::Vec3f& position, const osg::Vec3f& lookAt);
+        CharacterPreview(osg::Group* parent, Resource::ResourceSystem* resourceSystem, const MWWorld::Ptr& character,
+            int sizeX, int sizeY, const osg::Vec3f& position, const osg::Vec3f& lookAt);
         virtual ~CharacterPreview();
 
         int getTextureWidth() const;
@@ -77,7 +77,6 @@ namespace MWRender
     class InventoryPreview : public CharacterPreview
     {
     public:
-
         InventoryPreview(osg::Group* parent, Resource::ResourceSystem* resourceSystem, const MWWorld::Ptr& character);
 
         void updatePtr(const MWWorld::Ptr& ptr);
@@ -97,11 +96,10 @@ namespace MWRender
 
     class RaceSelectionPreview : public CharacterPreview
     {
-        ESM::NPC                        mBase;
-        MWWorld::LiveCellRef<ESM::NPC>  mRef;
+        ESM::NPC mBase;
+        MWWorld::LiveCellRef<ESM::NPC> mRef;
 
     protected:
-
         bool renderHeadOnly() override { return true; }
         void onSetup() override;
 
@@ -111,14 +109,11 @@ namespace MWRender
 
         void setAngle(float angleRadians);
 
-        const ESM::NPC &getPrototype() const {
-            return mBase;
-        }
+        const ESM::NPC& getPrototype() const { return mBase; }
 
-        void setPrototype(const ESM::NPC &proto);
+        void setPrototype(const ESM::NPC& proto);
 
     private:
-
         osg::ref_ptr<UpdateCameraCallback> mUpdateCameraCallback;
 
         float mPitchRadians;
