@@ -2,11 +2,12 @@
 #define OPENMW_MWRENDER_EFFECTMANAGER_H
 
 #include <memory>
-#include <string>
 #include <vector>
 
 #include <osg/ref_ptr>
 
+#include <components/vfs/pathutil.hpp>
+
 namespace osg
 {
     class Group;
@@ -33,7 +34,8 @@ namespace MWRender
         ~EffectManager();
 
         /// Add an effect. When it's finished playing, it will be removed automatically.
-        void addEffect (const std::string& model, const std::string& textureOverride, const osg::Vec3f& worldPosition, float scale, bool isMagicVFX = true);
+        void addEffect(VFS::Path::NormalizedView model, std::string_view textureOverride,
+            const osg::Vec3f& worldPosition, float scale, bool isMagicVFX = true, bool useAmbientLight = true);
 
         void update(float dt);
 
