@@ -1,9 +1,9 @@
 #ifndef GAME_RENDER_GLOBALMAP_H
 #define GAME_RENDER_GLOBALMAP_H
 
+#include <map>
 #include <string>
 #include <vector>
-#include <map>
 
 #include <osg/ref_ptr>
 
@@ -43,7 +43,7 @@ namespace MWRender
 
         void worldPosToImageSpace(float x, float z, float& imageX, float& imageY);
 
-        void exploreCell (int cellX, int cellY, osg::ref_ptr<osg::Texture2D> localMapTexture);
+        void exploreCell(int cellX, int cellY, osg::ref_ptr<osg::Texture2D> localMapTexture);
 
         /// Clears the overlay
         void clear();
@@ -64,8 +64,8 @@ namespace MWRender
          */
         void markForRemoval(osg::Camera* camera);
 
-        void write (ESM::GlobalMap& map);
-        void read (ESM::GlobalMap& map);
+        void write(ESM::GlobalMap& map);
+        void read(ESM::GlobalMap& map);
 
         osg::ref_ptr<osg::Texture2D> getBaseTexture();
         osg::ref_ptr<osg::Texture2D> getOverlayTexture();
@@ -82,14 +82,13 @@ namespace MWRender
          * x, y, width and height are the destination coordinates (top-left coordinate origin)
          * @param cpuCopy copy the resulting render onto mOverlayImage as well?
          */
-        void requestOverlayTextureUpdate(int x, int y, int width, int height, osg::ref_ptr<osg::Texture2D> texture, bool clear, bool cpuCopy,
-                                         float srcLeft = 0.f, float srcTop = 0.f, float srcRight = 1.f, float srcBottom = 1.f);
-
-        int mCellSize;
+        void requestOverlayTextureUpdate(int x, int y, int width, int height, osg::ref_ptr<osg::Texture2D> texture,
+            bool clear, bool cpuCopy, float srcLeft = 0.f, float srcTop = 0.f, float srcRight = 1.f,
+            float srcBottom = 1.f);
 
         osg::ref_ptr<osg::Group> mRoot;
 
-        typedef std::vector<osg::ref_ptr<osg::Camera> > CameraVector;
+        typedef std::vector<osg::ref_ptr<osg::Camera>> CameraVector;
         CameraVector mActiveCameras;
 
         CameraVector mCamerasPendingRemoval;
@@ -97,7 +96,8 @@ namespace MWRender
         struct ImageDest
         {
             ImageDest()
-                : mX(0), mY(0)
+                : mX(0)
+                , mY(0)
                 , mFrameDone(0)
             {
             }
@@ -111,8 +111,6 @@ namespace MWRender
 
         ImageDestMap mPendingImageDest;
 
-        std::vector< std::pair<int,int> > mExploredCells;
-
         osg::ref_ptr<osg::Texture2D> mBaseTexture;
         osg::ref_ptr<osg::Texture2D> mAlphaTexture;
 
@@ -136,4 +134,3 @@ namespace MWRender
 }
 
 #endif
-
