@@ -1,9 +1,10 @@
 #ifndef OPENMW_MWRENDER_GROUNDCOVER_H
 #define OPENMW_MWRENDER_GROUNDCOVER_H
 
-#include <components/terrain/quadtreeworld.hpp>
-#include <components/resource/scenemanager.hpp>
 #include <components/esm3/loadcell.hpp>
+#include <components/resource/scenemanager.hpp>
+#include <components/terrain/quadtreeworld.hpp>
+#include <components/vfs/pathutil.hpp>
 
 namespace MWWorld
 {
@@ -18,13 +19,16 @@ namespace osg
 namespace MWRender
 {
     typedef std::tuple<osg::Vec2f, float> GroundcoverChunkId; // Center, Size
-    class Groundcover : public Resource::GenericResourceManager<GroundcoverChunkId>, public Terrain::QuadTreeWorld::ChunkManager
+    class Groundcover : public Resource::GenericResourceManager<GroundcoverChunkId>,
+                        public Terrain::QuadTreeWorld::ChunkManager
     {
     public:
-        Groundcover(Resource::SceneManager* sceneManager, float density, float viewDistance, const MWWorld::GroundcoverStore& store);
+        Groundcover(Resource::SceneManager* sceneManager, float density, float viewDistance,
+            const MWWorld::GroundcoverStore& store);
         ~Groundcover();
 
-        osg::ref_ptr<osg::Node> getChunk(float size, const osg::Vec2f& center, unsigned char lod, unsigned int lodFlags, bool activeGrid, const osg::Vec3f& viewPoint, bool compile) override;
+        osg::ref_ptr<osg::Node> getChunk(float size, const osg::Vec2f& center, unsigned char lod, unsigned int lodFlags,
+            bool activeGrid, const osg::Vec3f& viewPoint, bool compile) override;
 
         unsigned int getNodeMask() override;
 
@@ -35,18 +39,22 @@ namespace MWRender
             ESM::Position mPos;
             float mScale;
 
-            GroundcoverEntry(const ESM::CellRef& ref) : mPos(ref.mPos), mScale(ref.mScale)
-            {}
+            GroundcoverEntry(const ESM::CellRef& ref)
+                : mPos(ref.mPos)
+                , mScale(ref.mScale)
+            {
+            }
         };
 
     private:
+        using InstanceMap = std::map<VFS::Path::Normalized, std::vector<GroundcoverEntry>, std::less<>>;
+
         Resource::SceneManager* mSceneManager;
         float mDensity;
         osg::ref_ptr<osg::StateSet> mStateset;
         osg::ref_ptr<osg::Program> mProgramTemplate;
         const MWWorld::GroundcoverStore& mGroundcoverStore;
 
-        typedef std::map<std::string, std::vector<GroundcoverEntry>> InstanceMap;
         osg::ref_ptr<osg::Node> createChunk(InstanceMap& instances, const osg::Vec2f& center);
         void collectInstances(InstanceMap& instances, float size, const osg::Vec2f& center);
     };
