@@ -1,9 +1,9 @@
 #ifndef OPENMW_MWRENDER_OBJECTPAGING_H
 #define OPENMW_MWRENDER_OBJECTPAGING_H
 
-#include <components/terrain/quadtreeworld.hpp>
+#include <components/esm3/refnum.hpp>
 #include <components/resource/resourcemanager.hpp>
-#include <components/esm3/loadcell.hpp>
+#include <components/terrain/quadtreeworld.hpp>
 
 #include <mutex>
 
@@ -11,10 +11,6 @@ namespace Resource
 {
     class SceneManager;
 }
-namespace MWWorld
-{
-    class ESMStore;
-}
 
 namespace MWRender
 {
@@ -24,20 +20,22 @@ namespace MWRender
     class ObjectPaging : public Resource::GenericResourceManager<ChunkId>, public Terrain::QuadTreeWorld::ChunkManager
     {
     public:
-        ObjectPaging(Resource::SceneManager* sceneManager);
+        ObjectPaging(Resource::SceneManager* sceneManager, ESM::RefId worldspace);
         ~ObjectPaging() = default;
 
-        osg::ref_ptr<osg::Node> getChunk(float size, const osg::Vec2f& center, unsigned char lod, unsigned int lodFlags, bool activeGrid, const osg::Vec3f& viewPoint, bool compile) override;
+        osg::ref_ptr<osg::Node> getChunk(float size, const osg::Vec2f& center, unsigned char lod, unsigned int lodFlags,
+            bool activeGrid, const osg::Vec3f& viewPoint, bool compile) override;
 
-        osg::ref_ptr<osg::Node> createChunk(float size, const osg::Vec2f& center, bool activeGrid, const osg::Vec3f& viewPoint, bool compile);
+        osg::ref_ptr<osg::Node> createChunk(float size, const osg::Vec2f& center, bool activeGrid,
+            const osg::Vec3f& viewPoint, bool compile, unsigned char lod);
 
         unsigned int getNodeMask() override;
 
         /// @return true if view needs rebuild
-        bool enableObject(int type, const ESM::RefNum & refnum, const osg::Vec3f& pos, const osg::Vec2i& cell, bool enabled);
+        bool enableObject(int type, ESM::RefNum refnum, const osg::Vec3f& pos, const osg::Vec2i& cell, bool enabled);
 
         /// @return true if view needs rebuild
-        bool blacklistObject(int type, const ESM::RefNum & refnum, const osg::Vec3f& pos, const osg::Vec2i& cell);
+        bool blacklistObject(int type, ESM::RefNum refnum, const osg::Vec3f& pos, const osg::Vec2i& cell);
 
         void clear();
 
@@ -47,7 +45,7 @@ namespace MWRender
 
         void reportStats(unsigned int frameNumber, osg::Stats* stats) const override;
 
-        void getPagedRefnums(const osg::Vec4i &activeGrid, std::vector<ESM::RefNum>& out);
+        void getPagedRefnums(const osg::Vec4i& activeGrid, std::vector<ESM::RefNum>& out);
 
     private:
         Resource::SceneManager* mSceneManager;
@@ -63,7 +61,10 @@ namespace MWRender
         {
             std::set<ESM::RefNum> mDisabled;
             std::set<ESM::RefNum> mBlacklist;
-            bool operator==(const RefTracker&other) const { return mDisabled == other.mDisabled && mBlacklist == other.mBlacklist; }
+            bool operator==(const RefTracker& other) const
+            {
+                return mDisabled == other.mDisabled && mBlacklist == other.mBlacklist;
+            }
         };
         RefTracker mRefTracker;
         RefTracker mRefTrackerNew;
@@ -75,13 +76,25 @@ namespace MWRender
         std::mutex mSizeCacheMutex;
         typedef std::map<ESM::RefNum, float> SizeCache;
         SizeCache mSizeCache;
+
+        std::mutex mLODNameCacheMutex;
+        typedef std::pair<std::string, unsigned char> LODNameCacheKey; // Key: mesh name, lod level
+        using LODNameCache = std::map<LODNameCacheKey, VFS::Path::Normalized>; // Cache: key, mesh name to use
+        LODNameCache mLODNameCache;
     };
 
     class RefnumMarker : public osg::Object
     {
     public:
-        RefnumMarker() : mNumVertices(0) { mRefnum.unset(); }
-        RefnumMarker(const RefnumMarker &copy, osg::CopyOp co) : mRefnum(copy.mRefnum), mNumVertices(copy.mNumVertices) {}
+        RefnumMarker()
+            : mNumVertices(0)
+        {
+        }
+        RefnumMarker(const RefnumMarker& copy, osg::CopyOp co)
+            : mRefnum(copy.mRefnum)
+            , mNumVertices(copy.mNumVertices)
+        {
+        }
         META_Object(MWRender, RefnumMarker)
 
         ESM::RefNum mRefnum;
