@@ -1,21 +1,22 @@
 #ifndef OPENMW_MWRENDER_RENDERINGMANAGER_H
 #define OPENMW_MWRENDER_RENDERINGMANAGER_H
 
-#include <osg/ref_ptr>
-#include <osg/Light>
-#include <osg/Camera>
-
-#include <components/settings/settings.hpp>
-
-#include <osgUtil/IncrementalCompileOperation>
-
 #include "objects.hpp"
-#include "navmeshmode.hpp"
 #include "renderinginterface.hpp"
 #include "rendermode.hpp"
 
+#include <components/settings/settings.hpp>
+#include <components/vfs/pathutil.hpp>
+
+#include <osg/Light>
+#include <osg/ref_ptr>
+
+#include <osgUtil/IncrementalCompileOperation>
+
 #include <deque>
 #include <memory>
+#include <span>
+#include <unordered_map>
 
 namespace osg
 {
@@ -42,7 +43,8 @@ namespace osgViewer
 namespace ESM
 {
     struct Cell;
-    struct RefNum;
+    struct FormId;
+    using RefNum = FormId;
 }
 
 namespace Terrain
@@ -73,6 +75,12 @@ namespace DetourNavigator
 namespace MWWorld
 {
     class GroundcoverStore;
+    class Cell;
+}
+
+namespace Debug
+{
+    struct DebugDrawer;
 }
 
 namespace MWRender
@@ -80,6 +88,7 @@ namespace MWRender
     class StateUpdater;
     class SharedUniformStateUpdater;
     class PerViewUniformStateUpdater;
+    class IntersectionVisitorWithIgnoreList;
 
     class EffectManager;
     class ScreenshotManager;
@@ -102,7 +111,7 @@ namespace MWRender
     {
     public:
         RenderingManager(osgViewer::Viewer* viewer, osg::ref_ptr<osg::Group> rootNode,
-            Resource::ResourceSystem* resourceSystem, SceneUtil::WorkQueue* workQueue, const std::string& resourcePath,
+            Resource::ResourceSystem* resourceSystem, SceneUtil::WorkQueue* workQueue,
             DetourNavigator::Navigator& navigator, const MWWorld::GroundcoverStore& groundcoverStore,
             SceneUtil::UnrefQueue& unrefQueue);
         ~RenderingManager();
@@ -126,7 +135,6 @@ namespace MWRender
 
         void setAmbientColour(const osg::Vec4f& colour);
 
-        void skySetDate(int day, int month);
         int skyGetMasserPhase() const;
         int skyGetSecundaPhase() const;
         void skySetMoonColour(bool red);
@@ -135,14 +143,15 @@ namespace MWRender
         void setSunColour(const osg::Vec4f& diffuse, const osg::Vec4f& specular, float sunVis);
         void setNight(bool isNight) { mNight = isNight; }
 
-        void configureAmbient(const ESM::Cell* cell);
-        void configureFog(const ESM::Cell* cell);
-        void configureFog(float fogDepth, float underwaterFog, float dlFactor, float dlOffset, const osg::Vec4f& colour);
+        void configureAmbient(const MWWorld::Cell& cell);
+        void configureFog(const MWWorld::Cell& cell);
+        void configureFog(
+            float fogDepth, float underwaterFog, float dlFactor, float dlOffset, const osg::Vec4f& colour);
 
         void addCell(const MWWorld::CellStore* store);
         void removeCell(const MWWorld::CellStore* store);
 
-        void enableTerrain(bool enable);
+        void enableTerrain(bool enable, ESM::RefId worldspace);
 
         void updatePtr(const MWWorld::Ptr& old, const MWWorld::Ptr& updated);
 
@@ -157,7 +166,6 @@ namespace MWRender
 
         /// Take a screenshot of w*h onto the given image, not including the GUI.
         void screenshot(osg::Image* image, int w, int h);
-        bool screenshot360(osg::Image* image);
 
         struct RayResult
         {
@@ -169,14 +177,16 @@ namespace MWRender
             float mRatio;
         };
 
-        RayResult castRay(const osg::Vec3f& origin, const osg::Vec3f& dest, bool ignorePlayer, bool ignoreActors=false);
+        RayResult castRay(const osg::Vec3f& origin, const osg::Vec3f& dest, bool ignorePlayer,
+            bool ignoreActors = false, std::span<const MWWorld::Ptr> ignoreList = {});
 
-        /// Return the object under the mouse cursor / crosshair position, given by nX and nY normalized screen coordinates,
-        /// where (0,0) is the top left corner.
-        RayResult castCameraToViewportRay(const float nX, const float nY, float maxDistance, bool ignorePlayer, bool ignoreActors=false);
+        /// Return the object under the mouse cursor / crosshair position, given by nX and nY normalized screen
+        /// coordinates, where (0,0) is the top left corner.
+        RayResult castCameraToViewportRay(
+            const float nX, const float nY, float maxDistance, bool ignorePlayer, bool ignoreActors = false);
 
-        /// Get the bounding box of the given object in screen coordinates as (minX, minY, maxX, maxY), with (0,0) being the top left corner.
-        osg::Vec4f getScreenBounds(const osg::BoundingBox &worldbb);
+        /// Get normalized screen coordinates of the bounding box's summit, where (0,0) is the top left corner
+        osg::Vec2f getScreenCoords(const osg::BoundingBox& bb);
 
         void setSkyEnabled(bool enabled);
 
@@ -184,7 +194,8 @@ namespace MWRender
 
         SkyManager* getSkyManager();
 
-        void spawnEffect(const std::string &model, const std::string &texture, const osg::Vec3f &worldPosition, float scale = 1.f, bool isMagicVFX = true);
+        void spawnEffect(VFS::Path::NormalizedView model, std::string_view texture, const osg::Vec3f& worldPosition,
+            float scale = 1.f, bool isMagicVFX = true, bool useAmbientLight = true);
 
         /// Clear all savegame-specific data
         void clear();
@@ -203,7 +214,7 @@ namespace MWRender
         void removeWaterRippleEmitter(const MWWorld::Ptr& ptr);
         void emitWaterRipple(const osg::Vec3f& pos);
 
-        void updatePlayerPtr(const MWWorld::Ptr &ptr);
+        void updatePlayerPtr(const MWWorld::Ptr& ptr);
 
         void removePlayer(const MWWorld::Ptr& player);
         void setupPlayer(const MWWorld::Ptr& player);
@@ -218,7 +229,7 @@ namespace MWRender
 
         void setViewDistance(float distance, bool delay = false);
 
-        float getTerrainHeightAt(const osg::Vec3f& pos);
+        float getTerrainHeightAt(const osg::Vec3f& pos, ESM::RefId worldspace);
 
         // camera stuff
         Camera* getCamera() { return mCamera.get(); }
@@ -232,7 +243,13 @@ namespace MWRender
 
         osg::Vec3f getHalfExtents(const MWWorld::ConstPtr& object) const;
 
-        void exportSceneGraph(const MWWorld::Ptr& ptr, const std::string& filename, const std::string& format);
+        // Return local bounding box. Safe to be called in parallel with cull thread.
+        osg::BoundingBox getCullSafeBoundingBox(const MWWorld::Ptr& ptr) const;
+
+        void exportSceneGraph(
+            const MWWorld::Ptr& ptr, const std::filesystem::path& filename, const std::string& format);
+
+        Debug::DebugDrawer& getDebugDrawer() const { return *mDebugDraw; }
 
         LandManager* getLandManager() const;
 
@@ -245,24 +262,32 @@ namespace MWRender
 
         void setNavMeshNumber(const std::size_t value);
 
-        void setActiveGrid(const osg::Vec4i &grid);
+        void setActiveGrid(const osg::Vec4i& grid);
 
         bool pagingEnableObject(int type, const MWWorld::ConstPtr& ptr, bool enabled);
-        void pagingBlacklistObject(int type, const MWWorld::ConstPtr &ptr);
+        void pagingBlacklistObject(int type, const MWWorld::ConstPtr& ptr);
         bool pagingUnlockCache();
-        void getPagedRefnums(const osg::Vec4i &activeGrid, std::vector<ESM::RefNum>& out);
+        void getPagedRefnums(const osg::Vec4i& activeGrid, std::vector<ESM::RefNum>& out);
 
         void updateProjectionMatrix();
 
         void setScreenRes(int width, int height);
 
-        void setNavMeshMode(NavMeshMode value);
+        void setNavMeshMode(Settings::NavMeshRenderMode value);
 
     private:
         void updateTextureFiltering();
         void updateAmbient();
         void setFogColor(const osg::Vec4f& color);
-        void updateThirdPersonViewMode();
+
+        struct WorldspaceChunkMgr
+        {
+            std::unique_ptr<Terrain::World> mTerrain;
+            std::unique_ptr<ObjectPaging> mObjectPaging;
+            std::unique_ptr<Groundcover> mGroundcover;
+        };
+
+        WorldspaceChunkMgr& getWorldspaceChunkMgr(ESM::RefId worldspace);
 
         void reportStats() const;
 
@@ -272,9 +297,10 @@ namespace MWRender
 
         const bool mSkyBlending;
 
-        osg::ref_ptr<osgUtil::IntersectionVisitor> getIntersectionVisitor(osgUtil::Intersector* intersector, bool ignorePlayer, bool ignoreActors);
+        osg::ref_ptr<osgUtil::IntersectionVisitor> getIntersectionVisitor(osgUtil::Intersector* intersector,
+            bool ignorePlayer, bool ignoreActors, std::span<const MWWorld::Ptr> ignoreList = {});
 
-        osg::ref_ptr<osgUtil::IntersectionVisitor> mIntersectionVisitor;
+        osg::ref_ptr<IntersectionVisitorWithIgnoreList> mIntersectionVisitor;
 
         osg::ref_ptr<osgViewer::Viewer> mViewer;
         osg::ref_ptr<osg::Group> mRootNode;
@@ -293,10 +319,11 @@ namespace MWRender
         std::unique_ptr<Pathgrid> mPathgrid;
         std::unique_ptr<Objects> mObjects;
         std::unique_ptr<Water> mWater;
-        std::unique_ptr<Terrain::World> mTerrain;
+        std::unordered_map<ESM::RefId, WorldspaceChunkMgr> mWorldspaceChunks;
+        Terrain::World* mTerrain;
         std::unique_ptr<TerrainStorage> mTerrainStorage;
-        std::unique_ptr<ObjectPaging> mObjectPaging;
-        std::unique_ptr<Groundcover> mGroundcover;
+        ObjectPaging* mObjectPaging;
+        Groundcover* mGroundcover;
         std::unique_ptr<SkyManager> mSky;
         std::unique_ptr<FogManager> mFog;
         std::unique_ptr<ScreenshotManager> mScreenshotManager;
@@ -306,13 +333,13 @@ namespace MWRender
         osg::ref_ptr<NpcAnimation> mPlayerAnimation;
         osg::ref_ptr<SceneUtil::PositionAttitudeTransform> mPlayerNode;
         std::unique_ptr<Camera> mCamera;
+        osg::ref_ptr<Debug::DebugDrawer> mDebugDraw;
 
         osg::ref_ptr<StateUpdater> mStateUpdater;
         osg::ref_ptr<SharedUniformStateUpdater> mSharedUniformStateUpdater;
         osg::ref_ptr<PerViewUniformStateUpdater> mPerViewUniformStateUpdater;
 
         osg::Vec4f mAmbientColor;
-        float mMinimumAmbientLuminance;
         float mNightEyeFactor;
 
         float mNearClip;
@@ -323,8 +350,9 @@ namespace MWRender
         float mFirstPersonFieldOfView;
         bool mUpdateProjectionMatrix = false;
         bool mNight = false;
+        const MWWorld::GroundcoverStore& mGroundCoverStore;
 
-        void operator = (const RenderingManager&);
+        void operator=(const RenderingManager&);
         RenderingManager(const RenderingManager&);
     };
 
