@@ -3,47 +3,47 @@
 #include <osg/Depth>
 #include <osg/PositionAttitudeTransform>
 
-#include <osgParticle/Operator>
 #include <osgParticle/BoxPlacer>
 #include <osgParticle/ModularEmitter>
 #include <osgParticle/ModularProgram>
+#include <osgParticle/Operator>
 #include <osgParticle/ParticleSystemUpdater>
 
-#include <components/settings/settings.hpp>
+#include <components/settings/values.hpp>
 
 #include <components/sceneutil/controller.hpp>
-#include <components/sceneutil/shadow.hpp>
-#include <components/sceneutil/visitor.hpp>
 #include <components/sceneutil/depth.hpp>
 #include <components/sceneutil/rtt.hpp>
+#include <components/sceneutil/shadow.hpp>
+#include <components/sceneutil/visitor.hpp>
 
-#include <components/resource/scenemanager.hpp>
 #include <components/resource/imagemanager.hpp>
+#include <components/resource/scenemanager.hpp>
 
 #include <components/vfs/manager.hpp>
 
-#include <components/misc/rng.hpp>
 #include <components/misc/resourcehelpers.hpp>
 #include <components/stereo/stereomanager.hpp>
 
 #include <components/nifosg/particle.hpp>
 
+#include "../mwworld/datetimemanager.hpp"
 #include "../mwworld/weather.hpp"
 
 #include "../mwbase/environment.hpp"
 #include "../mwbase/world.hpp"
 
-#include "vismask.hpp"
 #include "renderbin.hpp"
-#include "util.hpp"
 #include "skyutil.hpp"
+#include "util.hpp"
+#include "vismask.hpp"
 
 namespace
 {
     class WrapAroundOperator : public osgParticle::Operator
     {
     public:
-        WrapAroundOperator(osg::Camera *camera, const osg::Vec3 &wrapRange)
+        WrapAroundOperator(osg::Camera* camera, const osg::Vec3& wrapRange)
             : osgParticle::Operator()
             , mCamera(camera)
             , mWrapRange(wrapRange)
@@ -52,21 +52,13 @@ namespace
             mPreviousCameraPosition = getCameraPosition();
         }
 
-        osg::Object *cloneType() const override
-        {
-            return nullptr;
-        }
+        osg::Object* cloneType() const override { return nullptr; }
 
-        osg::Object *clone(const osg::CopyOp &op) const override
-        {
-            return nullptr;
-        }
+        osg::Object* clone(const osg::CopyOp& op) const override { return nullptr; }
 
-        void operate(osgParticle::Particle *P, double dt) override
-        {
-        }
+        void operate(osgParticle::Particle* P, double dt) override {}
 
-        void operateParticles(osgParticle::ParticleSystem *ps, double dt) override
+        void operateParticles(osgParticle::ParticleSystem* ps, double dt) override
         {
             osg::Vec3 position = getCameraPosition();
             osg::Vec3 positionDifference = position - mPreviousCameraPosition;
@@ -75,7 +67,6 @@ namespace
 
             std::vector<osg::Matrix> worldMatrices = ps->getWorldMatrices();
 
-
             if (!worldMatrices.empty())
             {
                 toWorld = worldMatrices[0];
@@ -84,18 +75,18 @@ namespace
 
             for (int i = 0; i < ps->numParticles(); ++i)
             {
-                osgParticle::Particle *p = ps->getParticle(i);
+                osgParticle::Particle* p = ps->getParticle(i);
                 p->setPosition(toWorld.preMult(p->getPosition()));
                 p->setPosition(p->getPosition() - positionDifference);
 
-                for (int j = 0; j < 3; ++j)  // wrap-around in all 3 dimensions
+                for (int j = 0; j < 3; ++j) // wrap-around in all 3 dimensions
                 {
                     osg::Vec3 pos = p->getPosition();
 
                     if (pos[j] < -mHalfWrapRange[j])
-                        pos[j] = mHalfWrapRange[j] + fmod(pos[j] - mHalfWrapRange[j],mWrapRange[j]);
+                        pos[j] = mHalfWrapRange[j] + fmod(pos[j] - mHalfWrapRange[j], mWrapRange[j]);
                     else if (pos[j] > mHalfWrapRange[j])
-                        pos[j] = fmod(pos[j] + mHalfWrapRange[j],mWrapRange[j]) - mHalfWrapRange[j];
+                        pos[j] = fmod(pos[j] + mHalfWrapRange[j], mWrapRange[j]) - mHalfWrapRange[j];
 
                     p->setPosition(pos);
                 }
@@ -107,15 +98,12 @@ namespace
         }
 
     protected:
-        osg::Camera *mCamera;
+        osg::Camera* mCamera;
         osg::Vec3 mPreviousCameraPosition;
         osg::Vec3 mWrapRange;
         osg::Vec3 mHalfWrapRange;
 
-        osg::Vec3 getCameraPosition()
-        {
-            return mCamera->getInverseViewMatrix().getTrans();
-        }
+        osg::Vec3 getCameraPosition() { return mCamera->getInverseViewMatrix().getTrans(); }
     };
 
     class WeatherAlphaOperator : public osgParticle::Operator
@@ -124,19 +112,14 @@ namespace
         WeatherAlphaOperator(float& alpha, bool rain)
             : mAlpha(alpha)
             , mIsRain(rain)
-        { }
-
-        osg::Object *cloneType() const override
         {
-            return nullptr;
         }
 
-        osg::Object *clone(const osg::CopyOp &op) const override
-        {
-            return nullptr;
-        }
+        osg::Object* cloneType() const override { return nullptr; }
 
-        void operate(osgParticle::Particle *particle, double dt) override
+        osg::Object* clone(const osg::CopyOp& op) const override { return nullptr; }
+
+        void operate(osgParticle::Particle* particle, double dt) override
         {
             constexpr float rainThreshold = 0.6f; // Rain_Threshold?
             float alpha = mIsRain ? mAlpha * rainThreshold : mAlpha;
@@ -144,7 +127,7 @@ namespace
         }
 
     private:
-        float &mAlpha;
+        float& mAlpha;
         bool mIsRain;
     };
 
@@ -155,7 +138,8 @@ namespace
         /// @param alpha the variable alpha value is recovered from
         AlphaFader(const float& alpha)
             : mAlpha(alpha)
-        { }
+        {
+        }
 
         void setDefaults(osg::StateSet* stateset) override
         {
@@ -171,19 +155,20 @@ namespace
         }
 
     protected:
-        const float &mAlpha;
+        const float& mAlpha;
     };
 
     // Helper for adding AlphaFaders to a subgraph
     class SetupVisitor : public osg::NodeVisitor
     {
     public:
-        SetupVisitor(const float &alpha)
+        SetupVisitor(const float& alpha)
             : osg::NodeVisitor(TRAVERSE_ALL_CHILDREN)
             , mAlpha(alpha)
-        { }
+        {
+        }
 
-        void apply(osg::Node &node) override
+        void apply(osg::Node& node) override
         {
             if (osg::StateSet* stateset = node.getStateSet())
             {
@@ -214,15 +199,16 @@ namespace
         }
 
     private:
-        const float &mAlpha;
+        const float& mAlpha;
     };
 
     class SkyRTT : public SceneUtil::RTTNode
     {
     public:
-        SkyRTT(osg::Vec2f size, osg::Group* earlyRenderBinRoot) :
-            RTTNode(static_cast<int>(size.x()), static_cast<int>(size.y()), 0, false, 1, StereoAwareness::Aware),
-            mEarlyRenderBinRoot(earlyRenderBinRoot)
+        SkyRTT(osg::Vec2f size, osg::Group* earlyRenderBinRoot)
+            : RTTNode(static_cast<int>(size.x()), static_cast<int>(size.y()), 0, false, 1, StereoAwareness::Aware,
+                MWRender::shouldAddMSAAIntermediateTarget())
+            , mEarlyRenderBinRoot(earlyRenderBinRoot)
         {
             setDepthBufferInternalFormat(GL_DEPTH24_STENCIL8);
         }
@@ -234,7 +220,7 @@ namespace
             camera->setNodeMask(MWRender::Mask_RenderToTexture);
             camera->setCullMask(MWRender::Mask_Sky);
             camera->addChild(mEarlyRenderBinRoot);
-            SceneUtil::ShadowManager::disableShadowsForStateSet(camera->getOrCreateStateSet());
+            SceneUtil::ShadowManager::instance().disableShadowsForStateSet(*camera->getOrCreateStateSet());
         }
 
     private:
@@ -245,16 +231,15 @@ namespace
 
 namespace MWRender
 {
-    SkyManager::SkyManager(osg::Group* parentNode, Resource::SceneManager* sceneManager, bool enableSkyRTT)
+    SkyManager::SkyManager(osg::Group* parentNode, osg::Group* rootNode, osg::Camera* camera,
+        Resource::SceneManager* sceneManager, bool enableSkyRTT)
         : mSceneManager(sceneManager)
-        , mCamera(nullptr)
+        , mCamera(camera)
         , mAtmosphereNightRoll(0.f)
         , mCreated(false)
         , mIsStorm(false)
-        , mDay(0)
-        , mMonth(0)
+        , mTimescaleClouds(Fallback::Map::getBool("Weather_Timescale_Clouds"))
         , mCloudAnimationTimer(0.f)
-        , mRainTimer(0.f)
         , mStormParticleDirection(MWWorld::Weather::defaultDirection())
         , mStormDirection(MWWorld::Weather::defaultDirection())
         , mClouds()
@@ -262,29 +247,30 @@ namespace MWRender
         , mCloudBlendFactor(0.f)
         , mCloudSpeed(0.f)
         , mStarsOpacity(0.f)
-        , mRemainingTransitionTime(0.f)
-        , mRainEnabled(false)
         , mRainSpeed(0.f)
         , mRainDiameter(0.f)
         , mRainMinHeight(0.f)
         , mRainMaxHeight(0.f)
         , mRainEntranceSpeed(1.f)
         , mRainMaxRaindrops(0)
+        , mRainRipplesEnabled(Fallback::Map::getBool("Weather_Rain_Ripples"))
+        , mSnowRipplesEnabled(Fallback::Map::getBool("Weather_Snow_Ripples"))
         , mWindSpeed(0.f)
         , mBaseWindSpeed(0.f)
         , mEnabled(true)
-        , mSunEnabled(true)
         , mSunglareEnabled(true)
         , mPrecipitationAlpha(0.f)
         , mDirtyParticlesEffect(false)
     {
-        osg::ref_ptr<CameraRelativeTransform> skyroot = new CameraRelativeTransform;
-        skyroot->setName("Sky Root");
+        mSkyRootNode = new CameraRelativeTransform;
+        mSkyRootNode->setName("Sky Root");
         // Assign empty program to specify we don't want shaders when we are rendering in FFP pipeline
         if (!mSceneManager->getForceShaders())
-            skyroot->getOrCreateStateSet()->setAttributeAndModes(new osg::Program(), osg::StateAttribute::OVERRIDE|osg::StateAttribute::PROTECTED|osg::StateAttribute::ON);
-        SceneUtil::ShadowManager::disableShadowsForStateSet(skyroot->getOrCreateStateSet());
-        parentNode->addChild(skyroot);
+            mSkyRootNode->getOrCreateStateSet()->setAttributeAndModes(new osg::Program(),
+                osg::StateAttribute::OVERRIDE | osg::StateAttribute::PROTECTED | osg::StateAttribute::ON);
+        mSceneManager->setUpNormalsRTForStateSet(mSkyRootNode->getOrCreateStateSet(), false);
+        SceneUtil::ShadowManager::instance().disableShadowsForStateSet(*mSkyRootNode->getOrCreateStateSet());
+        parentNode->addChild(mSkyRootNode);
 
         mEarlyRenderBinRoot = new osg::Group;
         // render before the world is rendered
@@ -294,17 +280,19 @@ namespace MWRender
 
         if (enableSkyRTT)
         {
-            mSkyRTT = new SkyRTT(Settings::Manager::getVector2("sky rtt resolution", "Fog"), mEarlyRenderBinRoot);
-            skyroot->addChild(mSkyRTT);
-            mRootNode = new osg::Group;
-            skyroot->addChild(mRootNode);
+            mSkyRTT = new SkyRTT(Settings::fog().mSkyRttResolution, mEarlyRenderBinRoot);
+            mSkyRootNode->addChild(mSkyRTT);
         }
-        else
-            mRootNode = skyroot;
 
-        mRootNode->setNodeMask(Mask_Sky);
-        mRootNode->addChild(mEarlyRenderBinRoot);
-        mUnderwaterSwitch = new UnderwaterSwitchCallback(skyroot);
+        mSkyNode = new osg::Group;
+        mSkyNode->setNodeMask(Mask_Sky);
+        mSkyNode->addChild(mEarlyRenderBinRoot);
+        mSkyRootNode->addChild(mSkyNode);
+
+        mUnderwaterSwitch = new UnderwaterSwitchCallback(mSkyRootNode);
+
+        mPrecipitationOcclusion = Settings::shaders().mWeatherParticleOcclusion;
+        mPrecipitationOccluder = std::make_unique<PrecipitationOccluder>(mSkyRootNode, parentNode, rootNode, camera);
     }
 
     void SkyManager::create()
@@ -313,7 +301,7 @@ namespace MWRender
 
         bool forceShaders = mSceneManager->getForceShaders();
 
-        mAtmosphereDay = mSceneManager->getInstance(Settings::Manager::getString("skyatmosphere", "Models"), mEarlyRenderBinRoot);
+        mAtmosphereDay = mSceneManager->getInstance(Settings::models().mSkyatmosphere.get(), mEarlyRenderBinRoot);
         ModVertexAlphaVisitor modAtmosphere(ModVertexAlphaVisitor::Atmosphere);
         mAtmosphereDay->accept(modAtmosphere);
 
@@ -325,11 +313,12 @@ namespace MWRender
         mEarlyRenderBinRoot->addChild(mAtmosphereNightNode);
 
         osg::ref_ptr<osg::Node> atmosphereNight;
-        if (mSceneManager->getVFS()->exists(Settings::Manager::getString("skynight02", "Models")))
-            atmosphereNight = mSceneManager->getInstance(Settings::Manager::getString("skynight02", "Models"), mAtmosphereNightNode);
+        if (mSceneManager->getVFS()->exists(Settings::models().mSkynight02.get()))
+            atmosphereNight = mSceneManager->getInstance(Settings::models().mSkynight02.get(), mAtmosphereNightNode);
         else
-            atmosphereNight = mSceneManager->getInstance(Settings::Manager::getString("skynight01", "Models"), mAtmosphereNightNode);
-        atmosphereNight->getOrCreateStateSet()->setAttributeAndModes(createAlphaTrackingUnlitMaterial(), osg::StateAttribute::ON|osg::StateAttribute::OVERRIDE);
+            atmosphereNight = mSceneManager->getInstance(Settings::models().mSkynight01.get(), mAtmosphereNightNode);
+        atmosphereNight->getOrCreateStateSet()->setAttributeAndModes(
+            createAlphaTrackingUnlitMaterial(), osg::StateAttribute::ON | osg::StateAttribute::OVERRIDE);
 
         ModVertexAlphaVisitor modStars(ModVertexAlphaVisitor::Stars);
         atmosphereNight->accept(modStars);
@@ -338,21 +327,25 @@ namespace MWRender
 
         mSun = std::make_unique<Sun>(mEarlyRenderBinRoot, *mSceneManager);
         mSun->setSunglare(mSunglareEnabled);
-        mMasser = std::make_unique<Moon>(mEarlyRenderBinRoot, *mSceneManager, Fallback::Map::getFloat("Moons_Masser_Size")/125, Moon::Type_Masser);
-        mSecunda = std::make_unique<Moon>(mEarlyRenderBinRoot, *mSceneManager, Fallback::Map::getFloat("Moons_Secunda_Size")/125, Moon::Type_Secunda);
+        mMasser = std::make_unique<Moon>(
+            mEarlyRenderBinRoot, *mSceneManager, Fallback::Map::getFloat("Moons_Masser_Size") / 125, Moon::Type_Masser);
+        mSecunda = std::make_unique<Moon>(mEarlyRenderBinRoot, *mSceneManager,
+            Fallback::Map::getFloat("Moons_Secunda_Size") / 125, Moon::Type_Secunda);
 
         mCloudNode = new osg::Group;
         mEarlyRenderBinRoot->addChild(mCloudNode);
 
         mCloudMesh = new osg::PositionAttitudeTransform;
-        osg::ref_ptr<osg::Node> cloudMeshChild = mSceneManager->getInstance(Settings::Manager::getString("skyclouds", "Models"), mCloudMesh);
+        osg::ref_ptr<osg::Node> cloudMeshChild
+            = mSceneManager->getInstance(Settings::models().mSkyclouds.get(), mCloudMesh);
         mCloudUpdater = new CloudUpdater(forceShaders);
         mCloudUpdater->setOpacity(1.f);
         cloudMeshChild->addUpdateCallback(mCloudUpdater);
         mCloudMesh->addChild(cloudMeshChild);
 
         mNextCloudMesh = new osg::PositionAttitudeTransform;
-        osg::ref_ptr<osg::Node> nextCloudMeshChild = mSceneManager->getInstance(Settings::Manager::getString("skyclouds", "Models"), mNextCloudMesh);
+        osg::ref_ptr<osg::Node> nextCloudMeshChild
+            = mSceneManager->getInstance(Settings::models().mSkyclouds.get(), mNextCloudMesh);
         mNextCloudUpdater = new CloudUpdater(forceShaders);
         mNextCloudUpdater->setOpacity(0.f);
         nextCloudMeshChild->addUpdateCallback(mNextCloudUpdater);
@@ -369,12 +362,11 @@ namespace MWRender
         if (mSceneManager->getForceShaders())
         {
             Shader::ShaderManager::DefineMap defines = {};
-            Stereo::Manager::instance().shaderStereoDefines(defines);
-            auto vertex = mSceneManager->getShaderManager().getShader("sky_vertex.glsl", defines, osg::Shader::VERTEX);
-            auto fragment = mSceneManager->getShaderManager().getShader("sky_fragment.glsl", defines, osg::Shader::FRAGMENT);
-            auto program = mSceneManager->getShaderManager().getProgram(vertex, fragment);
+            Stereo::shaderStereoDefines(defines);
+            auto program = mSceneManager->getShaderManager().getProgram("sky", defines);
             mEarlyRenderBinRoot->getOrCreateStateSet()->addUniform(new osg::Uniform("pass", -1));
-            mEarlyRenderBinRoot->getOrCreateStateSet()->setAttributeAndModes(program, osg::StateAttribute::ON|osg::StateAttribute::OVERRIDE);
+            mEarlyRenderBinRoot->getOrCreateStateSet()->setAttributeAndModes(
+                program, osg::StateAttribute::ON | osg::StateAttribute::OVERRIDE);
         }
 
         osg::ref_ptr<osg::Depth> depth = new SceneUtil::AutoDepth;
@@ -388,11 +380,6 @@ namespace MWRender
         mCreated = true;
     }
 
-    void SkyManager::setCamera(osg::Camera *camera)
-    {
-        mCamera = camera;
-    }
-
     void SkyManager::createRain()
     {
         if (mRainNode)
@@ -401,15 +388,17 @@ namespace MWRender
         mRainNode = new osg::Group;
 
         mRainParticleSystem = new NifOsg::ParticleSystem;
-        osg::Vec3 rainRange = osg::Vec3(mRainDiameter, mRainDiameter, (mRainMinHeight+mRainMaxHeight)/2.f);
+        osg::Vec3 rainRange = osg::Vec3(mRainDiameter, mRainDiameter, (mRainMinHeight + mRainMaxHeight) / 2.f);
 
         mRainParticleSystem->setParticleAlignment(osgParticle::ParticleSystem::FIXED);
-        mRainParticleSystem->setAlignVectorX(osg::Vec3f(0.1,0,0));
-        mRainParticleSystem->setAlignVectorY(osg::Vec3f(0,0,1));
+        mRainParticleSystem->setAlignVectorX(osg::Vec3f(0.1, 0, 0));
+        mRainParticleSystem->setAlignVectorY(osg::Vec3f(0, 0, 1));
 
         osg::ref_ptr<osg::StateSet> stateset = mRainParticleSystem->getOrCreateStateSet();
 
-        osg::ref_ptr<osg::Texture2D> raindropTex = new osg::Texture2D(mSceneManager->getImageManager()->getImage("textures/tx_raindrop_01.dds"));
+        constexpr VFS::Path::NormalizedView raindropImage("textures/tx_raindrop_01.dds");
+        osg::ref_ptr<osg::Texture2D> raindropTex
+            = new osg::Texture2D(mSceneManager->getImageManager()->getImage(raindropImage));
         raindropTex->setWrap(osg::Texture::WRAP_S, osg::Texture::CLAMP_TO_EDGE);
         raindropTex->setWrap(osg::Texture::WRAP_T, osg::Texture::CLAMP_TO_EDGE);
 
@@ -420,8 +409,8 @@ namespace MWRender
         stateset->setMode(GL_BLEND, osg::StateAttribute::ON);
 
         osg::ref_ptr<osg::Material> mat = new osg::Material;
-        mat->setAmbient(osg::Material::FRONT_AND_BACK, osg::Vec4f(1,1,1,1));
-        mat->setDiffuse(osg::Material::FRONT_AND_BACK, osg::Vec4f(1,1,1,1));
+        mat->setAmbient(osg::Material::FRONT_AND_BACK, osg::Vec4f(1, 1, 1, 1));
+        mat->setDiffuse(osg::Material::FRONT_AND_BACK, osg::Vec4f(1, 1, 1, 1));
         mat->setColorMode(osg::Material::AMBIENT_AND_DIFFUSE);
         stateset->setAttributeAndModes(mat);
 
@@ -440,11 +429,12 @@ namespace MWRender
         emitter->setPlacer(placer);
         mPlacer = placer;
 
-        // FIXME: vanilla engine does not use a particle system to handle rain, it uses a NIF-file with 20 raindrops in it.
-        // It spawns the (maxRaindrops-getParticleSystem()->numParticles())*dt/rainEntranceSpeed batches every frame (near 1-2).
-        // Since the rain is a regular geometry, it produces water ripples, also in theory it can be removed if collides with something.
+        // FIXME: vanilla engine does not use a particle system to handle rain, it uses a NIF-file with 20 raindrops in
+        // it. It spawns the (maxRaindrops-getParticleSystem()->numParticles())*dt/rainEntranceSpeed batches every frame
+        // (near 1-2). Since the rain is a regular geometry, it produces water ripples, also in theory it can be removed
+        // if collides with something.
         osg::ref_ptr<RainCounter> counter = new RainCounter;
-        counter->setNumberOfParticlesPerSecondToCreate(mRainMaxRaindrops/mRainEntranceSpeed*20);
+        counter->setNumberOfParticlesPerSecondToCreate(mRainMaxRaindrops / mRainEntranceSpeed * 20);
         emitter->setCounter(counter);
         mCounter = counter;
 
@@ -456,7 +446,7 @@ namespace MWRender
         updater->addParticleSystem(mRainParticleSystem);
 
         osg::ref_ptr<osgParticle::ModularProgram> program = new osgParticle::ModularProgram;
-        program->addOperator(new WrapAroundOperator(mCamera,rainRange));
+        program->addOperator(new WrapAroundOperator(mCamera, rainRange));
         program->addOperator(new WeatherAlphaOperator(mPrecipitationAlpha, true));
         program->setParticleSystem(mRainParticleSystem);
         mRainNode->addChild(program);
@@ -470,9 +460,12 @@ namespace MWRender
         mRainNode->setNodeMask(Mask_WeatherParticles);
 
         mRainParticleSystem->setUserValue("simpleLighting", true);
+        mRainParticleSystem->setUserValue("particleOcclusion", true);
         mSceneManager->recreateShaders(mRainNode);
 
-        mRootNode->addChild(mRainNode);
+        mSkyNode->addChild(mRainNode);
+        if (mPrecipitationOcclusion)
+            mPrecipitationOccluder->enable();
     }
 
     void SkyManager::destroyRain()
@@ -480,32 +473,35 @@ namespace MWRender
         if (!mRainNode)
             return;
 
-        mRootNode->removeChild(mRainNode);
+        mSkyNode->removeChild(mRainNode);
         mRainNode = nullptr;
         mPlacer = nullptr;
         mCounter = nullptr;
         mRainParticleSystem = nullptr;
         mRainShooter = nullptr;
+        mPrecipitationOccluder->disable();
     }
 
     SkyManager::~SkyManager()
     {
-        if (mRootNode)
+        if (mSkyRootNode)
         {
-            mRootNode->getParent(0)->removeChild(mRootNode);
-            mRootNode = nullptr;
+            mSkyRootNode->getParent(0)->removeChild(mSkyRootNode);
+            mSkyRootNode = nullptr;
         }
     }
 
     int SkyManager::getMasserPhase() const
     {
-        if (!mCreated) return 0;
+        if (!mCreated)
+            return 0;
         return mMasser->getPhaseInt();
     }
 
     int SkyManager::getSecundaPhase() const
     {
-        if (!mCreated) return 0;
+        if (!mCreated)
+            return 0;
         return mSecunda->getPhaseInt();
     }
 
@@ -519,6 +515,20 @@ namespace MWRender
         return mRainNode != nullptr;
     }
 
+    bool SkyManager::getRainRipplesEnabled() const
+    {
+        if (!mEnabled || mIsStorm)
+            return false;
+
+        if (hasRain())
+            return mRainRipplesEnabled;
+
+        if (mParticleNode && mCurrentParticleEffect == Settings::models().mWeathersnow.get())
+            return mSnowRipplesEnabled;
+
+        return false;
+    }
+
     float SkyManager::getPrecipitationAlpha() const
     {
         if (mEnabled && !mIsStorm && (hasRain() || mParticleNode))
@@ -539,13 +549,22 @@ namespace MWRender
             osg::Quat quat;
             quat.makeRotate(MWWorld::Weather::defaultDirection(), mStormParticleDirection);
             // Morrowind deliberately rotates the blizzard mesh, so so should we.
-            if (mCurrentParticleEffect == Settings::Manager::getString("weatherblizzard", "Models"))
-                quat.makeRotate(osg::Vec3f(-1,0,0), mStormParticleDirection);
+            if (mCurrentParticleEffect == Settings::models().mWeatherblizzard.get())
+                quat.makeRotate(osg::Vec3f(-1, 0, 0), mStormParticleDirection);
             mParticleNode->setAttitude(quat);
         }
 
+        const float timeScale = MWBase::Environment::get().getWorld()->getTimeManager()->getGameTimeScale();
+
         // UV Scroll the clouds
-        mCloudAnimationTimer += duration * mCloudSpeed * 0.003;
+        float cloudDelta = duration * mCloudSpeed / 400.f;
+        if (mTimescaleClouds)
+            cloudDelta *= timeScale / 60.f;
+
+        mCloudAnimationTimer += cloudDelta;
+        if (mCloudAnimationTimer >= 4.f)
+            mCloudAnimationTimer -= 4.f;
+
         mNextCloudUpdater->setTextureCoord(mCloudAnimationTimer);
         mCloudUpdater->setTextureCoord(mCloudAnimationTimer);
 
@@ -561,9 +580,10 @@ namespace MWRender
         }
 
         // rotate the stars by 360 degrees every 4 days
-        mAtmosphereNightRoll += MWBase::Environment::get().getWorld()->getTimeScaleFactor()*duration*osg::DegreesToRadians(360.f) / (3600*96.f);
+        mAtmosphereNightRoll += timeScale * duration * osg::DegreesToRadians(360.f) / (3600 * 96.f);
         if (mAtmosphereNightNode->getNodeMask() != 0)
-            mAtmosphereNightNode->setAttitude(osg::Quat(mAtmosphereNightRoll, osg::Vec3f(0,0,1)));
+            mAtmosphereNightNode->setAttitude(osg::Quat(mAtmosphereNightRoll, osg::Vec3f(0, 0, 1)));
+        mPrecipitationOccluder->update();
     }
 
     void SkyManager::setEnabled(bool enabled)
@@ -574,7 +594,7 @@ namespace MWRender
         const osg::Node::NodeMask mask = enabled ? Mask_Sky : 0u;
 
         mEarlyRenderBinRoot->setNodeMask(mask);
-        mRootNode->setNodeMask(mask);
+        mSkyNode->setNodeMask(mask);
 
         if (!enabled && mParticleNode && mParticleEffect)
         {
@@ -585,27 +605,29 @@ namespace MWRender
         mEnabled = enabled;
     }
 
-    void SkyManager::setMoonColour (bool red)
+    void SkyManager::setMoonColour(bool red)
     {
-        if (!mCreated) return;
-        mSecunda->setColor(red ? mMoonScriptColor : osg::Vec4f(1,1,1,1));
+        if (!mCreated)
+            return;
+        mSecunda->setColor(red ? mMoonScriptColor : osg::Vec4f(1, 1, 1, 1));
     }
 
     void SkyManager::updateRainParameters()
     {
         if (mRainShooter)
         {
-            float angle = -std::atan(mWindSpeed/50.f);
-            mRainShooter->setVelocity(osg::Vec3f(0, mRainSpeed*std::sin(angle), -mRainSpeed/std::cos(angle)));
+            float angle = -std::atan(mWindSpeed / 50.f);
+            mRainShooter->setVelocity(osg::Vec3f(0, mRainSpeed * std::sin(angle), -mRainSpeed / std::cos(angle)));
             mRainShooter->setAngle(angle);
 
-            osg::Vec3 rainRange = osg::Vec3(mRainDiameter, mRainDiameter, (mRainMinHeight+mRainMaxHeight)/2.f);
+            osg::Vec3 rainRange = osg::Vec3(mRainDiameter, mRainDiameter, (mRainMinHeight + mRainMaxHeight) / 2.f);
 
             mPlacer->setXRange(-rainRange.x() / 2, rainRange.x() / 2);
             mPlacer->setYRange(-rainRange.y() / 2, rainRange.y() / 2);
             mPlacer->setZRange(-rainRange.z() / 2, rainRange.z() / 2);
 
-            mCounter->setNumberOfParticlesPerSecondToCreate(mRainMaxRaindrops/mRainEntranceSpeed*20);
+            mCounter->setNumberOfParticlesPerSecondToCreate(mRainMaxRaindrops / mRainEntranceSpeed * 20);
+            mPrecipitationOccluder->updateRange(rainRange);
         }
     }
 
@@ -620,7 +642,8 @@ namespace MWRender
 
     void SkyManager::setWeather(const WeatherResult& weather)
     {
-        if (!mCreated) return;
+        if (!mCreated)
+            return;
 
         mRainEntranceSpeed = weather.mRainEntranceSpeed;
         mRainMaxRaindrops = weather.mRainMaxRaindrops;
@@ -667,9 +690,13 @@ namespace MWRender
             {
                 if (mParticleNode)
                 {
-                    mRootNode->removeChild(mParticleNode);
+                    mSkyNode->removeChild(mParticleNode);
                     mParticleNode = nullptr;
                 }
+                if (mRainEffect.empty())
+                {
+                    mPrecipitationOccluder->disable();
+                }
             }
             else
             {
@@ -678,7 +705,7 @@ namespace MWRender
                     mParticleNode = new osg::PositionAttitudeTransform;
                     mParticleNode->addCullCallback(mUnderwaterSwitch);
                     mParticleNode->setNodeMask(Mask_WeatherParticles);
-                    mRootNode->addChild(mParticleNode);
+                    mSkyNode->addChild(mParticleNode);
                 }
 
                 mParticleEffect = mSceneManager->getInstance(mCurrentParticleEffect, mParticleNode);
@@ -689,30 +716,45 @@ namespace MWRender
                 SetupVisitor alphaFaderSetupVisitor(mPrecipitationAlpha);
                 mParticleEffect->accept(alphaFaderSetupVisitor);
 
-                SceneUtil::FindByClassVisitor findPSVisitor(std::string("ParticleSystem"));
+                SceneUtil::FindByClassVisitor findPSVisitor("ParticleSystem");
                 mParticleEffect->accept(findPSVisitor);
 
+                const osg::Vec3 defaultWrapRange = osg::Vec3(1024, 1024, 800);
+                const bool occlusionEnabledForEffect
+                    = !mRainEffect.empty() || mCurrentParticleEffect == Settings::models().mWeathersnow.get();
+
                 for (unsigned int i = 0; i < findPSVisitor.mFoundNodes.size(); ++i)
                 {
-                    osgParticle::ParticleSystem *ps = static_cast<osgParticle::ParticleSystem *>(findPSVisitor.mFoundNodes[i]);
+                    osgParticle::ParticleSystem* ps
+                        = static_cast<osgParticle::ParticleSystem*>(findPSVisitor.mFoundNodes[i]);
 
                     osg::ref_ptr<osgParticle::ModularProgram> program = new osgParticle::ModularProgram;
                     if (!mIsStorm)
-                        program->addOperator(new WrapAroundOperator(mCamera,osg::Vec3(1024,1024,800)));
+                        program->addOperator(new WrapAroundOperator(mCamera, defaultWrapRange));
                     program->addOperator(new WeatherAlphaOperator(mPrecipitationAlpha, false));
                     program->setParticleSystem(ps);
                     mParticleNode->addChild(program);
 
                     for (int particleIndex = 0; particleIndex < ps->numParticles(); ++particleIndex)
                     {
-                        ps->getParticle(particleIndex)->setAlphaRange(osgParticle::rangef(mPrecipitationAlpha, mPrecipitationAlpha));
+                        ps->getParticle(particleIndex)
+                            ->setAlphaRange(osgParticle::rangef(mPrecipitationAlpha, mPrecipitationAlpha));
                         ps->getParticle(particleIndex)->update(0, true);
                     }
 
                     ps->setUserValue("simpleLighting", true);
+
+                    if (occlusionEnabledForEffect)
+                        ps->setUserValue("particleOcclusion", true);
                 }
 
                 mSceneManager->recreateShaders(mParticleNode);
+
+                if (mPrecipitationOcclusion && occlusionEnabledForEffect)
+                {
+                    mPrecipitationOccluder->enable();
+                    mPrecipitationOccluder->updateRange(defaultWrapRange);
+                }
             }
         }
 
@@ -720,13 +762,15 @@ namespace MWRender
         {
             mClouds = weather.mCloudTexture;
 
-            std::string texture = Misc::ResourceHelpers::correctTexturePath(mClouds, mSceneManager->getVFS());
+            const VFS::Path::Normalized texture
+                = Misc::ResourceHelpers::correctTexturePath(mClouds, mSceneManager->getVFS());
 
-            osg::ref_ptr<osg::Texture2D> cloudTex = new osg::Texture2D(mSceneManager->getImageManager()->getImage(texture));
+            osg::ref_ptr<osg::Texture2D> cloudTex
+                = new osg::Texture2D(mSceneManager->getImageManager()->getImage(texture));
             cloudTex->setWrap(osg::Texture::WRAP_S, osg::Texture::REPEAT);
             cloudTex->setWrap(osg::Texture::WRAP_T, osg::Texture::REPEAT);
 
-            mCloudUpdater->setTexture(cloudTex);
+            mCloudUpdater->setTexture(std::move(cloudTex));
         }
 
         if (mStormDirection != weather.mStormDirection)
@@ -741,13 +785,15 @@ namespace MWRender
 
             if (!mNextClouds.empty())
             {
-                std::string texture = Misc::ResourceHelpers::correctTexturePath(mNextClouds, mSceneManager->getVFS());
+                const VFS::Path::Normalized texture
+                    = Misc::ResourceHelpers::correctTexturePath(mNextClouds, mSceneManager->getVFS());
 
-                osg::ref_ptr<osg::Texture2D> cloudTex = new osg::Texture2D(mSceneManager->getImageManager()->getImage(texture));
+                osg::ref_ptr<osg::Texture2D> cloudTex
+                    = new osg::Texture2D(mSceneManager->getImageManager()->getImage(texture));
                 cloudTex->setWrap(osg::Texture::WRAP_S, osg::Texture::REPEAT);
                 cloudTex->setWrap(osg::Texture::WRAP_T, osg::Texture::REPEAT);
 
-                mNextCloudUpdater->setTexture(cloudTex);
+                mNextCloudUpdater->setTexture(std::move(cloudTex));
                 mNextStormDirection = weather.mStormDirection;
             }
         }
@@ -763,7 +809,7 @@ namespace MWRender
 
         if (mCloudColour != weather.mFogColor)
         {
-            osg::Vec4f clr (weather.mFogColor);
+            osg::Vec4f clr(weather.mFogColor);
             clr += osg::Vec4f(0.13f, 0.13f, 0.13f, 0.f);
 
             mCloudUpdater->setEmissionColor(clr);
@@ -809,7 +855,8 @@ namespace MWRender
 
     float SkyManager::getBaseWindSpeed() const
     {
-        if (!mCreated) return 0.f;
+        if (!mCreated)
+            return 0.f;
 
         return mBaseWindSpeed;
     }
@@ -824,50 +871,49 @@ namespace MWRender
 
     void SkyManager::sunEnable()
     {
-        if (!mCreated) return;
+        if (!mCreated)
+            return;
 
         mSun->setVisible(true);
     }
 
     void SkyManager::sunDisable()
     {
-        if (!mCreated) return;
+        if (!mCreated)
+            return;
 
         mSun->setVisible(false);
     }
 
-    void SkyManager::setStormParticleDirection(const osg::Vec3f &direction)
+    void SkyManager::setStormParticleDirection(const osg::Vec3f& direction)
     {
         mStormParticleDirection = direction;
     }
 
     void SkyManager::setSunDirection(const osg::Vec3f& direction)
     {
-        if (!mCreated) return;
+        if (!mCreated)
+            return;
 
         mSun->setDirection(direction);
     }
 
     void SkyManager::setMasserState(const MoonState& state)
     {
-        if(!mCreated) return;
+        if (!mCreated)
+            return;
 
         mMasser->setState(state);
     }
 
     void SkyManager::setSecundaState(const MoonState& state)
     {
-        if(!mCreated) return;
+        if (!mCreated)
+            return;
 
         mSecunda->setState(state);
     }
 
-    void SkyManager::setDate(int day, int month)
-    {
-        mDay = day;
-        mMonth = month;
-    }
-
     void SkyManager::setGlareTimeOfDayFade(float val)
     {
         mSun->setGlareTimeOfDayFade(val);
@@ -878,18 +924,19 @@ namespace MWRender
         mUnderwaterSwitch->setWaterLevel(height);
     }
 
-    void SkyManager::listAssetsToPreload(std::vector<std::string>& models, std::vector<std::string>& textures)
+    void SkyManager::listAssetsToPreload(
+        std::vector<VFS::Path::Normalized>& models, std::vector<VFS::Path::Normalized>& textures)
     {
-        models.emplace_back(Settings::Manager::getString("skyatmosphere", "Models"));
-        if (mSceneManager->getVFS()->exists(Settings::Manager::getString("skynight02", "Models")))
-            models.emplace_back(Settings::Manager::getString("skynight02", "Models"));
-        models.emplace_back(Settings::Manager::getString("skynight01", "Models"));
-        models.emplace_back(Settings::Manager::getString("skyclouds", "Models"));
+        models.push_back(Settings::models().mSkyatmosphere);
+        if (mSceneManager->getVFS()->exists(Settings::models().mSkynight02.get()))
+            models.push_back(Settings::models().mSkynight02);
+        models.push_back(Settings::models().mSkynight01);
+        models.push_back(Settings::models().mSkyclouds);
 
-        models.emplace_back(Settings::Manager::getString("weatherashcloud", "Models"));
-        models.emplace_back(Settings::Manager::getString("weatherblightcloud", "Models"));
-        models.emplace_back(Settings::Manager::getString("weathersnow", "Models"));
-        models.emplace_back(Settings::Manager::getString("weatherblizzard", "Models"));
+        models.push_back(Settings::models().mWeatherashcloud);
+        models.push_back(Settings::models().mWeatherblightcloud);
+        models.push_back(Settings::models().mWeathersnow);
+        models.push_back(Settings::models().mWeatherblizzard);
 
         textures.emplace_back("textures/tx_mooncircle_full_s.dds");
         textures.emplace_back("textures/tx_mooncircle_full_m.dds");
