@@ -1,13 +1,16 @@
 #ifndef OPENMW_MWRENDER_SKY_H
 #define OPENMW_MWRENDER_SKY_H
 
-#include <string>
 #include <memory>
+#include <string>
 #include <vector>
 
-#include <osg/ref_ptr>
 #include <osg/Vec4f>
+#include <osg/ref_ptr>
 
+#include <components/vfs/pathutil.hpp>
+
+#include "precipitationocclusion.hpp"
 #include "skyutil.hpp"
 
 namespace osg
@@ -37,24 +40,20 @@ namespace SceneUtil
 
 namespace MWRender
 {
-    ///@brief The SkyManager handles rendering of the sky domes, celestial bodies as well as other objects that need to be rendered
+    ///@brief The SkyManager handles rendering of the sky domes, celestial bodies as well as other objects that need to
+    /// be rendered
     /// relative to the camera (e.g. weather particle effects)
     class SkyManager
     {
     public:
-        SkyManager(osg::Group* parentNode, Resource::SceneManager* sceneManager, bool enableSkyRTT);
+        SkyManager(osg::Group* parentNode, osg::Group* rootNode, osg::Camera* camera,
+            Resource::SceneManager* sceneManager, bool enableSkyRTT);
         ~SkyManager();
 
         void update(float duration);
 
         void setEnabled(bool enabled);
 
-        void setHour (double hour);
-        ///< will be called even when sky is disabled.
-
-        void setDate (int day, int month);
-        ///< will be called even when sky is disabled.
-
         int getMasserPhase() const;
         ///< 0 new moon, 1 waxing or waning cresecent, 2 waxing or waning half,
         /// 3 waxing or waning gibbous, 4 full moon
@@ -63,7 +62,7 @@ namespace MWRender
         ///< 0 new moon, 1 waxing or waning cresecent, 2 waxing or waning half,
         /// 3 waxing or waning gibbous, 4 full moon
 
-        void setMoonColour (bool red);
+        void setMoonColour(bool red);
         ///< change Secunda colour to red
 
         void setWeather(const WeatherResult& weather);
@@ -76,9 +75,9 @@ namespace MWRender
 
         bool hasRain() const;
 
-        float getPrecipitationAlpha() const;
+        bool getRainRipplesEnabled() const;
 
-        void setRainSpeed(float speed);
+        float getPrecipitationAlpha() const;
 
         void setStormParticleDirection(const osg::Vec3f& direction);
 
@@ -95,9 +94,8 @@ namespace MWRender
         /// Set height of water plane (used to remove underwater weather particles)
         void setWaterHeight(float height);
 
-        void listAssetsToPreload(std::vector<std::string>& models, std::vector<std::string>& textures);
-
-        void setCamera(osg::Camera *camera);
+        void listAssetsToPreload(
+            std::vector<VFS::Path::Normalized>& models, std::vector<VFS::Path::Normalized>& textures);
 
         float getBaseWindSpeed() const;
 
@@ -105,6 +103,8 @@ namespace MWRender
 
         SceneUtil::RTTNode* getSkyRTT() { return mSkyRTT.get(); }
 
+        osg::Vec4f getSkyColor() const { return mSkyColour; }
+
     private:
         void create();
         ///< no need to call this, automatically done on first enable()
@@ -116,9 +116,10 @@ namespace MWRender
 
         Resource::SceneManager* mSceneManager;
 
-        osg::Camera *mCamera;
+        osg::Camera* mCamera;
 
-        osg::ref_ptr<osg::Group> mRootNode;
+        osg::ref_ptr<CameraRelativeTransform> mSkyRootNode;
+        osg::ref_ptr<osg::Group> mSkyNode;
         osg::ref_ptr<osg::Group> mEarlyRenderBinRoot;
 
         osg::ref_ptr<osg::PositionAttitudeTransform> mParticleNode;
@@ -150,17 +151,16 @@ namespace MWRender
         osg::ref_ptr<RainCounter> mCounter;
         osg::ref_ptr<RainShooter> mRainShooter;
 
+        bool mPrecipitationOcclusion = false;
+        std::unique_ptr<PrecipitationOccluder> mPrecipitationOccluder;
+
         bool mCreated;
 
         bool mIsStorm;
 
-        int mDay;
-        int mMonth;
-
+        bool mTimescaleClouds;
         float mCloudAnimationTimer;
 
-        float mRainTimer;
-
         // particle system rotation is independent of cloud rotation internally
         osg::Vec3f mStormParticleDirection;
         osg::Vec3f mStormDirection;
@@ -176,11 +176,8 @@ namespace MWRender
         osg::Vec4f mSkyColour;
         osg::Vec4f mFogColour;
 
-        std::string mCurrentParticleEffect;
+        VFS::Path::Normalized mCurrentParticleEffect;
 
-        float mRemainingTransitionTime;
-
-        bool mRainEnabled;
         std::string mRainEffect;
         float mRainSpeed;
         float mRainDiameter;
@@ -188,11 +185,12 @@ namespace MWRender
         float mRainMaxHeight;
         float mRainEntranceSpeed;
         int mRainMaxRaindrops;
+        bool mRainRipplesEnabled;
+        bool mSnowRipplesEnabled;
         float mWindSpeed;
         float mBaseWindSpeed;
 
         bool mEnabled;
-        bool mSunEnabled;
         bool mSunglareEnabled;
 
         float mPrecipitationAlpha;
