@@ -4,11 +4,12 @@
 #include <memory>
 #include <vector>
 
-#include <osg/ref_ptr>
-#include <osg/Vec3f>
 #include <osg/Vec3d>
+#include <osg/Vec3f>
+#include <osg/ref_ptr>
 
 #include <components/settings/settings.hpp>
+#include <components/vfs/pathutil.hpp>
 
 namespace osg
 {
@@ -46,12 +47,13 @@ namespace MWRender
     class Refraction;
     class Reflection;
     class RippleSimulation;
-    class RainIntensityUpdater;
+    class RainSettingsUpdater;
+    class Ripples;
 
     /// Water rendering
     class Water
     {
-        osg::ref_ptr<RainIntensityUpdater> mRainIntensityUpdater;
+        osg::ref_ptr<RainSettingsUpdater> mRainSettingsUpdater;
 
         osg::ref_ptr<osg::Group> mParent;
         osg::ref_ptr<osg::Group> mSceneRoot;
@@ -64,8 +66,7 @@ namespace MWRender
 
         osg::ref_ptr<Refraction> mRefraction;
         osg::ref_ptr<Reflection> mReflection;
-
-        const std::string mResourcePath;
+        osg::ref_ptr<Ripples> mRipples;
 
         bool mEnabled;
         bool mToggled;
@@ -81,21 +82,18 @@ namespace MWRender
 
         void createSimpleWaterStateSet(osg::Node* node, float alpha);
 
-        /// @param reflection the reflection camera (required)
-        /// @param refraction the refraction camera (optional)
-        void createShaderWaterStateSet(osg::Node* node, Reflection* reflection, Refraction* refraction);
+        void createShaderWaterStateSet(osg::Node* node);
 
         void updateWaterMaterial();
 
     public:
-        Water(osg::Group* parent, osg::Group* sceneRoot,
-              Resource::ResourceSystem* resourceSystem, osgUtil::IncrementalCompileOperation* ico,
-              const std::string& resourcePath);
+        Water(osg::Group* parent, osg::Group* sceneRoot, Resource::ResourceSystem* resourceSystem,
+            osgUtil::IncrementalCompileOperation* ico);
         ~Water();
 
         void setCullCallback(osg::Callback* callback);
 
-        void listAssetsToPreload(std::vector<std::string>& textures);
+        void listAssetsToPreload(std::vector<VFS::Path::Normalized>& textures);
 
         void setEnabled(bool enabled);
 
@@ -104,9 +102,9 @@ namespace MWRender
         bool isUnderwater(const osg::Vec3f& pos) const;
 
         /// adds an emitter, position will be tracked automatically using its scene node
-        void addEmitter (const MWWorld::Ptr& ptr, float scale = 1.f, float force = 1.f);
-        void removeEmitter (const MWWorld::Ptr& ptr);
-        void updateEmitterPtr (const MWWorld::Ptr& old, const MWWorld::Ptr& ptr);
+        void addEmitter(const MWWorld::Ptr& ptr, float scale = 1.f, float force = 1.f);
+        void removeEmitter(const MWWorld::Ptr& ptr);
+        void updateEmitterPtr(const MWWorld::Ptr& old, const MWWorld::Ptr& ptr);
         void emitRipple(const osg::Vec3f& pos);
 
         void removeCell(const MWWorld::CellStore* store); ///< remove all emitters in this cell
@@ -116,11 +114,9 @@ namespace MWRender
         void changeCell(const MWWorld::CellStore* store);
         void setHeight(const float height);
         void setRainIntensity(const float rainIntensity);
+        void setRainRipplesEnabled(bool enableRipples);
 
-        void update(float dt);
-
-        osg::Node* getReflectionNode();
-        osg::Node* getRefractionNode();
+        void update(float dt, bool paused);
 
         osg::Vec3d getPosition() const;
 
