@@ -2,20 +2,19 @@
 
 #include <components/compiler/opcodes.hpp>
 
+#include <components/interpreter/context.hpp>
 #include <components/interpreter/interpreter.hpp>
-#include <components/interpreter/runtime.hpp>
 #include <components/interpreter/opcodes.hpp>
+#include <components/interpreter/runtime.hpp>
 
 #include "../mwworld/esmstore.hpp"
 
 #include "../mwbase/environment.hpp"
-#include "../mwbase/windowmanager.hpp"
-#include "../mwbase/world.hpp"
 #include "../mwbase/mechanicsmanager.hpp"
+#include "../mwbase/windowmanager.hpp"
 
 #include "../mwmechanics/actorutil.hpp"
 
-#include "interpretercontext.hpp"
 #include "ref.hpp"
 
 namespace MWScript
@@ -24,113 +23,110 @@ namespace MWScript
     {
         class OpEnableWindow : public Interpreter::Opcode0
         {
-                MWGui::GuiWindow mWindow;
+            MWGui::GuiWindow mWindow;
 
-            public:
+        public:
+            OpEnableWindow(MWGui::GuiWindow window)
+                : mWindow(window)
+            {
+            }
 
-                OpEnableWindow (MWGui::GuiWindow window) : mWindow (window) {}
-
-                void execute (Interpreter::Runtime& runtime) override
-                {
-                    MWBase::Environment::get().getWindowManager()->allow (mWindow);
-                }
+            void execute(Interpreter::Runtime& runtime) override
+            {
+                MWBase::Environment::get().getWindowManager()->allow(mWindow);
+            }
         };
 
         class OpEnableRest : public Interpreter::Opcode0
         {
-            public:
-
-                void execute (Interpreter::Runtime& runtime) override
-                {
-                    MWBase::Environment::get().getWindowManager()->enableRest();
-                }
+        public:
+            void execute(Interpreter::Runtime& runtime) override
+            {
+                MWBase::Environment::get().getWindowManager()->enableRest();
+            }
         };
 
         template <class R>
         class OpShowRestMenu : public Interpreter::Opcode0
         {
         public:
-            void execute (Interpreter::Runtime& runtime) override
+            void execute(Interpreter::Runtime& runtime) override
             {
                 MWWorld::Ptr bed = R()(runtime, false);
 
-                if (bed.isEmpty() || !MWBase::Environment::get().getMechanicsManager()->sleepInBed(MWMechanics::getPlayer(),
-                                                                             bed))
+                if (bed.isEmpty()
+                    || !MWBase::Environment::get().getMechanicsManager()->sleepInBed(MWMechanics::getPlayer(), bed))
                     MWBase::Environment::get().getWindowManager()->pushGuiMode(MWGui::GM_Rest, bed);
             }
         };
 
         class OpShowDialogue : public Interpreter::Opcode0
         {
-                MWGui::GuiMode mDialogue;
+            MWGui::GuiMode mDialogue;
 
-            public:
+        public:
+            OpShowDialogue(MWGui::GuiMode dialogue)
+                : mDialogue(dialogue)
+            {
+            }
 
-                OpShowDialogue (MWGui::GuiMode dialogue)
-                : mDialogue (dialogue)
-                {}
-
-                void execute (Interpreter::Runtime& runtime) override
-                {
-                    MWBase::Environment::get().getWindowManager()->pushGuiMode(mDialogue);
-                }
+            void execute(Interpreter::Runtime& runtime) override
+            {
+                MWBase::Environment::get().getWindowManager()->pushGuiMode(mDialogue);
+            }
         };
 
         class OpGetButtonPressed : public Interpreter::Opcode0
         {
-            public:
-
-                void execute (Interpreter::Runtime& runtime) override
-                {
-                    runtime.push (MWBase::Environment::get().getWindowManager()->readPressedButton());
-                }
+        public:
+            void execute(Interpreter::Runtime& runtime) override
+            {
+                runtime.push(MWBase::Environment::get().getWindowManager()->readPressedButton());
+            }
         };
 
         class OpToggleFogOfWar : public Interpreter::Opcode0
         {
-            public:
-
-                void execute (Interpreter::Runtime& runtime) override
-                {
-                    runtime.getContext().report(MWBase::Environment::get().getWindowManager()->toggleFogOfWar() ? "Fog of war -> On"
-                                                                                                                : "Fog of war -> Off");
-                }
+        public:
+            void execute(Interpreter::Runtime& runtime) override
+            {
+                runtime.getContext().report(MWBase::Environment::get().getWindowManager()->toggleFogOfWar()
+                        ? "Fog of war -> On"
+                        : "Fog of war -> Off");
+            }
         };
 
         class OpToggleFullHelp : public Interpreter::Opcode0
         {
-            public:
-
-                void execute (Interpreter::Runtime& runtime) override
-                {
-                    runtime.getContext().report(MWBase::Environment::get().getWindowManager()->toggleFullHelp() ? "Full help -> On"
-                                                                                                                : "Full help -> Off");
-                }
+        public:
+            void execute(Interpreter::Runtime& runtime) override
+            {
+                runtime.getContext().report(MWBase::Environment::get().getWindowManager()->toggleFullHelp()
+                        ? "Full help -> On"
+                        : "Full help -> Off");
+            }
         };
 
         class OpShowMap : public Interpreter::Opcode0
         {
         public:
-
-            void execute (Interpreter::Runtime& runtime) override
+            void execute(Interpreter::Runtime& runtime) override
             {
-                std::string cell = ::Misc::StringUtils::lowerCase(runtime.getStringLiteral(runtime[0].mInteger));
+                std::string_view cell = runtime.getStringLiteral(runtime[0].mInteger);
                 runtime.pop();
 
-                // "Will match complete or partial cells, so ShowMap, "Vivec" will show cells Vivec and Vivec, Fred's House as well."
-                // http://www.uesp.net/wiki/Tes3Mod:ShowMap
+                // "Will match complete or partial cells, so ShowMap, "Vivec" will show cells Vivec and Vivec, Fred's
+                // House as well." http://www.uesp.net/wiki/Tes3Mod:ShowMap
 
-                const MWWorld::Store<ESM::Cell> &cells =
-                    MWBase::Environment::get().getWorld()->getStore().get<ESM::Cell>();
+                const MWWorld::Store<ESM::Cell>& cells = MWBase::Environment::get().getESMStore()->get<ESM::Cell>();
 
-                MWBase::WindowManager *winMgr = MWBase::Environment::get().getWindowManager();
+                MWBase::WindowManager* winMgr = MWBase::Environment::get().getWindowManager();
 
                 for (auto it = cells.extBegin(); it != cells.extEnd(); ++it)
                 {
-                    std::string name = it->mName;
-                    ::Misc::StringUtils::lowerCaseInPlace(name);
-                    if (name.length() >= cell.length() && name.substr(0, cell.length()) == cell)
-                        winMgr->addVisitedLocation(it->mName, it->getGridX(), it->getGridY());
+                    const auto& cellName = it->mName;
+                    if (Misc::StringUtils::ciStartsWith(cellName, cell))
+                        winMgr->addVisitedLocation(cellName, it->getGridX(), it->getGridY());
                 }
             }
         };
@@ -138,22 +134,16 @@ namespace MWScript
         class OpFillMap : public Interpreter::Opcode0
         {
         public:
-
-            void execute (Interpreter::Runtime& runtime) override
+            void execute(Interpreter::Runtime& runtime) override
             {
-                const MWWorld::Store<ESM::Cell> &cells =
-                    MWBase::Environment::get().getWorld ()->getStore().get<ESM::Cell>();
+                const MWWorld::Store<ESM::Cell>& cells = MWBase::Environment::get().getESMStore()->get<ESM::Cell>();
 
-                MWWorld::Store<ESM::Cell>::iterator it = cells.extBegin();
-                for (; it != cells.extEnd(); ++it)
+                for (auto it = cells.extBegin(); it != cells.extEnd(); ++it)
                 {
-                    std::string name = it->mName;
-                    if (name != "")
-                        MWBase::Environment::get().getWindowManager()->addVisitedLocation (
-                            name,
-                            it->getGridX(),
-                            it->getGridY()
-                        );
+                    const std::string& name = it->mName;
+                    if (!name.empty())
+                        MWBase::Environment::get().getWindowManager()->addVisitedLocation(
+                            name, it->getGridX(), it->getGridY());
                 }
             }
         };
@@ -161,22 +151,20 @@ namespace MWScript
         class OpMenuTest : public Interpreter::Opcode1
         {
         public:
-
-            void execute (Interpreter::Runtime& runtime, unsigned int arg0) override
+            void execute(Interpreter::Runtime& runtime, unsigned int arg0) override
             {
-                int arg=0;
-                if(arg0>0)
+                int arg = 0;
+                if (arg0 > 0)
                 {
                     arg = runtime[0].mInteger;
                     runtime.pop();
                 }
 
-
                 if (arg == 0)
                 {
                     MWGui::GuiMode modes[] = { MWGui::GM_Inventory, MWGui::GM_Container };
 
-                    for (int i=0; i<2; ++i)
+                    for (int i = 0; i < 2; ++i)
                     {
                         if (MWBase::Environment::get().getWindowManager()->containsMode(modes[i]))
                             MWBase::Environment::get().getWindowManager()->removeGuiMode(modes[i]);
@@ -202,20 +190,22 @@ namespace MWScript
         class OpToggleMenus : public Interpreter::Opcode0
         {
         public:
-            void execute(Interpreter::Runtime &runtime) override
+            void execute(Interpreter::Runtime& runtime) override
             {
-                bool state = MWBase::Environment::get().getWindowManager()->toggleHud();
+                bool state = MWBase::Environment::get().getWindowManager()->setHudVisibility(
+                    !MWBase::Environment::get().getWindowManager()->isHudVisible());
                 runtime.getContext().report(state ? "GUI -> On" : "GUI -> Off");
 
                 if (!state)
                 {
-                    while (MWBase::Environment::get().getWindowManager()->getMode() != MWGui::GM_None) // don't use isGuiMode, or we get an infinite loop for modal message boxes!
+                    while (MWBase::Environment::get().getWindowManager()->getMode()
+                        != MWGui::GM_None) // don't use isGuiMode, or we get an infinite loop for modal message boxes!
                         MWBase::Environment::get().getWindowManager()->popGuiMode();
                 }
             }
         };
 
-        void installOpcodes (Interpreter::Interpreter& interpreter)
+        void installOpcodes(Interpreter::Interpreter& interpreter)
         {
             interpreter.installSegment5<OpShowDialogue>(Compiler::Gui::opcodeEnableBirthMenu, MWGui::GM_Birth);
             interpreter.installSegment5<OpShowDialogue>(Compiler::Gui::opcodeEnableClassMenu, MWGui::GM_Class);
