@@ -7,10 +7,9 @@
 
 #include "interpretercontext.hpp"
 
-MWWorld::Ptr MWScript::ExplicitRef::operator() (Interpreter::Runtime& runtime, bool required,
-    bool activeOnly) const
+MWWorld::Ptr MWScript::ExplicitRef::operator()(Interpreter::Runtime& runtime, bool required, bool activeOnly) const
 {
-    std::string_view id = runtime.getStringLiteral(runtime[0].mInteger);
+    ESM::RefId id = ESM::RefId::stringRefId(runtime.getStringLiteral(runtime[0].mInteger));
     runtime.pop();
 
     if (required)
@@ -19,11 +18,9 @@ MWWorld::Ptr MWScript::ExplicitRef::operator() (Interpreter::Runtime& runtime, b
         return MWBase::Environment::get().getWorld()->searchPtr(id, activeOnly);
 }
 
-MWWorld::Ptr MWScript::ImplicitRef::operator() (Interpreter::Runtime& runtime, bool required,
-    bool activeOnly) const
+MWWorld::Ptr MWScript::ImplicitRef::operator()(Interpreter::Runtime& runtime, bool required, bool activeOnly) const
 {
-    MWScript::InterpreterContext& context
-    = static_cast<MWScript::InterpreterContext&> (runtime.getContext());
+    MWScript::InterpreterContext& context = static_cast<MWScript::InterpreterContext&>(runtime.getContext());
 
     return context.getReference(required);
 }
