@@ -14,16 +14,14 @@ namespace MWScript
     {
         static constexpr bool implicit = false;
 
-        MWWorld::Ptr operator() (Interpreter::Runtime& runtime, bool required = true,
-            bool activeOnly = false) const;
+        MWWorld::Ptr operator()(Interpreter::Runtime& runtime, bool required = true, bool activeOnly = false) const;
     };
 
     struct ImplicitRef
     {
         static constexpr bool implicit = true;
 
-        MWWorld::Ptr operator() (Interpreter::Runtime& runtime, bool required = true,
-            bool activeOnly = false) const;
+        MWWorld::Ptr operator()(Interpreter::Runtime& runtime, bool required = true, bool activeOnly = false) const;
     };
 }
 
