@@ -1,12 +1,11 @@
 #include "userextensions.hpp"
 
-#include <components/compiler/extensions.hpp>
 #include <components/compiler/opcodes.hpp>
 
-#include <components/interpreter/interpreter.hpp>
-#include <components/interpreter/runtime.hpp>
-#include <components/interpreter/opcodes.hpp>
 #include <components/interpreter/context.hpp>
+#include <components/interpreter/interpreter.hpp>
+#include <components/interpreter/opcodes.hpp>
+#include <components/interpreter/runtime.hpp>
 
 #include "ref.hpp"
 
@@ -19,52 +18,41 @@ namespace MWScript
     {
         class OpUser1 : public Interpreter::Opcode0
         {
-            public:
-
-                void execute (Interpreter::Runtime& runtime) override
-                {
-                    runtime.getContext().report ("user1: not in use");
-                }
+        public:
+            void execute(Interpreter::Runtime& runtime) override { runtime.getContext().report("user1: not in use"); }
         };
 
         class OpUser2 : public Interpreter::Opcode0
         {
-            public:
-
-                void execute (Interpreter::Runtime& runtime) override
-                {
-                    runtime.getContext().report ("user2: not in use");
-                }
+        public:
+            void execute(Interpreter::Runtime& runtime) override { runtime.getContext().report("user2: not in use"); }
         };
 
-        template<class R>
+        template <class R>
         class OpUser3 : public Interpreter::Opcode0
         {
-            public:
+        public:
+            void execute(Interpreter::Runtime& runtime) override
+            {
+                //                    MWWorld::Ptr ptr = R()(runtime);
 
-                void execute (Interpreter::Runtime& runtime) override
-                {
-//                    MWWorld::Ptr ptr = R()(runtime);
-
-                    runtime.getContext().report ("user3: not in use");
-                }
+                runtime.getContext().report("user3: not in use");
+            }
         };
 
-        template<class R>
+        template <class R>
         class OpUser4 : public Interpreter::Opcode0
         {
-            public:
+        public:
+            void execute(Interpreter::Runtime& runtime) override
+            {
+                //                    MWWorld::Ptr ptr = R()(runtime);
 
-                void execute (Interpreter::Runtime& runtime) override
-                {
-//                    MWWorld::Ptr ptr = R()(runtime);
-
-                    runtime.getContext().report ("user4: not in use");
-                }
+                runtime.getContext().report("user4: not in use");
+            }
         };
-        
 
-        void installOpcodes (Interpreter::Interpreter& interpreter)
+        void installOpcodes(Interpreter::Interpreter& interpreter)
         {
             interpreter.installSegment5<OpUser1>(Compiler::User::opcodeUser1);
             interpreter.installSegment5<OpUser2>(Compiler::User::opcodeUser2);
