@@ -1,12 +1,13 @@
 #include "sound_buffer.hpp"
 
 #include "../mwbase/environment.hpp"
-#include "../mwbase/world.hpp"
 #include "../mwworld/esmstore.hpp"
 
 #include <components/debug/debuglog.hpp>
-#include <components/settings/settings.hpp>
-#include <components/vfs/manager.hpp>
+#include <components/esm3/loadsoun.hpp>
+#include <components/misc/resourcehelpers.hpp>
+#include <components/settings/values.hpp>
+#include <components/vfs/pathutil.hpp>
 
 #include <algorithm>
 #include <cmath>
@@ -23,9 +24,8 @@ namespace MWSound
             float mAudioMaxDistanceMult;
         };
 
-        AudioParams makeAudioParams(const MWBase::World& world)
+        AudioParams makeAudioParams(const MWWorld::Store<ESM::GameSetting>& settings)
         {
-            const auto& settings = world.getStore().get<ESM::GameSetting>();
             AudioParams params;
             params.mAudioDefaultMinDistance = settings.find("fAudioDefaultMinDistance")->mValue.getFloat();
             params.mAudioDefaultMaxDistance = settings.find("fAudioDefaultMaxDistance")->mValue.getFloat();
@@ -35,11 +35,11 @@ namespace MWSound
         }
     }
 
-    SoundBufferPool::SoundBufferPool(const VFS::Manager& vfs, Sound_Output& output) :
-        mVfs(&vfs),
-        mOutput(&output),
-        mBufferCacheMax(std::max(Settings::Manager::getInt("buffer cache max", "Sound"), 1) * 1024 * 1024),
-        mBufferCacheMin(std::min(static_cast<std::size_t>(std::max(Settings::Manager::getInt("buffer cache min", "Sound"), 1)) * 1024 * 1024, mBufferCacheMax))
+    SoundBufferPool::SoundBufferPool(Sound_Output& output)
+        : mOutput(&output)
+        , mBufferCacheMax(Settings::sound().mBufferCacheMax * 1024 * 1024)
+        , mBufferCacheMin(
+              std::min(static_cast<std::size_t>(Settings::sound().mBufferCacheMin) * 1024 * 1024, mBufferCacheMax))
     {
     }
 
@@ -48,7 +48,7 @@ namespace MWSound
         clear();
     }
 
-    Sound_Buffer* SoundBufferPool::lookup(const std::string& soundId) const
+    Sound_Buffer* SoundBufferPool::lookup(const ESM::RefId& soundId) const
     {
         const auto it = mBufferNameMap.find(soundId);
         if (it != mBufferNameMap.end())
@@ -60,12 +60,47 @@ namespace MWSound
         return nullptr;
     }
 
-    Sound_Buffer* SoundBufferPool::load(const std::string& soundId)
+    Sound_Buffer* SoundBufferPool::lookup(std::string_view fileName) const
+    {
+        const auto it = mBufferFileNameMap.find(std::string(fileName));
+        if (it != mBufferFileNameMap.end())
+        {
+            Sound_Buffer* sfx = it->second;
+            if (sfx->getHandle() != nullptr)
+                return sfx;
+        }
+        return nullptr;
+    }
+
+    Sound_Buffer* SoundBufferPool::loadSfx(Sound_Buffer* sfx)
+    {
+        if (sfx->getHandle() != nullptr)
+            return sfx;
+
+        auto [handle, size] = mOutput->loadSound(sfx->getResourceName());
+        if (handle == nullptr)
+            return {};
+
+        sfx->mHandle = handle;
+
+        mBufferCacheSize += size;
+        if (mBufferCacheSize > mBufferCacheMax)
+        {
+            unloadUnused();
+            if (!mUnusedBuffers.empty() && mBufferCacheSize > mBufferCacheMax)
+                Log(Debug::Warning) << "No unused sound buffers to free, using " << mBufferCacheSize << " bytes!";
+        }
+        mUnusedBuffers.push_front(sfx);
+
+        return sfx;
+    }
+
+    Sound_Buffer* SoundBufferPool::load(const ESM::RefId& soundId)
     {
         if (mBufferNameMap.empty())
         {
-            for (const ESM::Sound& sound : MWBase::Environment::get().getWorld()->getStore().get<ESM::Sound>())
-                insertSound(Misc::StringUtils::lowerCase(sound.mId), sound);
+            for (const ESM::Sound& sound : MWBase::Environment::get().getESMStore()->get<ESM::Sound>())
+                insertSound(sound.mId, sound);
         }
 
         Sound_Buffer* sfx;
@@ -74,47 +109,65 @@ namespace MWSound
             sfx = it->second;
         else
         {
-            const ESM::Sound *sound = MWBase::Environment::get().getWorld()->getStore().get<ESM::Sound>().search(soundId);
+            const ESM::Sound* sound = MWBase::Environment::get().getESMStore()->get<ESM::Sound>().search(soundId);
             if (sound == nullptr)
                 return {};
             sfx = insertSound(soundId, *sound);
         }
 
-        if (sfx->getHandle() == nullptr)
+        return loadSfx(sfx);
+    }
+
+    Sound_Buffer* SoundBufferPool::load(std::string_view fileName)
+    {
+        Sound_Buffer* sfx;
+        const auto it = mBufferFileNameMap.find(std::string(fileName));
+        if (it != mBufferFileNameMap.end())
+            sfx = it->second;
+        else
         {
-            auto [handle, size] = mOutput->loadSound(sfx->getResourceName());
-            if (handle == nullptr)
-                return {};
-
-            sfx->mHandle = handle;
-
-            mBufferCacheSize += size;
-            if (mBufferCacheSize > mBufferCacheMax)
-            {
-                unloadUnused();
-                if (!mUnusedBuffers.empty() && mBufferCacheSize > mBufferCacheMax)
-                    Log(Debug::Warning) << "No unused sound buffers to free, using " << mBufferCacheSize << " bytes!";
-            }
-            mUnusedBuffers.push_front(sfx);
+            sfx = insertSound(fileName);
         }
 
-        return sfx;
+        return loadSfx(sfx);
     }
 
     void SoundBufferPool::clear()
     {
-        for (auto &sfx : mSoundBuffers)
+        for (auto& sfx : mSoundBuffers)
         {
-            if(sfx.mHandle)
+            if (sfx.mHandle)
                 mOutput->unloadSound(sfx.mHandle);
             sfx.mHandle = nullptr;
         }
+
+        mBufferFileNameMap.clear();
+        mBufferNameMap.clear();
         mUnusedBuffers.clear();
     }
 
-    Sound_Buffer* SoundBufferPool::insertSound(const std::string& soundId, const ESM::Sound& sound)
+    Sound_Buffer* SoundBufferPool::insertSound(std::string_view fileName)
     {
-        static const AudioParams audioParams = makeAudioParams(*MWBase::Environment::get().getWorld());
+        static const AudioParams audioParams
+            = makeAudioParams(MWBase::Environment::get().getESMStore()->get<ESM::GameSetting>());
+
+        float volume = 1.f;
+        float min = std::max(audioParams.mAudioDefaultMinDistance * audioParams.mAudioMinDistanceMult, 1.f);
+        float max = std::max(min, audioParams.mAudioDefaultMaxDistance * audioParams.mAudioMaxDistanceMult);
+
+        min = std::max(min, 1.0f);
+        max = std::max(min, max);
+
+        Sound_Buffer& sfx = mSoundBuffers.emplace_back(fileName, volume, min, max);
+
+        mBufferFileNameMap.emplace(fileName, &sfx);
+        return &sfx;
+    }
+
+    Sound_Buffer* SoundBufferPool::insertSound(const ESM::RefId& soundId, const ESM::Sound& sound)
+    {
+        static const AudioParams audioParams
+            = makeAudioParams(MWBase::Environment::get().getESMStore()->get<ESM::GameSetting>());
 
         float volume = static_cast<float>(std::pow(10.0, (sound.mData.mVolume / 255.0 * 3348.0 - 3348.0) / 2000.0));
         float min = sound.mData.mMinRange;
@@ -130,8 +183,8 @@ namespace MWSound
         min = std::max(min, 1.0f);
         max = std::max(min, max);
 
-        Sound_Buffer& sfx = mSoundBuffers.emplace_back("Sound/" + sound.mSound, volume, min, max);
-        sfx.mResourceName = mVfs->normalizeFilename(sfx.mResourceName);
+        Sound_Buffer& sfx = mSoundBuffers.emplace_back(
+            Misc::ResourceHelpers::correctSoundPath(VFS::Path::Normalized(sound.mSound)), volume, min, max);
 
         mBufferNameMap.emplace(soundId, &sfx);
         return &sfx;
