@@ -5,21 +5,18 @@
 
 #include "../mwmechanics/actorutil.hpp"
 
-#include <components/compiler/locals.hpp>
-
-#include "inventorystore.hpp"
-#include "player.hpp"
 #include "class.hpp"
+#include "inventorystore.hpp"
 
 namespace MWWorld
 {
-    ActionEquip::ActionEquip (const MWWorld::Ptr& object, bool force)
-    : Action (false, object)
-    , mForce(force)
+    ActionEquip::ActionEquip(const MWWorld::Ptr& object, bool force)
+        : Action(false, object)
+        , mForce(force)
     {
     }
 
-    void ActionEquip::executeImp (const Ptr& actor)
+    void ActionEquip::executeImp(const Ptr& actor)
     {
         MWWorld::Ptr object = getTarget();
         MWWorld::InventoryStore& invStore = actor.getClass().getInventoryStore(actor);
@@ -34,13 +31,13 @@ namespace MWWorld
 
         if (!mForce)
         {
-            std::pair <int, std::string> result = object.getClass().canBeEquipped (object, actor);
+            auto result = object.getClass().canBeEquipped(object, actor);
 
             // display error message if the player tried to equip something
             if (!result.second.empty() && actor == MWMechanics::getPlayer())
                 MWBase::Environment::get().getWindowManager()->messageBox(result.second);
 
-            switch(result.first)
+            switch (result.first)
             {
                 case 0:
                     return;
@@ -65,11 +62,11 @@ namespace MWWorld
         }
 
         if (it == invStore.end())
-            throw std::runtime_error("ActionEquip can't find item " + object.getCellRef().getRefId());
+            throw std::runtime_error("ActionEquip can't find item " + object.getCellRef().getRefId().toDebugString());
 
         // equip the item in the first free slot
-        std::vector<int>::const_iterator slot=slots_.first.begin();
-        for (;slot!=slots_.first.end(); ++slot)
+        std::vector<int>::const_iterator slot = slots_.first.begin();
+        for (; slot != slots_.first.end(); ++slot)
         {
             // if the item is equipped already, nothing to do
             if (invStore.getSlot(*slot) == it)
@@ -78,7 +75,7 @@ namespace MWWorld
             if (invStore.getSlot(*slot) == invStore.end())
             {
                 // slot is not occupied
-                invStore.equip(*slot, it, actor);
+                invStore.equip(*slot, it);
                 break;
             }
         }
@@ -91,17 +88,17 @@ namespace MWWorld
             bool reEquip = false;
             for (slot = slots_.first.begin(); slot != slots_.first.end(); ++slot)
             {
-                invStore.unequipSlot(*slot, actor, false);
+                invStore.unequipSlot(*slot, false);
                 if (slot + 1 != slots_.first.end())
                 {
-                    invStore.equip(*slot, invStore.getSlot(*(slot + 1)), actor);
+                    invStore.equip(*slot, invStore.getSlot(*(slot + 1)));
                 }
                 else
                 {
-                    invStore.equip(*slot, it, actor);
+                    invStore.equip(*slot, it);
                 }
 
-                //Fix for issue of selected enchated item getting remmoved on cycle
+                // Fix for issue of selected enchated item getting remmoved on cycle
                 if (invStore.getSlot(*slot) == enchItem)
                 {
                     reEquip = true;
