@@ -4,7 +4,7 @@
 
 #include <MyGUI_LanguageManager.h>
 
-#include <components/misc/stringops.hpp>
+#include <components/misc/strings/format.hpp>
 
 #include "../mwbase/environment.hpp"
 #include "../mwbase/mechanicsmanager.hpp"
@@ -16,19 +16,19 @@
 
 namespace MWWorld
 {
-    ActionHarvest::ActionHarvest (const MWWorld::Ptr& container)
-        : Action (true, container)
+    ActionHarvest::ActionHarvest(const MWWorld::Ptr& container)
+        : Action(true, container)
     {
-        setSound("Item Ingredient Up");
+        setSound(ESM::RefId::stringRefId("Item Ingredient Up"));
     }
 
-    void ActionHarvest::executeImp (const MWWorld::Ptr& actor)
+    void ActionHarvest::executeImp(const MWWorld::Ptr& actor)
     {
         if (!MWBase::Environment::get().getWindowManager()->isAllowed(MWGui::GW_Inventory))
             return;
 
         MWWorld::Ptr target = getTarget();
-        MWWorld::ContainerStore& store = target.getClass().getContainerStore (target);
+        MWWorld::ContainerStore& store = target.getClass().getContainerStore(target);
         store.resolve();
         MWWorld::ContainerStore& actorStore = actor.getClass().getContainerStore(actor);
         std::map<std::string, int> takenMap;
@@ -37,13 +37,15 @@ namespace MWWorld
             if (!it->getClass().showsInInventory(*it))
                 continue;
 
-            int itemCount = it->getRefData().getCount();
-            // Note: it is important to check for crime before move an item from container. Otherwise owner check will not work
-            // for a last item in the container - empty harvested containers are considered as "allowed to use".
+            int itemCount = it->getCellRef().getCount();
+            // Note: it is important to check for crime before move an item from container. Otherwise owner check will
+            // not work for a last item in the container - empty harvested containers are considered as "allowed to
+            // use".
             MWBase::Environment::get().getMechanicsManager()->itemTaken(actor, *it, target, itemCount);
-            actorStore.add(*it, itemCount, actor);
-            store.remove(*it, itemCount, getTarget());
-            takenMap[it->getClass().getName(*it)]+=itemCount;
+            actorStore.add(*it, itemCount);
+            store.remove(*it, itemCount);
+            std::string name{ it->getClass().getName(*it) };
+            takenMap[name] += itemCount;
         }
 
         // Spawn a messagebox (only for items added to player's inventory)
@@ -52,9 +54,9 @@ namespace MWWorld
             std::ostringstream stream;
             int lineCount = 0;
             const static int maxLines = 10;
-            for (auto & pair : takenMap)
+            for (const auto& pair : takenMap)
             {
-                std::string itemName = pair.first;
+                const std::string& itemName = pair.first;
                 int itemCount = pair.second;
                 lineCount++;
                 if (lineCount == maxLines)
@@ -62,7 +64,8 @@ namespace MWWorld
                 else if (lineCount > maxLines)
                     break;
 
-                // The two GMST entries below expand to strings informing the player of what, and how many of it has been added to their inventory
+                // The two GMST entries below expand to strings informing the player of what, and how many of it has
+                // been added to their inventory
                 std::string msgBox;
                 if (itemCount == 1)
                 {
