@@ -1,10 +1,10 @@
 #include "actionsoulgem.hpp"
 
 #include <components/debug/debuglog.hpp>
+#include <components/esm3/loadcrea.hpp>
 
-#include "../mwbase/windowmanager.hpp"
 #include "../mwbase/environment.hpp"
-#include "../mwbase/world.hpp"
+#include "../mwbase/windowmanager.hpp"
 
 #include "../mwmechanics/actorutil.hpp"
 
@@ -13,24 +13,24 @@
 namespace MWWorld
 {
 
-    ActionSoulgem::ActionSoulgem(const Ptr &object)
+    ActionSoulgem::ActionSoulgem(const Ptr& object)
         : Action(false, object)
     {
-
     }
 
-    void ActionSoulgem::executeImp(const Ptr &actor)
+    void ActionSoulgem::executeImp(const Ptr& actor)
     {
         if (actor != MWMechanics::getPlayer())
             return;
 
-        if(MWMechanics::isPlayerInCombat()) { //Ensure we're not in combat
+        if (MWMechanics::isPlayerInCombat())
+        { // Ensure we're not in combat
             MWBase::Environment::get().getWindowManager()->messageBox("#{sInventoryMessage5}");
             return;
         }
 
         const auto& target = getTarget();
-        const std::string targetSoul = target.getCellRef().getSoul();
+        const ESM::RefId& targetSoul = target.getCellRef().getSoul();
 
         if (targetSoul.empty())
         {
@@ -38,14 +38,14 @@ namespace MWWorld
             return;
         }
 
-        if (!MWBase::Environment::get().getWorld()->getStore().get<ESM::Creature>().search(targetSoul))
+        if (!MWBase::Environment::get().getESMStore()->get<ESM::Creature>().search(targetSoul))
         {
-            Log(Debug::Warning) << "Soul '" << targetSoul << "' not found (item: '" << target.getCellRef().getRefId() << "')";
+            Log(Debug::Warning) << "Soul '" << targetSoul << "' not found (item: '" << target.getCellRef().getRefId()
+                                << "')";
             return;
         }
 
         MWBase::Environment::get().getWindowManager()->showSoulgemDialog(target);
     }
 
-
 }
