@@ -7,7 +7,7 @@
 
 namespace MWWorld
 {
-    void ActionTrap::executeImp(const Ptr &actor)
+    void ActionTrap::executeImp(const Ptr& actor)
     {
         osg::Vec3f actorPosition(actor.getRefData().getPosition().asVec3());
         osg::Vec3f trapPosition(mTrapSource.getRefData().getPosition().asVec3());
@@ -17,7 +17,9 @@ namespace MWWorld
         // radius, because for most trap spells this is 1 foot, much less than the activation distance.
         // Using activation distance as the trap range.
 
-        if (actor == MWBase::Environment::get().getWorld()->getPlayerPtr() && MWBase::Environment::get().getWorld()->getDistanceToFacedObject() > trapRange) // player activated object outside range of trap
+        if (actor == MWBase::Environment::get().getWorld()->getPlayerPtr()
+            && MWBase::Environment::get().getWorld()->getDistanceToFacedObject()
+                > trapRange) // player activated object outside range of trap
         {
             MWMechanics::CastSpell cast(mTrapSource, mTrapSource);
             cast.mHitPosition = trapPosition;
@@ -28,7 +30,7 @@ namespace MWWorld
             MWMechanics::CastSpell cast(mTrapSource, actor);
             cast.mHitPosition = actorPosition;
             cast.cast(mSpellId);
-        }   
-        mTrapSource.getCellRef().setTrap("");
+        }
+        mTrapSource.getCellRef().setTrap(ESM::RefId());
     }
 }
