@@ -1,12 +1,20 @@
 #ifndef OPENMW_MWWORLD_CELLPRELOADER_H
 #define OPENMW_MWWORLD_CELLPRELOADER_H
 
-#include <map>
-#include <osg/ref_ptr>
-#include <osg/Vec3f>
-#include <osg/Vec4i>
+#include "positioncellgrid.hpp"
+
 #include <components/sceneutil/workqueue.hpp>
 
+#include <osg/ref_ptr>
+
+#include <map>
+#include <span>
+
+namespace osg
+{
+    class Stats;
+}
+
 namespace Resource
 {
     class ResourceSystem;
@@ -37,12 +45,13 @@ namespace MWWorld
     class CellPreloader
     {
     public:
-        CellPreloader(Resource::ResourceSystem* resourceSystem, Resource::BulletShapeManager* bulletShapeManager, Terrain::World* terrain, MWRender::LandManager* landManager);
+        CellPreloader(Resource::ResourceSystem* resourceSystem, Resource::BulletShapeManager* bulletShapeManager,
+            Terrain::World* terrain, MWRender::LandManager* landManager);
         ~CellPreloader();
 
         /// Ask a background thread to preload rendering meshes and collision shapes for objects in this cell.
         /// @note The cell itself must be in State_Loaded or State_Preloaded.
-        void preload(MWWorld::CellStore* cell, double timestamp);
+        void preload(MWWorld::CellStore& cell, double timestamp);
 
         void notifyLoaded(MWWorld::CellStore* cell);
 
@@ -55,34 +64,40 @@ namespace MWWorld
         void setExpiryDelay(double expiryDelay);
 
         /// The minimum number of preloaded cells before unused cells get thrown out.
-        void setMinCacheSize(unsigned int num);
+        void setMinCacheSize(std::size_t value) { mMinCacheSize = value; }
 
         /// The maximum number of preloaded cells.
-        void setMaxCacheSize(unsigned int num);
+        void setMaxCacheSize(std::size_t value) { mMaxCacheSize = value; }
 
         /// Enables the creation of instances in the preloading thread.
         void setPreloadInstances(bool preload);
 
-        unsigned int getMaxCacheSize() const;
+        std::size_t getMaxCacheSize() const { return mMaxCacheSize; }
+
+        std::size_t getCacheSize() const { return mPreloadCells.size(); }
 
         void setWorkQueue(osg::ref_ptr<SceneUtil::WorkQueue> workQueue);
 
-        typedef std::pair<osg::Vec3f, osg::Vec4i> PositionCellGrid;
-        void setTerrainPreloadPositions(const std::vector<PositionCellGrid>& positions);
+        void setTerrainPreloadPositions(std::span<const PositionCellGrid> positions);
 
-        bool syncTerrainLoad(const std::vector<CellPreloader::PositionCellGrid> &positions, double timestamp, Loading::Listener& listener);
-        void abortTerrainPreloadExcept(const PositionCellGrid *exceptPos);
-        bool isTerrainLoaded(const CellPreloader::PositionCellGrid &position, double referenceTime) const;
+        void syncTerrainLoad(Loading::Listener& listener);
+        void abortTerrainPreloadExcept(const PositionCellGrid* exceptPos);
+        bool isTerrainLoaded(const PositionCellGrid& position, double referenceTime) const;
+        void setTerrain(Terrain::World* terrain);
+
+        void reportStats(unsigned int frameNumber, osg::Stats& stats) const;
 
     private:
+        void clearAllTasks();
+
         Resource::ResourceSystem* mResourceSystem;
         Resource::BulletShapeManager* mBulletShapeManager;
         Terrain::World* mTerrain;
         MWRender::LandManager* mLandManager;
         osg::ref_ptr<SceneUtil::WorkQueue> mWorkQueue;
         double mExpiryDelay;
-        unsigned int mMinCacheSize;
-        unsigned int mMaxCacheSize;
+        std::size_t mMinCacheSize = 0;
+        std::size_t mMaxCacheSize = 0;
         bool mPreloadInstances;
 
         double mLastResourceCacheUpdate;
@@ -91,7 +106,7 @@ namespace MWWorld
         {
             PreloadEntry(double timestamp, osg::ref_ptr<SceneUtil::WorkItem> workItem)
                 : mTimeStamp(timestamp)
-                , mWorkItem(workItem)
+                , mWorkItem(std::move(workItem))
             {
             }
             PreloadEntry()
@@ -107,13 +122,17 @@ namespace MWWorld
         // Cells that are currently being preloaded, or have already finished preloading
         PreloadMap mPreloadCells;
 
-        std::vector<osg::ref_ptr<Terrain::View> > mTerrainViews;
+        std::vector<osg::ref_ptr<Terrain::View>> mTerrainViews;
         std::vector<PositionCellGrid> mTerrainPreloadPositions;
         osg::ref_ptr<TerrainPreloadItem> mTerrainPreloadItem;
         osg::ref_ptr<SceneUtil::WorkItem> mUpdateCacheItem;
 
         std::vector<PositionCellGrid> mLoadedTerrainPositions;
         double mLoadedTerrainTimestamp;
+        std::size_t mEvicted = 0;
+        std::size_t mAdded = 0;
+        std::size_t mExpired = 0;
+        std::size_t mLoaded = 0;
     };
 
 }
