@@ -7,13 +7,15 @@
 
 namespace MWWorld
 {
-    FailedAction::FailedAction(const std::string &msg, const Ptr& target)
-      : Action(false, target), mMessage(msg)
-    {   }
-
-    void FailedAction::executeImp(const Ptr &actor)
+    FailedAction::FailedAction(std::string_view msg, const Ptr& target)
+        : Action(false, target)
+        , mMessage(msg)
     {
-        if(actor == MWMechanics::getPlayer() && !mMessage.empty())
+    }
+
+    void FailedAction::executeImp(const Ptr& actor)
+    {
+        if (actor == MWMechanics::getPlayer() && !mMessage.empty())
             MWBase::Environment::get().getWindowManager()->messageBox(mMessage);
     }
 }
