@@ -1,11 +1,15 @@
 #include "localscripts.hpp"
 
 #include <components/debug/debuglog.hpp>
+#include <components/esm3/loadcont.hpp>
+#include <components/esm3/loadcrea.hpp>
+#include <components/esm3/loadnpc.hpp>
+#include <components/esm3/loadscpt.hpp>
 
-#include "esmstore.hpp"
 #include "cellstore.hpp"
 #include "class.hpp"
 #include "containerstore.hpp"
+#include "esmstore.hpp"
 
 namespace
 {
@@ -20,10 +24,10 @@ namespace
 
         bool operator()(const MWWorld::Ptr& ptr)
         {
-            if (ptr.getRefData().isDeleted())
+            if (ptr.mRef->isDeleted())
                 return true;
 
-            std::string script = ptr.getClass().getScript(ptr);
+            const ESM::RefId& script = ptr.getClass().getScript(ptr);
 
             if (!script.empty())
                 mScripts.add(script, ptr);
@@ -43,19 +47,19 @@ namespace
         bool operator()(const MWWorld::Ptr& containerPtr)
         {
             // Ignore containers without generated content
-            if (containerPtr.getType() == ESM::Container::sRecordId &&
-                containerPtr.getRefData().getCustomData() == nullptr)
+            if (containerPtr.getType() == ESM::Container::sRecordId
+                && containerPtr.getRefData().getCustomData() == nullptr)
                 return true;
 
             MWWorld::ContainerStore& container = containerPtr.getClass().getContainerStore(containerPtr);
-            for(MWWorld::ContainerStoreIterator it = container.begin(); it != container.end(); ++it)
+            for (const auto& ptr : container)
             {
-                std::string script = it->getClass().getScript(*it);
-                if(script != "")
+                const ESM::RefId& script = ptr.getClass().getScript(ptr);
+                if (!script.empty())
                 {
-                    MWWorld::Ptr item = *it;
+                    MWWorld::Ptr item = ptr;
                     item.mCell = containerPtr.getCell();
-                    mScripts.add (script, item);
+                    mScripts.add(script, item);
                 }
             }
             return true;
@@ -64,7 +68,8 @@ namespace
 
 }
 
-MWWorld::LocalScripts::LocalScripts (const MWWorld::ESMStore& store) : mStore (store)
+MWWorld::LocalScripts::LocalScripts(const MWWorld::ESMStore& store)
+    : mStore(store)
 {
     mIter = mScripts.end();
 }
@@ -74,49 +79,46 @@ void MWWorld::LocalScripts::startIteration()
     mIter = mScripts.begin();
 }
 
-bool MWWorld::LocalScripts::getNext(std::pair<std::string, Ptr>& script)
+bool MWWorld::LocalScripts::getNext(std::pair<ESM::RefId, Ptr>& script)
 {
-    if (mIter!=mScripts.end())
+    if (mIter != mScripts.end())
     {
-        std::list<std::pair<std::string, Ptr> >::iterator iter = mIter++;
+        auto iter = mIter++;
         script = *iter;
         return true;
     }
     return false;
 }
 
-void MWWorld::LocalScripts::add (const std::string& scriptName, const Ptr& ptr)
+void MWWorld::LocalScripts::add(const ESM::RefId& scriptName, const Ptr& ptr)
 {
-    if (const ESM::Script *script = mStore.get<ESM::Script>().search (scriptName))
+    if (const ESM::Script* script = mStore.get<ESM::Script>().search(scriptName))
     {
         try
         {
-            ptr.getRefData().setLocals (*script);
+            ptr.getRefData().setLocals(*script);
 
-            for (std::list<std::pair<std::string, Ptr> >::iterator iter = mScripts.begin(); iter!=mScripts.end(); ++iter)
-                if (iter->second==ptr)
+            for (auto iter = mScripts.begin(); iter != mScripts.end(); ++iter)
+                if (iter->second == ptr)
                 {
                     Log(Debug::Warning) << "Error: tried to add local script twice for " << ptr.getCellRef().getRefId();
                     remove(ptr);
                     break;
                 }
 
-            mScripts.emplace_back (scriptName, ptr);
+            mScripts.emplace_back(scriptName, ptr);
         }
         catch (const std::exception& exception)
         {
-            Log(Debug::Error)
-                << "failed to add local script " << scriptName
-                << " because an exception has been thrown: " << exception.what();
+            Log(Debug::Error) << "failed to add local script " << scriptName
+                              << " because an exception has been thrown: " << exception.what();
         }
     }
     else
-        Log(Debug::Warning)
-            << "failed to add local script " << scriptName
-            << " because the script does not exist.";
+        Log(Debug::Warning) << "failed to add local script " << scriptName << " because the script does not exist.";
 }
 
-void MWWorld::LocalScripts::addCell (CellStore *cell)
+void MWWorld::LocalScripts::addCell(CellStore* cell)
 {
     AddScriptsVisitor addScriptsVisitor(*this);
     cell->forEach(addScriptsVisitor);
@@ -132,48 +134,51 @@ void MWWorld::LocalScripts::clear()
     mScripts.clear();
 }
 
-void MWWorld::LocalScripts::clearCell (CellStore *cell)
+void MWWorld::LocalScripts::clearCell(CellStore* cell)
 {
-    std::list<std::pair<std::string, Ptr> >::iterator iter = mScripts.begin();
+    auto iter = mScripts.begin();
 
-    while (iter!=mScripts.end())
+    while (iter != mScripts.end())
     {
-        if (iter->second.mCell==cell)
+        if (iter->second.mCell == cell)
         {
-            if (iter==mIter)
-               ++mIter;
+            if (iter == mIter)
+                ++mIter;
 
-            mScripts.erase (iter++);
+            mScripts.erase(iter++);
         }
         else
             ++iter;
     }
 }
 
-void MWWorld::LocalScripts::remove (RefData *ref)
+void MWWorld::LocalScripts::remove(const MWWorld::CellRef* ref)
 {
-    for (std::list<std::pair<std::string, Ptr> >::iterator iter = mScripts.begin();
-        iter!=mScripts.end(); ++iter)
-        if (&(iter->second.getRefData()) == ref)
+    for (auto iter = mScripts.begin(); iter != mScripts.end(); ++iter)
+        if (&(iter->second.getCellRef()) == ref)
         {
-            if (iter==mIter)
+            if (iter == mIter)
                 ++mIter;
 
-            mScripts.erase (iter);
+            mScripts.erase(iter);
             break;
         }
 }
 
-void MWWorld::LocalScripts::remove (const Ptr& ptr)
+void MWWorld::LocalScripts::remove(const Ptr& ptr)
 {
-    for (std::list<std::pair<std::string, Ptr> >::iterator iter = mScripts.begin();
-        iter!=mScripts.end(); ++iter)
-        if (iter->second==ptr)
+    for (auto iter = mScripts.begin(); iter != mScripts.end(); ++iter)
+        if (iter->second == ptr)
         {
-            if (iter==mIter)
+            if (iter == mIter)
                 ++mIter;
 
-            mScripts.erase (iter);
+            mScripts.erase(iter);
             break;
         }
 }
+
+bool MWWorld::LocalScripts::isRunning(const ESM::RefId& scriptName, const Ptr& ptr) const
+{
+    return std::ranges::find(mScripts, std::pair(scriptName, ptr)) != mScripts.end();
+}
