@@ -10,10 +10,12 @@ namespace ESM
 
 namespace MWWorld
 {
-    void convertMagicEffects(ESM::CreatureStats& creatureStats, ESM::InventoryState& inventory,
-        ESM::NpcStats* npcStats = nullptr);
+    void convertMagicEffects(
+        ESM::CreatureStats& creatureStats, ESM::InventoryState& inventory, ESM::NpcStats* npcStats = nullptr);
 
     void convertStats(ESM::CreatureStats& creatureStats);
+
+    void convertEnchantmentSlots(ESM::CreatureStats& creatureStats, ESM::InventoryState& inventory);
 }
 
 #endif
