@@ -1,17 +1,23 @@
 #include "projectilemanager.hpp"
 
 #include <iomanip>
-#include <sstream>
 #include <memory>
 #include <optional>
+#include <sstream>
 
 #include <osg/PositionAttitudeTransform>
 
 #include <components/debug/debuglog.hpp>
 
 #include <components/esm3/esmwriter.hpp>
+#include <components/esm3/loadench.hpp>
+#include <components/esm3/loadmgef.hpp>
+#include <components/esm3/loadrace.hpp>
 #include <components/esm3/projectilestate.hpp>
 
+#include <components/esm/quaternion.hpp>
+#include <components/esm/vector3.hpp>
+
 #include <components/misc/constants.hpp>
 #include <components/misc/convert.hpp>
 #include <components/misc/resourcehelpers.hpp>
@@ -20,33 +26,32 @@
 #include <components/resource/scenemanager.hpp>
 
 #include <components/sceneutil/controller.hpp>
-#include <components/sceneutil/visitor.hpp>
 #include <components/sceneutil/lightmanager.hpp>
 #include <components/sceneutil/nodecallback.hpp>
+#include <components/sceneutil/visitor.hpp>
 
-#include <components/settings/settings.hpp>
+#include <components/settings/values.hpp>
 
-#include "../mwworld/manualref.hpp"
 #include "../mwworld/class.hpp"
 #include "../mwworld/esmstore.hpp"
 #include "../mwworld/inventorystore.hpp"
+#include "../mwworld/manualref.hpp"
 
-#include "../mwbase/soundmanager.hpp"
-#include "../mwbase/world.hpp"
 #include "../mwbase/environment.hpp"
+#include "../mwbase/soundmanager.hpp"
 #include "../mwbase/windowmanager.hpp"
+#include "../mwbase/world.hpp"
 
+#include "../mwmechanics/actorutil.hpp"
 #include "../mwmechanics/combat.hpp"
 #include "../mwmechanics/creaturestats.hpp"
 #include "../mwmechanics/spellcasting.hpp"
-#include "../mwmechanics/actorutil.hpp"
-#include "../mwmechanics/aipackage.hpp"
 #include "../mwmechanics/weapontype.hpp"
 
 #include "../mwrender/animation.hpp"
-#include "../mwrender/vismask.hpp"
 #include "../mwrender/renderingmanager.hpp"
 #include "../mwrender/util.hpp"
+#include "../mwrender/vismask.hpp"
 
 #include "../mwsound/sound.hpp"
 
@@ -55,9 +60,10 @@
 
 namespace
 {
-    ESM::EffectList getMagicBoltData(std::vector<std::string>& projectileIDs, std::set<std::string>& sounds, float& speed, std::string& texture, std::string& sourceName, const std::string& id)
+    ESM::EffectList getMagicBoltData(std::vector<ESM::RefId>& projectileIDs, std::set<ESM::RefId>& sounds, float& speed,
+        std::string& texture, std::string& sourceName, const ESM::RefId& id)
     {
-        const MWWorld::ESMStore& esmStore = MWBase::Environment::get().getWorld()->getStore();
+        const MWWorld::ESMStore& esmStore = *MWBase::Environment::get().getESMStore();
         const ESM::EffectList* effects;
         if (const ESM::Spell* spell = esmStore.get<ESM::Spell>().search(id)) // check if it's a spell
         {
@@ -76,50 +82,52 @@ namespace
         int count = 0;
         speed = 0.0f;
         ESM::EffectList projectileEffects;
-        for (std::vector<ESM::ENAMstruct>::const_iterator iter (effects->mList.begin());
-            iter!=effects->mList.end(); ++iter)
+        for (const ESM::IndexedENAMstruct& effect : effects->mList)
         {
-            const ESM::MagicEffect *magicEffect = MWBase::Environment::get().getWorld()->getStore().get<ESM::MagicEffect>().find (
-                iter->mEffectID);
+            const ESM::MagicEffect* magicEffect
+                = MWBase::Environment::get().getESMStore()->get<ESM::MagicEffect>().find(effect.mData.mEffectID);
 
             // Speed of multi-effect projectiles should be the average of the constituent effects,
             // based on observation of the original engine.
             speed += magicEffect->mData.mSpeed;
             count++;
 
-            if (iter->mRange != ESM::RT_Target)
+            if (effect.mData.mRange != ESM::RT_Target)
                 continue;
 
             if (magicEffect->mBolt.empty())
-                projectileIDs.emplace_back("VFX_DefaultBolt");
+                projectileIDs.emplace_back(ESM::RefId::stringRefId("VFX_DefaultBolt"));
             else
                 projectileIDs.push_back(magicEffect->mBolt);
 
-            static const std::string schools[] = {
-                "alteration", "conjuration", "destruction", "illusion", "mysticism", "restoration"
-            };
             if (!magicEffect->mBoltSound.empty())
                 sounds.emplace(magicEffect->mBoltSound);
             else
-                sounds.emplace(schools[magicEffect->mData.mSchool] + " bolt");
-            projectileEffects.mList.push_back(*iter);
+                sounds.emplace(MWBase::Environment::get()
+                                   .getESMStore()
+                                   ->get<ESM::Skill>()
+                                   .find(magicEffect->mData.mSchool)
+                                   ->mSchool->mBoltSound);
+            projectileEffects.mList.push_back(effect);
         }
-        
+
         if (count != 0)
             speed /= count;
 
         // the particle texture is only used if there is only one projectile
-        if (projectileEffects.mList.size() == 1) 
+        if (projectileEffects.mList.size() == 1)
         {
-            const ESM::MagicEffect *magicEffect = MWBase::Environment::get().getWorld()->getStore().get<ESM::MagicEffect>().find (
-                effects->mList.begin()->mEffectID);
+            const ESM::MagicEffect* magicEffect
+                = MWBase::Environment::get().getESMStore()->get<ESM::MagicEffect>().find(
+                    effects->mList.begin()->mData.mEffectID);
             texture = magicEffect->mParticle;
         }
-        
-        if (projectileEffects.mList.size() > 1) // insert a VFX_Multiple projectile if there are multiple projectile effects
+
+        if (projectileEffects.mList.size()
+            > 1) // insert a VFX_Multiple projectile if there are multiple projectile effects
         {
-            const std::string ID = "VFX_Multiple" + std::to_string(effects->mList.size());
-            std::vector<std::string>::iterator it;
+            const ESM::RefId ID = ESM::RefId::stringRefId("VFX_Multiple" + std::to_string(effects->mList.size()));
+            std::vector<ESM::RefId>::iterator it;
             it = projectileIDs.begin();
             it = projectileIDs.insert(it, ID);
         }
@@ -130,23 +138,14 @@ namespace
     {
         // Calculate combined light diffuse color from magical effects
         osg::Vec4 lightDiffuseColor;
-        float lightDiffuseRed = 0.0f;
-        float lightDiffuseGreen = 0.0f;
-        float lightDiffuseBlue = 0.0f;
-        for (std::vector<ESM::ENAMstruct>::const_iterator iter(effects.mList.begin());
-            iter != effects.mList.end(); ++iter)
+        for (const ESM::IndexedENAMstruct& enam : effects.mList)
         {
-            const ESM::MagicEffect *magicEffect = MWBase::Environment::get().getWorld()->getStore().get<ESM::MagicEffect>().find(
-                iter->mEffectID);
-            lightDiffuseRed += (static_cast<float>(magicEffect->mData.mRed) / 255.f);
-            lightDiffuseGreen += (static_cast<float>(magicEffect->mData.mGreen) / 255.f);
-            lightDiffuseBlue += (static_cast<float>(magicEffect->mData.mBlue) / 255.f);
+            const ESM::MagicEffect* magicEffect
+                = MWBase::Environment::get().getESMStore()->get<ESM::MagicEffect>().find(enam.mData.mEffectID);
+            lightDiffuseColor += magicEffect->getColor();
         }
         int numberOfEffects = effects.mList.size();
-        lightDiffuseColor = osg::Vec4(lightDiffuseRed / numberOfEffects
-            , lightDiffuseGreen / numberOfEffects
-            , lightDiffuseBlue / numberOfEffects
-            , 1.0f);
+        lightDiffuseColor /= numberOfEffects;
 
         return lightDiffuseColor;
     }
@@ -156,21 +155,20 @@ namespace MWWorld
 {
 
     ProjectileManager::ProjectileManager(osg::Group* parent, Resource::ResourceSystem* resourceSystem,
-                                         MWRender::RenderingManager* rendering, MWPhysics::PhysicsSystem* physics)
+        MWRender::RenderingManager* rendering, MWPhysics::PhysicsSystem* physics)
         : mParent(parent)
         , mResourceSystem(resourceSystem)
         , mRendering(rendering)
         , mPhysics(physics)
         , mCleanupTimer(0.0f)
     {
-
     }
 
     /// Rotates an osg::PositionAttitudeTransform over time.
     class RotateCallback : public SceneUtil::NodeCallback<RotateCallback, osg::PositionAttitudeTransform*>
     {
     public:
-        RotateCallback(const osg::Vec3f& axis = osg::Vec3f(0,-1,0), float rotateSpeed = osg::PI*2)
+        RotateCallback(const osg::Vec3f& axis = osg::Vec3f(0, -1, 0), float rotateSpeed = osg::PI * 2)
             : mAxis(axis)
             , mRotateSpeed(rotateSpeed)
         {
@@ -191,9 +189,8 @@ namespace MWWorld
         float mRotateSpeed;
     };
 
-
-    void ProjectileManager::createModel(State &state, const std::string &model, const osg::Vec3f& pos, const osg::Quat& orient,
-                                        bool rotate, bool createLight, osg::Vec4 lightDiffuseColor, std::string texture)
+    void ProjectileManager::createModel(State& state, VFS::Path::NormalizedView model, const osg::Vec3f& pos,
+        const osg::Quat& orient, bool rotate, bool createLight, osg::Vec4 lightDiffuseColor, const std::string& texture)
     {
         state.mNode = new osg::PositionAttitudeTransform;
         state.mNode->setNodeMask(MWRender::Mask_Effect);
@@ -204,7 +201,7 @@ namespace MWWorld
 
         if (rotate)
         {
-            osg::ref_ptr<osg::PositionAttitudeTransform> rotateNode (new osg::PositionAttitudeTransform);
+            osg::ref_ptr<osg::PositionAttitudeTransform> rotateNode(new osg::PositionAttitudeTransform);
             rotateNode->addUpdateCallback(new RotateCallback());
             state.mNode->addChild(rotateNode);
             attachTo = rotateNode;
@@ -214,18 +211,19 @@ namespace MWWorld
 
         if (state.mIdMagic.size() > 1)
         {
-            const VFS::Manager* const vfs = MWBase::Environment::get().getResourceSystem()->getVFS();
-
             for (size_t iter = 1; iter != state.mIdMagic.size(); ++iter)
             {
                 std::ostringstream nodeName;
                 nodeName << "Dummy" << std::setw(2) << std::setfill('0') << iter;
-                const ESM::Weapon* weapon = MWBase::Environment::get().getWorld()->getStore().get<ESM::Weapon>().find (state.mIdMagic.at(iter));
-                SceneUtil::FindByNameVisitor findVisitor(nodeName.str());
+                const ESM::Weapon* weapon
+                    = MWBase::Environment::get().getESMStore()->get<ESM::Weapon>().find(state.mIdMagic.at(iter));
+                std::string nameToFind = nodeName.str();
+                SceneUtil::FindByNameVisitor findVisitor(nameToFind);
                 attachTo->accept(findVisitor);
                 if (findVisitor.mFoundNode)
                     mResourceSystem->getSceneManager()->getInstance(
-                        Misc::ResourceHelpers::correctMeshPath(weapon->mModel, vfs), findVisitor.mFoundNode);
+                        Misc::ResourceHelpers::correctMeshPath(VFS::Path::Normalized(weapon->mModel)),
+                        findVisitor.mFoundNode);
             }
         }
 
@@ -239,11 +237,11 @@ namespace MWWorld
             projectileLight->setLinearAttenuation(0.1f);
             projectileLight->setQuadraticAttenuation(0.f);
             projectileLight->setPosition(osg::Vec4(pos, 1.0));
-            
+
             SceneUtil::LightSource* projectileLightSource = new SceneUtil::LightSource;
             projectileLightSource->setNodeMask(MWRender::Mask_Lighting);
             projectileLightSource->setRadius(66.f);
-            
+
             state.mNode->addChild(projectileLightSource);
             projectileLightSource->setLight(projectileLight);
         }
@@ -254,10 +252,10 @@ namespace MWWorld
 
         state.mEffectAnimationTime = std::make_shared<MWRender::EffectAnimationTime>();
 
-        SceneUtil::AssignControllerSourcesVisitor assignVisitor (state.mEffectAnimationTime);
+        SceneUtil::AssignControllerSourcesVisitor assignVisitor(state.mEffectAnimationTime);
         state.mNode->accept(assignVisitor);
 
-        MWRender::overrideFirstRootTexture(texture, mResourceSystem, projectile);
+        MWRender::overrideFirstRootTexture(texture, mResourceSystem, *projectile);
     }
 
     void ProjectileManager::update(State& state, float duration)
@@ -265,29 +263,32 @@ namespace MWWorld
         state.mEffectAnimationTime->addTime(duration);
     }
 
-    void ProjectileManager::launchMagicBolt(const std::string &spellId, const Ptr &caster, const osg::Vec3f& fallbackDirection, int slot)
+    void ProjectileManager::launchMagicBolt(
+        const ESM::RefId& spellId, const Ptr& caster, const osg::Vec3f& fallbackDirection, ESM::RefNum item)
     {
         osg::Vec3f pos = caster.getRefData().getPosition().asVec3();
         if (caster.getClass().isActor())
         {
-            // Note: we ignore the collision box offset, this is required to make some flying creatures work as intended.
+            // Note: we ignore the collision box offset, this is required to make some flying creatures work as
+            // intended.
             pos.z() += mPhysics->getRenderingHalfExtents(caster).z() * 2 * Constants::TorsoHeight;
         }
 
-        if (MWBase::Environment::get().getWorld()->isUnderwater(caster.getCell(), pos)) // Underwater casting not possible
+        if (MWBase::Environment::get().getWorld()->isUnderwater(
+                caster.getCell(), pos)) // Underwater casting not possible
             return;
 
         osg::Quat orient;
         if (caster.getClass().isActor())
-            orient = osg::Quat(caster.getRefData().getPosition().rot[0], osg::Vec3f(-1,0,0))
-                    * osg::Quat(caster.getRefData().getPosition().rot[2], osg::Vec3f(0,0,-1));
+            orient = osg::Quat(caster.getRefData().getPosition().rot[0], osg::Vec3f(-1, 0, 0))
+                * osg::Quat(caster.getRefData().getPosition().rot[2], osg::Vec3f(0, 0, -1));
         else
-            orient.makeRotate(osg::Vec3f(0,1,0), osg::Vec3f(fallbackDirection));
+            orient.makeRotate(osg::Vec3f(0, 1, 0), osg::Vec3f(fallbackDirection));
 
         MagicBoltState state;
         state.mSpellId = spellId;
         state.mCasterHandle = caster;
-        state.mSlot = slot;
+        state.mItem = item;
         if (caster.getClass().isActor())
             state.mActorId = caster.getClass().getCreatureStats(caster).getActorId();
         else
@@ -295,7 +296,8 @@ namespace MWWorld
 
         std::string texture;
 
-        state.mEffects = getMagicBoltData(state.mIdMagic, state.mSoundIds, state.mSpeed, texture, state.mSourceName, state.mSpellId);
+        state.mEffects = getMagicBoltData(
+            state.mIdMagic, state.mSoundIds, state.mSpeed, texture, state.mSourceName, state.mSpellId);
 
         // Non-projectile should have been removed by getMagicBoltData
         if (state.mEffects.mList.empty())
@@ -307,52 +309,53 @@ namespace MWWorld
             return;
         }
 
-        MWWorld::ManualRef ref(MWBase::Environment::get().getWorld()->getStore(), state.mIdMagic.at(0));
+        MWWorld::ManualRef ref(*MWBase::Environment::get().getESMStore(), state.mIdMagic.at(0));
         MWWorld::Ptr ptr = ref.getPtr();
 
         osg::Vec4 lightDiffuseColor = getMagicBoltLightDiffuseColor(state.mEffects);
 
-        auto model = ptr.getClass().getModel(ptr);
+        VFS::Path::Normalized model = ptr.getClass().getCorrectedModel(ptr);
         createModel(state, model, pos, orient, true, true, lightDiffuseColor, texture);
 
-        MWBase::SoundManager *sndMgr = MWBase::Environment::get().getSoundManager();
-        for (const std::string &soundid : state.mSoundIds)
+        MWBase::SoundManager* sndMgr = MWBase::Environment::get().getSoundManager();
+        for (const auto& soundid : state.mSoundIds)
         {
-            MWBase::Sound *sound = sndMgr->playSound3D(pos, soundid, 1.0f, 1.0f,
-                                                       MWSound::Type::Sfx, MWSound::PlayMode::Loop);
+            MWBase::Sound* sound
+                = sndMgr->playSound3D(pos, soundid, 1.0f, 1.0f, MWSound::Type::Sfx, MWSound::PlayMode::Loop);
             if (sound)
                 state.mSounds.push_back(sound);
         }
 
-        // in case there are multiple effects, the model is a dummy without geometry. Use the second effect for physics shape
+        // in case there are multiple effects, the model is a dummy without geometry. Use the second effect for physics
+        // shape
         if (state.mIdMagic.size() > 1)
         {
-            model = Misc::ResourceHelpers::correctMeshPath(
-                MWBase::Environment::get().getWorld()->getStore().get<ESM::Weapon>().find(state.mIdMagic[1])->mModel,
-                MWBase::Environment::get().getResourceSystem()->getVFS());
+            model = Misc::ResourceHelpers::correctMeshPath(VFS::Path::Normalized(
+                MWBase::Environment::get().getESMStore()->get<ESM::Weapon>().find(state.mIdMagic[1])->mModel));
         }
         state.mProjectileId = mPhysics->addProjectile(caster, pos, model, true);
         state.mToDelete = false;
         mMagicBolts.push_back(state);
     }
 
-    void ProjectileManager::launchProjectile(const Ptr& actor, const ConstPtr& projectile, const osg::Vec3f &pos, const osg::Quat &orient, const Ptr& bow, float speed, float attackStrength)
+    void ProjectileManager::launchProjectile(const Ptr& actor, const ConstPtr& projectile, const osg::Vec3f& pos,
+        const osg::Quat& orient, const Ptr& bow, float speed, float attackStrength)
     {
         ProjectileState state;
         state.mActorId = actor.getClass().getCreatureStats(actor).getActorId();
         state.mBowId = bow.getCellRef().getRefId();
-        state.mVelocity = orient * osg::Vec3f(0,1,0) * speed;
+        state.mVelocity = orient * osg::Vec3f(0, 1, 0) * speed;
         state.mIdArrow = projectile.getCellRef().getRefId();
         state.mCasterHandle = actor;
         state.mAttackStrength = attackStrength;
         int type = projectile.get<ESM::Weapon>()->mBase->mData.mType;
         state.mThrown = MWMechanics::getWeaponType(type)->mWeaponClass == ESM::WeaponType::Thrown;
 
-        MWWorld::ManualRef ref(MWBase::Environment::get().getWorld()->getStore(), projectile.getCellRef().getRefId());
+        MWWorld::ManualRef ref(*MWBase::Environment::get().getESMStore(), projectile.getCellRef().getRefId());
         MWWorld::Ptr ptr = ref.getPtr();
 
-        const auto model = ptr.getClass().getModel(ptr);
-        createModel(state, model, pos, orient, false, false, osg::Vec4(0,0,0,0));
+        const VFS::Path::Normalized model = ptr.getClass().getCorrectedModel(ptr);
+        createModel(state, model, pos, orient, false, false, osg::Vec4(0, 0, 0, 0));
         if (!ptr.getClass().getEnchantment(ptr).empty())
             SceneUtil::addEnchantedGlow(state.mNode, mResourceSystem, ptr.getClass().getEnchantmentColor(ptr));
 
@@ -368,7 +371,8 @@ namespace MWWorld
 
         for (auto& state : mMagicBolts)
         {
-            // casters are identified by actor id in the savegame. objects doesn't have one so they can't be identified back.
+            // casters are identified by actor id in the savegame. objects doesn't have one so they can't be identified
+            // back.
             // TODO: should object-type caster be restored from savegame?
             if (state.mActorId == -1)
                 continue;
@@ -398,11 +402,10 @@ namespace MWWorld
         {
             mCleanupTimer = 2.0f;
 
-            auto isCleanable = [](const ProjectileManager::State& state) -> bool
-            {
+            auto isCleanable = [](const ProjectileManager::State& state) -> bool {
                 const float farawayThreshold = 72000.0f;
                 osg::Vec3 playerPos = MWMechanics::getPlayer().getRefData().getPosition().asVec3();
-                return (state.mNode->getPosition() - playerPos).length2() >= farawayThreshold*farawayThreshold;
+                return (state.mNode->getPosition() - playerPos).length2() >= farawayThreshold * farawayThreshold;
             };
 
             for (auto& projectileState : mProjectiles)
@@ -421,7 +424,7 @@ namespace MWWorld
 
     void ProjectileManager::moveMagicBolts(float duration)
     {
-        static const bool normaliseRaceSpeed = Settings::Manager::getBool("normalise race speed", "Game");
+        const bool normaliseRaceSpeed = Settings::game().mNormaliseRaceSpeed;
         for (auto& magicBoltState : mMagicBolts)
         {
             if (magicBoltState.mToDelete)
@@ -434,30 +437,32 @@ namespace MWWorld
             MWWorld::Ptr caster = magicBoltState.getCaster();
             if (!caster.isEmpty() && caster.getClass().isActor())
             {
-                if (caster.getRefData().getCount() <= 0 || caster.getClass().getCreatureStats(caster).isDead())
+                if (caster.getCellRef().getCount() <= 0 || caster.getClass().getCreatureStats(caster).isDead())
                 {
                     cleanupMagicBolt(magicBoltState);
                     continue;
                 }
             }
 
-            const auto& store = MWBase::Environment::get().getWorld()->getStore();
+            const auto& store = *MWBase::Environment::get().getESMStore();
             osg::Quat orient = magicBoltState.mNode->getAttitude();
-            static float fTargetSpellMaxSpeed = store.get<ESM::GameSetting>().find("fTargetSpellMaxSpeed")->mValue.getFloat();
+            static float fTargetSpellMaxSpeed
+                = store.get<ESM::GameSetting>().find("fTargetSpellMaxSpeed")->mValue.getFloat();
             float speed = fTargetSpellMaxSpeed * magicBoltState.mSpeed;
             if (!normaliseRaceSpeed && !caster.isEmpty() && caster.getClass().isNpc())
             {
                 const auto npc = caster.get<ESM::NPC>()->mBase;
                 const auto race = store.get<ESM::Race>().find(npc->mRace);
-                speed *= npc->isMale() ? race->mData.mWeight.mMale : race->mData.mWeight.mFemale;
+                speed *= npc->isMale() ? race->mData.mMaleWeight : race->mData.mFemaleWeight;
             }
-            osg::Vec3f direction = orient * osg::Vec3f(0,1,0);
+            osg::Vec3f direction = orient * osg::Vec3f(0, 1, 0);
             direction.normalize();
             projectile->setVelocity(direction * speed);
 
             update(magicBoltState, duration);
 
-            // For AI actors, get combat targets to use in the ray cast. Only those targets will return a positive hit result.
+            // For AI actors, get combat targets to use in the ray cast. Only those targets will return a positive hit
+            // result.
             std::vector<MWWorld::Ptr> targetActors;
             if (!caster.isEmpty() && caster.getClass().isActor() && caster != MWMechanics::getPlayer())
                 caster.getClass().getCreatureStats(caster).getAiSequence().getCombatTargets(targetActors);
@@ -477,15 +482,17 @@ namespace MWWorld
                 continue;
             // gravity constant - must be way lower than the gravity affecting actors, since we're not
             // simulating aerodynamics at all
-            projectileState.mVelocity -= osg::Vec3f(0, 0, Constants::GravityConst * Constants::UnitsPerMeter * 0.1f) * duration;
+            projectileState.mVelocity
+                -= osg::Vec3f(0, 0, Constants::GravityConst * Constants::UnitsPerMeter * 0.1f) * duration;
 
             projectile->setVelocity(projectileState.mVelocity);
 
-            // rotation does not work well for throwing projectiles - their roll angle will depend on shooting direction.
+            // rotation does not work well for throwing projectiles - their roll angle will depend on shooting
+            // direction.
             if (!projectileState.mThrown)
             {
                 osg::Quat orient;
-                orient.makeRotate(osg::Vec3f(0,1,0), projectileState.mVelocity);
+                orient.makeRotate(osg::Vec3f(0, 1, 0), projectileState.mVelocity);
                 projectileState.mNode->setAttitude(orient);
             }
 
@@ -493,7 +500,8 @@ namespace MWWorld
 
             MWWorld::Ptr caster = projectileState.getCaster();
 
-            // For AI actors, get combat targets to use in the ray cast. Only those targets will return a positive hit result.
+            // For AI actors, get combat targets to use in the ray cast. Only those targets will return a positive hit
+            // result.
             std::vector<MWWorld::Ptr> targetActors;
             if (!caster.isEmpty() && caster.getClass().isActor() && caster != MWMechanics::getPlayer())
                 caster.getClass().getCreatureStats(caster).getAiSequence().getCombatTargets(targetActors);
@@ -524,22 +532,26 @@ namespace MWWorld
                 caster = target;
 
             // Try to get a Ptr to the bow that was used. It might no longer exist.
-            MWWorld::ManualRef projectileRef(MWBase::Environment::get().getWorld()->getStore(), projectileState.mIdArrow);
+            MWWorld::ManualRef projectileRef(*MWBase::Environment::get().getESMStore(), projectileState.mIdArrow);
             MWWorld::Ptr bow = projectileRef.getPtr();
             if (!caster.isEmpty() && projectileState.mIdArrow != projectileState.mBowId)
             {
                 MWWorld::InventoryStore& inv = caster.getClass().getInventoryStore(caster);
                 MWWorld::ContainerStoreIterator invIt = inv.getSlot(MWWorld::InventoryStore::Slot_CarriedRight);
-                if (invIt != inv.end() && Misc::StringUtils::ciEqual(invIt->getCellRef().getRefId(), projectileState.mBowId))
+                if (invIt != inv.end() && invIt->getCellRef().getRefId() == projectileState.mBowId)
                     bow = *invIt;
             }
-            if (projectile->getHitWater())
-                mRendering->emitWaterRipple(pos);
 
-            MWMechanics::projectileHit(caster, target, bow, projectileRef.getPtr(), pos, projectileState.mAttackStrength);
+            const auto hitPosition = Misc::Convert::toOsg(projectile->getHitPosition());
+
+            if (projectile->getHitWater())
+                mRendering->emitWaterRipple(hitPosition);
+
+            MWMechanics::projectileHit(
+                caster, target, bow, projectileRef.getPtr(), hitPosition, projectileState.mAttackStrength);
             projectileState.mToDelete = true;
         }
-        const MWWorld::ESMStore& esmStore = MWBase::Environment::get().getWorld()->getStore();
+        const MWWorld::ESMStore& esmStore = *MWBase::Environment::get().getESMStore();
         for (auto& magicBoltState : mMagicBolts)
         {
             if (magicBoltState.mToDelete)
@@ -560,10 +572,10 @@ namespace MWWorld
             assert(target != caster);
 
             MWMechanics::CastSpell cast(caster, target);
-            cast.mHitPosition = pos;
+            cast.mHitPosition = Misc::Convert::toOsg(projectile->getHitPosition());
             cast.mId = magicBoltState.mSpellId;
             cast.mSourceName = magicBoltState.mSourceName;
-            cast.mSlot = magicBoltState.mSlot;
+            cast.mItem = magicBoltState.mItem;
             // Grab original effect list so the indices are correct
             const ESM::EffectList* effects;
             if (const ESM::Spell* spell = esmStore.get<ESM::Spell>().search(magicBoltState.mSpellId))
@@ -574,7 +586,7 @@ namespace MWWorld
                 const MWWorld::Ptr& ptr = ref.getPtr();
                 effects = &esmStore.get<ESM::Enchantment>().find(ptr.getClass().getEnchantment(ptr))->mEffects;
             }
-            cast.inflict(target, caster, *effects, ESM::RT_Target);
+            cast.inflict(target, *effects, ESM::RT_Target);
 
             magicBoltState.mToDelete = true;
         }
@@ -590,10 +602,12 @@ namespace MWWorld
             if (magicBoltState.mToDelete)
                 cleanupMagicBolt(magicBoltState);
         }
-        mProjectiles.erase(std::remove_if(mProjectiles.begin(), mProjectiles.end(), [](const State& state) { return state.mToDelete; }),
-                mProjectiles.end());
-        mMagicBolts.erase(std::remove_if(mMagicBolts.begin(), mMagicBolts.end(), [](const State& state) { return state.mToDelete; }),
-                mMagicBolts.end());
+        mProjectiles.erase(std::remove_if(mProjectiles.begin(), mProjectiles.end(),
+                               [](const State& state) { return state.mToDelete; }),
+            mProjectiles.end());
+        mMagicBolts.erase(
+            std::remove_if(mMagicBolts.begin(), mMagicBolts.end(), [](const State& state) { return state.mToDelete; }),
+            mMagicBolts.end());
     }
 
     void ProjectileManager::cleanupProjectile(ProjectileManager::ProjectileState& state)
@@ -625,7 +639,7 @@ namespace MWWorld
         mMagicBolts.clear();
     }
 
-    void ProjectileManager::write(ESM::ESMWriter &writer, Loading::Listener &progress) const
+    void ProjectileManager::write(ESM::ESMWriter& writer, Loading::Listener& progress) const
     {
         for (std::vector<ProjectileState>::const_iterator it = mProjectiles.begin(); it != mProjectiles.end(); ++it)
         {
@@ -655,7 +669,7 @@ namespace MWWorld
             state.mPosition = ESM::Vector3(osg::Vec3f(it->mNode->getPosition()));
             state.mOrientation = ESM::Quaternion(osg::Quat(it->mNode->getAttitude()));
             state.mActorId = it->mActorId;
-            state.mSlot = it->mSlot;
+            state.mItem = it->mItem;
             state.mSpellId = it->mSpellId;
             state.mSpeed = it->mSpeed;
 
@@ -665,7 +679,7 @@ namespace MWWorld
         }
     }
 
-    bool ProjectileManager::readRecord(ESM::ESMReader &reader, uint32_t type)
+    bool ProjectileManager::readRecord(ESM::ESMReader& reader, uint32_t type)
     {
         if (type == ESM::REC_PROJ)
         {
@@ -680,23 +694,27 @@ namespace MWWorld
             state.mAttackStrength = esm.mAttackStrength;
             state.mToDelete = false;
 
-            std::string model;
+            VFS::Path::Normalized model;
             try
             {
-                MWWorld::ManualRef ref(MWBase::Environment::get().getWorld()->getStore(), esm.mId);
+                MWWorld::ManualRef ref(*MWBase::Environment::get().getESMStore(), esm.mId);
                 MWWorld::Ptr ptr = ref.getPtr();
-                model = ptr.getClass().getModel(ptr);
+                model = ptr.getClass().getCorrectedModel(ptr);
                 int weaponType = ptr.get<ESM::Weapon>()->mBase->mData.mType;
                 state.mThrown = MWMechanics::getWeaponType(weaponType)->mWeaponClass == ESM::WeaponType::Thrown;
 
-                state.mProjectileId = mPhysics->addProjectile(state.getCaster(), osg::Vec3f(esm.mPosition), model, false);
+                state.mProjectileId
+                    = mPhysics->addProjectile(state.getCaster(), osg::Vec3f(esm.mPosition), model, false);
             }
-            catch(...)
+            catch (const std::exception& e)
             {
+                Log(Debug::Warning) << "Failed to add projectile for " << esm.mId
+                                    << " while reading projectile record: " << e.what();
                 return true;
             }
 
-            createModel(state, model, osg::Vec3f(esm.mPosition), osg::Quat(esm.mOrientation), false, false, osg::Vec4(0,0,0,0));
+            createModel(state, model, osg::Vec3f(esm.mPosition), osg::Quat(esm.mOrientation), false, false,
+                osg::Vec4(0, 0, 0, 0));
 
             mProjectiles.push_back(state);
             return true;
@@ -711,16 +729,18 @@ namespace MWWorld
             state.mSpellId = esm.mSpellId;
             state.mActorId = esm.mActorId;
             state.mToDelete = false;
-            state.mSlot = esm.mSlot;
+            state.mItem = esm.mItem;
             std::string texture;
 
             try
             {
-                state.mEffects = getMagicBoltData(state.mIdMagic, state.mSoundIds, state.mSpeed, texture, state.mSourceName, state.mSpellId);
+                state.mEffects = getMagicBoltData(
+                    state.mIdMagic, state.mSoundIds, state.mSpeed, texture, state.mSourceName, state.mSpellId);
             }
-            catch(...)
+            catch (const std::exception& e)
             {
-                Log(Debug::Warning) << "Warning: Failed to recreate magic projectile from saved data (id \"" << state.mSpellId << "\" no longer exists?)";
+                Log(Debug::Warning) << "Failed to recreate magic projectile for " << esm.mId << " and spell "
+                                    << state.mSpellId << " while reading projectile record: " << e.what();
                 return true;
             }
 
@@ -729,27 +749,30 @@ namespace MWWorld
                                        // file's effect list, which is already trimmed of non-projectile
                                        // effects. We need to use the stored value.
 
-            std::string model;
+            VFS::Path::Normalized model;
             try
             {
-                MWWorld::ManualRef ref(MWBase::Environment::get().getWorld()->getStore(), state.mIdMagic.at(0));
+                MWWorld::ManualRef ref(*MWBase::Environment::get().getESMStore(), state.mIdMagic.at(0));
                 MWWorld::Ptr ptr = ref.getPtr();
-                model = ptr.getClass().getModel(ptr);
+                model = ptr.getClass().getCorrectedModel(ptr);
             }
-            catch(...)
+            catch (const std::exception& e)
             {
+                Log(Debug::Warning) << "Failed to get model for " << state.mIdMagic.at(0)
+                                    << " while reading projectile record: " << e.what();
                 return true;
             }
 
             osg::Vec4 lightDiffuseColor = getMagicBoltLightDiffuseColor(state.mEffects);
-            createModel(state, model, osg::Vec3f(esm.mPosition), osg::Quat(esm.mOrientation), true, true, lightDiffuseColor, texture);
+            createModel(state, model, osg::Vec3f(esm.mPosition), osg::Quat(esm.mOrientation), true, true,
+                lightDiffuseColor, texture);
             state.mProjectileId = mPhysics->addProjectile(state.getCaster(), osg::Vec3f(esm.mPosition), model, true);
 
-            MWBase::SoundManager *sndMgr = MWBase::Environment::get().getSoundManager();
-            for (const std::string &soundid : state.mSoundIds)
+            MWBase::SoundManager* sndMgr = MWBase::Environment::get().getSoundManager();
+            for (const auto& soundid : state.mSoundIds)
             {
-                MWBase::Sound *sound = sndMgr->playSound3D(esm.mPosition, soundid, 1.0f, 1.0f,
-                                                           MWSound::Type::Sfx, MWSound::PlayMode::Loop);
+                MWBase::Sound* sound = sndMgr->playSound3D(
+                    esm.mPosition, soundid, 1.0f, 1.0f, MWSound::Type::Sfx, MWSound::PlayMode::Loop);
                 if (sound)
                     state.mSounds.push_back(sound);
             }
