@@ -1,64 +1,72 @@
 #include "inisettings.hpp"
 
-#include <QTextStream>
-#include <QFile>
-#include <QStringList>
-#include <QString>
-#include <QRegExp>
 #include <QDebug>
+#include <QFile>
+#include <QRegularExpression>
+#include <QString>
+#include <QStringList>
 
-Wizard::IniSettings::IniSettings()
-{
-}
+#include <fstream>
 
-Wizard::IniSettings::~IniSettings()
-{
-}
+#include <components/files/qtconversion.hpp>
 
-QStringList Wizard::IniSettings::findKeys(const QString &text)
+QStringList Wizard::IniSettings::findKeys(const QString& text)
 {
     QStringList result;
 
-    for (const QString &key : mSettings.keys())
+    for (const QString& key : mSettings.keys())
     {
 
         if (key.startsWith(text))
             result << key;
-
     }
 
     return result;
 }
 
-bool Wizard::IniSettings::readFile(QTextStream &stream)
+bool Wizard::IniSettings::readFile(std::ifstream& stream, ToUTF8::FromType encoding)
 {
     // Look for a square bracket, "'\\["
     // that has one or more "not nothing" in it, "([^]]+)"
     // and is closed with a square bracket, "\\]"
-    QRegExp sectionRe(QLatin1String("^\\[([^]]+)\\]"));
+    QRegularExpression sectionRe("^\\[([^]]+)\\]$");
 
     // Find any character(s) that is/are not equal sign(s), "[^=]+"
     // followed by an optional whitespace, an equal sign, and another optional whitespace, "\\s*=\\s*"
     // and one or more periods, "(.+)"
-    QRegExp keyRe(QLatin1String("^([^=]+)\\s*=\\s*(.+)$"));
+    QRegularExpression keyRe(QLatin1String("^([^=]+)\\s*=\\s*(.+)$"));
 
     QString currentSection;
 
-    while (!stream.atEnd())
-    {
-        const QString line(stream.readLine());
+    ToUTF8::Utf8Encoder encoder(encoding);
 
+    std::string legacyEncLine;
+    while (std::getline(stream, legacyEncLine))
+    {
+        std::string_view lineBuffer = encoder.getUtf8(legacyEncLine);
+
+        // unify Unix-style and Windows file ending
+        if (!(lineBuffer.empty()) && (lineBuffer[lineBuffer.length() - 1]) == '\r')
+        {
+            lineBuffer = lineBuffer.substr(0, lineBuffer.length() - 1);
+        }
+
+        const QString line = QString::fromStdString(std::string(lineBuffer));
         if (line.isEmpty() || line.startsWith(QLatin1Char(';')))
             continue;
 
-        if (sectionRe.exactMatch(line))
+        QRegularExpressionMatch sectionMatch = sectionRe.match(line);
+        if (sectionMatch.hasMatch())
         {
-            currentSection = sectionRe.cap(1);
+            currentSection = sectionMatch.captured(1);
+            continue;
         }
-        else if (keyRe.indexIn(line) != -1)
+
+        QRegularExpressionMatch match = keyRe.match(line);
+        if (match.hasMatch())
         {
-            QString key = keyRe.cap(1).trimmed();
-            QString value = keyRe.cap(2).trimmed();
+            QString key = match.captured(1).trimmed();
+            QString value = match.captured(2).trimmed();
 
             // Append the section, but only if there is one
             if (!currentSection.isEmpty())
@@ -71,37 +79,54 @@ bool Wizard::IniSettings::readFile(QTextStream &stream)
     return true;
 }
 
-bool Wizard::IniSettings::writeFile(const QString &path, QTextStream &stream)
+bool Wizard::IniSettings::writeFile(const QString& path, std::ifstream& stream, ToUTF8::FromType encoding)
 {
     // Look for a square bracket, "'\\["
     // that has one or more "not nothing" in it, "([^]]+)"
     // and is closed with a square bracket, "\\]"
-    QRegExp sectionRe(QLatin1String("^\\[([^]]+)\\]"));
+    QRegularExpression sectionRe("^\\[([^]]+)\\]$");
 
     // Find any character(s) that is/are not equal sign(s), "[^=]+"
     // followed by an optional whitespace, an equal sign, and another optional whitespace, "\\s*=\\s*"
     // and one or more periods, "(.+)"
-    QRegExp keyRe(QLatin1String("^([^=]+)\\s*=\\s*(.+)$"));
+    QRegularExpression keyRe(QLatin1String("^([^=]+)\\s*=\\s*(.+)$"));
 
     const QStringList keys(mSettings.keys());
 
     QString currentSection;
     QString buffer;
 
-    while (!stream.atEnd()) {
+    ToUTF8::Utf8Encoder encoder(encoding);
 
-        const QString line(stream.readLine());
+    std::string legacyEncLine;
+    while (std::getline(stream, legacyEncLine))
+    {
+        std::string_view lineBuffer = encoder.getUtf8(legacyEncLine);
+        // unify Unix-style and Windows file ending
+        if (!(lineBuffer.empty()) && (lineBuffer[lineBuffer.length() - 1]) == '\r')
+        {
+            lineBuffer = lineBuffer.substr(0, lineBuffer.length() - 1);
+        }
 
-        if (line.isEmpty() || line.startsWith(QLatin1Char(';'))) {
+        const QString line = QString::fromStdString(std::string(lineBuffer));
+        if (line.isEmpty() || line.startsWith(QLatin1Char(';')))
+        {
             buffer.append(line + QLatin1String("\n"));
             continue;
         }
 
-        if (sectionRe.exactMatch(line)) {
+        QRegularExpressionMatch sectionMatch = sectionRe.match(line);
+        if (sectionMatch.hasMatch())
+        {
             buffer.append(line + QLatin1String("\n"));
-            currentSection = sectionRe.cap(1);
-        } else  if (keyRe.indexIn(line) != -1) {
-            QString key(keyRe.cap(1).trimmed());
+            currentSection = sectionMatch.captured(1);
+            continue;
+        }
+
+        QRegularExpressionMatch match = keyRe.match(line);
+        if (match.hasMatch())
+        {
+            QString key(match.captured(1).trimmed());
             QString lookupKey(key);
 
             // Append the section, but only if there is one
@@ -115,7 +140,8 @@ bool Wizard::IniSettings::writeFile(const QString &path, QTextStream &stream)
 
     // Add the new settings to the buffer
     QHashIterator<QString, QVariant> i(mSettings);
-    while (i.hasNext()) {
+    while (i.hasNext())
+    {
         i.next();
 
         QStringList fullKey(i.key().split(QLatin1Char('/')));
@@ -125,7 +151,8 @@ bool Wizard::IniSettings::writeFile(const QString &path, QTextStream &stream)
         const QString& key(fullKey.at(1));
 
         int index = buffer.lastIndexOf(section);
-        if (index == -1) {
+        if (index == -1)
+        {
             // Add the section to the end of the file, because it's not found
             buffer.append(QString("\n%1\n").arg(section));
             index = buffer.lastIndexOf(section);
@@ -134,37 +161,33 @@ bool Wizard::IniSettings::writeFile(const QString &path, QTextStream &stream)
         // Look for the next section
         index = buffer.indexOf(QLatin1Char('['), index + 1);
 
-        if (index == -1 ) {
+        if (index == -1)
+        {
             // We are at the last section, append it to the bottom of the file
             buffer.append(QString("\n%1=%2").arg(key, i.value().toString()));
             mSettings.remove(i.key());
             continue;
-        } else {
+        }
+        else
+        {
             // Not at last section, add the key at the index
             buffer.insert(index - 1, QString("\n%1=%2").arg(key, i.value().toString()));
             mSettings.remove(i.key());
         }
     }
 
-    // Now we reopen the file, this time we write
-    QFile file(path);
-
-    if (file.open(QIODevice::ReadWrite | QIODevice::Truncate | QIODevice::Text)) {
-        QTextStream in(&file);
-        in.setCodec(stream.codec());
-
-        // Write the updated buffer to an empty file
-        in << buffer;
-        file.flush();
-        file.close();
-    } else {
+    const auto iniPath = Files::pathFromQString(path);
+    std::ofstream file(iniPath, std::ios::out);
+    if (file.fail())
         return false;
-    }
+
+    file << encoder.getLegacyEnc(buffer.toStdString());
+    file.close();
 
     return true;
 }
 
-bool Wizard::IniSettings::parseInx(const QString &path)
+bool Wizard::IniSettings::parseInx(const QString& path)
 {
     QFile file(path);
 
@@ -175,7 +198,8 @@ bool Wizard::IniSettings::parseInx(const QString &path)
         const QByteArray pattern("\x21\x00\x1A\x01\x04\x00\x04\x97\xFF\x06", 10);
 
         int i = 0;
-        while ((i = data.indexOf(pattern, i)) != -1) {
+        while ((i = data.indexOf(pattern, i)) != -1)
+        {
 
             int next = data.indexOf(pattern, i + 1);
             if (next == -1)
@@ -184,7 +208,8 @@ bool Wizard::IniSettings::parseInx(const QString &path)
             QByteArray array(data.mid(i, (next - i)));
 
             // Skip some invalid entries
-            if (array.contains("\x04\x96\xFF")) {
+            if (array.contains("\x04\x96\xFF"))
+            {
                 ++i;
                 continue;
             }
@@ -208,7 +233,9 @@ bool Wizard::IniSettings::parseInx(const QString &path)
         }
 
         file.close();
-    } else {
+    }
+    else
+    {
         qDebug() << "Failed to open INX file: " << path;
         return false;
     }
