@@ -1,15 +1,15 @@
 #include "installationpage.hpp"
 
-#include <QTextCodec>
+#include <QComboBox>
 #include <QFileDialog>
 #include <QMessageBox>
 #include <QThread>
 
 #include "mainwizard.hpp"
 
-Wizard::InstallationPage::InstallationPage(QWidget *parent, Config::GameSettings &gameSettings) :
-    QWizardPage(parent),
-    mGameSettings(gameSettings)
+Wizard::InstallationPage::InstallationPage(QWidget* parent, Config::GameSettings& gameSettings)
+    : QWizardPage(parent)
+    , mGameSettings(gameSettings)
 {
     mWizard = qobject_cast<MainWizard*>(parent);
 
@@ -18,44 +18,40 @@ Wizard::InstallationPage::InstallationPage(QWidget *parent, Config::GameSettings
     mFinished = false;
 
     mThread = std::make_unique<QThread>();
-    mUnshield = std::make_unique<UnshieldWorker>(mGameSettings.value("morrowind-bsa-filesize").toLongLong());
+    mUnshield = std::make_unique<UnshieldWorker>(mGameSettings.value("morrowind-bsa-filesize").value.toLongLong());
     mUnshield->moveToThread(mThread.get());
 
-    connect(mThread.get(), SIGNAL(started()),
-            mUnshield.get(), SLOT(extract()));
+    connect(mThread.get(), &QThread::started, mUnshield.get(), &UnshieldWorker::extract);
 
-    connect(mUnshield.get(), SIGNAL(finished()),
-            mThread.get(), SLOT(quit()));
+    connect(mUnshield.get(), &UnshieldWorker::finished, mThread.get(), &QThread::quit);
 
-    connect(mUnshield.get(), SIGNAL(finished()),
-            this, SLOT(installationFinished()), Qt::QueuedConnection);
+    connect(mUnshield.get(), &UnshieldWorker::finished, this, &InstallationPage::installationFinished,
+        Qt::QueuedConnection);
 
-    connect(mUnshield.get(), SIGNAL(error(QString, QString)),
-            this, SLOT(installationError(QString, QString)), Qt::QueuedConnection);
+    connect(mUnshield.get(), &UnshieldWorker::error, this, &InstallationPage::installationError, Qt::QueuedConnection);
 
-    connect(mUnshield.get(), SIGNAL(textChanged(QString)),
-            installProgressLabel, SLOT(setText(QString)), Qt::QueuedConnection);
+    connect(
+        mUnshield.get(), &UnshieldWorker::textChanged, installProgressLabel, &QLabel::setText, Qt::QueuedConnection);
 
-    connect(mUnshield.get(), SIGNAL(textChanged(QString)),
-            logTextEdit, SLOT(appendPlainText(QString)),  Qt::QueuedConnection);
+    connect(mUnshield.get(), &UnshieldWorker::textChanged, logTextEdit, &QPlainTextEdit::appendPlainText,
+        Qt::QueuedConnection);
 
-    connect(mUnshield.get(), SIGNAL(textChanged(QString)),
-            mWizard, SLOT(addLogText(QString)),  Qt::QueuedConnection);
+    connect(mUnshield.get(), &UnshieldWorker::textChanged, mWizard, &MainWizard::addLogText, Qt::QueuedConnection);
 
-    connect(mUnshield.get(), SIGNAL(progressChanged(int)),
-            installProgressBar, SLOT(setValue(int)),  Qt::QueuedConnection);
+    connect(mUnshield.get(), &UnshieldWorker::progressChanged, installProgressBar, &QProgressBar::setValue,
+        Qt::QueuedConnection);
 
-    connect(mUnshield.get(), SIGNAL(requestFileDialog(Wizard::Component)),
-            this, SLOT(showFileDialog(Wizard::Component)), Qt::QueuedConnection);
+    connect(mUnshield.get(), &UnshieldWorker::requestFileDialog, this, &InstallationPage::showFileDialog,
+        Qt::QueuedConnection);
 
-    connect(mUnshield.get(), SIGNAL(requestOldVersionDialog()),
-            this, SLOT(showOldVersionDialog())
-            , Qt::QueuedConnection);
+    connect(mUnshield.get(), &UnshieldWorker::requestOldVersionDialog, this, &InstallationPage::showOldVersionDialog,
+        Qt::QueuedConnection);
 }
 
 Wizard::InstallationPage::~InstallationPage()
 {
-    if (mThread->isRunning()) {
+    if (mThread->isRunning())
+    {
         mUnshield->stopWorker();
         mThread->quit();
         mThread->wait();
@@ -76,15 +72,16 @@ void Wizard::InstallationPage::initializePage()
     // That way installing all three components would yield 300%
     // When one component is done the bar will be filled by 33%
 
-    if (field(QLatin1String("installation.retailDisc")).toBool() == true) {
+    if (field(QLatin1String("installation.retailDisc")).toBool() == true)
+    {
         installProgressBar->setMaximum((components.count() * 100));
-    } else {
-        if (components.contains(QLatin1String("Tribunal"))
-                && !mWizard->mInstallations[path].hasTribunal)
+    }
+    else
+    {
+        if (components.contains(QLatin1String("Tribunal")) && !mWizard->mInstallations[path].hasTribunal)
             installProgressBar->setMaximum(100);
 
-        if (components.contains(QLatin1String("Bloodmoon"))
-                && !mWizard->mInstallations[path].hasBloodmoon)
+        if (components.contains(QLatin1String("Bloodmoon")) && !mWizard->mInstallations[path].hasBloodmoon)
             installProgressBar->setMaximum(installProgressBar->maximum() + 100);
     }
 
@@ -106,16 +103,16 @@ void Wizard::InstallationPage::startInstallation()
 
         if (components.contains(QLatin1String("Bloodmoon")))
             mUnshield->setInstallComponent(Wizard::Component_Bloodmoon, true);
-    } else {
+    }
+    else
+    {
         // Morrowind should already be installed
         mUnshield->setInstallComponent(Wizard::Component_Morrowind, false);
 
-        if (components.contains(QLatin1String("Tribunal"))
-                && !mWizard->mInstallations[path].hasTribunal)
+        if (components.contains(QLatin1String("Tribunal")) && !mWizard->mInstallations[path].hasTribunal)
             mUnshield->setInstallComponent(Wizard::Component_Tribunal, true);
 
-        if (components.contains(QLatin1String("Bloodmoon"))
-                && !mWizard->mInstallations[path].hasBloodmoon)
+        if (components.contains(QLatin1String("Bloodmoon")) && !mWizard->mInstallations[path].hasBloodmoon)
             mUnshield->setInstallComponent(Wizard::Component_Bloodmoon, true);
 
         // Set the location of the Morrowind.ini to update
@@ -129,12 +126,17 @@ void Wizard::InstallationPage::startInstallation()
     // Set the right codec to use for Morrowind.ini
     QString language(field(QLatin1String("installation.language")).toString());
 
-    if (language == QLatin1String("Polish")) {
-        mUnshield->setIniCodec(QTextCodec::codecForName("windows-1250"));
-    } else if (language == QLatin1String("Russian")) {
-        mUnshield->setIniCodec(QTextCodec::codecForName("windows-1251"));
-    }  else {
-        mUnshield->setIniCodec(QTextCodec::codecForName("windows-1252"));
+    if (language == QLatin1String("Polish"))
+    {
+        mUnshield->setIniEncoding(ToUTF8::FromType::WINDOWS_1250);
+    }
+    else if (language == QLatin1String("Russian"))
+    {
+        mUnshield->setIniEncoding(ToUTF8::FromType::WINDOWS_1251);
+    }
+    else
+    {
+        mUnshield->setIniEncoding(ToUTF8::FromType::WINDOWS_1252);
     }
 
     mThread->start();
@@ -143,17 +145,18 @@ void Wizard::InstallationPage::startInstallation()
 void Wizard::InstallationPage::showFileDialog(Wizard::Component component)
 {
     QString name;
-    switch (component) {
+    switch (component)
+    {
 
-    case Wizard::Component_Morrowind:
-        name = QLatin1String("Morrowind");
-        break;
-    case Wizard::Component_Tribunal:
-        name = QLatin1String("Tribunal");
-        break;
-    case Wizard::Component_Bloodmoon:
-        name = QLatin1String("Bloodmoon");
-        break;
+        case Wizard::Component_Morrowind:
+            name = QLatin1String("Morrowind");
+            break;
+        case Wizard::Component_Tribunal:
+            name = QLatin1String("Tribunal");
+            break;
+        case Wizard::Component_Bloodmoon:
+            name = QLatin1String("Bloodmoon");
+            break;
     }
     logTextEdit->appendHtml(tr("<p>Attempting to install component %1.</p>").arg(name));
     mWizard->addLogText(tr("Attempting to install component %1.").arg(name));
@@ -161,16 +164,20 @@ void Wizard::InstallationPage::showFileDialog(Wizard::Component component)
     QMessageBox msgBox;
     msgBox.setWindowTitle(tr("%1 Installation").arg(name));
     msgBox.setIcon(QMessageBox::Information);
-    msgBox.setText(QObject::tr("Select a valid %1 installation media.<br><b>Hint</b>: make sure that it contains at least one <b>.cab</b> file.").arg(name));
+    msgBox.setText(
+        QObject::tr("Select a valid %1 installation media.<br><b>Hint</b>: make sure that it contains at least one "
+                    "<b>.cab</b> file.")
+            .arg(name));
     msgBox.exec();
 
-    QString path = QFileDialog::getExistingDirectory(this,
-                                                     tr("Select %1 installation media").arg(name),
-                                                     QDir::rootPath());
+    QString path
+        = QFileDialog::getExistingDirectory(this, tr("Select %1 installation media").arg(name), QDir::rootPath());
 
-    if (path.isEmpty()) {
-        logTextEdit->appendHtml(tr("<p><br/><span style=\"color:red;\"> \
-                                    <b>Error: The installation was aborted by the user</b></span></p>"));
+    if (path.isEmpty())
+    {
+        logTextEdit->appendHtml(
+            tr("<p><br/><span style=\"color:red;\">"
+               "<b>Error: The installation was aborted by the user</b></span></p>"));
 
         mWizard->addLogText(QLatin1String("Error: The installation was aborted by the user"));
         mWizard->mError = true;
@@ -190,15 +197,17 @@ void Wizard::InstallationPage::showOldVersionDialog()
     QMessageBox msgBox;
     msgBox.setWindowTitle(tr("Morrowind Installation"));
     msgBox.setIcon(QMessageBox::Information);
-    msgBox.setText(QObject::tr("There may be a more recent version of Morrowind available.<br><br>Do you wish to continue anyway?"));
+    msgBox.setText(QObject::tr(
+        "There may be a more recent version of Morrowind available.<br><br>Do you wish to continue anyway?"));
     msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
     msgBox.setDefaultButton(QMessageBox::No);
 
     int ret = msgBox.exec();
     if (ret == QMessageBox::No)
     {
-        logTextEdit->appendHtml(tr("<p><br/><span style=\"color:red;\"> \
-                                    <b>Error: The installation was aborted by the user</b></span></p>"));
+        logTextEdit->appendHtml(
+            tr("<p><br/><span style=\"color:red;\">"
+               "<b>Error: The installation was aborted by the user</b></span></p>"));
 
         mWizard->addLogText(QLatin1String("Error: The installation was aborted by the user"));
         mWizard->mError = true;
@@ -224,14 +233,12 @@ void Wizard::InstallationPage::installationFinished()
     emit completeChanged();
 }
 
-void Wizard::InstallationPage::installationError(const QString &text, const QString &details)
+void Wizard::InstallationPage::installationError(const QString& text, const QString& details)
 {
     installProgressLabel->setText(tr("Installation failed!"));
 
-    logTextEdit->appendHtml(tr("<p><br/><span style=\"color:red;\"> \
-                               <b>Error: %1</b></p>").arg(text));
-    logTextEdit->appendHtml(tr("<p><span style=\"color:red;\"> \
-                               <b>%1</b></p>").arg(details));
+    logTextEdit->appendHtml(tr("<p><br/><span style=\"color:red;\"><b>Error: %1</b></p>").arg(text));
+    logTextEdit->appendHtml(tr("<p><span style=\"color:red;\"><b>%1</b></p>").arg(details));
 
     mWizard->addLogText(QLatin1String("Error: ") + text);
     mWizard->addLogText(details);
@@ -241,34 +248,44 @@ void Wizard::InstallationPage::installationError(const QString &text, const QStr
     msgBox.setWindowTitle(tr("An error occurred"));
     msgBox.setIcon(QMessageBox::Critical);
     msgBox.setStandardButtons(QMessageBox::Ok);
-    msgBox.setText(tr("<html><head/><body><p><b>The Wizard has encountered an error</b></p> \
-                      <p>The error reported was:</p><p>%1</p> \
-                      <p>Press &quot;Show Details...&quot; for more information.</p></body></html>").arg(text));
+    msgBox.setText(
+        tr("<html><head/><body><p><b>The Wizard has encountered an error</b></p>"
+           "<p>The error reported was:</p><p>%1</p>"
+           "<p>Press &quot;Show Details...&quot; for more information.</p></body></html>")
+            .arg(text));
 
     msgBox.setDetailedText(details);
     msgBox.exec();
 
-
     emit completeChanged();
 }
 
 bool Wizard::InstallationPage::isComplete() const
 {
-    if (!mWizard->mError) {
+    if (!mWizard->mError)
+    {
         return mFinished;
-    } else {
+    }
+    else
+    {
         return true;
     }
 }
 
 int Wizard::InstallationPage::nextId() const
 {
-    if (field(QLatin1String("installation.retailDisc")).toBool() == true) {
+    if (field(QLatin1String("installation.retailDisc")).toBool() == true)
+    {
         return MainWizard::Page_Conclusion;
-    } else {
-        if (!mWizard->mError) {
+    }
+    else
+    {
+        if (!mWizard->mError)
+        {
             return MainWizard::Page_Import;
-        } else {
+        }
+        else
+        {
             return MainWizard::Page_Conclusion;
         }
     }
