@@ -1,10 +1,14 @@
 #ifndef LANGUAGESELECTIONPAGE_HPP
 #define LANGUAGESELECTIONPAGE_HPP
 
-#include <QWizardPage>
+#include <QString>
+#include <QWizard>
 
 #include "ui_languageselectionpage.h"
 
+class QObject;
+class QWidget;
+
 namespace Wizard
 {
     class MainWizard;
@@ -13,15 +17,15 @@ namespace Wizard
     {
         Q_OBJECT
     public:
-        LanguageSelectionPage(QWidget *parent);
+        LanguageSelectionPage(QWidget* parent);
 
-        int nextId() const;
+        int nextId() const override;
 
     private:
-        MainWizard *mWizard;
+        MainWizard* mWizard;
 
     protected:
-        void initializePage();
+        void initializePage() override;
     };
 }
 
