@@ -1,31 +1,39 @@
 #include "mainwizard.hpp"
 
-#include <QDebug>
 #include <QDateTime>
-#include <QMessageBox>
-#include <QTextCodec>
+#include <QDebug>
 #include <QDir>
+#include <QMessageBox>
+#include <QProcess>
+
+#include <components/files/qtconfigpath.hpp>
+#include <components/files/qtconversion.hpp>
+#include <components/misc/utf8qtextstream.hpp>
+#include <components/process/processinvoker.hpp>
 
-#include "intropage.hpp"
-#include "methodselectionpage.hpp"
-#include "languageselectionpage.hpp"
-#include "existinginstallationpage.hpp"
-#include "installationtargetpage.hpp"
 #include "componentselectionpage.hpp"
-#include "importpage.hpp"
 #include "conclusionpage.hpp"
+#include "existinginstallationpage.hpp"
+#include "importpage.hpp"
+#include "installationtargetpage.hpp"
+#include "intropage.hpp"
+#include "languageselectionpage.hpp"
+#include "methodselectionpage.hpp"
 
 #ifdef OPENMW_USE_UNSHIELD
 #include "installationpage.hpp"
 #endif
 
+#include <algorithm>
+
 using namespace Process;
 
-Wizard::MainWizard::MainWizard(QWidget *parent) :
-    QWizard(parent),
-    mInstallations(),
-    mError(false),
-    mGameSettings(mCfgMgr)
+Wizard::MainWizard::MainWizard(Files::ConfigurationManager&& cfgMgr, QWidget* parent)
+    : QWizard(parent)
+    , mInstallations()
+    , mCfgMgr(cfgMgr)
+    , mError(false)
+    , mGameSettings(mCfgMgr)
 {
 #ifndef Q_OS_MAC
     setWizardStyle(QWizard::ModernStyle);
@@ -44,18 +52,18 @@ Wizard::MainWizard::MainWizard(QWidget *parent) :
 
     mImporterInvoker = new ProcessInvoker();
 
-    connect(mImporterInvoker->getProcess(), SIGNAL(started()),
-            this, SLOT(importerStarted()));
+    connect(mImporterInvoker->getProcess(), &QProcess::started, this, &MainWizard::importerStarted);
 
-    connect(mImporterInvoker->getProcess(), SIGNAL(finished(int,QProcess::ExitStatus)),
-            this, SLOT(importerFinished(int,QProcess::ExitStatus)));
+    connect(mImporterInvoker->getProcess(), qOverload<int, QProcess::ExitStatus>(&QProcess::finished), this,
+        &MainWizard::importerFinished);
 
-    mLogError = tr("<html><head/><body><p><b>Could not open %1 for writing</b></p> \
-                   <p>Please make sure you have the right permissions \
-                   and try again.</p></body></html>");
+    mLogError = tr(
+        "<html><head/><body><p><b>Could not open %1 for writing</b></p>"
+        "<p>Please make sure you have the right permissions "
+        "and try again.</p></body></html>");
 
-    boost::filesystem::create_directories(mCfgMgr.getUserConfigPath());
-    boost::filesystem::create_directories(mCfgMgr.getUserDataPath());
+    std::filesystem::create_directories(mCfgMgr.getUserConfigPath());
+    std::filesystem::create_directories(mCfgMgr.getUserDataPath());
 
     setupLog();
     setupGameSettings();
@@ -63,11 +71,11 @@ Wizard::MainWizard::MainWizard(QWidget *parent) :
     setupInstallations();
     setupPages();
 
-    const boost::filesystem::path& installationPath = mCfgMgr.getInstallPath();
+    const std::filesystem::path& installationPath = mCfgMgr.getInstallPath();
     if (!installationPath.empty())
     {
-        const boost::filesystem::path& dataPath = installationPath / "Data Files";
-        addInstallation(toQString(dataPath));
+        const std::filesystem::path& dataPath = installationPath / "Data Files";
+        addInstallation(Files::pathToQString(dataPath));
     }
 }
 
@@ -78,12 +86,13 @@ Wizard::MainWizard::~MainWizard()
 
 void Wizard::MainWizard::setupLog()
 {
-    QString logPath(toQString(mCfgMgr.getLogPath()));
+    QString logPath(Files::pathToQString(mCfgMgr.getLogPath()));
     logPath.append(QLatin1String("wizard.log"));
 
     QFile file(logPath);
 
-    if (!file.open(QIODevice::WriteOnly | QIODevice::Text | QIODevice::Truncate)) {
+    if (!file.open(QIODevice::WriteOnly | QIODevice::Text | QIODevice::Truncate))
+    {
         QMessageBox msgBox;
         msgBox.setWindowTitle(tr("Error opening Wizard log file"));
         msgBox.setIcon(QMessageBox::Critical);
@@ -99,14 +108,15 @@ void Wizard::MainWizard::setupLog()
     qDebug() << logPath;
 }
 
-void Wizard::MainWizard::addLogText(const QString &text)
+void Wizard::MainWizard::addLogText(const QString& text)
 {
-    QString logPath(toQString(mCfgMgr.getLogPath()));
+    QString logPath(Files::pathToQString(mCfgMgr.getLogPath()));
     logPath.append(QLatin1String("wizard.log"));
 
     QFile file(logPath);
 
-    if (!file.open(QIODevice::ReadWrite | QIODevice::Text)) {
+    if (!file.open(QIODevice::ReadWrite | QIODevice::Text))
+    {
         QMessageBox msgBox;
         msgBox.setWindowTitle(tr("Error opening Wizard log file"));
         msgBox.setIcon(QMessageBox::Critical);
@@ -131,21 +141,22 @@ void Wizard::MainWizard::addLogText(const QString &text)
 
 void Wizard::MainWizard::setupGameSettings()
 {
-    QString userPath(toQString(mCfgMgr.getUserConfigPath()));
-    QString globalPath(toQString(mCfgMgr.getGlobalPath()));
-    QString message(tr("<html><head/><body><p><b>Could not open %1 for reading</b></p> \
-                    <p>Please make sure you have the right permissions \
-                    and try again.</p></body></html>"));
+    QString message(
+        tr("<html><head/><body><p><b>Could not open %1 for reading</b></p>"
+           "<p>Please make sure you have the right permissions "
+           "and try again.</p></body></html>"));
 
     // Load the user config file first, separately
     // So we can write it properly, uncontaminated
-    QString path(userPath + QLatin1String("openmw.cfg"));
+    QString path(Files::getUserConfigPathQString(mCfgMgr));
     QFile file(path);
 
     qDebug() << "Loading config file:" << path.toUtf8().constData();
 
-    if (file.exists()) {
-        if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
+    if (file.exists())
+    {
+        if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
+        {
             QMessageBox msgBox;
             msgBox.setWindowTitle(tr("Error opening OpenMW configuration file"));
             msgBox.setIcon(QMessageBox::Critical);
@@ -156,26 +167,25 @@ void Wizard::MainWizard::setupGameSettings()
             return;
         }
         QTextStream stream(&file);
-        stream.setCodec(QTextCodec::codecForName("UTF-8"));
+        Misc::ensureUtf8Encoding(stream);
 
-        mGameSettings.readUserFile(stream);
+        mGameSettings.readUserFile(stream, QFileInfo(path).dir().path());
     }
 
     file.close();
 
     // Now the rest
-    QStringList paths;
-    paths.append(userPath + QLatin1String("openmw.cfg"));
-    paths.append(QLatin1String("openmw.cfg"));
-    paths.append(globalPath + QLatin1String("openmw.cfg"));
+    QStringList paths = Files::getActiveConfigPathsQString(mCfgMgr);
 
-    for (const QString &path2 : paths)
+    for (const QString& path2 : paths)
     {
         qDebug() << "Loading config file:" << path2.toUtf8().constData();
 
         file.setFileName(path2);
-        if (file.exists()) {
-            if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
+        if (file.exists())
+        {
+            if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
+            {
                 QMessageBox msgBox;
                 msgBox.setWindowTitle(tr("Error opening OpenMW configuration file"));
                 msgBox.setIcon(QMessageBox::Critical);
@@ -186,9 +196,9 @@ void Wizard::MainWizard::setupGameSettings()
                 return;
             }
             QTextStream stream(&file);
-            stream.setCodec(QTextCodec::codecForName("UTF-8"));
+            Misc::ensureUtf8Encoding(stream);
 
-            mGameSettings.readFile(stream);
+            mGameSettings.readFile(stream, QFileInfo(path2).dir().path());
         }
         file.close();
     }
@@ -196,20 +206,22 @@ void Wizard::MainWizard::setupGameSettings()
 
 void Wizard::MainWizard::setupLauncherSettings()
 {
-    QString path(toQString(mCfgMgr.getUserConfigPath()));
+    QString path(Files::pathToQString(mCfgMgr.getUserConfigPath()));
     path.append(QLatin1String(Config::LauncherSettings::sLauncherConfigFileName));
 
-    QString message(tr("<html><head/><body><p><b>Could not open %1 for reading</b></p> \
-                    <p>Please make sure you have the right permissions \
-                    and try again.</p></body></html>"));
-
+    QString message(
+        tr("<html><head/><body><p><b>Could not open %1 for reading</b></p>"
+           "<p>Please make sure you have the right permissions "
+           "and try again.</p></body></html>"));
 
     QFile file(path);
 
     qDebug() << "Loading config file:" << path.toUtf8().constData();
 
-    if (file.exists()) {
-        if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
+    if (file.exists())
+    {
+        if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
+        {
             QMessageBox msgBox;
             msgBox.setWindowTitle(tr("Error opening OpenMW configuration file"));
             msgBox.setIcon(QMessageBox::Critical);
@@ -220,23 +232,22 @@ void Wizard::MainWizard::setupLauncherSettings()
             return;
         }
         QTextStream stream(&file);
-        stream.setCodec(QTextCodec::codecForName("UTF-8"));
+        Misc::ensureUtf8Encoding(stream);
 
         mLauncherSettings.readFile(stream);
     }
 
     file.close();
-
 }
 
 void Wizard::MainWizard::setupInstallations()
 {
     // Check if the paths actually contain a Morrowind installation
-    for (const QString& path : mGameSettings.getDataDirs())
+    for (const auto& path : mGameSettings.getDataDirs())
     {
 
-        if (findFiles(QLatin1String("Morrowind"), path))
-            addInstallation(path);
+        if (findFiles(QLatin1String("Morrowind"), path.value))
+            addInstallation(path.value);
     }
 }
 
@@ -246,27 +257,30 @@ void Wizard::MainWizard::runSettingsImporter()
 
     QString path(field(QLatin1String("installation.path")).toString());
 
-    QString userPath(toQString(mCfgMgr.getUserConfigPath()));
-    QFile file(userPath + QLatin1String("openmw.cfg"));
+    QFile file(Files::getUserConfigPathQString(mCfgMgr));
 
     // Construct the arguments to run the importer
     QStringList arguments;
 
     // Import plugin selection?
     if (field(QLatin1String("installation.retailDisc")).toBool() == true
-            || field(QLatin1String("installation.import-addons")).toBool() == true)
+        || field(QLatin1String("installation.import-addons")).toBool() == true)
         arguments.append(QLatin1String("--game-files"));
 
     arguments.append(QLatin1String("--encoding"));
 
     // Set encoding
     QString language(field(QLatin1String("installation.language")).toString());
-
-    if (language == QLatin1String("Polish")) {
+    if (language == QLatin1String("Polish"))
+    {
         arguments.append(QLatin1String("win1250"));
-    } else if (language == QLatin1String("Russian")) {
+    }
+    else if (language == QLatin1String("Russian"))
+    {
         arguments.append(QLatin1String("win1251"));
-    }  else {
+    }
+    else
+    {
         arguments.append(QLatin1String("win1252"));
     }
 
@@ -277,23 +291,26 @@ void Wizard::MainWizard::runSettingsImporter()
     // Now the paths
     arguments.append(QLatin1String("--ini"));
 
-    if (field(QLatin1String("installation.retailDisc")).toBool() == true) {
+    if (field(QLatin1String("installation.retailDisc")).toBool() == true)
+    {
         arguments.append(path + QDir::separator() + QLatin1String("Morrowind.ini"));
-    } else {
+    }
+    else
+    {
         arguments.append(mInstallations[path].iniPath);
     }
 
     arguments.append(QLatin1String("--cfg"));
-    arguments.append(userPath + QLatin1String("openmw.cfg"));
+    arguments.append(Files::getUserConfigPathQString(mCfgMgr));
 
     if (!mImporterInvoker->startProcess(QLatin1String("openmw-iniimporter"), arguments, false))
         return qApp->quit();
 }
 
-void Wizard::MainWizard::addInstallation(const QString &path)
+void Wizard::MainWizard::addInstallation(const QString& path)
 {
     qDebug() << "add installation in: " << path;
-    Installation install;// = new Installation();
+    Installation install; // = new Installation();
 
     install.hasMorrowind = findFiles(QLatin1String("Morrowind"), path);
     install.hasTribunal = findFiles(QLatin1String("Tribunal"), path);
@@ -305,7 +322,8 @@ void Wizard::MainWizard::addInstallation(const QString &path)
 
     // Try the parent directory
     // In normal Morrowind installations that's where Morrowind.ini is
-    if (!file.exists()) {
+    if (!file.exists())
+    {
         dir.cdUp();
         file.setFileName(dir.filePath(QLatin1String("Morrowind.ini")));
     }
@@ -316,9 +334,12 @@ void Wizard::MainWizard::addInstallation(const QString &path)
     mInstallations.insert(QDir::toNativeSeparators(path), install);
 
     // Add it to the openmw.cfg too
-    if (!mGameSettings.getDataDirs().contains(path)) {
-        mGameSettings.setMultiValue(QLatin1String("data"), path);
-        mGameSettings.addDataDir(path);
+    const auto& dataDirs = mGameSettings.getDataDirs();
+    if (std::none_of(
+            dataDirs.begin(), dataDirs.end(), [&](const Config::SettingValue& dir) { return dir.value == path; }))
+    {
+        mGameSettings.setMultiValue(QLatin1String("data"), { path });
+        mGameSettings.addDataDir({ path });
     }
 }
 
@@ -336,12 +357,9 @@ void Wizard::MainWizard::setupPages()
     setPage(Page_Import, new ImportPage(this));
     setPage(Page_Conclusion, new ConclusionPage(this));
     setStartId(Page_Intro);
-
 }
 
-void Wizard::MainWizard::importerStarted()
-{
-}
+void Wizard::MainWizard::importerStarted() {}
 
 void Wizard::MainWizard::importerFinished(int exitCode, QProcess::ExitStatus exitStatus)
 {
@@ -366,7 +384,8 @@ void Wizard::MainWizard::reject()
     msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
     msgBox.setText(tr("Are you sure you want to exit the Wizard?"));
 
-    if (msgBox.exec() == QMessageBox::Yes) {
+    if (msgBox.exec() == QMessageBox::Yes)
+    {
         QWizard::reject();
     }
 }
@@ -375,14 +394,19 @@ void Wizard::MainWizard::writeSettings()
 {
     // Write the encoding and language settings
     QString language(field(QLatin1String("installation.language")).toString());
-    mLauncherSettings.setValue(QLatin1String("Settings/language"), language);
+    mLauncherSettings.setLanguage(language);
 
-    if (language == QLatin1String("Polish")) {
-        mGameSettings.setValue(QLatin1String("encoding"), QLatin1String("win1250"));
-    } else if (language == QLatin1String("Russian")) {
-        mGameSettings.setValue(QLatin1String("encoding"), QLatin1String("win1251"));
-    }  else {
-        mGameSettings.setValue(QLatin1String("encoding"), QLatin1String("win1252"));
+    if (language == QLatin1String("Polish"))
+    {
+        mGameSettings.setValue(QLatin1String("encoding"), { "win1250" });
+    }
+    else if (language == QLatin1String("Russian"))
+    {
+        mGameSettings.setValue(QLatin1String("encoding"), { "win1251" });
+    }
+    else
+    {
+        mGameSettings.setValue(QLatin1String("encoding"), { "win1252" });
     }
 
     // Write the installation path so that openmw can find them
@@ -390,20 +414,24 @@ void Wizard::MainWizard::writeSettings()
 
     // Make sure the installation path is the last data= entry
     mGameSettings.removeDataDir(path);
-    mGameSettings.addDataDir(path);
+    mGameSettings.addDataDir({ path });
 
-    QString userPath(toQString(mCfgMgr.getUserConfigPath()));
+    QString userPath(Files::pathToQString(mCfgMgr.getUserConfigPath()));
     QDir dir(userPath);
 
-    if (!dir.exists()) {
-        if (!dir.mkpath(userPath)) {
+    if (!dir.exists())
+    {
+        if (!dir.mkpath(userPath))
+        {
             QMessageBox msgBox;
             msgBox.setWindowTitle(tr("Error creating OpenMW configuration directory"));
             msgBox.setIcon(QMessageBox::Critical);
             msgBox.setStandardButtons(QMessageBox::Ok);
-            msgBox.setText(tr("<html><head/><body><p><b>Could not create %1</b></p> \
-                              <p>Please make sure you have the right permissions \
-                              and try again.</p></body></html>").arg(userPath));
+            msgBox.setText(
+                tr("<html><head/><body><p><b>Could not create %1</b></p>"
+                   "<p>Please make sure you have the right permissions "
+                   "and try again.</p></body></html>")
+                    .arg(userPath));
             connect(&msgBox, &QDialog::finished, qApp, &QApplication::quit, Qt::QueuedConnection);
             msgBox.exec();
             return;
@@ -411,53 +439,60 @@ void Wizard::MainWizard::writeSettings()
     }
 
     // Game settings
-    QFile file(userPath + QLatin1String("openmw.cfg"));
+    QFile file(Files::getUserConfigPathQString(mCfgMgr));
 
-    if (!file.open(QIODevice::ReadWrite | QIODevice::Text | QIODevice::Truncate)) {
+    if (!file.open(QIODevice::ReadWrite | QIODevice::Text | QIODevice::Truncate))
+    {
         // File cannot be opened or created
         QMessageBox msgBox;
         msgBox.setWindowTitle(tr("Error writing OpenMW configuration file"));
         msgBox.setIcon(QMessageBox::Critical);
         msgBox.setStandardButtons(QMessageBox::Ok);
-        msgBox.setText(tr("<html><head/><body><p><b>Could not open %1 for writing</b></p> \
-                          <p>Please make sure you have the right permissions \
-                          and try again.</p></body></html>").arg(file.fileName()));
+        msgBox.setText(
+            tr("<html><head/><body><p><b>Could not open %1 for writing</b></p>"
+               "<p>Please make sure you have the right permissions "
+               "and try again.</p></body></html>")
+                .arg(file.fileName()));
         connect(&msgBox, &QDialog::finished, qApp, &QApplication::quit, Qt::QueuedConnection);
         msgBox.exec();
         return;
     }
 
     QTextStream stream(&file);
-    stream.setCodec(QTextCodec::codecForName("UTF-8"));
+    Misc::ensureUtf8Encoding(stream);
 
     mGameSettings.writeFile(stream);
     file.close();
 
     // Launcher settings
-    file.setFileName(userPath + QLatin1String(Config::LauncherSettings::sLauncherConfigFileName));
+    file.setFileName(
+        Files::pathToQString(mCfgMgr.getUserConfigPath() / Config::LauncherSettings::sLauncherConfigFileName));
 
-    if (!file.open(QIODevice::ReadWrite | QIODevice::Text | QIODevice::Truncate)) {
+    if (!file.open(QIODevice::ReadWrite | QIODevice::Text | QIODevice::Truncate))
+    {
         // File cannot be opened or created
         QMessageBox msgBox;
         msgBox.setWindowTitle(tr("Error writing OpenMW configuration file"));
         msgBox.setIcon(QMessageBox::Critical);
         msgBox.setStandardButtons(QMessageBox::Ok);
-        msgBox.setText(tr("<html><head/><body><p><b>Could not open %1 for writing</b></p> \
-                          <p>Please make sure you have the right permissions \
-                          and try again.</p></body></html>").arg(file.fileName()));
+        msgBox.setText(
+            tr("<html><head/><body><p><b>Could not open %1 for writing</b></p>"
+               "<p>Please make sure you have the right permissions "
+               "and try again.</p></body></html>")
+                .arg(file.fileName()));
         connect(&msgBox, &QDialog::finished, qApp, &QApplication::quit, Qt::QueuedConnection);
         msgBox.exec();
         return;
     }
 
     stream.setDevice(&file);
-    stream.setCodec(QTextCodec::codecForName("UTF-8"));
+    Misc::ensureUtf8Encoding(stream);
 
     mLauncherSettings.writeFile(stream);
     file.close();
 }
 
-bool Wizard::MainWizard::findFiles(const QString &name, const QString &path)
+bool Wizard::MainWizard::findFiles(const QString& name, const QString& path)
 {
     QDir dir(path);
 
@@ -466,10 +501,5 @@ bool Wizard::MainWizard::findFiles(const QString &name, const QString &path)
 
     // TODO: add MIME handling to make sure the files are real
     return (dir.entryList().contains(name + QLatin1String(".esm"), Qt::CaseInsensitive)
-            && dir.entryList().contains(name + QLatin1String(".bsa"), Qt::CaseInsensitive));
-}
-
-QString Wizard::MainWizard::toQString(const boost::filesystem::path& path)
-{
-    return QString::fromUtf8(path.string().c_str());
+        && dir.entryList().contains(name + QLatin1String(".bsa"), Qt::CaseInsensitive));
 }
