@@ -4,6 +4,7 @@
 #include <BulletCollision/CollisionShapes/btTriangleCallback.h>
 
 #include <type_traits>
+#include <utility>
 
 namespace BulletHelpers
 {
@@ -13,7 +14,8 @@ namespace BulletHelpers
     public:
         explicit ProcessTriangleCallback(Impl impl)
             : mImpl(std::move(impl))
-        {}
+        {
+        }
 
         void processTriangle(btVector3* triangle, int partId, int triangleIndex) override
         {
