@@ -1,22 +1,22 @@
 #include "debug.hpp"
-#include "exceptions.hpp"
 #include "recastmesh.hpp"
 #include "settings.hpp"
 #include "settingsutils.hpp"
+#include "tilespositionsrange.hpp"
+#include "version.hpp"
 
-#include <boost/filesystem.hpp>
-#include <boost/filesystem/fstream.hpp>
-
-#include <components/bullethelpers/operators.hpp>
+#include <components/debug/writeflags.hpp>
 
 #include <DetourNavMesh.h>
 #include <DetourStatus.h>
 
 #include <osg/io_utils>
 
-#include <ostream>
 #include <array>
-#include <string_view>
+#include <filesystem>
+#include <fstream>
+#include <ostream>
+#include <system_error>
 
 namespace DetourNavigator
 {
@@ -27,8 +27,9 @@ namespace DetourNavigator
 
     std::ostream& operator<<(std::ostream& stream, Status value)
     {
-#define OPENMW_COMPONENTS_DETOURNAVIGATOR_DEBUG_STATUS_MESSAGE(name) \
-    case Status::name: return stream << "DetourNavigator::Status::"#name;
+#define OPENMW_COMPONENTS_DETOURNAVIGATOR_DEBUG_STATUS_MESSAGE(name)                                                   \
+    case Status::name:                                                                                                 \
+        return stream << "DetourNavigator::Status::" #name;
         switch (value)
         {
             OPENMW_COMPONENTS_DETOURNAVIGATOR_DEBUG_STATUS_MESSAGE(Success)
@@ -36,10 +37,11 @@ namespace DetourNavigator
             OPENMW_COMPONENTS_DETOURNAVIGATOR_DEBUG_STATUS_MESSAGE(NavMeshNotFound)
             OPENMW_COMPONENTS_DETOURNAVIGATOR_DEBUG_STATUS_MESSAGE(StartPolygonNotFound)
             OPENMW_COMPONENTS_DETOURNAVIGATOR_DEBUG_STATUS_MESSAGE(EndPolygonNotFound)
+            OPENMW_COMPONENTS_DETOURNAVIGATOR_DEBUG_STATUS_MESSAGE(TargetPolygonNotFound)
             OPENMW_COMPONENTS_DETOURNAVIGATOR_DEBUG_STATUS_MESSAGE(MoveAlongSurfaceFailed)
             OPENMW_COMPONENTS_DETOURNAVIGATOR_DEBUG_STATUS_MESSAGE(FindPathOverPolygonsFailed)
-            OPENMW_COMPONENTS_DETOURNAVIGATOR_DEBUG_STATUS_MESSAGE(GetPolyHeightFailed)
             OPENMW_COMPONENTS_DETOURNAVIGATOR_DEBUG_STATUS_MESSAGE(InitNavMeshQueryFailed)
+            OPENMW_COMPONENTS_DETOURNAVIGATOR_DEBUG_STATUS_MESSAGE(FindStraightPathFailed)
         }
 #undef OPENMW_COMPONENTS_DETOURNAVIGATOR_DEBUG_STATUS_MESSAGE
         return stream << "DetourNavigator::Error::" << static_cast<int>(value);
@@ -62,12 +64,9 @@ namespace DetourNavigator
 
     std::ostream& operator<<(std::ostream& s, const Heightfield& v)
     {
-        s << "Heightfield {.mCellPosition=" << v.mCellPosition
-          << ", .mCellSize=" << v.mCellSize
-          << ", .mLength=" << static_cast<int>(v.mLength)
-          << ", .mMinHeight=" << v.mMinHeight
-          << ", .mMaxHeight=" << v.mMaxHeight
-          << ", .mHeights={";
+        s << "Heightfield {.mCellPosition=" << v.mCellPosition << ", .mCellSize=" << v.mCellSize
+          << ", .mLength=" << static_cast<int>(v.mLength) << ", .mMinHeight=" << v.mMinHeight
+          << ", .mMaxHeight=" << v.mMaxHeight << ", .mHeights={";
         for (float h : v.mHeights)
             s << h << ", ";
         s << "}";
@@ -78,46 +77,43 @@ namespace DetourNavigator
     {
         switch (v)
         {
-            case CollisionShapeType::Aabb: return s << "AgentShapeType::Aabb";
-            case CollisionShapeType::RotatingBox: return s << "AgentShapeType::RotatingBox";
-            case CollisionShapeType::Cylinder: return s << "AgentShapeType::Cylinder";
+            case CollisionShapeType::Aabb:
+                return s << "CollisionShapeType::Aabb";
+            case CollisionShapeType::RotatingBox:
+                return s << "CollisionShapeType::RotatingBox";
+            case CollisionShapeType::Cylinder:
+                return s << "CollisionShapeType::Cylinder";
         }
-        return s << "AgentShapeType::" << static_cast<std::underlying_type_t<CollisionShapeType>>(v);
+        return s << "CollisionShapeType::" << static_cast<std::underlying_type_t<CollisionShapeType>>(v);
     }
 
     std::ostream& operator<<(std::ostream& s, const AgentBounds& v)
     {
-        return s << "AgentBounds {" << v.mShapeType << ", " << v.mHalfExtents << "}";
+        return s << "AgentBounds {" << v.mShapeType << ", {" << v.mHalfExtents.x() << ", " << v.mHalfExtents.y() << ", "
+                 << v.mHalfExtents.z() << "}}";
     }
 
     namespace
     {
-        struct StatusString
-        {
-            dtStatus mStatus;
-            std::string_view mString;
+        using StatusString = Debug::FlagString<unsigned int>;
+
+        constexpr std::array dtStatuses{
+            StatusString{ DT_FAILURE, "DT_FAILURE" },
+            StatusString{ DT_SUCCESS, "DT_SUCCESS" },
+            StatusString{ DT_IN_PROGRESS, "DT_IN_PROGRESS" },
+            StatusString{ DT_WRONG_MAGIC, "DT_WRONG_MAGIC" },
+            StatusString{ DT_WRONG_VERSION, "DT_WRONG_VERSION" },
+            StatusString{ DT_OUT_OF_MEMORY, "DT_OUT_OF_MEMORY" },
+            StatusString{ DT_INVALID_PARAM, "DT_INVALID_PARAM" },
+            StatusString{ DT_BUFFER_TOO_SMALL, "DT_BUFFER_TOO_SMALL" },
+            StatusString{ DT_OUT_OF_NODES, "DT_OUT_OF_NODES" },
+            StatusString{ DT_PARTIAL_RESULT, "DT_PARTIAL_RESULT" },
         };
     }
 
-    static constexpr std::array dtStatuses {
-        StatusString {DT_FAILURE, "DT_FAILURE"},
-        StatusString {DT_SUCCESS, "DT_SUCCESS"},
-        StatusString {DT_IN_PROGRESS, "DT_IN_PROGRESS"},
-        StatusString {DT_WRONG_MAGIC, "DT_WRONG_MAGIC"},
-        StatusString {DT_WRONG_VERSION, "DT_WRONG_VERSION"},
-        StatusString {DT_OUT_OF_MEMORY, "DT_OUT_OF_MEMORY"},
-        StatusString {DT_INVALID_PARAM, "DT_INVALID_PARAM"},
-        StatusString {DT_BUFFER_TOO_SMALL, "DT_BUFFER_TOO_SMALL"},
-        StatusString {DT_OUT_OF_NODES, "DT_OUT_OF_NODES"},
-        StatusString {DT_PARTIAL_RESULT, "DT_PARTIAL_RESULT"},
-    };
-
     std::ostream& operator<<(std::ostream& stream, const WriteDtStatus& value)
     {
-        for (const auto& status : dtStatuses)
-            if (value.mStatus & status.mStatus)
-                stream << status.mString;
-        return stream;
+        return Debug::writeFlags(stream, value.mStatus, dtStatuses);
     }
 
     std::ostream& operator<<(std::ostream& stream, const Flag value)
@@ -148,7 +144,7 @@ namespace DetourNavigator
         else
         {
             bool first = true;
-            for (const auto flag : {Flag_walk, Flag_swim, Flag_openDoor, Flag_usePathgrid})
+            for (const auto flag : { Flag_walk, Flag_swim, Flag_openDoor, Flag_usePathgrid })
             {
                 if (value.mValue & flag)
                 {
@@ -167,11 +163,16 @@ namespace DetourNavigator
     {
         switch (value)
         {
-            case AreaType_null: return stream << "null";
-            case AreaType_water: return stream << "water";
-            case AreaType_door: return stream << "door";
-            case AreaType_pathgrid: return stream << "pathgrid";
-            case AreaType_ground: return stream << "ground";
+            case AreaType_null:
+                return stream << "null";
+            case AreaType_water:
+                return stream << "water";
+            case AreaType_door:
+                return stream << "door";
+            case AreaType_pathgrid:
+                return stream << "pathgrid";
+            case AreaType_ground:
+                return stream << "ground";
         }
         return stream << "unknown area type (" << static_cast<std::underlying_type_t<AreaType>>(value) << ")";
     }
@@ -182,8 +183,6 @@ namespace DetourNavigator
         {
             case ChangeType::remove:
                 return stream << "ChangeType::remove";
-            case ChangeType::mixed:
-                return stream << "ChangeType::mixed";
             case ChangeType::add:
                 return stream << "ChangeType::add";
             case ChangeType::update:
@@ -192,13 +191,40 @@ namespace DetourNavigator
         return stream << "ChangeType::" << static_cast<int>(value);
     }
 
-    void writeToFile(const RecastMesh& recastMesh, const std::string& pathPrefix,
-        const std::string& revision, const RecastSettings& settings)
+    std::ostream& operator<<(std::ostream& stream, const Version& value)
+    {
+        return stream << "Version {" << value.mGeneration << ", " << value.mRevision << "}";
+    }
+
+    std::ostream& operator<<(std::ostream& stream, const TilesPositionsRange& value)
+    {
+        return stream << "TilesPositionsRange {" << value.mBegin << ", " << value.mEnd << "}";
+    }
+
+    std::ostream& operator<<(std::ostream& stream, const AreaCosts& value)
+    {
+        return stream << "AreaCosts {"
+                      << ".mWater = " << value.mWater << ", .mDoor = " << value.mDoor
+                      << ", .mPathgrid = " << value.mPathgrid << ", .mGround = " << value.mGround << "}";
+    }
+
+    std::ostream& operator<<(std::ostream& stream, const DetourSettings& value)
+    {
+        return stream << "DetourSettings {"
+                      << ".mMaxPolys = " << value.mMaxPolys
+                      << ", .mMaxNavMeshQueryNodes = " << value.mMaxNavMeshQueryNodes
+                      << ", .mMaxPolygonPathSize = " << value.mMaxPolygonPathSize
+                      << ", .mMaxSmoothPathSize = " << value.mMaxSmoothPathSize << "}";
+    }
+
+    void writeToFile(const RecastMesh& recastMesh, const std::string& pathPrefix, const std::string& revision,
+        const RecastSettings& settings)
     {
         const auto path = pathPrefix + "recastmesh" + revision + ".obj";
-        boost::filesystem::ofstream file(boost::filesystem::path(path), std::ios::out);
+        std::ofstream file(std::filesystem::path(path), std::ios::out);
         if (!file.is_open())
-            throw NavigatorException("Open file failed: " + path);
+            throw std::system_error(
+                errno, std::generic_category(), "Failed to open file to write recast mesh: " + path);
         file.exceptions(std::ios::failbit | std::ios::badbit);
         file.precision(std::numeric_limits<float>::max_exponent10);
         std::vector<float> vertices = recastMesh.getMesh().getVertices();
@@ -243,9 +269,9 @@ namespace DetourNavigator
         };
 
         const auto path = pathPrefix + "all_tiles_navmesh" + revision + ".bin";
-        boost::filesystem::ofstream file(boost::filesystem::path(path), std::ios::out | std::ios::binary);
+        std::ofstream file(std::filesystem::path(path), std::ios::out | std::ios::binary);
         if (!file.is_open())
-            throw NavigatorException("Open file failed: " + path);
+            throw std::system_error(errno, std::generic_category(), "Failed to open file to write navmesh: " + path);
         file.exceptions(std::ios::failbit | std::ios::badbit);
 
         NavMeshSetHeader header;
