@@ -4,19 +4,22 @@
 #include "recastglobalallocator.hpp"
 
 #include <components/debug/debuglog.hpp>
+#include <components/files/conversion.hpp>
 
 namespace DetourNavigator
 {
-    std::unique_ptr<Navigator> makeNavigator(const Settings& settings, const std::string& userDataPath)
+    std::unique_ptr<Navigator> makeNavigator(const Settings& settings, const std::filesystem::path& userDataPath)
     {
         DetourNavigator::RecastGlobalAllocator::init();
 
         std::unique_ptr<NavMeshDb> db;
         if (settings.mEnableNavMeshDiskCache)
         {
+            const std::string path = Files::pathToUnicodeString(userDataPath / "navmesh.db");
+            Log(Debug::Info) << "Using " << path << " to store navigation mesh cache";
             try
             {
-                db = std::make_unique<NavMeshDb>(userDataPath + "/navmesh.db", settings.mMaxDbFileSize);
+                db = std::make_unique<NavMeshDb>(path, settings.mMaxDbFileSize);
             }
             catch (const std::exception& e)
             {
