@@ -1,21 +1,27 @@
 ﻿#ifndef OPENMW_COMPONENTS_DETOURNAVIGATOR_NAVIGATOR_H
 #define OPENMW_COMPONENTS_DETOURNAVIGATOR_NAVIGATOR_H
 
-#include "objectid.hpp"
-#include "navmeshcacheitem.hpp"
-#include "recastmeshtiles.hpp"
-#include "waitconditiontype.hpp"
+#include <cassert>
+#include <filesystem>
+#include <optional>
+
+#include "cellgridbounds.hpp"
 #include "heightfieldshape.hpp"
+#include "objectid.hpp"
 #include "objecttransform.hpp"
+#include "recastmeshtiles.hpp"
+#include "sharednavmeshcacheitem.hpp"
+#include "updateguard.hpp"
+#include "waitconditiontype.hpp"
 
+#include <components/esm/refid.hpp>
 #include <components/resource/bulletshape.hpp>
 
-#include <string_view>
-
 namespace ESM
 {
     struct Cell;
     struct Pathgrid;
+    class RefId;
 }
 
 namespace Loading
@@ -27,13 +33,15 @@ namespace DetourNavigator
 {
     struct Settings;
     struct AgentBounds;
+    struct Stats;
 
     struct ObjectShapes
     {
         osg::ref_ptr<const Resource::BulletShapeInstance> mShapeInstance;
         ObjectTransform mTransform;
 
-        ObjectShapes(const osg::ref_ptr<const Resource::BulletShapeInstance>& shapeInstance, const ObjectTransform& transform)
+        ObjectShapes(
+            const osg::ref_ptr<const Resource::BulletShapeInstance>& shapeInstance, const ObjectTransform& transform)
             : mShapeInstance(shapeInstance)
             , mTransform(transform)
         {
@@ -47,11 +55,12 @@ namespace DetourNavigator
         osg::Vec3f mConnectionEnd;
 
         DoorShapes(const osg::ref_ptr<const Resource::BulletShapeInstance>& shapeInstance,
-                   const ObjectTransform& transform, const osg::Vec3f& connectionStart, const osg::Vec3f& connectionEnd)
+            const ObjectTransform& transform, const osg::Vec3f& connectionStart, const osg::Vec3f& connectionEnd)
             : ObjectShapes(shapeInstance, transform)
             , mConnectionStart(connectionStart)
             , mConnectionEnd(connectionEnd)
-        {}
+        {
+        }
     };
 
     /**
@@ -65,12 +74,15 @@ namespace DetourNavigator
     {
         virtual ~Navigator() = default;
 
+        virtual ScopedUpdateGuard makeUpdateGuard() = 0;
+
         /**
          * @brief addAgent should be called for each agent even if all of them has same half extents.
          * @param agentBounds allows to setup bounding cylinder for each agent, for each different half extents
          * there is different navmesh.
+         * @return true if agent is successfully added or false if agent bounds are not supported.
          */
-        virtual void addAgent(const AgentBounds& agentBounds) = 0;
+        virtual bool addAgent(const AgentBounds& agentBounds) = 0;
 
         /**
          * @brief removeAgent should be called for each agent even if all of them has same half extents
@@ -78,82 +90,76 @@ namespace DetourNavigator
          */
         virtual void removeAgent(const AgentBounds& agentBounds) = 0;
 
-        /**
-         * @brief setWorldspace should be called before adding object from new worldspace
-         * @param worldspace
-         */
-        virtual void setWorldspace(std::string_view worldspace) = 0;
-
-        /**
-         * @brief updateBounds should be called before adding object from loading cell
-         * @param playerPosition corresponds to the bounds center
-         */
-        virtual void updateBounds(const osg::Vec3f& playerPosition) = 0;
+        // Updates bounds for recast mesh and navmesh tiles, removes tiles outside the range.
+        virtual void updateBounds(ESM::RefId worldspace, const std::optional<CellGridBounds>& cellGridBounds,
+            const osg::Vec3f& playerPosition, const UpdateGuard* guard)
+            = 0;
 
         /**
          * @brief addObject is used to add complex object with allowed to walk and avoided to walk shapes
          * @param id is used to distinguish different objects
          * @param shape members must live until object is updated by another shape removed from Navigator
          * @param transform allows to setup objects geometry according to its world state
-         * @return true if object is added, false if there is already object with given id
          */
-        virtual bool addObject(const ObjectId id, const ObjectShapes& shapes, const btTransform& transform) = 0;
+        virtual void addObject(
+            const ObjectId id, const ObjectShapes& shapes, const btTransform& transform, const UpdateGuard* guard)
+            = 0;
 
         /**
          * @brief addObject is used to add doors.
          * @param id is used to distinguish different objects.
          * @param shape members must live until object is updated by another shape or removed from Navigator.
          * @param transform allows to setup objects geometry according to its world state.
-         * @return true if object is added, false if there is already object with given id.
          */
-        virtual bool addObject(const ObjectId id, const DoorShapes& shapes, const btTransform& transform) = 0;
+        virtual void addObject(
+            const ObjectId id, const DoorShapes& shapes, const btTransform& transform, const UpdateGuard* guard)
+            = 0;
 
         /**
          * @brief updateObject replace object geometry by given data.
          * @param id is used to find object.
          * @param shape members must live until object is updated by another shape removed from Navigator.
          * @param transform allows to setup objects geometry according to its world state.
-         * @return true if object is updated, false if there is no object with given id.
          */
-        virtual bool updateObject(const ObjectId id, const ObjectShapes& shapes, const btTransform& transform) = 0;
+        virtual void updateObject(
+            const ObjectId id, const ObjectShapes& shapes, const btTransform& transform, const UpdateGuard* guard)
+            = 0;
 
         /**
          * @brief updateObject replace object geometry by given data.
          * @param id is used to find object.
          * @param shape members must live until object is updated by another shape removed from Navigator.
          * @param transform allows to setup objects geometry according to its world state.
-         * @return true if object is updated, false if there is no object with given id.
          */
-        virtual bool updateObject(const ObjectId id, const DoorShapes& shapes, const btTransform& transform) = 0;
+        virtual void updateObject(
+            const ObjectId id, const DoorShapes& shapes, const btTransform& transform, const UpdateGuard* guard)
+            = 0;
 
         /**
          * @brief removeObject to make it no more available at the scene.
          * @param id is used to find object.
-         * @return true if object is removed, false if there is no object with given id.
          */
-        virtual bool removeObject(const ObjectId id) = 0;
+        virtual void removeObject(const ObjectId id, const UpdateGuard* guard) = 0;
 
         /**
          * @brief addWater is used to set water level at given world cell.
          * @param cellPosition allows to distinguish cells if there is many in current world.
          * @param cellSize set cell borders. std::numeric_limits<int>::max() disables cell borders.
          * @param shift set global shift of cell center.
-         * @return true if there was no water at given cell if cellSize != std::numeric_limits<int>::max() or there is
-         * at least single object is added to the scene, false if there is already water for given cell or there is no
-         * any other objects.
          */
-        virtual bool addWater(const osg::Vec2i& cellPosition, int cellSize, float level) = 0;
+        virtual void addWater(const osg::Vec2i& cellPosition, int cellSize, float level, const UpdateGuard* guard) = 0;
 
         /**
          * @brief removeWater to make it no more available at the scene.
          * @param cellPosition allows to find cell.
-         * @return true if there was water at given cell.
          */
-        virtual bool removeWater(const osg::Vec2i& cellPosition) = 0;
+        virtual void removeWater(const osg::Vec2i& cellPosition, const UpdateGuard* guard) = 0;
 
-        virtual bool addHeightfield(const osg::Vec2i& cellPosition, int cellSize, const HeightfieldShape& shape) = 0;
+        virtual void addHeightfield(
+            const osg::Vec2i& cellPosition, int cellSize, const HeightfieldShape& shape, const UpdateGuard* guard)
+            = 0;
 
-        virtual bool removeHeightfield(const osg::Vec2i& cellPosition) = 0;
+        virtual void removeHeightfield(const osg::Vec2i& cellPosition, const UpdateGuard* guard) = 0;
 
         virtual void addPathgrid(const ESM::Cell& cell, const ESM::Pathgrid& pathgrid) = 0;
 
@@ -163,24 +169,14 @@ namespace DetourNavigator
          * @brief update starts background navmesh update using current scene state.
          * @param playerPosition setup initial point to order build tiles of navmesh.
          */
-        virtual void update(const osg::Vec3f& playerPosition) = 0;
-
-        /**
-         * @brief updatePlayerPosition starts background navmesh update using current scene state only when player position has been changed.
-         * @param playerPosition setup initial point to order build tiles of navmesh.
-         */
-        virtual void updatePlayerPosition(const osg::Vec3f& playerPosition) = 0;
-
-        /**
-         * @brief disable navigator updates
-         */
-        virtual void setUpdatesEnabled(bool enabled) = 0;
+        virtual void update(const osg::Vec3f& playerPosition, const UpdateGuard* guard) = 0;
 
         /**
          * @brief wait locks thread until tiles are updated from last update call based on passed condition type.
          * @param waitConditionType defines when waiting will stop
+         * @param listener optional listener for a progress bar
          */
-        virtual void wait(Loading::Listener& listener, WaitConditionType waitConditionType) = 0;
+        virtual void wait(WaitConditionType waitConditionType, Loading::Listener* listener) = 0;
 
         /**
          * @brief getNavMesh returns navmesh for specific agent half extents
@@ -196,14 +192,14 @@ namespace DetourNavigator
 
         virtual const Settings& getSettings() const = 0;
 
-        virtual void reportStats(unsigned int frameNumber, osg::Stats& stats) const = 0;
+        virtual Stats getStats() const = 0;
 
         virtual RecastMeshTiles getRecastMeshTiles() const = 0;
 
         virtual float getMaxNavmeshAreaRealRadius() const = 0;
     };
 
-    std::unique_ptr<Navigator> makeNavigator(const Settings& settings, const std::string& userDataPath);
+    std::unique_ptr<Navigator> makeNavigator(const Settings& settings, const std::filesystem::path& userDataPath);
 
     std::unique_ptr<Navigator> makeNavigatorStub();
 }
