@@ -1,10 +1,9 @@
 #ifndef OPENMW_COMPONENTS_DETOURNAVIGATOR_RECASTMESHPROVIDER_H
 #define OPENMW_COMPONENTS_DETOURNAVIGATOR_RECASTMESHPROVIDER_H
 
-#include "tileposition.hpp"
 #include "recastmesh.hpp"
 #include "tilecachedrecastmeshmanager.hpp"
-#include "version.hpp"
+#include "tileposition.hpp"
 
 #include <functional>
 #include <memory>
@@ -18,9 +17,10 @@ namespace DetourNavigator
     public:
         RecastMeshProvider(TileCachedRecastMeshManager& impl)
             : mImpl(impl)
-        {}
+        {
+        }
 
-        std::shared_ptr<RecastMesh> getMesh(std::string_view worldspace, const TilePosition& tilePosition) const
+        std::shared_ptr<RecastMesh> getMesh(ESM::RefId worldspace, const TilePosition& tilePosition) const
         {
             return mImpl.get().getNewMesh(worldspace, tilePosition);
         }
