@@ -31,10 +31,7 @@ namespace DetourNavigator
 
     inline TileBounds toNavMeshCoordinates(const RecastSettings& settings, const TileBounds& value)
     {
-        return TileBounds {
-            toNavMeshCoordinates(settings, value.mMin),
-            toNavMeshCoordinates(settings, value.mMax)
-        };
+        return TileBounds{ toNavMeshCoordinates(settings, value.mMin), toNavMeshCoordinates(settings, value.mMax) };
     }
 
     inline float fromNavMeshCoordinates(const RecastSettings& settings, float value)
@@ -50,6 +47,7 @@ namespace DetourNavigator
         return position;
     }
 
+    // Returns value in NavMesh coordinates
     inline float getTileSize(const RecastSettings& settings)
     {
         return static_cast<float>(settings.mTileSize) * settings.mCellSize;
@@ -65,24 +63,28 @@ namespace DetourNavigator
         return static_cast<int>(v);
     }
 
+    // Returns integer tile position for position in navmesh coordinates
     inline TilePosition getTilePosition(const RecastSettings& settings, const osg::Vec2f& position)
     {
         return TilePosition(getTilePosition(settings, position.x()), getTilePosition(settings, position.y()));
     }
 
+    // Returns integer tile position for position in navmesh coordinates
     inline TilePosition getTilePosition(const RecastSettings& settings, const osg::Vec3f& position)
     {
         return getTilePosition(settings, osg::Vec2f(position.x(), position.z()));
     }
 
+    // Returns tile bounds in navmesh coordinates
     inline TileBounds makeTileBounds(const RecastSettings& settings, const TilePosition& tilePosition)
     {
-        return TileBounds {
+        return TileBounds{
             osg::Vec2f(tilePosition.x(), tilePosition.y()) * getTileSize(settings),
             osg::Vec2f(tilePosition.x() + 1, tilePosition.y() + 1) * getTileSize(settings),
         };
     }
 
+    // Returns border size relative to cell size
     inline float getBorderSize(const RecastSettings& settings)
     {
         return static_cast<float>(settings.mBorderSize) * settings.mCellSize;
@@ -98,6 +100,7 @@ namespace DetourNavigator
         return std::floor(std::sqrt(settings.mMaxTilesNumber / osg::PI)) - 1;
     }
 
+    // Returns tile bounds in real coordinates
     inline TileBounds makeRealTileBoundsWithBorder(const RecastSettings& settings, const TilePosition& tilePosition)
     {
         TileBounds result = makeTileBounds(settings, tilePosition);
