@@ -1,9 +1,9 @@
 #ifndef OPENMW_ESM_CELLSTATE_H
 #define OPENMW_ESM_CELLSTATE_H
 
-#include "cellid.hpp"
-
 #include "components/esm/defs.hpp"
+#include "components/esm/refid.hpp"
+#include "timestamp.hpp"
 
 namespace ESM
 {
@@ -15,16 +15,16 @@ namespace ESM
     /// \note Does not include references
     struct CellState
     {
-        CellId mId;
-
+        RefId mId;
+        bool mIsInterior;
         float mWaterLevel;
 
-        int mHasFogOfWar; // Do we have fog of war state (0 or 1)? (see fogstate.hpp)
+        int32_t mHasFogOfWar; // Do we have fog of war state (0 or 1)? (see fogstate.hpp)
 
         TimeStamp mLastRespawn;
 
-        void load (ESMReader &esm);
-        void save (ESMWriter &esm) const;
+        void load(ESMReader& esm);
+        void save(ESMWriter& esm) const;
     };
 }
 
