@@ -3,27 +3,22 @@
 
 #include <vector>
 
+#include "components/esm/common.hpp"
 #include "components/esm/esmcommon.hpp"
+#include "components/esm3/formatversion.hpp"
 
 namespace ESM
 {
     class ESMReader;
     class ESMWriter;
 
-#pragma pack(push)
-#pragma pack(1)
-
     struct Data
     {
-        /* File format version. This is actually a float, the supported
-            versions are 1.2 and 1.3. These correspond to:
-            1.2 = 0x3f99999a and 1.3 = 0x3fa66666
-        */
-        unsigned int version;
-        int type;           // 0=esp, 1=esm, 32=ess (unused)
+        ESM::ESMVersion version;
+        int32_t type; // 0=esp, 1=esm, 32=ess (unused)
         std::string author; // Author's name
-        std::string desc;   // File description
-        int records;        // Number of records
+        std::string desc; // File description
+        int32_t records; // Number of records
     };
 
     struct GMDT
@@ -37,13 +32,9 @@ namespace ESM
         NAME32 mPlayerName;
     };
 
-#pragma pack(pop)
-
     /// \brief File header record
     struct Header
     {
-        static constexpr int CurrentFormat = 1; // most recent known format
-
         // Defines another files (esm or esp) that this file depends upon.
         struct MasterData
         {
@@ -56,13 +47,13 @@ namespace ESM
         std::vector<unsigned char> mSCRS; // Used in .ess savegames only, screenshot
 
         Data mData;
-        int mFormat;
+        FormatVersion mFormatVersion;
         std::vector<MasterData> mMaster;
 
         void blank();
 
-        void load (ESMReader &esm);
-        void save (ESMWriter &esm);
+        void load(ESMReader& esm);
+        void save(ESMWriter& esm);
     };
 
 }
