@@ -6,34 +6,33 @@
 namespace ESM
 {
 
-    void StolenItems::write(ESMWriter &esm) const
+    void StolenItems::write(ESMWriter& esm) const
     {
         for (StolenItemsMap::const_iterator it = mStolenItems.begin(); it != mStolenItems.end(); ++it)
         {
-            esm.writeHNString("NAME", it->first);
-            for (std::map<std::pair<std::string, bool>, int>::const_iterator ownerIt = it->second.begin();
-                 ownerIt != it->second.end(); ++ownerIt)
+            esm.writeHNRefId("NAME", it->first);
+            for (auto ownerIt = it->second.begin(); ownerIt != it->second.end(); ++ownerIt)
             {
                 if (ownerIt->first.second)
-                    esm.writeHNString("FNAM", ownerIt->first.first);
+                    esm.writeHNRefId("FNAM", ownerIt->first.first);
                 else
-                    esm.writeHNString("ONAM", ownerIt->first.first);
+                    esm.writeHNRefId("ONAM", ownerIt->first.first);
                 esm.writeHNT("COUN", ownerIt->second);
             }
         }
     }
 
-    void StolenItems::load(ESMReader &esm)
+    void StolenItems::load(ESMReader& esm)
     {
         while (esm.isNextSub("NAME"))
         {
-            std::string itemid = esm.getHString();
+            ESM::RefId itemid = esm.getRefId();
 
-            std::map<std::pair<std::string, bool>, int> ownerMap;
+            std::map<std::pair<ESM::RefId, bool>, int> ownerMap;
             while (esm.isNextSub("FNAM") || esm.isNextSub("ONAM"))
             {
                 const bool isFaction = (esm.retSubName() == "FNAM");
-                std::string owner = esm.getHString();
+                ESM::RefId owner = esm.getRefId();
                 int count;
                 esm.getHNT(count, "COUN");
                 ownerMap.emplace(std::make_pair(std::move(owner), isFaction), count);
