@@ -4,7 +4,7 @@
 #include <string>
 #include <vector>
 
-#include "components/esm/defs.hpp"
+#include <components/esm/position.hpp>
 
 namespace ESM
 {
@@ -18,17 +18,16 @@ namespace ESM
 
         struct Dest
         {
-            Position    mPos;
+            Position mPos;
             std::string mCellName;
         };
 
         std::vector<Dest> mList;
 
         /// Load one destination, assumes the subrecord name was already read
-        void add(ESMReader &esm);
-
-        void save(ESMWriter &esm) const;
+        void add(ESMReader& esm);
 
+        void save(ESMWriter& esm) const;
     };
 
 }
