@@ -26,23 +26,19 @@
 */
 #include "loadrace.hpp"
 
-#include <stdexcept>
 #include <cstring>
-#include <iostream> // FIXME: for debugging only
-#include <iomanip>  // FIXME: for debugging only
+#include <stdexcept>
 
-#include "formid.hpp"
 #include "reader.hpp"
 //#include "writer.hpp"
 
 void ESM4::Race::load(ESM4::Reader& reader)
 {
-    mFormId = reader.hdr().record.id;
-    reader.adjustFormId(mFormId);
-    mFlags  = reader.hdr().record.flags;
+    mId = reader.getFormIdFromHeader();
+    mFlags = reader.hdr().record.flags;
 
     std::uint32_t esmVer = reader.esmVersion();
-    bool isTES4 = esmVer == ESM::VER_080 || esmVer == ESM::VER_100;
+    bool isTES4 = (esmVer == ESM::VER_080 || esmVer == ESM::VER_100) && !reader.hasFormVersion();
     bool isFONV = esmVer == ESM::VER_132 || esmVer == ESM::VER_133 || esmVer == ESM::VER_134;
     bool isFO3 = false;
 
@@ -53,10 +49,10 @@ void ESM4::Race::load(ESM4::Reader& reader)
     while (reader.getSubRecordHeader())
     {
         const ESM4::SubRecordHeader& subHdr = reader.subRecordHeader();
-        //std::cout << "RACE " << ESM::printName(subHdr.typeId) << std::endl;
+        // std::cout << "RACE " << ESM::printName(subHdr.typeId) << std::endl;
         switch (subHdr.typeId)
         {
-            case ESM4::SUB_EDID:
+            case ESM::fourCC("EDID"):
             {
                 reader.getZString(mEditorId);
                 // TES4
@@ -77,8 +73,10 @@ void ESM4::Race::load(ESM4::Reader& reader)
                 // Imperial    0x00000907
                 break;
             }
-            case ESM4::SUB_FULL: reader.getLocalizedString(mFullName); break;
-            case ESM4::SUB_DESC:
+            case ESM::fourCC("FULL"):
+                reader.getLocalizedString(mFullName);
+                break;
+            case ESM::fourCC("DESC"):
             {
                 if (subHdr.dataSize == 1) // FO3?
                 {
@@ -86,18 +84,13 @@ void ESM4::Race::load(ESM4::Reader& reader)
                     break;
                 }
 
-                reader.getLocalizedString(mDesc); break;
-            }
-            case ESM4::SUB_SPLO: // bonus spell formid (TES5 may have SPCT and multiple SPLO)
-            {
-                FormId magic;
-                reader.getFormId(magic);
-                mBonusSpells.push_back(magic);
-//              std::cout << "RACE " << printName(subHdr.typeId) << " " << formIdToString(magic) << std::endl;
-
+                reader.getLocalizedString(mDesc);
                 break;
             }
-            case ESM4::SUB_DATA: // ?? different length for TES5
+            case ESM::fourCC("SPLO"): // bonus spell formid (TES5 may have SPCT and multiple SPLO)
+                reader.getFormId(mBonusSpells.emplace_back());
+                break;
+            case ESM::fourCC("DATA"): // ?? different length for TES5
             {
 // DATA:size 128
 // 0f 0f ff 00 ff 00 ff 00 ff 00 ff 00 ff 00 00 00
@@ -110,7 +103,7 @@ void ESM4::Race::load(ESM4::Reader& reader)
 // 00 00 00 00 9a 99 99 3e 00 00 a0 40 02 00 00 00
 #if 0
                 unsigned char mDataBuf[256/*bufSize*/];
-                reader.get(&mDataBuf[0], subHdr.dataSize);
+                reader.get(mDataBuf, subHdr.dataSize);
 
                 std::ostringstream ss;
                 ss << ESM::printName(subHdr.typeId) << ":size " << subHdr.dataSize << "\n";
@@ -146,7 +139,7 @@ void ESM4::Race::load(ESM4::Reader& reader)
                     reader.get(mWeightFemale);
                     reader.get(mRaceFlags);
                 }
-                else if (subHdr.dataSize >= 128 && subHdr.dataSize <= 164) // TES5
+                else if (subHdr.dataSize == 128 || subHdr.dataSize == 164) // TES5
                 {
                     mIsTES5 = true;
 
@@ -181,21 +174,21 @@ void ESM4::Race::load(ESM4::Reader& reader)
                     reader.get(dummy2); // size
                     reader.get(dummy2); // head biped object
                     reader.get(dummy2); // hair biped object
-                    reader.get(dummy);  // injured health % (0.f..1.f)
+                    reader.get(dummy); // injured health % (0.f..1.f)
                     reader.get(dummy2); // shield biped object
-                    reader.get(dummy);  // health regen
-                    reader.get(dummy);  // magicka regen
-                    reader.get(dummy);  // stamina regen
-                    reader.get(dummy);  // unarmed damage
-                    reader.get(dummy);  // unarmed reach
+                    reader.get(dummy); // health regen
+                    reader.get(dummy); // magicka regen
+                    reader.get(dummy); // stamina regen
+                    reader.get(dummy); // unarmed damage
+                    reader.get(dummy); // unarmed reach
                     reader.get(dummy2); // body biped object
-                    reader.get(dummy);  // aim angle tolerance
+                    reader.get(dummy); // aim angle tolerance
                     reader.get(dummy2); // unknown
-                    reader.get(dummy);  // angular accleration rate
-                    reader.get(dummy);  // angular tolerance
+                    reader.get(dummy); // angular accleration rate
+                    reader.get(dummy); // angular tolerance
                     reader.get(dummy2); // flags
 
-                    if (subHdr.dataSize > 128)
+                    if (subHdr.dataSize == 164)
                     {
                         reader.get(dummy2); // unknown 1
                         reader.get(dummy2); // unknown 2
@@ -211,43 +204,47 @@ void ESM4::Race::load(ESM4::Reader& reader)
                 else
                 {
                     reader.skipSubRecordData();
-                    std::cout << "RACE " << ESM::printName(subHdr.typeId) << " skipping..."
-                        << subHdr.dataSize << std::endl;
+                    // std::cout << "RACE " << ESM::printName(subHdr.typeId) << " skipping..." << subHdr.dataSize <<
+                    // std::endl;
                 }
 #endif
                 break;
             }
-            case ESM4::SUB_DNAM:
+            case ESM::fourCC("DNAM"):
             {
-                reader.get(mDefaultHair[0]); // male
-                reader.get(mDefaultHair[1]); // female
+                reader.getFormId(mDefaultHair[0]); // male
+                reader.getFormId(mDefaultHair[1]); // female
 
                 break;
             }
-            case ESM4::SUB_CNAM: // Only in TES4?
-            //              CNAM       SNAM                     VNAM
-            // Sheogorath   0x0  0000  98 2b  10011000 00101011
-            // Golden Saint 0x3  0011  26 46  00100110 01000110
-            // Dark Seducer 0xC  1100  df 55  11011111 01010101
-            // Vampire Race 0x0  0000  77 44  01110111 10001000
-            // Dremora      0x7  0111  bf 32  10111111 00110010
-            // Argonian     0x0  0000  dc 3c  11011100 00111100
-            // Nord         0x5  0101  b6 03  10110110 00000011
-            // Breton       0x5  0101  48 1d  01001000 00011101 00000000 00000907 (Imperial)
-            // Wood Elf     0xD  1101  2e 4a  00101110 01001010 00019204 00019204 (HighElf)
-            // khajiit      0x5  0101  54 5b  01010100 01011011 00023FE9 00023FE9 (Argonian)
-            // Dark Elf     0x0  0000  72 54  01110010 01010100 00019204 00019204 (HighElf)
-            // Orc          0xC  1100  74 09  01110100 00001001 000224FD 000224FD (Nord)
-            // High Elf     0xF  1111  e6 21  11100110 00100001
-            // Redguard     0xD  1101  a9 61  10101001 01100001
-            // Imperial     0xD  1101  8e 35  10001110 00110101
-            {
-                reader.skipSubRecordData();
-                break;
-            }
-            case ESM4::SUB_PNAM: reader.get(mFaceGenMainClamp); break; // 0x40A00000 = 5.f
-            case ESM4::SUB_UNAM: reader.get(mFaceGenFaceClamp); break; // 0x40400000 = 3.f
-            case ESM4::SUB_ATTR: // Only in TES4?
+            case ESM::fourCC("CNAM"):
+                //              CNAM       SNAM                     VNAM
+                // Sheogorath   0x0  0000  98 2b  10011000 00101011
+                // Golden Saint 0x3  0011  26 46  00100110 01000110
+                // Dark Seducer 0xC  1100  df 55  11011111 01010101
+                // Vampire Race 0x0  0000  77 44  01110111 10001000
+                // Dremora      0x7  0111  bf 32  10111111 00110010
+                // Argonian     0x0  0000  dc 3c  11011100 00111100
+                // Nord         0x5  0101  b6 03  10110110 00000011
+                // Breton       0x5  0101  48 1d  01001000 00011101 00000000 00000907 (Imperial)
+                // Wood Elf     0xD  1101  2e 4a  00101110 01001010 00019204 00019204 (HighElf)
+                // khajiit      0x5  0101  54 5b  01010100 01011011 00023FE9 00023FE9 (Argonian)
+                // Dark Elf     0x0  0000  72 54  01110010 01010100 00019204 00019204 (HighElf)
+                // Orc          0xC  1100  74 09  01110100 00001001 000224FD 000224FD (Nord)
+                // High Elf     0xF  1111  e6 21  11100110 00100001
+                // Redguard     0xD  1101  a9 61  10101001 01100001
+                // Imperial     0xD  1101  8e 35  10001110 00110101
+                {
+                    reader.skipSubRecordData();
+                    break;
+                }
+            case ESM::fourCC("PNAM"):
+                reader.get(mFaceGenMainClamp);
+                break; // 0x40A00000 = 5.f
+            case ESM::fourCC("UNAM"):
+                reader.get(mFaceGenFaceClamp);
+                break; // 0x40400000 = 3.f
+            case ESM::fourCC("ATTR"): // Only in TES4?
             {
                 if (subHdr.dataSize == 2) // FO3?
                 {
@@ -279,7 +276,7 @@ void ESM4::Race::load(ESM4::Reader& reader)
             //          |             |
             //          +-------------+
             //
-            case ESM4::SUB_NAM0: // start marker head data /* 1 */
+            case ESM::fourCC("NAM0"): // start marker head data /* 1 */
             {
                 curr_part = 0; // head part
 
@@ -290,7 +287,7 @@ void ESM4::Race::load(ESM4::Reader& reader)
                 }
                 else if (isTES4)
                     mHeadParts.resize(9); // assumed based on Construction Set
-                else
+                else // Optimized for TES5
                 {
                     mHeadPartIdsMale.resize(5);
                     mHeadPartIdsFemale.resize(5);
@@ -299,16 +296,16 @@ void ESM4::Race::load(ESM4::Reader& reader)
                 currentIndex = 0xffffffff;
                 break;
             }
-            case ESM4::SUB_INDX:
+            case ESM::fourCC("INDX"):
             {
                 reader.get(currentIndex);
                 // FIXME: below check is rather useless
-                //if (headpart)
+                // if (headpart)
                 //{
                 //    if (currentIndex > 8)
                 //        throw std::runtime_error("ESM4::RACE::load - too many head part " + currentIndex);
                 //}
-                //else // bodypart
+                // else // bodypart
                 //{
                 //    if (currentIndex > 4)
                 //        throw std::runtime_error("ESM4::RACE::load - too many body part " + currentIndex);
@@ -316,9 +313,13 @@ void ESM4::Race::load(ESM4::Reader& reader)
 
                 break;
             }
-            case ESM4::SUB_MODL:
+            case ESM::fourCC("MODL"):
             {
-                if (curr_part == 0) // head part
+                if (currentIndex == 0xffffffff)
+                {
+                    reader.skipSubRecordData();
+                }
+                else if (curr_part == 0) // head part
                 {
                     if (isMale || isTES4)
                         reader.getZString(mHeadParts[currentIndex].mesh);
@@ -336,23 +337,29 @@ void ESM4::Race::load(ESM4::Reader& reader)
 
                     // TES5 seems to have no body parts at all, instead keep EGT models
                 }
-                else if (curr_part == 2) // egt
-                {
-                    //std::cout << mEditorId << " egt " << currentIndex << std::endl; // FIXME
-                    reader.skipSubRecordData();  // FIXME TES5 egt
-                }
+                // else if (curr_part == 2) // egt
+                // {
+                //     // std::cout << mEditorId << " egt " << currentIndex << std::endl; // FIXME
+                //     reader.skipSubRecordData(); // FIXME TES5 egt
+                // }
                 else
                 {
-                    //std::cout << mEditorId << " hkx " << currentIndex << std::endl; // FIXME
-                    reader.skipSubRecordData();  // FIXME TES5 hkx
+                    // std::cout << mEditorId << " hkx " << currentIndex << std::endl; // FIXME
+                    reader.skipSubRecordData(); // FIXME TES5 hkx
                 }
 
                 break;
             }
-            case ESM4::SUB_MODB: reader.skipSubRecordData(); break; // always 0x0000?
-            case ESM4::SUB_ICON:
+            case ESM::fourCC("MODB"):
+                reader.skipSubRecordData();
+                break; // always 0x0000?
+            case ESM::fourCC("ICON"):
             {
-                if (curr_part == 0) // head part
+                if (currentIndex == 0xffffffff)
+                {
+                    reader.skipSubRecordData();
+                }
+                else if (curr_part == 0) // head part
                 {
                     if (isMale || isTES4)
                         reader.getZString(mHeadParts[currentIndex].texture);
@@ -367,12 +374,12 @@ void ESM4::Race::load(ESM4::Reader& reader)
                         reader.getZString(mBodyPartsFemale[currentIndex].texture);
                 }
                 else
-                    reader.skipSubRecordData();  // FIXME TES5
+                    reader.skipSubRecordData(); // FIXME TES5
 
                 break;
             }
             //
-            case ESM4::SUB_NAM1: // start marker body data /* 4 */
+            case ESM::fourCC("NAM1"): // start marker body data /* 4 */
             {
 
                 if (isFO3 || isFONV)
@@ -386,7 +393,7 @@ void ESM4::Race::load(ESM4::Reader& reader)
                 {
                     curr_part = 1; // body part
 
-                    mBodyPartsMale.resize(5);   // 0 = upper body, 1 = legs, 2 = hands, 3 = feet, 4 = tail
+                    mBodyPartsMale.resize(5); // 0 = upper body, 1 = legs, 2 = hands, 3 = feet, 4 = tail
                     mBodyPartsFemale.resize(5); // 0 = upper body, 1 = legs, 2 = hands, 3 = feet, 4 = tail
                 }
                 else // TES5
@@ -399,28 +406,38 @@ void ESM4::Race::load(ESM4::Reader& reader)
 
                 break;
             }
-            case ESM4::SUB_MNAM: isMale = true; break;  /* 2, 5, 7 */
-            case ESM4::SUB_FNAM: isMale = false; break; /* 3, 6, 8 */
+            case ESM::fourCC("MNAM"):
+                isMale = true;
+                break; /* 2, 5, 7 */
+            case ESM::fourCC("FNAM"):
+                isMale = false;
+                break; /* 3, 6, 8 */
             //
-            case ESM4::SUB_HNAM:
+            case ESM::fourCC("HNAM"):
             {
-                std::size_t numHairChoices = subHdr.dataSize / sizeof(FormId);
-                mHairChoices.resize(numHairChoices);
-                for (unsigned int i = 0; i < numHairChoices; ++i)
-                    reader.get(mHairChoices.at(i));
+                // FIXME: this is a texture name in FO4
+                if (subHdr.dataSize % sizeof(ESM::FormId32) != 0)
+                    reader.skipSubRecordData();
+                else
+                {
+                    std::size_t numHairChoices = subHdr.dataSize / sizeof(ESM::FormId32);
+                    mHairChoices.resize(numHairChoices);
+                    for (unsigned int i = 0; i < numHairChoices; ++i)
+                        reader.getFormId(mHairChoices.at(i));
+                }
 
                 break;
             }
-            case ESM4::SUB_ENAM:
+            case ESM::fourCC("ENAM"):
             {
-                std::size_t numEyeChoices = subHdr.dataSize / sizeof(FormId);
+                std::size_t numEyeChoices = subHdr.dataSize / sizeof(ESM::FormId32);
                 mEyeChoices.resize(numEyeChoices);
                 for (unsigned int i = 0; i < numEyeChoices; ++i)
-                    reader.get(mEyeChoices.at(i));
+                    reader.getFormId(mEyeChoices.at(i));
 
                 break;
             }
-            case ESM4::SUB_FGGS:
+            case ESM::fourCC("FGGS"):
             {
                 if (isMale || isTES4)
                 {
@@ -437,7 +454,7 @@ void ESM4::Race::load(ESM4::Reader& reader)
 
                 break;
             }
-            case ESM4::SUB_FGGA:
+            case ESM::fourCC("FGGA"):
             {
                 if (isMale || isTES4)
                 {
@@ -454,7 +471,7 @@ void ESM4::Race::load(ESM4::Reader& reader)
 
                 break;
             }
-            case ESM4::SUB_FGTS:
+            case ESM::fourCC("FGTS"):
             {
                 if (isMale || isTES4)
                 {
@@ -472,27 +489,27 @@ void ESM4::Race::load(ESM4::Reader& reader)
                 break;
             }
             //
-            case ESM4::SUB_SNAM: //skipping...2 // only in TES4?
+            case ESM::fourCC("SNAM"): // skipping...2 // only in TES4?
             {
                 reader.skipSubRecordData();
                 break;
             }
-            case ESM4::SUB_XNAM:
+            case ESM::fourCC("XNAM"):
             {
-                FormId race;
+                ESM::FormId race;
                 std::int32_t adjustment;
-                reader.get(race);
+                reader.getFormId(race);
                 reader.get(adjustment);
                 mDisposition[race] = adjustment;
 
                 break;
             }
-            case ESM4::SUB_VNAM:
+            case ESM::fourCC("VNAM"):
             {
                 if (subHdr.dataSize == 8) // TES4
                 {
-                    reader.get(mVNAM[0]); // For TES4 seems to be 2 race formids
-                    reader.get(mVNAM[1]);
+                    reader.getFormId(mVNAM[0]); // For TES4 seems to be 2 race formids
+                    reader.getFormId(mVNAM[1]);
                 }
                 else if (subHdr.dataSize == 4) // TES5
                 {
@@ -504,14 +521,14 @@ void ESM4::Race::load(ESM4::Reader& reader)
                 else
                 {
                     reader.skipSubRecordData();
-                    std::cout << "RACE " << ESM::printName(subHdr.typeId) << " skipping..."
-                        << subHdr.dataSize << std::endl;
+                    // std::cout << "RACE " << ESM::printName(subHdr.typeId) << " skipping..." << subHdr.dataSize <<
+                    // std::endl;
                 }
 
                 break;
             }
             //
-            case ESM4::SUB_ANAM: // TES5
+            case ESM::fourCC("ANAM"): // TES5
             {
                 if (isMale)
                     reader.getZString(mModelMale);
@@ -519,22 +536,24 @@ void ESM4::Race::load(ESM4::Reader& reader)
                     reader.getZString(mModelFemale);
                 break;
             }
-            case ESM4::SUB_KSIZ: reader.get(mNumKeywords); break;
-            case ESM4::SUB_KWDA:
+            case ESM::fourCC("KSIZ"):
+                reader.get(mNumKeywords);
+                break;
+            case ESM::fourCC("KWDA"):
             {
-                std::uint32_t formid;
+                ESM::FormId formid;
                 for (unsigned int i = 0; i < mNumKeywords; ++i)
                     reader.getFormId(formid);
                 break;
             }
             //
-            case ESM4::SUB_WNAM: // ARMO FormId
+            case ESM::fourCC("WNAM"): // ARMO FormId
             {
                 reader.getFormId(mSkin);
-                //std::cout << mEditorId << " skin " << formIdToString(mSkin) << std::endl; // FIXME
+                // std::cout << mEditorId << " skin " << formIdToString(mSkin) << std::endl; // FIXME
                 break;
             }
-            case ESM4::SUB_BODT: // body template
+            case ESM::fourCC("BODT"): // body template
             {
                 reader.get(mBodyTemplate.bodyPart);
                 reader.get(mBodyTemplate.flags);
@@ -545,34 +564,54 @@ void ESM4::Race::load(ESM4::Reader& reader)
 
                 break;
             }
-            case ESM4::SUB_BOD2: // TES5
+            case ESM::fourCC("BOD2"):
             {
-                reader.get(mBodyTemplate.bodyPart);
-                mBodyTemplate.flags = 0;
-                mBodyTemplate.unknown1 = 0; // probably padding
-                mBodyTemplate.unknown2 = 0; // probably padding
-                mBodyTemplate.unknown3 = 0; // probably padding
-                reader.get(mBodyTemplate.type);
+                if (subHdr.dataSize == 8 || subHdr.dataSize == 4) // TES5, FO4
+                {
+                    reader.get(mBodyTemplate.bodyPart);
+                    mBodyTemplate.flags = 0;
+                    mBodyTemplate.unknown1 = 0; // probably padding
+                    mBodyTemplate.unknown2 = 0; // probably padding
+                    mBodyTemplate.unknown3 = 0; // probably padding
+                    mBodyTemplate.type = 0;
+                    if (subHdr.dataSize == 8)
+                        reader.get(mBodyTemplate.type);
+                }
+                else
+                {
+                    reader.skipSubRecordData();
+                }
 
                 break;
             }
-            case ESM4::SUB_HEAD: // TES5
+            case ESM::fourCC("HEAD"): // TES5
             {
-                FormId formId;
+                ESM::FormId formId;
                 reader.getFormId(formId);
 
-                // FIXME: no order? head, mouth, eyes, brow, hair
-                if (isMale)
-                    mHeadPartIdsMale[currentIndex] = formId;
-                else
-                    mHeadPartIdsFemale[currentIndex] = formId;
+                if (currentIndex != 0xffffffff)
+                {
+                    // FIXME: no order? head, mouth, eyes, brow, hair
+                    if (isMale)
+                    {
+                        if (currentIndex >= mHeadPartIdsMale.size())
+                            mHeadPartIdsMale.resize(currentIndex + 1);
+                        mHeadPartIdsMale[currentIndex] = formId;
+                    }
+                    else
+                    {
+                        if (currentIndex >= mHeadPartIdsFemale.size())
+                            mHeadPartIdsFemale.resize(currentIndex + 1);
+                        mHeadPartIdsFemale[currentIndex] = formId;
+                    }
+                }
 
-                //std::cout << mEditorId << (isMale ? " male head " : " female head ")
-                        //<< formIdToString(formId) << " " << currentIndex << std::endl; // FIXME
+                // std::cout << mEditorId << (isMale ? " male head " : " female head ")
+                // << formIdToString(formId) << " " << currentIndex << std::endl; // FIXME
 
                 break;
             }
-            case ESM4::SUB_NAM3: // start of hkx model
+            case ESM::fourCC("NAM3"): // start of hkx model
             {
                 curr_part = 3; // for TES5 NAM3 indicates the start of hkx model
 
@@ -612,78 +651,132 @@ void ESM4::Race::load(ESM4::Reader& reader)
             // ManakinRace
             // ManakinRace
             // ManakinRace FX0
-            case ESM4::SUB_NAME: // TES5 biped object names (x32)
+            case ESM::fourCC("NAME"): // TES5 biped object names (x32)
             {
                 std::string name;
                 reader.getZString(name);
-                //std::cout << mEditorId << " " << name << std::endl;
+                // std::cout << mEditorId << " " << name << std::endl;
 
                 break;
             }
-            case ESM4::SUB_MTNM: // movement type
-            case ESM4::SUB_ATKD: // attack data
-            case ESM4::SUB_ATKE: // attach event
-            case ESM4::SUB_GNAM: // body part data
-            case ESM4::SUB_NAM4: // material type
-            case ESM4::SUB_NAM5: // unarmed impact?
-            case ESM4::SUB_LNAM: // close loot sound
-            case ESM4::SUB_QNAM: // equipment slot formid
-            case ESM4::SUB_HCLF: // default hair colour
-            case ESM4::SUB_UNES: // unarmed equipment slot formid
-            case ESM4::SUB_TINC:
-            case ESM4::SUB_TIND:
-            case ESM4::SUB_TINI:
-            case ESM4::SUB_TINL:
-            case ESM4::SUB_TINP:
-            case ESM4::SUB_TINT:
-            case ESM4::SUB_TINV:
-            case ESM4::SUB_TIRS:
-            case ESM4::SUB_PHWT:
-            case ESM4::SUB_AHCF:
-            case ESM4::SUB_AHCM:
-            case ESM4::SUB_MPAI:
-            case ESM4::SUB_MPAV:
-            case ESM4::SUB_DFTF:
-            case ESM4::SUB_DFTM:
-            case ESM4::SUB_FLMV:
-            case ESM4::SUB_FTSF:
-            case ESM4::SUB_FTSM:
-            case ESM4::SUB_MTYP:
-            case ESM4::SUB_NAM7:
-            case ESM4::SUB_NAM8:
-            case ESM4::SUB_PHTN:
-            case ESM4::SUB_RNAM:
-            case ESM4::SUB_RNMV:
-            case ESM4::SUB_RPRF:
-            case ESM4::SUB_RPRM:
-            case ESM4::SUB_SNMV:
-            case ESM4::SUB_SPCT:
-            case ESM4::SUB_SPED:
-            case ESM4::SUB_SWMV:
-            case ESM4::SUB_WKMV:
+            case ESM::fourCC("MTNM"): // movement type
+            case ESM::fourCC("ATKD"): // attack data
+            case ESM::fourCC("ATKE"): // attach event
+            case ESM::fourCC("GNAM"): // body part data
+            case ESM::fourCC("NAM4"): // material type
+            case ESM::fourCC("NAM5"): // unarmed impact?
+            case ESM::fourCC("LNAM"): // close loot sound
+            case ESM::fourCC("QNAM"): // equipment slot formid
+            case ESM::fourCC("HCLF"): // default hair colour
+            case ESM::fourCC("UNES"): // unarmed equipment slot formid
+            case ESM::fourCC("TINC"):
+            case ESM::fourCC("TIND"):
+            case ESM::fourCC("TINI"):
+            case ESM::fourCC("TINL"):
+            case ESM::fourCC("TINP"):
+            case ESM::fourCC("TINT"):
+            case ESM::fourCC("TINV"):
+            case ESM::fourCC("TIRS"):
+            case ESM::fourCC("PHWT"):
+            case ESM::fourCC("AHCF"):
+            case ESM::fourCC("AHCM"):
+            case ESM::fourCC("MPAI"):
+            case ESM::fourCC("MPAV"):
+            case ESM::fourCC("DFTF"):
+            case ESM::fourCC("DFTM"):
+            case ESM::fourCC("FLMV"):
+            case ESM::fourCC("FTSF"):
+            case ESM::fourCC("FTSM"):
+            case ESM::fourCC("MTYP"):
+            case ESM::fourCC("NAM7"):
+            case ESM::fourCC("NAM8"):
+            case ESM::fourCC("PHTN"):
+            case ESM::fourCC("RNAM"):
+            case ESM::fourCC("RNMV"):
+            case ESM::fourCC("RPRF"):
+            case ESM::fourCC("RPRM"):
+            case ESM::fourCC("SNMV"):
+            case ESM::fourCC("SPCT"):
+            case ESM::fourCC("SPED"):
+            case ESM::fourCC("SWMV"):
+            case ESM::fourCC("WKMV"):
+            case ESM::fourCC("SPMV"):
+            case ESM::fourCC("ATKR"):
+            case ESM::fourCC("CTDA"):
+            case ESM::fourCC("CIS1"):
+            case ESM::fourCC("CIS2"):
+            case ESM::fourCC("MODT"): // Model data
+            case ESM::fourCC("MODC"):
+            case ESM::fourCC("MODS"):
+            case ESM::fourCC("MODF"): // Model data end
             //
-            case ESM4::SUB_YNAM: // FO3
-            case ESM4::SUB_NAM2: // FO3
-            case ESM4::SUB_VTCK: // FO3
-            case ESM4::SUB_MODT: // FO3
-            case ESM4::SUB_MODD: // FO3
-            case ESM4::SUB_ONAM: // FO3
-            {
+            case ESM::fourCC("YNAM"): // FO3
+            case ESM::fourCC("NAM2"): // FO3
+            case ESM::fourCC("VTCK"): // FO3
+            case ESM::fourCC("MODD"): // FO3
+            case ESM::fourCC("ONAM"): // FO3
+            case ESM::fourCC("APPR"): // FO4
+            case ESM::fourCC("ATKS"): // FO4
+            case ESM::fourCC("ATKT"): // FO4
+            case ESM::fourCC("ATKW"): // FO4
+            case ESM::fourCC("BMMP"): // FO4
+            case ESM::fourCC("BSMB"): // FO4
+            case ESM::fourCC("BSMP"): // FO4
+            case ESM::fourCC("BSMS"): // FO4
 
-                //std::cout << "RACE " << ESM::printName(subHdr.typeId) << " skipping..." << subHdr.dataSize << std::endl;
+            case ESM::fourCC("FMRI"): // FO4
+            case ESM::fourCC("FMRN"): // FO4
+            case ESM::fourCC("HLTX"): // FO4
+            case ESM::fourCC("MLSI"): // FO4
+            case ESM::fourCC("MPGN"): // FO4
+            case ESM::fourCC("MPGS"): // FO4
+            case ESM::fourCC("MPPC"): // FO4
+            case ESM::fourCC("MPPF"): // FO4
+            case ESM::fourCC("MPPI"): // FO4
+            case ESM::fourCC("MPPK"): // FO4
+            case ESM::fourCC("MPPM"): // FO4
+            case ESM::fourCC("MPPN"): // FO4
+            case ESM::fourCC("MPPT"): // FO4
+            case ESM::fourCC("MSID"): // FO4
+            case ESM::fourCC("MSM0"): // FO4
+            case ESM::fourCC("MSM1"): // FO4
+            case ESM::fourCC("NNAM"): // FO4
+            case ESM::fourCC("NTOP"): // FO4
+            case ESM::fourCC("PRPS"): // FO4
+            case ESM::fourCC("PTOP"): // FO4
+            case ESM::fourCC("QSTI"): // FO4
+            case ESM::fourCC("RBPC"): // FO4
+            case ESM::fourCC("SADD"): // FO4
+            case ESM::fourCC("SAKD"): // FO4
+            case ESM::fourCC("SAPT"): // FO4
+            case ESM::fourCC("SGNM"): // FO4
+            case ESM::fourCC("SRAC"): // FO4
+            case ESM::fourCC("SRAF"): // FO4
+            case ESM::fourCC("STCP"): // FO4
+            case ESM::fourCC("STKD"): // FO4
+            case ESM::fourCC("TETI"): // FO4
+            case ESM::fourCC("TTEB"): // FO4
+            case ESM::fourCC("TTEC"): // FO4
+            case ESM::fourCC("TTED"): // FO4
+            case ESM::fourCC("TTEF"): // FO4
+            case ESM::fourCC("TTET"): // FO4
+            case ESM::fourCC("TTGE"): // FO4
+            case ESM::fourCC("TTGP"): // FO4
+            case ESM::fourCC("UNWP"): // FO4
+            case ESM::fourCC("WMAP"): // FO4
+            case ESM::fourCC("ZNAM"): // FO4
                 reader.skipSubRecordData();
                 break;
-            }
             default:
                 throw std::runtime_error("ESM4::RACE::load - Unknown subrecord " + ESM::printName(subHdr.typeId));
         }
     }
 }
 
-//void ESM4::Race::save(ESM4::Writer& writer) const
+// void ESM4::Race::save(ESM4::Writer& writer) const
 //{
-//}
+// }
 
-//void ESM4::Race::blank()
+// void ESM4::Race::blank()
 //{
-//}
+// }
