@@ -1,17 +1,18 @@
 #ifndef OPENMW_CONSTRAINEDFILESTREAMBUF_H
 #define OPENMW_CONSTRAINEDFILESTREAMBUF_H
 
-#include <components/platform/file.hpp>
-
+#include <filesystem>
 #include <streambuf>
 
+#include <components/platform/file.hpp>
+
 namespace Files
 {
     /// A file streambuf constrained to a specific region in the file, specified by the 'start' and 'length' parameters.
     class ConstrainedFileStreamBuf final : public std::streambuf
     {
     public:
-        ConstrainedFileStreamBuf(const std::string& fname, std::size_t start, std::size_t length);
+        ConstrainedFileStreamBuf(const std::filesystem::path& fname, std::size_t start, std::size_t length);
 
         int_type underflow() final;
 
