@@ -1,15 +1,14 @@
 #ifndef OPENMW_COMPONENTS_FILES_OPENFILE_H
 #define OPENMW_COMPONENTS_FILES_OPENFILE_H
 
+#include <filesystem>
 #include <iosfwd>
 #include <memory>
 #include <string>
 
-#include <boost/filesystem/fstream.hpp>
-
 namespace Files
 {
-    std::unique_ptr<boost::filesystem::ifstream> openBinaryInputFileStream(const std::string& path);
+    std::unique_ptr<std::ifstream> openBinaryInputFileStream(const std::filesystem::path& path);
 }
 
 #endif
