@@ -1,8 +1,8 @@
 #ifndef OPENMW_COMPONENTS_FONTLOADER_H
 #define OPENMW_COMPONENTS_FONTLOADER_H
 
-#include <MyGUI_XmlDocument.h>
 #include <MyGUI_Version.h>
+#include <MyGUI_XmlDocument.h>
 
 #include <components/myguiplatform/myguidatamanager.hpp>
 #include <components/to_utf8/to_utf8.hpp>
@@ -25,24 +25,23 @@ namespace Gui
     class FontLoader
     {
     public:
-        FontLoader (ToUTF8::FromType encoding, const VFS::Manager* vfs, float scalingFactor);
+        /// @param exportFonts export the converted fonts (Images and XML with glyph metrics) to files?
+        FontLoader(ToUTF8::FromType encoding, const VFS::Manager* vfs, float scalingFactor, bool exportFonts);
 
-        void overrideLineHeight(MyGUI::xml::ElementPtr _node, const std::string& _file, MyGUI::Version _version);
+        void overrideLineHeight(MyGUI::xml::ElementPtr _node, std::string_view _file, MyGUI::Version _version);
 
-        int getFontHeight();
-
-        static std::string getFontForFace(const std::string& face);
+        static std::string_view getFontForFace(std::string_view face);
 
     private:
         ToUTF8::FromType mEncoding;
         const VFS::Manager* mVFS;
-        int mFontHeight;
         float mScalingFactor;
+        bool mExportFonts;
 
         void loadFonts();
         void loadFont(const std::string& fontName, const std::string& fontId);
 
-        void loadBitmapFont (const std::string& fileName, const std::string& fontId);
+        void loadBitmapFont(const std::string& fileName, const std::string& fontId);
         void loadTrueTypeFont(const std::string& fileName, const std::string& fontId);
 
         FontLoader(const FontLoader&);
