@@ -1,12 +1,13 @@
 #include "content.hpp"
+#include "element.hpp"
 
 namespace LuaUi
 {
     sol::protected_function loadContentConstructor(LuaUtil::LuaState* state)
     {
-        sol::function loader = state->loadInternalLib("content");
+        sol::protected_function loader = state->loadInternalLib("content");
         sol::set_environment(state->newInternalLibEnvironment(), loader);
-        sol::table metatable = loader().get<sol::table>();
+        sol::table metatable = LuaUtil::LuaState::throwIfError(loader()).get<sol::table>();
         if (metatable["new"].get_type() != sol::type::function)
             throw std::logic_error("Expected function");
         return metatable["new"].get<sol::protected_function>();
@@ -14,6 +15,8 @@ namespace LuaUi
 
     bool isValidContent(const sol::object& object)
     {
+        if (object.is<Element>())
+            return true;
         if (object.get_type() != sol::type::table)
             return false;
         sol::table table = object;
