@@ -1,12 +1,15 @@
 #include "helpviewer.hpp"
 
-#include <QString>
 #include <QDesktopServices>
+#include <QString>
 #include <QUrl>
 
+#include <components/version/version.hpp>
+
 void Misc::HelpViewer::openHelp(const char* url)
 {
-    QString link {OPENMW_DOC_BASEURL};
+    std::string_view docsUrl = Version::getDocumentationUrl();
+    QString link = QString::fromUtf8(docsUrl.data(), docsUrl.size());
     link.append(url);
     QDesktopServices::openUrl(QUrl(link));
 }
