@@ -1,7 +1,9 @@
 #pragma once
 
-namespace Misc {
-    namespace HelpViewer {
+namespace Misc
+{
+    namespace HelpViewer
+    {
         void openHelp(const char* url);
     }
 }
